#ifndef QCALC_CONTEXT_H
#define QCALC_CONTEXT_H

#include "qcalcCommon.h"
#include "qcalcOutputItem.h"
#include "qcalcJoinItem.h"
#include "qcalcView.h"
#include "qcalcTimeLine.h"

// This file defines all the context widgets.

// Everything here should be quite self-explanatory

class BlackBoxContextWidget;
class IOContextWidget;
class PluginContextWidget;
class DefaultContextWidget;
class SelectionContextWidget;
class ExtractorContextWidget;
class JoinContextWidget;
class TimeLineWidget;

extern TimeLineHandler timeLineHander;


class ContextWidget: public QWidget
{
    Q_OBJECT

    public:
        ContextWidget(QWidget* parent, CalcView* view);

        virtual ~ContextWidget(){;}
		void showItemTab(QList<QGraphicsItem *> items);
		void setTimeLine(QString &name);

        QTabWidget* tabWidget;
		CalcView* _view;
		
    public slots:
		
		void setView(bool withItems);
		void newInput(const QString& text);
		void closeEvent(QCloseEvent* ce){ce->ignore();}
		void clearTT();  // not used


    protected:
        BlackBoxContextWidget* bbView;
        IOContextWidget* ioView;
        PluginContextWidget* pluginView;
		JoinContextWidget* joinView;
		DefaultContextWidget* defaultView;
		SelectionContextWidget* selectionView;
		ExtractorContextWidget* extractorView;
		TimeLineWidget* timeLineView;
		QLabel* noViewLabel;
};


class BlackBoxContextWidget: public QScrollArea
{
    Q_OBJECT
    
public:
    BlackBoxContextWidget(QWidget* parent, CalcView* view);
    
    virtual ~BlackBoxContextWidget(){}
    
    void setBlackBoxInfo(VOperatorItem* bb);
    
    protected slots:
    
    void updateName();
    void updateOutputTexts();
    void updateInputTexts();
    
protected:
    
    VOperatorItem* blackBox;
    
    QWidget* inputWidget;
    QVBoxLayout* inputLayout;
    QWidget* outputWidget;
    QVBoxLayout* outputLayout;
    
    QVector<QTextEdit*> inputDescriptions;
    QVector<QTextEdit*> outputDescriptions;
    QVector<QLabel*> inputLabels;
    QVector<QLabel*> outputLabels;
    QVector<QGroupBox*> inputBoxes;
    QVector<QGroupBox*> outputBoxes;
    
    QLineEdit* nameEditor;
    
};


class IOContextWidget: public QScrollArea
{
    Q_OBJECT
    
public:
    IOContextWidget(QWidget* parent, CalcView* view);
    
    virtual ~IOContextWidget(){;}
    
    void setInfo(VCalcItem* item);
    
    protected slots:
    
    void updateName();
    void updateOutputTexts();
    void updateInputTexts();
    
protected:
    
    VCalcItem* IOitem;
    
    QWidget* inputWidget;
    QVBoxLayout* inputLayout;
    QWidget* outputWidget;
    QVBoxLayout* outputLayout;
    
    QVector<QTextEdit*> inputDescriptions;
    QVector<QTextEdit*> outputDescriptions;
    QVector<QLabel*> inputLabels;
    QVector<QLabel*> outputLabels;
    QVector<QGroupBox*> inputBoxes;
    QVector<QGroupBox*> outputBoxes;
    
    QLineEdit* nameEditor;
    
};

class PluginContextWidget: public QTabWidget
{
    Q_OBJECT
    public:
        PluginContextWidget(QWidget* parent=NULL);

        virtual ~PluginContextWidget(){}

        void setPluginInfo(VOperatorItem* plugin);

    protected:

        QTextEdit* ioWidget;
        QTextEdit* helpWidget;
        QTextBrowser* docWidget;
};

class JoinContextWidget: public QWidget
{
	Q_OBJECT
	
	public:
		
		JoinContextWidget(QWidget* parent=NULL, CalcView * view=NULL);
		
		virtual ~JoinContextWidget(){}
		
		void setJoinParameters(bool withItems);
		void setTTivlTab(int i);
		void setTTsourceTab(int i);

		QList<VInputItem*> listInputs();
		QList<VJoinItem*> listJoins();

		// Timeline stuff
		QComboBox*  _TimeIvlSelect;
        QguWLinput* _IvlSource;
        QComboBox*  _TimeTagSelect;
        QguWLinput* _TagSource;
		QComboBox* tlBox;
		
		QPushButton* saveAsButton;
		QPushButton* deleteButton;
		
        QLabel*     IvlSource_label;
        QLabel*     TagSource_label;

		
	public slots:
		
		void setJoinStatus(bool);
        
        void setAlgorithm();
        void setWidthField();
        void setGap();
        void setEnd();
        void setGapField();
        void setMinBoxField();

		void addedTimeLine(const QString&);
		void deletedTimeLine(const QString&);
		void setTimeLineTab(const QString name);
//		void setTimeLineTab(int n);
		void newSelection(const QString&);
		void deleteSelected();		
		void saveChanges();		
		void saveAs();		
		void setTTivl(int i);		
		void setTTsource(int i);
		
	protected:
		QLabel* warningLabel;
		QCheckBox* _JoinToTimelineBox;
		QCheckBox* _keepXrefBox;
        QguWLinput* _width_field;
        QComboBox*  _gap;
        QComboBox*  _end;
        QguWLinput* _gap_field;
        QguWLinput* _box_min_field;
        QComboBox*  _algorithm;

        CalcView *_view;
		
};


class DefaultContextWidget: public QWidget
{
	Q_OBJECT
    public:
        DefaultContextWidget(QWidget* parent, CalcView* view);

        virtual ~DefaultContextWidget(){}
		
	public slots:

		void addInput();
		void addOutput();
		void addExtractor();
		void addJoin();
		void updateBlackBoxMenu();
		
    protected:
		
		CalcView* view;
		QMenu* bbOp;
};

class SelectionContextWidget: public QWidget
{
	Q_OBJECT
    public:
        SelectionContextWidget(QWidget* parent, CalcView* view);

        virtual ~SelectionContextWidget(){}


	protected:
		CalcView* view;
};

class ExtractorContextWidget: public QScrollArea
{
	Q_OBJECT
    public:
        ExtractorContextWidget(ContextWidget* parent);

        virtual ~ExtractorContextWidget(){}
		
		void setExtractorParameters(QList<VExtractionItem*>& items);
		
	public slots:

		void CheckChanged(bool);
		void selectedComponent(QAction*);
		void selectedEvent(QAction*);
		void selectedMeta(QAction*);
		void selectedTime(QAction*);
		void SuffixChanged(const QString&);
		void EventChanged(const QString&);
		void MetaChanged(const QString&);
		void userSetComponent(const char *comp, const char *type);
		void setTarget(int);
		
	protected:
		QList<VExtractionItem*> items;
		
		QLabel* warningLabel;
		
		QComboBox * target;

		
		QCheckBox * compCheckBox;
		QPushButton * ComponentButton;
		QPushButton * EventButton;
		QPushButton * MetaButton;
		QMenu* ComponentMenu0;
		QMenu* ComponentMenu1;
		QMenu* ComponentMenu2;
		QMenu* ComponentMenu3;
		QMenu* ComponentMenuText;
		QMenu* ComponentMenuTime;
		QMenu* ComponentMenuTimeIvl;
		QMenu* ComponentMenuMeta;
		QPushButton * TimeButton;
		QMenu* TimeMenu;
		QMenu* TimetagMenu;
		QLineEdit* SuffixComponent;
		QLineEdit* SuffixEvent;
		QLineEdit* SuffixMeta;
		QLineEdit* SuffixTime;
		QLabel* Suffix;
		
		QCheckBox * arrayCheckBox;
		QPushButton * arrayComponentButton;
		QPushButton * arrayTimeButton;
		QMenu* arrayTimeMenu;
		QLineEdit* arraySuffixComponent;
		QLineEdit* arraySuffixTime;
		QLabel* arraySuffix;
		QLabel *talkBack;
		ContextWidget *ctxtWidget;
};

#endif
