//  qjui.h
//
//  Qsas - join Header File
//
//  Tony Allen
//  Dec 2001
//

#ifndef QSAS_DATA_JUIW_H_
#define QSAS_DATA_JUIW_H_

#include "qt_version.h"
#include <QApplication.h>
#include <QStatusBar.h>
#include <QMessageBox.h>
#include <QClipboard>
#include <qsplitter.h>
#include <QListWidget>
#include <QListWidgetItem>
#include <QLinkedList>
#include "qslot.h"
#include "qxml.h"
#include "DvObj.h"

class QjuiWindow;

#define QJUI_VERSION  " V_2.0 [10 Jan 2014]"

class QjuiWidgetItem;

class QjuiWidgetItem : public QListWidgetItem
{
	public:
		QjuiWidgetItem(const QString & text, QListWidget * parent);

		QjuiWidgetItem(const QjuiWidgetItem& wi);

		virtual ~QjuiWidgetItem();

		QString I_width_field;
        QString I_gap_field;
        QString I_minBox_field;
    
		int I_algorithm;
        int I_gap;
        int I_end;

		void setChoices(int w);
		
		QjuiWindow* owner;

		QString WLcontext;

		// The following field is only used at restore time
		// As old versions of qjui could have folders,
		// folder entries are now loaded, marked "toBeDeleted", and then deleted
		bool toBeDeleted;
		
        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	protected:
};

class QjuiWidget;

class QjuiWidget : public QListWidget
{
	Q_OBJECT

	public:
		QjuiWidget(QWidget * parent, QjuiWindow * owner);

		~QjuiWidget();

		QjuiWindow* owner;
		void setItemChoices(int w);

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
        		
	protected:
		virtual void dragEnterEvent(QDragEnterEvent* event);

		virtual void dropEvent(QDropEvent* event);

		virtual void mousePressEvent(QMouseEvent* event);

		void displayMenu(const QPoint& p);
		QMimeData * mimeData(const QList<QListWidgetItem *> items) const;
		QStringList mimeTypes () const;
		QMenu* RButtonMenu;
    
	public slots:
		void setAlgorithm(int);
		void setWidthField();
        void setGap();
        void setEnd();
        void setGapField();
        void setMinBoxField();
		void updateOnSelection();
		void update1OnSelection();
		void update2OnSelection();
		void update3OnSelection();
		void update4OnSelection();
		void openEditor(QListWidgetItem*);
		void deleteListItems();
		void renameSelectedItem();
		void copy();
		void paste();
		void moveItemUp();
		void moveItemDown();
		
};

   
class QjuiWindow : public QMainWindow {

Q_OBJECT

  QSplitter * _top_splitter;
  QjuiWidget * _join_widget;
  QMenu* options_menu;
  QAction *verboseAction;
  QGroupBox * join_widget;
  QGroupBox * time_widget;
  QGroupBox * feedback_widget;   
  QCheckBox * pad;
  QCheckBox *noJoin;
    
public slots:
  void saveXML();
  void saveAsXML();
  void include(){restore(false);}
  void restore(bool replace=true);
  void restoreXML(QString path, bool replace=true);
  void saveSession(QString);
  void restoreSession(QString);
  void defaultAll();
  void deleteAllItems();
  void verboseToggle();
  bool joinObjects(DvJoinList &JoinedList);
  void DoJoin();
  bool writeFile();
  void keepChoices();
  void applyChoices();
  void setTTsource(int);
  void tagsHadDrop();
  void setTTivl(int);
  void setTimeFeedback();
  void setIntervalOption();
  void messageHandler(QString txt);
  
  void launchManExport();
  void launchManJoin();
  void launchManual();
  void launchVersion();
  void setNoJoin(bool);

signals:

public:
  QjuiWindow( QWidget *parent=0, const char *name=0, bool qexp=false );
  ~QjuiWindow();
  void setDefaults();
  DvEvent getInterval();
  DvEvent getIntervalFromData();
  DvObject_var getTimeTags();
  bool getScalarInterval(double &start, double &end);
  bool getScalarIntervalFromData(double &start, double &end);
  DvObject_var getTagsFromObj();

 void status(const QString& str)
  {
      statusBar()->showMessage(str);
	  if(qApp) qApp->processEvents();
  }

 void status(const QString& str,int ms)
  {
    statusBar()->showMessage(str,ms);
    if(qApp) qApp->processEvents();
  }

  QBoxLayout *buttons_layout;
  QWidget * _buttons_widget;

  QPushButton *dismissBtn;
  QPushButton *WriteBtn;
  QPushButton *JoinBtn;
  QWidget*    _top_widget;

  QguWLinput* _width_field;
  QComboBox*  _gap;
  QComboBox*  _end;
  QguWLinput* _gap_field;
  QguWLinput* _box_min_field;
  QComboBox*  _algorithm;
  QComboBox*  _TimeIvlSelect;

  QguWLinput* _IvlSource;
  QComboBox*  _TimeTagSelect;
  QguWLinput* _TagSource;
  QLabel*     IvlSource_label;
  QLabel*     TagSource_label;
  QLabel* Start_field;
  QLabel* End_field;
  QLabel* Ntags_field;
  QLabel* spacing_field;
  
  QString  _width_field_Def;
  int  _end_Def;
  int  _gap_Def;
  QString  _gap_field_Def;
  int  _algorithm_Def;
  QString _IvlSource_Def;
  int  _TimeIvlSelect_Def;
  QString _TagSource_Def;
  int  _TimeTagSelect_Def;
  QString  _box_min_field_Def;

  QString  _width_field_cpy;
  int  _end_cpy;
  int  _gap_cpy;
  QString  _gap_field_cpy;
  int  _algorithm_cpy;
  QString _IvlSource_cpy;
  int  _TimeIvlSelect_cpy;
  QString _TagSource_cpy;
  int  _TimeTagSelect_cpy;
  QString  _box_min_field_cpy;

  QString _saveset_filter;

  
  bool verbose;
  bool isJoin;

    QString _filename;        ///< Filename used to save the current session or working list content
    QString SAVE_EXTENSION;   ///< File extension used to save the current session or working list content

protected:
  void dragEnterEvent( QDragEnterEvent *event);

};


QjuiWindow * QJUICreatePopup(QWidget *parent, const char *Title, bool qexp=false);
void QJUImakeWindow(QWidget *parent, const char * Title);
void QEXPmakeWindow(QWidget *parent, const char * Title);
void QjuiShowJoin();
void QjuiShowExport();

                                                  
# endif // QSAS_DATA_JUIW_H_
