//  qpui.h
//
//  Qsas - Plot Widgets Interface File
//
//  Janet Barnes
//  July 2002
//
//  Alban Rochel
//  2006?
//
//  Tony Allen
//  Dec 2007, June 2013, Jan 2014

#ifndef QSAS_PLOT_UI_H_
#define QSAS_PLOT_UI_H_

#include <QSplitter.h>
#include <QStatusBar.h>
#include <QSpinBox>
#include <QTabWidget>
#include <QTabBar>
#include <QLayout>
#include <QCheckBox>

#include <QApplication>
#include <QMimeData>
#include <QStyle>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QPainter>
#include <QScrollArea>
#include <QList>
#include <QMessageBox>

#include "qt_version.h"
#include "qslot.h"
#include "QbTimeEditors.h"
#include "IntervalDialog.h"

#include "floatspinbox.h"
#include "IntervalWidgets.h"
#include "ColourTable.h"

#include "PlotWindow.h"
#include "qxml.h"

#define QPUI_VERSION  " V_3.2.0 [ Jan 2014 ]"

#if QT_VERSION >= 0x040300
#define QPUI_CHAR_HEIGHT 130 // Truetype fonts handled by Qt
#else
#define QPUI_CHAR_HEIGHT 90 // Hershey fonts handled by PLplot itself
#endif

#define QPUI_PAGE_HGAP 0.05
#define QPUI_PAGE_VGAP 0.1
#define QPUI_PAGE_LMARGIN 0.1 //0.2
#define QPUI_PAGE_RMARGIN 0.1
#define QPUI_PAGE_TMARGIN 0.05 //0.03
#define QPUI_PAGE_BMARGIN 0.08 //0.05

#define QPUI_PANEL_LOFFSET 0.2
#define QPUI_PANEL_WIDTH 0.7
#define QPUI_PANEL_TOFFSET 0.05
#define QPUI_PANEL_HEIGHT 0.90

#define QPUI_LINE_WIDTH 1
#define QPUI_TICKS_LENGTH 0.7

#define QPUI_TITLE_CHAR_SIZE_ADD 0.3 //1.0
#define QPUI_TITLE_DISP 1.0 //0.7
#define QPUI_SUBTITLE_CHAR_SIZE_ADD 0.1
#define QPUI_SUBTITLE_DISP 2.0 //1.5
#define QPUI_LABELS_CHAR_SIZE_ADD 0.0
#define QPUI_LABELS_DISP 3.0 //2.0
#define QPUI_SYMBOL_SIZE 1.0
#define QPUI_LEGEND_CHAR_SIZE_ADD 0.0
#define QPUI_LEGEND_BOX_LINE_WIDTH 1
#define QPUI_LEGEND_LINE_LENGTH 0.2
#define QPUI_LEGEND_MAX_CHAR_LENGTH 25
#define QPUI_LEGEND_JUST 0.5
#define QPUI_LEGEND_HEIGHT 0.4
#define QPUI_LEGEND_WIDTH 0.25

#define QPUI_COLOUR_BAR_DISP 1.5
#define QPUI_COLOUR_BAR_WIDTH 2.5
#define QPUI_COLOUR_BAR_HEIGHT 1.0
#define QPUI_COLOUR_BAR_VOFFSET 0.0

#define QPUI_SKIP_LOG_VALUE -1.e31

#define QPUI_LOWEST_COLOUR_INDEX 16
#define QPUI_HIGHEST_COLOUR_INDEX 255
#define QPUI_NCOLOURS_MAX 256

#define QPUI_SPECTROGRAM_LOFFSET 0.1
#define QPUI_SPECTROGRAM_WIDTH 0.7
#define QPUI_SPECTROGRAM_TOFFSET 0.1
#define QPUI_SPECTROGRAM_HEIGHT 0.2

#define QP_LINEAR 0
#define QP_LOG 1

#define QP_FROM_META 0
#define QP_FROM_DATA 1
#define QP_FROM_USER 2
#define QP_AUTO_ALL_SELECTED 3

#define T_LAB_DATA_XCOMP 0
#define T_LAB_DATA_XDATA 1
#define T_LAB_DATA_META 2
#define T_LAB_DATA_UNITS 3
#define T_LAB_DATA_OFFSET 4
#define T_LAB_DATA_LAB_VERT 5
#define T_LAB_DATA_UNIT_VERT 6
#define T_LAB_DATA_CHAR 7
#define T_LAB_DATA_FONT_FAMILY 8
#define T_LAB_DATA_FONT_STYLE 9
#define T_LAB_DATA_FONT_WEIGHT 10
#define T_LAB_DATA_COLOUR 11
#define T_LAB_DATA_POWER 12

#define NAME_COL 0
#define LINK_COL 1

#define PATH_SEPARATOR "&&" // String for paths separator, one that will never be used by anyone

#define Q_UNICODE_POINT "#[0x25cf]" //   x.2
#define Q_UNICODE_PLUS "#[0x002b]"                  //   x1
#define Q_UNICODE_ASTERISK "#[0x2217]"              //   x1 
#define Q_UNICODE_CIRCLE "#[0x25cb]" //   x.8
#define Q_UNICODE_CROSS "#[0x00d7]"          // x1
//#define Q_UNICODE_SQUARE "#[0x25fb]" //  x.8 - not windows
#define Q_UNICODE_SQUARE "#[0x25a2]" //  x.8 - rounded corners (places well)
//#define Q_UNICODE_SQUARE "#[0x25a1]" //  x.8
#define Q_UNICODE_TRIANGLE "#[0x25b3]"             //   x.6
#define Q_UNICODE_CIRCLED_POINT "#[0x2299]" //   x.6
#define Q_UNICODE_FILLED_DIAMOND "#[0x2666]" //   x.8
#define Q_UNICODE_DIAMOND "#[0x2662]" //    x.8
#define Q_UNICODE_STAR "#[0x2606]" //   x.5
#define Q_UNICODE_WIDECROSS "#[0x256c]" //   x.6
//#define Q_UNICODE_FILLEDSQUARE "#[0x25fc]" //  x.8 - not windows
#define Q_UNICODE_FILLEDSQUARE "#[0x25a9]" //  x.8 - checkerboard square (places well)
//#define Q_UNICODE_FILLEDSQUARE "#[0x25a0]" //  x.8
#define Q_UNICODE_FILLEDCIRCLE "#[0x25cf]" //  x.8
#define Q_UNICODE_FILLEDSTAR "#[0x2605]" //  x.5
#define Q_UNICODE_LEFT_ARROW "#[0x2190]" // x.8
#define Q_UNICODE_RIGHT_ARROW "#[0x2192]" // x.8
#define Q_UNICODE_UP_ARROW "#[0x2191]" // x.8
#define Q_UNICODE_DOWN_ARROW "#[0x2193]" //x.8
#define Q_UNICODE_CUSTOM_SHAPE "-"
#define Q_UNICODE_NOSYMBOL " " // should never be used


extern ColourTablePalette stdPalette;
extern ColourTablePalette profilePalette;
extern ColourTablePalette importedPalette;

extern ColourTablePalette savedPalette; // Used to reference and save used and non-standard colour tables
extern int importedPaletteOffset; // Used to offset the indexes, when merging a session with another one

PLFLT Q_plsize(PLINT code, const char *ch, PLFLT size); // utility for Q_plpoin() in qpui_trace.cc
void Q_plmtex(const char * side, PLFLT disp, PLFLT pos, PLFLT just, const char * text); // alternative to plmtex that handles Granny strings and Latex syntax

class QpuiItem;

class PageItem;
class FrameItem;
class PanelItem;
class PlotItem;
class TraceItem;
class SpectrumItem;
class EventItem;
class TLabItem;

class PlotList;

typedef vector <double> dblBins;


typedef enum QpItemType_e
{
	QP_TRACE=0,
	QP_SPECTROGRAM=1,
	QP_CONST_XT=2,
	QP_CONST_Y=3,
	QP_EVENT=4,
	QP_PANEL=5,
	QP_FRAME=6,
	QP_PAGE=7,
	QP_NONE=8,
	QP_REG_LINE=9,
	QP_T_LAB=10,
	QP_PLOT=11
} QpItemType;

typedef enum QpFrameType_e
{
	QP_TS=0,
	QP_XY=1
	
} QpFrameType;

typedef enum QpTabType_e
{
	QP_DATA_TAB=0,
	QP_TRACE_TAB=1,
	QP_PLOT_TYPE_TAB=2,
	QP_VALUE_TAB=3,
	QP_COLOUR_TAB=4,
	QP_Y_TAB=5,
	QP_LEGEND_TAB=6,
	QP_XT_TAB=7,
	QP_POSITION_TAB=8,
	QP_FRAME_TAB=9,
	QP_TITLE_TAB=10,
	QP_PAGE_TAB=11,
	QP_LAYOUT_TAB=12,
	QP_EVENT_TAB=13,
	QP_IGNORE_TAB=14,
	QP_LAB_DATA_TAB=15
	
  
} QpTabType;

typedef enum QpuiPageSize_e
{
	QPUI_A4=0,
	QPUI_CUSTOM=1,
	QPUI_EXECUTIVE=2,
	QPUI_LEGAL=3,
	QPUI_LETTER=4,
	QPUI_DEFAULT=5
} QpuiPageSize;

typedef enum QpuiPageDPI_e
{
	QPUI_72_DPI=0,
	QPUI_150_DPI=1,
	QPUI_300_DPI=2,
	QPUI_600_DPI=3,
	QPUI_1200_DPI=4
} QpuiPageDPI;

typedef enum QpuiUnits_e
{
	QPUI_PIXELS=0,
	QPUI_INCHES=1,
	QPUI_CM=2,
	QPUI_MM=3,
	QPUI_PSPOINTS=4
} QpuiUnits;

typedef enum QpuiPlotRatio_e
{
    QPUI_FREE=0,
    QPUI_FIXED=1,
    QPUI_SQUARE_ASPECT=2,
    QPUI_ISOTROPIC=3
} QpuiPlotRatio;

typedef enum QpuiReturnStatus_e
{
	QPUI_SUCCESS=0,
	QPUI_FAILURE=1
} QpuiReturnStatus;

typedef enum QpuiLinkStatus_e
{
	QPUI_UNLINKED=0,
	QPUI_LINKED=1
} QpuiLinkStatus;

typedef enum QpuiUpdateStatus_e
{
	QPUI_NO_UPDATES=0,
	QPUI_UPDATED=1,
	QPUI_INCOMPATIBLE=2
} QpuiUpdateStatus;

typedef enum QpuiPageOrientation_e
{
	QPUI_PORTRAIT=0,
	QPUI_LANDSCAPE
} QpuiPageOrientation;

typedef enum QpuiTickMarks_e
{
	QPUI_NOTICKS=0,
	QPUI_IN=1,
	QPUI_OUT=2,
	QPUI_GRID=3
} QpuiTickMarks;

typedef enum QpuiTickLabels_e
{
	QPUI_AUTOMATIC=0,
	QPUI_DECIMAL=1,
	QPUI_EXPONENTIAL=2,
	QPUI_TIME=3
} QpuiTickLabels;


typedef enum QpLineStyle_e
{
	
	QP_NOLINE=0,
	QP_SOLID=1,
	QP_DASHED=2,
	QP_DOTDASH=3,
	QP_DOTTED=4,
	QP_DASHDOTDOTDOT=5
} QpLineStyle;

typedef enum QpuiColours_e
{
	QPUI_WHITE=0,
	QPUI_BLACK=1,
	QPUI_RED=2,
	QPUI_GREEN=3,
	QPUI_BLUE=4,
	QPUI_CYAN=5,
	QPUI_MAGENTA=6,
	QPUI_YELLOW=7,
	QPUI_ORANGE=8,
	QPUI_YELLOWGREEN=9
} QpuiColours;

typedef enum QpuiColourTable_e
{
	QPUI_RAINBOW=0,
	QPUI_RAINBOW_EXT=1,
	QPUI_GRAY_WHITE_LOW=2,
	QPUI_GRAY_WHITE_HIGH=3,
	QPUI_RED_BLUE=4,
	QPUI_INV_RED_BLUE=5,
	QPUI_EARTH=6,
	QPUI_USER_TABLE=7
} QpuiColourTable;

typedef enum QpuiSymbolShape_e
{
	QPUI_NOSYMBOL=0,
	QPUI_POINT=1,
	QPUI_PLUS=2,
	QPUI_ASTERISK=3,
	QPUI_CIRCLE=23,
	QPUI_CROSS=5,
	QPUI_SQUARE=6,
	QPUI_TRIANGLE=7,
// 	QPUI_CIRCLED_CROSS=8,
	QPUI_CIRCLED_POINT=9,
	QPUI_LENS_FLARE=10,
	QPUI_DIAMOND=11,
	QPUI_STAR=12,
  //QPUI_FILLEDTRIANGLE=13, // Not in plplot: square
	QPUI_WIDECROSS=14,
  //QPUI_FILLEDASTERISK=15, // Not in plplot: david star
	QPUI_FILLEDSQUARE=16,
	QPUI_FILLEDCIRCLE=17,
	QPUI_FILLEDSTAR=18,
	QPUI_LEFT_ARROW=28,
	QPUI_RIGHT_ARROW=29,
	QPUI_UP_ARROW=30,
	QPUI_DOWN_ARROW=31,
	QPUI_CUSTOM_SHAPE=32
} QpuiSymbolShape;


typedef enum QpuiConnectStyle_e
{
	QPUI_STRAIGHT=0,
	QPUI_HIST=1,
	QPUI_NOCONNECT=2
} QpuiConnectStyle;

typedef enum QpuiObjectType_e
{
	QPUI_DATA_SERIES=0,
	QPUI_TIME_SERIES=1,
	QPUI_TIME_CONSTANT=2,
	QPUI_SCALAR_Y_CONSTANT=3,
	QPUI_SCALAR_X_CONSTANT=4
} QpuiObjectType;

typedef enum QpuiJustification_e
{
	QPUI_HCENTRE=0,
	QPUI_LEFT=1,
	QPUI_RIGHT=2
} QpuiJustification;

typedef enum QpuiLegendJust_e
{
	QPUI_VCENTRE=0,
	QPUI_TOP=1,
	QPUI_BOTTOM=2
} QpuiLegendJust;

typedef enum QpuiCharFont_e
{
	QPUI_NORMAL=0,
	QPUI_ROMAN=1,
	QPUI_ITALIC=2,
	QPUI_SCRIPT=3
} QpuiCharFont;

typedef enum QpuiVectorComp_e
{
	QPUI_VECTOR_X=0,
	QPUI_VECTOR_Y=1,
	QPUI_VECTOR_Z=2,
	QPUI_VECTOR_RMAG=3,
	QPUI_VECTOR_LAT_DEG=4,
	QPUI_VECTOR_THETA_DEG=5,
	QPUI_VECTOR_PHI_DEG=6,
	QPUI_VECTOR_LAT_RAD=7,
	QPUI_VECTOR_THETA_RAD=8,
	QPUI_VECTOR_PHI_RAD=9
} QpuiVectorComp;

typedef enum QpuiTimeIntervalComp_e
{
	QPUI_START_TIME=0,
	QPUI_END_TIME=1,
	QPUI_CENTRE_TIME=2,
	QPUI_START_AND_END_TIME=3
} QpuiTimeIntervalComp;

typedef enum QpuiLegendAlign_e
{
	QPUI_INSIDE_TOP_RIGHT=0,
	QPUI_INSIDE_TOP_LEFT=1,
	QPUI_INSIDE_BOTTOM_RIGHT=2,
	QPUI_INSIDE_BOTTOM_LEFT=3,
	QPUI_ABOVE_TOP_CENTRE=4,
	QPUI_OUTSIDE_TOP_RIGHT=5,
	QPUI_USER_DEFINED=6
} QpuiLegendAlign;

typedef enum QpuiLegendBorder_e
{
	QPUI_NO_BORDER=0,
	QPUI_LINE=1
} QpuiLegendBorder;

typedef enum QpuiLegendBackground_e
{
	QPUI_TRANSPARENT=0,
	QPUI_OPAQUE=1
} QpuiLegendBackground;

typedef enum QpuiPageFlds_e
{
	PAGE_SIZE=0,
	PAGE_UNITS=1,
	PAGE_WIDTH=2,
	PAGE_HEIGHT=3,
	PAGE_ORIENTATION=4,
	XWIN_NUM=5,
	IDENT=6,
	NUM_ROWS=7,
	NUM_COLS=8,
	HGAP=9,
	VGAP=10,
	LMARGIN=11,
	RMARGIN=12,
	TMARGIN=13,
	BMARGIN=14,
	AUTOLAYOUT=15,
	PAGE_DPI=16
} QpuiPageFlds;

const int QpuiPageNumFlds=17;

typedef enum QPanelFlds_e
{
	LOFFSET=0,
	WIDTH=1,
	TOFFSET=2,
	HEIGHT=3,
	ASPECT_RATIO=4,
	PANEL_LINE_WIDTH=5,
	TICKS_LENGTH=6,
	PANEL_LINE_COLOUR=7,
	TOP_AXIS=8,
	BOTTOM_AXIS=9,
	LEFT_AXIS=10,
	RIGHT_AXIS=11,
	TOP_TICKS_LABELS=12,
	BOTTOM_TICKS_LABELS=13,
	LEFT_TICKS_LABELS=14,
	RIGHT_TICKS_LABELS=15,
	TOP_LABELS=16,
	BOTTOM_LABELS=17,
	LEFT_LABELS=18,
	RIGHT_LABELS=19,
	X_IS_ZERO=20,
	Y_IS_ZERO=21,
	AUTOTITLE=22,
	TITLE=23,
	TITLE_CHAR_SIZE=24,
	TITLE_DISP=25,
	SUBTITLE=26,
	SUBTITLE_CHAR_SIZE=27,
	SUBTITLE_DISP=28,
	TITLE_JUST=29,
	TITLE_FONT_FAMILY=30,
	TITLE_COLOUR=31,
	LABELS_CHAR_SIZE=32,
	LABELS_DISP=33,
	LABELS_FONT=34,
	LABELS_COLOUR=35,
	LABELS_ROTATION=36,
	FRAME_TYPE=37,
	TICKS_CHAR_SIZE=38,
	LR_ALTERNATING_AXIS=39,
	LR_ALTERNATING_TICKS_LABELS=40,
	LR_ALTERNATING_LABELS=41,
	RL_ALTERNATING_AXIS=42,
	RL_ALTERNATING_TICKS_LABELS=43,
	RL_ALTERNATING_LABELS=44,
	TITLE_FONT_STYLE=45,
	TITLE_FONT_WEIGHT=46
} QpPanelFlds;

const int QpuiPanelNumFlds=47;

typedef enum QpAxes_e
{
	X_AXIS=0,
	Y_AXIS=1,
	T_AXIS=2,
	Z_AXIS=3,
	COL_AXIS=4
} QpAxes;

typedef enum QpuiAxisFlds_e
{
	AXIS_AUTOSCALE=0,

 AXIS_MIN=1,
 AXIS_MAX=2,
 AXIS_T_DATA=3,
 AXIS_LINLOG=4,
 AXIS_NUM_MAJOR_TICKS=5,
 AXIS_NUM_MINOR_TICKS=6,
 AXIS_MAJOR_TICKMARKS=7,
 AXIS_MINOR_TICKMARKS=8,
 AXIS_LABEL=9,
 AXIS_AUTOLABEL=10,
 AXIS_LABEL_CHAR_SIZE=11,
 AXIS_LABEL_DISP=13,
 AXIS_LABEL_ROTATION=14,
 AXIS_LABEL_FONT_FAMILY=15,
 AXIS_LABEL_COLOUR=16,
 AXIS_TICK_TIME_FORMAT=17,
 AXIS_TICK_TIME_FORMAT_AUTO=18,
 AXIS_TIME_LABELS_FORMAT=19,
 AXIS_TIME_LABELS_FORMAT_AUTO=20,
 AXIS_LABEL_FONT_STYLE=21,
 AXIS_LABEL_FONT_WEIGHT=22,
 AXIS_EXP_LABEL_Y=23,
 AXIS_EXP_LABEL_X=24

} QpuiAxisFlds;

const int QpuiAxisNumFlds=24;

typedef enum QpuiLegendFlds_e
{
	SHOW_LEGEND=0,
	LEGEND_BORDER=1,
	LEGEND_NUM_ROWS=4,
	LEGEND_NUM_COLS=5,
	LEGEND_CHAR_SIZE=6,
	LEGEND_MAX_CHAR_LENGTH=7,
	LEGEND_LINE_LENGTH=8,
	LEGEND_JUST=9,
	LEGEND_FONT_FAMILY=10,
	LEGEND_ALIGN=11,
	LEGEND_L_OFFSET=12,
	LEGEND_T_OFFSET=13,
	LEGEND_BACKGROUND=14,
	LEGEND_WIDTH=15,
	LEGEND_HEIGHT=16,
	LEGEND_FONT_STYLE=17,
	LEGEND_FONT_WEIGHT=18
} QpuiLegendFlds;

const int QpuiLegendNumFlds=19;

typedef enum QpuiEventFlds_e
{
	EVENT_LABEL=0,
	EVENT_LABEL_CHAR_SIZE=1,
	EVENT_LABEL_FONT_FAMILY=2,
	EVENT_LABEL_COLOUR=3,
	EVENT_X_DATA=4,
	EVENT_Y_DATA=5,
	EVENT_LABEL_FONT_STYLE=6,
	EVENT_LABEL_FONT_WEIGHT=7,
    EVENT_LABEL_JUST=9
} QpuiEventFlds;

const int QpuiEventNumFlds=9;


typedef enum QpuiTraceFlds_e
{
 TRACE_PLOT_TYPE=0,
 TRACE_YDATA=1,
 TRACE_XDATA=2,
 TRACE_YCOMP=3,
 TRACE_XCOMP=4,
 TRACE_CONNECT_STYLE=5,
 TRACE_LINE_STYLE=6,
 TRACE_LINE_WIDTH=7,
 TRACE_LINE_COLOUR=8,
 TRACE_SYMBOL_SHAPE=9,
 TRACE_SYMBOL_SIZE=10,
 TRACE_SYMBOL_COLOUR=11,
 TRACE_CONSTANT_DATA=12,
 TRACE_TCOMP=13,
 TRACE_LEGEND_TEXT=14,
 TRACE_SHOW_LEGEND=15,
 LEGEND_AUTO_TEXT=16,
 TRACE_GAP_DETECTION=17,
 TRACE_GAP_THRESHOLD=18,
 TRACE_GAP_THRESHOLD_TYPE=19,
 TRACE_LINE_SCALE=20,
 TRACE_CUSTOM_SYMBOL=21,
 TRACE_TIDATA=22,
 TRACE_TICOMP=23,
 TRACE_COL_COMP=24,
 TRACE_COL_DATA=25,
 TRACE_SHOW_COL_BAR=26,
 TRACE_EXP_COL_BAR_X=27,
 TRACE_EXP_COL_BAR_Y=28,
 TRACE_AUTO_COL_BAR=29,
 TRACE_LOG_COL_BAR=30,
 TRACE_COL_MIN=31,
 TRACE_COL_MAX=32,
 TRACE_SET_COL_TABLE_PROF=33,
 TRACE_USER_COL_LAB=34,
 TRACE_COL_BAR_W=35,
 TRACE_COL_BAR_H=36,
 TRACE_COL_BAR_HOFF=37,
 TRACE_COL_BAR_VOFF=38,
 TRACE_COL_BAR_TICK=39,
 TRACE_SET_C_COL=40,
 TRACE_SET_C_SIZE=41,
 TRACE_SET_COL_TABLE_SESS=42,
 TRACE_SET_COL_TABLE_STD=43
} QpuiTraceFlds;

typedef enum QpuiDataFlds_e
{
	USE_TIME_AS_SEC=0
} QpuiDataFlds;

const int QpuiTraceNumFlds=21;

typedef enum QpuiSpectroGramFlds_e
{
	SPECTROGRAM_PLOT_TYPE=0,

 SPECTROGRAM_ZDATA=1,
 COLOUR_SCALE_AUTOSCALE=2,
 COLOUR_SCALE_MIN=3,
 COLOUR_SCALE_MAX=4,
 COLOUR_SCALE_LINLOG=5,
 COLOUR_SCALE_SHOW_COLOUR_BAR=6,
 COLOUR_SCALE_COLOUR_BAR_WIDTH=7,
//  COLOUR_SCALE_TABLE=8,
 COLOUR_SCALE_COLOUR_BAR_DISP=9,
 COLOUR_SCALE_LABEL=10,
 COLOUR_SCALE_AUTOLABEL=11,
 SPECTROGRAM_YDATA_START=12,
 SPECTROGRAM_XDATA_START=13,
 SPECTROGRAM_XCOMP_START=14,
 SPECTROGRAM_YDATA_END=15,
 SPECTROGRAM_XDATA_END=16,
 SPECTROGRAM_XCOMP_END=17,
 SPECTROGRAM_YCOMP_START=18,
 SPECTROGRAM_YCOMP_END=19,
 SPECTROGRAM_ZCOMP=20,
 COLOUR_SCALE_COLOUR_BAR_HEIGHT=21,
 COLOUR_SCALE_COLOUR_BAR_VOFFSET=22,
 COLOUR_SCALE_TICK_LENGTH=23,
 COLOUR_SCALE_TABLE_STD=24,
 COLOUR_SCALE_TABLE_PROFILE=25,
 COLOUR_SCALE_TABLE_SESSION=26,
 COLOUR_SCALE_AXIS_EXP_LABEL_X=27,
 COLOUR_SCALE_AXIS_EXP_LABEL_Y=28,
 SPECTROGRAM_XY_SWAP=29,
 SPECTROGRAM_TICOMP=30,
 SPECTROGRAM_TIDATA=31,
 SPECTROGRAM_INTERLACE=32,
 SPECTROGRAM_MIN_CELL_WIDTH=33,
 SPECTROGRAM_GAP=34
} QpuiSpectroGramFlds;

const int QpuiSpectroGramNumFlds=29;

double getSpacingMedian(DvObject_var D0);

class FontButton: public QPushButton
{
	Q_OBJECT
	
	public:
		FontButton(QWidget* parent=NULL, int family=PL_FCI_SANS, bool italic=false, bool bold=false);
		
		~FontButton();
		
		void getFontParams(PLINT& family, PLINT& style, PLINT& weight);
		void setFontParams(PLINT family, PLINT style, PLINT weight);

#if QT_VERSION < 0x040300
        int getFontIndex();
#endif
	
	signals:
		void familyChanged(int);
		void styleChanged(bool);
		void weightChanged(bool);
		
	public slots:

		void setSansFont();
		void setSerifFont();
		void setMonotypeFont();
		void setDecorativeFont();
		void setItalic(bool i);
		void checkItalic();
		void setBold(bool b);
		void checkBold();

        void setRomanFont();
        void setNormalFont();
        void setItalicFont();

		
	protected:
		
		void update();
		
		int family;
		bool italic;
		bool bold;
		
		QMenu* menu;

		QAction* sansAction;
		QAction* serifAction;
		QAction* monotypeAction;
		QAction* decorativeAction;
		QAction* italicAction;
		QAction* boldAction;
#if QT_VERSION < 0x040300
        QAction* normalFontAction;
        QAction* romanFontAction;
        QAction* italicFontAction;
#endif
};

class ColourTableButton: public QPushButton
{
    Q_OBJECT

    public:
        ColourTableButton(QWidget* parent=0);

        void setCurrentTable(const ColourTable&);

        ColourTable getCurrentTable() const;

        void setStdPalette(int i); // for compatibility with old combobox
        void setProfilePalette(int i);
        void setSessionPalette(int i);
        
    public slots:
        void updateMenuEntries();

        void selected(QAction*);

    signals:

        void selectedStd(int);
        void selectedProfile(int);
        void selectedSession(int);
        
    protected:
        QMenu* menu;
        ColourTable currentTable;
};

// QPUI version 3 classes - AJA Jan 2008

class QpWindow;

// QpTab top level class
// ----------------------
class QpTab:  public QWidget
{
	Q_OBJECT

	public:
		QpTab(QTabWidget *tabWidget,  const QString& name, QpTabType _tabType);
		~QpTab(){;}
		void setSlotColour(bool active);

		QPalette GUIpalette;
		QpTabType TabType;
	
};

// QpValueReturnTab Class
// ----------------------

class QpValueReturnTab:  public QpTab
{
	Q_OBJECT

	public:
		QpValueReturnTab(QTabWidget *tabWidget,  const QString& name, PageItem * page);
	
		QpValueReturnTab(QpValueReturnTab* tab, PageItem* _page); // Copy constructor for copy/paste functions

		~QpValueReturnTab(){ 
			killTimer(_timer);
		}

		PageItem*  _page_item;
		FrameItem* _frame_item;
		PanelItem* _panel_item;

		QPushButton *_rePlot;
		QPushButton *_reset;

		ScalarIntervalWidget* _xaxis_range_fld;
		TimeIntervalWidget * _taxis_range_fld;
		QCheckBox *_xaxis_rescale_fld;
		QCheckBox *_xaxis_swap_fld;


		ScalarIntervalWidget* _yaxis_range_fld;
		QCheckBox *_yaxis_rescale_fld;
		QCheckBox *_yaxis_swap_fld;
		
		QLabel *message_lbl;
		QCheckBox *_replot_fld;

		double _x_from;
		double _x_to;
		double _y_from;
		double _y_to;

		vector <double> _x_from_old;
		vector <double>  _x_to_old;
		vector <double>  _y_from_old;
		vector <double>  _y_to_old;
		vector <int>     num_old_frames;
		vector <int>     num_old_panels;
		vector <FrameItem*>  _frame_item_old;
		vector <PanelItem*>  _panel_item_old;
		vector <int>  _panel_autoscale_old;
		vector <int>  _frame_autoscale_old;
        
		int _timer;

	protected:
		
	public slots:
		void catchValueRet(int);
		void reject(){;}
		void resetValues();
		void rePlot();
		void getValues();
	protected:
		void create(); // common part for both constructors
};


// class QpAxisTab
//------------------

class QpAxisTab : public QpTab
{
	Q_OBJECT

	public:
		QpAxisTab(QTabWidget *tabWidget,  const QString& name, QpuiItem * item, QpTabType _tabType);
		QpAxisTab(QpAxisTab* tab, QpuiItem* item, QpTabType _tabType);
		~QpAxisTab(){;}
			// methods to be re-implemented for each axis type
		virtual void setAxisMinMax(){;}
		virtual void setAutoLabelText(){;}
		virtual void setTfromObj(){;}
		void refreshYZRanges();
		void handleAutoScale(int);
	
			QpuiItem * I_Item;
			QpAxes I_axis;
	
			PanelItem * I_Panel;
			QComboBox *_axis_autoscale_fld;
			QLabel* linlog_label;
			QComboBox *_axis_linlog_fld;
			QLabel* t_drop_label;
			QguWLinput *_t_data_fld;

			QCheckBox *_axis_y_nudge_fld;

	
			ScalarIntervalWidget* _axis_xy_fld;
			TimeIntervalWidget* _axis_time_fld;

			QSpinBox *_axis_major_ticks_num_fld;
			QComboBox *_axis_major_tickmarks_fld;
			QSpinBox *_axis_minor_ticks_num_fld;
			QComboBox *_axis_minor_tickmarks_fld;
			QGroupBox* _exp_label_params_groupbox; // hidden for T axes
			FloatSpinBox * _exp_label_offset_y_fld;
			FloatSpinBox * _exp_label_offset_x_fld;
			QLineEdit *_label_fld;
			FloatSpinBox *_label_char_size_fld;
			FloatSpinBox *_label_disp_fld;
			QCheckBox *_axis_autolabel_fld;
			FloatSpinBox *_label_rotation_fld;
			FontButton *_label_font_fld;
			QPushButton *_label_colour_fld;
			QPushButton *_event_label_fld;
			QGroupBox *frame_type_groupbox;
			QPushButton *_frame_type_btn;
	
			QLabel* tick_time_fmt_label;
			QLabel* time_labels_fmt_label;
			QLineEdit* tick_time_fmt_fld;
			QLineEdit* time_labels_fmt_fld;
			QCheckBox* tick_time_fmt_auto_fld;
			QCheckBox* time_labels_fmt_auto_fld;
	
			int axis_auto_last;
			QColor _label_colour;
            bool userSetLinLog;


			QDomElement toXMLElement(QDomDocument&) const;
			void fromXMLElement(QDomElement&);
	
	//IntervalEditorWindow* editorWindow;

	public slots:
		void setAxisAutoScale(int);
		void setAxisLinLog(int);
		void setAxisRange();
		void setAxisNumMajorTicks(int);
		void setAxisMajorTickMarks(int);
		void setAxisNumMinorTicks(int);
		void setAxisMinorTickMarks(int);
        void setAxisExpLabelYOffset(double);
        void setAxisExpLabelXOffset(double);
		void setAxisLabel();
		void setAxisLabelCharSize(double);
		void setAxisLabelDisp(double);
		void setAxisAutoLabel(bool);
		void setAxisLabelRotation(double);
		void setAxisLabelFamily(int);
		void setAxisLabelStyle(bool);
		void setAxisLabelWeight(bool);
		void setAxisLabelColour();
		void launchFrameTypeSelector();
	protected:
		void create();

};

// class QpXAxisTab
//------------------

class QpXAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpXAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame);
		QpXAxisTab(QpXAxisTab* tab, FrameItem* frame);
		~QpXAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
	
		FrameItem * I_Frame;
	
	public slots:
    
		void interactiveRescale(double start, double end);

};


// class QpTAxisTab
//------------------

class QpTAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpTAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame);
		QpTAxisTab(QpTAxisTab* tab, FrameItem* frame);
		~QpTAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
		void setTfromObj();
	
		FrameItem * I_Frame;
	
	public slots:
		void setTData();
    
		void setTickTimeFormat(const QString&);
		void setTickTimeFormatAuto(int);
		void setTimeLabelsFormat(const QString&);
		void setTimeLabelsFormatAuto(int);

		void interactiveRescale(DvEvent);
};

// class QpYAxisTab
//------------------

class QpYAxisTab : public QpAxisTab
{
	Q_OBJECT

	public:
		QpYAxisTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel);
		QpYAxisTab(QpYAxisTab* tab, PanelItem* panel);
		~QpYAxisTab(){;}
		void setAxisMinMax();
		void setAutoLabelText();
	
		PanelItem * I_Panel;

	public slots:
		void updateNudge();
};

// class QpLayoutTab
//------------------

class QpLayoutTab : public QpTab
{
	Q_OBJECT
	public:
		QpLayoutTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page);
		QpLayoutTab(QpLayoutTab* tab, PageItem* _page);
	
		~QpLayoutTab(){;}
		void updateRowCol(int numFrames);

		PageItem * I_Page;
		QSpinBox* _row_num_fld;
		QSpinBox* _col_num_fld;
		QCheckBox* _autolayout_fld;
		FloatSpinBox* _hgap_fld;
		FloatSpinBox* _vgap_fld;
		FloatSpinBox* _lmargin_fld;
		FloatSpinBox* _rmargin_fld;
		FloatSpinBox* _tmargin_fld;
		FloatSpinBox* _bmargin_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setNumRows(int);
		void setNumCols(int);
		void setHGap(double);
		void setVGap(double);
		void setLMargin(double);
		void setRMargin(double);
		void setTMargin(double);
		void setBMargin(double);
		void setAutoLayout(bool);
	protected:
		void create(); // common part for both constructors


};

// class QpPageTab
//------------------

class QpPageTab : public QpTab
{
	Q_OBJECT
	public:
		QpPageTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page);
		QpPageTab(QpPageTab* tab, PageItem* _page);
        ~QpPageTab(){;}

		PageItem * I_Page;
	
		QComboBox* _page_size_fld;
		QComboBox* _units_fld;
		QLineEdit* _width_fld;
		QLineEdit* _height_fld;
		QComboBox* _dpi_fld;
		QComboBox* _orientation_fld;
		QCheckBox* _show_ident_fld;
		int oldUnits;
        PlotID* _xwin_num_fld;
        QGridLayout *page_output_layout;
        
        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setPageSize(int);
		void setUnits(int);
		void setDPI(int);
		void setWidth(const QString&);
		void setHeight(const QString&);
		void setOrientation(int);
		void setIdent(bool);
	
	protected:
		void create(); // common part for both constructors

};

class QpEventTab: public QpTab
{
	Q_OBJECT

	public:
		QpEventTab( QTabWidget *tabWidget,  const QString& name, EventItem * _event);
		QpEventTab(QpEventTab* tab, EventItem* _event);
		~QpEventTab(){;}

		EventItem * I_event;
		QguWLinput* _event_label_fld;
		QLabel* _y_label_fld;
		QLabel* _x_label_fld;
		QguWLinput* _y_data_fld;
		QguWLinput* _x_data_fld;
		FloatSpinBox* _char_size_fld;
		FontButton* _event_label_font_fld;
		QPushButton* _event_label_colour_fld;
        FloatSpinBox* _event_label_just_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);

	signals:

	public slots:
		void setEventLabel();
		void setEventLabelCharSize(double);
        void setEventLabelJust(double);
		void setEventLabelFamily(int);
		void setEventLabelStyle(bool);
		void setEventLabelWeight(bool);
		void setEventLabelColour();
		void setYData();
		void setXData();
		void getValues();
	protected:
		void create();

};



// class QpDisplayTab
//------------------

class QpDisplayTab : public QpTab
{
	Q_OBJECT
	public:
		QpDisplayTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpDisplayTab(QpDisplayTab* tab, FrameItem* _frame);
		~QpDisplayTab(){;}

		FrameItem* I_Frame;
  
		FloatSpinBox* _line_width_fld;
		QPushButton* _line_colour_fld;
		FloatSpinBox* _ticks_length_fld;
		FloatSpinBox* _ticks_char_size_fld;
		QCheckBox* _show_top_axis_fld;
		QCheckBox* _show_bottom_axis_fld;
		QCheckBox* _show_left_axis_fld;
		QCheckBox* _show_right_axis_fld;
		QCheckBox* _show_lr_alternating_axis_fld;
		QCheckBox* _show_rl_alternating_axis_fld;
		QCheckBox* _show_top_ticks_labels_fld;
		QCheckBox* _show_bottom_ticks_labels_fld;
		QCheckBox* _show_left_ticks_labels_fld;
		QCheckBox* _show_right_ticks_labels_fld;
		QCheckBox* _show_lr_alternating_ticks_labels_fld;
		QCheckBox* _show_rl_alternating_ticks_labels_fld;
		QCheckBox* _show_top_labels_fld;
		QCheckBox* _show_bottom_labels_fld;
		QCheckBox* _show_left_labels_fld;
		QCheckBox* _show_right_labels_fld;
		QCheckBox* _show_lr_alternating_labels_fld;
		QCheckBox* _show_rl_alternating_labels_fld;
		QCheckBox* _show_xiszero_fld;
		QCheckBox* _show_yiszero_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setLineWidth(double);
		void setLineColour();
		void setTicksLength(double);
		void setTicksCharSize(double);
		void setTopAxis(bool);
		void setBottomAxis(bool);
		void setLeftAxis(bool);
		void setRightAxis(bool);
		void setLRAlternatingAxis(bool);
		void setRLAlternatingAxis(bool);
		void setTopTicksLabels(bool);
		void setBottomTicksLabels(bool);
		void setLeftTicksLabels(bool);
		void setRightTicksLabels(bool);
		void setLRAlternatingTicksLabels(bool);
		void setRLAlternatingTicksLabels(bool);
		void setTopLabels(bool);
		void setBottomLabels(bool);
		void setLeftLabels(bool);
		void setRightLabels(bool);
		void setLRAlternatingLabels(bool);
		void setRLAlternatingLabels(bool);
		void setYIsZero(bool);
		void setXIsZero(bool);
	protected:
		void create();

};

// class QpPropertiesTab
// --------------------------

class QpPropertiesTab : public QpTab
{
	Q_OBJECT
	public:
		QpPropertiesTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpPropertiesTab(QpPropertiesTab* tab, FrameItem* _frame);
		~QpPropertiesTab(){;}
		double setIsotropicRatio();

		FrameItem* I_Frame;
		QLabel* _autolayout_lbl;
		FloatSpinBox* _loffset_fld;
		FloatSpinBox* _width_fld;
		FloatSpinBox* _toffset_fld;
		FloatSpinBox* _height_fld;
        QComboBox* _aspect_ratio_fld;
		QComboBox* _frame_type_fld;
		double _aspect_ratio;
		double _isotropic_y_range;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
	public slots:
		void setLOffset(double);
		void setTOffset(double);
		void setWidth(double);
		void setHeight(double);
		void setAspectRatio(int);
        
	protected:
		void create();

};

// class QpTitleTab
// -----------------

class QpTitleTab : public QpTab
{
	Q_OBJECT
	public:
		QpTitleTab(QTabWidget *tabWidget,  const QString& name, FrameItem * _frame);
		QpTitleTab(QpTitleTab* tab, FrameItem* _frame);
		~QpTitleTab(){;}

		FrameItem* I_Frame;
		QCheckBox* _autotitle_fld;
		QLineEdit* _title_fld;
		FloatSpinBox* _title_char_size_fld;
		FloatSpinBox* _title_disp_fld;
		QLineEdit* _subtitle_fld;
		FloatSpinBox* _subtitle_char_size_fld;
		FloatSpinBox* _subtitle_disp_fld;
		QComboBox* _title_just_fld;
		FontButton* _title_font_fld;
		QPushButton* _title_colour_fld;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
					
	public slots:
		void setAutoTitle(bool);
		void setTitle(const QString &);
		void setTitleCharSize(double);
		void setTitleDisp(double);
		void setSubTitle(const QString &);
		void setSubTitleCharSize(double);
		void setSubTitleDisp(double);
		void setTitleJust(int);
		void setTitleFamily(int);
		void setTitleStyle(bool);
		void setTitleWeight(bool);
		void setTitleColour();
	protected:
		void create();
	
};



// class QpTLabDataTab
//------------------

class QpTLabDataTab : public QpTab
{
	Q_OBJECT
	public:

		QpTLabDataTab(QTabWidget *tabWidget,  const QString& name, TLabItem * _plot, QpItemType plotType);
		QpTLabDataTab(QpTLabDataTab* tab, TLabItem* _plot);
		~QpTLabDataTab(){;}
			
		TLabItem * I_LabItem;
		QpItemType I_PlotType;
	
		QLabel* _x_label_fld;
		QguWLinput* _x_data_fld;
		QLineEdit *metaText;
		QLineEdit *unitText;
		FloatSpinBox *powText;
        QDomElement toXMLElement(QDomDocument&, QpItemType) const;
        void fromXMLElement(QDomElement&);
		DvString old_BaseXName;
		DvString old_XComp;
		FloatSpinBox *_offset_y_fld;
		FloatSpinBox *_label_char_size_fld;
		QCheckBox *_label_V_fld;
		QCheckBox *_unit_V_fld;
		FontButton *_label_font_fld;
		QPushButton *_label_colour_fld;
		
		bool UserUnit;
		bool UserText;
		QColor _label_colour;

	public slots:
		void setXData();
		void setMeta(const QString &);
		void setUnit(const QString &);
		void setPow(double);
		void setOffset(double);
		void setLabVert(bool);
		void setUnitVert(bool);
		void setCharSize(double);
		void setFontFamily(int);
		void setFontStyle(bool);
		void setFontWeight(bool);
		void setLabelColour();
		   

	protected:
		void create();
        
};

// class QpDataTab
//------------------

class QpDataTab : public QpTab
{
	Q_OBJECT
	public:

		QpDataTab(QTabWidget *tabWidget,  const QString& name, PlotItem * _plot, QpItemType plotType);
		QpDataTab(QpDataTab* tab, PlotItem* _plot);
		~QpDataTab();
		void setDataTypeTitle();
	
		PlotItem * I_Plot;
		QpItemType I_PlotType;
	
		QLabel* _y_label_fld;
		QguWLinput* _y_data_fld;
		QLabel* _x_label_fld;
		QguWLinput* _x_data_fld;
		QLineEdit *_legend_text_fld;
		QCheckBox* _show_legend_fld;
		QCheckBox* _auto_legend_fld;

		QLabel* _mask_label_fld;
		QguWLinput* _mask_data_fld;

        QWidget* _gap_params_container;
        QCheckBox* _gap_detection_fld;
        FloatSpinBox* _gap_threshold_fld;
        QComboBox* _gap_threshold_type_fld;
        double gap_threshold; // in seconds
		
		QGroupBox *meta_groupbox;
		QMenu *metaMenu;
        QLineEdit *metaText;
		vector <QAction*> metaActions;
	
        
	// for Spectrum
		QCheckBox* _XYtoggle;
		QguWLinput* _z_data_fld;
		QguWLinput* _y_start_data_fld;
		QLabel* _x_start_label_fld;
		QguWLinput* _x_start_data_fld;
		QCheckBox *gap_toggle;
		FloatSpinBox *minCellWidth;
		
		QCheckBox *interlace_toggle;
		QCheckBox *timeAsSec_toggle;
		QGroupBox *data_type_groupbox;
		DvString old_BaseName;
		DvString old_Comp;
		DvString old_BaseTIName;
		DvString old_BaseXName;
		DvString old_XComp;
		DvString old_YComp;

        QDomElement toXMLElement(QDomDocument&, QpItemType) const;
        void fromXMLElement(QDomElement&, QpItemType);

	public slots:
		void setYData();
		void setXData();
		void setTIData();
		void resetLinReg();
		void setTI_ZData();
		void setLegendText(const QString &);
		void setAutoLegend(bool);
		void setShowLegend();
		void useTimeAsSec(bool);
		void gap_toggle_changed(bool);
		void interlace_toggle_changed(bool);
		void propagateLink();
        void gap_detection_toggled(bool);
        void gap_threshold_changed(double);
        void gap_threshold_type_changed(int);
        void updateGapThreshold();
		void setMetaMenu();
		void setMetaText();
		void minCellWidth_changed(double value);
   
	// for Spectrum
		void setZData();
		void setXYswap();
		void setYStartData();
		void checkYStartData(const QString &);
		void setXStartData();
		void launchTypeSelector();
	protected:
		void create();
        
};


// class QpTraceTab
//------------------

class QpTraceTab : public QpTab
{
	Q_OBJECT
	public:
		QpTraceTab(QTabWidget *tabWidget,  const QString& name, TraceItem * _trace);
		QpTraceTab(QpTraceTab* tab, TraceItem* trace);
		~QpTraceTab(){;}
   
		TraceItem * I_Trace;
  
		QComboBox* _line_style_fld;
		FloatSpinBox* _line_width_fld;
		QComboBox* _connect_style_fld;
		FloatSpinBox* _line_scale_fld;
		QPushButton* _line_colour_fld;
		QComboBox* _symbol_shape_fld;
		FloatSpinBox* _symbol_size_fld;
		QLineEdit* _custom_symbol_fld;
		QPushButton* _symbol_colour_fld;
		void setLineColour(QColor c);
		void setSymbolColour(QColor c);
		QLabel *_description_fld;
		QguWLinput* _c_data_fld;
		QLabel *_c_label_fld;
		QLabel *maxLabel;
		QLabel *minLabel;
		QLabel* colour_table_label;
		QLabel * colLabLab;
		QCheckBox *c_range_auto;
		QCheckBox *c_range_log;
		QLineEdit *_c_min_fld;
		QLineEdit *_c_max_fld;
		ColourTableButton *colourTable;
		QCheckBox *_show_colour_bar;
		QLineEdit *colLabSlot;
		bool userColLab;
		FloatSpinBox *_exp_label_offset_x;
		FloatSpinBox *_exp_label_offset_y;
		FloatSpinBox *_colour_bar_width;
		FloatSpinBox *_colour_bar_height;
		FloatSpinBox *_colour_bar_disp;
		FloatSpinBox *_colour_bar_voffset;
		FloatSpinBox *_colourscale_ticklength;
		bool useColBarOn;
		int oldSymbolShape;
		QCheckBox *_colour_on;
		QCheckBox *_size_on;
		
        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setLineStyle(int choice);
		void setLineWidth(double choice);
		void setConnectStyle(int choice);
		void setLineScale(double choice);
		void setLineColour();
		void setSymbolShape(int choice);
		void setSymbolSize(double choice);
		void setSymbolColour();
		void setCustomSymbol(const QString& text);
		void setColData();
		void setColDataOff();
		void setColUserMin(const QString &);
		void setColUserMax(const QString &);
		void setAutoColour(bool on);
		void setLogColour(bool on);
		void setUserColLab(const QString& );
		void setAxisExpLabelXOffset(double val);
		void setAxisExpLabelYOffset(double val);
		void showColourBar(bool);
		void setColourTableProfile(int);
		void setColourTableSession(int);
		void setColourTableStd(int);
        void setColBarWidth(double);
        void setColBarHeight(double);
        void setColBarHoffset(double);
        void setColBarVoffset(double);
        void setColBarTickLen(double);
		void setColourOn(bool);
		void setSizeOn(bool);
	
	protected:
		void create();
		DvString old_BaseName;
		DvString old_Comp;


};


// class QpLegendTab
//------------------

class QpLegendTab : public QpTab
{
	Q_OBJECT
	public:
		QpLegendTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel);
		QpLegendTab(QpLegendTab* tab, PanelItem* panel);
		~QpLegendTab(){;}
	
		PanelItem * I_Panel;
		QCheckBox *_show_legend_fld;
		QComboBox *_legend_border_fld;
		QComboBox *_legend_background_fld;
		QSpinBox *_legend_num_rows_fld;
		QSpinBox *_legend_num_cols_fld;
		FloatSpinBox *_legend_width_fld;
		FloatSpinBox *_legend_height_fld;
		QSpinBox *_legend_max_char_length_fld;
		FloatSpinBox *_legend_line_length_fld;
		FloatSpinBox *_legend_char_size_fld;
		QComboBox *_legend_just_fld;
		FontButton *_legend_font_fld;
		QComboBox *_legend_align_fld;
		FloatSpinBox *_legend_loffset_fld;
		FloatSpinBox *_legend_toffset_fld;
		QGroupBox* legend_box_groupbox;
		QGroupBox* legend_text_groupbox;
		QGroupBox* legend_position_groupbox;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
	public slots:
		void setShowLegend(bool);
		void setLegendBorder(int);
		void setLegendBackground(int);
		void setLegendNumRows(int);
		void setLegendNumCols(int);
		void setLegendWidth(double);
		void setLegendHeight(double);
		void setLegendMaxCharLength(int);
		void setLegendLineLength(double);
		void setLegendCharSize(double);
		void setLegendJust(int);
// 	void setLegendFont(int);
		void setLegendFamily(int);
		void setLegendStyle(bool);
		void setLegendWeight(bool);
		void setLegendAlign(int);
		void setLegendLoffset(double);
		void setLegendToffset(double);
		void updateNumTraces();
	protected:
		void create();
	
};


class QpColourscaleTab : public QpTab
{
	Q_OBJECT
	public:
		QpColourscaleTab(QTabWidget *tabWidget,  const QString& name, SpectrumItem * _spectro);
		QpColourscaleTab(QpColourscaleTab* tab, SpectrumItem* _spectro);
		~QpColourscaleTab(){;}

		SpectrumItem * I_Spectrum;
  
		QComboBox* _colourscale_auto_fld;
		QComboBox* _colourscale_linlog_fld;
		QguWLinput* _colourscale_min_fld;
		QguWLinput* _colourscale_max_fld;
		QCheckBox* _colourscale_show_colour_bar_fld;
		FloatSpinBox* _colourscale_colour_bar_width_fld;
		FloatSpinBox* _colourscale_colour_bar_height_fld;
		FloatSpinBox* _colourscale_colour_bar_voffset_fld;
		FloatSpinBox* _colourscale_colour_bar_disp_fld;
		FloatSpinBox* _colourscale_ticklength_fld;
        FloatSpinBox * _exp_label_offset_y_fld;
        FloatSpinBox * _exp_label_offset_x_fld;
        ColourTableButton* _colourtable_fld;
		QLineEdit* _colourscale_label_fld;
		QCheckBox* _colourscale_autolabel_fld;
	
		int colour_auto_last;
    bool userSetColLinLog;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);


	public slots:
		void setColourScaleMin();
		void setColourScaleMax();
		void setColourScaleLabel();
		void setColourscaleMinMax();
		void setColourScaleAutoLabel();
		void setAutoScale(int);
		void setLinLog(int);
		void setColourScaleAutoLabelToggle(bool On);
		void setColourbarWidth(double width);
		void setColourbarVoffset(double offset);
		void setColourbarHeight(double height);
		void setColourbarDisplacement(double disp);
        void setColourtableStd(int choice);
        void setColourtableProfile(int choice);
        void setColourtableSession(int choice);
		void setShowColourBar(bool On);
		void setTicksLength(double);
        void setAxisExpLabelXOffset(double);
        void setAxisExpLabelYOffset(double);
	protected:
		void create();
	
};

// Function generating the dash styles
// ----------------------
void setDashStyle(QpLineStyle style, double scale=1.);

// Function getting the character height in normalized device coordinates
// ----------------------
PLFLT getCharHeight();

int getPGPColourIndex(const QColor&, bool reset=false);
PLINT symbolIndexConversion(int index);
const char * symbolUnicodeConversion(int index);
string SafePGPString(const string&);
float roundf(float,short);

void print_iden();

// Plot Utility calls
//------------------
QpItemType QpRemapPlotType(int _type);


class QpuiColourScale
{
	public:
		bool _autoscale;
		bool _recalc_range;
		double _min;
		double _max;
		int _linlog;
		bool _show_colour_bar;
		float _colour_bar_width;
		int _colour_table;
		float _colour_bar_disp;
		string _label;
		bool _autolabel;
		float _colour_bar_height;
		float _colour_bar_voffset;

};

// class  QpWindow
// -----------------------

class QpWindow : public QMainWindow
{
	Q_OBJECT
 
	QWidget*    _top_widget;
	QGridLayout* _top_gl;
	QSplitter * _top_splitter;
	QStatusBar* _status_bar;
	QMenuBar *_menu_bar;
	PlotList* plotList;

    QToolBar *_file_tool_bar;
    QAction *restoreAct;
    QAction *saveAsAct;
    QAction *deleteAct;
    QAction *newPageAct;
    QAction *newTSAct;
    QAction *newSubAct;
    QAction *newXYAct;
    QAction *newTraceAct;
    QAction *newSpecAct;
    QAction *eventAct;
    QAction *cutAct;
    QAction *copyAct;
    QAction *pasteAct;
    QAction *helpAct;
    
	public slots:
		void status(const QString& str) {_status_bar->showMessage(str,2000);};
        void doPlot();
        void plotHidden();
		void deleteAllItems();
        void saveAsXML();
        void restoreXML(bool replace=true);
        void restoreXML(QString path, bool replace=true);

		void launchManPlot();
		void launchManual();
		void launchVersion();
        void save();
        void include();
        void saveSession(QString);
        void restoreSession(QString);
        void bringToFront(){this->activateWindow(); this->raise();}
        void toggleToolbar();

	signals:
        void refreshColourTableButtonsSignal();

	public:
		QpWindow( QWidget *parent=0);
		~QpWindow(){;}
		void resetDefaultBtn();
        void saveAsXML(QString path);

		QMenuBar* menu_bar(){ return _menu_bar;}

		int _releaseVersion;
		int _majorVersion;
		int _minorVersion;

        void refreshColourTableButtons();

        QString _filename;        ///< Filename used to save the current session or working list content
        QString SAVE_EXTENSION;   ///< File extension used to save the current session or working list content
        
	protected:
		void keyPressEvent(QKeyEvent * e );
        QPushButton *plotBtn;
        QPushButton *plotSelBtn;
		QPushButton *dismissBtn;
        
        void createToolBars();  // SEH
        void createActions();   // SEH
        QAction* toolBar;
        
};



void QPUImakeWindow(QWidget*,const char*);
void QpuiShow();
extern QpWindow *QPUI_widget;
DvObject_var makeIndexObject(DvObject_var &zdo, int indx);

// ------------------------------

// ----- QpuiItem class --------

class QpuiItem: public QTreeWidgetItem
{	
	public:
		QpuiItem(const QString& name="", QpuiItem* parent=NULL);
		QpuiItem(const QString& name="", PlotList* parent=NULL);
        ~QpuiItem(){;}
		
		virtual PageItem * page(){ return NULL; }
		virtual FrameItem * frame(){ return NULL; }
		virtual PanelItem * panel(){ return NULL; }

		QpuiItem * parent(){return (QpuiItem *)QTreeWidgetItem::parent();}
		
		bool setName(const QString& newName);
		void setNextName();
		void setIcon(QpItemType_e type);
		inline QString getName() const {return text(0);}		
		inline PlotList* Tree() const {return (PlotList*) this->treeWidget();}		
		QString getPath();

		inline bool hasChildren() const {return (childCount() != 0);}
		int childNumber();
		QpuiItem* getChildByName(const QString& name) const;
		void insertAbove(QpuiItem* item);
		void insertBelow(QpuiItem* item);
		void expandChildren();
				
		bool moveOK(QpuiItem* item);
		QpuiItem* duplicateItem(QpuiItem* parent);
		void insertItem(QpuiItem* item);
		bool isAncestorOf(QpuiItem* other);
		bool ancestorInList(QList <QTreeWidgetItem*> &list);
		int level();
        virtual void addCustomColourTableTo(ColourTablePalette&);
		
		void clear();
				
		virtual void showTabs(){;}
		virtual void setDefaultOptions(){;}
		virtual QpFrameType frameType(){return QP_TS;}
		virtual void updateAxes(){;}

        
		QpItemType I_Type;
		QTabWidget *tabWidget;
		bool userSetName;

//        virtual QDomElement toXMLElement(QDomDocument&) const{QDomElement xml=doc.createElement("Plot_Item");return xml;}
        virtual void fromXMLElement(QDomElement&){;}
		
	protected:		

};

// ------------------------------

class PageItem: public QpuiItem
{
	public:
		
		PageItem(const QString& name="", PlotList* parent=NULL, int position=-1);
		PageItem(PageItem* page, PlotList* parent, int position=-1);
		~PageItem();
		
		void setName(const QString& newName); // also sets plot Window message
		
		PageItem * page(){ return this; }
						
		QpuiReturnStatus plot(PlotWindow* win);
		void set_colour_table(const QpuiColourTable& colour_table=QPUI_RAINBOW);
        void updateIsotropicFrames();
		void setLayout();
		void showTabs();
		void setFramesAuto(bool active);
		void setDefaultOptions();
		void setSmooth(bool smooth);

        double aspectRatio() const;
        
		int panelsInColumn(int colno);
		
		int _print_id;

		bool containsSpectrum();
		double _conversion_factors[5];
		static double _width_in_inches[6];
		static double _height_in_inches[6];
		
		QpValueReturnTab* ValueReturnTab;
		QpPageTab* PageTab;
		QpLayoutTab* LayoutTab;
		
		
        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
        PlotWindow* focusWindow();
		
};

// ------------------------------

class FrameItem: public QpuiItem
{

	public:
		
		FrameItem(const QString& name="", PageItem* parent=NULL, QpFrameType frame_type=QP_TS, int position=-1);
		FrameItem(FrameItem* frame, PageItem* parent, int position=-1);
		~FrameItem();

		bool panelBlank(int position);
		
		PageItem * page(){ 
			if(parent()) return parent()->page();
			else return NULL;
		}
		FrameItem * frame(){return this;}
		
		string getTitle();
		void showTabs();
		bool getXmetaRange(double& min, double& max);
		bool getXRange(double& min, double& max);
		void getYObjects();
		int numberInStack();
		QpuiReturnStatus plot(PlotWindow* win);
		void postPlot();
		QpFrameType frameType(){return _frameType;}
		void setPanelProperties(double loffset, double width, double toffset, double height, int n_rows, int n_cols);
		int panelNum(PanelItem *panelItem);
		void changeFrame(int frameType);
		bool containsSpectrum();
		void updateAxes();
		void setDefaultOptions();
		void updateIsotropic(bool resize = true);
		
		QpPropertiesTab* PropertiesTab;
		QpDisplayTab*   DisplayTab;
		QpTitleTab*     TitleTab;
		QpAxisTab * xtAxisTab; // pointer to x or t tab
		QpXAxisTab * xAxisTab;
		QpTAxisTab * tAxisTab;
		
		QColor _line_colour;
		QColor _title_colour;
		QpFrameType _frameType;

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
		bool restoring;

};

// ------------------------------

class PanelItem: public QpuiItem
{

	public:
		
		PanelItem(const QString& name="", FrameItem* parent=NULL, int position=-1);
		PanelItem(PanelItem* panel, FrameItem* parent, int position=-1);
		~PanelItem();
		
		PageItem * page(){ 
			if(parent()) return parent()->page();
			else return NULL;
		}
		FrameItem * frame(){ 
			if(parent()) return parent()->frame();
			else return NULL;
		}
		PanelItem * panel(){return this;}
		
		bool isBlank();
		bool hasTLabel();
		
		void showTabs();
		int  getStackNumber();
		bool getXmetaRange(double& panel_xmin, double& panel_xmax);
		int  getXmetaIsLog();
		bool getXRange(double& panel_xmin, double& panel_xmax);
		void getYObjects();
		bool getYmetaRange(double& ymin, double& ymax);
		bool getYRange(double& _ymin, double& _ymax);
		void getLabel(const QpAxes& axis, DvString& label);
		int numberTraces();
		int numberSpectra();
		int numberThisSpectrum(SpectrumItem *item);
		QpuiReturnStatus plot(double _height, double _width, double _toffset, double _loffset, PlotWindow* win);
		void postPlot(bool show_top_axis, bool show_bottom_axis, bool show_title, bool show_T_lab);
		void prePlot();
		DvString getTitle();
		QpFrameType frameType(){return frame()->frameType();}
		void updateYZRange();
		void updateAxes();
		void setDefaultOptions();
		bool isInterlaced();
		int numTLabel(TLabItem *item);
		void handleBoundary(DvString format, DvTime start, DvTime end, bool &showStart, bool &showEnd);
		
		QpLegendTab* LegendTab;
		QpYAxisTab * yAxisTab;
		
		int   _legend_entry;
		float _legend_cell_width;
		float _legend_cell_height;

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
};

class TLabItem: public QpuiItem
{

	public:
		
		TLabItem(const QString& name="", PanelItem* parent=NULL, const QString& WLcontext="", int position=-1);
		TLabItem(TLabItem* plot, PanelItem* parent, int position=-1);
		~TLabItem();
		
		PageItem * page(){ 
			if(parent()) return parent()->page();
			else return NULL;
		}
		FrameItem * frame(){ 
			if(parent()) return parent()->frame();
			else return NULL;
		}
		PanelItem * panel(){ return parent()->panel();}
				
		QpFrameType frameType();
		QpuiReturnStatus plot();
		void replaceXData(QString obj_name);
		void replaceXComponent(QString comp);
		void showTabs();
        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		void tickLabels(PLFLT tstart, PLFLT tend, PLFLT ttick1, PLFLT disp, PLINT length, DvObject_var &data, const char *linelabel, const char *units);
		
		QString WLcontext;

		QpTLabDataTab * TLabDataTab;
		
	protected:
};



class PlotItem: public QpuiItem
{

	public:
		
		PlotItem(const QString& name="", PanelItem* parent=NULL, const QString& WLcontext="", int position=-1);
		PlotItem(PlotItem* plot, PanelItem* parent, int position=-1);
		~PlotItem();
		
		PageItem * page(){ 
			if(parent()) return parent()->page();
			else return NULL;
		}
		FrameItem * frame(){ 
			if(parent()) return parent()->frame();
			else return NULL;
		}
		PanelItem * panel(){ 
			if(parent()) return parent()->panel();
			else return NULL;
		}
				
		void setLinkText(const QString& linkText){
			this->linkText=linkText;
			this->setText(1, linkText);
		}
		QString getLinkText() const {return linkText;}
		
		virtual void changeFrameType(int frameType){;}
		QpFrameType frameType();
		virtual	QpuiReturnStatus plot(){return QPUI_FAILURE;}
		virtual void replaceData(QString obj_name){;}
		virtual void replaceComponent(QString comp){;}
		virtual void replaceTIdata(QString obj_name){;}
		virtual void replaceTIcomp(QString obj_name){;}
		virtual void replaceXData(QString obj_name){;}
		virtual void replaceXComponent(QString comp){;}
		virtual void clearObj_id(){;}
		virtual int getXmetaIsLog(){return 0;}
		virtual int getYmetaIsLog(){return 0;}
		virtual bool getXmetaRange(double& min, double& max){return false;}
		virtual bool getYmetaRange(double&, double&){return false;}
		virtual bool getXRange(double& min, double& max){return false;}
		virtual bool getYRange(double& ymin, double& ymax){return false;}
		virtual void get_dobjs(){;}
		void setLinks(QList<PlotItem*>& linkList);
		void removeLink(PlotItem* plot_item);
		QString WLcontext;

		QpDataTab * DataTab;
			
		int with;
		double missing_value;
		QList<PlotItem*> linkedPlots;

		
	protected:
		QString linkText;
};


class TraceItem : public PlotItem
{
	
	public:

		TraceItem(const QString& name="", PanelItem* parent=NULL, const QString& WLcontext="", QpItemType plotType=QP_TRACE, int position=-1);
		TraceItem(TraceItem* trace, PanelItem* parent, int position=-1);
		~TraceItem();
		
        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
		
		int getXmetaIsLog();
		int getYmetaIsLog();
		bool getXmetaRange(double& min, double& max);
		bool getYmetaRange(double& min, double& max);
		bool getXRange(double& min, double& max);
		bool getYRange(double& ymin, double& ymax);
		void getLabel(const QpAxes& axis, DvString& label);
		DvString getLegendText();
		DvString getTitle();
		void getFrameName(string& name); // replace getName(string& name)
		void changeFrameType(int frameType);
		void showTabs();
		QpuiReturnStatus plotLegend();
		QpuiReturnStatus plot();
		void replaceData(QString obj_name);
		void replaceComponent(QString comp);
		void replaceTIdata(QString comp);
		void replaceTIcomp(QString comp);
		void replaceXData(QString obj_name);
		void replaceXComponent(QString comp);
		void refreshTabs();
		void clearObj_id();
		void updateAxes();
		void get_dobjs();
		void setDefaultOptions();
		void updateColourBar();
		void replace_C_Component(QString comp);
		void replaceCdata(QString base);
		void setDataYObj(QString txt);
		void setDataXObj(QString txt);
        void plotHist(int n_pts, PLFLT *x_data, PLFLT *y_data);

		QpTraceTab * TraceTab;

		DvObject_var _ydo;
		DvObject_var _xdo;
		DvObject_var _cdo; // for colour points by extra variable c
		DvObject_var linFit_var;
		
		int _x_obj_id_getXRange;
		int _y_obj_id_getYRange;
		double _x_range_min; // min used last y range
		double _x_range_max;   // max used last y range
		double _x_from; // min from data for autoscale
		double _x_to;   // max from data for autoscale
		double _y_from; // min from data for autoscale
		double _y_to;   // max from data for autoscale
		
		QColor _line_colour;
		QColor C1_colour;
		QColor C2_colour;
		QColor C3_colour;
		QColor C4_colour;
		QColor R_colour;
		QColor X_colour;
		QColor Y_colour;
		QColor Z_colour;
		QColor T_colour;
		QColor P_colour;
		QColor L_colour;
		QColor _symbol_colour;
		bool userLineColour;
		bool userSymbolColour;
};


class EventItem : public PlotItem
{
	
	public:

		EventItem(const QString& name="", PanelItem* parent=NULL, const QString& WLcontext="", int position=-1);
		EventItem(EventItem* item, PanelItem* parent, int position=-1);
		~EventItem();
		
		void setDataYObj(QString txt);
		void setDataXObj(QString txt);
		
		QpuiReturnStatus plot();
		void showTabs();

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);
	
		QpEventTab *eventTab;
		QColor _event_label_colour;
};


class SpectrumItem : public PlotItem
{

	public:

		SpectrumItem(const QString& name="", PanelItem* parent=NULL, const QString& WLcontext="", int position=-1);
		SpectrumItem(SpectrumItem* item, PanelItem* parent, int position=-1);
		~SpectrumItem();
		
		
		void setDataZObj(QString txt);
		void setDataYObj(QString txt);
		void setDataXObj(QString txt);
		
		void getLabel(const QpAxes axis, DvString& label);
		int  getXData(DvObject_var &xdo, DvObject_var &xdo_end);
		int  getXData(DvObject_var &xdo, DvObject_var &xdo_plus, DvObject_var &xdo_minus);
		void refreshX();
		int  getXData(DvObject_var &xdo);
		int  getXData(DvObject_var &xdo, int nSpectra, int nItem);
		int  getYData(DvObject_var &ydo, DvObject_var &ydo_end);
		int  getYData(DvObject_var &ydo, DvObject_var &ydo_plus, DvObject_var &ydo_minus);
		int  getYData(DvObject_var &ydo);
		bool  isZConformal(int nx_len, int ny_len);
		string getTitle();
		void getSpectrogramName(string& name); // replaces getName(string&)
		int  getXmetaIsLog();
		int  getYmetaIsLog();
		bool getXRange(double&, double&);
		bool getXmetaRange(double&, double&);
		bool getYmetaRange(double&, double&);
		bool getYRange(double& ymin, double& ymax);
		bool getZRange(double &zmin, double &zmax);
		bool getZmetaRange(double& zmin,double& zmax);
		void updateAxes();
		void get_dobjs();
		void setDefaultOptions();

		DvObject_var getZDO();
		int  fill_X_Data();
		int  fill_Y_Data();
		int  fill_Z_Data(int XtoY, bool inPlot=true);
		void changeFrameType(int frameType);
		void showTabs();
		QpuiReturnStatus plot();
		void replaceData(QString obj_name);
		void replaceComponent(QString comp);
		void replaceTIdata(QString comp);
		void replaceTIcomp(QString comp);
		void refreshTabs();
		void clearObj_id();

        virtual QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&);

        void addCustomColourTableTo(ColourTablePalette&);
        
		QpColourscaleTab* ColourscaleTab;

		DvObject_var _zdo;
		int _z_obj_id;
		vector <dblBins> _z_data_all;
		DvObject_var _ydo;
		int _y_start_obj_id;
		vector <dblBins> _y_start_all;
		int _y_end_obj_id;
		vector <dblBins> _y_end_all;
		DvObject_var _xdo;
		int _x_start_obj_id;
		vector <double> _x_start_all;
		int _x_end_obj_id;
		vector <double> _x_end_all;
		QString y_data_start_str;
		QString y_data_end_str;
		QString x_data_start_str;
		QString x_data_end_str;
		int _nx_data;
		int _ny_data;
		bool XYtoggle_in_fill_x;
		bool XYtoggle_in_fill_y;
		int zdo_id_getZRange;
		int ydo_id_getYRange;
		int xdo_id_getXRange;
		
		double x_range_min;
		double x_range_max;
		double x_range_min_z;
		double x_range_max_z;
		double y_range_min_z;
		double y_range_max_z;
		double x_min_data;
		double x_max_data;
		double y_min_data;
		double y_max_data;
		double z_min_data;
		double z_max_data;
};



// -------------- PlotList class -------


class PlotList : public QTreeWidget
{
	Q_OBJECT
		
	public:
		PlotList(QWidget* parent=NULL);
		~PlotList();
				
		QTabWidget *tabWidget;
		
		void plot(bool hidden=false);
		QpuiItem* sameLevelAbove(QpuiItem *item);			
		QpuiItem* sameLevelBelow(QpuiItem *item);			
		PlotItem* itemFromTab; // used for tab to tell list what item it works on
		
		QpTabType I_LastTab;
		
	public slots:
		void plotSelectedItems();
		void setDefaultSelectedItems();
		void updateOnSelection();
		void duplicate();
		void cut();
		void copy();
		void paste();
		void deleteSelectedItems();
		void deleteAllItems();
		bool isAncestorOf(QpuiItem* other);

        // --- SEH ----
        void  fastSelectComponent_x();
        void  fastSelectComponent_y();
        void  fastSelectComponent_z();
        void  fastSelectComponent_r();
		
        void  fastSelect_SC1();
        void  fastSelect_SC2();
        void  fastSelect_SC3();
        void  fastSelect_SC4();
		
        void  fastSelect_DashedLine();
        void  fastSelect_DottedLine();
        void  fastSelect_SolidLine();
        // --- eo SEH ---

		QpuiItem * getItemFromPath(QString &path);

		TraceItem* makeVector_RXYZ_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_XYZ_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_XYZ_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_LP_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_TP_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_X_Y_Z_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_X_Y_Z_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_L_P_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeVector_R_T_P_Item(FrameItem* parent=NULL, QpFrameType frameType=QP_TS);
		TraceItem* makeC1_2_3_4_Item(FrameItem* parent=NULL);
		TraceItem* makeC1234_Item(FrameItem* parent=NULL);
		TraceItem* make2Panel_Item(FrameItem* parent=NULL);
		TraceItem* make3Panel_Item(FrameItem* parent=NULL);
		TraceItem* make4Panel_Item(FrameItem* parent=NULL);
		PanelItem* makePanelItem(QpuiItem* parent=NULL, const QString& WL="");
		PanelItem* getPanelItem(QpuiItem* parent=NULL, const QString& WL="");
		FrameItem* makeTSFrameItem(QpuiItem* parent=NULL);
		FrameItem* makeXYFrameItem(QpuiItem* parent=NULL);
		TraceItem* makeTraceItem(QpuiItem* parent=NULL, const QString& WL="");
		SpectrumItem* makeSpectrumItem(QpuiItem* parent=NULL, const QString& WL="");
		TLabItem* makeTLabItem(QpuiItem* parent=NULL, const QString& WL="");
		EventItem* makeEventItem(QpuiItem* parent=NULL, const QString& WL="");
		void makeConstYTraceItem();
		void makeConstXTraceItem();
		void makeRegLineTraceItem();
		TraceItem* makeConstTrace(PanelItem* parent, const QString& WL, QpItemType plotType, QpFrameType frameType);
		PageItem* makePageItem();
		PageItem* getPageItem(QpuiItem* parent=NULL);
		void makePlotItem(QpuiItem * parent, const QString& obj_name);
		PlotItem* makePlot(PanelItem * parent, QpItemType plotType, QpFrameType frameType, const QString& WL);
		void makeMultiTraceItem(FrameItem* parent, QVector <int> &nTraces, QVector <TraceItem *> &stack, QpFrameType frameType);
	
		void setPageOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setFrameOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setTraceOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setTLabDataOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setEventOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setSpectrumOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setLegendOption(int option, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setAxisOption(int option, int axis=T_AXIS, int iChoice=0, float fChoice=0.0, const QString& sChoice ="", bool bChoice =false, const QColor& cChoice = QColor(0, 0, 0));
		void setYRangeAllSelected();
		void setPlotOption(int option,  bool bChoice =false);
		void renameSelectedItem();
		void setPageLayout(PageItem *pageItem);
		
		
		void displayMenu(const QPoint& p);
		void mousePressEvent(QMouseEvent* event);
		QStringList mimeTypes() const;
		QMimeData* mimeData(const QList<QTreeWidgetItem *> items ) const;
//		bool dropMimeData(QTreeWidgetItem *parent, int index, const QMimeData *data, Qt::DropAction action);
		void dropEvent(QDropEvent *event);

		void setPlotType(QAction *);
		void setFrameType(QAction*);
		
		void currentChanged(const QModelIndex& current, const QModelIndex& previous);
		void openObjectEditor(QTreeWidgetItem *item, int column);
				
		void moveUp();
		void moveDown();
		void deleteItem(QpuiItem* item);
		void linkSelected();
		void unlinkSelected();
		void getLinkFlags(QList<QString>& io_list, QpuiItem* parent);
		void getLinkedItems(QList<PlotItem*>& io_list, QpuiItem* current, const QString& flag);
		void linkPlots(QList<PlotItem*>& linkedItems);
		
		void changePlotType(PlotItem *item, QpItemType plotType);

        QDomElement toXMLElement(QDomDocument&) const;
        void fromXMLElement(QDomElement&, bool _replace);
		
	protected:
		QDomDocument *copyDoc;
		int LinkNumber;
		void linkTraceItems(QVector<TraceItem *>& stack, bool onceOnly);


	signals:
		void raiseWindow();
		
	protected slots:
		void dragEnterEvent(QDragEnterEvent *event);
		void dragMoveEvent(QDragMoveEvent *event);
		void displayIntervalWidget(double x, double y, int pos_x, int pos_y);
		
		
	public:
		QList<PageItem*> _plot_page_list;
		bool isLink;

		QMenu * RButtonMenu;
		QTreeWidgetItem * dropRow;
		QList<QTreeWidgetItem*> copyList;
		bool isCopy;
        QList<PlotWindow*> *plotWindows;
};


// helper routines, not called by user
PLINT tickscaler(PLFLT tstart, PLFLT tend, DvObject_var & data);
PLFLT data_interpolate(PLFLT t, DvObject_var & data);
void qlabel(PLFLT t, PLINT length, PLINT powten, char* label, DvObject_var & data, bool isHr=false);
void qpldtik( PLFLT vmin, PLFLT vmax, PLFLT *tick, PLINT *nsubt, PLBOOL ld );
void qpldtfac( PLFLT vmin, PLFLT vmax, PLFLT *factor, PLFLT *start );

# endif // QSAS_PLOT_UI_H_
