//  qslot.h
//
//  Qsas - drop slot Header File
//
//  Tony Allen


#ifndef QSAS_SL_H_
#define QSAS_SL_H_

#define QSLOT_VERSION  "QSL V_2.1 [ Aug 2007]"

#include <string.h>
#include <vector>

#include <QLineEdit>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qslider.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <QDragEnterEvent>
#include <QMimeData>
#include <QGraphicsItem>
#include <QVBoxLayout>
#include <QDialog>
#include "qt_version.h"

#include "qsas_version.h"
#include <QGroupBox>
#include <QMenu>
#include <QTreeView>
#include <QListWidget>
#include "qsw_helpwindow.h"
#include "qdutil.h"

#define QS_ANY 10

#define QS_WL_OBJ 1
#define QS_ALIAS  2
#define QS_NARROW 3

#define QS_DBL  1
#define QS_INT  2
#define QS_TIME 3
#define QS_TI   4
#define QS_TEXT 5

#define QS_0D     0
#define QS_1D     1
#define QS_2D     2
#define QS_ARRAY  3
#define QS_VECTOR 4

#define QS_SEQ     1
#define QS_NOT_SEQ 2

#define QS_TT     1

#define COL_OK 0
#define COL_NOT_OK 1
#define COL_COMP 2
#define COL_INACTIVE 3
#define COL_INCOMPLETE 4

class QguWLinput;
class RecordSlider;
class timeSlider ;

class Qcondition
{
   
public:
   Qcondition(int objType=QS_WL_OBJ, int dataType=QS_ANY, int dimType=QS_ANY, int seqType=QS_ANY, int ttType=QS_ANY)
   {
		condObj = objType;
		condData = dataType;
		condDim = dimType;
		condSeq = seqType;
		condTT = ttType;
   }
   ~Qcondition(){;}
   
   int condObj;
   int condData;
   int condDim;
   int condSeq;
   int condTT;

};

/// \class QguLineEdit 
/// \brief This is the QguLineEdit class
///
/// This class specialises the QLineEdit widget itself inside QguWLinput to allow us to overload drop and paste
///
/// A line edit allows the user to enter and edit a single line of plain text with a useful collection of editing functions, 
/// including undo and redo, cut and paste, and drag and drop
///
/// It inherits QLineEdit which is a built-in QT3 class
///
/// \remarks { 
/// This class includes Q_OBJECT which is a QT macro that is used in all sub-classes of QObject in order to
/// benefit from the QT's Meta object system which are signals and slots mechanisms for inter-object communication,
/// runtime type information and the dynamic property system }

class QguLineEdit : public QLineEdit
{
   Q_OBJECT
   
public:
   QguLineEdit(QWidget *parent, const char *name);
   ~QguLineEdit();
   void mouseDoubleClickEvent ( QMouseEvent * event );

   QStringList mimeTypes() const;

   QguWLinput *WLinput;
   void setActive(bool);
   bool active;


signals:
   void hadDropEvent();
   
   void gotFocus();
   
   
public slots:

protected slots:
	void focusInEvent(QFocusEvent *);

protected:
   virtual void dropEvent(QDropEvent *e);
//   virtual void focusInEvent(QFocusEvent *);

   QColor activeColor, inactiveColor;
   QColor activeText, inactiveText;

};

class QguWLinput;


class QguWLinput : public QWidget
{
   Q_OBJECT
     
public:
   QguWLinput(QWidget *parent, const char *name, int width=23);
   ~QguWLinput();
   void setDefaultComp(int targetDim, int targetIsSeq, QString &obj_name);
   QString getDefaultComp(int targetDim, int targetIsSeq);
   DvObject_var getObject();
   bool validObject();
   void setColour(int validity);
   void setText(QString txt);
   void setText(DvString txt);
   void setText(const char * txt);
   void setTextKeepComp(const QString& txt);
   bool compatibleComp(DvObject_var &dobj, DvString comp);
//   void setAliasKeepComp(const QString &txt);
   QString text();
   QString textObject();
   double value();
   void setDouble(double value);
   void setCursorPosition(int posn);
   void emitCmpSelected(const char *txt, const char *typ){ emit componentSelected(txt, typ);}
   void setAlignment(Qt::AlignmentFlag flag);

   void setMaxLength(int len);
   void setReadOnly(bool readOnly);
   void clear();
   bool hasFocus();
   void clearFocus();
   void setEnabled(bool enabled);
   dimTripleStack parseArrayReduce();
   dimTriple parseTimeSlice();
   QguComp_e getComponent();
   bool hasComponent();
   bool reValidate(bool propagate=false);
   bool validateCondition(const QString &txt, Qcondition &cond);
   void set3DSubsetDef(const char *setting, bool sliceTime=false);
   QString get3DSubsetDef(DvObject_var &dobj);
   
   QMenu *getDefaultPopup(int seqChoice, bool event = false);
   QMenu *getTrangePopup();
   QMenu *getTimePopup();
   QMenu *getTimeFromELPopup();
   QMenu *getTivlFromEvtTablePopup();
   QMenu *getTivlSeqSlicePopup();
   QMenu *getVectorPopup(DvObject_var dobj=DvObject_var());
   QMenu *getArrayPopup();
   void  setDropdown(QMenu *dropdownChoice);
      
   QToolButton * dropDownArray;
   QguLineEdit *lineBox;
   QMenu * defaultPopup;
   QMenu * TrangePopup;
   QMenu * TimePopup;
   QMenu * TimeFromELPopup;
   QMenu * TivlFromEvtTablePopup;
   QMenu * TivlSlicePopup;
   QMenu * vectorPopup;
   QMenu * arrayPopup;
   QMenu * activeMenu;

   bool active;   ///< Public flag set to signify that this QguWLinput is active (accepts input)
   bool takesAlias;
   string rtpDefault;
   
	vector <Qcondition> acceptConditions; // vector of conditions that would be acceptable
	void setAcceptable(Qcondition newCondition)
	{
		acceptConditions.clear();
		acceptConditions.push_back(newCondition);
	}
	void addAcceptable(Qcondition newCondition)
	{
		acceptConditions.push_back(newCondition);
	}
  
   int with;    ///< Public member defining how the list view is being saved (saved with/without data)
   QString ObjectName; ///< The WL context for the valid object or the alias name
   QString ObjectName_nil; ///< nil string used when input did not validate OK
   DvObject_var nilDVObject; ///< nil object for use when invalid slot input text so that a .is_nil() object is returned
   QPalette *WLI_palette;
   
   DvObject_var DVOS_cache;
   
   bool isValid() const;
   QGraphicsItem *graphicsParent;
   QWidget *widgetParent;
   
signals:
   /// \fn void hadDropEvent()
   /// \brief signal emitted drop received
   /// \param    None
   /// \exception None
   /// \return   None
   ///          
   void hadDropEvent();

   /// \fn void validated_alias()
   /// \brief Function signal that is emitted when entered text is alias on calculator list
   /// \param     text validated
   /// \exception None
   /// \return    None
   ///  
   void validated_alias();
   
   /// \fn void edited_OK()
   /// \brief Function signal that is emitted when entered text is valid for the slot
   /// \param     text validated
   /// \exception None
   /// \return    None
   ///  
   void edited_OK();

   void slotEmpty(); // used to reset after slot emptied
   
   /// \fn void componentSelected(const char *, const char *)
   /// \brief signal emitted when dropdown component menu activated for time or vector
   /// \param     component of type const char *, flag of type const char * = "VECTOR", "ARRAY" or "TIME"
   /// \exception None
   /// \return   None
   ///    
   void componentSelected(const char *, const char *);
  
   /// \fn void TimeChosen()
   /// \brief signal emitted when dropdown component menu selected a time
   /// \param     None
   /// \exception None
   /// \return   None
   ///   
   void TimeChosen();
   
   void tryAlias(bool);

   void validated_WL_OBJ(const QString &);
   
	void dropDetected(const char*);

public slots:
   void setDropdownText_event();
   void setDropdownText_r();
   void setDropdownText_x();
   void setDropdownText_y();
   void setDropdownText_z();
   void setDropdownText_ld();
   void setDropdownText_td();
   void setDropdownText_pd();
   void setDropdownText_lr();
   void setDropdownText_tr();
   void setDropdownText_pr();
   void setDropdownText_lt();
   void setDropdownText_dobj();
   void openArrayReduce();
   void openSliceT_seq();
   void openSliceT_not_seq();
   void openSliceS();
   void openSliceS_seq();
   void openSliceS_not_seq();
   void openSliceV();
   void openSelectMetadata();
   void setDropdownTstart();
   void setDropdownTcentre();
   void setDropdownTend();
   void setTimeDropdownDefault();
   void setETDropdownDefault();
   void setVectorDropdownDefault();
   void setArrayDropdownDefault();
   void resetColour();
   void selectTivlSeqSubset();
   void selectTivlEvent();
   void selectEventAndTime();
   void dropEvent(QDropEvent *e);
   void validateObjChanged(const QString&);
   void validateDeletion(const QString&);
   void validatePathChanged(const QString&, const QString&);
   void validateInput(const QString&);
   void resetCache();
   void dropdownClicked(bool);

   
protected:
 //  virtual void mouseDoubleClickEvent(QMouseEvent *e);
    int _validity;
};



//class indexSlider : public QGroupBox {
class indexSlider : public QWidget {
  
Q_OBJECT

public:
  indexSlider( int atNum, QString title, int maxN, QWidget *parent=0, const char * name=0 );
  ~indexSlider();
  
  QguWLinput *numField;
  QLabel * echoLabel;
  int num;
  int maxNum;
  
  void setLabel(const QString& label){echoLabel->setText(label);}
  
public slots:
  void decRec();
  void incRec();
  void adjustRec();
  
signals:
   void indexChanged();
   
protected:
 //  QGroupBox *index_slider_groupbox;
};


class rangeSlider : public QGroupBox {
  
Q_OBJECT

public:
  rangeSlider(int idx, int MaxN, DvObject_var &depend_i, QWidget *parent=0, char * name=0 );
  ~rangeSlider();
  
  indexSlider * fromSlide;
  indexSlider * toSlide;
  QLabel *fromLabel;
  QLabel *toLabel;
  vector <DvString> binTextSeq;
  QComboBox *actionChoice;
  
public slots:
  void updateOnIdx();
  void updateRangeWidget(int);
  
signals:
  
   
protected:
  
};


class ArrayReduceWindow : public QDialog {

Q_OBJECT

  QMenuBar *_menu_bar;
  QswHelpWindow *arrHelp_window;
  QPushButton *dismissBtn;
  
public slots:
  void launchVersion();
  void launchManArrayReduce();
  void launchManual();
  void setDefaults();
  void setToLastChoice();
  void dismiss();
  void acceptSelection();
  void updateFeedback();
    
signals:

public:
    
  ArrayReduceWindow(QString baseName, vector <size_t> &arrayDim, QWidget *parent=0, const char *name=0, bool modal=false ); 

  ~ArrayReduceWindow(){;};
  QguWLinput *parentSlot; 
  vector <rangeSlider *> sliderStack;
  bool isSequence;
#if QT_VERSION < 0x040000
  QHGroupBox *buttons_layout;
#else
  QGroupBox * buttons_groupbox;
#endif
  timeSlider *slicer;
  
protected:
  
};


class TimeSliceWindow : public QDialog {

Q_OBJECT

  QMenuBar *_menu_bar;
  
public slots:
  void setDefaults();
  void setToLastChoice();
  void dismiss();
  void acceptSelection();
  void updateFeedback();
    
signals:

public:
    
  TimeSliceWindow(QString baseName, const char * source,QWidget *parent=0, 
		  const char *name=0,bool acceptTSubset=false, bool acceptTSlice=false); 

  ~TimeSliceWindow(){;};
  QguWLinput *parentSlot;   
#if QT_VERSION < 0x040000 
  QHGroupBox *buttons_layout;
#else
  QGroupBox *buttons_groupbox;
#endif
  QComboBox  *vectorChoice;
  timeSlider *slicer;
  
protected:
  
};

class TimeIntervalSlider : public QGroupBox {

    Q_OBJECT

    public:
        TimeIntervalSlider(const QString & title, QWidget * parent=0, const char * name =0);
        void updateTitle();
  
        QSlider *recBar;
        QguWLinput *recField;
        int record;
        int maxRecord;
        DvObject_var obj;
        QString titleBase;
  
    public slots:
        void incRec();
        void decRec();
        void lastRec();
        void firstRec();
        void adjustRec(int);
        void setFromTyping();
  
    signals:
        void sliderChanged();
};

class SelectETWindow : public QDialog {

    Q_OBJECT

  
    public slots:
        void validated();
        void cancelled();
    
    signals:

    public:
    
        SelectETWindow(QString baseName, QWidget *parent=0, 
                        const char *name=0,
                        bool modal=false ); 

        SelectETWindow(){;};
        QguWLinput *parentSlot;
        QPushButton * validateBtn;
        QPushButton * cancelBtn;
		QComboBox *pullDown;
        TimeIntervalSlider * slider;
        int record;
        int max_rec;
  
    protected:
  
};

class SelectTivlWindow : public QDialog {

    Q_OBJECT

  
    public slots:
        void validated();
        void cancelled();
    
    signals:

    public:
    
        SelectTivlWindow(QString baseName, QWidget *parent=0, 
                        const char *name=0,
                        bool modal=false ); 

        ~SelectTivlWindow(){;};
        QguWLinput *parentSlot;
        QPushButton * validateBtn;
        QPushButton * cancelBtn;
        TimeIntervalSlider * slider;
        int record;
        int max_rec;
  
    protected:
  
};

class SelectTivlSubsetWindow : public QDialog {

    Q_OBJECT

//             QMenuBar *_menu_bar;
  
    public slots:
        void validated();
        void cancelled();
        
    public:
        int record_start, record_end;
        int max_rec;
    
    signals:

    public:
    
        SelectTivlSubsetWindow(QString baseName, QWidget *parent=0, 
                         const char *name=0,
                         bool modal=false ); 

        ~SelectTivlSubsetWindow(){;};
        QguWLinput *parentSlot;
        TimeIntervalSlider * slider1;
        TimeIntervalSlider * slider2;
        QPushButton * validateBtn;
        QPushButton * cancelBtn;

  
    protected:
  
};


class selectMetadataWindow : public QMainWindow{

Q_OBJECT

  QswHelpWindow *metaHelp_window;
  QPushButton *dismissBtn;
  
public slots:
  void selectAndClose();
  void dismiss();
  void launchManual();
    
signals:

public:
    
  selectMetadataWindow(DvObject_var &dobj,  QWidget *parent=0); 

  ~selectMetadataWindow();
  QListWidget * xrefList;
  QguWLinput *parentSlot; 
    
protected:
  
};


/// \class DataSliceWindow qdutil.h
/// \brief This is the DataSliceWindow class
///
/// This class specialises the QMainWindow class which provies a main application window.
///
/// Main windows are most often used to provide menus, toolbars and a status bar around a large central widget.
///
/// It inherits QMainWindow which is a built-in QT3 class
///
/// \remarks {
/// This class includes Q_OBJECT which is a QT3 macro that is used in all sub-classes of QObject in order to
/// benefit from the QT's Meta object system which are signals and slots mechanisms for inter-object communication,
/// runtime type information and the dynamic property system }

class DataSliceWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  DataSliceWindow( QWidget *parent=0);
  /// \fn ~DataSliceWindow()
  /// \brief Destructor of the DataSliceWindow class
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  virtual ~DataSliceWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QComboBox *typeField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptSlice();
  void setUserInput();
  void setNewName();
  void setAccept();
  void dismiss();
  void setType(int);

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};


class RecordSlider : public QGroupBox {

Q_OBJECT

public:
  RecordSlider(const QString & title, QWidget * parent=0, const char * name =0);
  void updateTitle(int rec);
  void updateTitleEnd();
  void setTime(DvTime tobj);
  void setValue(double val);
  bool valueUnchanged();
  void setAcceptScalar();
  
  QSlider *recBar;
  QguWLinput *recField;
  QguWLinput *valueField;
  
  int record;
  int maxRecord;
  DvObject_var ttags;
  QString titleBase;
  
public slots:
  void incRec();
  void decRec();
  void lastRec();
  void firstRec();
  void adjustRec(int);
  void setFromTyping();
  void setFromValueFld();
  void sliderChanged(int);
  void setRecText(int rec);
  

signals:
  void recordSliderChanged();
};

class timeSlider : public QGroupBox {

Q_OBJECT

public:
  timeSlider(DvObject_var &dobj, bool acceptTSlice, bool acceptTSubset, QWidget *parent=0, char * name=0);
  virtual ~timeSlider(){;};
  bool isDefault();
  void dragEnterEvent(QDragEnterEvent *event);
  void dropEvent(QDropEvent * event);
  
  QComboBox *sliceChoice;
  RecordSlider *fromSlider;
  RecordSlider *toSlider;
  RecordSlider *atSlider;
  QVBoxLayout * mainLayout;
  int Inothing;
  int Isubset;
  int Iaverage;
  int Islice;
  DvObject_var tt;
  
public slots:
  void updateTimeSlider(int opChoice);

  
};


# endif // QSAS_SL_H_
