#include "qcalcEvaluator.h"


void QEvaluatorVectProd::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	// do arithmetic...

	result = val1->vec(val2);
	
	if(result.is_nil())
	{
		message=QString("Arithmetic>> Not valid for these input types");
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}

// ---------------------

void QEvaluatorDiff3::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var result;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	if( val2->totalSize() > 1 )
	{
		message=QString("Gap input is not a scalar value");
		ok=false;
		return; 
	}
	double gapSize = val2->asDouble();	

	// do arithmetic...

	// 3 point estimate	

	result = val1->differentiate(DV_DERIV_3PT, gapSize);

	if(result.is_nil())
	{
		message=QString("Differentiation>> Not valid for these input types");
		ok=false;
		return;
	}
	
	out[0]->setValue(result, true);
}

// ----------------------------------


void QEvaluatorDiff5::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
    ok=true;
    DvObject_var result;
    
    QEvaluator::evaluate(in, out, ok, message);
    if(!ok)
        return;
    
    DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
    if(!ok)
    {
        message=QString("Input ")+in[0]->getName()+QString(" invalid");
        ok=false;
        return;
    }
    DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
    if(!ok)
    {
        message=QString("Input ")+in[1]->getName()+QString(" invalid");
        ok=false;
        return;
    }
    
    if( val2->totalSize() > 1 )
    {
        message=QString("Gap input is not a scalar value");
        ok=false;
        return; 
    }
    double gapSize = val2->asDouble();	
    
    // Do derivative
    
    result = val1->differentiate(DV_DERIV_5PT, gapSize);
    
    if(result.is_nil())
    {
        message=QString("Differentiation>> Not valid for these input types");
        ok=false;
        return;
    }
    
    out[0]->setValue(result, true);
}

// ----------------------------------


void QEvaluatorCreateReg::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
    ok=true;
    DvObject_var result;
    
    QEvaluator::evaluate(in, out, ok, message);
    if(!ok)
        return;
    
    DvString name = out[0]->getName().toLatin1().data();
    
    
    DvObject_var val1=in[0]->connectedSlot()->getValue(ok);
    if(!ok)
    {
        message=QString("Input ")+in[0]->getName()+QString(" invalid");
        ok=false;
        return;
    }
    DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
    if(!ok)
    {
        message=QString("Input ")+in[1]->getName()+QString(" invalid");
        ok=false;
        return;
    }
    DvObject_var val3=in[2]->connectedSlot()->getValue(ok);
    if(!ok)
    {
        message=QString("Input ")+in[2]->getName()+QString(" invalid");
        ok=false;
        return;
    }
    
   
    int n = val1->asInt();	
    
    if(name.contains("_T")){
        DvTime start = val2->asTime();
        double spacing = val3->asDouble();
        
        result = new DvObject(start, (size_t)n);
        result->assign_regular(spacing);
        result->change_xref(SI_CONVERSION, "1>s");
    }
    else if(name.contains("_D")){
        double start = val2->asDouble();
        double spacing = val3->asDouble();
        
        result = new DvObject(start, (size_t)n);
        result->assign_regular(spacing);
    }
    else{
        int start = val2->asInt();
        int spacing = val3->asInt();
        
        result = new DvObject(start, (size_t)n);
        result->assign_regular(spacing);
    }
            
    if(result.is_nil())
    {
        message=QString("Create Regular>> Not valid for these inputs");
        ok=false;
        return;
    }
    
    out[0]->setValue(result, true);
}

// ---------------------

void QEvaluatorIntDef::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var retObj;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	DvObject_var dobj=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	if( val2->seqSize() > 1 )
	{
		message=QString("Method input is not a scalar value");
		ok=false;
		return; 
	}
	int gapMethod = val2->asInt();	
	DvString gapStr = DV_REMOVE;
	if(gapMethod == 0) gapStr = DV_NN;
	else if(gapMethod == 1) gapStr = DV_LINEAR;

	// Do integral
	
	retObj = dobj->integrate(gapStr, DV_DEF_INTEGRAL);

	if( retObj.is_nil() ) {
		message=QString("NULL object from integral");
		ok=false;
		return;
	}
	
	out[0]->setValue(retObj, true);
}

// ---------------------

void QEvaluatorIntIndef::evaluate(QVector<VInSlot*>& in, QVector<VOutSlot*>& out, bool& ok, QString& message)
{
	ok=true;
	DvObject_var retObj;
	
	QEvaluator::evaluate(in, out, ok, message);
	if(!ok)
		return;
	
	DvObject_var dobj=in[0]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[0]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	DvObject_var val2=in[1]->connectedSlot()->getValue(ok);
	if(!ok)
	{
		message=QString("Input ")+in[1]->getName()+QString(" invalid");
		ok=false;
		return;
	}
	
	if( val2->seqSize() > 1 )
	{
		message=QString("Method input is not a scalar value");
		ok=false;
		return; 
	}
	int gapMethod = val2->asInt();	
	DvString gapStr = DV_REMOVE;
	if(gapMethod == 0) gapStr = DV_NN;
	else if(gapMethod == 1) gapStr = DV_LINEAR;
	
	// Do integral
	
	retObj = dobj->integrate(gapStr, DV_INDEF_INTEGRAL);

	if( retObj.is_nil() ) {
		message=QString("NULL object from integral");
		ok=false;
		return;
	}
	
	out[0]->setValue(retObj, true);
}



