#include "qcalcGraphicElements.h"
#include "qcalcCalcItem.h"
#include "qcalcSlots.h"
// #include "qcalcTimeSlots.h"
#include "qcalcOperatorItem.h"
#include <QGraphicsScene>
#include <cmath>
#include "qcalcScene.h"

CalcToolTip::CalcToolTip(CalcItem* parent):
	QGraphicsRectItem(parent)
{
	setRect(-BASE_WIDTH/2., -BASE_HEIGHT/2., BASE_WIDTH, BASE_HEIGHT);
}

void CalcToolTip::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	QColor color1;
	QColor color2;

	color2=QColor(255,255,0,100);
	color1=QColor(150,150,0);

	painter->setRenderHints(QPainter::Antialiasing);
	
	QBrush brush(color2);
	QPen pen(color1);
	pen.setWidthF(1.);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRoundRect(boundingRect(), 15, 15);
	
	painter->drawText(boundingRect(),Qt::AlignHCenter|Qt::AlignVCenter|Qt::TextWordWrap, " Select a component:\n Calculation will continue");
	
}

Connector::Connector(OutSlot* from, InSlot* to, const QPointF& toPos, QGraphicsItem* parent):
		QGraphicsLineItem(parent)
{
	if(from==NULL)
	{
		std::cerr << "Connector constructor err: null InSlot"<< std::endl;
		return;
	}
	this->from=from;
	from->addConnector(this);
	fromPos=mapFromItem(from, 0., 0.);
	if(to!=NULL)
	{
		to->setConnector(this);
		this->to=to;
		this->toPos=mapFromItem(to, 0., 0.);
	}
	else
	{
		this->to=NULL;
		this->toPos=toPos;
	}
	
	setFlag(QGraphicsItem::ItemIsMovable, false);
	setFlag(QGraphicsItem::ItemIsSelectable, false);
	setFlag(QGraphicsItem::ItemIsFocusable, false);
	
	setZValue(2.);
	
	page=CalcScene::currentPage;
	
}

Connector::~Connector()
{
	from->removeConnector(this);
	if(to!=NULL)
	{
		to->removeConnector(this);
	}
	scene()->removeItem(this);
}

QRectF Connector::boundingRect() const
{
	return QRectF(line().p1(), QSizeF(line().p2().x() - line().p1().x(),
		      line().p2().y() - line().p1().y()))
			.normalized()
			.adjusted(-5., -5., 5., 5.);

}

QPainterPath Connector::shape() const
{
	QPainterPath path=QGraphicsLineItem::shape();
	path.addPolygon(arrowHead);
	return path;
}

void Connector::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	painter->setRenderHints(QPainter::Antialiasing);
	QPen pen;
	if(to!=NULL)
	{
		pen.setStyle(Qt::SolidLine);
	}
	else
	{
		pen.setStyle(Qt::DotLine);
	}
	pen.setWidthF(1.);
	pen.setColor(Qt::black);
	QBrush b;
	b.setColor(Qt::black);
	b.setStyle(Qt::SolidPattern);
	painter->setPen(pen);
	painter->setBrush(b);
	painter->drawLine(line());
	
	arrowHead.clear();
	QPointF tip;
	if(to!=NULL)
	{
		tip=mapFromItem(to, 0., 0.);
	}
	else
	{
		tip=toPos;
	}
	QPointF tail=mapFromItem(from, 0., 0.);
	double angle;
	if(tip.x()!=tail.x())
	{
		angle=atan((tip.y()-tail.y())/(tip.x()-tail.x()));
	}
	else
	{
		if(tip.y()<tail.y())
		{
			angle=-M_PI/2.;
		}
		else angle=M_PI/2.;
	}
	double fact=1.;
	if(tip.x() < tail.x()) fact=-1.;
	QPointF p1(tip.x()-fact*10.*cos(angle+0.5), tip.y()-fact*10.*sin(angle+0.5));
	QPointF p2(tip.x()-fact*10.*cos(angle-0.5), tip.y()-fact*10.*sin(angle-0.5));
	QPointF p3(tip.x()-fact*5.*cos(angle), tip.y()-fact*5.*sin(angle));
	arrowHead << tip << p1 << p3 << p2;
	
	painter->drawPolygon(arrowHead);
}

void Connector::updatePosition()
{
	fromPos=mapFromItem(from, 0., 0.);
	if(to!=NULL)
	{
		this->toPos=mapFromItem(to, 0., 0.);
	}
	setLine(QLineF(fromPos, toPos));
}

void Connector::setEnd(const QPointF& toPos)
{
	if(to!=NULL) // removes existing connector
	{
		to->removeConnector(this);
	}
	to=NULL;
	this->toPos=toPos;
	setLine(QLineF(fromPos, toPos));
}

void Connector::setEnd(InSlot* to)
{
	// First check if this does not close a loop
	
	if(this->to!=to && this->to!=NULL)
	{
		this->to->removeConnector(this);
	}
	this->to=to;
	to->setConnector(this);
	toPos=mapFromItem(to, 0., 0.);
	setLine(QLineF(fromPos, toPos));
}

RotoZoomButton::RotoZoomButton(QGraphicsItem* parent):
		QGraphicsRectItem(parent)
{
	setRect(-HEADER_HEIGHT/2.+1., -HEADER_HEIGHT/2.+1., HEADER_HEIGHT-1., HEADER_HEIGHT-1.);
	hasMouseFocus=false;
#if QT_VERSION < 0x040400
	setAcceptsHoverEvents(true);
#else
	setAcceptHoverEvents(true);
#endif
}

void RotoZoomButton::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	QColor color1;
	QColor color2;
	if(hasMouseFocus)
	{
		color2=((CalcItem*)parentItem())->color1;
		color1=((CalcItem*)parentItem())->outlineColor;
	}
	else
	{
		color1=((CalcItem*)parentItem())->color1;
		color2=((CalcItem*)parentItem())->outlineColor;
	}
	painter->setRenderHints(QPainter::Antialiasing);
	
	QBrush brush(color2);
	QPen pen(color2);
	pen.setWidthF(1.);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRoundRect(boundingRect(), 25, 25);
	
	pen.setColor(color1);
	painter->setPen(pen);
	brush.setStyle(Qt::NoBrush);
	painter->setBrush(brush);

	painter->drawEllipse(QRectF(-3, -3, 6, 6));
	painter->drawArc(QRectF(-6, -6, 12, 12), 30*16, 210*16);
	
	painter->drawLine((int)(6.5*cos(30./180.*M_PI)), (int)(-6.5*sin(30./180.*M_PI)), (int)(5*cos(40./180.*M_PI)), (int)(-5*sin(40./180.*M_PI)));
	painter->drawLine((int)(6.5*cos(30./180.*M_PI)), (int)(-6.5*sin(30./180.*M_PI)), (int)(8*cos(40./180.*M_PI)), (int)(-8*sin(40./180.*M_PI)));
	
	painter->drawLine((int)(6.5*cos(240./180.*M_PI)), (int)(-6.5*sin(240./180.*M_PI)), (int)(5*cos(230./180.*M_PI)), (int)(-5*sin(230./180.*M_PI)));
	painter->drawLine((int)(6.5*cos(240./180.*M_PI)), (int)(-6.5*sin(240./180.*M_PI)), (int)(8*cos(230./180.*M_PI)), (int)(-8*sin(230./180.*M_PI)));

	brush.setStyle(Qt::SolidPattern);
	brush.setColor(color2);
	painter->setBrush(brush);
	QPointF points[4] = {
		QPointF(3.*sqrt(2.)/2.-0.5, 3.*sqrt(2.)/2.+0.5),
		QPointF(5.5, 6.5),
		QPointF(6.5, 5.5),
		QPointF(3.*sqrt(2.)/2.+0.5, 3.*sqrt(2.)/2.-0.5)
	};
	painter->drawConvexPolygon(points, 4);
}

void RotoZoomButton::hoverEnterEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=true;
	update();
}

void RotoZoomButton::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=false;
	update();
}

CalcItem* RotoZoomButton::getItem()
{
	return (CalcItem*)parentItem();
}


CloseButton::CloseButton(QGraphicsItem* parent):
		QGraphicsRectItem(parent)
{
	setRect(-HEADER_HEIGHT/2.+1., -HEADER_HEIGHT/2.+1., HEADER_HEIGHT-1., HEADER_HEIGHT-1.);
	hasMouseFocus=false;
#if QT_VERSION < 0x040400
	setAcceptsHoverEvents(true);
#else
	setAcceptHoverEvents(true);
#endif
}

void CloseButton::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	QColor color1;
	QColor color2;
	if(hasMouseFocus)
	{
		color2=((CalcItem*)parentItem())->color1;
		color1=((CalcItem*)parentItem())->outlineColor;
	}
	else
	{
		color1=((CalcItem*)parentItem())->color1;
		color2=((CalcItem*)parentItem())->outlineColor;
	}
	painter->setRenderHints(QPainter::Antialiasing);
	
	QBrush brush(color2);
	QPen pen(color2);
	pen.setWidthF(1.);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRoundRect(boundingRect(), 25, 25);
	
	pen.setColor(color1);
	painter->setPen(pen);
	painter->drawLine(QPointF(-HEADER_HEIGHT/4., -HEADER_HEIGHT/4.), QPointF(HEADER_HEIGHT/4., HEADER_HEIGHT/4.));
	painter->drawLine(QPointF(HEADER_HEIGHT/4., -HEADER_HEIGHT/4.), QPointF(-HEADER_HEIGHT/4., HEADER_HEIGHT/4.));
	
}

void CloseButton::hoverEnterEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=true;
	update();
}/**/

void CloseButton::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=false;
	update();
}

CalcItem* CloseButton::getItem()
{
	return (CalcItem*)parentItem();
}

ExpandButton::ExpandButton(QGraphicsItem* parent):
		QGraphicsRectItem(parent)
{
	setRect(-HEADER_HEIGHT/2.+1., -HEADER_HEIGHT/2.+1., HEADER_HEIGHT-1., HEADER_HEIGHT-1.);
	hasMouseFocus=false;
#if QT_VERSION < 0x040400
	setAcceptsHoverEvents(true);
#else
	setAcceptHoverEvents(true);
#endif
}

void ExpandButton::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	
	QColor color1;
	QColor color2;
	if(hasMouseFocus)
	{
		color2=((CalcItem*)parentItem())->color1;
		color1=((CalcItem*)parentItem())->outlineColor;
	}
	else
	{
		color1=((CalcItem*)parentItem())->color1;
		color2=((CalcItem*)parentItem())->outlineColor;
	}
	painter->setRenderHints(QPainter::Antialiasing);
	
	QBrush brush(color2);
	QPen pen(color2);
	pen.setWidthF(1.);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRoundRect(boundingRect(), 25, 25);
	
	pen.setColor(color1);
	painter->setPen(pen);
	painter->drawLine(QPointF(-HEADER_HEIGHT/4., 0.), QPointF(HEADER_HEIGHT/4., 0.));
	
	if(!((OperatorItem*)parentItem())->isExpanded())
	{
		painter->drawLine(QPointF(0., -HEADER_HEIGHT/4.), QPointF(0., HEADER_HEIGHT/4.));
	}
}

void ExpandButton::hoverEnterEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=true;
	update();
}

void ExpandButton::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
	hasMouseFocus=false;
	update();
}

CalcItem* ExpandButton::getItem()
{
	return (CalcItem*)parentItem();
}


////////////////////////////////////////////////////////////////

ErrorButton::ErrorButton(CalcItem* parent):
		QGraphicsRectItem(parent)
{
	setRect(-HEADER_HEIGHT/2.+1., -HEADER_HEIGHT/2.+1., HEADER_HEIGHT-1., HEADER_HEIGHT-1.);
}

void ErrorButton::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
	
	QColor color1=Qt::white;
	QColor color2=Qt::red;

	painter->setRenderHints(QPainter::Antialiasing);
	
	QBrush brush(color2);
	QPen pen(color2);
	pen.setWidthF(1.);
	painter->setPen(pen);
	painter->setBrush(brush);
	painter->drawRoundRect(boundingRect(), 25, 25);
	
	pen.setColor(color1);
	painter->setPen(pen);
	QPolygonF poly;
	poly << QPointF(0., -HEADER_HEIGHT/3.) << QPointF(-HEADER_HEIGHT/3., HEADER_HEIGHT/3.) << QPointF(HEADER_HEIGHT/3., HEADER_HEIGHT/3.);
	painter->drawPolygon(poly);
	
	painter->drawLine(0, (int)(-HEADER_HEIGHT/6), 0, (int)(HEADER_HEIGHT/6));
	painter->drawPoint(0, (int)(HEADER_HEIGHT/6+2));
}

CalcItem* ErrorButton::getItem()
{
	return (CalcItem*)parentItem();
}

