#include "qcalcInputItem.h"
#include "qcalcSlots.h"
#include "qcalcScene.h"
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneDragDropEvent>
#include "qdutil.h"

using namespace QSAS;
extern TimeLineHandler timeLineHandler;

QVector<VInputItem*> InputItems;

VInputItem::VInputItem():
		VCalcItem()
{
	VOutSlot* slot=new VOutSlot(this);
	outputs.push_back(slot);
	joinSettings=new JoinSettings;
    not_joined_value = new DvObject();
	value = new DvObject();
	gui=NULL;
	input_id=counter++;
	name=QString("Input #%1").arg(input_id);

    joinSettings->noJoin=true;
	joinSettings->algorithm=0;
	joinSettings->width=60.;
	joinSettings->gap=0;
    joinSettings->gapSize=6.;
    joinSettings->end=0;
    joinSettings->minBoxSize=3;
    valueText="";
	
	InputItems.append(this);
}

VInputItem::~VInputItem()
{
	if(gui!=NULL)
	{
		QGraphicsScene * scene=gui->scene();
		if(scene!=NULL)
		{
			scene->removeItem(gui);
		}
		gui->parentItem=NULL; // thus gui knows it is being deleted
		
		delete gui;
	}
	
	int index=InputItems.indexOf(this);
	if(index>=0)
		InputItems.remove(index);
	
	delete joinSettings;
}


QDomElement VInputItem::toXMLElement(QDomDocument& document) const
{
	QDomElement tag=document.createElement("Input");
    if(gui!=NULL)
    {
		tag.appendChild(gui->toXMLElement(document));
    }
	tag.appendChild(createTextXMLNode(document, "XPosnInBlackBox", positionInBlackBox.x()));
	tag.appendChild(createTextXMLNode(document, "YPosnInBlackBox", positionInBlackBox.y()));
	tag.appendChild(createTextXMLNode(document, "ID", calc_id));

    tag.appendChild(createTextXMLNode(document, "Data", valueText));
	tag.appendChild(createTextXMLNode(document, "Page", page));
	tag.appendChild(createTextXMLNode(document, "Join_No_Join", joinSettings->noJoin));
	tag.appendChild(createTextXMLNode(document, "Join_Algorithm", joinSettings->algorithm));
	tag.appendChild(createTextXMLNode(document, "Join_Width", joinSettings->width));
	tag.appendChild(createTextXMLNode(document, "Join_Gap", joinSettings->gap));
    tag.appendChild(createTextXMLNode(document, "Join_Gap_Size", joinSettings->gapSize));
    tag.appendChild(createTextXMLNode(document, "Join_End", joinSettings->end));
    tag.appendChild(createTextXMLNode(document, "Join_Min_Box", joinSettings->minBoxSize));

    return tag;
}


void VInputItem::setTags(DvObject_var tt)
{
	tags = tt;
}


void VInputItem::reevaluateOutSlots()
{
   
    if(not_joined_value.is_nil()){
		outputs[0]->setValue(not_joined_value, false);
		return;
	}
    if(joinSettings->noJoin)
    {	
        value=not_joined_value;
    }
    else
    {
		QplugAppendTextDisplay( "Joining...\n ");
		if(not_joined_value->seqSize() <= 1 )
        {
            value=not_joined_value;
        }
        else
        {
			switch(joinSettings->gap)
			{
				case 1:
				{
					DvString gap_opt_ptr(DV_ZERO_FILL);
					not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);	
					break;
				}
				case 2:
				{
					DvString gap_opt_ptr(DV_FILL);
					not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
					break;
				}
				case 3:
				{
					DvString gap_opt_ptr(DV_NN);
					not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
					break;
				}
				case 4:
				{
					DvString gap_opt_ptr(DV_REMOVE);
					not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
					break;
				}
				case 0:
				default:
				{
					DvString gap_opt_ptr(DV_LINEAR);
					not_joined_value->change_xref(DV_GAP_OPTION, gap_opt_ptr);
					break;
				}
			}
            
            switch(joinSettings->end)
            {
                case 1:
                {
                    DvString end_opt_ptr(DV_ZERO_FILL);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 2:
                {
                    DvString end_opt_ptr(DV_FILL);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 3:
                {
                    DvString end_opt_ptr(DV_NN);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 4:
                {
                    DvString end_opt_ptr(DV_REMOVE);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
                case 0:
                default:
                {
                    DvString end_opt_ptr(DV_LINEAR);
                    not_joined_value->change_xref(DV_END_OPTION, end_opt_ptr);
                    break;
                }
            }
			
				
			if(joinSettings->algorithm == 0){
				double gap=joinSettings->gapSize;
				if(gap > 1.e-20) not_joined_value->change_xref(DV_GAP_WIDTH, gap);
				value = not_joined_value->linearJoin(*tags);
			}
			else {
				double boxcar_width=joinSettings->width;
				not_joined_value->change_xref(DV_BOX_WIDTH, boxcar_width);
                int min_box = joinSettings->minBoxSize;
                not_joined_value->change_xref(DV_MIN_BOXCAR, min_box);
                value = not_joined_value->boxcarJoin(*tags);
                
			}				
        
            if(value.is_nil() || value->seqSize() == 0){
				QplugAppendTextDisplay( "Input failed on join: Check Timeline is valid");
                if(value->nError() > 0){
                    for(size_t n=0; n<value->nError(); n++) QplugAppendTextDisplay(value->getError(n).c_str());
                }
                
                outputs[0]->setValue(value, false);
                return;
            }
			
			
        }
    }
	
	outputs[0]->setValue(value, true);

    InputItem *gui = this->getGUI();
    DvString newTitle = this->getName().toLatin1().data();
    
    newTitle = newTitle.front("(");
    if(value->is_dbl()) newTitle += "(dbl)";
    else if (value->is_int()) newTitle += "(int)";
    else if (value->is_str()) newTitle += "(str)";
    else if (value->is_time()) newTitle += "(time)";
    else if (value->is_event()) newTitle += "(event)";
    gui->setTitle(newTitle.c_str());
    setName(newTitle.c_str());
    
}



InputItem* VInputItem::getGUI()
{
	if(gui==NULL)
	{
		gui=new InputItem(this, outputs[0]);
	}
	return gui;
}

void VInputItem::removeGUI()
{
	if(gui!=NULL)
	{
		gui->scene()->removeItem(gui);
		delete gui;
		gui=NULL;
	}
}



void VInputItem::fromXMLElement(QDomElement& el, QGraphicsScene* scene)
{
	VCalcItem::fromXMLElement(el, scene);


	QDomElement guiElement=el.firstChildElement("GUI");
	QString data="";
	QString val;

	if(getXMLNodeText(el, "XPosnInBlackBox", val))
	{
		positionInBlackBox.setX(val.toDouble());
	}
	if(getXMLNodeText(el, "YPosnInBlackBox", val))
	{
		positionInBlackBox.setY(val.toDouble());
	}

	if(getXMLNodeText(el, "Data", val))
	{
		data=val;
	}
	if(getXMLNodeText(el, "Page", val))
	{
		page=val.toInt();
	}
	if(getXMLNodeText(el, "Join_No_Join", val))
	{
		joinSettings->noJoin=val.toInt();
	}
	if(getXMLNodeText(el, "Join_Algorithm", val))
	{
		joinSettings->algorithm=val.toInt();
	}
	if(getXMLNodeText(el, "Join_Width", val))
	{
		joinSettings->width=val.toDouble();
	}
    if(getXMLNodeText(el, "Join_Gap", val))
    {
        joinSettings->gap=val.toInt();
    }
    if(getXMLNodeText(el, "Join_End", val))
    {
        joinSettings->end=val.toInt();
    }
    if(getXMLNodeText(el, "Join_Min_Box", val))
    {
        joinSettings->minBoxSize=val.toInt();
    }
    if(getXMLNodeText(el, "Join_Gap_Size", val))
    {
        joinSettings->gapSize=val.toDouble();
    }
	if(!guiElement.isNull())
	{
		InputItem* item=this->getGUI();
        scene->addItem(item);
        item->fromXMLElement(guiElement);
		item->blockSig();
		item->setValue(data);
		item->allowSig();
	}
	else
	{
		DvObject_var obj = getObjectAndComp(data.toLatin1().data());
		setValue(obj, data);
       
	}
    
}


InputItem::InputItem(VInputItem* parentItem, VOutSlot* output, QGraphicsItem* parent):
		CalcItem(parent)
{
	this->parentItem=parentItem;
	
	outlineColor=QColor(100, 75, 00);
	color1=QColor(255, 255, 100);
	color2=QColor(230, 190, 80);
	
	title=parentItem->getName();
	
	width=BASE_WIDTH;
	height=BASE_HEIGHT;
	setRect(-width/2., -height/2., width, height);
	
	edit=new QguWLinput(NULL, "");
	edit->graphicsParent = this;
	
	edit->setMaximumWidth((int)(BASE_WIDTH-6));
	edit->setMinimumWidth((int)(BASE_WIDTH-6));
	edit->setMaximumHeight((int)(HEADER_HEIGHT-2));
	edit->setMinimumHeight((int)(HEADER_HEIGHT-2));
    edit->setAcceptable(Qcondition());
    edit->addAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    edit->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
    edit->addAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	
	QGraphicsProxyWidget * editProxy=new QGraphicsProxyWidget(this);
	editProxy->setWidget(edit);
	editProxy->moveBy(-edit->width()/2., 0.);
	
	OutSlot* slot=new OutSlot(this, output);
	slot->setPos(0., height/2.);
	outSlots.push_back(slot);
	
	rzButton->setPos(-width/2.+1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	closeButton->setPos(width/2.-HEADER_HEIGHT/2., -height/2.+HEADER_HEIGHT/2.);
	errButton->setPos(width/2.-1.5*HEADER_HEIGHT, -height/2.+HEADER_HEIGHT/2.);
	
	connect(edit, SIGNAL(edited_OK()), this, SLOT(setValue()));
	
	setAcceptDrops(false);
	edit->setAcceptDrops(false);
	
//	setValue(parentItem->valueText);


}

InputItem::~InputItem()
{
	foreach(InSlot* slot, inSlots)
	{
		delete slot;
	}
	foreach(OutSlot* slot, outSlots)
	{
		delete slot;
	}
}


QVariant InputItem::itemChange(GraphicsItemChange change, const QVariant& value)
{
	if (change == ItemPositionChange && scene()) {
		QPointF newPos = value.toPointF();
		
		updateConnectorsPositions();
		
		return newPos;
	}
	else if(change==ItemSelectedChange && scene())
	{
		if(value.toBool())
		{
			setZValue(1.);
		}
		else setZValue(0.);
		parentItem->setSelected(value.toBool());
	}
	return QGraphicsItem::itemChange(change, value);
}

void InputItem::dragEnterEvent(QGraphicsSceneDragDropEvent * e)
{
	e->accept();
	QGraphicsItem::dragEnterEvent(e);
}

void InputItem::dragLeaveEvent(QGraphicsSceneDragDropEvent * e)
{
	e->accept();
	QGraphicsItem::dragLeaveEvent(e);
}

void InputItem::dropEvent(QGraphicsSceneDragDropEvent* e)
{
	e->accept();
	QGraphicsItem::dropEvent(e);
	setValue(e->mimeData()->text());
} 

void InputItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	CalcItem::paint(painter, option, widget);
}

void InputItem::setValue()
{
    setValue(edit->text());
}

void InputItem::setValue(const QString& text)
{
	DvObject_var obj = getObjectAndComp(text.toLatin1().data());
	if(obj.is_ok())
	{
		if( obj->is_event() ) emit newInput(QString("scalar"));
		else if(obj->seqSize() > 1) emit newInput(text);
		else emit newInput(QString("scalar"));
		
		parentItem->setValue(obj, text);
	}

	
	if(edit->text() != text)
	{
		edit->setText(text);
		edit->reValidate();
	}
}

QDomElement InputItem::toXMLElement(QDomDocument& doc) const
{
	return CalcItem::toXMLElement(doc);
}

void InputItem::fromXMLElement(QDomElement& el)
{
	CalcItem::fromXMLElement(el);
}
