/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
#ifndef _QTPL_H
#define _QTPL_H

#include <qsplitter.h>
#include <qtextedit.h>
#include "qt_version.h"

#include <QScrollArea>
#include <QLayout>

#include "qtpl_parser.h"
#include "qslot.h"
#include "qplug_if.h"
#include "PlotWindow.h"

#ifdef WINDOWS
/* SJS attempt to undefine X and Y 6Aug2014 */
#undef X
#undef Y
#include <windows.h>
#else
#include <dlfcn.h>     // dynamic-load stuff
#endif

extern PlotID *plotWin_id;
extern PlugPlotWidgets *plotWidgets;

#define QTPL_VERSION  "QTPL V_2.0 [13 Feb 2004]"
#define QTPL_MINWIDTH 600
#define QTPL_MINHEIGHT 300

class qtplMenu : public QMenu
{
   Q_OBJECT

public:
   qtplMenu(QWidget *parent=0, const char *name=0);
   void addFolderToMenu(const char *folder, bool first= true);
   QString menuPath;
   
signals:
   
public slots:
   void qtplOpen(QAction*);

protected:

private:
   
};



class QplugSlot : public QGroupBox
{
   Q_OBJECT

public:
   QplugSlot(const QString &title, const QString & text, 
                    QString qtplType,
                    QWidget *parent=0, const char *name=0);
   int argIndex;
   QString type;
   QString defaultValue;
   QString toolTipText;
   QguWLinput *ioSlot;
   
signals:
   
public slots:

protected:

private:
   
};



class qplugWindow : public QMainWindow {

Q_OBJECT
  
public slots:
  void dismiss();
  void defaultAll();
  void runPlugin();
  void launchVersion();
  void launchManPlugins();
  void launchPluginHelp();
  void launchDocPlugin();
  void launchManual();
  void saveLog();
  void clearText();
  
signals:

  
public:
    
  qplugWindow(QWidget *parent=0, const char *name=0, QtplParserFile * qtplFile=0);
    ~qplugWindow(){
        if(plotWin_id) {
            delete plotWin_id;
            plotWin_id = NULL;
        }
#ifdef WINDOWS
        if( handle && FreeLibrary(handle)==0) QplugAppendTextDisplay("Error freeing the library");
#else
        if( handle && dlclose(handle) != 0 ) QplugAppendTextDisplay(dlerror());
#endif
        handle = NULL;
    }
  void QplugInit ();
  void QplugBuildInputArgsList (vector <DvObject_var> &inputs);
  void QplugSetDefaults();
  void QplugGetEntryName();
  
#ifdef QT_VERSION
#if QT_VERSION <0x030000
  QMultiLineEdit * textArea;
#else 
  QTextEdit * textArea;		
#endif // end of #if
#endif // end of #ifdef
  QWidget *inputWidget;
  QVBoxLayout *inputVLayout;
  //Q_V_GROUP_BOX *inputBox;
  QWidget *outputWidget;
  QVBoxLayout *outputVLayout;
  QWidget *ioWidget;
  QHBoxLayout *ioHLayout;
  QScrollArea *scrollW;
  QtplParserFile *qtplf;
  QString PathAndNameSO;
  QString try_entry_name;
  QString qtplName;
  vector <QplugSlot *> iSlots;
  vector <QplugSlot *> oSlots;
  int errorSet;
    
protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *runBtn;
  QPushButton *defaultBtn;
  QPushButton *dismissBtn;
  QPushButton *saveBtn;
  QPushButton *clearBtn;
    
#ifdef WINDOWS
    HINSTANCE handle;
#else
    void *handle;                      // for dynamic load
#endif
};

   
qplugWindow * QtplOpenByName(QString name, QWidget *parent, bool show=true);
int runBatchPlugin(DvString name, QWidget *parent, vector <DvString> &in, vector <DvString> &out);
void QplugSetTextDisplay (QTextEdit *);

  
#endif


