//Dwt_set_plot.cc

#include "Dwt_set_plot.h"

QplugReturnStatus ReadFilter(const char* filter_name, DvObject_var &gfilt)		 
{
   QString filter_bank_path;
   char* file_name;
   if ( !(filter_bank_path = QSAS_profiles->current()->FilterBankPath).isEmpty() ){
      file_name = (char *)malloc(strlen(filter_bank_path.toLatin1().data())+12);
      strcpy(file_name, filter_bank_path.toLatin1().data());
   } else {
       QplugAppendTextDisplay("Error: Cannot read FILTER_BANK.");
       QplugAppendTextDisplay("Check path in current profile");
       return QPLUG_FAILURE;
   }
   strcat(file_name, "/Bank.flt");
   int len = gfilt->seqSize();
   
   ifstream  fin(file_name, ios::in);
   if(!fin){
       QplugAppendTextDisplay("Error: Cannot open filter file.");
       QplugAppendTextDisplay("Check path in current profile.");
       return QPLUG_FAILURE;
   }
   char s[100], ch[20], readch;
   double gvalue;
   int rlen;
   
   fin.getline(s, 100, '\n');
   fin.getline(s, 100, '\n');
   fin.getline(s, 100, '\n');
   fin >> ch >> rlen;
   while((strcmp(ch, filter_name) !=0) || rlen != len){
	for(int i=0; i<rlen; i++) fin >> gvalue;
	if((readch=fin.get()) == EOF) {
	   QplugAppendTextDisplay("Error: No match found for the selected filter/length combination\n");
           return QPLUG_FAILURE;
	}	
	fin >> ch >> rlen;
   }
   for(int l=0; l<len; l++){
		fin >> gvalue;
	 	gfilt->dbl(l) = gvalue;
   }
   fin.close();
   delete file_name;

  
   return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus  Run_Dwt(DvObject_var ts_in, int jsize, DvObject_var hf, DvObject_var gf, DvObject_var &wavlcoef)			
{

	long ts_size = ts_in->seqSize();
	int j = 1;
	long Mj = (long)floor(ts_size/2.0);

    valarray<double> vin, vout, wout, wcoef, wtmp;
	vin.resize(ts_size, 0.0);
    vin = ts_in->toDouble();  
	
    while(j <= jsize ){  
      int Nj = (int)Mj;
      vout.resize(Nj, 0.0);
      wout.resize(Nj, 0.0);
      for(int t=0; t<Nj; t++){
		int tt = 2*t +1;
		vout[t] = gf->dbl(0) * vin[tt]; 
		wout[t] = hf->dbl(0) * vin[tt];  
		for(int l=1; l<hf->seqSize(); l++){
			--tt;
			if(tt < 0) tt = 2*Nj - 1;
			vout[t] = vout[t] + gf->dbl(l) * vin[tt];
			wout[t] = wout[t] + hf->dbl(l) * vin[tt];
		}
	  }
	  int N = wcoef.size();
	  wtmp.resize(N);
	  wtmp = wcoef;
	  
	  wcoef.resize(N + wout.size());    
	  for(int i=0; i<N; i++) wcoef[i] = wtmp[i];  
	  for(int i=0; i<wout.size(); i++) wcoef[N+i] = wout[i];  

	  vin.resize(vout.size());
      vin = vout;
      Mj = (long)(Mj/2);
      ++j;
	}
	int N = wcoef.size();
	wtmp.resize(N);
	wtmp = wcoef;
	wcoef.resize(N + vout.size());    
	for(int i=0; i<N; i++) wcoef[i] = wtmp[i];  
	for(int i=0; i<vout.size(); i++) wcoef[N+i] = vout[i];  

	wavlcoef = new DvObject(wcoef);
	
	return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus  Run_MoDwt(DvObject_var ts_in, int jsize, DvObject_var hf, DvObject_var gf, DvObject_var &wavlcoef)			
{

	long ts_size = ts_in->seqSize();
	int j = 1, k = 1, tt;

	valarray<double> vin, vout, wout, wcoef, wtmp;    
	vin.resize(ts_size, 0.0);
    vin = ts_in->toDouble();
	 
    while(j <= jsize ){  
      vout.resize(ts_size, 0.0);
      wout.resize(ts_size, 0.0);
      for(int t=0; t<ts_size; t++){
		tt = t;

		vout[t]= gf->dbl(0) * vin[tt]; 
		wout[t]= hf->dbl(0) * vin[tt]; 
		for(int l=1; l<hf->seqSize(); l++){
			tt -= k; 
			if(tt < 0) tt = tt + ts_size;
			vout[t] = vout[t] + gf->dbl(l) * vin[tt];
			wout[t] = wout[t] + hf->dbl(l) * vin[tt];		
		}


      }
  
	  int N = wcoef.size();
	  wtmp.resize(N);
	  wtmp = wcoef;
	  
	  wcoef.resize(N + wout.size());    
	  for(int i=0; i<N; i++) wcoef[i] = wtmp[i];  
	  for(int i=0; i<wout.size(); i++) wcoef[N+i] = wout[i];  
		
	  vin.resize(vout.size());
	  vin = vout;
	  ++j;
	  k = 2*k;
	}
	int N = wcoef.size();
	wtmp.resize(N);
	wtmp = wcoef;
	wcoef.resize(N + vout.size());    
	for(int i=0; i<N; i++) wcoef[i] = wtmp[i];  
	for(int i=0; i<vout.size(); i++) wcoef[N+i] = vout[i];  
		
 
	wavlcoef = new DvObject(wcoef);

    return QPLUG_SUCCESS;
}
/*-------------------------------------------------------------*/

QplugReturnStatus SPectro_set(DvObject_var wav_mat, DvObject_var ttag,
		int level, PGSpectro *spectrop, DvObject_var gfilter, 
		Yes_or_No dwt_yes_no, const char *x_lab, const char *y_lab, const char *w_lab)
{
	int nx;
	if(dwt_yes_no == YES)  nx = wav_mat->seqSize();
	else  nx = (int) (wav_mat->seqSize()/(level+1.0));

	int nz = ttag->seqSize(); //size of data without padding
	int filtlen = gfilter->seqSize();
	
	vector<size_t> dims;
	dims.push_back(level);
	dims.push_back(nx);
	DvObject_var z = new DvObject(0.0, dims); //z[level][nx]; 
      
	spectrop->z = new double[level*nx];//for DWT or MODWT
	spectrop->nx = nx;                  
	spectrop->ny = level;

	spectrop->domain.xndcmax = 0.8;
	spectrop->domain.yndcmin   = 0.1;     
	spectrop->domain.yndcmax   = 0.4;

	spectrop->xlabel = x_lab;        
	spectrop->ylabel = y_lab; 
	spectrop->wedge.label = w_lab; //Wedge label;
	spectrop->ylinlog = PGSPECTRO_LOG;
     
	spectrop->skip_missing = YES;
	spectrop->zmissing = 0.0;
	spectrop->zlinlog = PGSPECTRO_LOG; //displays Log(|w_{j,t}|^2)

	float delta_t = (float)ttag->get_spacing(); //sampling interval
	if(delta_t <= 0.0 ){
        QplugAppendTextDisplay("Error: Sampling interval <= zero.\n");
        return QPLUG_FAILURE;
	}
     
	spectrop->ybinlow = new double[level];
	spectrop->ybinhigh = new double[level];

     //float lamda_j = pow(2.0,j+1)*delta_t; tau_j = pow(2.0,j)*delta_t;
	for (int j=0; j<level; j++){	
        spectrop->ybinlow[j] = pow(2.0,j)*delta_t; 
        spectrop->ybinhigh[j] = 2.0*spectrop->ybinlow[j];	   
	}            
	spectrop->domain.yworldmin = spectrop->ybinlow[0];
	spectrop->domain.yworldmax = spectrop->ybinhigh[level-1];
 

	DvObject_var eg = new DvObject(0.0); 
	if(dwt_yes_no == YES){
        //gfilt_var MUST be of unit energy =sum_{l} (g_l)^2
        for(int l=0; l<filtlen; l++) eg += l * gfilter->dbl(l) * gfilter->dbl(l);
	}
	else {
		for(int l=0; l<filtlen; l++){
			gfilter->dbl(l) *= sqrt(2.0);
			eg +=  l * gfilter->dbl(l) * gfilter->dbl(l);
        }
    }
	
    QplugReturnStatus ret_stat;
    
	//now we update the spectrop
    ret_stat = SPectro_fill(nz, filtlen, eg, wav_mat, spectrop, dwt_yes_no, z);

  
    return QPLUG_SUCCESS;
}

/*-------------------------------------------------------------*/
QplugReturnStatus SPectro_fill(int nz,  int Llen, DvObject_var eg, 
		DvObject_var wseq, PGSpectro *spectrop, Yes_or_No dwt_yes_no, DvObject_var z)
{

	int level = z->Dims()[0]; //spectrop->ny;
	int nx = z->Dims()[1];
	DvObject_var tmpseq;
	DvObject_var wavmax =  new DvObject(0.0, level);
	DvObject_var wavmin =  new DvObject(0.0, level);
	DvObject_var tmpwavj;
	int ii, jump, nj = nx/2, nwj, nstart = 0, t=0; 
 
	if(dwt_yes_no == YES){
    	for (int k=0; k<level; k++){
			int j = k+1;
			int Lj = (int)((pow(2.0, j) - 1)*(Llen - 1)+1);
			int nujH = (int)( pow(2.0, j-1)*(Llen - 1) - ceil(eg->dbl()));
			int wearlyt= 0;
			int ttj = (int)(pow(2.0,j)*(wearlyt+1) - 1 - nujH);
			while(ttj <0){
				++wearlyt;
				ttj = (int)(pow(2.0,j)*(wearlyt+1) - 1 - nujH);
			} 
       
			//compute the associated early time in X_t
			int xearlyt = (int) (pow(2.0,j)*(wearlyt+1)-1-nujH);

			int r = (int) (pow(2.0,j-1));
			jump = (int)(pow(2.0,j) - 1.0);   

			//pick up the right j-level wav. coef.
			tmpwavj = wseq->subSequence(nstart, nstart+nj-1);

			//divide by the number of time we repeat the energy
       		*tmpwavj /= pow(2.0,j); //works fine
			//shift the wav. coef. to line up time to X_t
			if(wearlyt > 0) tmpwavj->cycleMod(0, -wearlyt, tmpwavj->seqSize());
	
			//--------next fill the matrix z	  
			if(xearlyt == 0){
	   			//fill to the right and shift left by r
	   			ii = 0;	    
	   			for(t=0; t<nx; t+=2*r) {
	       			for(int l=0; l<2*r; l++){
		   				z->dbl(0,k,t+l) = tmpwavj->dbl(ii);
					}
					++ii; 	
	   	 		}      
	    		z->cycleMod(k*nx, -r, nx);
			}
			else if(xearlyt >0 & xearlyt == r){	      
	    		ii = -1;
	    		for(t=xearlyt; t<nx-2*r; t += 2*r){
	       			++ii;
	       			//fill by r before t (t-r,..,t-1) and by r-1 after t
	       			for(int l=0; l<r+1; l++) {
						z->dbl(0,k,t-l) = tmpwavj->dbl(ii);
					}
	       			if(r>1) for(int l=1; l<r; l++) z->dbl(0,k,t+l) = tmpwavj->dbl(ii);

	     		}
	     		++ii; 
	     		for(int l=0; l<r+1; l++) z->dbl(0,k,t-l) = tmpwavj->dbl(ii);
	     		if(r>1)  for(int l=1; l<r; l++) z->dbl(0,k,t+l) = tmpwavj->dbl(ii);   
			} 
			else if(xearlyt > r){	   
	     		ii = -1;
	     		for(t=xearlyt; t<nx-2*r; t+=2*r){
	        		//fill by r before t (t-r,..,t-1) and by r-1 after t
	         		++ii;
		 			for(int l=0; l<r+1; l++) {
						z->dbl(0,k,t-l) = tmpwavj->dbl(ii);
					}
		 			if(r>1) {
		       			for(int l=1; l<r; l++) z->dbl(0,k,t+l) = tmpwavj->dbl(ii);
		 			}
	      		}
	      		++ii;
	      		for(int l=0; l<r+1; l++) z->dbl(0,k,t-l) = tmpwavj->dbl(ii);
	      		while(nx-t-1>0 & t<nx){
		    		++t;
		   			z->dbl(0,k,t) = tmpwavj->dbl(ii);
	      		}
	      		//fill between t=0 and t=xearlyt-r
	      		for(int l=0; l<xearlyt-r; l++) z->dbl(0,k,l) = tmpwavj->dbl(ii);
		
			} 
			else if(xearlyt >0 & xearlyt<r) {
	      		//fill always to the left
	       		for(t=0; t<xearlyt+1; t++) z->dbl(0,k,t) = tmpwavj->dbl(0);
	       		ii = 1;	       
               	for(t= xearlyt+1; t<nx-jump; t+=jump+1){ 
	           		z->dbl(0,k,t) = tmpwavj->dbl(ii);
                   	for(int i=t+1; i< t+1+jump; i++) z->dbl(0,k,i) = tmpwavj->dbl(ii);	
					++ii ;  
 	        	}	       
				int tend = nx - (jump - xearlyt);
				if(tend >0){
		   			for(t=tend; t<nx; t++) z->dbl(0,k,t) = tmpwavj->dbl(0);
	        	}
	      		//now shift to the right by r-1
	      		if(r>1) z->cycleMod(k*nx, r-1, nx);
			}
			nstart += nj;
			nj = nj/2;
			wavmax->dbl(k) = tmpwavj->max();
			wavmin->dbl(k) = tmpwavj->min();
      
		}//end of for loop over k	   		
		spectrop->zscalemax = (float)(wavmax->max());  
		spectrop->zscalemin = (float)(spectrop->zscalemax/pow(2.0,1.5*level));   	 	    	    
	} 
	else {
		tmpseq = wseq->subSequence(0, (level*nx)-1);//off scal. coef
		spectrop->zscalemax = (float)tmpseq->max();
		spectrop->zscalemin = (float)(spectrop->zscalemax/pow(2.0,1.5*level));
		
		//gfilt_var must be be of unit energy
		for(int k=0; k<level; k++){ 
			int j = k+1;
			int Lj = (int)((pow(2.0, j) - 1)*(Llen - 1)+1);
			int nujH = (int)( pow(2.0, j-1)*(Llen - 1) - ceil(eg->dbl()));
			int w_earlyt= 0;
			int ttj = (int)(w_earlyt - nujH);

			while(ttj <0){
				++w_earlyt;
				ttj = (int)(w_earlyt - nujH);
			} 
			int x_earlyt = (int)(w_earlyt - nujH);
			tmpwavj = wseq->subSequence(k*nx, k*nx+nx-1);
	       	if(w_earlyt > 0) tmpwavj->cycleMod(0, -w_earlyt, nx);
			for(int i=0; i<nx; i++) {
				z->dbl(0, k, i) = tmpwavj->dbl(i);
			}
		}
	} 
	//now fill in the vector z. spectro->z[k]=z[i][j]; //z[nx][ny] 
	
	if(dwt_yes_no == YES && isPowerOf2(nz) == NO){
         spectrop->nx = nz;   
         spectrop->z = new double[level*nz];
         spectrop->zscalemax = (double)(z->max());
         spectrop->zscalemin = (double)(spectrop->zscalemax/pow(2.0,1.5*level));	 
         for(int i=0; i<nz; i++){
            for(int j=0; j<level; j++){
                int k=level*i+j;
	        	spectrop->z[k] = z->dbl(0,j,i);
            }
         }
	} 
	else {
		for(int i=0; i<nx; i++){
			for(int j=0; j<level; j++){
                int k=level*i+j;
				spectrop->z[k] = z->dbl(0,j,i);
			}
		}
    }     
 
 return QPLUG_SUCCESS;
}

 /*-------------------------------------------------------------*/
 Yes_or_No isPowerOf2(long m){
	float f = log((float)m)/log(2.0);
	if((f - floor(f)) != 0.0) return NO;
	else return YES;
 }
 
 	
/*-------------------------------------------------------------*/
void clean_all(PGSpectro *spectrop/*, PGXYtrace *tracep*/){
//       delete [] tracep->x, delete [] tracep->y;
//       delete [] tracep->datestart, delete [] tracep->dateend;
      delete [] spectrop->z;
      delete [] spectrop->ybinlow;
      delete [] spectrop->ybinhigh;
}      
	 

