/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/* PROJECT:     Cluster UK CDHF
* COMPONENT:   IRM Onboard Timetags Plugin 
* MODULE:      IRMTags
* LANGUAGE:	   C++
* FUNCTION:
* PURPOSE:     Shift IRM Timetags to Onboard UT 
* DESCRIPTION:
* ARGUMENTS: [pased via call_list]
*    IN:
*     ANY_TIME_SERIES      ts_in        input time series data object
*     VECTOR_TIME_SERIES   IRMPos       IRM Position in GEOC joined to ts_in
*
*    OUT:
*     ANY_TIME_SERIES      ts_out       same as ts_in but with timetags
*                                              shifted to UT Onboard         
* GLOBALS:
* RETURN:
*    TYPE:     QplugReturnStatus
*    VALUES:   see qplug_if.h
* FUNCTIONS_USED: 
* ALGORITHM:   pretty obvious
* HISTORY:	   Steve Schwartz  Nov 98
*              AJA March 2000
***********************************************************************/

#ifndef __cplusplus
#define __cplusplus
#endif

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"
#include "IRMTags.h"

using namespace QSAS;

#ifdef __cplusplus
  extern "C" {
#endif
#ifdef __cplusplus
  }
#endif

QplugReturnStatus  IRMTags  ( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{

int     at_ok;
char    outstring[200];
int ndata;

// Objects:

QplugReturnStatus      IRMTags_return;   // used to return from sub-modules

QplugAppendTextDisplay("IRM Timetag shift to onboard UT. Nov 1998\n");

int nlist = inputs.size();        // are there enough args? 
if (nlist < 2){
   QplugAppendTextDisplay( "Not enough entries in argument list from GUI\n");
   return QPLUG_FAILURE;
}  //endif


//	...next, get the input data objects and fill arrays

DvObject_var ts_in = inputs[0];    // FIRST INPUT ARGUMENT = IRM Data Object

      
DvObject_var irmpos = inputs[1];      // SECOND INPUT ARGUMENT = IRM Position

if( irmpos.is_nil() || !irmpos->isThreeVector() ){
       QplugAppendTextDisplay("ERROR>>Second Argument not a vector TS\n");	
       return QPLUG_FAILURE;
} //endif

 string attribute_value = irmpos->getXrefText(FRAME);
 
 if (TestGeocFrame(attribute_value.c_str()) != QPLUG_SUCCESS){ 
    sprintf(outstring,"IRMTags>> IRM Pos Frame given as %s, not geoc_xyz.\n Proceed with caution\n", attribute_value.c_str());           
    QplugAppendTextDisplay(outstring);
 }
 
 
#ifdef VERBOSE_IRM
      sprintf(outstring,"Frame = %s\n",attribute_value.c_str());
      QplugAppendTextDisplay(outstring);
#endif
 
//      ... test that DO and Pos objects are joined

DvObject_var ts_tt  = ts_in->getTimeTags();
  
 ndata = ts_in->seqSize();

// ensure joined

irmpos = irmpos->Join(ts_in); 

 // Create and fill output data object; add Mod re shifted tags
// Note that copy operation automatically passes over all Xrefs (attributes)

 DvObject_var tt_out = new DvObject(ts_tt);
 DvObject_var ts_out = new DvObject(ts_in);

IRMTags_return = IRMTags_generate (irmpos, tt_out, ndata);
if (IRMTags_return != QPLUG_SUCCESS) {
      QplugAppendTextDisplay (
          "IRMTags >> New tags failed to be generated!\n");
      return QPLUG_FAILURE;  
}

#ifdef VERBOSE_IRM
  QplugAppendTextDisplay("IRMTags New timetags generated.\n");
#endif


 ts_out->change_xref(DEPEND_0, tt_out);


 // Add attribute describing what we've done
 
 ts_out->change_xref("Analysis_Notes", "IRMTags used to shift Timetags to On-board UT");

 outputs.push_back(ts_out);     
 

#ifdef VERBOSE_IRM
 QplugAppendTextDisplay(">>IRMTags  object done successfully\n");
#endif


return QPLUG_SUCCESS;
                                                 // cleanup and finish 
}  // end IRMTags



/***********************************************************************
***************************************************  IRMTags_generate
***********************************************************************/

QplugReturnStatus  IRMTags_generate (DvObject_var &pos, DvObject_var &outtag, int ndata)
{

    int     i;
    double   dr,temp;

    for(i=0; i<ndata; i++){
        dr=0.0;
        temp= pos->dbl(i,0) - gndstn_x; temp =temp*temp;dr += temp;
        temp= pos->dbl(i,1) - gndstn_y; temp =temp*temp;dr += temp;
        temp= pos->dbl(i,2) - gndstn_z; temp =temp*temp;dr += temp;
        dr=sqrt(dr);
        outtag->time(i) -= dr/c_re_s;
    }  
    
    return QPLUG_SUCCESS;
}

/**********************************************************************
****************************************************** TestGeocFrame  
**********************************************************************/

/* Borrowed fragments from Tony's QRTN QrGetFrameFromAttribute routine */

QplugReturnStatus TestGeocFrame(const char * attstring){
 
char *str;          /* need to copy string as strtok changes its arg */
char *ptr;
 
str = new char[strlen(attstring)+1];
strcpy(str,attstring);

  /* Frame is of format vector>frame_xyz" */
   
 ptr = strtok(str,">");
 if(ptr != NULL) ptr = strtok(NULL, "_");
 
 if ( (ptr != NULL)  && (strcmp(ptr, "geoc") == 0) ){
    delete[] str;
    return QPLUG_SUCCESS;
 } else {
    delete[] str;
    return QPLUG_FAILURE;
 }
}
