/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "MinVar.h"
#include "qdutil.h"
#include "qpui.h"
#include "qsas_plplot_time_helpers.h"

using namespace QSAS;

void mvplot(DvObject_var &vts_out, 
            DvObject_var &Eigenvalues, DvObject_var &Rmatx, int constrained)
{
    int i;
	char stringret[10];
	double day_epoch;
	PLFLT max1, max2, min1, min2, int1, int2, diff, hold;
	PLFLT *secs;
	PLFLT *mvmax, *mvint, *mvmin;
	int yr, mth, dy, hr, mn;
	double sc;
	int pgid;  /* hold qsas id to restore on return */
	string str1, str2, str3;
	PLFLT xval[4], yval[9];
	int np;
	PLFLT line_x[2], line_y[2];

	DvObject_var tt = vts_out->getTimeTags();
	if (tt.is_nil() ) return;

	np = tt->seqSize();

	secs = new PLFLT[np];

    for(i=0;i<np;i++){
        secs[i] = (PLFLT) tt->time(i).getEpoch1970();   //try to work with real time SJS
    }
    
    // unpack vts_out and cast to float (find array bounds as we go)

	mvmax = new PLFLT[np];
	mvint = new PLFLT[np];
	mvmin = new PLFLT[np];

	mvmin[0] = vts_out->dbl(0,0);
	mvint[0] = vts_out->dbl(0,1);
	mvmax[0] = vts_out->dbl(0,2);

	max1=mvmax[0];int1=mvint[0];min1=mvmin[0];
	max2=mvmax[0];int2=mvint[0];min2=mvmin[0];

	for(i=1;i<np;i++){
		mvmin[i] = vts_out->dbl(i,0);
		mvint[i] = vts_out->dbl(i,1);
		mvmax[i] = vts_out->dbl(i,2);
		if(mvmin[i] < min1) min1=mvmin[i];
		if(mvint[i] < int1) int1=mvint[i];
		if(mvmax[i] < max1) max1=mvmax[i];
		if(mvmin[i] > min2) min2=mvmin[i];
		if(mvint[i] > int2) int2=mvint[i];
		if(mvmax[i] > max2) max2=mvmax[i];
	}
	diff = max2-max1;
	diff = diff/10.0;
	max1 -= diff;
	max2 += diff;
	diff = (max2-max1)/2.0;
	hold = int1;
	int1 = (hold + ((int2-hold)/2.0)) - diff;
	int2 = (hold + ((int2-hold)/2.0)) + diff;
	hold = min1;
	min1 = (hold + ((min2-hold)/2.0)) - diff;
	min2 = (hold + ((min2-hold)/2.0)) + diff;

    PLFLT tickPos = 0;

	double fact_x=.75/5.8;
	double fact_y=.75/5.8/1.414;
	xval[0] = 0.7*fact_x; xval[1] = 3.7*fact_x; xval[2] = 6.7*fact_x; xval[3] = 7.3*fact_x;
	yval[0] = 0.75*fact_y; yval[1] = 2.5*fact_y; yval[2] = 3.5*fact_y; yval[3] = 4.5*fact_y; yval[4] = 5.5*fact_y;
	yval[5] = 6.5*fact_y; yval[6] = 9.5*fact_y; yval[7] = 10.0*fact_y; yval[8] = 10.5*fact_y;

    if(plotWin_id == NULL) plotWin_id = new PlotID("Minimum Variance");
    PlotWindow *win= plotWin_id->newPlotWindow((int)(150*8.27), (int)(150.*11.69), 150); 
	win->setSmooth(true);
	pladv(0);
	plcol0(0);
	plschr(QPUI_CHAR_HEIGHT, 0.5);

    plwid(7);

	plvpor(xval[0], xval[1], yval[5], yval[6]);
	plwind(int1, int2, max1, max2);
	plbox("bcmts", 0., 0, "bcnts", 0., 0);
	win->setSmooth(false);
	plline(np, mvint, mvmax);

	plpoin(1, &mvint[0], &mvmax[0], 4);
	plpoin(1, &mvint[0], &mvmax[0], 17);
	win->setSmooth(true);

	plmtex("t", 3.5, 0.5, 0.5, "Intermediate (MV frame)");
	plmtex("l", 5., 0.5, 0.5, "Maximum (MV frame)");

	win->setSmooth(false);
	pllsty(4);
	line_x[0]=int1; line_x[1]=int2;
	line_y[0]=0.; line_y[1]=0.;
	plline(2, line_x, line_y);
	line_x[0]=0.; line_x[1]=0.;
	line_y[0]=max1; line_y[1]=max2;
	plline(2, line_x, line_y);
	pllsty(1);
	win->setSmooth(true);

	plvpor(xval[1], xval[2], yval[5], yval[6]);
	plwind(min1, min2, max1, max2);
	plbox("bcmts", 0., 0, "bcmts", 0., 0);
	win->setSmooth(false);
	plline(np, mvmin, mvmax);
	plpoin(1, &mvmin[0], &mvmax[0], 4);
	plpoin(1, &mvmin[0], &mvmax[0], 17);

	pllsty(4);
	line_x[0]=min1; line_x[1]=min2;
	line_y[0]=0.; line_y[1]=0.;
	plline(2, line_x, line_y);
	line_x[0]=0.; line_x[1]=0.;
	line_y[0]=max1; line_y[1]=max2;
	plline(2, line_x, line_y);
	pllsty(1);

	win->setSmooth(true);
	plmtex("t", 3.5, 0.5, 0.5, "Minimum (MV frame)");
	plmtex("r", 5., 0.5, 0.5, "Maximum (MV frame)");

	plvpor(xval[0], xval[1], yval[2], yval[5]);
	plwind(int1, int2, min1, min2);
	plbox("bcnts", 0., 0, "bcnts", 0., 0);
	win->setSmooth(false);
	plline(np, mvint, mvmin);
	plpoin(1, &mvint[0], &mvmin[0], 4);
	plpoin(1, &mvint[0], &mvmin[0], 17);

	win->setSmooth(true);
	plmtex("b", 3.5, 0.5, 0.5, "Intermediate (MV frame)");
	plmtex("l", 5., 0.5, 0.5, "Minimum (MV frame)");
	win->setSmooth(false);

    pllsty(4);
	line_x[0]=int1; line_x[1]=int2;
	line_y[0]=0.; line_y[1]=0.;
	plline(2, line_x, line_y);
	line_x[0]=0.; line_x[1]=0.;
	line_y[0]=min1; line_y[1]=min2;
	plline(2, line_x, line_y);
	pllsty(1);

	win->setSmooth(true);
	
    char fmt_ticks[50];
    char fmt_label[50];
    int separator=0;

    if(QSAS_profiles->current()->TimeRep=="Colon")
    {
        separator = 1;
    }

    PLINT auto_nsubx;
    PLFLT auto_xtick;
    computeFormat(fabs(secs[np-1] - secs[0]), fmt_ticks, fmt_label, &auto_xtick, &auto_nsubx, separator);
    pltimefmt(fmt_ticks);

    plschr(QPUI_CHAR_HEIGHT, 0.5);

    PLLabelDefaults exp_label_opt;
    exp_label_opt.exp_label_disp = 0.;
    exp_label_opt.exp_label_pos= 1.1;
    exp_label_opt.exp_label_just=0.;
    plslabelfunc(NULL, &exp_label_opt);
  
	plvpor(xval[1], xval[2], yval[4], yval[5]);
	plwind(secs[0], secs[np-1], max1, max2);
	plbox("dbcts", 0., 0, "bcmvts", tickPos, 0);
	win->setSmooth(false);
	plline(np, secs, mvmax);

	win->setSmooth(true);
	plmtex("r", 6., 0.5, 0.5, "Max.");

	plvpor(xval[1], xval[2], yval[3], yval[4]);
	plwind(secs[0], secs[np-1], int1, int2);
	plbox("dbcts", 0., 0, "bcmvts", tickPos, 0);
	win->setSmooth(false);
	plline(np, secs, mvint);

	win->setSmooth(true);
	plmtex("r", 6., 0.5, 0.5, "Int.");

	plvpor(xval[1], xval[2], yval[2], yval[3]);
	plwind(secs[0], secs[np-1], min1, min2);
	plbox("dfbcnts", 0., 0, "bcmvts", tickPos, 0);
	win->setSmooth(false);
	plline(np, secs, mvmin);

    plslabelfunc(NULL, NULL);

	win->setSmooth(true);
	plmtex("r", 6., 0.5, 0.5, "Min.");
	plmtex("b", 3.5, 0.5, 0.5, "Universal Time");

	plvpor(0., 1., yval[7], yval[8]);
	plwind(0.0, 1.0, 0.0, 1.0);

	DvString name = vts_out->getXrefText("FIELDNAM");
	switch(constrained){
		case(0):
			str1 = "Variance Analysis: "; str1 += name.c_str();
			break;
		case(1):
			str1 = "Constrained Variance Analysis: "; str1 += name.c_str();
			break;
		default:
			constrained = 0;
			str1 = "Constrained Variance Analysis: "; str1 += name.c_str();
			break;
	}

	plschr(QPUI_CHAR_HEIGHT, 0.8);
	plptex(0.5, 0.6, 1., 0., 0.5, str1.c_str());
	plschr(QPUI_CHAR_HEIGHT, 0.5);

	plvpor(xval[0], xval[2], yval[0], yval[1]);
	plwind(0.0, 1.0, 0.0, 7.0);
	str1 = "Time Interval (UT): ";
	str1 +=  tt->time(0).getISOstring().c_str();
	str1 += " - ";
	str2 = tt->time(np-1).getISOstring().c_str();
	for(i=11;i<str2.length();i++){
		str1 += str2.at(i);
	}
	plptex(0.5, 6.5, 1., 0.0, 0.5, str1.c_str());

	DvString units = vts_out->getXrefText("UNITS");
	str1 = "Units: "; str1 += units.c_str();
    str1 += "    Frame: ";
    DvString frm = Rmatx->getXrefText("Frame").c_str(); // original frame
    frm = frm.after(">");
    str1 +=  frm;

	DvString SC_id = vts_out->getXrefText("SC_id");
	if(!SC_id.empty()){
		str2 = " Spacecraft: "; str2 += SC_id.c_str();
		str1 += str2;
	}

	plptex(0.5, 5.0, 1., 0., 0.5, str1.c_str());

	plptex(0.25, 3.5, 1., 0., 0.5, "#gl");
	plptex(0.60, 3.5, 1., 0., 0.5, "Direction");
	plcol0(0);
	char str[300];
	for(i=0;i<3;i++){
		if (i == constrained) plcol0(1);
		hold = 2.5 - (float)i;
		sprintf(&(str[0]), "%.3g",  Eigenvalues->dbl(i));
		plptex(0.25, hold, 1., 0.0, 0.5, str);
		sprintf(&(str[0]), "(%6.3f, %6.3f, %6.3f)", 
			Rmatx->dbl(0,i,0), Rmatx->dbl(0,i,1), Rmatx->dbl(0,i,2));
		plptex(0.60, hold, 1., 0.0, 0.5, str);
		plcol0(0);
	}
    
    win->activateWindow();
    win->raise();
    win->setVisible(false);  // this seems to be necessary
    win->showNormal(); // includes showVisible(true)

	// if(pgid != 0) cpgslct(pgid); /* restore pgplot device selected prior to call */

    delete [] secs;
    delete [] mvmax;
    delete [] mvint;
    delete [] mvmin;

    return; 

}

/*----------------------------------------------------------------------*/

