#include "QCCWindow.h"

#if QT_VERSION > 0x050000
#include <QPageSize>
#endif

PlugPlotWidgets::PlugPlotWidgets(PlotWindow *_win)
{
	// Build interface
    win = _win;
    this->setContentsMargins(2,2,2,2);
    
    // Init plplot
    
    plmkstrm(&strm);
    plsetqtdev((void*)win->plot);
    plsdev("qt");
    plinit();

    this->setObjectName("PlugWindow");
    
    QGridLayout *layout = new QGridLayout(this);
    layout->setObjectName("plug layout");
    
	layout->addWidget(new QLabel("Title"), 0, 1, Qt::AlignRight);
	editTitle=new QLineEdit;
    editTitle->setMinimumWidth(300);
    layout->addWidget(editTitle, 0, 2, 1, 3, Qt::AlignLeft);
	connect(editTitle, SIGNAL(textEdited(const QString&)), this, SLOT(newTitle(const QString&)));

    layout->addWidget(new QLabel("Axis Units"), 1, 1, Qt::AlignRight);
	editAxis=new QLineEdit;
    editAxis->setMinimumWidth(50);
    layout->addWidget(editAxis, 1,2, Qt::AlignLeft);
	connect(editAxis, SIGNAL(textEdited(const QString&)), this, SLOT(newFrame(const QString&)));
	
	bgColour=Qt::cyan;
	buttonBgColour=new QPushButton("Box Colour");
	QPalette pBg; 
	pBg.setColor(QPalette::Active, QPalette::ButtonText, bgColour); 
	buttonBgColour->setPalette(pBg);
    layout->addWidget(buttonBgColour, 1, 4, Qt::AlignLeft);
	connect(buttonBgColour, SIGNAL(clicked()), this, SLOT(changeColourBg()));
	
	projX=new QCheckBox("No X component");
	layout->addWidget(projX, 2, 1, Qt::AlignLeft);
	connect(projX, SIGNAL(stateChanged(int)), this, SLOT(projXChanged(int)));
	projY=new QCheckBox("No Y component");
	layout->addWidget(projY, 2, 2, Qt::AlignLeft);
	connect(projY, SIGNAL(stateChanged(int)), this, SLOT(projYChanged(int)));
	projZ=new QCheckBox("No Z component");
	layout->addWidget(projZ, 2, 3, Qt::AlignLeft);
	connect(projZ, SIGNAL(stateChanged(int)), this, SLOT(projZChanged(int)));
	
	layout->addWidget(new QLabel("C1"), 3, 1, Qt::AlignCenter);
	layout->addWidget(new QLabel("C2"), 3, 2, Qt::AlignCenter);
	layout->addWidget(new QLabel("C3"), 3, 3, Qt::AlignCenter);
	layout->addWidget(new QLabel("C4"), 3, 4, Qt::AlignCenter);
	
	layout->addWidget(new QLabel("X"), 4, 0, Qt::AlignLeft);
	layout->addWidget(new QLabel("Y"), 5, 0, Qt::AlignLeft);
	layout->addWidget(new QLabel("Z"), 6, 0, Qt::AlignLeft);
//	layout->addWidget(new QLabel("Colour"), 7, 0, Qt::AlignLeft);
	
	QDoubleValidator * v=new QDoubleValidator(-1.e7, 1.e7, 2, 0);
	
	editX1=new QLineEdit;
    editX1->setValidator(v);
	layout->addWidget(editX1, 4, 1, Qt::AlignCenter);
	connect(editX1, SIGNAL(textEdited(const QString&)), this, SLOT(newX1(const QString&)));
	
	editY1=new QLineEdit;
    editY1->setValidator(v);
	layout->addWidget(editY1, 5, 1, Qt::AlignCenter);
	connect(editY1, SIGNAL(textEdited(const QString&)), this, SLOT(newY1(const QString&)));
	
	editZ1=new QLineEdit;
    editZ1->setValidator(v);
	layout->addWidget(editZ1, 6, 1, Qt::AlignCenter);
	connect(editZ1, SIGNAL(textEdited(const QString&)), this, SLOT(newZ1(const QString&)));
	
	editX2=new QLineEdit;
    editX2->setValidator(v);
	layout->addWidget(editX2, 4, 2, Qt::AlignCenter);
	connect(editX2, SIGNAL(textEdited(const QString&)), this, SLOT(newX2(const QString&)));
	
	editY2=new QLineEdit;
    editY2->setValidator(v);
	layout->addWidget(editY2, 5, 2, Qt::AlignCenter);
	connect(editY2, SIGNAL(textEdited(const QString&)), this, SLOT(newY2(const QString&)));
	
	editZ2=new QLineEdit;
    editZ2->setValidator(v);
	layout->addWidget(editZ2, 6, 2, Qt::AlignCenter);
	connect(editZ2, SIGNAL(textEdited(const QString&)), this, SLOT(newZ2(const QString&)));
	
	editX3=new QLineEdit;
    editX3->setValidator(v);
	layout->addWidget(editX3, 4, 3, Qt::AlignCenter);
	connect(editX3, SIGNAL(textEdited(const QString&)), this, SLOT(newX3(const QString&)));
	
	editY3=new QLineEdit;
    editY3->setValidator(v);
	layout->addWidget(editY3, 5, 3, Qt::AlignCenter);
	connect(editY3, SIGNAL(textEdited(const QString&)), this, SLOT(newY3(const QString&)));
	
	editZ3=new QLineEdit;
    editZ3->setValidator(v);
	layout->addWidget(editZ3, 6, 3, Qt::AlignCenter);
	connect(editZ3, SIGNAL(textEdited(const QString&)), this, SLOT(newZ3(const QString&)));
	
	editX4=new QLineEdit;
    editX4->setValidator(v);
	layout->addWidget(editX4, 4, 4, Qt::AlignCenter);
	connect(editX4, SIGNAL(textEdited(const QString&)), this, SLOT(newX4(const QString&)));
	
	editY4=new QLineEdit;
    editY4->setValidator(v);
	layout->addWidget(editY4, 5, 4, Qt::AlignCenter);
	connect(editY4, SIGNAL(textEdited(const QString&)), this, SLOT(newY4(const QString&)));
	
	editZ4=new QLineEdit;
    editZ4->setValidator(v);
	layout->addWidget(editZ4, 6, 4, Qt::AlignCenter);
	connect(editZ4, SIGNAL(textEdited(const QString&)), this, SLOT(newZ4(const QString&)));
	
	buttonCol1=new QPushButton("Change");
	QPalette p1; p1.setColor(QPalette::Active ,QPalette::ButtonText, Qt::black); buttonCol1->setPalette(p1);
	layout->addWidget(buttonCol1, 7, 1, Qt::AlignCenter);
	connect(buttonCol1, SIGNAL(clicked()), this, SLOT(changeColour1()));
	
	buttonCol2=new QPushButton("Change");
	QPalette p2; p2.setColor(QPalette::Active ,QPalette::ButtonText, Qt::red); buttonCol2->setPalette(p2);
	layout->addWidget(buttonCol2, 7, 2, Qt::AlignCenter);
	connect(buttonCol2, SIGNAL(clicked()), this, SLOT(changeColour2()));
	
	buttonCol3=new QPushButton("Change");
	QPalette p3; p3.setColor(QPalette::Active ,QPalette::ButtonText, Qt::green); buttonCol3->setPalette(p3);
	layout->addWidget(buttonCol3, 7, 3, Qt::AlignCenter);
	connect(buttonCol3, SIGNAL(clicked()), this, SLOT(changeColour3()));
	
	buttonCol4=new QPushButton("Change");
	QPalette p4; p4.setColor(QPalette::Active ,QPalette::ButtonText, Qt::magenta); buttonCol4->setPalette(p4);
	layout->addWidget(buttonCol4, 7, 4, Qt::AlignCenter);
	connect(buttonCol4, SIGNAL(clicked()), this, SLOT(changeColour4()));
	
	
	resetColourIndex();
		
	// Init variables
	
	for(int i=0; i<4; ++i)
	{
		for(int j=0; j<3; ++j)
			clusterParameters[i].position[j]=0.;
	}

	clusterParameters[0].colour=Qt::black;
	clusterParameters[1].colour=Qt::red;
	clusterParameters[2].colour=Qt::green;
	clusterParameters[3].colour=Qt::magenta;
	
	alt=45;
	az=45;
	projection[0]=projection[1]=projection[2]=false;

    sliderAlt = win->addSlider(Qt::Vertical, 0, 90, 5);
    sliderAlt->setValue(45);
    
    sliderAz = win->addSlider(Qt::Horizontal, 0, 360, 5);
    sliderAz->setValue(45);
     connect(sliderAlt, SIGNAL(sliderMoved(int)), this, SLOT(altChanged(int)));
     connect(sliderAz, SIGNAL(sliderMoved(int)), this, SLOT(azChanged(int)));
    
    
}

void PlugPlotWidgets::setParameters(RtnVector& v1, RtnVector& v2, RtnVector& v3, RtnVector& v4, const QString& title, const QString& frame)
{
	this->title=title;
	this->frame=frame;
	
	editTitle->setText(title);
	editAxis->setText(frame);
	
	for(int i=0; i<3; i++)
	{
		clusterParameters[0].position[i]=v1.ptr->dbl(i);
		clusterParameters[1].position[i]=v2.ptr->dbl(i);
		clusterParameters[2].position[i]=v3.ptr->dbl(i);
		clusterParameters[3].position[i]=v4.ptr->dbl(i);
	}
	
	char text[15];
	sprintf(text, "%.2f", clusterParameters[0].position[0]);
	editX1->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[0].position[1]);
	editY1->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[0].position[2]);
	editZ1->setText(QString(text));
	
	sprintf(text, "%.2f", clusterParameters[1].position[0]);
	editX2->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[1].position[1]);
	editY2->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[1].position[2]);
	editZ2->setText(QString(text));
	
	sprintf(text, "%.2f", clusterParameters[2].position[0]);
	editX3->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[2].position[1]);
	editY3->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[2].position[2]);
	editZ3->setText(QString(text));
	
	sprintf(text, "%.2f", clusterParameters[3].position[0]);
	editX4->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[3].position[1]);
	editY4->setText(QString(text));
	sprintf(text, "%.2f", clusterParameters[3].position[2]);
	editZ4->setText(QString(text));

	replot();
	
	
}

void PlugPlotWidgets::altChanged(int newAlt)
{
	this->alt=(double)newAlt;
	replot();
}

void PlugPlotWidgets::azChanged(int newAz)
{
	this->az=(double)newAz;
	replot();
}

void PlugPlotWidgets::projXChanged(int state)
{
	projection[0]=(state!=0);
	replot();
}

void PlugPlotWidgets::projYChanged(int state)
{
	projection[1]=(state!=0);
	replot();
}

void PlugPlotWidgets::projZChanged(int state)
{
	projection[2]=(state!=0);
	replot();
}

void PlugPlotWidgets::newX1(const QString& s)
{
	clusterParameters[0].position[0]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newY1(const QString& s)
{
	clusterParameters[0].position[1]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newZ1(const QString& s)
{
	clusterParameters[0].position[2]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newX2(const QString& s)
{
	clusterParameters[1].position[0]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newY2(const QString& s)
{
	clusterParameters[1].position[1]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newZ2(const QString& s)
{
	clusterParameters[1].position[2]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newX3(const QString& s)
{
	clusterParameters[2].position[0]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newY3(const QString& s)
{
	clusterParameters[2].position[1]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newZ3(const QString& s)
{
	clusterParameters[2].position[2]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newX4(const QString& s)
{
	clusterParameters[3].position[0]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newY4(const QString& s)
{
	clusterParameters[3].position[1]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newZ4(const QString& s)
{
	clusterParameters[3].position[2]=s.toDouble();
	replot();
}

void PlugPlotWidgets::newTitle(const QString& s)
{
	title=s;
	replot();
}

void PlugPlotWidgets::newFrame(const QString& s)
{
	frame=s;
	replot();
}

void PlugPlotWidgets::changeColour1()
{
	QPalette pal=buttonCol1->palette();
	clusterParameters[0].colour=QColorDialog::getColor(clusterParameters[0].colour, this);
	pal.setColor(QPalette::Active ,QPalette::ButtonText, clusterParameters[0].colour);
	buttonCol1->setPalette(pal);
	replot();
}

void PlugPlotWidgets::changeColour2()
{
	QPalette pal=buttonCol2->palette();
	clusterParameters[1].colour=QColorDialog::getColor(clusterParameters[1].colour, this);
	pal.setColor(QPalette::Active ,QPalette::ButtonText, clusterParameters[1].colour);
	buttonCol2->setPalette(pal);
	replot();
}

void PlugPlotWidgets::changeColour3()
{
	QPalette pal=buttonCol3->palette();
	clusterParameters[2].colour=QColorDialog::getColor(clusterParameters[2].colour, this);
	pal.setColor(QPalette::Active ,QPalette::ButtonText, clusterParameters[2].colour);
	buttonCol3->setPalette(pal);
	replot();
}

void PlugPlotWidgets::changeColour4()
{
	QPalette pal=buttonCol4->palette();
	clusterParameters[3].colour=QColorDialog::getColor(clusterParameters[3].colour, this);
	pal.setColor(QPalette::Active ,QPalette::ButtonText, clusterParameters[3].colour);
	buttonCol4->setPalette(pal);
	replot();
}

void PlugPlotWidgets::changeColourBg()
{
	QPalette pal=buttonBgColour->palette();
	bgColour=QColorDialog::getColor(bgColour, this);
	pal.setColor(QPalette::Active ,QPalette::ButtonText, bgColour);
	buttonBgColour->setPalette(pal);
	replot();
}

void PlugPlotWidgets::replot()
{
	PLINT old_strm;
	
	PLFLT min_x, max_x, min_y, max_y, min_z, max_z;
	
	PLFLT t_x[]={clusterParameters[0].position[0],
			clusterParameters[1].position[0],
			clusterParameters[2].position[0],
			clusterParameters[3].position[0]};
	PLFLT t_y[]={clusterParameters[0].position[1],
			clusterParameters[1].position[1],
			clusterParameters[2].position[1],
			clusterParameters[3].position[1]};
	PLFLT t_z[]={clusterParameters[0].position[2],
			clusterParameters[1].position[2],
			clusterParameters[2].position[2],
			clusterParameters[3].position[2]};

	min_x=*(min_element(t_x, t_x+sizeof(t_x)/sizeof(t_x[0])));
	max_x=*(max_element(t_x, t_x+sizeof(t_x)/sizeof(t_x[0])));
	min_y=*(min_element(t_y, t_y+sizeof(t_y)/sizeof(t_y[0])));
	max_y=*(max_element(t_y, t_y+sizeof(t_y)/sizeof(t_y[0])));
	min_z=*(min_element(t_z, t_z+sizeof(t_z)/sizeof(t_z[0])));
	max_z=*(max_element(t_z, t_z+sizeof(t_z)/sizeof(t_z[0])));
	
	if(min_x>=max_x || min_y>=max_y || min_z>=max_z) return;

	double length=max_x-min_x; min_x-=0.1*length; max_x+=0.1*length;
	length=max_y-min_y; min_y-=0.1*length; max_y+=0.1*length;
	length=max_z-min_z; min_z-=0.1*length; max_z+=0.1*length;
    
	plgstrm(&old_strm);
	plsstrm(strm);

	plwid(100); // to force reset so it gets used
	pladv(0);


    plvpor(0., 1., 0., 1.);
    PLFLT xLen = max_x - min_x;
    PLFLT yLen = max_y - min_y;
    PLFLT zLen = max_z - min_z;

    plwind(-1.4, 1.3, -1.2, 1.7);

    plwid(2);
    
	plw3d(1., yLen/xLen, 1.2*zLen/xLen,
		min_x, max_x,
		min_y, max_y,
		min_z, max_z,
	     alt, az);

	PLFLT cubex[10], cubey[10], cubez[10];

	plcol0(getColourIndex(bgColour));
	// Plot facet x=min_x
	cubex[0]=cubex[1]=cubex[2]=cubex[3]=min_x;
	cubey[0]=cubey[1]=min_y; cubey[2]=cubey[3]=max_y;
	cubez[0]=cubez[3]=min_z; cubez[1]=cubez[2]=max_z;
	plfill3(4, cubex, cubey, cubez);
	
	// Plot facet x=max_x
	cubex[0]=cubex[1]=cubex[2]=cubex[3]=max_x;
	plfill3(4, cubex, cubey, cubez);
	
	// Plot facet y=min_y
	cubey[0]=cubey[1]=cubey[2]=cubey[3]=min_y;
	cubex[0]=cubex[1]=min_x; cubex[2]=cubex[3]=max_x;
	cubez[0]=cubez[3]=min_z; cubez[1]=cubez[2]=max_z;
	plfill3(4, cubex, cubey, cubez);
	
	// Plot facet y=max_y
	cubey[0]=cubey[1]=cubey[2]=cubey[3]=max_y;
	plfill3(4, cubex, cubey, cubez);
	
	// Plot facet z=min_z
	cubez[0]=cubez[1]=cubez[2]=cubez[3]=min_z;
	cubex[0]=cubex[1]=min_x; cubex[2]=cubex[3]=max_x;
	cubey[0]=cubey[3]=min_y; cubey[1]=cubey[2]=max_y;
	plfill3(4, cubex, cubey, cubez);
	
	// Plot facet z=max_z
	cubez[0]=cubez[1]=cubez[2]=cubez[3]=max_z;
	plfill3(4, cubex, cubey, cubez);
	
	plcol0(0);
	
	//draw box contour
	pllsty(2);
	
	cubex[0]=cubex[3]=cubex[4]=cubex[5]=cubex[6]=cubex[9]=min_x;
	cubex[1]=cubex[2]=cubex[7]=cubex[8]=max_x;
	cubey[0]=cubey[1]=cubey[4]=cubey[5]=cubey[8]=cubey[9]=min_y;
	cubey[2]=cubey[3]=cubey[6]=cubey[7]=max_y;
	cubez[0]=cubez[1]=cubez[2]=cubez[3]=cubez[4]=min_z;
	cubez[5]=cubez[6]=cubez[7]=cubez[8]=cubez[9]=max_z;
	plline3(10, cubex, cubey, cubez);
	
	cubex[0]=cubex[1]=min_x;
	cubey[0]=cubey[1]=max_y;
	cubez[0]=min_z; cubez[1]=max_z;
	plline3(2, cubex, cubey, cubez);
	
	cubex[0]=cubex[1]=max_x;
	plline3(2, cubex, cubey, cubez);
	
	cubey[0]=cubey[1]=min_y;
	plline3(2, cubex, cubey, cubez);
	
	// plot axes
	pllsty(1);
    plschr(0., 0.8);

	plbox3("bnstu", (QString("x ")+frame).toLatin1().data(), 0, 0,
	       "bnstu", (QString("y ")+frame).toLatin1().data(), 0, 0,
			 "bnstu", (QString("z ")+frame).toLatin1().data(), 0, 0);
	
	for(int i=0; i<4; i++)
	{
		double x=(projection[0]?min_x:clusterParameters[i].position[0]);
		double y=(projection[1]?min_y:clusterParameters[i].position[1]);
		double z=(projection[2]?min_z:clusterParameters[i].position[2]);
		
		plssym(0.,4.);
		plcol0(getColourIndex(clusterParameters[i].colour));
		plpoin3(1, &x, &y, &z, 17);
// 		plptex3(x, y, z, 1., 0., 0., 0., 0., 0., 0.5, "X");
// 		plcol0(0);
		plwid(4);
		plssym(0.,1.);
		plcol0(0);
		plpoin3(1, &x, &y, &z, '1'+i);
		plwid(2);
	}
	
	// draw tetrahedron
	plcol0(0);
	
	cubex[0]=(projection[0]?min_x:clusterParameters[0].position[0]);
	cubey[0]=(projection[1]?min_y:clusterParameters[0].position[1]);
	cubez[0]=(projection[2]?min_z:clusterParameters[0].position[2]);
	
	cubex[1]=(projection[0]?min_x:clusterParameters[1].position[0]);
	cubey[1]=(projection[1]?min_y:clusterParameters[1].position[1]);
	cubez[1]=(projection[2]?min_z:clusterParameters[1].position[2]);
	
	cubex[2]=(projection[0]?min_x:clusterParameters[2].position[0]);
	cubey[2]=(projection[1]?min_y:clusterParameters[2].position[1]);
	cubez[2]=(projection[2]?min_z:clusterParameters[2].position[2]);
	
	cubex[3]=(projection[0]?min_x:clusterParameters[0].position[0]);
	cubey[3]=(projection[1]?min_y:clusterParameters[0].position[1]);
	cubez[3]=(projection[2]?min_z:clusterParameters[0].position[2]);
	
	cubex[4]=(projection[0]?min_x:clusterParameters[3].position[0]);
	cubey[4]=(projection[1]?min_y:clusterParameters[3].position[1]);
	cubez[4]=(projection[2]?min_z:clusterParameters[3].position[2]);
	
	cubex[5]=(projection[0]?min_x:clusterParameters[2].position[0]);
	cubey[5]=(projection[1]?min_y:clusterParameters[2].position[1]);
	cubez[5]=(projection[2]?min_z:clusterParameters[2].position[2]);
	
	plline3(6, cubex, cubey, cubez);
	
	cubex[0]=(projection[0]?min_x:clusterParameters[1].position[0]);
	cubey[0]=(projection[1]?min_y:clusterParameters[1].position[1]);
	cubez[0]=(projection[2]?min_z:clusterParameters[1].position[2]);
	
	cubex[1]=(projection[0]?min_x:clusterParameters[3].position[0]);
	cubey[1]=(projection[1]?min_y:clusterParameters[3].position[1]);
	cubez[1]=(projection[2]?min_z:clusterParameters[3].position[2]);
	
	plline3(2, cubex, cubey, cubez);
	
	// add title
	
	// add phi and theta
	
	plschr(0., 0.8);
	char text[40];
	sprintf(text, "#gh=%d", (int)(90.-alt));
	plmtex("rv", -1., 0.05, 1., text);
	sprintf(text, "#gf=%d", (int)az);
	plmtex("rv", -1., 0.08, 1., text);
	
	plschr(0., 0.9);
	plmtex("t", -1.2, 0.5, 0.5, title.toLatin1().data());
	
    win->plot->update();
    plsstrm(old_strm);

}

void PlugPlotWidgets::resetColourIndex()
{
	PLINT iR,iG,iB;
	
	userR.clear();
	userG.clear();
	userB.clear();
	for(int i=0;i<16;++i)
	{
		plgcol0(i,&iR,&iG,&iB);
		userR.push_back(iR);
		userG.push_back(iG);
		userB.push_back(iB);
	}
}

int PlugPlotWidgets::getColourIndex(const QColor& c)
{
	// find PGPlot colour index closest to this colour
	int R=c.red();
	int G=c.green();
	int B=c.blue();
	int min_diff=200000; // bigger than 255*255*3
	//int colIndex=0;
		
	int indexNearest=0;
		
	for(int i=0;i<userR.size();i++)
	{
		int diff=(R-userR[i])*(R-userR[i]) + (G-userG[i])*(G-userG[i]) + (B-userB[i])*(B-userB[i]);
		if(diff==0)
		{
			return i; // found in existing table
		}
		else if(diff<min_diff)
		{
			min_diff = diff;
			indexNearest=i;
		}
	}
		
	if( userR.size() < 40)
	{
		// create new colour and set in local copy
		
		userR.push_back(R);
		userG.push_back(G);
		userB.push_back(B);
	}
	else
	{
		// use nearest colour
		return indexNearest;
	}
	
	// set up colour table as new colour added
	plscmap0n(userR.size());
	plscol0(userR.size()-1, R, G, B);
	
	return userR.size()-1;
}



