/*****************************************************************
* PROJECT:  Cluster UK CDHF
* COMPONENT:  Plug In library
* MODULE:  TrigoFunct
* LANGUAGE:  C++
* FUNCTION: QSAS TrigoFunct  Plug In
* PURPOSE: .
* 	
* ARGUMENTS:  
*    IN: ANY DATA OBJECT
*        integer: function identifier
*	 integer: deg/rad
*    OUT: 
*	ANY DATA OBJECT 
*
* RETURN: QplugReturnStatus
*
* History: version 1.0 Created Jul. 2002. 
* 	   version 1.1 July 2002.
* A. Serroukh, 
******************************************************************/

#include<cstdio>
#include<cmath>
#include<string.h>
#include<cstdlib>
#include<fstream>

#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"


using namespace QSAS;

/*****************************************************************/
extern "C" QplugReturnStatus DvTrigoFunction(vector<DvObject_var>& inputs, vector<DvObject_var>& outputs){

    DvObject_var  any_obj = inputs[0];
    DvObject_var  obj_fct = inputs[1]; 
    DvObject_var  deg_or_rad = inputs[2];
     
    bool indeg = false;
    bool hyperbolic = true;
    
    DvObject_var  outObj; 
   
    //-----unpack and test the narrow operations ------------   
    if( any_obj.is_nil() ){
     QplugAppendTextDisplay("Error>> input neither const nor sequence.\n");
     return QPLUG_FAILURE;
    } 
               
    if(obj_fct.is_nil()){
      QplugAppendTextDisplay("Error>> undefined function\n");
      return QPLUG_FAILURE;
    }       
    int obj_fct_id = obj_fct->asInt();

    if(deg_or_rad->asInt() == 0) indeg = true;  
    if(obj_fct_id < 4 ) hyperbolic = false; 

    if(deg_or_rad.is_nil()){
        QplugAppendTextDisplay("Error>> Deg/Rad unspecified.\n");
        return QPLUG_FAILURE;
    }        
    if(deg_or_rad->asInt() == 0) indeg = true;  
   
            	 
    switch(obj_fct_id){
        case 0:           
            outObj = any_obj->cos();
            break;
        case 1:  
            outObj = any_obj->sin();
            break;
        case 2:  
            outObj = any_obj->tan();
            break;
        case 3:   
            outObj = any_obj->cot();	      
            break;
        case 4:  
            outObj = any_obj->cosh(); 
            break;
        case 5: 
            outObj = any_obj->sinh();	
            break;	
        case 6:  
            outObj = any_obj->tanh();
            break;
        case 7:           
            outObj = any_obj->coth();
            break;				
        default:         
            QplugAppendTextDisplay("Error>> Please choose from sin, cos,...\n");
            return QPLUG_FAILURE;
     } 
      
    if( outObj.is_nil()  ){
       QplugAppendTextDisplay("Error>> unvalid input to the trigonometry function.\n");
       QplugAppendTextDisplay("Error>> outTrig is nil object. \n");
       return QPLUG_FAILURE;
    } 
     
    outputs.push_back(outObj);  

    return QPLUG_SUCCESS;

}
