/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qplug_if.h"
#include "DvObj.h"

#include "qrtn.h"

extern "C" QplugReturnStatus   DvCoordRotation (vector<DvObject_var>& inputs, vector<DvObject_var>& outputs)
{

QplugReturnStatus	status;

RtnVector vec;

DvObject_var ts_in;
DvObject_var fr_ptr;

long check;
char *attr_value;

status = QPLUG_SUCCESS;

// identify plug-in version

QplugAppendTextDisplay (
        "MAH Coord Rotation: qrtn_plugin.cc (AJA)  8 Aug 1999, 18 Aug 2000, Feb 2006\n");


//	...first, unpack the input data objects

ts_in = inputs[0];
if ( ts_in.is_nil()  ) {	
    QplugAppendTextDisplay ("CoordRotation>> ERROR - input object not valid\n");
    return QPLUG_FAILURE; 
}  
  
if (  !ts_in->isThreeVector() ) {	
    QplugAppendTextDisplay ("CoordRotation>> ERROR - input not a Vector!\n");
    return QPLUG_FAILURE; 
}        

 vec.n_recs = ts_in->seqSize();

//       next item in list is int identifying target frame
//       acronyms are set to integer values by #define in qrtn.h
//       0  GSE,  Geocentric Solar Ecliptic
//       1  GSM,  Geocentric Solar Magnetic
//       2  GEOC, Geographic (geocentric)
//       3  SM,   Solar Magnetic
//       4  GEI,  Geocentric Inertial
//       5  MAGD,  Geomagnetic (dipole)
//       6  HAE,  Heliospheric Aries Ecliptic
//       7  HEE,  Heliospheric Earth Ecliptic
//       8  HEEQ, Heliospheric Earth Equatorial
//       9  GSEQ, Geocentric Solar Equatorial


    fr_ptr = inputs[1];
 
    if ( fr_ptr.is_nil() ) { 
        QplugAppendTextDisplay ("CoordRotation>> ERROR - argument not an integer flag!\n");
        return QPLUG_FAILURE; 
    }  

    vec.out_frame = fr_ptr->asInt();

    // Find original frame
 
    vec.in_frame = vec.QrGetFrameFromAttr(ts_in);

    // Find if this is a position vector

    vec.IsPositionVector = vec.QrIsPositionVector(ts_in);
    if(vec.IsPositionVector == 1){
      vec.IsPositionVector = 0; /* issue warning & continue, delta r rotates */
      
      if(vec.QrOriginsDiffer(vec.in_frame, vec.out_frame)) {
        QplugAppendTextDisplay 
          ("WARNING: CoordRotation plug-in provides frame rotation only.\n");
        QplugAppendTextDisplay 
          ("Transformation is between geocentric and heliocentric frames.\n");
        QplugAppendTextDisplay 
          ("Position vector coordinate transformations will be WRONG.\n");
        QplugAppendTextDisplay 
          ("Note: Spacecraft separations will rotate OK.\n");
      }
    }

    DvObject_var ts_out = new DvObject(ts_in);
    vec.ptr = ts_out;

    vec.t = ts_in->getTimeTags();
    if( vec.t->is_nil() ) {
        QplugAppendTextDisplay ("Cannot find time tags, some frames are time dependent\n");
        return QPLUG_FAILURE;
    }

    //  End of Plug-in wrapper, call C function to do rotation
 
    check = vec.QrDoRotation();

    if (check == QRTN_OK || check == SAME_FRAME) {
 
        //	..add  the output series to the list
  
        outputs.push_back(vec.ptr);
        DvString rmName = GetSlotText(1);
        rmName.clean();
        DvObject_var rmtx;
        if( !(rmName.empty()) ) {
            rmtx = vec.QrGetRotnMatx();
            outputs.push_back(rmtx);
        }
        
        if(check == SAME_FRAME){
            QplugAppendTextDisplay ("CoordRotation: Same frame, null operation\n");
        }
        else{
            QplugAppendTextDisplay ("CoordRotation: vector rotated \n");
        }
    }
    else{
        QplugAppendTextDisplay (vec.QrErrorMessage(check));
        status = QPLUG_FAILURE;
    } // end if on check


    return status;

}

