#include "IntervalSliders.h"
#include "IntervalWidgets.h"
#include <QTimer>


#define SLIDER_HEIGHT 20
#define TEXT_HEIGHT 20
#define SEPARATION 1
#define TOTAL_HEIGHT SLIDER_HEIGHT+TEXT_HEIGHT+SEPARATION

#define COLOUR_BAR_1 QColor(255, 255, 255)
#define COLOUR_BAR_2 QColor(230, 230, 230)
#define COLOUR_BAR_OUT QColor(180, 180, 180)
#define COLOUR_EVENT_1 QColor(250, 200, 200, 150)
#define COLOUR_EVENT_2 QColor(230, 170, 170, 150)
#define COLOUR_EVENT_OUT QColor(200, 130, 130)
#define COLOUR_CURRENT_1 QColor(170, 170, 170, 150)
#define COLOUR_CURRENT_2 QColor(150, 150, 150, 150)
#define COLOUR_CURRENT_OUT QColor(130, 130, 130, 150)

#define MOUSE_MODE_HOVER 0
#define MOUSE_MODE_MOVE_START 1
#define MOUSE_MODE_MOVE_END 2
#define MOUSE_MODE_MOVE_ALL 3
#define MOUSE_MODE_DOUBLE_CLICK 4

#define MARGIN 5 // Margin for "magnetism" or grabbing an extremity of the arrow

TimeSlider::TimeSlider(TimeIntervalWidget* parent):
	QWidget(parent)
{
	this->parent=parent;
	availabilityRange=parent->currentInterval;
	viewRange = parent->currentInterval;
    viewRange.set(viewRange.start()-viewRange.duration()/10., viewRange.end()+viewRange.duration()/10.);
	savedViewRange=viewRange;

	setMinimumSize(100, TOTAL_HEIGHT);
	
	displayedString="";
    barString="";
	hasAvailabilityRange=false;
	mouseMode=MOUSE_MODE_HOVER;
	setMouseTracking(true);
	setFocusPolicy(Qt::StrongFocus);
	updateDisplayedString();
}

TimeSlider::~TimeSlider()
{}

void TimeSlider::paintEvent(QPaintEvent*)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	double xstart, xend; // multi-purpose;
	QPainter p(this);
	QPen pen;
	QBrush brush;
	
	// Paint background bar
	pen.setStyle(Qt::SolidLine);
	pen.setColor(COLOUR_BAR_OUT);
	p.setPen(pen);
		
	QLinearGradient backgroundGradient(QPointF(0,  ymin), QPointF(0,  ymax));
	backgroundGradient.setColorAt(0., COLOUR_BAR_1);
	backgroundGradient.setColorAt(1., COLOUR_BAR_2);
	QBrush backgroundBrush(backgroundGradient);
	p.setBrush(backgroundBrush);
	
	p.drawRect(0, (int)ymin, (int)(width()-1), (int)(ymax-ymin-1));
    
	if(hasAvailabilityRange)
	{        
        
		DvTime ts = availabilityRange.start();
		xstart=timeToX(ts);
		DvTime te = availabilityRange.end();
		xend=timeToX(te);

		brush.setColor(COLOUR_BAR_OUT);
		brush.setStyle(Qt::DiagCrossPattern);
		p.setBrush(brush);
		if(xstart>=0.)
		{
			p.drawRect(0, (int)ymin, (int)xstart, (int)(ymax-ymin-1));
		}

		if(xend<width())
		{
			p.drawRect((int)xend, (int)ymin, (int)(width()-xend), (int)(ymax-ymin-1));
		}
	}

	// Paint events
	pen.setColor(COLOUR_EVENT_OUT);
	p.setPen(pen);
	
	QLinearGradient eventGradient(QPointF(0,  ymin), QPointF(0,  ymax));
	eventGradient.setColorAt(0., COLOUR_EVENT_1);
	eventGradient.setColorAt(1., COLOUR_EVENT_2);
	QBrush eventBrush(eventGradient);
	p.setBrush(eventBrush);

    if(parent->events.is_ok()){
        for(int i=0; i<parent->events->seqSize(); ++i)
        {
            DvTime ts = parent->events->event(i).start();
            xstart=timeToX(ts);
            DvTime te = parent->events->event(i).end();
            xend=timeToX(te);
            QRectF rect(xstart, ymin, xend-xstart, ymax-ymin-1.);
            rect=rect.normalized();
            p.drawRect(rect);
            QRectF box=p.boundingRect(rect, Qt::AlignCenter, QString("%1").arg(i));
            if(box.width()<=rect.width())
            {
                p.drawText(rect, Qt::AlignCenter, QString("%1").arg(i));
            }
        }
    }
        // Paint zoom in/zoom out buttons
    p.setBrush(backgroundBrush);
    pen.setColor(COLOUR_BAR_OUT);
    p.setPen(pen);
    QRectF zoomIn(width()-1-TEXT_HEIGHT, ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
    p.drawRect(zoomIn);
#else
    p.drawRoundedRect(zoomIn, 4., 4.);
#endif
    QPen linePen;
    
    QRectF zoomOut(width()-1-2.*TEXT_HEIGHT, ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
    p.drawRect(zoomOut);
#else
    p.drawRoundedRect(zoomOut, 4., 4.);
#endif
    
    
    linePen.setWidthF(2.0);
    linePen.setColor(COLOUR_CURRENT_OUT);
    p.setPen(linePen);
    // draw "+" and "-" symbols
    p.drawLine(QPointF(width()-TEXT_HEIGHT+5., ymin-SEPARATION-TEXT_HEIGHT/2.), QPointF(width()-5., ymin-SEPARATION-TEXT_HEIGHT/2.));
    p.drawLine(QPointF(width()-TEXT_HEIGHT/2., 5.), QPointF(width()-TEXT_HEIGHT/2., ymin-SEPARATION-5.));
    p.drawLine(QPointF(width()-2.*TEXT_HEIGHT+5., ymin-SEPARATION-TEXT_HEIGHT/2.), QPointF(width()-TEXT_HEIGHT-5., ymin-SEPARATION-TEXT_HEIGHT/2.));
    

    // Paint slide view left/right buttons
    p.setPen(pen);
//  QRectF slideLeft(0., ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
    QRectF slideLeft(0., ymax-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT-1);
#if QT_VERSION < 0x040300
    p.drawRect(slideLeft);
#else
    p.drawRoundedRect(slideLeft, 4., 4.);
#endif
    
    QRectF slideRight(width()-1-TEXT_HEIGHT, ymax-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT-1);
#if QT_VERSION < 0x040300
    p.drawRect(slideRight);
#else
    p.drawRoundedRect(slideRight, 4., 4.);
#endif
    
    // Draw symbols
    p.setPen(linePen);
    QPolygonF poly;
    poly << QPointF(width()-5., ymax-1-TEXT_HEIGHT/2.) << QPointF(width()-TEXT_HEIGHT+5., ymax-1-5.) << QPointF(width()-TEXT_HEIGHT+5., ymax-1-TEXT_HEIGHT/2-5.);
    p.drawConvexPolygon(poly);
    poly.clear();
    poly << QPointF(5., ymax-1-TEXT_HEIGHT/2.) << QPointF(TEXT_HEIGHT-5., ymax-1-5.) << QPointF(TEXT_HEIGHT-5., ymax-1-TEXT_HEIGHT/2-5.);
    p.drawConvexPolygon(poly);
    
	// Paint current time bar
	DvTime ts = parent->currentInterval.start();
	xstart=timeToX(ts);
	DvTime te = parent->currentInterval.end();
	xend=timeToX(te);
	pen.setColor(COLOUR_CURRENT_OUT);
	p.setPen(pen);
	
	QLinearGradient currentGradient(QPointF(0,  ymin), QPointF(0,  ymax));
	currentGradient.setColorAt(0., COLOUR_CURRENT_1);
	currentGradient.setColorAt(1., COLOUR_CURRENT_2);
	QBrush currentBrush(currentGradient);
	p.setBrush(currentBrush);
	
	if(xstart<xend)
	{
		const QPointF points[5] =
		{
			QPointF(xstart, ymin),
			QPointF(xstart, ymax),
			QPointF(xend, ymax),
			QPointF(xend+(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(xend, ymin)
		};
		p.drawConvexPolygon(points, 5);
	}
	else
	{
		const QPointF points[5] =
		{
			QPointF(xstart, ymin),
			QPointF(xstart, ymax),
			QPointF(xend, ymax),
			QPointF(xend-(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(xend, ymin)
		};
		p.drawConvexPolygon(points, 5);
	}

    if(!barString.isEmpty())
    {
        QRectF rect2(xstart, ymin, xend-xstart, ymax-ymin-1.);
        rect2=rect2.normalized();
        QRectF box2=p.boundingRect(rect2, Qt::AlignCenter, barString);
        if(box2.width()<=rect2.width())
        {
            p.drawText(rect2, Qt::AlignCenter, barString);
        }
    }

	pen.setStyle(Qt::DotLine);
	p.setPen(pen);
	p.drawLine(QPointF(xend, ymin), QPointF(xend, ymax));
	pen.setStyle(Qt::SolidLine);
	p.setPen(pen);
	
	// Draw context string
	
	QRectF box(0., ymin-SEPARATION-TEXT_HEIGHT, width(), TEXT_HEIGHT);
	pen.setColor(Qt::black);
	p.setPen(pen);
	p.drawText(box, Qt::AlignCenter, displayedString);
}

int TimeSlider::getEventIndex(DvTime& time)
{
	QVector<int> indexes;
	indexes.clear();
    
    if(parent->events.is_nil()) return -1;
    
	for(int i=0; i<parent->events->seqSize(); ++i)
	{
		if((time>=parent->events->event(i).start() && time<=parent->events->event(i).end()) || (time>=parent->events->event(i).end() && time<=parent->events->event(i).start()) )
		{
			indexes.push_back(i);
		}
	}
	if(indexes.size()==0) return -1;
	if(indexes.size()==1) return indexes[0];
	
	// choose the event which has its centre closest to x
	int chosen=indexes[0];
	double dist=fabs(time.getDiffSecs(parent->events->event(chosen).start()+parent->events->event(chosen).duration()/2.));
	double currentDist;
	for(int i=1; i<indexes.size(); ++i)
	{
		currentDist=fabs(time.getDiffSecs(parent->events->event(indexes[i]).start()+parent->events->event(indexes[i]).duration()/2.));
		if(currentDist<dist)
		{
			dist=currentDist;
			chosen=indexes[i];
		}
	}
	return chosen;
}

void TimeSlider::mouseDoubleClickEvent(QMouseEvent* e)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	if(mouseMode==MOUSE_MODE_HOVER)
	{
		mouseMode = MOUSE_MODE_DOUBLE_CLICK;
		qApp->processEvents(); // get rid of mouse press and mouse Release events		
		if(e->y()>=ymin && e->y()<=ymax)
		{
			DvTime t = xToTime(e->x());
			int index=getEventIndex(t);
			if(index >=0)
			{
                parent->currentInterval = parent->events->event(index);
				emit intervalChanged();
				qApp->processEvents(); // ensure interval changed before event set to ensure nameFld visible
				update();
				emit eventDoubleClick(index);
			}
		}
	}
 	QTimer::singleShot(200, this, SLOT(resetMode()) );	
	
	QWidget::mouseDoubleClickEvent(e);
}

void TimeSlider::resetMode()
{
	mouseMode=MOUSE_MODE_HOVER;
}

void TimeSlider::mouseMoveEvent(QMouseEvent* e)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;

	if(mouseMode==MOUSE_MODE_HOVER)
	{
		if(e->y()>=ymin && e->y()<=ymax)
		{
            if(e->x()<TEXT_HEIGHT)
            {
                setCursor(Qt::ArrowCursor);
                displayedString=QString("Slide view range left");
                update();
            }
            else if(e->x()>=width()-TEXT_HEIGHT)
            {
                setCursor(Qt::ArrowCursor);
                displayedString=QString("Slide view range right");
                update();
            }
            else
            {
			    // if Mouse over event, display event number
			    DvTime mouseTime=xToTime(e->x());
			    int index=getEventIndex(mouseTime);
			    if(index>=0)
			    {
				    QString str1=parent->events->event(index).start().getISOstring().c_str();
				    str1.truncate(23);
				    QString str2=parent->events->event(index).end().getISOstring().c_str();
				    str2.truncate(23);
				    displayedString=QString("Event #%1: %2 to %3").arg(index).arg(str1).arg(str2);
				    update();
			    }
			    else
			    {
				    QString str=mouseTime.getISOstring().c_str();
				    str.truncate(23);
				    displayedString=str;
				    update();
			    }
			    
				DvTime ts = parent->currentInterval.start();
			    double xstart=timeToX(ts);
				DvTime te = parent->currentInterval.end();
			    double xend=timeToX(te);
			    
                if(e->x() > xstart-MARGIN && e->x() < xstart+MARGIN&& fabs(xstart-xend)>2)
			    {
				    setCursor( Qt::SizeHorCursor);
			    }
                else if(e->x() > xend-MARGIN && e->x() < xend+MARGIN&& fabs(xstart-xend)>2)
			    {
				    setCursor( Qt::SizeHorCursor);
			    }
			    else if( (e->x() < xstart-MARGIN && e->x() < xend-MARGIN) || (e->x() > xstart+MARGIN && e->x() > xend+MARGIN) )
			    {
				    setCursor(Qt::ArrowCursor);
			    }
			    else
			    {
				    setCursor(Qt::OpenHandCursor);
			    }
            }
			
		}
		else
		{
			setCursor(Qt::ArrowCursor);
			if(e->y()<ymin-SEPARATION) // Possibly on a button
			{
				if(e->x()<width()-TEXT_HEIGHT && e->x()>=width()-2.*TEXT_HEIGHT)
				{
					displayedString=QString("Zoom view range out");
					update();
				}
				else if(e->x()>=width()-TEXT_HEIGHT)
				{
					displayedString=QString("Zoom view range in");
					update();
				}
				else
				{
					updateDisplayedString();
				}
			}
			else
			{
				updateDisplayedString();
			}
		}
	}
	else if(mouseMode==MOUSE_MODE_MOVE_ALL)
	{
		DvTime mouseTime=xToTime(e->x());
		double intervalWidth=parent->currentInterval.duration();
		parent->currentInterval.set(mouseTime-intervalWidth*relativePosition, mouseTime-intervalWidth*relativePosition+intervalWidth);
        QString str1=parent->currentInterval.start().getISOstring().c_str();
        str1.truncate(23);
        QString str2=parent->currentInterval.end().getISOstring().c_str();
        str2.truncate(23);
        displayedString=QString("%1 to %2").arg(str1).arg(str2);
		update();
	}
	else if(mouseMode==MOUSE_MODE_MOVE_START)
	{
		DvTime mouseTime=xToTime(e->x());
		parent->currentInterval.set_start(mouseTime);
        QString str1=parent->currentInterval.start().getISOstring().c_str();
        str1.truncate(23);
        QString str2=parent->currentInterval.end().getISOstring().c_str();
        str2.truncate(23);
        displayedString=QString("%1 to %2").arg(str1).arg(str2);
		update();
	}
	else if(mouseMode==MOUSE_MODE_MOVE_END)
	{
		DvTime mouseTime=xToTime(e->x());
		parent->currentInterval.set_end(mouseTime);
        QString str1=parent->currentInterval.start().getISOstring().c_str();
        str1.truncate(23);
        QString str2=parent->currentInterval.end().getISOstring().c_str();
        str2.truncate(23);
        displayedString=QString("%1 to %2").arg(str1).arg(str2);
		update();
	}
}

void TimeSlider::setViewRange(DvEvent range)
{
	if(range.duration_abs()<1.)
	{
		DvEvent newRange=range;
		newRange.set_end(range.end()+1.);
		viewRange=range;
	}
	else
	{
		viewRange=range;
	}
	savedViewRange = range;
	update();
}

void TimeSlider::setViewRange(DvObject_var& eventSeq)
{
	if(eventSeq.is_nil()) return;
	if(!eventSeq->is_event()) return;
	
	
	DvTime min=eventSeq->event(0).start();
	DvTime max=min;
	for(int i=0; i<eventSeq->seqSize(); ++i)
	{
		if(eventSeq->event(i).start()<min)
			min=eventSeq->event(i).start();
		if(eventSeq->event(i).start()>max)
			max=eventSeq->event(i).start();
		if(eventSeq->event(i).end()<min)
			min=eventSeq->event(i).end();
		if(eventSeq->event(i).end()>max)
			max=eventSeq->event(i).end();
	}
	DvEvent ivl(min, max);
	setViewRange(ivl);
}

void TimeSlider::unsetAvailabilityRange()
{
	hasAvailabilityRange=false;
	update();
}

void TimeSlider::setAvailabilityRange(DvEvent& range)
{
	hasAvailabilityRange=true;
	availabilityRange=range;
	update();
}

void TimeSlider::leaveEvent(QEvent*)
{
	updateDisplayedString();
}

void TimeSlider::mousePressEvent(QMouseEvent* e)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	if(mouseMode!=MOUSE_MODE_HOVER)
		return;
	
	
	// Now identify what's been clicked on
	
	if(e->y()<ymin-SEPARATION) // Possibly on a button
	{
		if(e->x()<width()-TEXT_HEIGHT && e->x()>=width()-2.*TEXT_HEIGHT)
		{
			zoomOut();
		}
		else if(e->x()>=width()-TEXT_HEIGHT)
		{
			zoomIn();
		}
		else
		{
			updateDisplayedString();
		}
	}
	else if(e->y()>=ymin)
	{
        if(e->x()<TEXT_HEIGHT)
        {
            offsetLeft();
        }
        else if(e->x()>=width()-TEXT_HEIGHT)
        {
            offsetRight();
        }
        else
        {
			DvTime t_start = parent->currentInterval.start();
		    double xstart=timeToX(t_start);
			
			DvTime t_end = parent->currentInterval.end();
		    double xend=timeToX(t_end);
		    double intervalWidth=(xend>xstart? parent->currentInterval.duration() : parent->currentInterval.duration());
		    
		    if(e->x() < xstart-MARGIN && e->x() < xend-MARGIN)
		    {
			    // offset to the left
			    parent->currentInterval.set(parent->currentInterval.start()-intervalWidth, parent->currentInterval.end()-intervalWidth);
			    update();
			    emit intervalChanged();
		    }
		    else if(e->x() > xstart+MARGIN && e->x() > xend+MARGIN)
		    {
			    // offset to the right
			    parent->currentInterval.set(parent->currentInterval.start()+intervalWidth, parent->currentInterval.end()+intervalWidth);
			    update();
			    emit intervalChanged();
		    }
		    else if(e->x()>xstart-MARGIN && e->x() < xstart+MARGIN && fabs(xstart-xend)>2)
		    {
			    // moving start
			    mouseMode=MOUSE_MODE_MOVE_START;
		    }
            else if(e->x() > xend-MARGIN && e->x() < xend+MARGIN && fabs(xstart-xend)>2)
		    {
			    // moving end
			    mouseMode=MOUSE_MODE_MOVE_END;
		    }
		    else
		    {
			    // moving all interval
			    mouseMode=MOUSE_MODE_MOVE_ALL;
			    DvTime mouseTime=xToTime(e->x());
			    relativePosition=mouseTime.getDiffSecs(parent->currentInterval.start())/(parent->currentInterval.duration());
		    }
        }
	}
}

void TimeSlider::mouseReleaseEvent(QMouseEvent* e)
{
	if(mouseMode==MOUSE_MODE_HOVER)
	{
		QWidget::mouseReleaseEvent(e);
		return;
	}

	if(mouseMode==MOUSE_MODE_DOUBLE_CLICK) return;
	
	mouseMode=MOUSE_MODE_HOVER;
	emit intervalChanged();
}

void TimeSlider::updateDisplayedString()
{
	QString fromString=QString(viewRange.start().getISOstring(0).c_str());
	fromString.truncate(23);
	QString toString=QString(viewRange.end().getISOstring(0).c_str());
	toString.truncate(23);
	displayedString=("View range: ")+fromString+QString(" to ")+toString;
	update();
}

double TimeSlider::timeToX(DvTime& time)
{
// 	return offset/duration*width();
    double duration=viewRange.duration();
	if(duration < 1.e-9) return 0;
    double offset=time.getDiffSecs(viewRange.start());
    return offset/duration*(width()-2.*TEXT_HEIGHT)+TEXT_HEIGHT;
}

DvTime TimeSlider::xToTime(double x)
{
// 	double duration=viewRange.duration();
// 	return viewRange.start()+x/((double)width())*duration;
    double duration=viewRange.duration();
    return viewRange.start()+(x-TEXT_HEIGHT)/((double)width()-2.*TEXT_HEIGHT)*duration;
}

void TimeSlider::resetView()
{
	viewRange=savedViewRange;
	update();
}

void TimeSlider::resetWidget()
{
	if(parent->events.is_nil() || parent->events->seqSize() == 0) parent->currentInterval = savedViewRange;
	else parent->currentInterval = parent->events->event(0);
	update();
}

void TimeSlider::changeViewRangeSlot()
{
	TimeIntervalDialog* dialog=new TimeIntervalDialog(viewRange, this);
	dialog->exec();
	if(dialog->result()==QDialog::Accepted)
	{
		viewRange=dialog->newRange;
		update();
	}
}

void TimeSlider::offsetView(double factor)
{
	double range=viewRange.duration();
	DvTime start=viewRange.start();
	
	viewRange.set(start+range*factor, start+range*factor+range);
	repaint();
}

void TimeSlider::zoom(double factor)
{
	double range=viewRange.duration();
	DvTime centre=viewRange.start()+viewRange.duration()/2.;
	
	viewRange.set(centre-range/2.*factor, centre+range/2.*factor);
	update();
}

void TimeSlider::offsetLeft()
{
	for(int i=0; i<10; i++) {
		offsetView(-0.025); 
		qApp->processEvents();
		QTime wait; 
		wait.start(); 
		while(wait.elapsed() < 10) ;
	}
}

void TimeSlider::offsetRight()
{
	for(int i=0; i<10; i++) {
		offsetView(0.025); 
		qApp->processEvents();
		QTime wait; 
		wait.start(); 
		while(wait.elapsed() < 10) ;
	}
}

TimeIntervalDialog::TimeIntervalDialog(DvEvent& interval, QWidget* parent, Qt::WindowFlags f):
		QDialog(parent, f)
{	
	QVBoxLayout* mainLayout=new QVBoxLayout;
	widget=new TimeIntervalWidget(this);
	widget->setInterval(interval);
	
	mainLayout->addWidget(widget);
	
	QHBoxLayout* buttonsLayout=new QHBoxLayout;
	
	QPushButton* okButton=new QPushButton("OK");
	QPushButton* cancelButton=new QPushButton("Cancel");
	buttonsLayout->addWidget(okButton);
	buttonsLayout->addWidget(cancelButton);
	
	mainLayout->addLayout(buttonsLayout);
	
	setLayout(mainLayout);
	
	connect(okButton, SIGNAL(clicked()), this, SLOT(validated()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelled()));
}

TimeIntervalDialog::~TimeIntervalDialog()
{
	delete widget;
}

void TimeIntervalDialog::validated()
{
	newRange=widget->getInterval();
	accept();
}

void TimeIntervalDialog::cancelled()
{
	reject();
}


// ------------     class ScalarIntervalSlider   ------------------
// ----------------------------------------------------------------


ScalarIntervalSlider::ScalarIntervalSlider(ScalarIntervalWidget* parent, double * start, double * end):
		QWidget(parent)
{
	this->parent=parent;
	currentStart=start; currentEnd=end;
	availabilityStart=*start; availabilityEnd=*end;
	viewStart=*start; viewEnd=*end;
	savedViewStart=*start; savedViewEnd=*end;

	setMinimumSize(100, TOTAL_HEIGHT);
	
	displayedString="";
    barString="";
	hasAvailabilityRange=false;
	mouseMode=MOUSE_MODE_HOVER;
	setMouseTracking(true);
	setFocusPolicy(Qt::StrongFocus);
	updateDisplayedString();
}

ScalarIntervalSlider::~ScalarIntervalSlider()
{}

void ScalarIntervalSlider::paintEvent(QPaintEvent*)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	double xstart, xend; // multi-purpose;

	QPainter p(this);
	QPen pen;
	QBrush brush;
	
	// Paint background bar
	pen.setStyle(Qt::SolidLine);
	pen.setColor(COLOUR_BAR_OUT);
	p.setPen(pen);
		
	QLinearGradient backgroundGradient(QPointF(0,  ymin), QPointF(0,  ymax));
	backgroundGradient.setColorAt(0., COLOUR_BAR_1);
	backgroundGradient.setColorAt(1., COLOUR_BAR_2);
	QBrush backgroundBrush(backgroundGradient);
	p.setBrush(backgroundBrush);
	
	p.drawRect(0, (int)ymin, width()-1, (int)(ymax-ymin-1));

	if(hasAvailabilityRange)
	{
		xstart=valToX(availabilityStart);
		xend=valToX(availabilityEnd);

		brush.setColor(COLOUR_BAR_OUT);
		brush.setStyle(Qt::DiagCrossPattern);
		p.setBrush(brush);
		if(xstart>=0.)
		{
			p.drawRect(0, (int)ymin, (int)xstart, (int)(ymax-ymin-1));
		}

		if(xend<width())
		{
			p.drawRect((int)xend, (int)ymin, (int)(width()-xend), (int)(ymax-ymin-1));
		}
	}
	
	// Paint current time bar
	xstart=valToX(*currentStart);
	xend=valToX(*currentEnd);
	
	pen.setColor(COLOUR_CURRENT_OUT);
	p.setPen(pen);
	
	QLinearGradient currentGradient(QPointF(0,  ymin), QPointF(0,  ymax));
	currentGradient.setColorAt(0., COLOUR_CURRENT_1);
	currentGradient.setColorAt(1., COLOUR_CURRENT_2);
	QBrush currentBrush(currentGradient);
	p.setBrush(currentBrush);
	
	if(xstart<xend)
	{
		const QPointF points[5] =
		{
			QPointF(xstart, ymin),
			QPointF(xstart, ymax),
			QPointF(xend, ymax),
			QPointF(xend+(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(xend, ymin)
		};
		p.drawConvexPolygon(points, 5);
	}
	else
	{
		const QPointF points[5] =
		{
			QPointF(xstart, ymin),
			QPointF(xstart, ymax),
			QPointF(xend, ymax),
			QPointF(xend-(ymax-ymin)/2., ymin+(ymax-ymin)/2.),
			QPointF(xend, ymin)
		};
		p.drawConvexPolygon(points, 5);
	}
    if(!barString.isEmpty())
    {
        QRectF rect2(xstart, ymin, xend-xstart, ymax-ymin-1.);
        rect2=rect2.normalized();
        QRectF box2=p.boundingRect(rect2, Qt::AlignCenter, barString);
        if(box2.width()<=rect2.width())
        {
            p.drawText(rect2, Qt::AlignCenter, barString);
        }
    }
	pen.setStyle(Qt::DotLine);
	p.setPen(pen);
	p.drawLine(QPointF(xend, ymin), QPointF(xend, ymax));
	pen.setStyle(Qt::SolidLine);
	p.setPen(pen);

	// Paint zoom in/zoom out buttons
	p.setBrush(backgroundBrush);
	pen.setColor(COLOUR_BAR_OUT);
	p.setPen(pen);
	QRectF zoomIn(width()-1-TEXT_HEIGHT, ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
	p.drawRect(zoomIn);
#else
	p.drawRoundedRect(zoomIn, 4., 4.);
#endif
	QPen linePen;
	
	QRectF zoomOut(width()-1-2.*TEXT_HEIGHT, ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
	p.drawRect(zoomOut);
#else
	p.drawRoundedRect(zoomOut, 4., 4.);
#endif
	
	
	linePen.setWidthF(2.0);
	linePen.setColor(COLOUR_CURRENT_OUT);
	p.setPen(linePen);
	// draw "+" and "-" symbols
	p.drawLine(QPointF(width()-TEXT_HEIGHT+5., ymin-SEPARATION-TEXT_HEIGHT/2.), QPointF(width()-5., ymin-SEPARATION-TEXT_HEIGHT/2.));
	p.drawLine(QPointF(width()-TEXT_HEIGHT/2., 5.), QPointF(width()-TEXT_HEIGHT/2., ymin-SEPARATION-5.));
	p.drawLine(QPointF(width()-2.*TEXT_HEIGHT+5., ymin-SEPARATION-TEXT_HEIGHT/2.), QPointF(width()-TEXT_HEIGHT-5., ymin-SEPARATION-TEXT_HEIGHT/2.));
	

	// Paint slide view left/right buttons
	p.setPen(pen);
	QRectF slideLeft(0., ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
	p.drawRect(slideLeft);
#else
	p.drawRoundedRect(slideLeft, 4., 4.);
#endif
	
	QRectF slideRight(TEXT_HEIGHT, ymin-SEPARATION-TEXT_HEIGHT, TEXT_HEIGHT, TEXT_HEIGHT);
#if QT_VERSION < 0x040300
	p.drawRect(slideRight);
#else
	p.drawRoundedRect(slideRight, 4., 4.);
#endif
	
	// Draw symbols
	p.setPen(linePen);
	QPolygonF poly;
	poly << QPointF(TEXT_HEIGHT*2.-5., ymin-SEPARATION-TEXT_HEIGHT/2.) << QPointF(TEXT_HEIGHT+5., ymin-SEPARATION-5.) << QPointF(TEXT_HEIGHT+5., 5.);
	p.drawConvexPolygon(poly);
	poly.clear();
	poly << QPointF(5., ymin-SEPARATION-TEXT_HEIGHT/2.) << QPointF(TEXT_HEIGHT-5., ymin-SEPARATION-5.) << QPointF(TEXT_HEIGHT-5., 5.);
	p.drawConvexPolygon(poly);
	
	// Draw context string
	
	QRectF box(0., ymin-SEPARATION-TEXT_HEIGHT, width(), TEXT_HEIGHT);
	pen.setColor(Qt::black);
	p.setPen(pen);
	p.drawText(box, Qt::AlignCenter, displayedString);
}

void ScalarIntervalSlider::mouseMoveEvent(QMouseEvent* e)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	if(mouseMode==MOUSE_MODE_HOVER)
	{
		if(e->y()>=ymin && e->y()<=ymax)
		{
			// if Mouse over event, display event number
			double mouseVal=xToVal(e->x());
			displayedString=QString("%1").arg(mouseVal);
			update();
			
			double xstart=valToX(*currentStart);
			double xend=valToX(*currentEnd);
			
            if(e->x() > xstart-MARGIN && e->x() < xstart+MARGIN && fabs(xstart-xend)>2)
			{
				setCursor( Qt::SizeHorCursor);
			}
			else if(e->x() > xend-MARGIN && e->x() < xend+MARGIN && fabs(xstart-xend)>2)
			{
				setCursor( Qt::SizeHorCursor);
			}
			else if((e->x() < xstart-MARGIN && e->x() < xend-MARGIN ) || (e->x() > xstart+MARGIN && e->x() > xend+MARGIN) )
			{
				setCursor(Qt::ArrowCursor);
			}
			else
			{
				setCursor(Qt::OpenHandCursor);
			}
			
		}
		else
		{
			setCursor(Qt::ArrowCursor);
			if(e->y()<ymin-SEPARATION) // Possibly on a button
			{
				if(e->x()<TEXT_HEIGHT)
				{
					displayedString=QString("Slide viewport left");
					update();
				}
				else if(e->x()<2*TEXT_HEIGHT)
				{
					displayedString=QString("Slide viewport right");
					update();
				}
				else if(e->x()<width()-TEXT_HEIGHT && e->x()>=width()-2.*TEXT_HEIGHT)
				{
					displayedString=QString("Zoom viewport out");
					update();
				}
				else if(e->x()>=width()-TEXT_HEIGHT)
				{
					displayedString=QString("Zoom viewport in");
					update();
				}
				else
				{
					updateDisplayedString();
				}
			}
			else
			{
				updateDisplayedString();
			}
		}
	}
	else if(mouseMode==MOUSE_MODE_MOVE_ALL)
	{
		double mouseVal=xToVal(e->x());

		double intervalWidth=*currentEnd-*currentStart;
		*currentStart=mouseVal-intervalWidth*relativePosition; *currentEnd=mouseVal-intervalWidth*relativePosition+intervalWidth;
		updateDisplayedString();
		update();
		emit intervalChanged();
	}
	else if(mouseMode==MOUSE_MODE_MOVE_START)
	{
		double mouseVal=xToVal(e->x());
		*currentStart=mouseVal;
		updateDisplayedString();
		update();
		emit intervalChanged();
	}
	else if(mouseMode==MOUSE_MODE_MOVE_END)
	{
		double mouseVal=xToVal(e->x());
		*currentEnd=mouseVal;
		updateDisplayedString();
		update();
		emit intervalChanged();
	}
}

void ScalarIntervalSlider::setViewRange(double start, double end)
{
	savedViewStart=start; savedViewEnd=end;
	viewStart=start; viewEnd=end;
	updateDisplayedString();
	update();
}

void ScalarIntervalSlider::unsetAvailabilityRange()
{
	hasAvailabilityRange=false;
	update();
}

void ScalarIntervalSlider::setAvailabilityRange(double start, double end)
{
	hasAvailabilityRange=true;
	availabilityStart=start;
	availabilityEnd=end;
	update();
}

void ScalarIntervalSlider::leaveEvent(QEvent*)
{
	updateDisplayedString();
}

void ScalarIntervalSlider::mousePressEvent(QMouseEvent* e)
{
	double ymin=(height()-TOTAL_HEIGHT)/2.;
	double ymax=ymin+SLIDER_HEIGHT;
	
	if(mouseMode!=MOUSE_MODE_HOVER)
		return;
	
	
	// Now identify what's been clicked on
	
	if(e->y()<ymin-SEPARATION) // Possibly on a button
	{
		if(e->x()<TEXT_HEIGHT)
		{
			offsetLeft();
		}
		else if(e->x()<2*TEXT_HEIGHT)
		{
			offsetRight();
		}
		else if(e->x()<width()-TEXT_HEIGHT && e->x()>=width()-2.*TEXT_HEIGHT)
		{
			zoomOut();
		}
		else if(e->x()>=width()-TEXT_HEIGHT)
		{
			zoomIn();
		}
		else
		{
			updateDisplayedString();
		}
	}
	else if(e->y()>=ymin)
	{
		double xstart=valToX(*currentStart);
		double xend=valToX(*currentEnd);
		double intervalWidth=(xend>xstart? *currentEnd-*currentStart : *currentStart-*currentEnd);
		
		if(e->x() < xstart-MARGIN && e->x() < xend-MARGIN)
		{
			// offset to the left
			*currentStart-=intervalWidth;
			*currentEnd-=intervalWidth;
			update();
			emit intervalChanged();
		}
		else if(e->x() > xstart+MARGIN && e->x() > xend+MARGIN)
		{
			// offset to the right
			*currentStart+=intervalWidth;
			*currentEnd+=intervalWidth;
			update();
			emit intervalChanged();
		}
        else if(e->x()>xstart-MARGIN && e->x() < xstart+MARGIN && fabs(xstart-xend)>2)
		{
			// moving start
			mouseMode=MOUSE_MODE_MOVE_START;
		}
        else if(e->x() > xend-MARGIN && e->x() < xend+MARGIN && fabs(xstart-xend)>2)
		{
			// moving end
			mouseMode=MOUSE_MODE_MOVE_END;
		}
		else
		{
			// moving all interval
			mouseMode=MOUSE_MODE_MOVE_ALL;
			double mouseVal=xToVal(e->x());
			relativePosition=(mouseVal-*currentStart)/(*currentEnd-*currentStart);
		}
	}
}

void ScalarIntervalSlider::mouseReleaseEvent(QMouseEvent* e)
{
	if(mouseMode==MOUSE_MODE_HOVER)
	{
		QWidget::mouseReleaseEvent(e);
		return;
	}
	
	mouseMode=MOUSE_MODE_HOVER;
	emit intervalChanged();
}

void ScalarIntervalSlider::updateDisplayedString()
{
	QString fromString=QString("%1").arg(viewStart);
	QString toString=QString("%1").arg(viewEnd);
	displayedString=("View range: ")+fromString+QString(" to ")+toString;
	update();
}

double ScalarIntervalSlider::valToX(double val)
{
	double length=viewEnd-viewStart;
	double offset=val-viewStart;
	return offset/length*width();
}

double ScalarIntervalSlider::xToVal(double x)
{
	double length=viewEnd-viewStart;
	return viewStart+x/((double)width())*length;
}

void ScalarIntervalSlider::resetView()
{
	viewStart=savedViewStart;
	viewEnd=savedViewEnd;
	update();
}

void ScalarIntervalSlider::changeViewRangeSlot()
{
// 	TimeIntervalDialog* dialog=new TimeIntervalDialog(viewRange, this);
// 	dialog->exec();
// 	if(dialog->result()==QDialog::Accepted)
// 	{
// 		viewRange=dialog->newRange;
// 		update();
// 	}
}

void ScalarIntervalSlider::offsetView(double factor)
{
	double length=viewEnd-viewStart;

	viewStart+=length*factor;
	viewEnd+=length*factor;
	repaint();
}

void ScalarIntervalSlider::zoom(double factor)
{
	double length=viewEnd-viewStart;
	double centre=viewStart+length/2.;

	viewStart=centre-length/2.*factor;
	viewEnd=centre+length/2.*factor;
	update();
}

void ScalarIntervalSlider::offsetLeft()
{
	for(int i=0; i<10; i++) {
		offsetView(-0.025); 
		qApp->processEvents();
		QTime wait; 
		wait.start(); 
		while(wait.elapsed() < 10) ;
	}
}

void ScalarIntervalSlider::offsetRight()
{
	for(int i=0; i<10; i++) {
		offsetView(0.025); 
		qApp->processEvents();
		QTime wait; 
		wait.start(); 
		while(wait.elapsed() < 10) ;
	}
}


