#include "qprofui.h"
#include <QMessageBox>
#include <QMenu>

PresetSelector::PresetSelector(QWidget * parent) : QDialog(parent)
{

	QGridLayout * layout=new QGridLayout(this);
	
	QComboBox * combo=new QComboBox(this);
	layout->addWidget(combo, 0, 0, 1, 2);
	
	std::vector<QString> names;
	
	QSAS_profiles->names(names);
	
	for(int i=0; i<names.size(); ++i)
	{
		if(names[i]!="Default" ) combo->addItem(names[i]);
	}
	
	connect(combo, SIGNAL(activated(const QString&)), this, SLOT(setSelected(const QString&)));
	
	QPushButton * ok=new QPushButton("OK", this);
	layout->addWidget(ok, 1, 0);
	connect(ok, SIGNAL(clicked()), this, SLOT(okPressed()));
	
	QPushButton * cancel=new QPushButton("Cancel", this);
	layout->addWidget(cancel, 1, 1);
	connect(cancel, SIGNAL(clicked()), this, SLOT(cancelPressed()));
	
	selectedPreset=combo->itemText(0);
	
	setWindowTitle("Select a preset");
	
	resize(250, 150);
}

void PresetSelector::setSelected(const QString& select)
{
	selectedPreset=select;
}

void PresetSelector::okPressed()
{
	done(QDialog::Accepted);
}

void PresetSelector::cancelPressed()
{
	done(QDialog::Rejected);
}

QProfileEditor::QProfileEditor()
{
	
 	QWidget* mainWidget=new QWidget(this);
	this->setCentralWidget(mainWidget);
	QGridLayout * layout=new QGridLayout(mainWidget);
	
	profilesList=new QComboBox(mainWidget);

	layout->addWidget(profilesList, 0, 1);
	
	QTabWidget* tabWidget=new QTabWidget(mainWidget);
	
	pathWidget=new PathConfig;
	tabWidget->addTab(pathWidget, "Paths setup");
	pathWidget->setSelectedProfile(QSAS_profiles->current());
	
	plotWidget=new PlotConfig;
	tabWidget->addTab(plotWidget, "Plot (1/2)");
	plotWidget->setSelectedProfile(QSAS_profiles->current());
	
	plotWidget2=new PlotConfig2;
	tabWidget->addTab(plotWidget2, "Plot (2/2)");
	plotWidget2->setSelectedProfile(QSAS_profiles->current());
	
	guiWidget=new GuiConfig;
	tabWidget->addTab(guiWidget, "User Interface");
	guiWidget->setSelectedProfile(QSAS_profiles->current());
	
	miscWidget=new MiscConfig;
	tabWidget->addTab(miscWidget, "Debug");
	miscWidget->setSelectedProfile(QSAS_profiles->current());
	
	layout->addWidget(tabWidget, 1, 0, 1, 4);
	
	layout->addWidget(new QLabel("Options marked \"*\" will take effect next time you start QSAS", mainWidget), 2, 0, 1, 3);

	applyCloseButton=new QPushButton("Apply and close", mainWidget);
	layout->addWidget(applyCloseButton, 3, 0);
	connect(applyCloseButton, SIGNAL(clicked()), this, SLOT(apply_and_close()));

	applyButton=new QPushButton("Apply", mainWidget);
	layout->addWidget(applyButton, 3, 1);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	
	saveAsButton=new QPushButton("Save as...", mainWidget);
	layout->addWidget(saveAsButton, 3, 2);
	connect(saveAsButton, SIGNAL(clicked()), this, SLOT(saveAs()));
	
	deleteButton=new QPushButton("Delete...", mainWidget);
	layout->addWidget(deleteButton, 3, 3);
	connect(deleteButton, SIGNAL(clicked()), this, SLOT(deletePreset()));


	setWindowTitle("Profiles Editor");
	
	layout->setRowStretch(0, 1);
	layout->setRowStretch(1, 10);
	layout->setRowStretch(2, 1);
	layout->setRowStretch(3, 1);
	
	resize(600, 600);

	updateProfilesList();

	this->show();
}


void QProfileEditor::setSelectedProfile(int index)
{
	QString name=profilesList->itemText(index);
	QProfile * selected=QSAS_profiles->fromName(name);
	if(selected!=NULL)
	{
		plotWidget->setSelectedProfile(selected);
		plotWidget2->setSelectedProfile(selected);
		pathWidget->setSelectedProfile(selected);
		guiWidget->setSelectedProfile(selected);
		miscWidget->setSelectedProfile(selected);
		
		if(name == "Default") {
			applyCloseButton->setEnabled(false);
			applyButton->setEnabled(false);
			deleteButton->setEnabled(false);
		}
		else{
			applyCloseButton->setEnabled(true);
			applyButton->setEnabled(true);
			deleteButton->setEnabled(true);
		}
		
		QSAS_profiles->setCurrent(name);
		emit(changedNames());
	}
}

void QProfileEditor::updateProfilesList()
{
	std::vector<QString> names;
	QSAS_profiles->names(names);
	int selected=0;
	profilesList->clear();

	for(int i=0; i<names.size(); ++i)
	{
		if(names[i]==QSAS_profiles->current()->getName()) selected=i;
		profilesList->addItem(names[i]);

	}
	profilesList->setCurrentIndex(selected);
	
	// cannot change Default without saveAs
	
	if(selected == 0) {
		applyCloseButton->setEnabled(false);
		applyButton->setEnabled(false);
		deleteButton->setEnabled(false);
	}
	else{
		applyCloseButton->setEnabled(true);
		applyButton->setEnabled(true);
		deleteButton->setEnabled(true);
	}
	
	connect(profilesList, SIGNAL(activated(int)), this, SLOT(setSelectedProfile(int)));
}

void QProfileEditor::apply()
{
	QProfile * prof = QSAS_profiles->current();
	plotWidget->updateProfile(prof);
	plotWidget2->updateProfile(prof);
	pathWidget->updateProfile(prof);
	miscWidget->updateProfile(prof);
	guiWidget->updateProfile(prof);

    emit(refreshColourTableButtons());
	
	if(QSAS_profiles->current()->fontSize>=5)
	{
		QFont f;
		f.setPixelSize(QSAS_profiles->current()->fontSize);
		qApp->setFont(f);
	}
	else
	{
		qApp->setFont(defaultFontQSAS);
	}
	
	// set default start for Gregorian Calendar

	string GregorianStr = QSAS_profiles->current()->gregorian_start.toLatin1().data();
	GregorianStr += " 000:000:000.000";
	DvTime t(GregorianStr.c_str());
	t.setGregorianStartMJD(t.getMJD());
	
	// set default decimal places for seconds in time fields
	t.setSecResolution(QSAS_profiles->current()->secDecimal);

		// Repeat for DVOS

	DvTime dt(GregorianStr.c_str());
	dt.setGregorianStartMJD(t.getMJD());
	
	// set default decimal places for seconds in time fields
	dt.setSecResolution(QSAS_profiles->current()->secDecimal);
	
	QSAS_profiles->save();

}

void QProfileEditor::apply_and_close()
{
	apply();
	
	close();
}

void QProfileEditor::saveAs()
{
	//Get name,
	bool ok;

	QString newName=QInputDialog::getText(this, "Name for this preset?", "Name:", QLineEdit::Normal, QString("Custom"), &ok);

	if(ok)
	{
		if(newName=="Default")
		{
			QMessageBox::critical(this, "Save failed", "\"Default\" is protected preset name");
			return;
		}
		
		bool alreadyExisting=true;
		QProfile * prof=QSAS_profiles->fromName(newName);
		if(prof==NULL)
		{
			alreadyExisting=false;
			prof=new QProfile(newName);
		}
		
		plotWidget->updateProfile(prof);
		plotWidget2->updateProfile(prof);
		pathWidget->updateProfile(prof);
		miscWidget->updateProfile(prof);
		guiWidget->updateProfile(prof);
		
		if(!alreadyExisting) QSAS_profiles->add(prof);
		
		QSAS_profiles->setCurrent(newName);
	
		updateProfilesList();
	
		emit(changedNames());
		
		if(QSAS_profiles->current()->fontSize>=5)
		{
			QFont f;
			f.setPixelSize(QSAS_profiles->current()->fontSize);
			qApp->setFont(f);
		}
		else
		{
			qApp->setFont(defaultFontQSAS);
		}
		
		// set default start for Gregorian Calendar

		string GregorianStr = QSAS_profiles->current()->gregorian_start.toLatin1().data();
		GregorianStr += " 000:000:000.000";
		DvTime t(GregorianStr.c_str());
		t.setGregorianStartMJD(t.getMJD());
	
		// set default decimal places for seconds in time fields
		t.setSecResolution(QSAS_profiles->current()->secDecimal);

		// Repeat for DVOS
		DvTime dt(GregorianStr.c_str());
		dt.setGregorianStartMJD(t.getMJD());
	
		// set default decimal places for seconds in time fields
		dt.setSecResolution(QSAS_profiles->current()->secDecimal);

	}

}

void QProfileEditor::deletePreset()
{
	PresetSelector * selector=new PresetSelector(this);
	
	if(selector->exec())
	{
		QSAS_profiles->remove(selector->selectedPreset);
		plotWidget->setSelectedProfile(QSAS_profiles->current());
		plotWidget2->setSelectedProfile(QSAS_profiles->current());
		pathWidget->setSelectedProfile(QSAS_profiles->current());
		miscWidget->setSelectedProfile(QSAS_profiles->current());
		guiWidget->setSelectedProfile(QSAS_profiles->current());
		
		updateProfilesList();

		emit(changedNames());
	}
}

void QProfileEditor::setQpuiWidget(QpWindow * w)
{
	if(guiWidget!=NULL)
	{
		guiWidget->setQpuiWidget(w);
	}
}
