/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>  
#include <math.h>
#include <ctype.h>
#include <sys/param.h>

#include <qfiledialog.h> 
#include <qglobal.h>
#include <qfont.h>
#include <qwidget.h>

#include "qt_version.h"
#include "qwui.h"
#include "qslot.h"
#include "qdutil.h"
#include "cdf.h"
#include "DvObj.h"

QFont defaultFontQSAS;

extern qwuiWindow* qsasWindow;


 using namespace QSAS;


 
// QSAS main window is a class on its own

QWidget *qwuiWindow::toggleBar;

qwuiWindow::qwuiWindow(QWidget *parent, const char *name) 
   :  QMainWindow(0)
{
    
	double qsas_height = QSAS_profiles->current()->Height; 
	if(qsas_height == 0) qsas_height = 48.; // default if no env set or is bad

	// ensure QSAS_HOME env is set
	char * qsas_home_env = getenv("QSAS_HOME"); 
	if( qsas_home_env != NULL) {
        qsas_home = QString(qsas_home_env);
		qsas_home=qsas_home.remove(QString("\""));
	}
	else{   
		bool ok;
		QString user_choice = QwuiTextInputDialog("Environment QSAS_HOME not set", 
                                   "Please provide the full path to QSAS_HOME Directory",
				   "/usr/local/QSAS_2", &ok, this);
		if( !ok || user_choice.isEmpty() ) user_choice = QString(".");
		qsas_home = QString(user_choice.toLatin1().data());   // deep copy
	}

	setAcceptDrops(true);
	help_window = new QswHelpWindow();
	
	// Menu insertion moved to qsas_main to avoid circular dependencies
	
	QFontMetrics qsas_font_mt(qApp->font());
	int char_width = qsas_font_mt.width("M");
	int char_height = qsas_font_mt.height();
   
	resize( char_width*48, (int)(char_height*qsas_height) );
	centralW = new QWidget(this);
	centralBox = new QVBoxLayout(centralW);
	
	toggleBar = new QWidget(centralW);
	QHBoxLayout * toggleLayout=new QHBoxLayout(toggleBar);

	centralBox->addWidget(toggleBar);

	StripBadData = new QCheckBox("Strip NaN   ", toggleBar);
	StripBadData->setChecked(FALSE);
	StripBadData->setMaximumHeight(18);
   
	StripFills = new QCheckBox("Strip Fill Values   ", toggleBar);
	StripFills->setChecked(FALSE);
	StripFills->setMaximumHeight(18);
   
	overWrite = new QCheckBox("Replace", toggleBar);
	overWrite->setChecked(FALSE);
	overWrite->setMaximumHeight(18);
	
	toggleLayout->addWidget(StripBadData);
	toggleLayout->addWidget(StripFills);
	toggleLayout->addWidget(overWrite);
	
#if QT_VERSION < 0x040300
	toggleLayout->setMargin(0);
#else
	toggleLayout->setContentsMargins(0,0,0,0);
#endif
    
    this->_filename = qsas_home;
	this->_filename += QString("/tmp");
	if(QSAS_profiles->current()->SaveRestorePath.isEmpty()) {
		 this->_sessionFolder = QString(this->_filename.toLatin1().data());
		#if defined(Q_OS_MACX)
			if( !this->_sessionFolder.contains(".qss")) this->_sessionFolder += QString("/SampleSession.qss");
		#endif
	}
	else {
		this->_sessionFolder = QSAS_profiles->current()->SaveRestorePath;
	}
	
	this->_filename += QString("/saveSet");
	this->SAVE_EXTENSION = QString(".qx_WL");
	this->_filename += this->SAVE_EXTENSION;
	this->fromSession = false;

	wlview=new WList(centralW);
	
	centralBox->addWidget(wlview);
	
	setCentralWidget( centralW );
	connect(wlview, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));

    restoreAction = new QAction(QIcon(qsas_home+"/icons/reload.png"), tr("Restore Working List"), this);
    restoreAction->setStatusTip(tr("Restore working list"));
    connect(restoreAction, SIGNAL(triggered()), this, SLOT(restoreWL())); 

    saveAction = new QAction(QIcon(qsas_home+"/icons/filesaveas.png"), tr("Save Working List"), this);
    saveAction->setStatusTip(tr("Save working list"));
    connect(saveAction, SIGNAL(triggered()), this, SLOT(saveWL()));

    openAction = new QAction(QIcon(qsas_home+"/icons/fileopen.png"), tr("Open File"), this);
    openAction->setStatusTip(tr("Open file"));

    selectorAction = new QAction(QIcon(qsas_home+"/icons/dbOpen.png"), tr("Open Data selector"), this);
    selectorAction->setStatusTip(tr("Database"));

    deleteAction = new QAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), tr("Delete"), this);
//    deleteAction = new QAction(QIcon(qsas_home+"/icons/editdelete.png"), tr("Delete selected item"), this);
    deleteAction->setStatusTip(tr("Delete selected item"));
    connect(deleteAction, SIGNAL( triggered()), wlview, SLOT(deleteObject())); 

    dellistAction = new QAction(QIcon(qsas_home+"/icons/clearlist.png"), tr("Clear Working list"), this);
    dellistAction->setStatusTip(tr("Clear Working list"));
    connect(dellistAction, SIGNAL( triggered()), wlview, SLOT(deleteAllDataItems())); 
    
    joinAction = new QAction(QIcon(qsas_home+"/icons/join.png"), tr("Join"), this);
    joinAction->setStatusTip(tr("Join"));

    calcAction = new QAction(QIcon(qsas_home+"/icons/xcalc.png"), tr("Calculator"), this);
    calcAction->setStatusTip(tr("Calculator"));
 
    plotAction = new QAction(QIcon(qsas_home+"/icons/plot2.png"), tr("Plot"), this);
    plotAction->setStatusTip(tr("Plot"));


//    timeAction = new QAction(style()->standardIcon(QStyle::SP_TitleBarShadeButton, 0, this), tr("New time interval"), this);
    timeAction = new QAction(QIcon(qsas_home+"/icons/newtime.png"), tr("New time interval"), this);
    timeAction->setStatusTip(tr("Create new Time interval"));
    connect(timeAction, SIGNAL( triggered()), wlview, SLOT(createTInterval()));

 
    mainToolBar = addToolBar(tr("Shortcuts"));
    mainToolBar->setIconSize(QSize(24, 24));
    mainToolBar->hide();
    mainToolBar->addAction(restoreAction);
    mainToolBar->addAction(saveAction);
    mainToolBar->addAction(openAction);
    mainToolBar->addAction(selectorAction);
    mainToolBar->addAction(deleteAction);
    mainToolBar->addAction(dellistAction);
    mainToolBar->addAction(timeAction);
    mainToolBar->addAction(calcAction);
    mainToolBar->addAction(joinAction);
    mainToolBar->addAction(plotAction);
    mainToolBar->insertSeparator(calcAction);  
    mainToolBar->insertSeparator(deleteAction);  
    mainToolBar->insertSeparator(openAction);  

}


void qwuiWindow::makeWindows()
{
	wlview->blockSignals(true);
	setStandardConstants( );
	wlview->blockSignals(false);
	
	/* create some test time interval objects to put in the list */
	DvObject_var ti_1 = new DvObject(DvEvent(DvTime(2001, 9, 4), DvTime(2001, 9, 5)));  
	QwuiPutOnWL(ti_1, "Time Intervals/SampleTI");
	
	wlview->collapseConst();



}
 

void qwuiWindow::saveSession(){
 
 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveSession \n");
	fflush(stdout);
 }
 
  QuMsgBar(" ", false);

// ensure default is set
   if(_sessionFolder.isEmpty())
  {
     saveSessionAs();
     return;
  }

// ensure default has right filter
  int posn = _sessionFolder.lastIndexOf(".qss");
  if (posn < 0 ){
     saveSessionAs();
     return;  
  }

// ensure default exists still
// user might delete it

  QDir sessionDir = QDir(_sessionFolder);
  if( !sessionDir.exists() ){
    bool OK = sessionDir.mkdir(_sessionFolder);
    if (!OK){
      QuMsgBar("Failed to create directory", _sessionFolder.toLatin1().data(), true);
      return;
    }
  }
  // remove trailing '/'

  if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
 
  emit saveSessionRequest(_sessionFolder);
    
  // set session folder and save
  this->_filename = QString(_sessionFolder.toLatin1().data());
  this->_filename += QString("/saveSet");
  this->_filename += this->SAVE_EXTENSION;
  saveWL();
 
}
  
void qwuiWindow::setSaveAs310(){
    save310->setChecked(true);
    save315->setChecked(false);
    setSaveVersion(310);
}

void qwuiWindow::setSaveAs315(){
    save315->setChecked(true);
    save310->setChecked(false);
    setSaveVersion(315);
}

void qwuiWindow::saveSessionAs(){
 
  if(QuSlotDebug())
  {
	printf("qwuiWindow::saveSessionAs \n");
	fflush(stdout);
  }
 
  QString fn;
  QuMsgBar(" ", false);
  
  QString newSession = _sessionFolder;
  // use path only
  int posn = newSession.lastIndexOf(".qss");
  
  if (posn > 0) {
     newSession.truncate(posn);
    posn = newSession.lastIndexOf("/");
    if (posn > 0) newSession.truncate(posn+1);
  }

  
  SessionFolderDialog *dialog = new SessionFolderDialog(this,"FolderSession");
  dialog->setDirectory(newSession);
  if(dialog->exec() == QDialog::Accepted){
    fn = dialog->getSessionFolder();
  
    if(fn.isEmpty())
    {
       QuMsgBar("Not .qss directory", true);
    }
    else
    {
       _sessionFolder=fn;
       if (_sessionFolder[_sessionFolder.length()-1] == '/') _sessionFolder.truncate(_sessionFolder.length()-1);
	 
	  // set QSAS main window title
	  QString capt = QString(fn.toLatin1().data());
	  int pos_end = capt.lastIndexOf(".qss");
	  capt.truncate(pos_end);
	  int pos_strt = capt.lastIndexOf("/");
	  capt.remove(0,pos_strt+1);

	  this->setWindowTitle( "Saving as   '" + capt + "'"  );
      this->repaint();
       saveSession();
	  this->setWindowTitle(QString(QSAS_VERSION_ID) + "   '" + capt + "'"  );


    }
  }
 
}
 
  
void qwuiWindow::restoreSession(){
    
/*    
    // use path only
    int posn = _sessionFolder.lastIndexOf(".qss");
    
    if (posn > 0) {
        _sessionFolder.truncate(posn);
        posn = _sessionFolder.lastIndexOf("/");
        if (posn > 0) _sessionFolder.truncate(posn+1);
    }
 */   
    QuMsgBar(" ", false);
    QString fn=QFileDialog::getExistingDirectory(this, "Select session folder to restore", _sessionFolder);
    if(!fn.isEmpty())
    {
        QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
        
        qApp->processEvents();
        
        if (fn[fn.length()-1] == '/')  fn.truncate(fn.length()-1);
        
        _sessionFolder = QString(fn.toLatin1().data());
        
        // restore WL
        // keep original name
        QString fileN = QString(_filename.toLatin1().data());
        
        // set session folder and restore
        this->_filename = QString(fn.toLatin1().data());
        this->_filename += QString("/saveSet");
        this->_filename += this->SAVE_EXTENSION;
        this->fromSession = true;
        
        // tell plot and calc GUIs to clear their listviews to minimise wasted revalidation
        emit clearListRequest();
        
        // empty the working list done in restore(true)
        
        qApp->processEvents();
        
        if( restoreWL(true) == false ){
            status("Restore session: old format WL? Open in QSAS 2.4 and re-save.",10000);
            qApp->processEvents();
            QApplication::restoreOverrideCursor();
        }
        else {
            status(fn+" restored",10000);
            
            // restore each GUI
            emit restoreSessionRequest(fn);
            
            qApp->processEvents();
            

            // set QSAS main window title
            QString capt = QString(_sessionFolder.toLatin1().data());
            int pos_end = capt.lastIndexOf(".qss");
            capt.truncate(pos_end);
            int pos_strt = capt.lastIndexOf("/");
            capt.remove(0,pos_strt+1);
            
            this->setWindowTitle("Restoring:   '" + capt + "'"  );
            this->repaint();
            
            // restore each GUI
            emit restoreSessionRequest(fn);
            
            qApp->processEvents();
            
            this->setWindowTitle(QString(QSAS_VERSION_ID) + "   '" + capt + "'"  );

           
             // put file name back
            _filename = QString(fileN.toLatin1().data());
                       
            QApplication::restoreOverrideCursor();
        }
    }
    else
        status("Restore session cancelled",10000);
}

void qwuiWindow::restoreNamedSession(QString fn){
    
    
    if (fn[fn.length()-1] == '/')  fn.truncate(fn.length()-1);
    
    _sessionFolder = QString(fn.toLatin1().data());
    
    // restore WL
    // keep original name
    QString fileN = QString(_filename.toLatin1().data());
    
    // set session folder and restore
    this->_filename = QString(fn.toLatin1().data());
    this->_filename += QString("/saveSet");
    this->_filename += this->SAVE_EXTENSION;
    this->fromSession = true;
    
    // tell plot and calc GUIs to clear their listviews to minimise wasted revalidation
    emit clearListRequest();
    
    // empty the working list done in restore(true)
    
    qApp->processEvents();
    
    if( restoreWL(true) == false ){
        qApp->processEvents();
        QApplication::restoreOverrideCursor();
    }
    else {
        
        // restore each GUI
        emit restoreSessionRequest(fn);
        
        qApp->processEvents();
        
        // put file name back
        _filename = QString(fileN.toLatin1().data());
        
        QApplication::restoreOverrideCursor();
    }
}

void qwuiWindow::restoreCalc(QString fn){
    
    // restore calc GUI
    emit restoreCalcRequest(fn);
    qApp->processEvents();
    
}


void qwuiWindow::restorePlot(QString fn){
    
    // restore calc GUI
    emit restorePlotRequest(fn);
    qApp->processEvents();
    
}

void qwuiWindow::readData(DvString& fileName, DvString& varName, DvEvent& range){

	emit readDataRequest(fileName, varName, range);
	qApp->processEvents();
}

void qwuiWindow::calcPage(int p){

	emit calcPageRequest(p);
	qApp->processEvents();
}

void qwuiWindow::directPrint(const char * name){

	emit directPrintRequest(name);
	qApp->processEvents();
}

void qwuiWindow::updatePlot(){

	emit updatePlotRequest();
	qApp->processEvents();
}

void qwuiWindow::includeInWL()
{
	restoreWL(false);
}

bool qwuiWindow::restoreWL(bool replace)
{
	QString fn;
    
    if(QSAS_profiles->current()->SaveRestorePath.isEmpty()) {
		 fn = _filename;
	}
	else {
		fn = QSAS_profiles->current()->SaveRestorePath;
	}

	if(!fromSession){
		fn=QFileDialog::getOpenFileName(this, tr("select file to open"), fn, "WL savesets (*.qx_WL)");
		_filename=fn;
	}
	fromSession = false; // reset for save from GUI

	if(_filename.isEmpty() ) return false;
   
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	qApp->processEvents();
		
	// XML saveset

	char * xmlTxt = readXML(_filename.toLatin1().data());
	char * xml = xmlTxt;

	if (xml == NULL ) {
		printf("QwuiWindow::restore unable to read xml file ");
		QApplication::restoreOverrideCursor();
		return false;
	}

	wlview->blockSignals(true);

    if(replace)
    {
		wlview->deleteAllDataItems();
		wlview->deleteAllTimeIntervalItems();
		wlview->deleteAllConstantItems();
    }

	size_t start;
	DvString tag;
	char *ptr = getNextTag(xml, start, tag);
	while(ptr){
		if(tag == "QSAS_ID") {
			printf("QwuiWindow::restore ID =  %s ------\n   --------------------------------------\n ", xml+start);
			setRestoreVersion(xml+start);
		}
		else if(tag == "STRIP_BAD_DATA_TOGGLE") {
			StripBadData->setChecked( DvString(xml+start).toBool());
		}
		else if(tag == "STRIP_FILLS_TOGGLE") {
			StripFills->setChecked( DvString(xml+start).toBool());
		}
		else if(tag == "OVER_WRITE_TOGGLE") {
			overWrite->setChecked( DvString(xml+start).toBool());
		}
		else if(tag == "WORKING_LIST") {
            DvClearXML(); // start with empty XML object list
			DvList WL_temp;
			WL_temp.fromXML(xml+start);
			DvNode *node = WL_temp.first();
			while(node){
				QString name = node->name().c_str();
				QwuiPutOnWL(node->obj(), name, false); // don't replace
				node = node->next;
			}
            DvClearXML(); // stop objects hanging around because they have var pointer on XML list
		}
		xml = ptr;
		ptr = getNextTag(xml, start, tag);
	}
	
	wlview->expandAll();
	wlview->collapseConst();

	wlview->blockSignals(false);
	wlview->setNoSelection();
		
	QApplication::restoreOverrideCursor();
	qApp->processEvents();

	if(xmlTxt) delete xmlTxt;
	
	return true;
}

void qwuiWindow::saveWL(){

	if(QuSlotDebug())
	{
		printf("qwuiWindow::saveWL \n");
		fflush(stdout);
	}

	if(_filename.isEmpty())
	{
		saveWLAs();
		return;
	}
    
	std::ofstream ofp;
 
	ofp.open(_filename.toLatin1().data(), std::fstream::out );

	ofp << "<QSAS_ID>";
	ofp << QSAS_VERSION_ID;
	ofp << "</QSAS_ID>\n";	
	
	ofp << "<STRIP_BAD_DATA_TOGGLE>";
	ofp << (int) StripBadData->isChecked();
	ofp << "</STRIP_BAD_DATA_TOGGLE>\n";

	ofp << "<STRIP_FILLS_TOGGLE>";
	ofp << (int) StripFills->isChecked();
	ofp << "</STRIP_FILLS_TOGGLE>\n";

	ofp << "<OVER_WRITE_TOGGLE>";
	ofp << (int) overWrite->isChecked();
	ofp << "</OVER_WRITE_TOGGLE>\n";      

	write_WLtoXML(ofp);
	
	ofp.close();
	
	status(_filename+" saved",10000);
	return;   
}

void qwuiWindow::saveWLAs(){

 if(QuSlotDebug())
 {
	printf("qwuiWindow::saveWLAs \n");
	fflush(stdout);
 }
 
  
  QString _filter = QString("save (*.qx_WL)");

  QString fn=QFileDialog::getSaveFileName(this, "Select file - Save WL Settings", _filename, _filter);
  if(!fn.isEmpty())
  {
    int pos = fn.lastIndexOf(".qx_WL");
    if (pos < 0) {
      pos = fn.lastIndexOf(".");
      if (pos > 0) fn.truncate(pos);
      fn += ".qx_WL";
    }
    _filename=fn;
    saveWL();
  }
  else
    status("saving aborted ",10000); 
}



void qwuiWindow::setPreferences(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::setPreferences \n");
	fflush(stdout);
 }
 
  
  QuMsgBar("TBD: Window to set user preferences\n global settings", false);
  
}


void qwuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManual \n");
	fflush(stdout);
 }
 
   qsasWindow->help_window->manEntry("index.html");
}

void qwuiWindow::launchManMainW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManMainW \n");
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_main.html");
}

void qwuiWindow::launchManSampleW(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSampleW \n");
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_intro.html");
}

void qwuiWindow::launchManDataH(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManDataH \n");
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void qwuiWindow::launchManSaveRestore(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchManSaveRestore \n");
	fflush(stdout);
 }
 
  qsasWindow->help_window->manEntry("QSAS_save_restore.html");
}

void qwuiWindow::launchVersion(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchVersion \n");
	fflush(stdout);
 }
 
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n";
  text += QString(QSAS_VERSION_DATE);
  // DVOS
  text += "\nDVOS version: ";
  text += getDVOS_versionStr().c_str();
  text += ", ";
  text += getDVOS_versionDate().c_str();
  // QT
  text += "\nQT version detected:  ";
  text += QString(QT_VERSION_STR);
  // cdf 
  CDFid id;
  long version, release, increment;
  long numDims=0;
  long dimSizes[1] = {0};
  CDFsetFileBackward(BACKWARDFILEoff);
  QString file_path=QString(getenv("QSAS_HOME"));
  file_path+="/empty";
  CDFstatus status = CDFlib(CREATE_, CDF_, file_path.toLatin1().data(), numDims, dimSizes, &id, NULL_);
  char message[CDF_STATUSTEXT_LEN+1];
  CDFerror(status, message);
   if(QuSlotDebug())  printf("%s\n",message);
  if(status == CDF_OK) {
  	CDFgetVersion (id, &version, &release, &increment);
	CDFdelete(id);
  	text += "\nCDF version detected:  ";
  	QString cdf_version_str = QString("CDF %1.%2.%3").arg(version).arg(release).arg(increment);
  	text += cdf_version_str;
  }
  Qwui_msgBox(text.toLatin1().data());

}

void qwuiWindow::launchContact(){ 

 if(QuSlotDebug())
 {
	printf("qwuiWindow::launchContact \n");
	fflush(stdout);
 }

  qsasWindow->help_window->manEntry("ContactUs.html");


}


void qwuiWindow::closeEvent( QCloseEvent* ce)
{     
	char * fastClose = (char *) getenv ("EXIT_MODE");
	if(fastClose != NULL && strcmp(fastClose, "QSAS_FAST_EXIT") == 0) 
	{
		qApp->exit(0);
	}
	else {
		
		switch( QMessageBox::information( this, "Qwui Window",
                                          "Exit Qsas Main Window?",
                                          "Exit", "Save Session", "Cancel",
                                          0, 1 ) ) 
    	{
        	case 0:
			{
				qApp->exit(0);
				break;
			}
        	case 1:
			{
				saveSession();
            	ce->ignore();	    
            	break;
			}
        	case 2:
        	default: 
        	{
		   	 	ce->ignore();
            	break;
			}
    	}
	}
}
 void qwuiWindow::dragEnterEvent(QDragEnterEvent *event)
{   
       this->activateWindow();
       this->raise();
}


void qwuiWindow::Qwui_msgBox( const char *  text)
{
    QMessageBox::information( this, "QSAS information", text, "OK" ); 
    return;
}

  

SessionFolderDialog::SessionFolderDialog( QWidget *parent, const char *name ) 
    : QFileDialog(parent) 
{
    this->setWindowTitle("Type name of .qss folder for Save");
    this->setFileMode(QFileDialog::Directory);
    setOption(QFileDialog::ShowDirsOnly, true);
    setAcceptMode(QFileDialog::AcceptSave);
	QString qss(".qss");
	setDefaultSuffix(qss);
    selectFile("NewSession.qss");
    setNameFilter("*.qss");
    
}


QString SessionFolderDialog::getSessionFolder(){
	QStringList files = this->selectedFiles();
  	QString dirName;
  	if (!files.isEmpty()) dirName = files[0];
  
	if( !dirName.isEmpty() && !dirName.endsWith(".qss") && !dirName.endsWith(".qss/") )
	{
		dirName.append(".qss");
		QuMsgBar(".qss appended to folder name", true);
	}
	return dirName;
}


//---------------------------------------------------------

QString QwuiPutOnWL(DvObject_var obj, const QString& path, bool replace)
{
    if(obj.is_nil() && obj->nError() == 0) return QString("");
    
    // Fix MMS FGM 4vectors
    size_t rComp;
    if( (rComp = Qwui4VecRcomp(obj)) < 4){
        DvObject_var robj = QwuiGet4VecR(obj, rComp);
        obj = QwuiGet3Vec(obj, rComp);
        
        QwuiPutOnWL(robj, path+"_R", replace);
        return QwuiPutOnWL(obj, path, replace);
    }
    
    
	bool ok = true;
	QString newPath=path;
	if( path.isEmpty() ) newPath = "Data/unnamed";
	if(newPath.endsWith('/') ) newPath.truncate(newPath.size()-1); // remove training '/'

	if(  !newPath.startsWith("Constants/") && !newPath.startsWith("Data/") && !newPath.startsWith("Time Intervals/" ) && !newPath.startsWith("Temp/" ) ) {
		// decide where to put it
		if(obj->is_event()) newPath.prepend("Time Intervals/");
		else newPath.prepend("Data/"); 
	}
			
	WLItem* item = qsasWindow->wlview->itemFromPath(newPath);
	if(qsasWindow->overWrite->isChecked()) replace = true;
	
   // remove and '[' or ']' from WL name to prevent calculator problems	
	newPath.replace("[", "(");
	newPath.replace("]", ")");

	if(item==NULL)
	{
		item=qsasWindow->wlview->createItem(newPath);

		if(item!=NULL)
		{
			newPath=item->getPath();
		}

        ok = QuSetObjByName(obj, newPath.toLatin1().data());
	}
	else
	{
		if(replace)
		{
			ok = QuSetObjByName(obj, newPath.toLatin1().data());
		}
		else
		{
			while(qsasWindow->wlview->itemFromPath(newPath)!=NULL)
			{
				newPath.append("A");
			}
			item=qsasWindow->wlview->createItem(newPath);
			
			ok = QuSetObjByName(obj, newPath.toLatin1().data());
		}
	}

	if(item==NULL) return QString();

	if( !ok) 
	{
		qsasWindow->wlview->deleteItem(item);
		return QString();
	}
	
	qsasWindow->wlview->setCurrentItem(item);
	
	
	qsasWindow->wlview->emitObjChanged(newPath);

 //   qsasWindow->setWindowTitle(QString(QSAS_VERSION_ID) );

	return newPath;
	
}

size_t Qwui4VecRcomp(DvObject_var obj){
    
    size_t rComp = 4;
    if(obj->arraySize() == 4 && obj->nDims() == 1){
        if(obj->xref_exists(REPRESENTATION_1)){
            bool isCartesian=false;
            DvObject_var rep = obj->get_xref(REPRESENTATION_1);
            for(size_t i=0; i< rep->arraySize(); i++){
                if(rep->asStr(i) == "X") isCartesian=true;
                if(rep->asStr(i) == "R") rComp = i;
            }
            if(isCartesian) return rComp;
        }
    }
    return rComp;
}

DvObject_var QwuiGet4VecR(DvObject_var obj, size_t rComp){

    DvObject_var r = obj->element(rComp);
    r->copy_xrefs_from(obj);
    r->delete_xref(COORDINATE_SYSTEM);
    r->delete_xref(TENSOR_ORDER);
    r->delete_xref(REPRESENTATION_1);
    r->delete_xref(LABL_PTR_1);
    
    r->change_xref(LABLAXIS, "Bmag");
    DvObject_var zeroObj = new DvObject(0.0);
    r->change_xref(VALIDMIN, zeroObj);
    
    DvObject_var fieldNam = r->get_xref(FIELDNAM);
    DvString fnStr = fieldNam->asStr();
    fnStr = fnStr.replace("vector", "magnitude");
    r->change_xref(FIELDNAM, fnStr);
    
    DvObject_var catDesc = r->get_xref(CATDESC);
    DvString cdStr = catDesc->asStr();
    cdStr = cdStr.replace("vector", "magnitude");
    r->change_xref(CATDESC, cdStr);
    
    
    
    return r;
}

DvObject_var QwuiGet3Vec(DvObject_var obj, size_t rComp){

    DvObject_var newObj = obj->subDimension(rComp, true);
    DvObject_var rep = obj->get_xref(REPRESENTATION_1);
    rep = rep->subDimension(rComp, true);
    newObj->change_xref(REPRESENTATION_1, rep);
    
    return newObj;
}

//---------------------------------------------------------

QSize qsasGUISize(){ 
   return qsasWindow->frameSize();
}

//---------------------------------------------------------

void QwuiClearTemp(){
	qsasWindow->wlview->deleteAllTempItems();
}

//---------------------------------------------------------

QString QwuiTextInputDialog(const char *caption, const char *label,
                          const char * defText, bool *OK, QWidget * parent){

QString input = QInputDialog::getText(parent, QString(caption), QString(label), QLineEdit::Normal, QString(defText), OK);

return input;

}
