//  qdi.cc
//
// Direct Import Widget
//
// 2002 - 2016: Tony Allen
//

#include <stdlib.h>
#include "qcheckbox.h"

#include "qbu.h"
#include "qdi.h"

#include "qie_caa.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;

// -----------------------------------------------------------------------

void setAsSeparator(QListWidgetItem* item)
{
    QFont font=item->font();
    font.setBold(true);
    item->setFont(font);

    if(item->listWidget()!=NULL)
    {
        QRect rect=item->listWidget()->visualItemRect(item);

        QLinearGradient grad(QPoint(0,0), QPoint(0, rect.height()));
        grad.setColorAt(0, QColor(255, 255, 255));
        grad.setColorAt(1, QColor(230, 230, 230));
        QBrush brush(grad);
        item->setBackground(brush);
    }

    item->setTextAlignment(Qt::AlignCenter);
    QBrush brush2(Qt::black);
    item->setForeground(brush2);
    
    item->setFlags(Qt::NoItemFlags);
}


// -------------------------------------------------------------
// ---- DVOS Native QIE interface ----------------------------
// -------------------------------------------------------------

// -----------------------------------------------------------------------


DirectAccessWindow::DirectAccessWindow( QWidget *parent, const char *name ) 
  : QMainWindow(parent)
{
	// Status bar
	msgBar = statusBar();
	
	// Menu
	
	menuBar()->addSeparator();   
	QMenu * help= menuBar()->addMenu( " Help");
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Import", this, SLOT( launchManDirectImport() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	// Top widget
	
	QWidget * topWidget=new QWidget(this);
	QVBoxLayout * topLayout=new QVBoxLayout;
	
	// List panel
	
	QHBoxLayout * listLayout=new QHBoxLayout;
	
    QVBoxLayout* varLayout=new QVBoxLayout;
    QLabel* varLabel=new QLabel("Variables");
    varLabel->setAlignment(Qt::AlignCenter);
    varLayout->addWidget(varLabel);
    varList=new QListWidget;
    varList-> setSortingEnabled(false);
    connect( varList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(setSliderForItem(QListWidgetItem*)) );
    connect( varList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearGattrList()) );
    connect( varList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(varDoubleClick(QListWidgetItem*)) );
    activeList = varList;
    varLayout->addWidget(varList);
    listLayout->addLayout(varLayout);

    QVBoxLayout* GattrLayout=new QVBoxLayout;
    QLabel* GattrLabel=new QLabel("Global Attributes");
    GattrLabel->setAlignment(Qt::AlignCenter);
    GattrLayout->addWidget(GattrLabel);
    GattrList=new QListWidget;
    GattrList-> setSortingEnabled(true);
    connect( GattrList, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(clearVarList()) );
    connect( GattrList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(GattrDoubleClick(QListWidgetItem*)) );
    GattrLayout->addWidget(GattrList);
    listLayout->addLayout(GattrLayout);
	
	topLayout->addLayout(listLayout);
	
	// Subset panel
	
	subset_panel=new QGroupBox(topWidget);
	QVBoxLayout * subset_layout=new QVBoxLayout;
	
	useTcheck = new QCheckBox;
	useTcheck->setText("Get time for subsetting ");
	useTcheck->setDown(false);
	connect(useTcheck, SIGNAL(clicked( )), this, SLOT(tRangeToggle( )) );
	subset_layout->addWidget(useTcheck);
	
	fromSlider = new  RecordSlider("From  ", subset_panel, "fromSlider");
	subset_layout->addWidget(fromSlider);
	
	toSlider =   new  RecordSlider("To  ", subset_panel, "toSlider");
	subset_layout->addWidget(toSlider);
	
	subset_panel->setLayout(subset_layout);
	topLayout->addWidget(subset_panel);
	
	// Buttons
	
	QHBoxLayout * button_layout=new QHBoxLayout;
	
	getVarBtn = new QPushButton( "Get Data");
	getVarBtn->setMinimumSize(getVarBtn->sizeHint());
	getVarBtn->setDefault(true);
	connect( getVarBtn, SIGNAL(clicked()), this, SLOT(getData()) );
	button_layout->addWidget(getVarBtn);

	browseVarBtn = new QPushButton( "Browse Data");
	browseVarBtn->setMinimumSize(browseVarBtn->sizeHint());
	connect( browseVarBtn, SIGNAL(clicked()), this, SLOT(browseData()) );
	button_layout->addWidget(browseVarBtn);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	button_layout->addWidget(dismissBtn);
	
	topLayout->addLayout(button_layout);
	
	topWidget->setLayout(topLayout);
	setCentralWidget(topWidget);
	
	QieDataCont = new QieData();
	
	setAttribute(Qt::WA_DeleteOnClose);
	

}

// ------------------------------------------

void DirectAccessWindow::close(){

 if(QuSlotDebug())
 {
  printf("DirectAccesstWindow::close \n");
  fflush(stdout);
 }
  
  qApp->processEvents();

  delete QieDataCont;
  QMainWindow::close();
}



// ------------------------------------------

void DirectAccessWindow::tRangeToggle(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::tRangeToggle \n");
  fflush(stdout);
 }
 
 
  QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
	
  // get time variables (note they are cached in earlier calls)
  for(int i=0; i<QieDataCont->QieVars.size(); i++){
	 if(QieDataCont->QieVars[i]->datatype == DATA_TIME)  QieDataCont->readVarData(i) ;
  }
	
  QApplication::restoreOverrideCursor();
  
  if(QieDataCont->d0_var_num != -1){
  	DvObject_var d0 = QieDataCont->QieVars[QieDataCont->d0_var_num]->Vobj;
	 	
	fromSlider->ttags = d0;
	toSlider->ttags   = d0;
	
  }

  updateOnSelection();
  useTcheck->setEnabled(false); // prevent being unset
}

// ------------------------------------------

void DirectAccessWindow::launchManual(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManual \n");
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("index.html");
}

// ------------------------------------------

void DirectAccessWindow::launchManDirectImport(){ 

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchManDirectImport \n");
  fflush(stdout);
 }
 
  
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

// ------------------------------------------

void DirectAccessWindow::launchVersion(){

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::launchVersion \n");
  fflush(stdout);
 }
 
  
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Direct Access:  ";
  text += QString(QIE_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}

// ------------------------------------------

int DirectAccessWindow::getFileContents(const char *file){

   // keep global copy of path to add as xref later 


	getVarBtn->setEnabled(false);
	browseVarBtn->setEnabled(false);
	dismissBtn->setEnabled(false); 
	  
	Selected_File_Path = QString(file);

	if(QuSlotDebug())  setDebug(true);
	
	int nRecs = QieDataCont->queryFile(file);
	if( nRecs > 0 ){
		
		// Set up display if file read OK
		
		// Set up global list
		GattrList->clear();
		DvNode *Gnode = QieDataCont->GlobalList.first();
		while(Gnode){
			QListWidgetItem* newGattrItem=new QListWidgetItem(Gnode->name().c_str(), GattrList);
			QSize size=newGattrItem->sizeHint();
			size.setWidth(180);
			size.setHeight(20);
			newGattrItem->setSizeHint(size);
			Gnode = Gnode->next;
		}
		// set up variable list

			QLinkedList<QListWidgetItem*> metaList;
			QLinkedList<QListWidgetItem*> dataList;
			varList->clear();

			QListWidgetItem* topItem=new QListWidgetItem("Data", varList);
			setAsSeparator(topItem);

			for(int i=0; i<QieDataCont->QieVars.size(); i++){

				bool isData = true;
				if(QieDataCont->QieVars[i]->nRecs == -1){
					if(QieDataCont->QieVars[i]->recVary)
					{
						QieDataCont->QieVars[i]->nRecs = nRecs; // set here as flat files don't know it
					}
					else
					{
						QieDataCont->QieVars[i]->nRecs = 1;
					}
				}
				DvString typeXref;
				DvNode *Xnode = QieDataCont->QieVars[i]->Vxref.find("PARAMETER_TYPE");
				if(!Xnode){
					Xnode = QieDataCont->QieVars[i]->Vxref.find("VAR_TYPE");
				}
				if (Xnode) {
					typeXref = Xnode->obj()->asStr();
				}
				if ( !typeXref.empty() ){
					if ( typeXref == "Support_Data" ) isData = false;
					if ( typeXref == "support_data" ) isData = false;
					if ( typeXref == "metadata" ) isData = false;
				}

				QListWidgetItem* newItem=new QListWidgetItem(QieDataCont->QieVars[i]->name.c_str(), NULL);
				QSize size=newItem->sizeHint();
				size.setWidth(180);
				size.setHeight(20);
				newItem->setSizeHint(size);
            
				if(isData){
					dataList.push_back(newItem);
				}
				else{
					metaList.push_back(newItem);
				}
			
			}

			foreach(QListWidgetItem* i, dataList)
			{
				varList->addItem(i);
			}

			QListWidgetItem* sepItem=new QListWidgetItem("Support Data", varList);
			setAsSeparator(sepItem);

			foreach(QListWidgetItem* i, metaList)
			{
				varList->addItem(i);
			}
		
		}
		else{
			getVarBtn->setEnabled(false);
			browseVarBtn->setEnabled(false);
			msgBar->showMessage("WARNING: No data records found: empty file or missing header." );

		}

	getVarBtn->setEnabled(true);
	browseVarBtn->setEnabled(true);
	dismissBtn->setEnabled(true);
	
	return nRecs;
}

// ------------------------------------------

void DirectAccessWindow::setSliders(int nRecs){

	// set initial values for sliders
	if( !(getVarBtn->isEnabled()) ) return; // not enabled if file read failed
	 
	if(nRecs == 0) {
		fromSlider->recBar->setRange(0, 0);
		toSlider->recBar->setRange(0, 0);
		getVarBtn->setEnabled(false);
		browseVarBtn->setEnabled(false);
		fromSlider->maxRecord = 0;
		toSlider->maxRecord = 0;
		msgBar->showMessage("WARNING: No records were read, corrupt file?" );
	}
	else{
		fromSlider->recBar->setRange(0, nRecs-1);
		toSlider->recBar->setRange(0, nRecs-1);
		fromSlider->maxRecord = nRecs-1;
		toSlider->maxRecord = nRecs-1;
		
		fromSlider->recBar->setValue(0);
		toSlider->recBar->setValue(toSlider->maxRecord);
	}
	//fromSlider->updateTitle();
	//toSlider->updateTitle();

}

void DirectAccessWindow::updateOnSelection(QieVariable *V){
    
	fromSlider->updateTitle(0);
	toSlider->updateTitleEnd();
	
}


// ------------------------------------------

void DirectAccessWindow::setSliderForItem(QListWidgetItem *item){
        
	bool rangeChanged = false;
    int nRecs;
    int vnum;
    QieVariable *V;
    
    if(item == 0) return;

    // This slot is needed for data files with sparse records,
    // i.e. files where different variables may have different time tags
    
    bool intOK;
    int fromRec = fromSlider->recField->text().toInt(&intOK);
    if (!intOK) fromRec = 0;
    int toRec   = toSlider->recField->text().toInt(&intOK);
    if (!intOK) toRec = toSlider->maxRecord;

    if( activeList == varList) {
    
        if(item != 0){
            vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
            if( vnum == -1 ) return;
        }
        V = QieDataCont->QieVars[vnum];
        nRecs = V->nRecs;
    }
    else return;
      
	if(nRecs-1 != toSlider->maxRecord)    rangeChanged = true;
	    
    if(nRecs == 0) {
        fromSlider->recBar->setRange(0, 0);
        toSlider->recBar->setRange(0, 0);
        getVarBtn->setEnabled(false);
        browseVarBtn->setEnabled(false);
        fromSlider->maxRecord = 0;
        toSlider->maxRecord = 0;
        msgBar->showMessage("WARNING: No records were read, corrupt file?" );
    }
    else if(nRecs == 1) {
        fromSlider->recBar->setRange(0, 0);
        toSlider->recBar->setRange(0, 0);
        getVarBtn->setEnabled(true);
        browseVarBtn->setEnabled(true);
        fromSlider->maxRecord = 0;
        toSlider->maxRecord = 0;
        msgBar->showMessage("WARNING: single entry for variable" );
    }
    else{
        fromSlider->maxRecord = nRecs-1;
        toSlider->maxRecord = nRecs-1;
        getVarBtn->setEnabled(true);
        browseVarBtn->setEnabled(true);
        fromSlider->recBar->setRange(0, nRecs-1);
        toSlider->recBar->setRange(0, nRecs-1);
        msgBar->showMessage(" " );

    }

	DvObject_var D0 = QieDataCont->getD0_obj(vnum);
	
    	// attach correct tt for variable
    	fromSlider->ttags = D0;
    	toSlider->ttags   = D0;
	
	
    // reset current slider positions to where they were if safe
	//    if(toRec == 0) toSlider->recBar->setValue(toSlider->maxRecord);
    if(toRec > toSlider->maxRecord || rangeChanged) toSlider->recBar->setValue(toSlider->maxRecord);
    else toSlider->recBar->setValue(toRec);
        
    if(fromRec > fromSlider->maxRecord) fromSlider->recBar->setValue(0);
    else fromSlider->recBar->setValue(fromRec);

    fromSlider->setEnabled(true);
    toSlider->setEnabled(true);

     // now ttags are set, add time feedback to title
    fromSlider->updateTitle(fromRec);
	toSlider->updateTitle(toRec);
}

// -------------------------------------------------------

void DirectAccessWindow::browseData()
{

 if(QuSlotDebug())
 {
  printf("DirectAccessWindow::browseData \n");
  fflush(stdout);
 }
 
  DvObject_var  dobj;
  DvObject_var  nil_obj;
  int vnum;
  DataBrowseWindow *QBROWSER_widget;

  // browse either list 
  
  if( activeList == varList) {
    
      QListWidgetItem* item=varList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No variable selected");
       return;
     }   
    vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
  
    if( vnum == -1 ) {
      msgBar->showMessage( "No Data Object selected to browse");
      return;
    }

	// read data in from file 
	
	QieDataCont->readVarData(vnum); 

    dobj = QieDataCont->QieVars[vnum]->Vobj;
     
	
	// Attach cleaned xrefs to data object (fixes ptr to var attributes)
	QieDataCont->attachXrefs(vnum, "");
	
    //    open browser
    if( dobj.is_ok() ){

   
      QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text().toLatin1().data(), dobj, "");
      if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->parentsBtn->setEnabled(false);
      }
    }
  }
  else if( activeList == GattrList){
      QListWidgetItem* item=GattrList->currentItem();
     if(item == 0){
       msgBar->showMessage( "No G Attr selected");
       return;
     }
    
     // open G attr
	 DvNode *Gnode = QieDataCont->GlobalList.find(item->text().toLatin1().data());
     if(Gnode) dobj = Gnode->obj();

     //    open browser
     if( dobj.is_ok() ){
	 
        QBROWSER_widget = QbuLaunchBrowse(nil_obj, this, item->text().toLatin1().data(), dobj, "");
        if(QBROWSER_widget != 0) {
			QBROWSER_widget->disallowSaves = true;
			if(QBROWSER_widget->browser != 0) QBROWSER_widget->editMenu->setEnabled(false);
			QBROWSER_widget->parentsBtn->setEnabled(false);
		}
     }
  }
  

}

// -------------------------------------------------------------

void DirectAccessWindow::clearVarList(){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::clearVarList \n");
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = varList->firstChild();
//     while(next){
//         varList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    varList->clearSelection();
    activeList = GattrList;
}

// -----------------------------------------------------------


void DirectAccessWindow::clearGattrList(){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::clearGattrList \n");
        fflush(stdout);
    }
 

//     Q_LIST_VIEW_ITEM * next = GattrList->firstChild();
//     while(next){
//         GattrList->setSelected(next, false);
//         next->listView()->update();
//         next = next->nextSibling();
//     }
  
    GattrList->clearSelection();
    activeList = varList;
}

// ----------------------------------------------

void DirectAccessWindow::keyPressEvent(QKeyEvent * e )
{  
  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if(   getVarBtn->isDefault()) {
		 getVarBtn->setDefault(false);
		 browseVarBtn->setDefault(true);
	  }
	  else if(browseVarBtn->isDefault()) {
		 browseVarBtn->setDefault(false);
		 dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault())  {
		 dismissBtn->setDefault(false);
		 getVarBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(getVarBtn->isDefault()) getData();
	 else if (browseVarBtn->isDefault()) browseData();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// ---------------------------------------------

void DirectAccessWindow::varDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::varDoubleClick \n");
        fflush(stdout);
    }
 
    if( !(getVarBtn->isEnabled()) ) return; // get data disabled
    if(item == 0) return;
    activeList = varList;
    varList->setCurrentItem(item);
    getData();

}

// ---------------------------------------------

void DirectAccessWindow::GattrDoubleClick(QListWidgetItem* item){

    if(QuSlotDebug())
    {
        printf("DirectAccessWindow::GattrDoubleClick \n");
        fflush(stdout);
    }
 

    if(item == 0) return;
    activeList = GattrList;
    GattrList->setCurrentItem(item);
    browseData();

}

// ------------------------------------------------

void DirectAccessWindow::getData()
{

	if(QuSlotDebug())
	{
		printf("DirectAccessWindow::getData \n");
		fflush(stdout);
	}
	getVarBtn->setEnabled(false);
	browseVarBtn->setEnabled(false);
	dismissBtn->setEnabled(false);
	
	int vnum;
	int anum;

	// import from either list 
  
	if( activeList == varList) {
  
		QListWidgetItem *item = varList->currentItem();
		if(item == 0) {
			msgBar->showMessage( "No variable selected");
			getVarBtn->setEnabled(true);
			browseVarBtn->setEnabled(true);
			dismissBtn->setEnabled(true);
			return;
		}
		bool intOK;
		int fromRec = fromSlider->recField->text().toInt(&intOK);
		if(!intOK) fromRec = 0;
  
		int toRec = toSlider->recField->text().toInt(&intOK);
		if(!intOK) toRec = toSlider->maxRecord;
		vnum = QieDataCont->findVarNumByName(item->text().toLatin1().data() );
		if(vnum == -1 ) {
			msgBar->showMessage( "No Data Object selected");
			getVarBtn->setEnabled(true);
			browseVarBtn->setEnabled(true);
			dismissBtn->setEnabled(true);
			return;
		}

		if( !(QieDataCont->QieVars[vnum]->recVary) ){
			toRec = 0;
			fromRec = 0;
		}
		
		QieDataCont->readVarData(vnum);

	//    Add object name to WL
		if( QieDataCont->QieVars[vnum]->Vobj.is_ok() && QieDataCont->QieVars[vnum]->Vobj->seqSize() > toRec){

			QieDataCont->attachXrefs(vnum);
			DvObject_var dobj = QieDataCont->QieVars[vnum]->Vobj->subSequence(fromRec, toRec);

			// Add parents xref for QSAS browse window use
			DvObject_var file_seq = new DvObject(QieDataCont->inFile.name.c_str());

			dobj->change_xref("Parents", file_seq); 

			string sanitizedVarName = QuSanitizeName(item->text().toLatin1().data());
			QwuiPutOnWL(dobj, QString(sanitizedVarName.c_str()));
		}
	}
	else if(activeList == GattrList){
		
        QListWidgetItem* item=GattrList->currentItem();
		if(item == 0){
			msgBar->showMessage( "No G Attr selected");
			getVarBtn->setEnabled(true);
			browseVarBtn->setEnabled(true);
			dismissBtn->setEnabled(true);
			return;
		}
    
		// open G attr
		DvNode *Gnode = QieDataCont->GlobalList.find(item->text().toLatin1().data());
		if(Gnode) {
			DvObject_var dobj = Gnode->obj();

			//   put G attr on WL
			if( dobj.is_ok() ){

				string sanitizedGattrName = QuSanitizeName(item->text().toLatin1().data());
				QwuiPutOnWL(dobj, QString(sanitizedGattrName.c_str()));
				
			}
		}
	}
	
	getVarBtn->setEnabled(true);
	browseVarBtn->setEnabled(true);
	dismissBtn->setEnabled(true);

}



// Utility call for qwui
//----------------------

 void DaLaunchWindow(QString file, QWidget *parent)
{
  // direct access window
  
	DirectAccessWindow *QDA_widget = new DirectAccessWindow(parent, "import");
	QSize qsasGUISize = qsasWindow->frameSize();
	QDA_widget->move(qsasGUISize.width(),20);
	
	if ( !file.isNull() ){  // open file and get data
 
		QDA_widget->setWindowTitle(file);
		
		QDA_widget->msgBar->showMessage(" " );
		
		QDA_widget->show();

		// open file
		int nRecs = QDA_widget->getFileContents(file.toLatin1().data());


		// set sliders
		
		QDA_widget->setSliders(nRecs);

	}
	else {
		QDA_widget->msgBar->showMessage("no file selected?");
		QDA_widget->show();
	}

  return;
}
