
//  DataSelector.cc
//
// Data Selection Widget
//
// March 2009: Tony Allen
//

#include "qt_version.h"
#include "qdutil.h"
#include "qdi.h"
#include "qwui.h"
#include <sys/stat.h>

#include "DataSelector.h"

#define DA_PIXEL_WIDTH 5
#define DA_PIXEL_HEIGHT 12
#define DMAP_MODE 0664

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release;

unsigned int DataBaseItem::item_counter=0;

// io streams for map classes
// --------------------------

ofstream& operator<<(ofstream& v_os, const dataEntry& entry)
{ 
	v_os << entry.variableName << '\n';
	int size=entry.fileName.size();
	v_os << size<< '\n';
	for(int i=0; i<entry.fileName.size(); i++)
	{
		v_os << entry.fileName[i] << ',';
		v_os << setprecision(11) << entry.start[i]<< ',';
		v_os << entry.end[i] << '\n';
	}
	return v_os;

}


ifstream& operator>>(ifstream& v_is,  dataEntry& entry)
{ 
	char buff[201];
	v_is.getline(&(buff[0]), 200);
	entry.variableName = string(buff, v_is.gcount()-1);
	
	int cnt;
	v_is >> cnt;
	char c = v_is.get(); // skip newline
	
	entry.fileName.resize(cnt);
	entry.start.resize(cnt);
	entry.end.resize(cnt);
	for(int i=0; i<cnt; i++)
	{
		v_is.getline(&(buff[0]), 200, ',');
		entry.fileName[i] = string(buff, v_is.gcount());

		double value;
		v_is >> setprecision(11) >> value;
		entry.start[i] = value;
		c = v_is.get(); // skip comma
		v_is >> value;
		entry.end[i] = value;
		c = v_is.get(); // skip newline
	}
	return v_is;

}

ofstream& operator<<(ofstream& v_os, const variableEntry& entry)
{ 
	v_os << entry.variableName << '\n';
	int size=entry.start.size();
	v_os << size << '\n';
	for(int k=0; k<size; k++)
	{
		v_os << setprecision(11) << entry.start[k] << ',';
		v_os << entry.end[k] << '\n';
	}
	return v_os;

}

ifstream& operator>>(ifstream& v_is,  variableEntry& entry)
{ 
	char buff[201];

	v_is.getline(&(buff[0]), 200);
	entry.variableName = string(buff, v_is.gcount()-1);
	double value;
	int count;
	v_is >> count;
	char c = v_is.get(); // skip newline

	if(count > 0)
	{
		v_is >> setprecision(11) >> value;
		entry.start.assign(count, value);
		c = v_is.get(); // skip comma
		v_is >> value;
		entry.end.assign(count, value);
		c = v_is.get(); // skip newline
	}
	for(int k=1; k<count; k++)
	{
		v_is >> value;
		entry.start[k] = value;
		c = v_is.get(); // skip comma
		v_is >> value;
		entry.end[k] = value;
		c = v_is.get(); // skip newline
	}
	return v_is;

}

ofstream& operator<<(ofstream& v_os, const variableList& varList)
{
	int size=varList.size();
	v_os << size << '\n';
	for (int i=0; i<varList.size(); i++) 
	{
		v_os << varList[i] << '$';
	}
	return v_os;
}

ifstream& operator>>(ifstream& v_is, variableList& varList)
{
	int size = varList.size();

	char c; // = v_is.get(); // skip newline
	
	for(int i=0; i<size; i++)
	{
		variableEntry entry;
		v_is >> entry;
		c = v_is.get(); // skip $
		varList[i] = entry;
	}
	return v_is;
}

ofstream& operator<<(ofstream& v_os, const mapEntry& entry)
{ 
	v_os << entry.fileName << ':';
	v_os << entry.varList << '/';
	return v_os;

}

ifstream& operator>>(ifstream& v_is,  mapEntry& entry)
{ 
	char buff[201];
	// done this way as Windows has C:/ in file name
	char c;
	v_is.getline(&(buff[0]), 200);

	int n=v_is.gcount()-1;
	while(buff[n] != ':' && n > 0){
		n--;
	}
	
	entry.varList.clear();
	entry.varList.resize(atof(&(buff[n+1])));
	
	entry.fileName = string(buff, n); // drops ':'
	
	v_is >> entry.varList;
	c = v_is.get(); // skip '/'
	return v_is;

}

ofstream& operator<<(ofstream& v_os, const mapList& mList)
{
	int size=mList.size();
	v_os << size << '\n';
	for (int i=0; i<mList.size(); i++) 
	{
		v_os << mList[i] << '\n';
	}
	return v_os;
}

ifstream& operator>>(ifstream& v_is, mapList& mList)
{
	int size;
	v_is >> size;
	char c = v_is.get(); // skip newline
	if(size >  0)
	{
		mapEntry entry;
		v_is >> entry;
	 	c = v_is.get(); // skip newline
		mList.assign(size, entry);
	}
	for(int i=1; i<size; i++)
	{
		mapEntry entry;
		v_is >> entry;
	 	c = v_is.get(); // skip newline
		mList[i] = entry;
	}

	return v_is;
}

// class DataSelector
// ------------------

DataSelector::DataSelector( QWidget *parent, const char *name ):QWidget(parent)
{	
	def_db_dir="";
	interruptFlag = false;
	
   _top_gl = new QGridLayout( this );

   _top_gl->setSpacing(4);
   _top_gl->setMargin(4);
   _top_gl->setRowStretch(0,10);
   _top_gl->setRowStretch(1,1);
   
  // splitter to manage database list view and data selection widget

   _top_splitter = new QSplitter( this );
   _top_gl->addWidget( _top_splitter, 0,0,1,2 );
	
// databases list view of available data sources

   _database_listview=new DataBaseListView(_top_splitter);
   _database_listview->setColumnWidth(0,300);
   _database_listview->setColumnWidth(1,250);
   _database_listview->setMinimumWidth(550);
   _database_listview->_data_selector=this;
           
   
	// RHS Tabs
	QTabWidget *tabWidget = new QTabWidget(_top_splitter);
	
	// Data Selector
	QWidget *selectionTab = new QWidget(tabWidget);

   QGridLayout* dataselection_layout = new QGridLayout( selectionTab);
   dataselection_layout->setSpacing(4);

	QLabel *blank = new QLabel(" ");
   dataselection_layout->addWidget( blank, 0, 0 );

  // add to data selection list button
   _add_button = new QPushButton( ">> Add >>", selectionTab );
   _add_button->setMinimumSize(_add_button->sizeHint());
   _add_button->setEnabled(false);
   dataselection_layout->addWidget( _add_button, 1, 0 );
   connect( _add_button, SIGNAL(clicked()), this, SLOT(add_to_selection_list()) );

  // remove from data selection list button
   _remove_button = new QPushButton( "Remove <<", selectionTab );
   _remove_button->setMinimumSize(_remove_button->sizeHint());
   _remove_button->setEnabled(false);
   dataselection_layout->addWidget( _remove_button, 2, 0 );
   connect( _remove_button, SIGNAL(clicked()),  this, SLOT(remove_from_selection_list()) );

  // clear all items from data selection list button
   _clear_button = new QPushButton( "Clear <<<", selectionTab );
   _clear_button->setMinimumSize(_clear_button->sizeHint());
   _clear_button->setEnabled(false);
   dataselection_layout->addWidget( _clear_button, 3, 0 );
   connect( _clear_button, SIGNAL(clicked()), this, SLOT(clear_selection_list()) );

  // list of selected data 
   _data_selection_listview=new DataSelectionListView(this, selectionTab);
   _data_selection_listview->setColumnWidth(1, 150);
   _data_selection_listview->setColumnWidth(0, 200);
   _data_selection_listview->setMinimumSize(_data_selection_listview->sizeHint());

	dataselection_layout->addWidget( _data_selection_listview, 0,1,6,1 );
	selectionTab->setMinimumSize(selectionTab->sizeHint());	

	// Database Manager
	managerTab = new DB_Manager(tabWidget);	
	managerTab->_folder_listview = _database_listview;
	managerTab->_data_selector = this;
	managerTab->setMinimumSize(managerTab->sizeHint());	

	tabWidget->addTab(managerTab, "Manage Databases");
	tabWidget->addTab(selectionTab, "Data Selection");
	tabWidget->adjustSize();
	tabWidget->setCurrentIndex(tabWidget->indexOf(selectionTab));
	
   _time_editor = new TimeIntervalWidget( this, 1);
   _time_editor->adjustSize();

   connect(_time_editor,SIGNAL(intervalUpdated()),this,SLOT(time_updated()) );
 
   _top_gl->addWidget( _time_editor, 1,0,1,2 );
  
   QHBoxLayout *buttons_layout=new QHBoxLayout();
   QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);
	
	progressBarLbl = new QLabel(this);
	progressBarLbl->setFixedWidth(150);
	buttons_layout->addWidget(progressBarLbl);  
	
	progressBar = new QProgressBar(this);
	progressBar->setValue(0);
	progressBar->setFixedWidth(100);
	buttons_layout->addWidget(progressBar);  
	
	interrupt = new QPushButton( "Interrupt", this );
	interrupt->setMinimumSize(interrupt->sizeHint());
	connect( interrupt, SIGNAL(clicked()), this, SLOT(issueInterrupt()) );
	interrupt->setEnabled(false);
	buttons_layout->addWidget(interrupt);

	QLabel * blank3 = new QLabel(this);
	blank3->setFixedWidth(50);
	blank3->setText("   ");
	buttons_layout->addWidget(blank3);  
  
	_get_data_button = new QPushButton( "Get Data", this );
	_get_data_button->setMinimumSize(_get_data_button->sizeHint());
	_get_data_button->setEnabled(false);
	connect( _get_data_button, SIGNAL(clicked()), this, SLOT(get_data()) );
	buttons_layout->addWidget(_get_data_button);
  
	QLabel * blank2 = new QLabel(this);
	blank2->setFixedWidth(50);
	blank2->setText("   ");
	buttons_layout->addWidget(blank2);  
	
	quit = new QPushButton( "Dismiss", this );
	quit->setMinimumSize(quit->sizeHint());
	buttons_layout->addWidget(quit);  
  
	_top_gl->addLayout(buttons_layout,2,1);
	adjustSize(); 

}

void DataSelector::add_item_to_selection(DataBaseItem* db_item)
{

	if(db_item)
	{
		if(db_item->db_item_type == LVI_FOLDER) return;
		_data_selection_listview->add_item(db_item);
	}

	// clear item in database view
	// nb: don't use node->setSelected(bool) //TODO pay attention to recursion issues!

	_clear_button->setEnabled(true);
	_remove_button->setEnabled(true);
   _get_data_button->setEnabled(true);

}

void DataSelector::add_to_selection_list()
{
	QList<QTreeWidgetItem*> items=_database_listview->selectedItems();
	
	foreach(QTreeWidgetItem* item, items)
	{
		add_item_to_selection((DataBaseItem*)item);
	}
}

void DataSelector::remove_from_selection_list()
{
	QList<QTreeWidgetItem*> items=_data_selection_listview->selectedItems();
	
	foreach(QTreeWidgetItem* i, items)
	{
		delete i;
	}
	
	// if nothing left, disable clear and remove
	if(_data_selection_listview->topLevelItemCount()==0)
	{
		_clear_button->setEnabled(false);
		_remove_button->setEnabled(false);
		_get_data_button->setEnabled(false);
	}
}

void DataSelector::clear_selection_list()
{
	_data_selection_listview->clear();
	_clear_button->setEnabled(false);
	_remove_button->setEnabled(false);
	_get_data_button->setEnabled(false);

}

void DataSelector::update_database_selection()
{
  _add_button->setEnabled(!_database_listview->selectedItems().isEmpty());
}

void DataSelector::issueInterrupt()
{
	interruptFlag = true;
	progressBarLbl->setText("Cancelled...");
	progressBarLbl->repaint();
	this->repaint();
}

void DataSelector::disableInterrupt()
{
	progressBar->setValue(0);
	progressBarLbl->setText(" ");
	interrupt->setEnabled(false);
	interruptFlag = false;
	this->resetButtons(false);
	QApplication::restoreOverrideCursor();
}

void DataSelector::activateInterrupt(const char *txt)
{
	interruptFlag = false;
	progressBarLbl->setText(txt);
	progressBarLbl->repaint();
	this->repaint();
	this->resetButtons(true);	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
}

bool DataSelector::isInterrupted(const char *txt)
{
		qApp->processEvents();
		if(interruptFlag == true)
		{
			DSwindow->statusBar()->showMessage(txt);
			disableInterrupt();
			return true;
		}
		return false;
}

// ----------------------------------------

void DataSelector::resetButtons(bool turnOff)
{
	if(QuSlotDebug())
	{
		printf("DataSelector::resetButtons, turnOff = %d\n", turnOff);
		fflush(stdout);
	}

	static bool getDataLast = true;
	static bool removeLast = true;
	static bool clearLast = true;
	static bool reSaveLast = true;
	
	if(turnOff)
	{
		interrupt->setEnabled(true);
		getDataLast = _get_data_button->isEnabled();
		_get_data_button->setEnabled(false);
		quit->setEnabled(false);
		_add_button->setEnabled(false);
		removeLast = _remove_button->isEnabled();
		_remove_button->setEnabled(false);
		clearLast = _clear_button->isEnabled();
		_clear_button->setEnabled(false);
		
		// manager buttons
		managerTab->db_erase->setEnabled(false);
		managerTab->db_clear->setEnabled(false);
		managerTab->db_delete->setEnabled(false);
		managerTab->db_update->setEnabled(false);
		managerTab->db_add->setEnabled(false);
		managerTab->db_remove->setEnabled(false);
		managerTab->db_create->setEnabled(false);
		reSaveLast = managerTab->db_resave->isEnabled();
		managerTab->db_resave->setEnabled(false);

		
		// DB menu
		DSwindow->database_menu->setEnabled(false);
		
	}
	else
	{
		interrupt->setEnabled(false);
		_get_data_button->setEnabled(getDataLast);
		quit->setEnabled(true);
		_add_button->setEnabled(true);
		_remove_button->setEnabled(removeLast);
		_clear_button->setEnabled(clearLast);
		
		// Manager buttons
		managerTab->db_erase->setEnabled(true);
		managerTab->db_clear->setEnabled(true);
		managerTab->db_delete->setEnabled(true);
		managerTab->db_update->setEnabled(true);
		managerTab->db_add->setEnabled(true);
		managerTab->db_remove->setEnabled(true);
		managerTab->db_create->setEnabled(true);
		managerTab->db_resave->setEnabled(reSaveLast);
		
		// DB Menu
		DSwindow->database_menu->setEnabled(true);
		
	}

}

void DataSelector::get_data()
{
	qApp->processEvents();
	activateInterrupt("Fetching Data...");
	
	int num = _data_selection_listview->topLevelItemCount();
	if( num == 0 ) 
	{
		disableInterrupt();
		return;
	}
	
	DvEvent tintv = _time_editor->getInterval();
		
	progressBar->setMaximum(num);
	progressBar->setValue(1);
	progressBar->repaint();


	// go through selection list getting variables from same files

	DSwindow->statusBar()->showMessage("Checking Database...");
	DSwindow->statusBar()->repaint();

	qApp->processEvents();
		
	dataFetchList fetchList;
	QTreeWidgetItemIterator it(_data_selection_listview);
	while (*it)
	{
		DataSelectionItem* data_select_item = dynamic_cast<DataSelectionItem*>(*it);
		if(data_select_item)
		{
			vector<string> fnames;
			data_select_item->get_files(tintv, fnames);
			string varName = data_select_item->text(0).toLatin1().data();
			int inList = -1;
			for(int i=0; i<fetchList.size(); i++)
			{
				if( fnames.size() == 0 || fetchList[i].fileList.size() == 0) continue;
				// find if this file is already needed
				if(fnames[0] == fetchList[i].fileList[0])
				{
					inList = i;
					break;
				}
			}
			if( inList == -1)
			{
				dataFetchEntry newEntry;
				newEntry.fileList = fnames;
				newEntry.toFetchList.push_back(varName);
				fetchList.push_back(newEntry);
			}
			else
			{
				fetchList[inList].toFetchList.push_back(varName);
			}

		}   // end if
		
		++it;
	}
			
	// get data object
	bool OK=true;
	for (int i=0; i<fetchList.size(); i++)
	{
		progressBar->setValue(i+1);
		progressBar->repaint();
				
		if(isInterrupted("Fetch Data Interrupted")) return;

		vector <DvObject_var> objList;
		dataFetchEntry fetchEntry;
		fetchEntry.copyFrom(fetchList[i]);
		
		if(fetchEntry.fileList.size() == 0) {
			DSwindow->statusBar()->showMessage("No Data in range for variable(s)");
			OK=false;
			continue;
		}
		
		bool interrupted = getVarList(fetchEntry.fileList, fetchEntry.toFetchList, tintv, objList);
		if(interrupted) 
		{
			disableInterrupt();
			DSwindow->statusBar()->showMessage("Fetch Data Interrupted");
			return;
		}
		// unpack objects to WL
		for(int j=0; j<objList.size(); j++)
		{
			if(objList[j].is_nil()) continue;

			string sanitizedVarName = QuSanitizeName(fetchEntry.toFetchList[j].c_str());
			QwuiPutOnWL(objList[j], QString(sanitizedVarName.c_str()));
		}
	}

	if(OK) DSwindow->status("Done");
	disableInterrupt();
}

// -------------------------------------------------------

void DataSelector::set_time_interval()
{
	if(QuSlotDebug())
	{
		printf("DataSelector::set_time_interval \n");
		fflush(stdout);
	}
	
	// get time range from each database
	
	double start = 1.e30;
	double end = -1.e30;

	int num=_database_listview->topLevelItemCount();
	DataBaseItem* item;
	for(int i=0; i<num; ++i)
	{
		item=(DataBaseItem*)_database_listview->topLevelItem(i);
		item->getTimeRange(start, end);
	}

	if(start < end) 
	{
		_time_editor->setAvailabilityRange(DvEvent(DvTime(start), DvTime(end)));
		_time_editor->setInterval(DvTime(start), DvTime(end), true); // also sets view range
	}
	
	this->update_data_availability();
}

// -------------------------------------------------------

void DataSelector::time_updated()
{
	if(QuSlotDebug())
	{
		printf("DataSelector::time_updated \n");
		fflush(stdout);
	}

	this->update_data_availability();
}

// -------------------------------------------------------

void DataSelector::update_data_availability()
{
	if(QuSlotDebug())
	{
		printf("DataSelector::update_data_availability \n");
		fflush(stdout);
	}

	// update data availability pixel maps
	// fetch time interval
	DvEvent tintv = _time_editor->getInterval();

	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	qApp->processEvents();

  // valid time interval, so get time tags over which to determine availability from current resolution setting

	QLinkedList<DataBaseItem*> items=_database_listview->getChildren();

    emit setInterval(tintv.start_abs().getEpoch2000(), tintv.end_abs().getEpoch2000());
	
	QApplication::restoreOverrideCursor();

	qApp->processEvents();

}
// ---------------------------------------------------------

bool  DataSelector::read_database(const char *db_filename)
{
	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::read_database %s \n", db_filename);
		fflush(stdout);
	}
	bool canReSave = false;
	
	// trap old syntax DB (should be .qsdb)
	if( strstr(db_filename, ".qdb") != NULL ) {
		printf("Old syntax DB %s \n", db_filename);
		return false;
	}
	
	activateInterrupt("Reading Database...");
	// keep db root path for subsequent calls
	string rootName = db_filename;
	int pos = rootName.rfind("/");
	if(pos != string::npos) db_dir = rootName.substr(0,pos).c_str();
	rootName = rootName.substr(pos+1, rootName.size() - pos - 6);

	qApp->processEvents();
	
	// close if already open (may have changed)
	QList<QTreeWidgetItem*> items=_database_listview->findItems(rootName.c_str(), Qt::MatchExactly, 0);
	foreach(QTreeWidgetItem* item, items)
	{
		delete (DataBaseItem*)item;
	}
	
	// read in db file

		ifstream db_file(db_filename);

		if ( db_file.is_open() )
		{			
			DataBaseItem* DBroot= new DataBaseItem(LVI_FOLDER, _database_listview, rootName);
			DBroot->setExpanded(true);
			DBroot->DB_NameAndPath = db_filename;
			DBroot->fullPathAndName = "";
			int numEntries;
			db_file >> numEntries;
	
			progressBar->setMaximum(numEntries);
			progressBar->setValue(1);
			qApp->processEvents();
			progressBar->repaint();

			for(int i=0; i<numEntries; i++)
			{
				progressBar->setValue(i+1);
				progressBar->repaint();
				
				string entry;
				db_file >> entry;

				// create List view entries
				for(int i=0; i<entry.size(); i++) if(entry[i] == '}') entry[i] = ' ';
				
				int pos = entry.find(",");
				string nameAndPath = entry.substr(0,pos).c_str();
				entry.replace(0,pos+1,""); // entry now has map name with path
				
				pos = nameAndPath.rfind("/");
				string folder = nameAndPath.substr(0,pos);
				string name = nameAndPath.c_str();
				name = name.replace(0, pos+1, "");
				DataBaseItem* parentDir=DBroot->findChildByName(folder);

				DataBaseItem* newDBitem;
				if(parentDir)
				{
					newDBitem=new DataBaseItem(LVI_FOLDER, parentDir, name);
					parentDir->setExpanded(true);
				}
				else
				{
					newDBitem=new DataBaseItem(LVI_FOLDER, DBroot, name);
				}
				
				newDBitem->setExpanded(false);
				newDBitem->fullPathAndName = nameAndPath;
				newDBitem->DB_NameAndPath = db_filename;
				newDBitem->mapName = entry;
				
				// read map for this folder and create each DataBaseItem
				newDBitem->read_map(entry);

				if(isInterrupted(" ")) return true;

			}
			db_file.close();
		
			if(DBroot->removeEmptyFolders()) canReSave = true;
			
			if(DBroot->childCount() == 0)
			{
				DSwindow->statusBar()->showMessage("No readable data files in Database directories");
				disableInterrupt();
				return false;
			}
			_database_listview->setSortingEnabled(false);
			qApp->processEvents();

			set_time_interval();
	
		}

	disableInterrupt();
	if(canReSave) managerTab->db_resave->setEnabled(true);
	
    _database_listview->updateWidgets();
    update_data_availability();
	return false;
}

// -----------------------------------------
void DataSelector::set_def_db()
{
	// create database file
	if(def_db_dir=="") 
	{
		def_db_dir = QSAS_profiles->current()->ConfigPath.toLatin1().data();
		if(def_db_dir == "")
		{
			const char *userEnv;
#ifdef WINDOWS
			userEnv = getenv("USERPROFILE");
#else
			userEnv = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
			if(userEnv) def_db_dir = userEnv;
		}
		if( def_db_dir != "") db_dir = def_db_dir;
	}
}
// -----------------------------------------

string DataSelector::write_db(vector <string> &db_entry_list, const char *title)
{	
	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::write_db \n" );
		fflush(stdout);
	}

	set_def_db(); // sets up db_dir if not already done

	QString saveAsFile = db_dir.c_str();
	saveAsFile += "/Database.qsdb";
	QString database_name = QFileDialog::getSaveFileName(this, tr(title),  saveAsFile, tr("(*.qsdb)"));
	if(database_name.isEmpty())
	{
		DSwindow->status("Cancelled...");
		return "";
	}

	if( !database_name.endsWith(".qsdb")) database_name.append(".qsdb");
    string db_name = database_name.toLatin1().data();

	update();
	qApp->processEvents();
	
    ofstream db_file(db_name.c_str());
   
    if ( db_file.is_open() )
    {
        db_file << db_entry_list.size();
        db_file << "\n";
        for(int i=0; i<db_entry_list.size(); i++)
        {
            db_file << db_entry_list[i];
            db_file << "\n";
        }
        db_file.close();
    }
    else
    {
        DSwindow->statusBar()->showMessage("Could Not write file");
        write_db(db_entry_list, "(Write permission failure, select alternative database file");
    }

	
	return db_name;
}

// ---------------------------

void DataSelector::open_database()
{
	QString openDir = db_dir.c_str();
	
	if(def_db_dir=="") 
	{
		def_db_dir = QSAS_profiles->current()->ConfigPath.toLatin1().data();
		if(def_db_dir == "")
		{
			const char *userEnv;
#ifdef WINDOWS
			userEnv = getenv("USERPROFILE");
#else
			userEnv = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
			if(userEnv) def_db_dir = userEnv;
		}
		
		if(def_db_dir != "") 
		{
			db_dir = def_db_dir;
			openDir = def_db_dir.c_str();
		}
	}
	
	#if defined(Q_OS_MACX)
		openDir += "/Database.qsdb";
	#endif

	
	QString qs_db_filename = QFileDialog::getOpenFileName(qsasWindow,  tr("Open Database File"), openDir, tr("Database (*.qsdb)"));

	if ( !qs_db_filename.isEmpty() ) 
	{
		bool interrupted = read_database (qs_db_filename.toLatin1().data());
		if(interrupted) DSwindow->statusBar()->showMessage("Interrupt detected while reading DB");
	}
}

//---------------------------------------------------------------------

bool DataSelector::getVarList(vector <string> &fileList, vector <string> &varToGetList, DvEvent &tintv, vector <DvObject_var> &objList)
{
	DvObject_var nilObj;
	
	// utility to get variables from list for interval using files in list

	// initialise object list with null var objects
	objList.assign(varToGetList.size(), nilObj);

	int nFiles = fileList.size();
	// read each file in turn
	for(int ifile=0; ifile<nFiles; ifile++)
	{
		if(isInterrupted("Fetch Data Interrupted")) return true;
				
		QieData *QieDataCont = new QieData();
		int nRecs = QieDataCont->queryFile(fileList[ifile].c_str());

		if( nRecs > 0 ){
			
			QString txt = QString("File %1 of %2 : %3").arg(ifile+1).arg(nFiles).arg(fileList[ifile].c_str());
			DSwindow->statusBar()->showMessage(txt.toLatin1().data());
			DSwindow->statusBar()->repaint();
			qApp->processEvents();

			// re initialise data block size
			// int nRecs = QieDataCont->readDataRecords();

			// for each variable get data in selected interval from file
			for(int jvar=0; jvar<varToGetList.size(); jvar++)
			{
				DSwindow->statusBar()->showMessage(QString("file %1 for var %2").arg(ifile+1).arg(varToGetList[jvar].c_str()));
				DSwindow->statusBar()->repaint();
				qApp->processEvents(); // call here to give interrupt chance
				
				int varN = QieDataCont->findVarNumByName(varToGetList[jvar].c_str());
				int recStart, recEnd;
				
				// find records in interval (Themis has multiple time lines, so do for each var)

				DvObject_var ttv = QieDataCont->readTT(varN, nRecs);
				
				// default: file entirely inside interval
				int seqSize = ttv->seqSize();
				recStart = 0;
				recEnd = seqSize-1;

				if( !(tintv.start_abs() <= ttv->time(0) && tintv.end_abs() >= ttv->time(recEnd)) )
				{
					// find records as not completely contained in ivl
					for(int k=0; k<seqSize; k++)
					{
						if(tintv.start_abs() > ttv->time(k)) recStart = k+1;
						if(tintv.end_abs() < ttv->time(seqSize-k-1)) recEnd = seqSize - k - 2;
					}
					if(recEnd < 0 || recStart > seqSize -1) continue; // failed to find good interval
				}

				// get var for this range

				QieDataCont->readVarData(varN);
				
				QieDataCont->attachXrefs(varN);
								
				DvObject_var Vobj = QieDataCont->QieVars[varN]->Vobj->subSequence(recStart, recEnd);
				
				// merging (merge takes care of data order and overlap)
				objList[jvar] = objList[jvar]->mergeWith(Vobj);
			}
		}
	
		delete QieDataCont;
	}
	
	// put variables on WL
	for(int jvar=0; jvar<objList.size(); jvar++)
	{
		//    Add object name to WL
		if( objList[jvar].is_ok() ){
			// Add parents xref for QSAS browse window use
			DvObject_var file_seq = new DvObject(DvString(""), fileList.size());
			for(int ifile=0; ifile<fileList.size(); ifile++) file_seq->str(ifile) = fileList[ifile]; 
			objList[jvar]->change_xref("Parents", file_seq); 
		}
    }

	return false;
}



// DataSelectorWindow  class
// -----------------------------------

DataSelectorWindow::DataSelectorWindow( QWidget *parent, const char *name )
: QMainWindow(0)
{
  _data_selector = new DataSelector(this,"data_selector");
  setCentralWidget( _data_selector );
  _data_selector->DSwindow = this;
  menuBar()->clear();

  QMenu* file_menu =   menuBar()->addMenu("File" );

  file_menu->clear();
  file_menu->addAction("Save As...", this, SLOT(saveAsXML()));
  file_menu->addSeparator();
  file_menu->addAction("&Include...", this, SLOT(include()), Qt::CTRL+Qt::Key_I);
  file_menu->addAction("&Restore...", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
  file_menu->addSeparator();
  file_menu->addAction("Close", this, SLOT(hide()));


  database_menu =   menuBar()->addMenu("Database" );

  database_menu->clear();
  database_menu->addAction("Open Database...", this, SLOT(open_database()));
  database_menu->addAction("Close Database", this, SLOT(close_database()));
  database_menu->addAction("Create Database", this, SLOT(create_database()));
  menuBar()->addSeparator();

  this->setMinimumSize(this->sizeHint());

  QMenu* edit_menu = menuBar()->addMenu("Edit" );
  
  edit_menu->clear();
  edit_menu->addAction("Sort View Down", _data_selector->_database_listview, SLOT(sortDown()));
  edit_menu->addAction("Sort View Up", _data_selector->_database_listview, SLOT(sortUp()));
  edit_menu->addSeparator();
  edit_menu->addAction("Show Files", _data_selector->_database_listview, SLOT(show_files()));
  edit_menu->addSeparator();
  
  QAction *delAct = edit_menu->addAction("Delete from View", _data_selector->_database_listview, SLOT(deleteItem()), Qt::Key_Delete);
  QList<QKeySequence> keyList;
  keyList.append(Qt::Key_Delete);
  keyList.append(Qt::Key_Backspace);
  keyList.append(Qt::CTRL+Qt::Key_X);
  delAct->setShortcuts(keyList);

  
  QMenu * help_menu = menuBar()->addMenu("Help");

  help_menu->addAction("Qsas Help Topics...",this, SLOT(launch_QSAS_Help()));
  help_menu->addAction("Help on &Data Selection...", this,SLOT(launch_DS_Help()));
  help_menu->addSeparator();
  help_menu->addAction("About Qsas...",qsasWindow->help_window,SLOT(about()));
 
  this->connect(qsasWindow, SIGNAL(saveSessionRequest(QString)),  this, SLOT(saveSession(QString)) );
  this->connect(qsasWindow, SIGNAL(restoreSessionRequest(QString)), this, SLOT(restoreSession(QString)) );
  this->connect( _data_selector->quit, SIGNAL(clicked()), this, SLOT(close()) );

	// tell QIE what to use for fp_display file pointer
	if(QuSlotDebug())  setDebug(true);

}

// -------------------------------
void DataSelectorWindow::close_database()
{	
	QLinkedList<DataBaseItem*> items=_data_selector->_database_listview->getSelectedDBs();
	if(items.isEmpty())
		return;

	foreach(DataBaseItem* item, items)
	{
		delete item;
	}
}

// -------------------------------
void DataSelectorWindow::create_database()
{	
	_data_selector->managerTab->createDB();
}


// ------------------------

void DataSelectorWindow::launch_QSAS_Help()
{ qsasWindow->help_window->manEntry("QSAS_help_index.html"); }  

// ------------------------

void DataSelectorWindow::launch_DS_Help()
{ qsasWindow->help_window->manEntry("QSAS_data_selection.html"); }

// -------------------------

void DataSelectorWindow::open_database()  { _data_selector->open_database();}

// -----------------------------

    
void DataSelectorWindow::saveSession(QString folder){

	if(folder.isEmpty()) return;
	  
  // set session folder and save
	QString fileN = QString(folder.toLatin1().data());
	fileN += QString("/saveSet.qx_dbSelect");
	
	saveAsXML(fileN);
	
}


void DataSelectorWindow::restoreSession(QString folder){

	if ( QuSlotDebug() )
	{
		printf( "DataSelectorWindow::restoreSession \n");
		fflush(stdout);
	}
    
	if(folder.isEmpty()) return;

	QString fileN =  QString(folder.toLatin1().data());
	fileN += QString("/saveSet.qx_dbSelect");
    
	restoreXML(fileN, true);
  
}

void  DataSelectorWindow::saveAsXML(QString path)
{
	ofstream file;
	file.open(path.toLatin1().data(),ios::out);

	if(file.is_open())
	{
		DataSelectionItem* ds_item;
		QDomDocument doc("QSAS_Data_Selector_save");

		QDomElement root=doc.createElement("QSAS_Data_Selector");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		int cnt=_data_selector->_database_listview->topLevelItemCount();
		DataBaseItem* item;
		for(int i=0; i<cnt; ++i)
		{
			item=(DataBaseItem*)(_data_selector->_database_listview->topLevelItem(i));
			root.appendChild(createTextXMLNode(doc, "DB_Item",item->DB_NameAndPath));
		}

		QTreeWidgetItemIterator it(_data_selector->_data_selection_listview);
		while(*it)
		{
			QDomElement selectionItem=doc.createElement("Selection_Item");
			ds_item=(DataSelectionItem*)(*it);
			string itemText0 = string(ds_item->text(0).toLatin1().data());
			string itemText1 = string(ds_item->text(1).toLatin1().data());
			selectionItem.appendChild(createTextXMLNode(doc, "Variable",itemText0));
			selectionItem.appendChild(createTextXMLNode(doc, "Source",itemText1));
			selectionItem.appendChild(createTextXMLNode(doc, "Map_Name",ds_item->mapName));
			QDomElement entryItem=doc.createElement("DataEntry");
			selectionItem.appendChild(entryItem);
                
			entryItem.appendChild(createTextXMLNode(doc, "Name",ds_item->databaseEntry.variableName));

			int size=ds_item->databaseEntry.fileName.size();
			for(int i=0; i<size; i++)
			{
				QDomElement el=doc.createElement("File_Name");
				entryItem.appendChild(el);
				el.appendChild(doc.createTextNode(ds_item->databaseEntry.fileName[i].c_str()));
				el.setAttribute("Start",ds_item->databaseEntry.start[i]);
				el.setAttribute("End",ds_item->databaseEntry.end[i]);
			}
			root.appendChild(selectionItem);
                
			++it;
		}

		DvEvent TIobj = _data_selector->_time_editor->getViewRange();
		
		root.appendChild(createTextXMLNode(doc, "Range_From", TIobj.start().getISOstring(1)));
		root.appendChild(createTextXMLNode(doc, "Range_To", TIobj.end().getISOstring(1)));
            
		QString xml = doc.toString();
		file << xml.toLatin1().data();
		file.close();
	}

}
void  DataSelectorWindow::saveAsXML()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), "QSAS DB selector save (*.qx_dbSelect);; XML files (*.xml)");

    if(!path.isEmpty())
    {
		if(!path.contains(".qx_dbSelect") && !path.contains(".xml")){
			path += ".qx_dbSelect";
		}
		saveAsXML(path);
	}
}

void  DataSelectorWindow::restoreXML(QString path, bool replace)
{

	if( !QFile::exists(path) ) return;
	    
	QDomDocument doc;
	QFile myFile(path);
	doc.setContent(&myFile);
	myFile.close();
        
	QDomElement docElem = doc.documentElement();
	if(!docElem.isNull())
	{
		QString s;
		// clear list view of open .qsdb files
		if(replace) _data_selector->_database_listview->clear();

		string guiID="0.0.00";

		if(getXMLNodeText(docElem, "QSAS_Version", s))
		{
			guiID=s.toLatin1().data();
		}
		setRestoreVersion(guiID.c_str());
			
		QDomElement dbItem=docElem.firstChildElement("DB_Item");
		while(!dbItem.isNull())
		{
            _data_selector->read_database(dbItem.firstChild().nodeValue().toLatin1().data());
			
			dbItem=dbItem.nextSiblingElement("DB_Item");
		}
			
		// clear selection list view
		if(replace) _data_selector->_data_selection_listview->clear();
			
		QDomElement selectionItem=docElem.firstChildElement("Selection_Item");
		while(!selectionItem.isNull())
		{
			QString text0="";
			QString text1="";
			QString mName="";
				
			if(getXMLNodeText(selectionItem, "Variable", s))
			{
				text0=s;
			}
			if(getXMLNodeText(selectionItem, "Source", s))
			{
				text1=s;
			}
			if(getXMLNodeText(selectionItem, "Map_Name", s))
			{
				mName=s;
			}
			if(!text0.isEmpty() && !text1.isEmpty() && !mName.isEmpty())
			{
				DataSelectionItem *ds_item = new DataSelectionItem(_data_selector->_data_selection_listview, text0.toLatin1().data(), text1.toLatin1().data());
				ds_item->mapName = mName.toLatin1().data();
				QDomElement entryItem=docElem.firstChildElement("Data_Entry");
				if(!entryItem.isNull())
				{
					if(getXMLNodeText(entryItem, "Name", s))
					{
						ds_item->databaseEntry.variableName=s.toLatin1().data();
					}
					ds_item->databaseEntry.fileName.clear();
					ds_item->databaseEntry.start.clear();
					ds_item->databaseEntry.end.clear();
					QDomElement fileItem=entryItem.firstChildElement("File_Name");
					while(!fileItem.isNull())
					{
						ds_item->databaseEntry.fileName.push_back(fileItem.firstChildElement().nodeValue().toLatin1().data());
						ds_item->databaseEntry.start.push_back(fileItem.attribute("Start", "0").toDouble());
						ds_item->databaseEntry.end.push_back(fileItem.attribute("End", "0").toDouble());
						fileItem=fileItem.nextSiblingElement("File_Name");
					}
				}
			}
				
			selectionItem=selectionItem.nextSiblingElement("Selection_Item");
		}
			
		DvEvent TIobj;
		if(getXMLNodeText(docElem, "Range_From", s))
		{
			TIobj.set_start(DvTime(s.toLatin1().data()));
		}
		if(getXMLNodeText(docElem, "Range_To", s))
		{
			TIobj.set_end(DvTime(s.toLatin1().data()));
		}
		_data_selector->_time_editor->setViewRange(TIobj);
	}
	return;
}

void  DataSelectorWindow::restore(bool replace)
{
	QString path;

    QString restore_filter = QString("Data selection savesets (*.qx_dbSelect)");
    restore_filter += ";; XML files (*.xml);; All files (*)";
	
    path=QFileDialog::getOpenFileName(this, "Open...", QString(), restore_filter);
		
    if(!path.isEmpty())
    {
		restoreXML(path, replace);
	}
  
}

  
// Class DataSelectionListView
// ---------------------------

DataSelectionListView::DataSelectionListView(DataSelector* data_selector, QWidget* parent):
	QTreeWidget(parent)
{
	setColumnCount(2);
	QStringList headers;
	headers << "Variable" << "DataSource";
	setHeaderLabels(headers);
	viewport()->setAcceptDrops(true);
	_data_selector=data_selector;
	
	menu=new QMenu(this);
	menu->addAction("Show available data range", this, SLOT(show_data_info()));
	menu->addAction("Show data coverage in selected range", this, SLOT(show_data_coverage()));
	setSelectionMode(QAbstractItemView::ExtendedSelection);

}

void DataSelectionListView::show_data_info()
{
	DvEvent time_interval = _data_selector->_time_editor->getInterval();
	
	DataSelectionItem * item = dynamic_cast<DataSelectionItem*>(currentItem());

	if(item)
	{
		string infoText;
		item->get_data_info(infoText);
		_data_selector->DSwindow->statusBar()->showMessage(infoText.c_str()); 
	}
	else
	{
		_data_selector->DSwindow->statusBar()->showMessage("No data variable item selected"); 
	}
}

void DataSelectionListView::show_data_coverage()
{
	DvEvent time_interval = _data_selector->_time_editor->getInterval();

	DataSelectionItem * item = dynamic_cast<DataSelectionItem*>(currentItem());

	if(item)
	{
		int percentage = item->get_data_coverage(time_interval);
		QString txt = QString("Data coverage for %1 in selected interval is %2\%").arg(item->text(0)).arg(percentage);
		_data_selector->DSwindow->statusBar()->showMessage(txt); 
	}
	else
	{
		_data_selector->DSwindow->statusBar()->showMessage("No data variable item selected"); 
	}
}



void DataSelectionListView::mousePressEvent(QMouseEvent* e)
{
	if(e->button()==Qt::RightButton)
	{
		menu->exec(QCursor::pos());
	}
	QTreeWidget::mousePressEvent(e);
}


bool DataSelectionListView::add_item(DataBaseItem* db_item)
{
	QList<QTreeWidgetItem *> list=findItems(db_item->text(0), Qt::MatchExactly, 0);

	if(list.isEmpty())
	{
		DataSelectionItem* i=new DataSelectionItem(this, db_item);
		i->setSelected(true);
		return true;
	}
	
	return false;
}

//  class DataSelectionItem
// ------------------------

DataSelectionItem::DataSelectionItem(DataSelectionListView* parent, DataBaseItem *db_item):
		QTreeWidgetItem(parent)
{
	setText(0, db_item->text(0));
	setText(1, db_item->parent()->text(0));
	mapName = db_item->mapName.c_str();
	databaseEntry.copyFrom(db_item->databaseEntry);

}

DataSelectionItem::DataSelectionItem(DataSelectionListView* parent, const char * name, const char* source):
		QTreeWidgetItem(parent)
{
	setText(0, name);
	setText(1, source);
}

//--------------------------

int DataSelectionItem::get_data_coverage(DvEvent  &time_interval)
{
		
	double percent=0;
	for(int i=0; i<databaseEntry.fileName.size(); i++)
	{
		DvEvent ti_file(databaseEntry.start[i], databaseEntry.end[i]);
		DvEvent TIinFile = time_interval.event_intersect(ti_file);
		
		percent += TIinFile.duration_abs(); // duration is zero if inteval is empty
	}
	
	percent /= time_interval.duration();
	percent *= 100;
	return (int) (percent + 0.5);
}

// -------------------------------

void DataSelectionItem::get_data_info(string& info_txt)
{
	
	double t1 = 1.e30;
	double t2 = -1.e30;
	for(int i=0; i<databaseEntry.start.size(); i++)
	{
		if(databaseEntry.start[i] < t1) t1 = databaseEntry.start[i];
		if(databaseEntry.end[i]  > t2) t2 = databaseEntry.end[i];
	}
	if(t1 > 9.e29 || t2 < -9e29)
	{
		info_txt = string("No data available for ");
		info_txt += text(0).toLatin1().data();
	}
	else
	{
		info_txt = text(0).toLatin1().data();
		info_txt += " : available from ";
		info_txt += DvTime(t1).getISOstring();
		info_txt += " to ";
		info_txt += DvTime(t2).getISOstring() ;
	}
}

void DataSelectionItem::get_files(const DvEvent& time_interval, vector<string>& fnames)
{
	fnames.clear();
	for(int i=0; i<databaseEntry.start.size(); i++)
	{
		DvEvent tfile(databaseEntry.start[i],  databaseEntry.end[i]);
		if(time_interval.intersects(tfile) )
		{
			// add name if not already there (multiple time pairs for file are contiguous)
			if(fnames.empty() || fnames.back() != databaseEntry.fileName[i]) fnames.push_back(string(databaseEntry.fileName[i].c_str()));
		}
	}
				
} 


// Class DataBaseListView
// ------------------------

DataBaseListView::DataBaseListView(QWidget* parent):
	QTreeWidget(parent)
{
	setColumnCount(2);
	QStringList headers;
	headers << "Data Source" << "Data Availability";
	setHeaderLabels(headers);

	menu=new QMenu(this);
	menu->addAction("Sort View Down", this, SLOT(sortDown()));
	menu->addAction("Sort View Up", this, SLOT(sortUp()));
	menu->addSeparator();
	menu->addAction("Show Files", this, SLOT(show_files()));
	menu->addSeparator();
	menu->addAction("Delete", this, SLOT(deleteItem()));
	setSelectionMode(QAbstractItemView::ExtendedSelection);


	connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), this, SLOT(doubleClick(QTreeWidgetItem*, int)));
}

// void DataBaseListView::drawRow(QPainter* painter, const QStyleOptionViewItem & option, const QModelIndex& index) const
// {
//     QTreeWidget::drawRow(painter, option, index);
//     QRect rect=visualItemRect(itemFromIndex(index));
// 
// painter->drawRect(rect);
// 
//     
// }

DataBaseItem* DataBaseListView::findItemById(const int id)
{
    DataBaseItem* result=NULL;

    int num=topLevelItemCount();

    DataBaseItem* current;
    
    for(int i=0; i<num && result==NULL; ++i)
    {
        current=((DataBaseItem*)topLevelItem(i));
        if(current->item_id==id)
        {
            result=current;
        }
        else
        {
            result=current->findChildById(id);
        }
    }

    return result;
}

QMimeData* DataBaseListView::mimeData(const QList<QTreeWidgetItem*> items) const 
{
    QMimeData * data=QTreeWidget::mimeData(items);

    QStringList dataList;
    DataBaseItem* db_item;
    foreach(QTreeWidgetItem* item, items)
    {
        db_item=dynamic_cast<DataBaseItem*>(item);
        if(db_item!=NULL)
        {
            if(db_item->db_item_type==LVI_VARIABLE)
            {
                dataList << QString("%1").arg(((DataBaseItem*)db_item)->item_id);
            }
        }
    }

    QString dataString=dataList.join("//");

    data->setData("application/qdsitems", dataString.toLatin1().data());

    return data;
}

QStringList DataBaseListView::mimeTypes() const
{
    QStringList res;
    res.append("application/qdsitems");
    return res;
}


QLinkedList<DataBaseItem*> DataBaseListView::getSelectedDBs()
{
	if(QuSlotDebug())
	{
		printf("DataBaseListView::getSelectedDBs\n");
		fflush(stdout);
	}
	
	QLinkedList<DataBaseItem*> res;
	res.clear();
	DataBaseItem* temp;
	
	if(this->topLevelItemCount() == 1)
	{
		// special case of nothing selected, but only one DB anyway
		// stops me being annoyed when I forget to select something
		temp=(DataBaseItem*)this->topLevelItem(0);
		if(temp) res.push_back(temp);
		return res;
	}

	QList<QTreeWidgetItem*> items=selectedItems();

	foreach(QTreeWidgetItem* item, items)
	{
		temp=(DataBaseItem*)item;
		if(temp->isSelected())
		{
			if(!res.contains(temp->rootItem))
				res.push_back(temp->rootItem);
		}
	}

	return res;
}

void DataBaseListView::selectionChanged (const QItemSelection & i1, const QItemSelection & i2)
{
    QTreeWidget::selectionChanged(i1, i2);
	_data_selector->update_database_selection();
}

void DataBaseListView::show_files()
{
	DataBaseItem * db_item = dynamic_cast<DataBaseItem*>(currentItem());

	if(db_item && db_item->db_item_type == LVI_VARIABLE)
	{
		db_item->show_files();
	}
	else
	{
		_data_selector->DSwindow->statusBar()->showMessage("No variable Selected");
	}
}

void DataBaseListView::sortDown()
{
	this->setSortingEnabled(true);
	this->sortItems(0, Qt::AscendingOrder);
	this->setSortingEnabled(false);
	_data_selector->managerTab->db_resave->setEnabled(true);
}

void DataBaseListView::sortUp()
{
	this->setSortingEnabled(true);
	this->sortItems(0, Qt::DescendingOrder);
	this->setSortingEnabled(false);
	_data_selector->managerTab->db_resave->setEnabled(true);
}

void DataBaseListView::deleteItem()
{

	QList<QTreeWidgetItem*> items=selectedItems();
	foreach(QTreeWidgetItem* item, items)
	{
		delete (DataBaseItem*)item;
	}
	_data_selector->set_time_interval();
	_data_selector->managerTab->db_resave->setEnabled(true);	
}


void DataBaseListView::paintEvent(QPaintEvent* event)
{

    QTreeWidget::paintEvent(event);
}

void DataBaseListView::updateWidgets()
{
    int num=topLevelItemCount();
    for(int i=0; i<num; ++i)
    {
        ((DataBaseItem*)topLevelItem(i))->recursiveUpdateWidget();
    }
}

void DataBaseListView::doubleClick(QTreeWidgetItem* item, int)
{
	_data_selector->add_item_to_selection(dynamic_cast<DataBaseItem*>(item));
}

void DataBaseListView::mousePressEvent(QMouseEvent* e)
{
	if(e->button()==Qt::RightButton)
	{
		menu->exec(QCursor::pos());
	}
	QTreeWidget::mousePressEvent(e);
}

QLinkedList<DataBaseItem*> DataBaseListView::getChildren()
{
	QLinkedList<DataBaseItem*> list;
	list.clear();

	int num=topLevelItemCount();

	DataBaseItem* temp;
	for(int i=0; i<num; ++i)
	{
		temp=dynamic_cast<DataBaseItem*>(topLevelItem(i));
		if(temp!=NULL)
		{
			list.push_back(temp);
			temp->appendChildrenTo(list);
		}
	}
	return list;
}


// DataBaseItem Class
// ------------------

DataBaseItem::DataBaseItem(LviType _type, QTreeWidget* parent, const string& name):
	QTreeWidgetItem(parent)
{
	db_item_type=_type;
	setText(0, name.c_str());
	_filelist_dialog = 0;
	
    hasWidget=false;
//     updateWidget();

	rootItem=this;
    if(db_item_type==LVI_VARIABLE)
    {
        setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
    }
    else
    {
        setFlags(Qt::ItemIsSelectable|Qt::ItemIsDropEnabled|Qt::ItemIsEnabled);
    }
    
    item_id=item_counter;
    item_counter++;
}

DataBaseItem::DataBaseItem(LviType _type, QTreeWidgetItem* parent, const string& name):
	QTreeWidgetItem(parent)
{
	db_item_type=_type;
	setText(0, name.c_str());
	_filelist_dialog = 0;

    hasWidget=false;
//     updateWidget();
	if(parent) rootItem=((DataBaseItem*)parent)->rootItem;

    if(db_item_type==LVI_VARIABLE)
    {
        setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
    }
    else
    {
        setFlags(Qt::ItemIsSelectable|Qt::ItemIsDropEnabled|Qt::ItemIsEnabled);
    }

    item_id=item_counter;
    item_counter++;
}

DataBaseItem::~DataBaseItem()
{
    if(hasWidget)
    {
        widget->item=NULL;
    }
}

void DataBaseItem::updateWidget()
{
    if(!hasWidget && db_item_type==LVI_VARIABLE)
    {
        //DEBUG
//         struct timeval t1, t2;
//         std::cout << text(0).toLatin1().data() << " " ;
//         QApplication::processEvents();
//         gettimeofday(&t1, NULL);
        //END DEBUG
        widget=new AvailabilityWidget(this, treeWidget());
        treeWidget()->setItemWidget(this, 1, widget);
        hasWidget=true;
        //DEBUG
//         QApplication::processEvents();
//         gettimeofday(&t2, NULL);
//         std::cout << ((t2.tv_sec*1000000.+t2.tv_usec)-(t1.tv_sec*1000000.+t1.tv_usec))/1000. << " ms" << std::endl;
        //END DEBUG
    }
}

void DataBaseItem::recursiveUpdateWidget()
{
    updateWidget();
    int num=childCount();
    for(int i=0; i<num; ++i)
    {
        ((DataBaseItem*)child(i))->recursiveUpdateWidget();
    }
}

void DataBaseItem::getSelectedFolders(vector<string>& list)
{
	if(db_item_type==LVI_VARIABLE)
		return;

	int num=childCount();

	if(isSelected())
	{
		for(int i=0; i<num; ++i)
		{
			((DataBaseItem*)child(i))->getSubFolders(list);
		}
		list.push_back(fullPathAndName);
	}
	else
	{
		for(int i=0; i<num; ++i)
		{
			((DataBaseItem*)child(i))->getSelectedFolders(list);
		}
	}
}

void DataBaseItem::getSubFolders(vector<string>& list)
{
	if(db_item_type==LVI_VARIABLE)
		return;

	int num=childCount();

	for(int i=0; i<num; ++i)
	{
		((DataBaseItem*)child(i))->getSubFolders(list);
	}
	list.push_back(fullPathAndName);
}

void DataBaseItem::get_files( DvEvent &time_interval, vector<string> &fnames)
{
	fnames.clear();
	for(int i=0; i<databaseEntry.start.size(); i++)
	{
		DvEvent tfile(databaseEntry.start[i],  databaseEntry.end[i]);
		if(time_interval.intersects(tfile) )
		{
			// add name if not already there (multiple time pairs for file are contiguous)
			if(fnames.empty() || fnames.back() != databaseEntry.fileName[i]) fnames.push_back(string(databaseEntry.fileName[i].c_str()));
		}
	}
}

void DataBaseItem::show_files()
{
	DvEvent tivl = ((DataBaseListView*)treeWidget())->_data_selector->_time_editor->getInterval();
	
	if(_filelist_dialog==NULL)
	{
		QPoint p = QCursor::pos();
		_filelist_dialog = new DataFileListDialog(this, treeWidget(), "fileList");
		_filelist_dialog->setMinimumSize(_filelist_dialog->sizeHint());
		QPalette palette = QPalette(qApp->palette());
		_filelist_dialog->setPalette(palette);
		_filelist_dialog->move(p);
	}
	
	vector <string> file_names;
	this->get_files(tivl, file_names);
	_filelist_dialog->_filelist_listview->clear();
	int sz=file_names.size();
	for(int i=0; i<sz; i++)
	{
		new QListWidgetItem(file_names[i].c_str(), _filelist_dialog->_filelist_listview);
	}

	_filelist_dialog->setNumFiles(sz);
  
	_filelist_dialog->show();
}

void DataBaseItem::appendChildrenTo(QLinkedList<DataBaseItem*>& list)
{
	int num=childCount();
	DataBaseItem* temp;
	for(int i=0; i<num; ++i)
	{
		temp=dynamic_cast<DataBaseItem*>(child(i));
		if(temp!=NULL)
		{
			list.push_back(temp);
			temp->appendChildrenTo(list);
		}
	}
}
bool DataBaseItem::removeEmptyFolders()
{	
	bool folderDeleted=false;
	if(db_item_type==LVI_VARIABLE)
	{
		return folderDeleted;
	}
	
	int num=childCount();

	QLinkedList<DataBaseItem*> list;
	for(int i=0; i<num; ++i)
	{
		list.push_back((DataBaseItem*)child(i));
	}

	foreach(DataBaseItem* item, list)
	{
		if(item->db_item_type==LVI_FOLDER)
		{
			// remove empty sub-folders first as item may BECOME empty
			if(item->removeEmptyFolders()) folderDeleted = true;
			if(item->childCount()==0)	
			{
				delete item;
				folderDeleted = true;
			}
		}
	}
	return folderDeleted;
}

void DataBaseItem::read_map(string fileName)
{
	mapList map_listing;
	// open and read map
	if( !loadMap(fileName, map_listing)) return;

	// collect variable names and associated file lists
	dataList databaseList;
	for(int i=0; i<map_listing.size(); i++)
	{
		for(int j=0; j<map_listing[i].varList.size(); j++)
		{
			// find if variable j is already known in unpacked list
			int nInList = databaseList.locate(map_listing[i].varList[j].variableName);
			if(nInList == -1)
			{
				// create new entry
				dataEntry newDataEntry;
				newDataEntry.db_start = 1.e30;
				newDataEntry.db_end = -1.e30;
				newDataEntry.variableName = map_listing[i].varList[j].variableName.c_str();
				nInList = databaseList.vEntry.size();
				databaseList.vEntry.push_back(newDataEntry);
			}
			for(int k=0; k<map_listing[i].varList[j].start.size(); k++)
			{
				databaseList.vEntry[nInList].start.push_back(map_listing[i].varList[j].start[k]);
				databaseList.vEntry[nInList].end.push_back(map_listing[i].varList[j].end[k]);
				databaseList.vEntry[nInList].fileName.push_back(map_listing[i].fileName.c_str()); // repeated for each time pair
				if(map_listing[i].varList[j].start[k] < databaseList.vEntry[nInList].db_start)
					databaseList.vEntry[nInList].db_start = map_listing[i].varList[j].start[k];
				if(map_listing[i].varList[j].end[k] > databaseList.vEntry[nInList].db_end)
					databaseList.vEntry[nInList].db_end = map_listing[i].varList[j].end[k];
			}
		}
	}
	
	// use inverted database to create new list view entries

	for(int i=0; i<databaseList.vEntry.size(); i++)
	{
		DataBaseItem *newDBitem = new DataBaseItem(LVI_VARIABLE, this, databaseList.vEntry[i].variableName.c_str());
		newDBitem->databaseEntry.copyFrom( databaseList.vEntry[i]);
		newDBitem->mapName = fileName.c_str();
		newDBitem->DB_NameAndPath = this->DB_NameAndPath.c_str(); // force deep copy
	}

	return;
}

DataBaseItem* DataBaseItem::findChildByName(const string& name)
{
	if(db_item_type != LVI_FOLDER) return NULL;
	
	if(fullPathAndName == name) return this;
	
	if(fullPathAndName != name.substr(0, fullPathAndName.size())) return NULL; 
	
	// incomplete, try descending
	
	int num=childCount();

	for(int i=0; i<num; ++i)
	{
		DataBaseItem* res = ((DataBaseItem*)child(i))->findChildByName(name);
		if(res) return res;
	}

	return NULL;
}

DataBaseItem* DataBaseItem::findChildById(const unsigned int id)
{
    DataBaseItem* result=NULL;
    int num=childCount();

    DataBaseItem* current;
    
    for(int i=0; i<num && result==NULL; ++i)
    {
        current=((DataBaseItem*)child(i));
        if(current->item_id==id)
        {
            result=current;
        }
        else
        {
            result=current->findChildById(id);
        }
    }

    return result;
}

void DataBaseItem::getTimeRange(double &start, double &end)
{
	if(db_item_type == LVI_FOLDER)
	{
		int num=childCount();
		DataBaseItem* temp;
		for(int i=0; i<num; ++i)
		{
			temp=(DataBaseItem*)child(i);
			temp->getTimeRange(start,end);
		}
	}
	else
	{
		if(databaseEntry.db_start < start) start = databaseEntry.db_start;
		if(databaseEntry.db_end > end) end = databaseEntry.db_end;
	}
	
	return;
}


// -----------------------------------

void DataBaseItem::getDBentries(vector <string> &db_entry_list)
{
	// recursive method to get DB entries from list view item
	if(db_item_type == LVI_FOLDER)
	{
		string entry(this->fullPathAndName.c_str());
		entry += ",";
		entry += this->mapName.c_str();
		db_entry_list.push_back(entry);

		int num=childCount();
		for(int i=0; i<num; ++i)
		{
		    ((DataBaseItem*)child(i))->getDBentries(db_entry_list);
		}
	}
	
}

// class:  AvailabilityWidget
// --------------------------

AvailabilityWidget::AvailabilityWidget(DataBaseItem* item, QWidget* parent):
	QWidget(parent)
{
	this->item=item;

    DataSelector* ds=((DataBaseListView*)item->treeWidget())->_data_selector;

	ts=0;
	te=1;

    connect(ds, SIGNAL(setInterval(double, double)), this, SLOT(setInterval(double, double)));
}

AvailabilityWidget::~AvailabilityWidget()
{
    if(item!=NULL)
    {
        item->hasWidget=false;
    }
}

void AvailabilityWidget::paintEvent(QPaintEvent*)
{
	QPainter p(this);
	p.drawPixmap(0, 0, pix);
	p.end();
}

void AvailabilityWidget::recompute()
{
    if(item==NULL)
    {
        return;
    }
 
	
    pix=QPixmap(width(), height());
	if(pix.isNull())
    {
		return;
    }
	pix.fill(qApp->palette().color(QPalette::Active, QPalette::Light));

	QPainter p(&pix);
	p.setPen(Qt::NoPen);
	int x=0;

	if(item==NULL)
	{
		p.end();
		update();
		return;
	}

	if(item->db_item_type==LVI_FOLDER)
	{
		p.end();
		update();
		return;
	}

	int da_bins=(width()-2)/DA_PIXEL_WIDTH;

	double resolution_sec = fabs(te-ts)/da_bins;

	vector<bool> avail;
	avail.clear();
	avail.assign(da_bins, false);

	for(int i=0; i<item->databaseEntry.start.size(); i++)
	{
		double t1 = item->databaseEntry.start[i];
		double t2 = item->databaseEntry.end[i];

		if(t1 > te) continue;
		if(t2 < ts) continue;

		// entry is somewhere in interval, dead reckon bin

		// find first bin, integer truncates to bin number
		int b1 = (int) ((t1 - ts ) / resolution_sec);

		// find end bin
		int b2 = (int)((t2 - ts ) / resolution_sec);

		for(int j=b1; j<= b2; j++)
		{
            if(j>=0 && j<avail.size())
			    avail[j] = true;
		}
	}

	for(int i=0;i<avail.size();i++)
	{
		if( avail[i] )
		{
			p.fillRect(x+2,2,DA_PIXEL_WIDTH-2,DA_PIXEL_HEIGHT, QColor("#118811"));
		}
		else
		{
			p.fillRect(x+2,2,DA_PIXEL_WIDTH-2,DA_PIXEL_HEIGHT,qApp->palette().color(QPalette::Active, QPalette::Midlight));
		}
		x+=DA_PIXEL_WIDTH;
	}
	p.end();
	update();
}

void AvailabilityWidget::resizeEvent(QResizeEvent*)
{
	recompute();
}


// DcmFileListDialog Class
//------------------------

DataFileListDialog::DataFileListDialog(DataBaseItem *item, QWidget *parent, const char *name) :  QDialog(parent)
{
	DB_item = item;
// 	db_item=NULL;
	timer_id=startTimer(500);
	
	QGridLayout * mainLayout = new QGridLayout( this );
	QGroupBox * listWidget=new QGroupBox(this);
	QVBoxLayout * listLayout=new QVBoxLayout;
	
	listWidget->setTitle("File List");

	// Label
	dcmfile_label=new QLabel(item->text(0).toLatin1().data());
	listLayout->addWidget(dcmfile_label);
	
	// List view

	_filelist_listview=new QListWidget(listWidget);
	listLayout->addWidget(_filelist_listview);
	connect( _filelist_listview, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(browse_metadata(QListWidgetItem *)));
	
	listWidget->setLayout(listLayout);
	mainLayout->addWidget( listWidget, 0, 0, 1, 2 );
	QPushButton *close = new QPushButton( "Close", this);
	close->setMinimumSize(close->sizeHint());
	connect( close, SIGNAL(clicked()), SLOT(reject()) );
	
	QPushButton *meta = new QPushButton( "View File Contents", this);
	meta->setMinimumSize(meta->sizeHint());
	connect( meta, SIGNAL(clicked()), this, SLOT(show_current_meta()) );
	
	mainLayout->addWidget(meta, 1, 0);
	mainLayout->addWidget(close, 1, 1);

	setLayout(mainLayout);
	setWindowTitle("File List");
	
}

void DataFileListDialog::setNumFiles(int n)
{
	dcmfile_label->setText(QString("%1: %2 File%3").arg(DB_item->text(0)).arg(n).arg((n>1?"s":"")));
}

void DataFileListDialog::show_current_meta( ) 
{
	QListWidgetItem *item = _filelist_listview->currentItem();
	browse_metadata(item);
}
			


void DataFileListDialog::browse_metadata(QListWidgetItem *item)
{
	if(item)
	{
		DaLaunchWindow(item->text(), this);
	}
}


// class dataFetchEntry
// --------------

void dataFetchEntry::copyFrom(dataFetchEntry &entry)
{
	fileList.assign(entry.fileList.size(), string(""));
	for(int i=0; i<entry.fileList.size(); i++)
	{	
		fileList[i] = string(entry.fileList[i].c_str());
	}
	
	toFetchList.assign(entry.toFetchList.size(), string(""));
	for(int i=0; i<entry.toFetchList.size(); i++)
	{	
		toFetchList[i] = string(entry.toFetchList[i].c_str());
	}
}

// class dataEntry
// --------------

void dataEntry::copyFrom(const dataEntry &entry)
{
	variableName = entry.variableName.c_str();
	fileName.assign(entry.fileName.size(), string(""));
	for(int i=0; i<entry.fileName.size(); i++)
	{
		fileName[i] = string(entry.fileName[i].c_str());
	}	
	if(entry.start.size() > 0)
	{
		start.assign(entry.start.size(), entry.start[0]);
		end.assign(entry.start.size(), entry.end[0]);
	}
	for(int i=1; i<entry.start.size(); i++)
	{
		start[i] = entry.start[i];
		end[i] = entry.end[i];
	}
	db_start = entry.db_start;
	db_end = entry.db_end;
}

// class dataList
// --------------

int dataList::locate(string name)
{
	for(int i=0; i<vEntry.size(); i++)
	{
		if(vEntry[i].variableName == name) return i;
	}
	return -1;
}


// Class DB_Manager
// ------------------------

DB_Manager::DB_Manager( QWidget *parent, const char *DB_name ):QWidget(parent)
{	

	QGridLayout* button_layout = new QGridLayout( this);
	button_layout->setSpacing(4);
	button_layout->setMargin(10);
	   
   QLabel *blank = new QLabel("          ");
   button_layout->addWidget( blank, 0,1);
   
	QLabel *contents_lbl = new QLabel( "Data Map Files", this);
	button_layout->addWidget( contents_lbl, 0, 0 );

	db_update = new QPushButton( " Update Data Maps ", this );
	db_update->setMinimumSize(db_update->sizeHint());
	db_update->setEnabled(true);
	button_layout->addWidget( db_update, 1, 0 );
	connect( db_update, SIGNAL(clicked()), this, SLOT(updateDB()) );
	db_update->setToolTip( "Modifies data maps for this DB. Use when data files added/deleted in directories");

	db_clear = new QPushButton( "Delete Data Maps", this );
	db_clear->setMinimumSize(db_clear->sizeHint());
	db_clear->setEnabled(true);
	button_layout->addWidget( db_clear, 2, 0 );
	connect( db_clear, SIGNAL(clicked()), this, SLOT(clearMaps()) );
	db_clear->setToolTip( "Delete all data maps for this DB. 'Update' will then make clean new map.");

	QLabel *space1_lbl = new QLabel( " ", this);
	button_layout->addWidget( space1_lbl, 3, 0 );

	QLabel *folder_lbl = new QLabel( "Database Contents", this);
	button_layout->addWidget( folder_lbl, 4, 0 );

	db_create = new QPushButton( "Create Database...", this );
	db_create->setMinimumSize(db_create->sizeHint());
	db_create->setEnabled(true);
	button_layout->addWidget( db_create, 5, 0 );
	connect( db_create, SIGNAL(clicked()), this, SLOT(createDB()) );
	db_create->setToolTip( "Create new database view of data directories.");

	db_add = new QPushButton( "Add Folder...", this );
	db_add->setMinimumSize(db_add->sizeHint());
	db_add->setEnabled(true);
	button_layout->addWidget( db_add, 6, 0 );
	connect( db_add, SIGNAL(clicked()), this, SLOT(addFolder()) );
	db_add->setToolTip( "Add a directory to this Database.");

	db_remove = new QPushButton( "Remove Folder", this );
	db_remove->setMinimumSize(db_remove->sizeHint());
	db_remove->setEnabled(true);
	button_layout->addWidget( db_remove, 7, 0 );
	connect( db_remove, SIGNAL(clicked()), this, SLOT(removeFolder()) );
	db_remove->setToolTip( "Permanently remove selected directories from this Database.");

	db_resave = new QPushButton( "Re-save Database View", this);
	db_resave->setMinimumSize(db_resave->sizeHint());
	db_resave->setEnabled(false);
	button_layout->addWidget( db_resave, 8, 0 );
	connect( db_resave, SIGNAL(clicked()), this, SLOT(reSaveDB()) );
	db_resave->setToolTip( "Re-save Database to reflect listview (e.g. after dragging folders).\n Empty directories are removed.");

	QLabel *space2_lbl = new QLabel( " ", this);
	button_layout->addWidget( space2_lbl, 9, 0 );

	QLabel *open_lbl = new QLabel("Remove Databases", this);
	button_layout->addWidget( open_lbl, 10, 0 );

	db_delete = new QPushButton( "Delete Database", this );
	db_delete->setMinimumSize(db_delete->sizeHint());
	db_delete->setEnabled(true);
	button_layout->addWidget( db_delete, 11, 0 );
	connect( db_delete, SIGNAL(clicked()), this, SLOT(deleteDB()) );
	db_delete->setToolTip( "Permanently delete this Database file, data maps are unaffected.");

	db_erase = new QPushButton( "Delete Database and Maps", this );
	db_erase->setMinimumSize(db_erase->sizeHint());
	db_erase->setEnabled(true);
	button_layout->addWidget( db_erase, 12, 0 );
	connect( db_erase, SIGNAL(clicked()), this, SLOT(eraseDB()) );
	db_erase->setToolTip( "Permanently delete this Database and all associated data map files.");

  	def_data_dir="";

	adjustSize(); 

}

// ------------------------

void DB_Manager::updateDB()
{
	_data_selector->DSwindow->statusBar()->clearMessage();
	
	// handle ALL selected databases
	QLinkedList<DataBaseItem*> DB_list;


	DB_list=_folder_listview->getSelectedDBs();
	
	if(DB_list.isEmpty())
	{
		_data_selector->DSwindow->statusBar()->showMessage("No Database Selected");
        return;
	}

    _data_selector->activateInterrupt("Updating Database...");

	foreach(DataBaseItem* db, DB_list)
	{
		string dbDir=db->DB_NameAndPath;

			ifstream db_file(dbDir.c_str());

			if( db_file.is_open() )
			{
				int numEntries;
				db_file >> numEntries;

				_data_selector->progressBar->setMaximum(numEntries);
				_data_selector->progressBar->setValue(1);
				qApp->processEvents();
				_data_selector->progressBar->repaint();

				for(int i=0; i<numEntries; i++)
				{
					_data_selector->progressBar->setValue(i+1);
					qApp->processEvents();
					_data_selector->progressBar->repaint();
					_data_selector->repaint();
					
					if(_data_selector->interruptFlag == true)
					{
						_data_selector->disableInterrupt();
						return;
					}
					
					string entry;
					db_file >> entry;
					// replace spaces in entries
					for(int i=0; i<entry.size(); i++) if(entry[i] == '}') entry[i] = ' ';
			
					int pos = entry.find(",");
					string nameAndPath = entry.substr(0,pos);
					entry.replace(0,pos+1,""); // entry now has map name with path
		
					_data_selector->DSwindow->update_map_file(entry.c_str(), nameAndPath.c_str(), dbDir.c_str());
				}
				db_file.close();
			}
		
		
		_data_selector->disableInterrupt();
		
		bool interrupted = _data_selector->read_database(dbDir.c_str());
		if(interrupted) 
		{
			_data_selector->DSwindow->statusBar()->showMessage("Interrupt detected while reading DB");
			return;
		}

	}
	
	return;
}

// -------------------------------
void DB_Manager::createDB()
{
	// set global data directory
	
	_data_selector->DSwindow->statusBar()->clearMessage();

	if(def_data_dir=="") 
	{
		def_data_dir = QSAS_profiles->current()->DataPath.toLatin1().data();
		if(def_data_dir=="")
		{
#ifdef WINDOWS
			def_data_dir = getenv("USERPROFILE");
#else
			def_data_dir = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
		}
	}
	QString data_dir_qs = def_data_dir.c_str();
	
	data_dir_qs = QFileDialog::getExistingDirectory(_data_selector->DSwindow, tr("Select Data Directory"), data_dir_qs, QFileDialog::ShowDirsOnly);
	if(data_dir_qs.isEmpty())
	{
		_data_selector->DSwindow->statusBar()->showMessage("No Data Directory selected...");
		return;
	}

	// remove trailing "/"
	if( data_dir_qs.endsWith("/") ) data_dir_qs.chop(1);
	data_dir = data_dir_qs.toLatin1().data();
	
	def_data_dir = data_dir.c_str();
	
	_data_selector->DSwindow->statusBar()->showMessage("Generating database entries...");
 
	vector <string> db_entry_list;
	_data_selector->activateInterrupt("Creating Database...");

	int numFolders = getNumFolders(data_dir.c_str());

	_data_selector->progressBar->setMaximum(numFolders);
	_data_selector->progressBar->setValue(1);
	_data_selector->nFolder=0;
	qApp->processEvents();
	this->repaint();
	
	_data_selector->set_def_db(); // first DB created will need the db_dir to be set
	
	bool interrupted = _data_selector->DSwindow->create_db_entries(data_dir.c_str(), _data_selector->db_dir.c_str(), db_entry_list);
	if(interrupted) 	_data_selector->DSwindow->statusBar()->showMessage("Create Database Interrupted");
	
	_data_selector->disableInterrupt();
	// even if interrupted, load what we have

	if(db_entry_list.size() == 0)
	{
		_data_selector->DSwindow->statusBar()->showMessage("Failed to Generate database entries...");
		return;
	}

	_data_selector->DSwindow->statusBar()->showMessage(string("Database Entries created for " + data_dir).c_str());

	string db_name = _data_selector->write_db(db_entry_list, "Name for Database file");
	if(db_name != "")
	{
		_data_selector->DSwindow->statusBar()->showMessage("Opening Database");
		interrupted = _data_selector->read_database(db_name.c_str());
		if(interrupted) _data_selector->DSwindow->statusBar()->showMessage("Interrupt detected while reading DB");

	}
}



// --------------------------------------

void DB_Manager::clearMaps(){

	_data_selector->DSwindow->statusBar()->clearMessage();

	// handle ALL selected databases
	QLinkedList<DataBaseItem*> DB_list=_folder_listview->getSelectedDBs();

	 if(DB_list.isEmpty())
	 {
		 _data_selector->DSwindow->statusBar()->showMessage("No Folder Selected");
	 }

	foreach(DataBaseItem* db, DB_list)
	{
		string db_nameandpath = db->DB_NameAndPath.c_str();
		
			ifstream db_file(db_nameandpath.c_str());
   
			if ( db_file.is_open() )
			{
				int numEntries;
				db_file >> numEntries;

				for(int i=0; i<numEntries; i++)
				{
					string entry;
					db_file >> entry;
					// replace spaces in entries
					for(int i=0; i<entry.size(); i++) if(entry[i] == '}') entry[i] = ' ';
			
					int pos = entry.find(",");
					entry.replace(0, pos+1, ""); // entry now has map name with path
		
					QFile mapFile(entry.c_str());
					if(mapFile.exists())
					{
						if(mapFile.remove())
						{
							cout << " map file " << entry.c_str() << " deleted" << endl;
						}
						else cout << " Do not have access permission to delete " << entry.c_str()  << endl;
					}
				}
				db_file.close();
			}
		
	}
	
	return;
}

// -----------------------------------

void DB_Manager::reSaveDB(){

	_data_selector->DSwindow->statusBar()->clearMessage();

	// handle ALL selected databases
	QLinkedList<DataBaseItem*> DB_list=_folder_listview->getSelectedDBs();
	if(DB_list.isEmpty())
	{
		_data_selector->DSwindow->statusBar()->showMessage("No Database Selected");
		return;
	}

	foreach(DataBaseItem* db, DB_list)
	{
		string db_nameandpath = db->DB_NameAndPath.c_str();	 
		vector <string> db_entry_list;
		
		int num=db->childCount();
		for(int i=0; i<num; ++i)
		{
		    ((DataBaseItem*)db->child(i))->getDBentries(db_entry_list);
		}

		writeDBfile(db_nameandpath.c_str(), db_entry_list);
		
	}
	
	db_resave->setEnabled(false);
	_data_selector->DSwindow->statusBar()->showMessage("Current view saved to database");


}
// -----------------------------------

void DB_Manager::removeFolder(){

	_data_selector->DSwindow->statusBar()->clearMessage();

	int numTopLevelItems=_folder_listview->topLevelItemCount();

	QVector<DataBaseItem*> DBlist;
	DataBaseItem* rootItem;
	for(int i=0; i<numTopLevelItems; ++i)
	{
		rootItem=(DataBaseItem*)_folder_listview->topLevelItem(i);
		vector <string> db_item_list;
		rootItem->getSelectedFolders(db_item_list);
		if(db_item_list.size() > 0)
		{
			// update databases and reopen modified DBs
			DBlist.push_back(rootItem);
	
			// get existing DB entries minus selected items
			vector <string> db_entry_list;
			string db_nameandpath = rootItem->DB_NameAndPath.c_str();
			deleteFoldersFromDB(db_item_list, db_entry_list, rootItem);
			
			// write modified entries
	
			writeDBfile(db_nameandpath.c_str(), db_entry_list);
		}
	}
	
	if(DBlist.isEmpty()) _data_selector->DSwindow->statusBar()->showMessage("No Folders Selected");

	// now delete modified BD and reopen;

	foreach(DataBaseItem* db, DBlist)
	{
		string db_nameandpath = db->DB_NameAndPath.c_str();
		delete db;
		
		// read DB back in
		_data_selector->read_database(db_nameandpath.c_str());

	}

}

// -----------------------------------------

void DB_Manager::addFolder(){

	_data_selector->DSwindow->statusBar()->clearMessage();

	// handle ALL selected databases
	QLinkedList<DataBaseItem*> DB_list=_folder_listview->getSelectedDBs();
	if(DB_list.isEmpty())
	{
		_data_selector->DSwindow->statusBar()->showMessage("No Database Selected");
		return;
	}

	// set global data directory
	static string def_data_dir="";
	if(def_data_dir=="") 
	{
		def_data_dir = QSAS_profiles->current()->DataPath.toLatin1().data();
		if(def_data_dir=="")
		{
#ifdef WINDOWS
			def_data_dir = getenv("USERPROFILE");
#else
			def_data_dir = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
		}
	}

	QString data_dir_qs = QFileDialog::getExistingDirectory(_data_selector->DSwindow,tr("Select Data Directory"),def_data_dir.c_str(), QFileDialog::ShowDirsOnly);
	if(data_dir_qs.isEmpty())
	{
		cout << "No Data Directory selected..." << endl;
		return;
	}

	// remove trailing "/"
	if( data_dir_qs.endsWith("/") ) data_dir_qs.chop(1);
	data_dir = data_dir_qs.toLatin1().data();
	def_data_dir = data_dir.c_str();
	
	vector <string> db_entry_list;

	foreach(DataBaseItem* db, DB_list)
	{
		string db_nameandpath = db->DB_NameAndPath.c_str();	 
		delete db;
				
		addFolderToDB(data_dir.c_str(), db_entry_list, db_nameandpath.c_str());

		writeDBfile(db_nameandpath.c_str(), db_entry_list);

		// close and reopen modified database
		
		_data_selector->read_database(db_nameandpath.c_str());
		
	}
		
}



void DB_Manager::deleteDB(){
	
	// handle ALL selected databases
	QLinkedList<DataBaseItem*> DB_list=_folder_listview->getSelectedDBs();

	if(DB_list.isEmpty())
	{
		_data_selector->DSwindow->statusBar()->showMessage("No Database Selected");
	}

	foreach(DataBaseItem* db, DB_list)
	{
		string db_nameandpath = db->DB_NameAndPath.c_str();

		QFile DBfile(db_nameandpath.c_str());
		if(DBfile.exists())
		{
			if(DBfile.remove())
			{
				cout << " Database " << db_nameandpath << " deleted" << endl;
			
			}
			else cout << " Do not have access permission to delete " << db_nameandpath  << endl;
		}
		delete db;
	}
	
	return;

}

// ----------------------------

void DB_Manager::eraseDB(){
  
	// go through database deleting maps
	this->clearMaps();
	
	// remove data map and clear list view
	this->deleteDB();
	
	return;
}

// ------------------------

void DB_Manager::deleteFoldersFromDB(vector <string> &folders, vector <string> &db_entry_list, DataBaseItem *db_root)
{
	string db_nameandpath = db_root->DB_NameAndPath.c_str();
	
		ifstream db_ifile(db_nameandpath.c_str());
   
		if(db_ifile.is_open() )
		{
			int numEntries;
			db_ifile >> numEntries;

			for(int i=0; i<numEntries; i++)
			{
				// read in entries from DB
				string entry;
				db_ifile >> entry;
			
				int pos = entry.find(",");
				string nameAndPath = entry.substr(0,pos);

				for(int i=0; i<nameAndPath.size(); i++) if(nameAndPath[i] == '}') nameAndPath[i] = ' ';
			
				bool found = false;
				// skip those in the list to drop
				for(int j=0; j<folders.size(); j++)
				{
					if(folders[j] == nameAndPath)
					{
						found = true;
						break;
					}
				}
				if( !found) db_entry_list.push_back(entry);
			}
			db_ifile.close();
		}
	
}

// ----------------------------------------------

void DB_Manager::addFolderToDB(const char *folder, vector <string> &db_entry_list, const char *DBnameandpath)
{
	string db_path =  DBnameandpath;
	int pos = db_path.rfind("/");
	db_path = db_path.substr(0, pos);
	

	// get existing DB entries
	
		ifstream db_ifile(DBnameandpath);
   
		if(db_ifile.is_open())
		{		
			int numEntries;
			db_ifile >> numEntries;

			for(int i=0; i<numEntries; i++)
			{
				string entry;
				db_ifile >> entry;
				db_entry_list.push_back(entry);
			}
			db_ifile.close();
		}
	
	
	// add folder
	int numFolders = getNumFolders(folder);
	_data_selector->progressBar->setMaximum(numFolders);
	_data_selector->progressBar->setValue(1);
	_data_selector->progressBarLbl->setText("Adding Folder...");
	_data_selector->nFolder=0;
	
	bool interrupted = _data_selector->DSwindow->create_db_entries(folder, db_path.c_str(), db_entry_list);
	_data_selector->progressBarLbl->setText(" ");

}

// -----------------------------------

string DataSelectorWindow::update_map_file(const char *existingMapName, const char *target_dir, const char *db_dir)
{
	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::update_map_file '%s' in %s for DB  %s\n",  existingMapName, target_dir, db_dir);
		fflush(stdout);
	}
	string mapName = existingMapName;
	mapList map_listing;
	vector <string> fileList;
	dirFileList dirList;
	
	// create list of files in dir (excludes sub-directories)
	dirList.getFilesInDir(target_dir);

	int mapExists = 1;
	if(mapName == "")
	{
		// if no existing binary map file to re-use, create one
		mapName = target_dir;
		mapName += "/.qs.map";
		// try to open or create map
		mapExists = newFile(mapName.c_str());
		if(mapExists == -1) 
		{
			// cannot create map in target dir, so look with databse file
			mapName = db_dir;
			mapName += "/";
			if(mapName != "/") // catch db_dir not set (avoid root dir)
			{
				mapName += altMapName(target_dir);
				mapExists = newFile(mapName.c_str());
			}
		}
		if(mapExists == -1) 
		{
			statusBar()->showMessage("User does not have write permission");
			return "";
		}
	}
	else mapExists = newFile(mapName.c_str()); // find out if this file still exists


	// read old map if it existed
	if(mapExists == 1)
	{
		// first delete entries no longer needed

		if( loadMap(mapName.c_str(), map_listing) )
		{
			vector <mapEntry>::iterator it = map_listing.begin();
			while (it != map_listing.end())
			{
				// remove from map if file missing

				if(  !dirList.nameFound(it->fileName)) it = map_listing.erase(it);
				else it++;
			}
		}
		else
		{
			if(QuSlotDebug())
			{
				printf("cannot read map file  %s \n", mapName.c_str() );
				fflush(stdout);
			}
		}
	
	}
	
	QDateTime tMap = getFileModDate(mapName.c_str());
	// create list of files to add to map

	for(int i=0; i<dirList.fileName.size(); i++)
	{
		// add to map if new
		if(!fileExists(map_listing, dirList.fileName[i].c_str()) ) fileList.push_back(string(dirList.fileName[i].c_str()));
				
		// add to map if changed
		else if(dirList.tFile[i] > tMap)  fileList.push_back(string(dirList.fileName[i].c_str()));
	}
	
	// we now have a dir list plus the old map list if it exists
	// Write new map

	if (write_map_contents(fileList, map_listing, mapName.c_str()) )
	{
		statusBar()->showMessage(mapName.c_str());
		qApp->processEvents();
		this->repaint();
		return mapName;
	}

	return "";
}

// -------------------------------

bool DataSelectorWindow::write_map_contents(vector <string> &file_list, mapList &map_listing, const char *mapName)
{	
	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::write_map_contents \n");
		fflush(stdout);
	}

	// add files to map listing
	for(int i=0; i<file_list.size(); i++)
	{
		mapEntry entry;
		entry.fileName = file_list.at(i).c_str();

		getFileMapEntries(entry.fileName, entry.varList);
	
		if( entry.varList.size() > 0) map_listing.push_back(entry);
		
	}
	file_list.clear();
	
		// try to give group write permission
		chmod(mapName, DMAP_MODE);
		
		ofstream b_os(mapName);

		// write to file
		if(b_os.is_open()) b_os << map_listing;
		
		map_listing.clear();
	

	return true;
}

// --------------------------------

bool DataSelectorWindow::create_db_entries(const char *target_dir, const char *db_dir, vector <string> &db_entry_list)
{
	if(QuSlotDebug())
	{
		printf("create_db_entries  %s \n", target_dir);
		fflush(stdout);
	}
	
	_data_selector->nFolder++;
	_data_selector->progressBar->setValue(_data_selector->nFolder);
	
	// check for interrupt
	if(_data_selector->isInterrupted(" ")) return true;
	
	// open directory
	QDir dir(target_dir);

	// map this directory
	string mapName = write_map_file(target_dir, db_dir);
	if ( mapName != "" )
	{
		string dir_mapped = target_dir;
		dir_mapped += ",";
		dir_mapped += mapName;
		for(int i=0; i<dir_mapped.size(); i++) if(dir_mapped[i] == ' ') dir_mapped[i] = '}';

		db_entry_list.push_back(dir_mapped);
	}
		
	// recursively map sub-directories
	qApp->processEvents();
	dir.setFilter(QDir::NoDotAndDotDot | QDir::Dirs);
	QFileInfoList dir_list = dir.entryInfoList();
	for(int i=0; i<dir_list.size(); i++) 
	{
		bool interrupted = create_db_entries(dir_list.at(i).absoluteFilePath().toLatin1().data(), db_dir, db_entry_list);
		if(interrupted) return true;
	}
	return false;
}

// -----------------------------------

bool DataSelectorWindow::file_exists(const char *pathName, const char *fileName)
{
	QDir dir(pathName);
	return dir.exists(fileName);
}

// -----------------------------------

string DataSelectorWindow::write_map_file(const char *target_dir, const char *db_dir)
{
	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::write_map_file  '%s' for DB '%s'\n", target_dir, db_dir);
		fflush(stdout);
	}

	string mapName = "";
	string data_map_name = altMapName(target_dir);

	// map this directory, create list of files to map

	// if no existing binary map file to re-use, create one
	if( file_exists(target_dir, ".qs.map") )
	{
		mapName = target_dir;
		mapName += "/.qs.map";
	}
	else if(file_exists(db_dir, data_map_name.c_str()))
	{
		// no map in target dir, so look with databse file
		mapName = db_dir;
		mapName += "/";
		mapName += data_map_name.c_str();
	}
	else mapName = "";
	return update_map_file(mapName.c_str(), target_dir, db_dir);
	
}

string DataSelectorWindow::altMapName(const char *target_dir)
{
	// construct name to be used if no data dir write permission
	string data_map_name;
	data_map_name = ".";
	data_map_name += target_dir;
	data_map_name += "_qs.map";
	int pos = data_map_name.find("/");
	while (pos != -1)  
	{
		data_map_name.replace(pos, 1, "_");
		pos = data_map_name.find("/");
	}
	pos = data_map_name.find(" ");
	while (pos != -1)  
	{
		data_map_name.replace(pos, 1, "_");
		pos = data_map_name.find(" ");
	}
	return data_map_name;
}

// -----------------------------------

void DataSelectorWindow::getFileMapEntries(string &fileName, variableList &varList)
{

	if(QuSlotDebug())
	{
		printf("DataSelectorWindow::getFileMapEntries for %s \n", fileName.c_str());
		fflush(stdout);
	}
	
	QieData *QieDataCont = new QieData();		// qie Contents Structure 

	QString statusLine = "reading... ";
	statusLine += fileName.c_str();
	statusBar()->showMessage(statusLine);
	qApp->processEvents();
	this->statusBar()->repaint();
	this->repaint();

	int nRecs = QieDataCont->queryFile(fileName.c_str());
	if( nRecs > 0 ){

		// file pointer is at start of data, we need number of records and to read header

	//	nRecs = QieDataCont->readDataRecords();	// read named file and get variables names
		// Get intervals for each variable (Themis has multiple time lines)
		for(int i=0; i<QieDataCont->QieVars.size(); i++)
		{
            QieVariable *var = QieDataCont->QieVars[i];
			
			// skip support data (but keep Status vars)
			DvString typeXref;DvNode *Xnode = QieDataCont->QieVars[i]->Vxref.find("PARAMETER_TYPE");
				if(!Xnode)
					Xnode = QieDataCont->QieVars[i]->Vxref.find("VAR_TYPE");
				if (Xnode) 
					typeXref = Xnode->obj()->asStr();

				if ( !typeXref.empty() ){
					if ( typeXref == "Support_Data" ) continue;
					if ( typeXref == "support_data" ) continue;
					if ( typeXref == "metadata" ) continue;
				}

			// add data variable to map 
			variableEntry vEntry;
			vEntry.variableName = var->name.c_str();
			DvObject_var tt = QieDataCont->readTT(i, nRecs);
 
			if(tt.is_nil() ) continue;
			
            if(tt->seqSize() < 2 || !tt->is_time()) continue;
			
			// get intervals of data
			double gapTol = fabs(tt->time(tt->seqSize()-1) - tt->time(0));
			if(tt->seqSize() > 50) gapTol = gapTol/20.;
			else if(tt->seqSize() > 5) gapTol = gapTol/2.;
			
			vEntry.start.push_back(tt->time(0).getEpoch2000());
			for(int k=1; k<tt->seqSize(); k++)
			{
				if(fabs(tt->time(k) - tt->time(k-1)) > gapTol)
				{
					// new interval
					vEntry.end.push_back(tt->time(k-1).getEpoch2000());
					vEntry.start.push_back(tt->time(k).getEpoch2000());
				}
			}
			// start and end are double seconds from epoch 2000
			vEntry.end.push_back(tt->time(tt->seqSize()-1).getEpoch2000());
			
			varList.push_back(vEntry);
          
		}
	}

	delete QieDataCont;
	return;
}  
// ------------------------------------------


bool loadMap(string mapName, mapList &map_listing)
{
	ifstream b_is(mapName.c_str());
		// read map entries

	if(b_is.is_open()) b_is >> map_listing;
	else return false;
	
	return true;
}

// ----------------------------

bool isFileWritable(const char *fileName)
{
	bool writable = false;
	QFile mf(fileName);
	if(!mf.exists()) mf.open(QIODevice::ReadWrite);
	if(mf.isWritable()) writable = true;
	mf.close();
	return writable;
}

// ------------------------

void writeDBfile(const char *dbFileName, vector <string> &db_entry_list)
{
	if(QuSlotDebug())
	{
		printf("Write DB file %s \n", dbFileName);
		fflush(stdout);
	}

		ofstream db_ofile(dbFileName);
   
		if ( db_ofile.is_open() )
		{
			db_ofile << db_entry_list.size();
			for(int i=0; i<db_entry_list.size(); i++)
			{
				db_ofile << db_entry_list[i];
				db_ofile << "\n";
			}
			db_ofile.close();
		}
		else cout << " failed to open DB for write " << dbFileName << endl;
	
}

// ------------------------

bool fileExists(mapList &map_listing, const char* fileName)
{
	for(int i=0; i<map_listing.size(); i++)
	{
		if(map_listing[i].fileName == string(fileName)) return true;
	}
	return false;
}

QDateTime getFileModDate(const char * fileName)
{	
	QFileInfo fileInfo(fileName);
	
	QDateTime tFile = fileInfo.lastModified();
	return tFile;
	
}

// ------------------------

int newFile(const char *fileName)
{	
	// if file exists, don't create
	QFileInfo fileInfo(fileName);
	if(fileInfo.exists())
	{
		if(fileInfo.size() == 0) return 0; // it exists but is empty
		else return 1; // it exists, and has content
	}
	// create new file as not present
	QFile qmf(fileName);
	if ( !qmf.open(QIODevice::ReadWrite)) return -1; // cannot open file
	qmf.close();
	
	return 0; // file created OK and is empty
}

// ------------------------

bool dirFileList::getFilesInDir( const char *target_dir)
{
	QDir dir(target_dir);
	QStringList filters;
	filters << "*.cef" << "*.cdf" << "*.qft"<< "*.qfd"<< "*.TAB";
	dir.setNameFilters(filters);
	dir.setFilter(QDir::Files);

	QFileInfoList dir_list = dir.entryInfoList();
	if(dir_list.size() <= 0) return false ;

	QList<QFileInfo>::iterator dir_itr=dir_list.begin();
	while(dir_itr != dir_list.end())
	{
		if(dir_itr->isDir()) dir_itr++;
		else
		{
			fileName.push_back(string(dir_itr->absoluteFilePath().toLatin1().data()));
			tFile.push_back(dir_itr->lastModified());
			dir_itr++;
		}
	}
	return true;
}

// ------------------------

bool dirFileList::nameFound(string &fileN)
{
	for(int i=0; i<fileName.size(); i++)
	{
		if(fileN == fileName[i]) return true;
	}
	return false;
}

// ------------------------

int getNumFolders(const char *data_dir)
{
	int nFolders = 1; // this folder
	QDir dir(data_dir);
	dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);

	QFileInfoList dir_list = dir.entryInfoList();
	if(dir_list.size() <= 0) return nFolders ;
	nFolders += dir_list.size();
	
	QList<QFileInfo>::iterator dir_itr=dir_list.begin();
	while(dir_itr != dir_list.end())
	{
		if(dir_itr->isDir()) 
		{
			nFolders += getNumFolders(dir_itr->absoluteFilePath().toLatin1().data());
			dir_itr++;
		}
	}
	return nFolders;
}


