
 
#include <qstring.h>
#include <qfiledialog.h> 

#include "qsasMenus.h"
#include "qplug_if.h"
#include "qie_VOtable.h"
#include "qpsd.h"

 extern qwuiWindow* qsasWindow;
#include "qcalcWindow.h"
extern CalcWindow *QCALC_Proto_widget;


void AddQwuiMenus(){
   
	// these are here because cygwin/windows does not allow forward declaration
   
	static mainMenu * file = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu(file );
	file->insertFileItems();
	file->setTitle( " File ");
  
	static mainMenu * edit = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu(edit );
	edit->insertEditItems();
	edit->setTitle(" Edit ");
  

	// Add to test the Object Calculator
	static mainMenu *eCalculate = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu( eCalculate );
	eCalculate->insertCalcItems();
	eCalculate->setTitle(" Analysis ");

	
	static mainMenu *plugins = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu(plugins );
	plugins->insertPluginMenu();
	plugins->setTitle(" Plug-Ins ");
     
	static mainMenu * plot = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu(plot );
	plot->insertPlotItems();
	plot->setTitle(" Plot    ");

	static mainMenu * profiles = new mainMenu(qsasWindow);
	qsasWindow->menuBar()->addMenu(profiles );
	profiles->insertProfilesItems();
	profiles->setTitle(" Profiles ");
  
	qsasWindow->menuBar()->addSeparator();   
   
	static mainMenu * help = new mainMenu( qsasWindow );
	qsasWindow->menuBar()->addMenu(help );
	help->insertHelpItems();
	help->setTitle(" Help ");
   
	// populate time and maths menus
	qsasWindow->mathOps->addFolderToMenu("Arithmetic");
   

	qsasWindow->timeOps->addFolderToMenu("TimeOperations");
    qsasWindow->filterOps->addFolderToMenu("Filtering");
    qsasWindow->vectorOps->addFolderToMenu("Vectors");
    qsasWindow->statOps->addFolderToMenu("Statistics");
	qsasWindow->unitOps->addFolderToMenu("UnitOperations");
	qsasWindow->eventTableOps->addFolderToMenu("EventTables");
   
	// populate plugin menu
	plugins->refreshPlugins(true);
   
 }

// class mainMenu
// -----------------------

mainMenu::mainMenu(QWidget *parent, const char *name)
    :  QMenu(parent)
{        
   // initialise NULL pointer    
    objTypeSelector = NULL;
    setTearOffEnabled(true);
	def_dir = QString("");
	
}

// -----------------------

void mainMenu::menuOp(QAction *act)
{ 

  QString pluginName; 
  
  if( act->text() == QString("Browse") ){
  	QString user_lib = QSAS_profiles->current()->UserLibPath;
	pluginName = QFileDialog::getOpenFileName(this, "Select qtpl template", user_lib, "*.qtpl");
    QtplOpenByName(pluginName, qsasWindow);
    return;
  }
  else if( act->text() == QString("Refresh Menu") ){
      this->refreshPlugins(false);
      return;
  }
  else if( act->text() == QString("Run Batch File") ){
      launchBatchProcess();
      return;
  }
  
}


void mainMenu::refreshPlugins(bool first)
{
    if(first){
        plotWin_id = NULL; // initialise before plugins get called
        plotWidgets = NULL;
    }
    
  for(int i=0; i<qsasWindow->pluginFolders.size(); i++){
    qsasWindow->pluginFolders[i]->clear();
    if(first) {
	
      this->addMenu(qsasWindow->pluginFolders[i]);
    }
    qsasWindow->pluginFolders[i]->addFolderToMenu(qsasWindow->pluginFolders[i]->title().toLatin1().data(), first);
  }

  if(first){
    this->addSeparator();
    this->addAction("Browse" );
    this->addAction("Refresh Menu" );
    this->addSeparator();
    this->addAction("Run Batch File" );

  }
  QuMsgBar("Plugin Menus rebuilt", false);
}

void mainMenu::insertFileItems(){

    QMenu *setSaveVersion = new QMenu( "Set QSAS Saveset Version", this);
    setSaveVersion->setToolTip("Allows saving in earlier QSAS versions");
    qsasWindow->save310 = setSaveVersion->addAction("Savesets readable by QSAS 3.1.0", qsasWindow, SLOT( setSaveAs310() ));
    qsasWindow->save315 = setSaveVersion->addAction("Latest Saveset version", qsasWindow, SLOT( setSaveAs315() ));
    qsasWindow->save310->setCheckable(true);
    qsasWindow->save315->setCheckable(true);
    qsasWindow->save315->setChecked(true);
    
	this->addAction(QIcon(qsasWindow->qsas_home+"/icons/dbOpen.png"), "Data Selector", this, SLOT( openDS() ));
	this->addAction(QIcon(qsasWindow->qsas_home+"/icons/fileopen.png"), "&Open Data File", this, SLOT( openCAA() ), Qt::CTRL+Qt::Key_O );
//	this->addAction(style()->standardIcon(QStyle::SP_DialogOpenButton, 0, this), "&Open Data File", this, SLOT( openCAA() ), Qt::CTRL+Qt::Key_O );
	this->addAction(style()->standardIcon(QStyle::SP_DialogSaveButton, 0, this), "&Write Data File", this, SLOT( openExport() ), Qt::CTRL+Qt::Key_W );
	this->addSeparator();
	this->addAction("Load Event Table", this, SLOT(openEvtTable()));
	this->addAction("Export Event Table", this, SLOT(exportET()));
	this->addSeparator();
	this->addAction(QIcon(qsasWindow->qsas_home+"/icons/filesaveas.png"), "Save Working List", qsasWindow, SLOT( saveWL() ) );
	this->addAction("Save Working List As...", qsasWindow, SLOT( saveWLAs() ) );
	this->addAction(QIcon(qsasWindow->qsas_home+"/icons/reload.png"), "Restore Working List...", qsasWindow, SLOT( restoreWL() ) );
	this->addAction("Include in WL...", qsasWindow, SLOT( includeInWL() ) );
	this->addSeparator();
	this->addAction("&Save Session", qsasWindow, SLOT( saveSession() ), Qt::CTRL+Qt::Key_S );
	this->addAction("Save Session As...", qsasWindow, SLOT( saveSessionAs() ), Qt::SHIFT+Qt::CTRL+Qt::Key_S );
	this->addAction("&Restore Session", qsasWindow, SLOT( restoreSession() ), Qt::CTRL+Qt::Key_R );
	this->addSeparator();
    this->addMenu(setSaveVersion );
	this->addSeparator();
	this->addAction(style()->standardIcon(QStyle::SP_DialogCloseButton, 0, this), "&Quit", qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );
	connect(qApp, SIGNAL(lastWindowClosed() ), qApp, SLOT(quit()));
	connect(qsasWindow, SIGNAL(openQDA_Window(const char *  )), this, SLOT(openDataFile(const char * )));
	
	connect(qsasWindow->wlview, SIGNAL(exportET()), this, SLOT(exportET())); // see comments on mainMenu::exportET()
	// For Batch process
    connect(qsasWindow, SIGNAL(readDataRequest(DvString&, DvString&, DvEvent&)), this, SLOT(getVariable(DvString&, DvString&, DvEvent&))); 
    
	this->addSeparator();
	toolBar = this->addAction("Show Tool Bar", this, SLOT( toggleToolbar() ), Qt::CTRL+Qt::Key_T );
    if(QSAS_profiles->current()->ShowToolbar) this->toggleToolbar();


}

void mainMenu::insertEditItems(){

	this->addAction("new &Object", qsasWindow->wlview, SLOT( newObject() ) );
    this->addAction("new &Folder", qsasWindow->wlview, SLOT( newFolder() ), Qt::CTRL+Qt::Key_F );
    this->addSeparator();

    this->addAction("&Duplicate Object", qsasWindow->wlview, SLOT( duplicateObj() ), Qt::CTRL+Qt::Key_D );
    this->addAction("Subsample/Extract", qsasWindow->wlview, SLOT( sliceObj() ), Qt::CTRL+Qt::ALT+Qt::Key_G );
    this->addAction("&Edit Object",qsasWindow->wlview, SLOT(editObject()), Qt::CTRL+Qt::Key_E );
    this->addAction("re-&Name Object", qsasWindow->wlview, SLOT( rename() ), Qt::CTRL+Qt::Key_N );
    this->addAction("select &All Data", qsasWindow->wlview, SLOT(selectAllData()), Qt::CTRL+Qt::Key_A );
    this->addSeparator();

   QAction *delAct = this->addAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete Object", qsasWindow->wlview, SLOT( deleteObject() ), Qt::Key_Delete);
   QList<QKeySequence> keyList;
   keyList.append(Qt::Key_Delete);
   keyList.append(Qt::Key_Backspace);
   keyList.append(Qt::CTRL+Qt::Key_X);
   delAct->setShortcuts(keyList);
  
   this->addAction("Remove object's time tags", qsasWindow->wlview, SLOT( removeTimeTags() ));
   this->addSeparator();
   this->addAction(QIcon(qsasWindow->qsas_home+"/icons/clearlist.png"), "Clear Working List", qsasWindow->wlview, SLOT( deleteAllDataItems() ));
   this->addAction("Clear Time Interval List", qsasWindow->wlview, SLOT( deleteAllTimeIntervalItems() ));
   this->addSeparator();
   this->addAction("Move Up", qsasWindow->wlview, SLOT(moveUp()), Qt::CTRL+Qt::Key_Up );
   this->addAction("Move Down", qsasWindow->wlview, SLOT(moveDown()), Qt::CTRL+Qt::Key_Down );
   this->addSeparator();
   this->addAction("Sort selection", qsasWindow->wlview, SLOT(sortWLSelected()) );
   this->addAction("Sort Working List", qsasWindow->wlview, SLOT(sortWL()) );
   this->addAction("Set Sort Prefs...", qsasWindow->wlview, SLOT(sortWLPrefs()) );

     
    connect(qsasWindow->openAction, SIGNAL( triggered()), this, SLOT(openCAA()));
    connect(qsasWindow->selectorAction, SIGNAL( triggered()), this, SLOT(openDS()));
//  connect(qsasWindow->timeAction, SIGNAL( triggered()),  qsasWindow->wlview, SLOT(createSEHTInterval()));


  }
  
void mainMenu::insertHelpItems(){

	this->addAction(style()->standardIcon(QStyle::SP_MessageBoxInformation, 0, this), "QSAS &Manual", qsasWindow, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	this->addSeparator();
	this->addAction("...Getting Started", qsasWindow, SLOT( launchManSampleW() ));
	this->addAction("...Main Window", qsasWindow, SLOT( launchManMainW() ));
	this->addAction("...Data Handling", qsasWindow, SLOT( launchManDataH() ));
	this->addAction("...Save and Restore", qsasWindow, SLOT( launchManSaveRestore() ));
	this->addSeparator();
	this->addAction("Module Version", qsasWindow, SLOT( launchVersion() ));
	this->addAction("Contact QSAS Team", qsasWindow, SLOT( launchContact() ));
   
}

void mainMenu::insertCalcItems(){


	this->addAction(QIcon(qsasWindow->qsas_home+"/icons/xcalc.png"), "Calculator", this, SLOT( launchCalculator()) );
	this->addAction("Particle Tools (PSD)", this, SLOT( launchPSD() ));



	// End Calculator
   
   // old Arithmetic plugins
   qsasWindow->mathOps = new qtplMenu( qsasWindow, "Simple Maths" );
   this->addMenu(qsasWindow->mathOps );
   connect(qsasWindow->mathOps, SIGNAL(triggered(QAction *)), qsasWindow->mathOps,  SLOT(qtplOpen(QAction *)) );

   // Unit conversion plugins
   qsasWindow->unitOps = new qtplMenu( qsasWindow, "Change Units" );
   this->addMenu(qsasWindow->unitOps );
   connect(qsasWindow->unitOps, SIGNAL(triggered(QAction *)), qsasWindow->unitOps,  SLOT(qtplOpen(QAction *)) );

   qsasWindow->timeOps = new qtplMenu( qsasWindow, "Time Ops..." );
   qsasWindow->timeOps->addAction(QIcon(qsasWindow->qsas_home+"/icons/join.png"), "&Join", this, SLOT( launchJoin() ), Qt::CTRL+Qt::Key_J );
    qsasWindow->timeOps->addAction("Timetags to seconds offset", this, SLOT( launchTTOffset() ) );
    qsasWindow->timeOps->addAction("Seconds offset to Timetags", this, SLOT( launchSStoTT() ) );
   qsasWindow->timeOps->addAction("Delay between Timetags, tt2 - tt1", this, SLOT( launchTT1TT2toSec() ) );
   qsasWindow->timeOps->addAction("Time Interval to seconds", this, SLOT( launchTItoSec() ) );
   qsasWindow->timeOps->addAction("Time t2 - t1 in seconds", this, SLOT( launchT1T2toSec() ) );
   
   this->addMenu(qsasWindow->timeOps);
   connect(qsasWindow->timeOps, SIGNAL(triggered(QAction *)), qsasWindow->timeOps,  SLOT(qtplOpen(QAction *)) );

   qsasWindow->filterOps = new qtplMenu( qsasWindow, "Filtering" );
   this->addMenu(qsasWindow->filterOps );
   connect(qsasWindow->filterOps, SIGNAL(triggered(QAction *)), qsasWindow->filterOps,  SLOT(qtplOpen(QAction *)) );

    qsasWindow->vectorOps = new qtplMenu( qsasWindow, "Vector Methods" );
    qsasWindow->vectorOps->addAction("Scalar Product", this, SLOT( dotProd() ) );
    qsasWindow->vectorOps->addAction("Vector Product", this, SLOT( vecProd() ) );
    qsasWindow->vectorOps->addAction("Normalize Vector", this, SLOT( normalizeVec() ) );
    qsasWindow->vectorOps->addAction("Outer Product", this, SLOT( outerProd() ) );
    qsasWindow->vectorOps->addAction("Array to Vector", this, SLOT(array2vec()) );
    this->addMenu(qsasWindow->vectorOps );
    connect(qsasWindow->vectorOps, SIGNAL(triggered(QAction *)), qsasWindow->vectorOps,  SLOT(qtplOpen(QAction *)) );
   
    
    qsasWindow->arrayOps = new qtplMenu( qsasWindow, "Array Methods" );
    qsasWindow->arrayOps->addAction("Matrix Inverse", this, SLOT( arrayMatInv() ) );
    qsasWindow->arrayOps->addAction("Trace", this, SLOT( arrayTrace() ) );
    qsasWindow->arrayOps->addAction("Inner Product", this, SLOT( innerProd() ) );
    qsasWindow->arrayOps->addAction("Outer Product", this, SLOT( outerProd() ) );
    qsasWindow->arrayOps->addAction("Transpose", this, SLOT( arrayTranspose() ) );
    qsasWindow->arrayOps->addAction("Determinant", this, SLOT( arrayDeterminant() ) );
    qsasWindow->arrayOps->addAction("Adjoint", this, SLOT( arrayAdjoint() ) );
    qsasWindow->arrayOps->addAction("Element(i)", this, SLOT( arrayElement() ) );
    qsasWindow->arrayOps->addAction("Element(i,j,...)", this, SLOT( arrayElement_ijk() ) );
    qsasWindow->arrayOps->addAction("Remove(i)", this, SLOT( removeElement() ) );
    qsasWindow->arrayOps->addAction("Sub-Matrix", this, SLOT( subMatrix() ) );
    qsasWindow->arrayOps->addAction("A(i,j) * B(i,j) -> C(i,j)", this, SLOT( piecewiseMult() ) );
    qsasWindow->arrayOps->addAction("A(i,j) / B(i,j) -> C(i,j)", this, SLOT( piecewiseDivide() ) );
    this->addMenu(qsasWindow->arrayOps );
//    connect(qsasWindow->arrayOps, SIGNAL(triggered(QAction *)), qsasWindow->arrayOps,  SLOT(qtplOpen(QAction *)) );
    
    qsasWindow->statOps = new qtplMenu( qsasWindow, "Statistics" );
    qsasWindow->statOps->addAction("Linear Regression", this, SLOT( regression() ) );
    this->addMenu(qsasWindow->statOps );
    connect(qsasWindow->statOps, SIGNAL(triggered(QAction *)), qsasWindow->statOps,  SLOT(qtplOpen(QAction *)) );
    
   qsasWindow->eventTableOps = new qtplMenu( qsasWindow, "Event Tables" );
   this->addMenu(qsasWindow->eventTableOps );
   connect(qsasWindow->eventTableOps, SIGNAL(triggered(QAction *)), qsasWindow->eventTableOps,  SLOT(qtplOpen(QAction *)) );
   
   QMenu* angle_menu = this->addMenu("Angle Methods");
   angle_menu->addAction("Trig and Hyperbolic Trig Ops", this, SLOT(openTrig()));
   angle_menu->addAction("Convert Angle Depend (-180,180) to (0,360) deg", this, SLOT(angleDepTo360()));
   angle_menu->addAction("Convert Angle Depend (0,360) to (-180,180) deg", this, SLOT(angleDepTo180()));
   angle_menu->addAction("Convert Angle Depend (-90,90) to (0,180) deg", this, SLOT(angleDepTo0180()));
   angle_menu->addAction("Convert Angle Depend (0,180) to (-90,90) deg", this, SLOT(angleDepTo90()));
   angle_menu->addAction("Re-centre Angle Depend", this, SLOT(angleDepReCentre()));
   angle_menu->addAction("ATan2 Deg", this, SLOT(aTan2Deg()) );
   angle_menu->addAction("ATan2 Rad", this, SLOT(aTan2Rad()) );
      
   this->addAction("Subsample/Extract", qsasWindow->wlview, SLOT( sliceObj() ) );

   this->addAction("Derivative", this, SLOT( derivative() ) );

   this->addAction("Integral", this, SLOT( integral() ) );
   
    this->addAction("Merge TS Objects", this, SLOT(mergeTS()) );
  
    this->addAction("Create Regular", this, SLOT( createRegObj() ) );

    connect(qsasWindow->calcAction, SIGNAL( triggered()), this, SLOT(launchCalculator()));
    connect(qsasWindow->joinAction, SIGNAL( triggered()), this, SLOT(launchJoin()));
  
}

void mainMenu::insertPlotItems(){

    this->addAction(QIcon(qsasWindow->qsas_home+"/icons/plot2.png"), "&Plot Layout", this, SLOT( launchPlot() ), Qt::CTRL+Qt::Key_P );
    this->addAction("2D Polar View", this, SLOT( launchPolarPlot() ) );
    this->addAction("Surface View", this, SLOT( launchSurfacePlot() ) );
    this->addAction("3D View", this, SLOT( launch3DPlot() ) );
    
    connect(qsasWindow->plotAction, SIGNAL( triggered()), this, SLOT(launchPlot()));

}

void mainMenu::insertPluginMenu(){
   static qtplMenu *geoMenu = new qtplMenu(qsasWindow, "Geophysics");
   qsasWindow->pluginFolders.push_back(geoMenu);
   
   static qtplMenu *analysisMenu = new qtplMenu(qsasWindow, "Analysis"); 
   qsasWindow->pluginFolders.push_back(analysisMenu);
   
   connect(this, SIGNAL(triggered(QAction *)),this,  SLOT(menuOp(QAction *)) );
   connect(geoMenu, SIGNAL(triggered(QAction *)),geoMenu,  SLOT(qtplOpen(QAction *)) );
   connect(analysisMenu, SIGNAL(triggered(QAction *)),analysisMenu,  SLOT(qtplOpen(QAction *)) );
}

void mainMenu::insertProfilesItems()
{
	this->addAction("Edit profiles...", this, SLOT( launchEditProfile() ));
	this->addSeparator();

	QString currentName=QSAS_profiles->current()->getName();
	std::vector<QString> names;
	QSAS_profiles->names(names);
	for(int i=0; i<names.size(); ++i)
	{
		QAction* thisAction=this->addAction(names[i]);			
		connect(this, SIGNAL(triggered(QAction*)), this, SLOT(launchSelectProfile(QAction*)) );
		thisAction->setCheckable(true);

		if(names[i]==currentName)
		{
			checkedProfile = thisAction;
			checkedProfile->setChecked(true);
		}
		else
		{
			thisAction->setChecked(false);
		}
	}
}

// This belongs in qwui, BUT as the Windows build cannot make dlls with undefined symbols, and as QiSaveVOTable is defined in qie, which must be compiled after the GUI dlls, we have no other choice than put it somewhere around here...
void mainMenu::exportET()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::exportET \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
	if(item==NULL) return;
	if(item->isFolder()) return;
	
	DvObject_var table=getWLObject(item->getPath().toLatin1().data());
    
	if(table.is_nil()){
		QuMsgBar("ERROR, Export object empty\n", true);
		return;
	}
	
	if(!table->is_event()){
		DvEvent range = table->getTimeRange();
		table = new DvObject(range);
	}
	    
	if(table.is_ok())
	{
		QString fileName=item->getName();
		fileName.remove(" (evt table)").append(".xml");
		QString fn=QFileDialog::getSaveFileName(this, "Select file - Save WL Settings", fileName, "*.xml");
		if(!fn.isEmpty())
		{
			bool res=QiSaveVOTable(table, fn);
			if(!res) QuMsgBar("ERROR, Export failed\n", true);
		}
		else
		{
			QuMsgBar("ERROR, Failed selecting the file\n", true);
		}
	}
	else
	{
		QuMsgBar("ERROR, Wrong object type\n", true);
	}
}

void mainMenu::openDS(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openDS \n");
  fflush(stdout);
  }
 
 
  DsShow();

}

void mainMenu::toggleToolbar(){ 
    if(qsasWindow->mainToolBar->isVisible()){
        qsasWindow->mainToolBar->setVisible(false);
        toolBar->setText("Show Tool Bar");
    }
    else{
        qsasWindow->mainToolBar->setVisible(true);
        toolBar->setText("Hide Tool Bar");
    }
    
}


void mainMenu::openCAA(){ 

	if(QuSlotDebug())
	{
		printf("mainMenu::openCAA \n");
		fflush(stdout);
	}
 
	if(def_dir.isEmpty()) 
	{
		def_dir = QSAS_profiles->current()->DataPath.toLatin1().data();

		if(def_dir.isEmpty())
		{
#ifdef WINDOWS
			def_dir = getenv("USERPROFILE");
#else
			def_dir = getenv("PWD"); // getenv returns const char * so is used to deep copy to string
#endif
		}
	}
	
	QString file;

	// get user file choice
	file = QFileDialog::getOpenFileName(this, "Select Data File", def_dir, "All QSAS data files ( *.cdf *.cef *.qft *.qfd *.TAB);;*.cdf;;*.cef;;*.qft;;*.qfd;;*.TAB;;Try any file (*)");
 
	if ( !file.isEmpty() ) {
  
		// keep chosen directory for next time
		int posn = file.lastIndexOf("/");
		if (posn >= 0){
			def_dir = QString(file.toLatin1().data()); // force deep copy
			def_dir.truncate(posn); 
			} 
		DaLaunchWindow(file, this);
	}
}

void mainMenu::openExport(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::openExport \n");
  fflush(stdout);
  }
 
 
   QjuiShowExport();
}

void mainMenu::openEvtTable(){ 
	if(QuSlotDebug())
	{
		printf("mainMenu::openEvtTable \n");
		fflush(stdout);
	}
	
	QString file;
	static QString dir=".";
	
	//       get user file choice
	file = QFileDialog::getOpenFileName(this, "Select Data File", dir, "VOTables ( *.xml);;*.*" );
	
	if ( !file.isEmpty() )
	{
		DvObject_var obj=QiLoadVOTable(file);
		if(obj.is_ok())
		{
			file=file.remove(".xml");
			int posn = file.lastIndexOf("/");
			if (posn >= 0)
			{
				dir = QString(file.toLatin1().data()); // force deep copy
				dir.truncate(posn);
				file=file.remove(dir);
			}
			
			file=file.append(" (evt table)");
		
			QwuiPutOnWL(obj, file);
		}
	}

}
void mainMenu::launchPSD()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchPSD \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	static PSD_Window *psdWindow = NULL;
	
	if(psdWindow == NULL) {
		psdWindow = new PSD_Window();
		psdWindow->setObject(path.toLatin1().data());
	}
	psdWindow->move(qsasGUISize().width(),30);
	psdWindow->show();
}

void mainMenu::launchBatchProcess( )
{
    static batchProcess *batchWindow = NULL;
    if(batchWindow == NULL) batchWindow = new batchProcess(qsasWindow);
    batchWindow->show();
    
    return;
}


void mainMenu::launchPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchPlot \n");
  fflush(stdout);
  }
 
  
  QpuiShow();
  
}

void mainMenu::launchPolarPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchPolarPlot \n");
  fflush(stdout);
  }
 
  
  QpolarShow();
  
}


void mainMenu::launchSurfacePlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launchSurfacePlot \n");
  fflush(stdout);
  }
 
  
  QsurfaceShow();
  
}

void mainMenu::launch3DPlot(){ 

 if(QuSlotDebug())
 {
  printf("mainMenu::launch3DPlot \n");
  fflush(stdout);
  }
 
  
  Q3dviewShow();
  
}

void mainMenu::launchJoin()
{ 
   QjuiShowJoin();
}


void mainMenu::launchCalculator()
{
	QcalcShowCalcProto();
}

void mainMenu::openDataFile(const char *file){
   
   DaLaunchWindow(QString(file), 0);
   
}

void mainMenu::launchEditProfile()
{
	QProfileEditor * edit=new QProfileEditor;
	if(QPUI_widget!=NULL) edit->setQpuiWidget(QPUI_widget);
	connect(edit, SIGNAL(changedNames()), this, SLOT(updateProfileList()));
	if(QPUI_widget!=NULL)
	{
		connect(edit, SIGNAL(refreshColourTableButtons()), QPUI_widget, SIGNAL(refreshColourTableButtonsSignal()));
	}
}

void mainMenu::updateProfileList()
{
	clear();
	insertProfilesItems();
	QSAS_profiles->save();
}

void mainMenu::launchSelectProfile(QAction *act)
{
	if(QSAS_profiles->setCurrent(act->text()))
	{
		checkedProfile->setChecked(false);
		checkedProfile=act;
		act->setChecked(true);
		
		if(QSAS_profiles->current()->fontSize>=5)
		{
			QFont f;
			f.setPixelSize(QSAS_profiles->current()->fontSize);
			qApp->setFont(f);
		}
		else
		{
			qApp->setFont(defaultFontQSAS);
		}
		
	}

    if(QPUI_widget!=NULL)
    {
        QPUI_widget->refreshColourTableButtons();
    }
    
    QSAS_profiles->save();

	
}

void mainMenu::angleDepTo360()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo360 \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(360);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}

void mainMenu::angleDepTo180()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo180 \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(180);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}
void mainMenu::angleDepTo0180()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo0180 \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(0);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}
void mainMenu::angleDepTo90()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepTo90 \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(90);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}
void mainMenu::angleDepReCentre()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::angleDepReCentre \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	AngleDepWindow *angleWindow = new AngleDepWindow(-1);
	angleWindow->setObject(path.toLatin1().data());
	angleWindow->move(qsasGUISize().width(),30);
	angleWindow->show();
}

void mainMenu::arrayMatInv()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::arrayMatInv \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *InvWindow = new ArrayUnaryWindow(-1);
    InvWindow->setObject(path.toLatin1().data());
    InvWindow->move(qsasGUISize().width(),30);
    InvWindow->show();
}

void mainMenu::arrayTrace()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchTrace \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *TraceWindow = new ArrayUnaryWindow(0);
    TraceWindow->setObject(path.toLatin1().data());
    TraceWindow->move(qsasGUISize().width(),30);
    TraceWindow->show();
}

void mainMenu::arrayTranspose()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchTranspose \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *TransposeWindow = new ArrayUnaryWindow(1);
    TransposeWindow->setObject(path.toLatin1().data());
    TransposeWindow->move(qsasGUISize().width(),30);
    TransposeWindow->show();
}

void mainMenu::arrayDeterminant()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchDeterminant \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *DetWindow = new ArrayUnaryWindow(2);
    DetWindow->setObject(path.toLatin1().data());
    DetWindow->move(qsasGUISize().width(),30);
    DetWindow->show();
}

void mainMenu::arrayAdjoint()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::arrayAdjoint \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *AdjWindow = new ArrayUnaryWindow(3);
    AdjWindow->setObject(path.toLatin1().data());
    AdjWindow->move(qsasGUISize().width(),30);
    AdjWindow->show();
}


void mainMenu::outerProd()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::outerProd \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *outerWindow = new ArrayBinaryWindow(5);
    outerWindow->setObject(path.toLatin1().data());
    outerWindow->move(qsasGUISize().width(),30);
    outerWindow->show();
}

void mainMenu::innerProd()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::innerProd \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *innerWindow = new ArrayBinaryWindow(6);
    innerWindow->setObject(path.toLatin1().data());
    innerWindow->move(qsasGUISize().width(),30);
    innerWindow->show();
}


void mainMenu::arrayElement()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::arrayElement \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *ElemWindow = new ArrayBinaryWindow(2);
    ElemWindow->setObject(path.toLatin1().data());
    ElemWindow->move(qsasGUISize().width(),30);
    ElemWindow->show();
}

void mainMenu::arrayElement_ijk()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::arrayElement_ijk \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *ElemWindow = new ArrayBinaryWindow(3);
    ElemWindow->setObject(path.toLatin1().data());
    ElemWindow->move(qsasGUISize().width(),30);
    ElemWindow->show();
}
void mainMenu::removeElement()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::removeElement \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *ElemWindow = new ArrayBinaryWindow(4);
    ElemWindow->setObject(path.toLatin1().data());
    ElemWindow->move(qsasGUISize().width(),30);
    ElemWindow->show();
}

void mainMenu::piecewiseMult()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchPiecewiseMult \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *pwWindow = new ArrayBinaryWindow(0);
    pwWindow->setObject(path.toLatin1().data());
    pwWindow->move(qsasGUISize().width(),30);
    pwWindow->show();
}

void mainMenu::piecewiseDivide()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchPiecewiseDivide \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *pwWindow = new ArrayBinaryWindow(1);
    pwWindow->setObject(path.toLatin1().data());
    pwWindow->move(qsasGUISize().width(),30);
    pwWindow->show();
}

void mainMenu::subMatrix()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::subMatrix \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayTriWindow *subMatWindow = new ArrayTriWindow(0);
    subMatWindow->setObject(path.toLatin1().data());
    subMatWindow->move(qsasGUISize().width(),30);
    subMatWindow->show();
}


void mainMenu::regression()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::regression \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	RegressionWindow *regressionWindow = new RegressionWindow();
	regressionWindow->setObject(path.toLatin1().data());
	regressionWindow->move(qsasGUISize().width(),30);
	regressionWindow->show();
}

void mainMenu::derivative()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::derivative \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    DerivativeWindow *derivativeWindow = new DerivativeWindow();
    derivativeWindow->setObject(path.toLatin1().data());
    derivativeWindow->move(qsasGUISize().width(),30);
    derivativeWindow->show();
}

void mainMenu::createRegObj()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::createRegObj \n");
        fflush(stdout);
    }
    
    // ask user for selection
    CreateRegularWindow *createRegWindow = new CreateRegularWindow();
    createRegWindow->move(qsasGUISize().width(),30);
    createRegWindow->show();
}

void mainMenu::integral()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::integral \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	IntegralWindow *integralWindow = new IntegralWindow();
	integralWindow->setObject(path.toLatin1().data());
	integralWindow->move(qsasGUISize().width(),30);
	integralWindow->show();
}


void mainMenu::launchTTOffset()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchTTOffset \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    TTagToScalarWindow *ttWindow = new TTagToScalarWindow();
    ttWindow->setObject(path.toLatin1().data());
    ttWindow->move(qsasGUISize().width(),30);
    ttWindow->show();
    
}

void mainMenu::launchSStoTT()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::launchSStoTT \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ScalarToTTagWindow *ttWindow = new ScalarToTTagWindow();
    ttWindow->setObject(path.toLatin1().data());
    ttWindow->move(qsasGUISize().width(),30);
    ttWindow->show();
    
}

void mainMenu::launchTItoSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchTItoSec \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	TIToSecWindow *ttWindow = new TIToSecWindow();
	ttWindow->setObject(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}

void mainMenu::launchT1T2toSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchT1T2toSec \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	T1T2ToSecWindow *ttWindow = new T1T2ToSecWindow();
	ttWindow->setObjectStart(path.toLatin1().data());
	ttWindow->setObjectEnd(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}

void mainMenu::launchTT1TT2toSec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::launchT1T2toSec \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();
	
	// ask user for selection
	TT1TT2ToSecWindow *ttWindow = new TT1TT2ToSecWindow();
	ttWindow->setObjectStart(path.toLatin1().data());
	ttWindow->move(qsasGUISize().width(),30);
	ttWindow->show();
	
}


void mainMenu::mergeTS()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::mergeTS \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    MergeTSWindow *toMergeWindow = new MergeTSWindow(0);
    toMergeWindow->setObject(path.toLatin1().data());
    toMergeWindow->move(qsasGUISize().width(),30);
    toMergeWindow->show();
}

void mainMenu::aTan2Deg()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::aTan2Deg \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *aTan2Window = new ArrayBinaryWindow(7);
    aTan2Window->setObject(path.toLatin1().data());
    aTan2Window->move(qsasGUISize().width(),30);
    aTan2Window->show();
}

void mainMenu::aTan2Rad()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::aTan2Rad \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayBinaryWindow *aTan2Window = new ArrayBinaryWindow(8);
    aTan2Window->setObject(path.toLatin1().data());
    aTan2Window->move(qsasGUISize().width(),30);
    aTan2Window->show();
}

void mainMenu::openTrig()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::openTrig \n");
        fflush(stdout);
    }
    
    QString pluginName = qsasWindow->qsas_home;
    pluginName += "/qtpl/Arithmetic/Trig.qtpl";   
    
    QtplOpenByName(pluginName, qsasWindow);
    return;
    
}

void mainMenu::dotProd()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::dotProd \n");
        fflush(stdout);
    }
    
    QString pluginName = qsasWindow->qsas_home;
    pluginName += "/qtpl/Arithmetic/DotProd.qtpl";   
    
    QtplOpenByName(pluginName, qsasWindow);
    return;
    
}

void mainMenu::vecProd()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::vecProd \n");
        fflush(stdout);
    }
    
    QString pluginName = qsasWindow->qsas_home;
    pluginName += "/qtpl/Arithmetic/VecProd.qtpl";   
    
    QtplOpenByName(pluginName, qsasWindow);
    return;
    
}

void mainMenu::normalizeVec()
{
    if(QuSlotDebug())
    {
        printf("mainMenu::normalizeVec \n");
        fflush(stdout);
    }
    
    WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();
    
    QString path;
    if(item==NULL || item->isFolder()) path = "";
    else  path = item->getPath();
    
    // ask user for selection
    ArrayUnaryWindow *normalizeWindow = new ArrayUnaryWindow(4);
    normalizeWindow->setObject(path.toLatin1().data());
    normalizeWindow->move(qsasGUISize().width(),30);
    normalizeWindow->show();
}


void mainMenu::array2vec()
{
	if(QuSlotDebug())
	{
		printf("mainMenu::vec2array \n");
		fflush(stdout);
	}
	
	WLItem* item = (WLItem*)qsasWindow->wlview->currentItem();

	QString path;
	if(item==NULL || item->isFolder()) path = "";
	else  path = item->getPath();

	// ask user for selection 
	MakeVectorWindow *toVectorWindow = new MakeVectorWindow(0);
	toVectorWindow->setObject(path.toLatin1().data());
	toVectorWindow->move(qsasGUISize().width(),30);
	toVectorWindow->show();  

}

void mainMenu::getVariable(DvString &fileName, DvString &varName, DvEvent &range){

    QieData *QieDataCont = new QieData();
    int nRecs = QieDataCont->queryFile(fileName.c_str());
    
    if( nRecs > 0 ){
        int varN = QieDataCont->findVarNumByName(varName.c_str());
        if(varN == -1) return;
        
        int recStart, recEnd;
				
        DvObject_var ttv = QieDataCont->readTT(varN, nRecs);
				
        int seqSize = ttv->seqSize();
        recStart = 0;
        recEnd = seqSize-1;

        if(!range.empty()){
            // find records in interval (Themis has multiple time lines, so do for each var)
 
            if( !(range.start_abs() <= ttv->time(0) && range.end_abs() >= ttv->time(recEnd)) )
            {
                // find records as not completely contained in ivl
                for(int k=0; k<seqSize; k++)
                {
                    if(range.start_abs() > ttv->time(k)) recStart = k+1;
                    if(range.end_abs() < ttv->time(seqSize-k-1)) recEnd = seqSize - k - 2;
                }
                if(recEnd < 0 || recStart > seqSize -1) return; // failed to find good interval
            }
        }
        
        // get var for this range

        QieDataCont->readVarData(varN);
				
        QieDataCont->attachXrefs(varN);
								
        DvObject_var dobj = QieDataCont->QieVars[varN]->Vobj->subSequence(recStart, recEnd);
        
        QwuiPutOnWL(dobj, varName.c_str());
    }
}

// class batchProcess
//-------------------

batchProcess::batchProcess(qwuiWindow *parent){
    
    qsas_home = parent->qsas_home;
    qwui = parent;
    pluginpath = "";
    exitOnError = false;
    
    QWidget * centreWidget = new QWidget(this);
    QGridLayout *batchLayout = new QGridLayout;

    centreWidget->setLayout(batchLayout);
    this->setCentralWidget(centreWidget);
    this->setWindowTitle("Batch Processing");

    QPushButton *selectFile = new QPushButton("Open batch file (.qbt)");
    connect(selectFile, SIGNAL(clicked()), this, SLOT(openBatchFile()));
                                              
                                              
    header = new QLineEdit();
    header->setMinimumWidth(700);

    playBatch = new QPushButton(" Run ",this);   
    connect(playBatch, SIGNAL(clicked()), this, SLOT(runProcess()));
    playBatch->setDefault(true);
    playBatch->setFocusPolicy(Qt::StrongFocus);

    stepBatch = new QPushButton(" Step ",this);
    connect(stepBatch, SIGNAL(clicked()), this, SLOT(stepProcess()));
    stepBatch->setFocusPolicy(Qt::StrongFocus);

    pauseBatch = new QPushButton("Pause",this); 
    pauseBatch->setCheckable(true); 
    pauseBatch->setChecked(false);
    pauseBatch->setFocusPolicy(Qt::StrongFocus);
                                             

    resumeBatch = new QPushButton("Resume",this); 
    connect(resumeBatch, SIGNAL(clicked()), this, SLOT(resumeProcess()));
    resumeBatch->setFocusPolicy(Qt::StrongFocus);
                                             
    stopBatch = new QPushButton(" Stop ",this); 
    stopBatch->setCheckable(true);
    stopBatch->setChecked(false);
    stopBatch->setFocusPolicy(Qt::StrongFocus);
 
    QPushButton *dismiss = new QPushButton("Dismiss",this);
    connect(dismiss, SIGNAL(clicked()), this, SLOT(dismissAndClose()));

    batchList = new QTextEdit();
    
    batchLayout->addWidget(selectFile,0,1,1,3);
    batchLayout->addWidget(header,1,0,1,8);
    batchLayout->addWidget(batchList,2,0,9,8);
                                              
    batchLayout->addWidget(playBatch,12,0);
                                              
    batchLayout->addWidget(pauseBatch,12,2);
    batchLayout->addWidget(resumeBatch,12,3);
    batchLayout->addWidget(stepBatch,12,4);
    batchLayout->addWidget(stopBatch,12,5);
    batchLayout->addWidget(dismiss,12,7);

    
    d=QDateTime::currentDateTime();    
 
}


void batchProcess::keyPressEvent(QKeyEvent * e )
{  
    // use return or enter key to perform default
    if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
        if(playBatch->isDefault()) runProcess();
        else if (resumeBatch->isDefault()) resumeProcess();
        return;
    }
    
    e->ignore(); 
    
}


bool batchProcess::stepProcess(){

    if ( !(batchInputFile.isOpen()) ) {
        
        batchInputFile.setFileName(batchFileName);
        batchInputFile.open( QIODevice::ReadOnly );
        if(batchInputFile.isOpen()) instream.setDevice( &batchInputFile );

    }
    
    // test in case open failed
    if ( batchInputFile.isOpen() ) {

        // Step process
        QString line;
 
        t.start();                   
        d = QDateTime::currentDateTime();
       
        line = instream.readLine().trimmed();
        while(line.startsWith("#") || line.startsWith("!") ) line = instream.readLine().trimmed();
        
        if(instream.atEnd() || line.startsWith(".stop") ) {
            batchInputFile.close();
            if(batchStatusFile.isOpen()) batchStatusFile.close();
            return false;
        }

        batchList->append(line);
        DvString fullLine = line.toLatin1().data();
        DvString command, arg1,arg2,arg3,arg4,arg5;
           
        command = fullLine.front(" ").trimmed(); 
        fullLine = fullLine.after(" ").trimmed();
        DvString fullArg = fullLine.c_str();
        arg1 = fullLine.front(" ").trimmed(); // remove possible quotes
        fullLine = fullLine.after(" ").trimmed();
        arg2 = fullLine.front(" ").trimmed(); 
        fullLine = fullLine.after(" ").trimmed();
        arg3 = fullLine.front(" ").trimmed(); 
        fullLine = fullLine.after(" ").trimmed();
        arg4 = fullLine.front(" ").trimmed(); 
        fullLine = fullLine.after(" ").trimmed();
        arg5 = fullLine.front(" ").trimmed(); 
 
        if(batchStatusFile.isOpen()) outstream << d.toString(Qt::ISODate); 

        if (command == "WRITELOG" && !arg1.empty()) {

            if(batchStatusFile.isOpen()) batchStatusFile.close();
            
            batchStatusFile.setFileName( arg1.c_str() ); 
            batchStatusFile.open( QIODevice::WriteOnly | QIODevice::Append );
            if(batchStatusFile.isOpen()) outstream.setDevice( &batchStatusFile );

        }  
        else if (command == "READ_DATA" && !arg1.empty() && !arg2.empty()) {
            DvTime t0;
            
            DvEvent range(t0,t0);
            if(!arg3.empty() && !arg4.empty()){
                DvTime t1(arg3); 
                DvTime t2(arg4);
                range.set(t1,t2);
            }	
            
			qwui->readData(arg1, arg2, range);
        }  
        else if (command == "RESTORE_WL" && !arg1.empty()) {
            qwui->fromSession = true; // prevent file selector popup
            qwui->_filename = arg1.c_str(); 
            qwui->restoreWL(true);	    
        }  
        else if (command == "RESTORE_CALC" && !arg1.empty()) {
            qwui->restoreCalc(arg1.c_str());            	    
        }  
        else if (command == "RESTORE_PLOT" && !arg1.empty()) {
            qwui->restorePlot(arg1.c_str());            	    
        }  
        else if (command == "RESTORE_SESSION" && !arg1.empty()) {
            qwui->restoreNamedSession(arg1.c_str());
        }  
        else if (command == "PLUGINPATH" ) {
            if(arg1.empty()) pluginpath = "";
            else{
                pluginpath = arg1.trimmed();
                if( !pluginpath.endsIn("/")) pluginpath += "/";
            }
        }  
        else if (command == "EXITONERROR" ) {
            exitOnError = true;
        }  
        else if (command == "SAVEWL") {
            if (!arg1.empty()) qwui->_filename = arg1.c_str();
            qwui->saveWL();
        } 
        else if (command == "CLEARWL") {
            qwui->wlview->deleteAllDataItems();
            qwui->wlview->deleteAllTimeIntervalItems();	    	  
        }
        else if (command == "CLEARTIME") {
            qwui->wlview->deleteAllTimeIntervalItems();	    
        }
        else if (command == "CLEARDATA") {
            qwui->wlview->deleteAllDataItems();
        } 
        else if (command == "INIT_PLUG_ARGS" && !arg1.empty() && !arg2.empty() ) {
            inPlugList.resize(arg1.toInt(), DvString());
            outPlugList.resize(arg2.toInt(), DvString());
        } 
        else if (command == "SET_IN_PLUG_ARG" && !arg1.empty() && !arg2.empty() ) {
            inPlugList[arg1.toInt()] = arg2;
        } 
        else if (command == "SET_OUT_PLUG_ARG" && !arg1.empty() && !arg2.empty() ) {
            outPlugList[arg1.toInt()] = arg2;
        } 
        else if (command == "RUNPLUGIN" && !arg1.empty()) {
            DvString pluginName = pluginpath;
            pluginName += arg1;
            
            int status = runBatchPlugin(pluginName, this, inPlugList, outPlugList);
            qApp->processEvents();
            
            if( status == -1 ) {
                QString txt("Failed to locate plugin: ");
                txt += pluginName.c_str();
                header->setText(txt);
            } 
            else if( status == 1 ) {
                QString txt("Failed to load plugin: ");
                txt += pluginName.c_str();
                header->setText(txt);
            } 
            else if( status == 2 ) {
                QString txt("Plugin problem with arguments: ");
                txt += pluginName.c_str();
                header->setText(txt);
            }             
            else if( status == 3 ) {
                QString txt("Plugin returned error: ");
                txt += pluginName.c_str();
                header->setText(txt);
            }  
            if(exitOnError && status != 0){
                batchInputFile.close();
                if(batchStatusFile.isOpen()) batchStatusFile.close();
                return false;
            }
        }
        else if (command == "PRINTDIRECT" && !arg1.empty()) {
            qwui->updatePlot();

            qwui->directPrint(arg1.c_str());  
   
        }
        else if (command == "CALC_PAGE" && !arg1.empty()) {
            qwui->calcPage(arg1.toInt());     
        } 
        else if (command == "CREATE_TIME" && !arg1.empty() && !arg2.empty() ) {
            // Time Interval
            DvTime t1(arg1); 
            DvTime t2(arg2);
            
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
            DvString timeName("Time Intervals/");
            if (!arg3.empty()) timeName += arg3.trimmed();
            else timeName += "Ivl";
            QwuiPutOnWL(evobj, timeName.c_str());
            
        } 
        else if (command == "CREATE_TIME_DURATION" && !arg1.empty() && !arg2.empty() ) {
            // Time Interval
            DvTime t1(arg1); 
            DvTime t2(arg1);
            double duration = arg2.toDouble();
            t2 += duration;
            
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
            DvString timeName("Time Intervals/");
            if (!arg3.empty()) timeName += arg3.trimmed();
            else timeName += "Ivl";
            QwuiPutOnWL(evobj, timeName.c_str());
            
        } 
        else if (command == "CREATE_TIME_SHIFTED" && !arg1.empty() && !arg2.empty() && !arg3.empty() ) {
            // Time Interval
            DvTime t1(arg1); 
            DvTime t2(arg2);
            double shift = arg3.toDouble();
            t1 += shift;
            t2 += shift;
            
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
            DvString timeName("Time Intervals/");
            if (!arg4.empty()) timeName += arg4.trimmed();
            else timeName += "Ivl";
            QwuiPutOnWL(evobj, timeName.c_str());
            
        } 
        else if (command == "CREATE_TIME_DURATION_SHIFTED" && !arg1.empty() && !arg2.empty() && !arg3.empty() ) {
            // Time Interval
            DvTime t1(arg1); 
            DvTime t2(arg1);
            
            double duration = arg2.toDouble();
            t2 += duration;
            
            double shift = arg3.toDouble();
            t1 += shift;
            t2 += shift;
            
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
            DvString timeName("Time Intervals/");
            if (!arg4.empty()) timeName += arg4.trimmed();
            else timeName += "Ivl";
            QwuiPutOnWL(evobj, timeName.c_str());
            
        } 
        else if (command == "CONSTANT_VEC" && !arg1.empty() && !arg2.empty() && !arg3.empty() && !arg4.empty() ) {
           // ----- VECTOR constant 3 values -----
            DvString constName = "Constants/"+arg1.trimmed();
            DvObject_var vobj = new DvObject(0.0, DIM3);
            vobj->dbl(0) = arg2.toDouble();		     
            vobj->dbl(1) = arg3.toDouble();		     
            vobj->dbl(2) = arg4.toDouble();		     
            QwuiPutOnWL(vobj, constName.c_str());	   
        } 
        else if (command == "CONSTANT_STR" && !arg1.empty() && !arg2.empty() ) {
            DvString constName = "Constants/"+arg1.trimmed();
            DvObject_var dobj = new DvObject(arg2.trimmed());
            QwuiPutOnWL(dobj, constName.c_str());	   
        } 
        else if (command == "CONSTANT_INT" && !arg1.empty() && !arg2.empty() ) {
            DvString constName = "Constants/"+arg1.trimmed();
            DvObject_var dobj = new DvObject(arg2.trimmed().toInt());
            QwuiPutOnWL(dobj, constName.c_str());	   
        } 
        else if (command == "CONSTANT_DBL" && !arg1.empty() && !arg2.empty() ) {
            DvString constName = "Constants/"+arg1.trimmed();
            DvObject_var dobj = new DvObject(arg2.trimmed().toDouble());
            QwuiPutOnWL(dobj, constName.c_str());	   
        }  
        else if (command == "WAIT" ) {
            playBatch->setDefault(false);
            resumeBatch->setDefault(true);
            return false;
        } 
        else if (command == "SHELL" && !arg1.empty() ) {
            QString cmd = fullArg.c_str();
            QProcess extraProc;
            extraProc.start(cmd);      
            if ( !extraProc.waitForFinished(30000) ) {
                batchList->append("Unable to process SHELL command\n   Check batch file");
            }
        } 
        else if (command == "CREATE_DOYTIME" && !arg1.empty() && !arg2.empty()) {
            DvString TimeObjName="Time Intervals/";
            if (!arg3.empty()) TimeObjName += arg3;
            else TimeObjName += "DOY_Ivl";
            
            DvTime t1;
            DvTime t2;
            double width = arg2.toDouble();
            
            int year=arg1.before("_").toInt();
            arg1 = arg1.back("_");
            int doy=arg1.before("_").toInt();;
            arg1 = arg1.back("_");
            int hour=arg1.substr(0,2).toInt();
            int min=arg1.substr(2,2).toInt();
            double sec=arg1.substr(4).toDouble();
            
            t1.setFromDOY(year,doy,hour,min,sec);
            t2.setFromDOY(year,doy,hour,min,sec);
            t2+= width;
            
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
            
            QwuiPutOnWL(evobj, TimeObjName.c_str());
        }
        else if (command == "CREATE_DOYTIME_SHIFT" && !arg1.empty() && !arg2.empty() && !arg3.empty()) {
            DvString TimeObjName="Time Intervals/";
            if (!arg4.empty()) TimeObjName += arg4;
            else TimeObjName += "DOY_Ivl";
        
            DvTime t1;
            DvTime t2;
            double width = arg2.toDouble();
        
            int year=arg1.before("_").toInt();
            arg1 = arg1.back("_");
            int doy=arg1.before("_").toInt();;
            arg1 = arg1.back("_");
            int hour=arg1.substr(0,2).toInt();
            int min=arg1.substr(2,2).toInt();
            double sec=arg1.substr(4).toDouble();
        
            t1.setFromDOY(year,doy,hour,min,sec);
            t2.setFromDOY(year,doy,hour,min,sec);
            t2+= width;
        
            if (!arg3.empty()) {
                double shift = arg3.toDouble();
                t1 += shift;
                t2 += shift;
            }
            DvEvent ev(t1,t2);
            DvObject_var evobj = new DvObject(ev);
        
            QwuiPutOnWL(evobj, TimeObjName.c_str());
        }
        else if (command == "DELETEITEM" && !arg1.empty()) {
            QString ToDelete = QString(arg1.c_str());
            if (!arg2.empty()) {
                ToDelete = QString(arg1.c_str())+" "+QString(arg2.c_str());  // e.g., "Time interval" - two arguments needed
            }

            WLItem *item = qwui->wlview->itemFromPath(ToDelete);
            if(item){
                qwui->wlview->deleteItem(item);
                qwui->wlview->emitDeleted(ToDelete);
            }
        }
    
        if(batchStatusFile.isOpen()) outstream <<  QString(" %1 secs : ").arg(t.elapsed()/1000.0, 7,'f',2) << line.toLatin1().data() << "\n";
        t.restart();
    }


    if(pauseBatch->isChecked()) {
        pauseBatch->setChecked(false);
        return false;
    }
    if(stopBatch->isChecked()) {
        stopBatch->setChecked(false);
        batchInputFile.close();
        if(batchStatusFile.isOpen()) batchStatusFile.close();
        
        return false;
    }
    
    return true;
}
                                              
void batchProcess::resumeProcess(){
    playBatch->setDefault(true);
    resumeBatch->setDefault(false);
    if ( batchInputFile.isOpen() ) {
        bool step = true;
        while(step){
            step = stepProcess();
        }
    }
    
}

void batchProcess::dismissAndClose(){
    
    batchList->clear();
    
    if(batchInputFile.isOpen() )  batchInputFile.close();
    if(batchStatusFile.isOpen()) batchStatusFile.close();

    this->close();
}
                                              
void batchProcess::runProcess(){
     
    batchList->clear();

    stopBatch->setChecked(false);

    batchInputFile.setFileName(batchFileName);
    
    if(batchInputFile.isOpen()) batchInputFile.close();
    batchInputFile.open( QIODevice::ReadOnly );

    if ( batchInputFile.isOpen() ) {
        instream.setDevice( &batchInputFile );
        bool step = true;
        while(step){
            step = stepProcess();
        }
    }
     
}

void batchProcess::openBatchFile(){
    
    batchFileName = QFileDialog::getOpenFileName(this, "Choose a file", qsas_home,"QSAS batch files (*.qbt);;Text files (*.txt);; All files (*.*)");
    if (batchFileName.isEmpty()) {
        return;
    }
    header->setText("Selected file: "+batchFileName);


}

