// Q3dView_window.cc
//


#include "q3dview.h"
#include "qdutil.h"


extern qwuiWindow* qsasWindow;
Q3dWindow *Q3d_widget=static_cast<Q3dWindow*>(NULL);


void Q3dMakeWindow( qwuiWindow *parent, const char* Title )
{
	if(Q3d_widget==NULL)
	{
		Q3d_widget = new Q3dWindow(parent);
		QSize qsasGUISize = qsasWindow->frameSize();
		Q3d_widget->adjustSize();  
		Q3d_widget->move((int)(qsasGUISize.width()), 25);

		Q3d_widget->connect(parent, SIGNAL(saveSessionRequest(QString)), Q3d_widget, SLOT(saveSession(QString)) );
		Q3d_widget->connect(parent, SIGNAL(restoreSessionRequest(QString)), Q3d_widget, SLOT(restoreSession(QString)) );

		Q3d_widget->setWindowTitle(Title);
		Q3d_widget->_filename+=QString("/saveSet");
		Q3d_widget->SAVE_EXTENSION=QString(".qx_3dview_plot");
		Q3d_widget->_filename+=Q3d_widget->SAVE_EXTENSION;
		
	}

}

void Q3dviewShow()
{
    Q3d_widget->show();
}


//     Q3dWindow CLASS
//     ----------------

Q3dWindow::Q3dWindow( QWidget *parent ):QMainWindow(0)
{
	// Constructor
	if(QuSlotDebug())
	{
		printf("Q3dWindow::Q3dWindow \n");
		fflush(stdout);
	}
    // used later by plot win
    startSlider = 0;
    startElevation = 70;
    startAzimuth = 30;
    elevation = NULL;
    azimuth = NULL;
    atSlider = NULL;
    delay = NULL;
	
	validData = false;
	
	// initialise defaults as they may get used before setPage() is called
	DPI = 72;
	conv = (double) DPI;
	escPushed = false;
		
	// set up colour group palettes
	QPalette Q3D_normal_palette = QPalette(qApp->palette());
	this->setPalette(Q3D_normal_palette);

	setAcceptDrops(true);
	setFocusPolicy(Qt::WheelFocus);

	QSplitter *_top_splitter=new QSplitter(Qt::Vertical, this);
	setCentralWidget( _top_splitter);

	tab_widget = new QTabWidget( _top_splitter);
	int margins = QSAS_profiles->current()->GUImargins;
	tab_widget->setContentsMargins(margins, margins, margins, margins);

	// data tab
	// ---------
	
	QWidget *dataTab=new QWidget();
	QGridLayout *dataTabLayout=new QGridLayout(dataTab);
	dataTabLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	dataTabLayout->setMargin(QSAS_profiles->current()->GUImargins);	
	dataTabLayout->setColumnStretch(0, 1);
	dataTabLayout->setColumnStretch(1, 3);
	dataTabLayout->setColumnStretch(2, 3);
	dataTabLayout->setColumnStretch(3, 3);
	dataTabLayout->setColumnStretch(4, 3);
	dataTabLayout->setColumnStretch(5, 10);
	dataTabLayout->setColumnStretch(6, 10);
	
	// data input
	QLabel * dataLabel = new QLabel("3D data: ");
	dataTabLayout->addWidget(dataLabel, 0, 0, Qt::AlignRight);

	dataSlot = new QguWLinput(dataTab, "data");
	dataSlot->setMinimumWidth(400);
	dataSlot->setToolTip("Accepts a 3D array or sequence of 3D arrays");
	dataSlot->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_ANY, QS_ANY));
	connect(dataSlot, SIGNAL(edited_OK()), this, SLOT(setDataObj()) );
	dataTabLayout->addWidget(dataSlot, 0, 1, 1, 6);
	
	//time selection
	atSlider = new  RecordSlider("At ", dataTab, "atSlider");
	atSlider->setToolTip("Select time (or record) at which to plot if a sequence of 3D records");
	dataTabLayout->addWidget(atSlider, 1, 0, 1, 7);
	connect(atSlider, SIGNAL(recordSliderChanged()), this, SLOT(setRecord()) );

    autoPlot = true;

	
	dataTab->adjustSize();
	tab_widget->addTab(dataTab, "Data");
	

	// Options Tab
	// ----------
	QWidget *optionsTab=new QWidget();
	QGridLayout *optionsTabLayout=new QGridLayout(optionsTab);
	optionsTabLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	optionsTabLayout->setMargin(QSAS_profiles->current()->GUImargins);	
	
	QLabel * titleLab = new QLabel("Plot Title: ");
	optionsTabLayout->addWidget(titleLab, 0, 0, Qt::AlignRight);
	
	titleSlot = new QLineEdit();
	titleSlot->setMinimumWidth(200);
	connect(titleSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserTitle()));
	optionsTabLayout->addWidget(titleSlot, 0, 1, 1, 3);

	QLabel * axisLab = new QLabel("Axis Type: ");
	optionsTabLayout->addWidget(axisLab, 0, 4, Qt::AlignRight);
	axisType = new QComboBox();
	axisType->insertItem(0, "Auto detect");
	axisType->insertItem(1, "Polar");
	axisType->insertItem(2, "Cartesian");
	axisType->setToolTip("R-Theta-Phi or X-Y-Z axes for box");
	optionsTabLayout->addWidget(axisType, 0, 5, Qt::AlignLeft);    
	connect(axisType, SIGNAL(currentIndexChanged(int)), this, SLOT(setAxisOpt()));
	

	RaxisLab = new QLabel("R axis Text: ");
	optionsTabLayout->addWidget(RaxisLab, 1, 0, Qt::AlignRight);
	
	RXLabSlot = new QLineEdit();
	RXLabSlot->setMinimumWidth(160);
	connect(RXLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserRXLab()));
	optionsTabLayout->addWidget(RXLabSlot, 1, 1);

	ThAxisLab = new QLabel("Theta axis Text: ");
	optionsTabLayout->addWidget(ThAxisLab, 1, 2, Qt::AlignRight);
	
	ThYLabSlot = new QLineEdit();
	ThYLabSlot->setMinimumWidth(160);
	connect(ThYLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserThYLab()));
	optionsTabLayout->addWidget(ThYLabSlot, 1, 3);

	PhiAxisLab = new QLabel("Phi axis Text: ");
	optionsTabLayout->addWidget(PhiAxisLab, 1, 4, Qt::AlignRight);
	
	PhiZLabSlot = new QLineEdit();
	PhiZLabSlot->setMinimumWidth(160);
	connect(PhiZLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserPhiZLab()));
	optionsTabLayout->addWidget(PhiZLabSlot, 1, 5);

	QLabel * typeLab = new QLabel("Plot Type: ");
	optionsTabLayout->addWidget(typeLab, 2, 0, Qt::AlignRight);
	plotType = new QComboBox();
	plotType->insertItem(0, "Data Points");
	plotType->insertItem(1, "R Ribbons, const Theta");
	plotType->insertItem(2, "R Ribbons, const Phi");
	plotType->insertItem(3, "Theta Ribbons, const R");
	plotType->insertItem(4, "Theta Ribbons, const Phi");
	plotType->insertItem(5, "Phi Ribbons, const R");
	plotType->insertItem(6, "Phi Ribbons, const Theta");
	plotType->setToolTip("Points show data value as colour at data point positions\nRibbons show data value as colour along one dimension");
	optionsTabLayout->addWidget(plotType, 2, 1, Qt::AlignLeft);
	connect(plotType, SIGNAL(currentIndexChanged(int)), this, SLOT(setPlotType(int)));

	ribbonLab = new QLabel("Ribbon width: ");
	ribbonLab->setEnabled(false);
	optionsTabLayout->addWidget(ribbonLab, 2, 2, Qt::AlignRight);
	beta = new FloatSpinBox(0.0, 1.0, 1, 0.1, NULL, "ribbonFraction");
	beta->setValue(1.0);
	beta->setToolTip("Fraction of bin filled by ribbons");
	optionsTabLayout->addWidget(beta, 2, 3, Qt::AlignLeft);
	beta->setEnabled(false);

    QLabel * swapLab = new QLabel("R dimension: ");
    optionsTabLayout->addWidget(swapLab, 2, 4, Qt::AlignRight);
    setRX = new QComboBox();
    setRX->insertItem(0, "auto");
    setRX->insertItem(1, "1st dim");
    setRX->insertItem(2, "2nd dim");
    setRX->insertItem(3, "3rd dim");
    optionsTabLayout->addWidget(setRX, 2, 5, Qt::AlignLeft);
    connect(setRX, SIGNAL(currentIndexChanged(int)), this, SLOT(setDataObj()) );

	shape_label=new QLabel("Symbol: ");
	optionsTabLayout->addWidget(shape_label,3,0,Qt::AlignRight);
	symbol_shape=new QComboBox();
	symbol_shape->addItem("Point");
	symbol_shape->addItem("Plus sign");
	symbol_shape->addItem("Asterisk");
	symbol_shape->addItem("Circle");
	symbol_shape->addItem("Cross");
	symbol_shape->addItem("Square");
	symbol_shape->addItem("Triangle");
	symbol_shape->addItem("Circled Point");
	symbol_shape->addItem("Lens Flare");
	symbol_shape->addItem("Diamond");
	symbol_shape->addItem("Star");
	symbol_shape->addItem("Wide cross");
	symbol_shape->addItem("Filled Square");
	symbol_shape->addItem("Filled Circle");
	symbol_shape->addItem("Filled Star");
	symbol_shape->addItem("Left Arrow");
	symbol_shape->addItem("Right Arrow");
	symbol_shape->addItem("Up Arrow");
	symbol_shape->addItem("Down Arrow");
	QFontMetrics symbol_shape_fm(symbol_shape->font());
	int symbol_shape_w=symbol_shape_fm.width("Filled Triangle");
	int symbol_shape_h=symbol_shape_fm.height();
	symbol_shape->setMinimumSize(symbol_shape_w,symbol_shape_h);
	symbol_shape->setToolTip("Select symbol shape");
	optionsTabLayout->addWidget(symbol_shape,3,1,Qt::AlignLeft);
	
	size_label = new QLabel("Symbol Size: ");
	optionsTabLayout->addWidget(size_label,3,2,Qt::AlignRight);
	symbol_size = new FloatSpinBox(0.1,20.,1,0.5,this, "symbol_size");
	symbol_size->setFixedWidth(100);
	optionsTabLayout->addWidget(symbol_size,3,3,Qt::AlignLeft);
	symbol_size->setValue(1.0);
	
	QLabel * alphaLab = new QLabel("Opacity: ");
	optionsTabLayout->addWidget(alphaLab, 3, 4, Qt::AlignRight);
	alphaSpin = new FloatSpinBox(-2.0, 1.0, 2, 0.1, NULL, "alpha");
	alphaSpin->setValue(0.4);
	alphaSpin->setToolTip("Alpha value for points and ribbons, 0 (transparent) to  1 (opaque)\n Colour runs linearlyfrom 1.0 at top to alpha at bottom.\nNegative alpha causes transparency to occur before bottom of colour bar");
	optionsTabLayout->addWidget(alphaSpin, 3, 5, Qt::AlignLeft);

	RXLog = new QCheckBox("Log Radius ");
	RXLog->setChecked(true);
	optionsTabLayout->addWidget(RXLog, 4, 0, Qt::AlignLeft);
	connect(RXLog, SIGNAL(toggled(bool)), this, SLOT(setLogRX()) );

	RXAuto = new QCheckBox("Auto radius");
	RXAuto->setChecked(true);
	connect(RXAuto, SIGNAL(toggled(bool)), this, SLOT(setAuto()));
	optionsTabLayout->addWidget(RXAuto, 4, 1, Qt::AlignRight);

	RXMinLab = new QLabel("Min Radius: ");
	optionsTabLayout->addWidget(RXMinLab, 4, 2, Qt::AlignRight);
	RXMin = new QLineEdit();
	RXMin->setMinimumWidth(30);
	RXMin->setToolTip("Minimum radial value (log(0) not allowed)");
	connect(RXMin, SIGNAL(textEdited(const QString& )), this, SLOT(changeRX()));
	optionsTabLayout->addWidget(RXMin, 4, 3, Qt::AlignLeft);
	
	RXMaxLab = new QLabel("Max Radius: ");
	optionsTabLayout->addWidget(RXMaxLab, 4, 4, Qt::AlignRight);
	RXMax = new QLineEdit();
	RXMax->setMinimumWidth(30);
	RXMax->setToolTip("Maximum radial value ");
	connect(RXMax, SIGNAL(textEdited(const QString& )), this, SLOT(changeRX()));
	optionsTabLayout->addWidget(RXMax, 4, 5, Qt::AlignLeft);
	
	
	YLog = new QCheckBox("Log Y axis");
	optionsTabLayout->addWidget(YLog, 5, 0, Qt::AlignLeft);
	connect(YLog, SIGNAL(toggled(bool)), this, SLOT(setLogY()) );
	YLog->setEnabled(false);
	
	ThYAuto = new QCheckBox("Auto angle");
	ThYAuto->setChecked(true);
	connect(ThYAuto, SIGNAL(toggled(bool)), this, SLOT(setAuto()));
	optionsTabLayout->addWidget(ThYAuto, 5, 1, Qt::AlignRight);

	ThYMinLab = new QLabel("Min angle: ");
	optionsTabLayout->addWidget(ThYMinLab, 5, 2, Qt::AlignRight);
	ThYMin = new QLineEdit();
	ThYMin->setMinimumWidth(30);
	ThYMin->setToolTip("Minimum angle");
	connect(ThYMin, SIGNAL(textEdited(const QString& )), this, SLOT(changeThY()));
	optionsTabLayout->addWidget(ThYMin, 5, 3, Qt::AlignLeft);
	
	ThYMaxLab = new QLabel("Max angle: ");
	optionsTabLayout->addWidget(ThYMaxLab, 5, 4, Qt::AlignRight);
	ThYMax = new QLineEdit();
	ThYMax->setMinimumWidth(30);
	ThYMax->setToolTip("Maximum angle for polar axis");
	connect(ThYMax, SIGNAL(textEdited(const QString& )), this, SLOT(changeThY()));
	optionsTabLayout->addWidget(ThYMax, 5, 5, Qt::AlignLeft);
		
	ZLog = new QCheckBox("Log Z axis");
	optionsTabLayout->addWidget(ZLog, 6, 0, Qt::AlignLeft);
	connect(ZLog, SIGNAL(toggled(bool)), this, SLOT(setLogZ()) );
	ZLog->setEnabled(false);
	
	PhiZAuto = new QCheckBox("Auto angle");
	PhiZAuto->setChecked(true);
	connect(PhiZAuto, SIGNAL(toggled(bool)), this, SLOT(setAuto()));
	optionsTabLayout->addWidget(PhiZAuto, 6, 1, Qt::AlignRight);

	PhiZMinLab = new QLabel("Min angle: ");
	optionsTabLayout->addWidget(PhiZMinLab, 6, 2, Qt::AlignRight);
	PhiZMin = new QLineEdit();
	PhiZMin->setMinimumWidth(30);
	PhiZMin->setToolTip("Minimum angle");
	connect(PhiZMin, SIGNAL(textEdited(const QString& )), this, SLOT(changePhiZ()));
	optionsTabLayout->addWidget(PhiZMin, 6, 3, Qt::AlignLeft);
	
	PhiZMaxLab = new QLabel("Max angle: ");
	optionsTabLayout->addWidget(PhiZMaxLab, 6, 4, Qt::AlignRight);
	PhiZMax = new QLineEdit();
	PhiZMax->setMinimumWidth(30);
	PhiZMax->setToolTip("Maximum angle for polar axis");
	connect(PhiZMax, SIGNAL(textEdited(const QString& )), this, SLOT(changePhiZ()));
	optionsTabLayout->addWidget(PhiZMax, 6, 5, Qt::AlignLeft);
		
	colourLog = new QCheckBox("Log ColourScale");
	colourLog->setChecked(true);
	connect(colourLog, SIGNAL(toggled(bool)), this, SLOT(setLogCol()));
	optionsTabLayout->addWidget(colourLog, 7, 0, Qt::AlignLeft);

	colourAuto = new QCheckBox("Auto ColourScale");
	colourAuto->setChecked(true);
	connect(colourAuto, SIGNAL(toggled(bool)), this, SLOT(autosetCol()));
	optionsTabLayout->addWidget(colourAuto, 7, 1, Qt::AlignRight);

	QLabel * colMinLab = new QLabel("Min value: ");
	optionsTabLayout->addWidget(colMinLab, 7, 2, Qt::AlignRight);
	colMin = new QLineEdit();
	colMin->setMinimumWidth(50);
	colMin->setMaximumWidth(100);
	colMin->setText("0.0");
	colMin->setToolTip("Minimum data value (log(0) not allowed)");
	connect(colMin, SIGNAL(textEdited(const QString & )), this, SLOT(setColUser()));
	optionsTabLayout->addWidget(colMin, 7, 3, Qt::AlignLeft);

	QLabel * colMaxLab = new QLabel("Max value: ");
	optionsTabLayout->addWidget(colMaxLab, 7, 4, Qt::AlignRight);
	colMax = new QLineEdit();
	colMax->setMinimumWidth(50);
	colMax->setMaximumWidth(100);
	colMax->setToolTip("Maximum data value (log(0) not allowed)");
	colMax->setText("1.0e6");
	connect(colMax, SIGNAL(textEdited(const QString & )), this, SLOT(setColUser()));
	optionsTabLayout->addWidget(colMax, 7, 5, Qt::AlignLeft);

	QLabel* colour_table_label = new QLabel("Colour Table: ");
	optionsTabLayout->addWidget(colour_table_label, 8, 0, Qt::AlignRight);
    colourTable = new ColourTableButton;
    optionsTabLayout->addWidget(colourTable, 8, 1, Qt::AlignLeft);

	QLabel * colLabLab = new QLabel("Colour bar label: ");
	optionsTabLayout->addWidget(colLabLab, 8, 2, Qt::AlignRight);
	
	colLabSlot = new QLineEdit();
	colLabSlot->setMinimumWidth(200);
	connect(colLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserColLab()));
	optionsTabLayout->addWidget(colLabSlot, 8, 3, 1, 3);

    QLabel* axis_width_label=new QLabel("Axis width: ");
    optionsTabLayout->addWidget(axis_width_label,9,0,Qt::AlignRight);
    axis_width_fld = new FloatSpinBox(0.1,20.,1,0.1, NULL, "axis_width");
    axis_width_fld->setValue(QSAS_profiles->current()->LineWidth);
    axis_width_fld->setFixedWidth(50);
    optionsTabLayout->addWidget(axis_width_fld,9,1,Qt::AlignLeft);
    
	QLabel* title_size_label = new QLabel("Title font size: ");
	optionsTabLayout->addWidget(title_size_label, 9, 2, Qt::AlignRight);
	title_size_fld = new FloatSpinBox(0.01, 40.,1, .1, NULL, "title_char_size");
	title_size_fld->setFixedWidth(50);
	optionsTabLayout->addWidget(title_size_fld, 9, 3, Qt::AlignLeft);
	title_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+0.3);

	QLabel* text_size_label = new QLabel("Text font size: ");
	optionsTabLayout->addWidget(text_size_label, 9, 4, Qt::AlignRight);
	text_size_fld = new FloatSpinBox(0.01, 40.,1, .1, NULL, "text_char_size");
	text_size_fld->setFixedWidth(50);
	optionsTabLayout->addWidget(text_size_fld, 9, 5, Qt::AlignLeft);
	text_size_fld->setValue(QSAS_profiles->current()->BaseFontSize);


	tab_widget->addTab(optionsTab, "Plot Options");
	
	// Page Tab
	// ----------
	QWidget *pageTab=new QWidget();

	QVBoxLayout *page_properties_top_layout=new QVBoxLayout(pageTab);
	page_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_size_groupbox = new QGroupBox(pageTab);
	page_size_groupbox->setTitle("Size");

	QGridLayout *page_size_layout = new QGridLayout;
	page_size_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_size_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* page_size_label = new QLabel("Page size: ");
	page_size_layout->addWidget(page_size_label, 1, 0, Qt::AlignRight);
	page_size = new QComboBox();
	page_size->insertItem(QPUI_A4, "A4");
	page_size->insertItem(QPUI_CUSTOM, "Custom");
	page_size->insertItem(QPUI_EXECUTIVE, "Executive");
	page_size->insertItem(QPUI_LEGAL, "Legal");
	page_size->insertItem(QPUI_LETTER, "Letter");
	page_size->insertItem(QPUI_DEFAULT, "Default");

	QFontMetrics page_size_fm(page_size->font());
	int page_size_w = page_size_fm.width("Executive");
	int page_size_h = page_size_fm.height();
	page_size->setMinimumSize(page_size_w, page_size_h);

	page_size->setToolTip("Select page size");
	page_size_layout->addWidget(page_size, 1, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageSize=="A4")
		page_size->setCurrentIndex(QPUI_A4);
	else if(QSAS_profiles->current()->PageSize=="Executive")
		page_size->setCurrentIndex(QPUI_EXECUTIVE);
	else if(QSAS_profiles->current()->PageSize=="Legal")
		page_size->setCurrentIndex(QPUI_LEGAL);
	else if(QSAS_profiles->current()->PageSize=="Letter")
		page_size->setCurrentIndex(QPUI_LETTER);
	else if(QSAS_profiles->current()->PageSize=="Default")
		page_size->setCurrentIndex(QPUI_DEFAULT);
	else page_size->setCurrentIndex(QPUI_CUSTOM);
	
	QLabel* units_label = new QLabel("Units: ");
	page_size_layout->addWidget(units_label, 1, 2, Qt::AlignRight);
	pageUnits = new QComboBox();
	pageUnits->insertItem(QPUI_PIXELS, "pixels");
	pageUnits->insertItem(QPUI_INCHES, "inches");
	pageUnits->insertItem(QPUI_CM, "cm");
	pageUnits->insertItem(QPUI_MM, "mm");
	pageUnits->insertItem(QPUI_PSPOINTS, "PS points");
	QFontMetrics units_fm(pageUnits->font());
	int units_w = units_fm.width("PS points");
	int units_h = page_size_fm.height();
	pageUnits->setMinimumSize(units_w, units_h);

	pageUnits->setToolTip("Select units for page size");
	page_size_layout->addWidget(pageUnits, 1, 3, Qt::AlignLeft);

	
	oldUnits = pageUnits->currentIndex();
	
	QLabel* dpi_label = new QLabel("Resolution: ");
	page_size_layout->addWidget(dpi_label, 1, 4, Qt::AlignRight);
	dpiPage = new QComboBox();
	dpiPage->insertItem(QPUI_72_DPI, "72 dpi");
	dpiPage->insertItem(QPUI_150_DPI, "150 dpi");
	dpiPage->insertItem(QPUI_300_DPI, "300 dpi");
	dpiPage->insertItem(QPUI_600_DPI, "600 dpi");
	dpiPage->insertItem(QPUI_1200_DPI, "1200 dpi");
	switch(QSAS_profiles->current()->PageDPI)
	{
		case 72: default:
			dpiPage->setCurrentIndex(QPUI_72_DPI);
			break;
		case 150:
			dpiPage->setCurrentIndex(QPUI_150_DPI);
			break;
		case 300:
			dpiPage->setCurrentIndex(QPUI_300_DPI);
			break;
		case 600:
			dpiPage->setCurrentIndex(QPUI_600_DPI);
			break;
		case 1200:
			dpiPage->setCurrentIndex(QPUI_1200_DPI);
			break;
	}

	page_size_layout->addWidget(dpiPage, 1, 5, Qt::AlignLeft);
	
	QLabel* width_label = new QLabel("Width: ");
	page_size_layout->addWidget(width_label, 3, 0, Qt::AlignRight);
	pageWidth = new QLineEdit;
	pageWidth->setFixedWidth(100);
	pageWidth->setValidator(new QDoubleValidator(pageWidth));

	page_size_layout->addWidget(pageWidth, 3, 1, Qt::AlignLeft);

	QLabel* height_label = new QLabel("Height: ");
	page_size_layout->addWidget(height_label, 3, 2, Qt::AlignRight);
	pageHeight = new QLineEdit;
	pageHeight->setFixedWidth(100);
	pageHeight->setValidator(new QDoubleValidator(pageHeight));
	page_size_layout->addWidget(pageHeight, 3, 3, Qt::AlignLeft);
	
	
	// set appropriate text in width and height fields
	if(QSAS_profiles->current()->PageSize=="Custom")
	{
		width = QSAS_profiles->current()->PageWidth;
		height = QSAS_profiles->current()->PageHeight;
		pageWidth->setText(QString().setNum(width));
		pageHeight->setText(QString().setNum(height));
	}

	QLabel* orientation_label = new QLabel("Orientation: ");
	page_size_layout->addWidget(orientation_label, 5, 0, Qt::AlignRight);
	pageOrientation = new QComboBox();
	pageOrientation->insertItem(QPUI_PORTRAIT, "Portrait");
	pageOrientation->insertItem(QPUI_LANDSCAPE, "Landscape");
	QFontMetrics orientation_fm(pageOrientation->font());
	int orientation_w = orientation_fm.width("Landscape");
	int orientation_h = orientation_fm.height();
	pageOrientation->setMinimumSize(orientation_w, orientation_h);

	pageOrientation->setToolTip("Select portrait or landscape");
	page_size_layout->addWidget(pageOrientation, 5, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageOrientation=="Landscape")
		pageOrientation->setCurrentIndex(QPUI_LANDSCAPE);
	else pageOrientation->setCurrentIndex(QPUI_PORTRAIT);


	
	page_size_groupbox->setLayout(page_size_layout);

	page_properties_top_layout->addWidget(page_size_groupbox);

	QGroupBox *page_output_groupbox = new QGroupBox(this);

	page_output_groupbox->setTitle("Plot Window");

	QGridLayout *page_output_layout = new QGridLayout;
	page_output_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_output_layout->setMargin(QSAS_profiles->current()->GUImargins);


    
    
    newWindow = new QCheckBox("New window for each plot");
    page_output_layout->addWidget(newWindow,0,0,Qt::AlignLeft);
    
    plotWin_id = new PlotID("3D Plot");
    page_output_layout->addWidget(plotWin_id, 0, 3, Qt::AlignLeft);
	
	page_output_groupbox->setLayout(page_output_layout);

	page_properties_top_layout->addWidget(page_output_groupbox);


	tab_widget->addTab(pageTab, "Page");

	tab_widget->adjustSize();

	// feedback text area
	textArea = new QTextEdit(_top_splitter);
	textArea->setReadOnly(true);


	// set up menu bar
	QMenu* file_menu = 	menuBar()->addMenu("&File");
	menuBar()->addSeparator();
	QMenu* help_menu = 	menuBar()->addMenu("&Help" );


	file_menu->clear();
    file_menu->addAction("Save As XML...", this, SLOT(saveAsXML()));
	file_menu->addSeparator();
    file_menu->addAction("Restore XML...", this, SLOT(restoreXML()));

	help_menu->clear();
	help_menu->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help_menu->addSeparator();
	help_menu->addAction("3D View Help", this, SLOT( launch3dviewPlotHelp() ));
	help_menu->addSeparator();
	help_menu->addAction("Module Version", this, SLOT( launchVersion() ));

	_status_bar = statusBar();
	_status_bar->showMessage(" ");

	QWidget * buttonsWidget = new QWidget(_top_splitter);
	QHBoxLayout *buttons_layout = new QHBoxLayout(buttonsWidget);

	plotBtn = new QPushButton("Plot",this);
	plotBtn->setMinimumSize(plotBtn->sizeHint());
	plotBtn->setFocusPolicy(Qt::StrongFocus);
	connect(plotBtn, SIGNAL(clicked()), this, SLOT(doPlot()));
	buttons_layout->addWidget(plotBtn);
	plotBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss", this);
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	dismissBtn->setFocusPolicy(Qt::StrongFocus);
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));


	_top_splitter->setStretchFactor(0, 10);
	_top_splitter->setStretchFactor(1, 10);
	_top_splitter->setStretchFactor(1, 1);
	
	adjustSize();
	userTitle = false;
	userThLab = false;
	userPhiLab = false;
	userRLab = false;
	userColLab = false;
	
	// ensure pointers are NULL for trap on free()
	// note z is still colour, phi or PhiZ is 3rd dimension
	z = 0;
	zLog = 0;
	nr = 0;
	ntheta = 0;
	nphi = 0;
	cgrid.nx = 0;
	cgrid.ny = 0;
	r = 0;
	r_plus = 0;
	r_minus = 0;
	theta = 0;
	theta_plus = 0;
	theta_minus = 0;
	phi = 0;
	phi_plus = 0;
	phi_minus = 0;

	connect(dpiPage, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	connect(pageUnits, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	connect(page_size, SIGNAL(currentIndexChanged(int)), this, SLOT(setPage()));
	setPage();
	
}

// ------------------------

void Q3dWindow::setGUItoPolar(bool isPolar)
{
	if(isPolar){
		RXLog->setText(QString("Log Radius"));
		RXAuto->setText(QString("Auto radius"));
		ThYAuto->setText(QString("Auto angle"));
		PhiZAuto->setText(QString("Auto angle"));
		YLog->setEnabled(false);
		ZLog->setEnabled(false);
		RXMinLab->setText("Min Radius: ");
		RXMaxLab->setText("Max Radius: ");
		ThYMinLab->setText("Min angle: ");
		ThYMaxLab->setText("Max angle: ");
		PhiZMinLab->setText("Min angle: ");
		PhiZMaxLab->setText("Max angle: ");
		RaxisLab->setText("R axis Text: ");
		ThAxisLab->setText("Theta axis Text: ");
		PhiAxisLab->setText("Phi axis Text: ");
		RXLog->setChecked(true);

	}
	else{
		RXLog->setText(QString("Log X"));
		RXAuto->setText(QString("Auto X range"));
		ThYAuto->setText(QString("Auto Y range"));
		PhiZAuto->setText(QString("Auto Z range"));
		YLog->setEnabled(true);
		ZLog->setEnabled(true);
		RXMinLab->setText("Min X: ");
		RXMaxLab->setText("Max X: ");
		ThYMinLab->setText("Min Y: ");
		ThYMaxLab->setText("Max Y: ");
		PhiZMinLab->setText("Min Z: ");
		PhiZMaxLab->setText("Max Z: ");
		RaxisLab->setText("X axis Text: ");
		ThAxisLab->setText("Y axis Text: ");
		PhiAxisLab->setText("Z axis Text: ");
		RXLog->setChecked(false);
	}
}

// ------------------------

Q3dWindow::~Q3dWindow()
{
	if (z != 0) delete[] z; 
	if (zLog != 0) delete[] zLog; 
	
	if (r != 0) delete[] r;
	if (r_plus != 0) delete[] r_plus;
	if (r_minus != 0) delete[] r_minus;
	if (theta != 0) delete[] theta;
	if (theta_plus != 0) delete[] theta_plus;
	if (theta_minus != 0) delete[] theta_minus;
	if (phi != 0) delete[] phi;
	if (phi_plus != 0) delete[] phi_plus;
	if (phi_minus != 0) delete[] phi_minus;

}

// ------------------------

void Q3dWindow::setPlotType(int type)
{
	if(type == 0) {
		symbol_size->setEnabled(true);
		size_label->setEnabled(true);
		symbol_shape->setEnabled(true);
		shape_label->setEnabled(true);
		beta->setEnabled(false);
		ribbonLab->setText("Ribbon width");
		ribbonLab->setEnabled(false);
	}
	else {
		symbol_size->setEnabled(false);
		size_label->setEnabled(false);
		symbol_shape->setEnabled(false);
		shape_label->setEnabled(false);
		beta->setEnabled(true);
		ribbonLab->setEnabled(true);
		if(type == 1 || type == 3) ribbonLab->setText("Ribbon phi width");
		else if(type == 2 || type == 5) ribbonLab->setText("Ribbon theta width");
		else if(type == 4 || type == 6) ribbonLab->setText("Ribbon R width");
	}
}


// ------------------------

void Q3dWindow::setAxisOpt()
{
	if(dobj.is_nil() ) {
		// handle case of changing toggle before data present
		bool polar = true;
		if(axisType->currentIndex() == 2) polar = false;
		setGUItoPolar(polar);
		return;
	}
	
	if( initData() ) return; // this will call setGUItoPolar()
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
	if(axisType->currentIndex() == 2) {
		plotType->setCurrentIndex(0);
		plotType->setEnabled(false);
	}
	else 
		plotType->setEnabled(true);
}

// ------------------------

void Q3dWindow::setDataObj()
{
	if ( initData() ) {
		Q3dviewAppendTextDisplay("Data object does not work for this plot setting");
		return;
	}
	setData();

	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
}


// ------------------------

void Q3dWindow::setRecord(bool andPlot)
{
	if(!validData) return;
	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
	if(andPlot) doPlot(true);
}


// ------------------------

void Q3dWindow::autosetRX()
{

	if(dobj.is_nil()) return;

	if(RXAuto->isChecked())
	{	
		if(RXLog->isChecked()) RXMax->setText( QString("%1").arg(pow(10., rmax)*userRmin) );
		else RXMax->setText( QString("%1").arg(rmax + rmin) );
		
		if(RXLog->isChecked()) RXMin->setText(QString("%1").arg(rminNonZero));
		else RXMin->setText(QString("%1").arg(rmin));	
	}
}

// ------------------------

void Q3dWindow::autosetThY()
{

	if(dobj.is_nil()) return;
	
	if(ThYAuto->isChecked())
	{
		if(YLog->isChecked()) {
			ThYMax->setText( QString("%1").arg((thetamax + thetamin)*rad2degTh) );
			ThYMin->setText(QString("%1").arg(thetaminNonZero*rad2degTh));
		}
		else {
			ThYMax->setText( QString("%1").arg(thetamax*rad2degTh ) );
			ThYMin->setText(QString("%1").arg(thetamin*rad2degTh));
		}
	}
}

// ------------------------

void Q3dWindow::autosetPhiZ()
{

	if(dobj.is_nil()) return;
	
	if(PhiZAuto->isChecked())
	{
		if(ZLog->isChecked()){
			PhiZMax->setText( QString("%1").arg((phimax + phimin)*rad2degPhi) );
			PhiZMin->setText(QString("%1").arg(phiminNonZero*rad2degPhi));
		}
		else {
			PhiZMax->setText( QString("%1").arg(phimax*rad2degPhi ) );
			PhiZMin->setText(QString("%1").arg(phimin*rad2degPhi));
		}
		
	}
}

// ------------------------

void Q3dWindow::autosetCol()
{
	if(dobj.is_nil() || !z) return;
		
	if(colourAuto->isChecked())
	{
		// set min/max
		zmin = arrmin(z, ndata);
		zmax = arrmax(z, ndata);
				
		if(colourLog->isChecked()){
			nonZeroMin();
		}

		colMax->setText(QString("%1").arg(zmax));
		colMin->setText(QString("%1").arg(zmin));
	}
}

// ------------------------

void Q3dWindow::changeView()
{
	// re-render viewed from new angles
	doPlot(true);

}

// ------------------------

void Q3dWindow::nonZeroMin()
{
	if(z == 0) return;

	zmin = zmax; // initialise to safe value
	PLFLT val;
	for(int i=0; i<ndata; i++){
		val = z[i];
		if(val < zmin && val > 1.e-30) zmin = val;
	}	

}

// ------------------------

void Q3dWindow::setColUser()
{
	
	colourAuto->setChecked(false);

}

// ------------------------

void Q3dWindow::setLogRX()
{
	// must handle RTh or XY case
	
	if(rmin <= 0) RXAuto->setChecked(true); // ensure negative values get reset accordingly

	if(!validData) return;
	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
}

// ------------------------

void Q3dWindow::setLogY()
{
	// must handle RTh or XY case
	
	if(thetamin <= 0) ThYAuto->setChecked(true); // ensure negative values get reset accordingly

	if(!validData) return;
	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
}
// ------------------------

void Q3dWindow::setLogZ()
{
	// must handle RTh or XY case
	
	if(phimin <= 0) PhiZAuto->setChecked(true); // ensure negative values get reset accordingly

	if(!validData) return;
	
	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
}

// ------------------------

void Q3dWindow::setLogCol()
{
	bool OK;
	double min = colMin->text().toDouble(&OK);
	if( ( !OK || min <= 1.0e-30 ) && colourLog->isChecked() ) 
	{
		colourAuto->setChecked(true); // ensure negative values get reset accordingly

		if(!validData) return;
	
		setData();
		setTitle();
		this->autosetCol();
	}

}

// ------------------------

void Q3dWindow::setAuto()
{	
	if(!validData) return;

	setData();
	setTitle();
	this->autosetCol();
	this->autosetRX();
	this->autosetThY();
	this->autosetPhiZ();
}


// ------------------------

void Q3dWindow::changeRX()
{
	if(!validData) return;
	
	RXAuto->setChecked(false);
	setData();
	setTitle();
	this->autosetCol();
}


// ------------------------

void Q3dWindow::changeThY()
{
	if(!validData) return;
	
	ThYAuto->setChecked(false);
	setData();
	setTitle();
	this->autosetCol();
}

// ------------------------

void Q3dWindow::changePhiZ()
{
	if(!validData) return;
	
	PhiZAuto->setChecked(false);
	setData();
	setTitle();
	this->autosetCol();
}

// ------------------------

void Q3dWindow::setTitle()
{
	if(dobj.is_nil()) return;
	
	setColLab(); // also set default col label
	setAxisLabels();

	timeText = "";
	if( atSlider->record <= atSlider->maxRecord){
		timeText = atSlider->ttags->asStr(atSlider->record);
	}
		
	if(userTitle) return;

	if (dobj->xref_exists(LABLAXIS))
		titleSlot->setText(dobj->getXrefText(LABLAXIS).c_str());
	else if (dobj->xref_exists(FIELDNAM))
		titleSlot->setText(dobj->getXrefText(FIELDNAM).c_str());
	else if (dobj->xref_exists(CATDESC))
		titleSlot->setText(dobj->getXrefText(CATDESC).c_str());
	else if (dobj->xref_exists(OBJECT_NAME))
		titleSlot->setText(dobj->getXrefText(OBJECT_NAME).c_str());


}

void Q3dWindow::setUserTitle()
{
	if(titleSlot->text().isEmpty()){
		userTitle = false;
	}
	else userTitle = true;
}


void Q3dWindow::setUserRXLab()
{
	userRLab = true;

}


void Q3dWindow::setUserThYLab()
{
	userThLab = true;

}

void Q3dWindow::setUserPhiZLab()
{
	userPhiLab = true;

}


void Q3dWindow::setAxisLabels()
{
	if(thetaDep.is_nil()) return;

	if(!userThLab){
		string thetalabel;
		if (thetaDep->xref_exists(LABLAXIS)) {
			thetalabel = string(thetaDep->getXrefText(LABLAXIS));
		} 
		else{
			if(axisType->currentIndex() == 1) thetalabel = string("Angle");
			else if(axisType->currentIndex() == 2) thetalabel = string("Y");
			else thetalabel = string("");
		}
		
		if (thetaDep->xref_exists(UNITS)) {
			thetalabel += " (";
			thetalabel += string(thetaDep->getXrefText(UNITS));
			thetalabel += ")";
		}

		ThYLabSlot->setText(thetalabel.c_str());

	}
	
	if(!userPhiLab){
		string philabel;
		if (phiDep->xref_exists(LABLAXIS)) {
			philabel = string(phiDep->getXrefText(LABLAXIS));
		} 
		else{
			if(axisType->currentIndex() == 1) philabel = string("Angle");
			else if(axisType->currentIndex() == 2) philabel = string("Z");
			else philabel = string("");
		}
		
		if (phiDep->xref_exists(UNITS)) {
			philabel += " (";
			philabel += string(phiDep->getXrefText(UNITS));
			philabel += ")";
		}

		PhiZLabSlot->setText(philabel.c_str());

	}
	
	if(!userRLab)
	{	
		string rlabel;

		if (rDep->xref_exists(LABLAXIS)) {
			rlabel = string(rDep->getXrefText(LABLAXIS));
		}
		else {
			if(axisType->currentIndex() == 1) rlabel = string("r");
			else if(axisType->currentIndex() == 2) rlabel = string("X");
			else rlabel = string("");
		}
		 
		
		if (rDep->xref_exists(UNITS)) {
			rlabel += " (";
			rlabel += string(rDep->getXrefText(UNITS));
			rlabel += ")";
		}

		RXLabSlot->setText(rlabel.c_str());

	}

	
}

void Q3dWindow::setColLab()
{
	if(dobj.is_nil()) return;
	
	if(userColLab) return;

	string zlabel;
	
	if (dobj->xref_exists(LABLAXIS)) {
		zlabel = string(dobj->getXrefText(LABLAXIS));
	}
	else zlabel = string("z");
	
	if (dobj->xref_exists(UNITS)){
		zlabel += " (";
		zlabel += string(dobj->getXrefText(UNITS));
		zlabel += ")";
	}

	colLabSlot->setText(zlabel.c_str());


}

void Q3dWindow::setUserColLab()
{
	userColLab = true;

}

void Q3dWindow::keyPressEvent(QKeyEvent * e )
{  
	if(QuSlotDebug())
	{
		printf("Q3dWindow::keyPressEvent \n");
		fflush(stdout);
	}

	// use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
		if(   plotBtn->isDefault()) {
			plotBtn->setDefault(false);
			animateBtn->setDefault(true);
		}
		else if(animateBtn->isDefault())  {
			animateBtn->setDefault(false);
			runBtn->setDefault(true);
		}
		else if(runBtn->isDefault())  {
			runBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault())  {
			dismissBtn->setDefault(false);
			plotBtn->setDefault(true);
		}
		escPushed = true;
		return;
	}
	
	// use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if(plotBtn->isDefault()) doPlot();
		else if (animateBtn->isDefault()) doAnimate();
		else if (runBtn->isDefault()) doRun();
		else if (dismissBtn->isDefault()) close();
		return;
	}

	// else pass on to list view 
	e->ignore(); // let selection change

}


// --------------------------------------

void Q3dWindow::launch3dviewPlotHelp()
{
	// slot invoked from Plotting Help menu sub item
	// launches browser showing Qsas Plot help docuumentation

	if(QuSlotDebug())
	{
		printf("Q3dWindow::launch3dviewPlotHelp \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_3dview.html");
}

void Q3dWindow::launchManual()
{
	// slot invoked from Qsas Manual Help menu sub item
	// launches browser showing Qsas help index

	if(QuSlotDebug())
	{
		printf("Q3dWindow::launchManual \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_help_index.html");
}

void Q3dWindow::launchVersion()
{
	// slot invoked from Module Version Help menu sub item
	// launches browser showing Qsas version numbers

	if(QuSlotDebug())
	{
		printf("Q3dWindow::launchVersion \n");
		fflush(stdout);
	}

	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n 3D View  ";
	text += QString(Q3D_VERSION);
	QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );
}


QDomElement Q3dWindow::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Tabs");
    
    xml.appendChild(createTextXMLNode(doc, "userTitle", userTitle));
    xml.appendChild(createTextXMLNode(doc, "userRLab", userRLab));
    xml.appendChild(createTextXMLNode(doc, "userThLab", userThLab));
    xml.appendChild(createTextXMLNode(doc, "userPhiLab", userThLab));
    xml.appendChild(createTextXMLNode(doc, "userColLab", userColLab));
    xml.appendChild(createTextXMLNode(doc, "userRmax", userRmax));
    xml.appendChild(createTextXMLNode(doc, "userRmin", userRmin));
    xml.appendChild(createTextXMLNode(doc, "userThmax", userThmax));
    xml.appendChild(createTextXMLNode(doc, "userThmin", userThmin));
    xml.appendChild(createTextXMLNode(doc, "usePhimax", userPhimax));
    xml.appendChild(createTextXMLNode(doc, "usePhimin", userPhimin));


    xml.appendChild(createTextXMLNode(doc, "dataSlot", dataSlot->text()));
	xml.appendChild(createTextXMLNode(doc, "atSlider", atSlider->record));
    xml.appendChild(createTextXMLNode(doc, "setRX", setRX->currentIndex()));
    if(elevation) xml.appendChild(createTextXMLNode(doc, "elevation", elevation->value()));
    if(azimuth) xml.appendChild(createTextXMLNode(doc, "azimuth", azimuth->value()));
    if(delay) xml.appendChild(createTextXMLNode(doc, "delay", delay->value()));

    xml.appendChild(createTextXMLNode(doc, "titleSlot", titleSlot->text()));
    xml.appendChild(createTextXMLNode(doc, "RXLabSlot", RXLabSlot->text()));
    xml.appendChild(createTextXMLNode(doc, "ThYLabSlot", ThYLabSlot->text()));
    xml.appendChild(createTextXMLNode(doc, "PhiZLabSlot", PhiZLabSlot->text()));
	
    xml.appendChild(createTextXMLNode(doc, "plotType", plotType->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "axisType", axisType->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "alphaSpin", alphaSpin->value()));

    xml.appendChild(createTextXMLNode(doc, "symbol_shape", symbol_shape->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "symbol_size", symbol_size->value()));
	xml.appendChild(createTextXMLNode(doc, "beta", beta->value()));

    xml.appendChild(createTextXMLNode(doc, "RXLog", RXLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "RXAuto", RXAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "RXMin", RXMin->text()));
    xml.appendChild(createTextXMLNode(doc, "RXMax", RXMax->text()));
	
    xml.appendChild(createTextXMLNode(doc, "YLog", YLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "ThYAuto", ThYAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "ThYMin", ThYMin->text()));
    xml.appendChild(createTextXMLNode(doc, "ThYMax", ThYMax->text()));

    xml.appendChild(createTextXMLNode(doc, "ZLog", ZLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "PhiZAuto", PhiZAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "PhiZMin", PhiZMin->text()));
    xml.appendChild(createTextXMLNode(doc, "PhiZMax", PhiZMax->text()));

	xml.appendChild(createTextXMLNode(doc, "RXMinLab", RXMinLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "RXMaxLab", RXMaxLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "ThYMinLab", ThYMinLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "ThYMaxLab", ThYMaxLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "PhiZMinLab", PhiZMinLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "PhiZMaxLab", PhiZMaxLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "RaxisLab", RaxisLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "ThAxisLab", ThAxisLab->isEnabled()));
	xml.appendChild(createTextXMLNode(doc, "PhiAxisLab", PhiAxisLab->isEnabled()));


    xml.appendChild(createTextXMLNode(doc, "colourLog", colourLog->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "colourAuto", colourAuto->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "colMin", colMin->text()));
    xml.appendChild(createTextXMLNode(doc, "colMax", colMax->text()));


    ColourTable ct=colourTable->getCurrentTable();

    int _colour_table_type;
    int _colour_table_id;
    if(stdPalette.contains(ct))
    {
        _colour_table_type=0;
        _colour_table_id=stdPalette.indexOf(ct);
    }
    else
    {
        _colour_table_type=1;
        _colour_table_id=savedPalette.indexOf(ct);
    }
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_Type", _colour_table_type));
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_ID", _colour_table_id));


    xml.appendChild(createTextXMLNode(doc, "colLabSlot", colLabSlot->text()));

    xml.appendChild(createTextXMLNode(doc, "page_size", page_size->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "pageUnits", pageUnits->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "dpiPage", dpiPage->currentIndex()));
	
    xml.appendChild(createTextXMLNode(doc, "pageWidth", pageWidth->text()));
    xml.appendChild(createTextXMLNode(doc, "pageHeight", pageHeight->text()));
	
    xml.appendChild(createTextXMLNode(doc, "pageOrientation", pageOrientation->currentIndex()));

    return xml;
}


void Q3dWindow::fromXMLElement(QDomElement& el)
{
    QString s;

	if(getXMLNodeText(el, "userTitle", s))
    {
		userTitle = s.toInt();
    }
	if(getXMLNodeText(el, "userRLab", s))
    {
		userRLab = s.toInt();
    }
	if(getXMLNodeText(el, "userThLab", s))
    {
		userThLab = s.toInt();
    }
	if(getXMLNodeText(el, "userPhiLab", s))
    {
		userPhiLab = s.toInt();
    }
	if(getXMLNodeText(el, "userColLab", s))
    {
		userColLab = s.toInt();
    }

	if(getXMLNodeText(el, "userRmax", s))
    {
		userRmax = s.toDouble();
    }
	if(getXMLNodeText(el, "userRmin", s))
    {
		userRmin = s.toDouble();
    }

	if(getXMLNodeText(el, "userThmax", s))
    {
		userThmax = s.toDouble();
    }
	if(getXMLNodeText(el, "userThmin", s))
    {
		userThmin = s.toDouble();
    }

	if(getXMLNodeText(el, "userPhimax", s))
    {
		userPhimax = s.toDouble();
    }
	if(getXMLNodeText(el, "userPhimin", s))
    {
		userPhimin = s.toDouble();
    }


	if(getXMLNodeText(el, "setRX", s))
    {
        setRX->setCurrentIndex(s.toInt());
    }

	if(getXMLNodeText(el, "titleSlot", s))
    {
        titleSlot->setText(s);
    }
	if(getXMLNodeText(el, "RXLabSlot", s))
    {
        RXLabSlot->setText(s);
    }
	if(getXMLNodeText(el, "ThYLabSlot", s))
    {
        ThYLabSlot->setText(s);
    }
	if(getXMLNodeText(el, "PhiZLabSlot", s))
    {
        PhiZLabSlot->setText(s);
    }


	if(getXMLNodeText(el, "plotType", s))
    {
        plotType->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "axisType", s))
    {
        axisType->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "alphaSpin", s))
    {
        alphaSpin->setValue(s.toDouble());
    }

	if(getXMLNodeText(el, "elevation", s))
    {
        if(elevation){
            elevation->blockSignals(true);
            elevation->setValue(s.toInt());
            elevation->blockSignals(false);
        }
        else startElevation = s.toInt();
    }

	if(getXMLNodeText(el, "azimuth", s))
    {
        if(azimuth){
            azimuth->blockSignals(true);
            azimuth->setValue(s.toInt());
            azimuth->blockSignals(false);
        }
        else startAzimuth = s.toInt();
    }

	if(getXMLNodeText(el, "delay", s))
    {
        if(delay){
            delay->setValue(s.toInt());
        }
    }


	if(getXMLNodeText(el, "symbol_shape", s))
    {
        symbol_shape->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "symbol_size", s))
    {
        symbol_size->setValue(s.toInt());
    }
	if(getXMLNodeText(el, "beta", s))
    {
        beta->setValue(s.toDouble());
    }
	

	if(getXMLNodeText(el, "RXLog", s))
    {
        RXLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "RXAuto", s))
    {
        RXAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "RXMin", s))
    {
        RXMin->setText(s);
    }
	if(getXMLNodeText(el, "RXMax", s))
    {
        RXMax->setText(s);
    }
	
	if(getXMLNodeText(el, "YLog", s))
    {
        YLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "ThYAuto", s))
    {
        ThYAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "ThYMin", s))
    {
        ThYMin->setText(s);
    }
	if(getXMLNodeText(el, "ThYMax", s))
    {
        ThYMax->setText(s);
    }
	
	if(getXMLNodeText(el, "ZLog", s))
    {
        ZLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "PhiZAuto", s))
    {
        PhiZAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "PhiZMin", s))
    {
       PhiZMin->setText(s);
    }
	if(getXMLNodeText(el, "PhiZMax", s))
    {
        PhiZMax->setText(s);
    }
	
	if(getXMLNodeText(el, "RXMinLab", s))
    {
        RXMinLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "RXMaxLab", s))
    {
        RXMaxLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "ThYMinLab", s))
    {
        ThYMinLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "ThYMaxLab", s))

	if(getXMLNodeText(el, "PhiZMinLab", s))
    {
        PhiZMinLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "PhiZMaxLab", s))
    {
        PhiZMaxLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "RaxisLab", s))
    {
        RaxisLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "ThAxisLab", s))
    {
        ThAxisLab->setEnabled(s.toInt());
    }
	if(getXMLNodeText(el, "PhiAxisLab", s))
    {
        PhiAxisLab->setEnabled(s.toInt());
    }


	if(getXMLNodeText(el, "colourLog", s))
    {
        colourLog->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "colourAuto", s))
    {
        colourAuto->setChecked(s.toInt());
    }
	if(getXMLNodeText(el, "colMin", s))
    {
        colMin->setText(s);
    }
	if(getXMLNodeText(el, "colMax", s))
    {
        colMax->setText(s);
    }


    int _colour_table_type(0);
    int _colour_table(0);
    if(getXMLNodeText(el, "Colour_Table_ID", s))
    {
        _colour_table=s.toInt();
    }
    if(getXMLNodeText(el, "Colour_Table_Type", s))
    {
        _colour_table_type=(s.toInt());
    }
    if(_colour_table_type!=0)
    {
        colourTable->setSessionPalette(_colour_table+importedPaletteOffset);
    }
    else
    {
        colourTable->setStdPalette(_colour_table);
    }


 	if(getXMLNodeText(el, "colLabSlot", s))
    {
        colLabSlot->setText(s);
    }
	
	if(getXMLNodeText(el, "page_size", s))
    {
        page_size->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "pageUnits", s))
    {
        pageUnits->setCurrentIndex(s.toInt());
    }
	if(getXMLNodeText(el, "dpiPage", s))
    {
        dpiPage->setCurrentIndex(s.toInt());
    }	
		
	if(getXMLNodeText(el, "pageWidth", s))
    {
        pageWidth->setText(s);
    }
	if(getXMLNodeText(el, "pageHeight", s))
    {
        pageHeight->setText(s);
    }

		if(getXMLNodeText(el, "pageOrientation", s))
    {
        pageOrientation->setCurrentIndex(s.toInt());
    }



	if(getXMLNodeText(el, "dataSlot", s))
    {
        dataSlot->setText(s);
		dataSlot->reValidate(true);
    }
	
	if(getXMLNodeText(el, "atSlider", s))
    {
        if(atSlider){
            atSlider->blockSignals(true);
            atSlider->adjustRec(s.toInt());
            atSlider->blockSignals(false);
        }
        else startSlider = s.toInt();
   }

}

void Q3dWindow::saveAsXML(QString path)
{
 
        ofstream file;
        file.open(path.toLatin1().data(),ios::out);

        if(file.is_open())
        {
            QDomDocument doc("QSAS_3dview_plot_save");

            QDomElement root=doc.createElement("QSAS_3dview_Plot");
            doc.appendChild(root);
            string guiID;
            guiID=QSAS_VERSION_ID;

            QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
            root.appendChild(version);

            root.appendChild(this->toXMLElement(doc));
            
            QString xml = doc.toString();
            file << xml.toLatin1().data();
            file.close();
        }
}
void Q3dWindow::saveAsXML()
{
   QString path=QFileDialog::getSaveFileName(this, "Save as...", QString("saveSet"), "QSAS 3D view save (*.qx_3dview_plot);; XML files (*.xml)");
    if(!path.isEmpty())
    {
		if(!path.contains(".qx_3dview_plot") && !path.contains(".xml")){
			path += ".qx_3dview_plot";
		}
		saveAsXML(path);
	}
}
void Q3dWindow::restoreXML()
{
    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS 3d view save (*.qx_3dview_plot);; XML files (*.xml);; All files (*)");
	restoreXML(path);
}

void Q3dWindow::restoreXML(QString path, bool replace)
{
	// restore
	
	if(!path.isEmpty())
    {
        QDomDocument doc;
        QFile myFile(path);
        doc.setContent(&myFile);
        myFile.close();

        QDomElement docElem = doc.documentElement();
        if(!docElem.isNull())
        {

            string guiID="0.0.00";

            QDomElement version=docElem.firstChildElement("QSAS_Version");
            if(!version.isNull())
            {
                guiID=version.firstChild().nodeValue().toLatin1().data();
            }
            setRestoreVersion(guiID.c_str());


            QDomElement tabElement=docElem.firstChildElement("Tabs");

            if(!tabElement.isNull())
            {
               this->fromXMLElement(tabElement);
            }
			return;
        }
    }
	return;
}



// ------------------------

void Q3dWindow::Q3dviewAppendTextDisplay(const char *txt)
{
	textArea->append(QString(txt));
	textArea->update();
	
}


void Q3dWindow::clearTextArea()
{
	textArea->clear();
}

// ------------------------

bool Q3dWindow::getDimensions(int &nx, int &ny, int &nz)
{

	if( dobj.is_nil() || dobj->nDims() != 3) {
		Q3dviewAppendTextDisplay("matrix not 3D");
		return false;
	}
	
	nx = dobj->Dims()[0];
	ny = dobj->Dims()[1];
	nz = dobj->Dims()[2];
	
	return true;

}

// ------------------------

bool Q3dWindow::initData()
{
	clearTextArea();
	
	isPolar = false; // default to cartesian, but reset if angle found

	if( dobj.is_ok()) {
		// data has been set successfully previously
		// free memory used and allocate with dimensions of new data object

		dobj = new DvObject();
		
		if (z != 0) delete[] z;
		if (zLog != 0) delete[] zLog;

		z = 0;
		zLog = 0;
		nr = 0;
		ntheta = 0;
		nphi = 0;
		
		if (r != 0) delete[] r;
		if (r_plus != 0) delete[] r_plus;
		if (r_minus != 0) delete[] r_minus;
		if (theta != 0) delete[] theta;
		if (theta_plus != 0) delete[] theta_plus;
		if (theta_minus != 0) delete[] theta_minus;
		if (phi != 0) delete[] phi;
		if (phi_plus != 0) delete[] phi_plus;
		if (phi_minus != 0) delete[] phi_minus;

		r = 0;
		r_plus = 0;
		r_minus = 0;
		theta = 0;
		theta_plus = 0;
		theta_minus = 0;
		phi = 0;
		phi_plus = 0;
		phi_minus = 0;
		
	}
	
	dobj = dataSlot->getObject();
	
	if(dobj.is_nil()) return true;
		
	// Initialising Data
	
	// set time slider
		
	atSlider->ttags = dobj->getDep0(); // may be nill
	
	if(dobj->seqSize() <= 1)
	{
		atSlider->recBar->setRange(0, 0);
		atSlider->setEnabled(false);
		atSlider->maxRecord = 0;
	}
	else{
		atSlider->recBar->setRange(0, atSlider->ttags->seqSize()-1);
		atSlider->setEnabled(true);
		atSlider->maxRecord = atSlider->ttags->seqSize()-1;
		
	}
	atSlider->recBar->setValue(0);
	
	// Obtaining r and theta from xref
	int nx, ny, nz;
	if( !getDimensions(nx, ny, nz) ) {
		Q3dviewAppendTextDisplay("Input object must be 3D matrix");
		validData = false;
		return true;
	}
	validData = true;
	
	//Make sure that depend 1, 2 and 3 exist. If they don't, add our own arbitary depend.
	if (!(dobj->xref_exists(DEPEND_1))) {
		vector <size_t> dimx(1);
		dimx[0] = nx;
		DvObject_var bin1 = new DvObject(0.0, dimx);

		for (int i = 0; i < nx; i++)
			bin1->dbl(i) = i+1;

		DvString lablaxis("(index)");
		bin1->change_xref(LABLAXIS, lablaxis);
		dobj->change_xref(DEPEND_1, bin1);
	}
	if (!(dobj->xref_exists(DEPEND_2))) {
		//Add our own depend according to the bin number.
		vector <size_t> dimy(1);
		dimy[0] = ny;
		DvObject_var bin2 = new DvObject(0.0, dimy);

		for (int i = 0; i < ny; i++)
			bin2->dbl(i) = i+1;

		DvString lablaxis("(index)");
		bin2->change_xref(LABLAXIS, lablaxis);
		dobj->change_xref(DEPEND_2, bin2);
	}
	if (!(dobj->xref_exists(DEPEND_3))) {
		//Add our own depend according to the bin number.
		vector <size_t> dimz(1);
		dimz[0] = nz;
		DvObject_var bin3 = new DvObject(0.0, dimz);

		for (int i = 0; i < nz; i++)
			bin3->dbl(i) = i+1;

		DvString lablaxis("(index)");
		bin3->change_xref(LABLAXIS, lablaxis);
		dobj->change_xref(DEPEND_3, bin3);
	}


	// find which dimension is theta and get fillvals and units
	rfillVal 		= NAN;
	thetafillVal 	= NAN;
	phifillVal 		= NAN;
	zfillVal 		= NAN;
	
	DvObject_var aObj; // local ptr used for attrs
	
	if(dobj->xref_exists(FILL_VALUE)) 
	{
		aObj = dobj->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) zfillVal = aObj->dbl(0);
	}

	// find depend that is an angle variable and set R and Th depends
	DvObject_var dep1 = dobj->get_xref(DEPEND_1);
	DvObject_var dep2 = dobj->get_xref(DEPEND_2);
	DvObject_var dep3 = dobj->get_xref(DEPEND_3);
	
	if(dep1.is_nil() || dep2.is_nil() || dep3.is_nil()) return true;

	RXindex = 1;
		
	int numAngles=0;

	if( dep1->isAngle()) 
	{
		RXindex = 3;
		numAngles++;
	}
	if(  dep2->isAngle()) 
	{
		numAngles++;
	}
	if(  dep3->isAngle()) 
	{
		numAngles++;
	}
	
	if(numAngles == 2) isPolar = true;
	else RXindex = 1; // reset to ist index for x in cartesian
	
	// Allow user to force other choice (useful for angle vs angle)
	if(setRX->currentIndex() != 0) 
	{
		RXindex = setRX->currentIndex();
	}

	// needed for isPhiAngle call only
	int kStart=0;
	int kEnd=0;
	double min=0.0;
	
	if(RXindex == 1){
		rDep = dep1;
		nr = nx;
		bool depIsDeg = dep2->isDeg();
		if( dep2->isPhiAngle(depIsDeg, 2, min, kStart, kEnd )){
			thetaDep = dep3;
			phiDep = dep2;
			ntheta = nz;
			nphi = ny;
			phiFirst=true;
		}
		else{
			thetaDep = dep2;
			phiDep = dep3;
			ntheta = ny;
			nphi = nz;
			phiFirst=false;
		}
	}
	else if(RXindex == 2){
		rDep = dep2;
		nr = ny;
		bool depIsDeg = dep1->isDeg();
		if( dep1->isPhiAngle(depIsDeg, 1, min, kStart, kEnd )){
			thetaDep = dep3;
			phiDep = dep1;
			ntheta = nz;
			nphi = nx;
			phiFirst=false;
		}
		else{
			thetaDep = dep1;
			phiDep = dep3;
			ntheta = nx;
			nphi = nz;
			phiFirst=true;
		}
	}
	else{
		rDep = dep3;
		nr = nz;
		bool depIsDeg = dep2->isDeg();
		if( dep2->isPhiAngle(depIsDeg, 2, min, kStart, kEnd )){
			thetaDep = dep1;
			phiDep = dep2;
			ntheta = nx;
			nphi = ny;
			phiFirst=false;
		}
		else{
			thetaDep = dep2;
			phiDep = dep1;
			ntheta = ny;
			nphi = nx;
			phiFirst=true;
		}
	}
	
	// find fillval for Theta
	if(thetaDep->xref_exists(FILL_VALUE)) 
	{
		aObj = thetaDep->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) thetafillVal = aObj->dbl();
	}
	
	// find fillval for Phi
	if(phiDep->xref_exists(FILL_VALUE)) 
	{
		aObj = phiDep->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) phifillVal = aObj->dbl();
	}
	
	// find fillval for R
	if(rDep->xref_exists(FILL_VALUE)) 
	{
		aObj = rDep->get_xref(FILL_VALUE);
		if (aObj.is_ok() && aObj->is_dbl()) rfillVal = aObj->dbl();
	}

	r = new PLFLT[nr];
	r_plus = new PLFLT[nr];
	r_minus = new PLFLT[nr];

	theta = new PLFLT[ntheta];
	theta_plus = new PLFLT[ntheta];
	theta_minus = new PLFLT[ntheta];
	
	phi = new PLFLT[nphi];
	phi_plus = new PLFLT[nphi];
	phi_minus = new PLFLT[nphi];


	// n is total number of data points
	ndata = nr * ntheta * nphi;

	z = new PLFLT[ndata];
	zLog = new PLFLT[ndata];
	
	// use explicit axis choice
	if(axisType->currentIndex() == 1){
		 isPolar = true;
		 if(numAngles < 2){
			Q3dviewAppendTextDisplay("Data needs DEPEND_i metadata with SI_Conversion specifying deg or rad");
		}
	}
	else if(axisType->currentIndex() == 2) isPolar = false;
	
	setGUItoPolar(isPolar);
	
	return false; // OK
}


// ------------------------

void Q3dWindow::setData()
{
	if(dobj.is_nil() || !dobj->is_dbl()) return;
	if(thetaDep.is_nil() ) return;
	if(phiDep.is_nil() ) return;
	if(rDep.is_nil() ) return;
	
	if(!validData) return;
	
	zLogEmpty = true;

	// Load dimension data
		
	if(isPolar && thetaDep->isDeg() ) 
	{
		ang2radTh = M_PI / 180.;
		rad2degTh = 1. / ang2radTh;
	}
	else 
	{
		// data in radians or cartesian plots
		ang2radTh = 1.0;
		rad2degTh = 1.0;
	}
	
	if(isPolar && phiDep->isDeg() ) 
	{
		ang2radPhi = M_PI / 180.;
		rad2degPhi = 1. / ang2radPhi;
	}
	else 
	{
		// data in radians or cartesian plots
		ang2radPhi = 1.0;
		rad2degPhi = 1.0;
	}

	// r dimension

	// allow for record varying depends
	int rAtRec = 0;
	if(rDep->seqSize() == dobj->seqSize()) rAtRec = atSlider->record;

	if( nr != rDep->Dims()[0]){
		Q3dviewAppendTextDisplay("data dimension does not match depend for r/X");
		return;
	}
	
	for (int i = 0; i < nr; i++) {
		r[i] = rDep->asDouble( rAtRec * nr + i);
		if (r[i] == rfillVal || !isfinite(r[i]) ) r[i] = 0;
	}

	// theta dimension
	
	// allow for record varying depends
	int thetaAtRec = 0;
	if(thetaDep->seqSize() == dobj->seqSize()) thetaAtRec = atSlider->record;

	if( ntheta != thetaDep->Dims()[0]){
		Q3dviewAppendTextDisplay("data dimension does not match depend for theta/Y");
		return;
	}

	for (int j = 0; j < ntheta; j++) {
		double val = thetaDep->asDouble( thetaAtRec * ntheta + j);
		if ( val == thetafillVal || !isfinite(val)) theta[j] = 0.;
		else theta[j] = val * ang2radTh;
	}
	
	// phi demension
	
	// allow for record varying depends
	int phiAtRec = 0;
	if(phiDep->seqSize() == dobj->seqSize()) phiAtRec = atSlider->record;

	if( nphi != phiDep->Dims()[0]){
		Q3dviewAppendTextDisplay("data dimension does not match depend for phi/Z");
		return;
	}

	for (int k = 0; k < nphi; k++) {
		double val = phiDep->asDouble( phiAtRec * nphi + k);
		if ( val == phifillVal || !isfinite(val)) phi[k] = 0.;
		else phi[k] = val * ang2radPhi;
	}
	
		
	
	//  Get delta plus and minus for theta
	// if deltas don't exist set bin edge to same as point
	
	setDeltaAngle(theta_plus, DELTA_PLUS, theta, ntheta, thetaDep, thetafillVal, ang2radTh);
	setDeltaAngle(theta_minus, DELTA_MINUS, theta, ntheta, thetaDep, thetafillVal, ang2radTh);
	
	//  Get delta plus and minus for phi
	// if deltas don't exist set bin edge to same as point
	
	setDeltaAngle(phi_plus, DELTA_PLUS, phi, nphi, phiDep, phifillVal, ang2radPhi);
	setDeltaAngle(phi_minus, DELTA_MINUS, phi, nphi, phiDep, phifillVal, ang2radPhi);
	
	//  Get delta plus and minus for r
	// if deltas don't exist set bin edge to same as point

	setDeltaRad(r_plus, DELTA_PLUS);
	setDeltaRad(r_minus, DELTA_MINUS);
	
	fullTh = false;
	if (   abs(sin((theta_plus[0] - theta_minus[ntheta - 1]) / 2)) < 0.01
		|| abs(sin((theta_plus[ntheta - 1] - theta_minus[0]) / 2)) < 0.01) {
		
		if(isPolar) fullTh = true;
				
	} 

	fullPhi = false;
	if (   abs(sin((phi_plus[0] - phi_minus[nphi - 1]) / 2)) < 0.01
		|| abs(sin((phi_plus[nphi - 1] - phi_minus[0]) / 2)) < 0.01) {
		
		if(isPolar) fullPhi = true;
				
	} 

	// fill the z data array

	// allow for record varying depends
	int zAtRec = atSlider->record;
	
	int iz;
	for (int i = 0; i < nr; i++) {
		for (int j = 0; j < ntheta; j++) {
			for (int k=0; k < nphi; k++) {
			
				iz = i*ntheta*nphi + j*nphi + k;
				switch(RXindex){
					case 1: default:
						if(phiFirst) z[iz] = dobj->dbl(zAtRec, i, k, j);
						else z[iz] = dobj->dbl(zAtRec, i, j, k);
						break;
					case 2:
						if(phiFirst) z[iz] = dobj->dbl(zAtRec, k, i, j);
						else z[iz] = dobj->dbl(zAtRec, j, i, k);
						break;
					case 3:
						if(phiFirst) z[iz] = dobj->dbl(zAtRec, k, j, i);
						else z[iz] = dobj->dbl(zAtRec, j, k, i );
						break;
				}
				
				if (z[iz] == zfillVal) z[iz] = NAN;
				
			}
		}
	}

	// find min and max values
	rmin = arrmin(r_minus, nr);
	thetamin = arrmin(theta_minus, ntheta);
	phimin = arrmin(phi_minus, nphi);
	rmax = arrmax(r_plus, nr);
	thetamax = arrmax(theta_plus, ntheta);
	phimax = arrmax(phi_plus, nphi);
	
	MinMax3dgrid(z, nr, ntheta, nphi, zmin, zmax, NAN);

	QString txt;
	
	rminNonZero = arrminNonZero(r_minus, nr);

	// remove -ve values if log axis in polar
	if(rmin <= 0 && RXLog->isChecked() ) {
		rmin = rminNonZero;
		
		QString txt = QString("Negative or zero r values: rmin moved to %1").arg(rmin);
		Q3dviewAppendTextDisplay(txt.toLatin1().data());

		for (int i = 0; i < nr; i++) {
			if(r[i] < rmin) r[i] = rmin;
			if(r_plus[i] < rmin) r_plus[i] = 1.1*rmin;
			if(r_minus[i] < rmin) r_minus[i] = rmin;
		}
	}

	
	if( !isPolar && thetamin <= 0 && YLog->isChecked()) {
		
		QString txt = QString("Negative Y values: %1 added to Y positions").arg(thetamin);
		Q3dviewAppendTextDisplay(txt.toLatin1().data());

		for (int i = 0; i < ntheta; i++) {
			theta[i] -= thetamin;
			theta_plus[i] -= thetamin;
			theta_minus[i] -= thetamin;
		}
		thetamax -= thetamin; 
		thetamin = 0.;
	}
	thetaminNonZero  = arrminNonZero(theta_minus, ntheta);
	
	if( !isPolar && phimin <= 0 && ZLog->isChecked()) {
		
		QString txt = QString("Negative Z values: %1 added to Z positions").arg(phimin);
		Q3dviewAppendTextDisplay(txt.toLatin1().data());

		for (int i = 0; i < nphi; i++) {
			phi[i] -= phimin;
			phi_plus[i] -= phimin;
			phi_minus[i] -= phimin;
		}
		phimax -= phimin; 
		phimin = 0.;
	}
	phiminNonZero  = arrminNonZero(phi_minus, nphi);
	
	// set user R Max/Min to use in plots

	if(RXAuto->isChecked()){
		userRmax = rmax;
		if(RXLog->isChecked()) userRmin = rminNonZero;
		else userRmin = rmin;
	}	
	else{
		bool OK;
		double userValue = RXMin->text().toDouble(&OK);
		if(OK) userRmin = (PLFLT) userValue;
		else userRmin = rmin;

		userValue = RXMax->text().toDouble(&OK);
		if(OK) userRmax = (PLFLT) userValue;
		else userRmax = rmax;
	}

	// set user Theta Max/Min to use in plots

	if(ThYAuto->isChecked()){
		userThmax = thetamax;
		if(YLog->isChecked()) userThmin = thetaminNonZero;
		else userThmin = thetamin;
	}	
	else{
		bool OK;
		double userValue = ThYMin->text().toDouble(&OK);
		if(OK) userThmin = (PLFLT) userValue * ang2radTh;
		else userThmin = thetamin;

		userValue = ThYMax->text().toDouble(&OK);
		if(OK) userThmax = (PLFLT) userValue * ang2radTh;
		else userThmax = thetamax;
	}

	// set user Phi Max/Min to use in plots

	if(PhiZAuto->isChecked()){
		userPhimax = phimax;
		if(ZLog->isChecked()) userPhimin = phiminNonZero;
		else userPhimin = phimin;
	}	
	else{
		bool OK;
		double userValue = PhiZMin->text().toDouble(&OK);
		if(OK) userPhimin = (PLFLT) userValue * ang2radPhi;
		else userPhimin = phimin;

		userValue = PhiZMax->text().toDouble(&OK);
		if(OK) userPhimax = (PLFLT) userValue * ang2radPhi;
		else userPhimax = phimax;
	}

	// set start and stop indices for R/X
	
	// move nr_start to above userRmin 
	nr_start = 0;
	for(int nthis=0; nthis<nr-1; nthis++)
	{
		if(r[nthis] >= userRmin ) {
			nr_start = nthis;
			break;
		}
	}
	
	nr_end = nr;
	// move nr_end to below userRmax, but never lower than 2
	for(int nthis=nr; nthis>1; nthis--)
	{
		if(r[nthis-1] <= userRmax ) {
			nr_end = nthis;
			break;
		}
	}

	for (int i = 0; i < nr; i++) {
		// plot starts when r_plus above userRmin, so catch small r and r_minus
		if(r_minus[i] < userRmin) r_minus[i] = userRmin;
	}	
	
	for (int i = 0; i < nr; i++) {
		// plot ends when r_minus above userRmax, so catch large r and r_plus
		if(r_plus[i] >= userRmax) r_plus[i] = userRmax;
	}	
	
	// set start and stop indices for Th/Y

	nth_start = 0;
	// move nth_start to above userThmin 
	for(int nthis=0; nthis<ntheta-1; nthis++)
	{
		if(theta[nthis] >= userThmin ) {
			nth_start = nthis;
			break;
		}
	}
	
	nth_end = ntheta;
	// move nth_end to below userThmax, but never lower than 2
	for(int nthis=ntheta; nthis>1; nthis--)
	{
		if(theta[nthis-1] <= userThmax ) {
			nth_end = nthis;
			break;
		}
	}
	
	for (int i = 0; i < ntheta; i++) {
		// plot starts when theta_plus above userThmin, so catch small theta and theta_minus
		if(theta_minus[i] < userThmin) theta_minus[i] = userThmin;
	}	
	
	for (int i = 0; i < ntheta; i++) {
		// plot ends when theta_minus above userThmax, so catch large theta and theta_plus
		if(theta_plus[i] > userThmax) theta_plus[i] = userThmax;
	}	

	// set start and stop indices for Phi/Z

	nphi_start = 0;
	// move nphi_start to above userPhimin 
	for(int nthis=0; nthis<nphi-1; nthis++)
	{
		if(phi[nthis] >= userPhimin ) {
			nphi_start = nthis;
			break;
		}
	}
	
	nphi_end = nphi;
	// move nphi_end to below userPhimax, but never lower than 2
	for(int nthis=nphi; nthis>1; nthis--)
	{
		if(phi[nthis-1] <= userPhimax ) {
			nphi_end = nthis;
			break;
		}
	}
	
	for (int i = 0; i < nphi; i++) {
		// plot starts when phi_plus above userPhimin, so catch small phi and phi_minus
		if(phi_minus[i] < userPhimin) phi_minus[i] = userPhimin;
	}	
	
	for (int i = 0; i < nphi; i++) {
		// plot ends when phi_minus above userPhimax, so catch large phi and phi_plus
		if(phi_plus[i] > userPhimax) phi_plus[i] = userPhimax;
	}	


	if(isPolar){
	
		if (RXLog->isChecked()) {
			//log graph

			if (userRmin < 1.e-30) {
				txt = QString("Log Radial Min: %1 too small").arg(userRmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
				userRmin = rminNonZero;
				txt = QString("Reset to: %1").arg(userRmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
			}
		
			for (int i = 0; i < nr; i++) {
				r[i] = SaferLog10(r[i] / userRmin);
				r_plus[i] = SaferLog10(r_plus[i] / userRmin);
				r_minus[i] = SaferLog10(r_minus[i] / userRmin);
			}
			rmax = SaferLog10(rmax / userRmin);
			userRmax = SaferLog10(userRmax / userRmin);

		}
	}
	else{
		// Cartesian plot
		
		if (RXLog->isChecked()) {
			//log graph

			if (userRmin < 1.e-30) {
				txt = QString("Log X Min: %1 too small").arg(userRmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
				userRmin = rminNonZero;
				txt = QString("Reset to: %1").arg(userRmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
			}
		
			for (int i = 0; i < nr; i++) {
				r[i] = SaferLog10(r[i]);
				r_plus[i] = SaferLog10(r_plus[i]);
				r_minus[i] = SaferLog10(r_minus[i]);
			}
			rmax = SaferLog10(rmax);
			userRmax = SaferLog10(userRmax);
			userRmin = SaferLog10(userRmin);

		}
		
		if (YLog->isChecked()) {
			//log graph

			if (userThmin < 1.e-30) {
				txt = QString("Log Y Min: %1 too small").arg(userThmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
				userThmin = thetaminNonZero;
				txt = QString("Reset to: %1").arg(userThmin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
			}
		
			for (int i = 0; i < ntheta; i++) {
				theta[i] = SaferLog10(theta[i]);
				theta_plus[i] = SaferLog10(theta_plus[i]);
				theta_minus[i] = SaferLog10(theta_minus[i]);
			}
			thetamax = SaferLog10(thetamax);
			userThmax = SaferLog10(userThmax);
			userThmin = SaferLog10(userThmin);

		}	

		
		if (ZLog->isChecked()) {
			//log graph

			if (userPhimin < 1.e-30) {
				txt = QString("Log Z Min: %1 too small").arg(userPhimin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
				userPhimin = phiminNonZero;
				txt = QString("Reset to: %1").arg(userPhimin);
				Q3dviewAppendTextDisplay(txt.toLatin1().data());
			}
		
			for (int i = 0; i < nphi; i++) {
				phi[i] = SaferLog10(phi[i]);
				phi_plus[i] = SaferLog10(phi_plus[i]);
				phi_minus[i] = SaferLog10(phi_minus[i]);
			}
			phimax = SaferLog10(phimax);
			userPhimax = SaferLog10(userPhimax);
			userPhimin = SaferLog10(userPhimin);

		}	
	}
		

}

// --------------------------------------

void Q3dWindow::setDeltaAngle(PLFLT *result, const char *delta, PLFLT *angle, int nAngle, DvObject_var &angleDep, double fillVal, double ang2rad)
{
	int sign = 1;
	if(strcmp(delta, DELTA_MINUS) == 0) sign = -1;

	bool noDelta = true;
	
	if(isPolar)
	{
		DvObject_var obj_delta = angleDep->get_xref(delta);
		if( obj_delta.is_ok()  ) {
			// if sequence get matrix for this record
			int atRec = 0;
			if(obj_delta->seqSize() > atSlider->record) atRec = atSlider->record;
			
			if (obj_delta->arraySize() == 1) {						//Sometimes the delta_plus is a scalar instead of an array.
				double val = obj_delta->asDouble(atRec);

				if ( val != fillVal){
					noDelta = false;
					for (int j = 0; j < nAngle; j++)  result[j] = angle[j] + (val * sign ) * ang2rad;
				} 
				else{
					Q3dviewAppendTextDisplay("No suitable delta found for Angle");
				}
			}
			else {
				for (int j = 0; j < nAngle; j++) {
					double val = obj_delta->asDouble(atRec * nAngle + j);
					if (val != fillVal){
						noDelta = false;
						result[j] = angle[j] + (val * sign) * ang2rad;
					}
					else result[j] = angle[j];
				}
			}
		}
		else{
			Q3dviewAppendTextDisplay("No delta_plus not found for angle");
		}
	}
	else{

		DvObject_var obj_delta = angleDep->get_xref(delta);
		if( obj_delta.is_ok() ) {
		
			// if sequence get matrix for this record
			int atRec = 0;
			if(obj_delta->seqSize() > atSlider->record) atRec = atSlider->record;
		
			if (obj_delta->arraySize() == 1) {
				// Sometimes delta+ is a scalar instead of an array.
				double val = obj_delta->asDouble(atRec);
				
				if (val != fillVal) { 
					noDelta = false;
					for (int j = 0; j < nAngle; j++)  result[j] = angle[j] + val * sign ;
				} 
				else{
					Q3dviewAppendTextDisplay("No suitable delta_plus not found for r");
				}
			} 
			else {
				for (int j = 0; j < nAngle; j++) {
					double val = obj_delta->asDouble(atRec * nAngle + j);
					if (val != fillVal){
						noDelta = false;
						result[j] = angle[j] + (val * sign);
					}
					else result[j] = angle[j];
				}
			}
		}
		else{
			Q3dviewAppendTextDisplay("No delta_plus not found for angle");
		}
	}
	
	// find if data increasing or decreasing
	//find the first non-fillVal and the last non-fillVal radial bin and compare their values.
	int imin =0;
	while(isnan(angle[imin]) && angle[imin] != fillVal && imin < nAngle) imin++;

	int imax = nAngle-1;
	while(isnan(angle[imax]) && angle[imax] != fillVal && imax >= 0) imax--;

	if(angle[imax] < angle[imin]) sign *= -1;

	
	// get appropriate delta for d_plus or d_minus with increasing or decreasing
	
	if(noDelta && sign == 1){
		for (int j = 0; j < nAngle-1; j++){
			result[j] = 0.5 * (angle[j] + angle[j+1]);
		}
		result[nAngle-1] = angle[nAngle-1];
	}

	if(noDelta && sign == -1){
		result[0] = angle[0];
		for (int j = 1; j < nAngle; j++){
			result[j] = 0.5 * (angle[j] + angle[j-1]);
		}
	}
	
	return;
}

// --------------------------------------

void Q3dWindow::setDeltaRad(PLFLT *result, const char *delta)
{
	bool noDelta = true;

	int sign = 1;
	if(strcmp(delta, DELTA_MINUS) == 0) sign = -1;

	DvObject_var obj_delta = rDep->get_xref(delta);
	if (rDep.is_ok()) {
		
		// if sequence get matrix for this record
		int atRec = 0;
		if(obj_delta->seqSize() > atSlider->record) atRec = atSlider->record;
		
		if (obj_delta->arraySize() == 1) {
		 	//Sometimes delta+ is a scalar instead of an array.
			double val = obj_delta->asDouble(atRec);
			
			if (val != rfillVal) {
				noDelta = false;
				for (int i = 0; i < nr; i++){
					result[i] = sign * val  + r[i];
				}
			} 
			else{
				Q3dviewAppendTextDisplay("No suitable delta_plus not found for r");
			}
		} 
		else {
			noDelta = false;
			for (int i = 0; i < nr; i++) {
				double val = obj_delta->asDouble(atRec * nr + i);
				if (val != rfillVal)
					result[i] = sign * val + r[i];
				else
					result[i] = r[i];
			}
		}
	}
	else{
		Q3dviewAppendTextDisplay("No delta_plus not found for r");
	}
	
	// find if data increasing or decreasing
	//find the first non-fillVal and the last non-fillVal radial bin and compare their values.
	int imin =0;
	while(isnan(r[imin]) && r[imin] != rfillVal && imin < nr) imin++;

	int imax = nr-1;
	while(isnan(r[imax]) && r[imax] != rfillVal && imax >= 0) imax--;

	if(r[imax] < r[imin]) sign *= -1;

	
	// get appropriate delta for d_plus or d_minus with increasing or decreasing
	
	if(noDelta && sign == 1){
		for (int j = 0; j < nr-1; j++){
			result[j] = 0.5 * (r[j] + r[j+1]);
		}
		r[nr-1] = r[nr-1];
	}

	if(noDelta && sign == -1){
		result[0] = r[0];
		for (int j = 1; j < nr; j++){
			result[j] = 0.5 * (r[j] + r[j-1]);
		}
	}

	return;
}
	
void Q3dWindow::stopAnimate()
{
	if(animateBtn->text() == QString(" || ") ) doAnimate(); // call button toggle if user did not already pause it
	
}
	
void Q3dWindow::doAnimate()
{
	static bool animationRunning=false;
 
    doPlot(true); // ensure window exists
	
	if(animationRunning){
		Q3dviewAppendTextDisplay("\n View animation paused");
		animateBtn->setText("  Animate View  ");
		animationRunning = false;
	}
	else{
		
		// set time out until interrupt possible
     	QTimer::singleShot(30000, this, SLOT(stopAnimate()));
		
		Q3dviewAppendTextDisplay("\n Run view animation");
		animateBtn->setText(" || ");
		animationRunning = true;
		
		
		if(delay->value() != 0){
			while(animationRunning){
				QTime wait;
				wait.start();
				while(wait.elapsed() < delay->value()) ; // do nothing
				stepAnimate();
				qApp->processEvents();
			}
		}
		else while(animationRunning) {
			stepAnimate();
			qApp->processEvents();
		}
	}

	animationRunning=false;
	animateBtn->setText("  Animate View  ");

}

void Q3dWindow::stepAnimate()
{

	static int daz=1;
	static int del=1;
	int azNow = azimuth->value();
	int elNow = elevation->value();
		
	// turn slowly at ends of motion
	if(azNow > 85  && daz > 0 ) daz = 0;
	else if(azNow > 85 && daz == 0) daz = -1;
	else if(azNow < -85 && daz < 0) daz = 0;
	else if(azNow < -85 && daz == 0) daz = 1;

	if(elNow > 85  && del > 0) del = 0;
	else if(elNow > 85 && del == 0) del = -1;
	else if(elNow < 5 && del < 0) del = 0;
	else if(elNow < 5 && del == 0) del = 1;

	azNow += daz;
	elNow += del;
	elevation->setValue(elNow);
	azimuth->setValue(azNow);
					
	doPlot(true);
}

// --------------------------------------

void Q3dWindow::doRun()
{
	Q3dviewAppendTextDisplay("\n Time animation");

	escPushed = false;
	static bool Running=false;
    autoPlot=false;


	if(runBtn->text()  == QString(" || ") ){
		qApp->processEvents(); // get rid of multiple clicks now
		Q3dviewAppendTextDisplay("\n Time animation paused");
		runBtn->setText("  Animate Time  ");
		Running = false;
//		this->raise();
	}
	else{
		bool colAutoSet = colourAuto->isChecked();

		runBtn->setText(" || ");
		
		// find max colour
		if(colAutoSet){

			// force fixed col max/min
			Q3dviewAppendTextDisplay("\n Finding Colour range for Time Animation...");
            atSlider->blockSignals(true);
			atSlider->adjustRec(0);
			setRecord(false);
			double colMinVal = zmin;
			double colMaxVal = zmax;
			if(zmin < 1.e-30) zmin = 1.;
			
			for(int nt=1; nt<=atSlider->maxRecord; nt++){
				atSlider->adjustRec(nt);
				setRecord(false);
				if(colMinVal > zmin && zmin > 1.e-30) colMinVal = zmin;
				if(colMaxVal < zmax) colMaxVal = zmax;
				qApp->processEvents();
				if(escPushed) break;

			}
            atSlider->blockSignals(false);
			colMin->setText(QString("%1").arg(colMinVal));
			colMax->setText(QString("%1").arg(colMaxVal));
			colourAuto->setChecked(false);
		}
		
		// start at beginning if at end
		if(atSlider->record >= atSlider->maxRecord ) atSlider->adjustRec(0);
		
		Q3dviewAppendTextDisplay("\n Running time animation");
		Running = true;

		if(delay->value() != 0){
			while(Running ){
				QTime wait;
				wait.start();
				while(wait.elapsed() < delay->value()) ; // do nothing
				
				doPlot(true);
				if(atSlider->record >= atSlider->maxRecord) {
					atSlider->adjustRec(0);
					colourAuto->setChecked(colAutoSet); // reset auto col range
					Q3dviewAppendTextDisplay("\n Animation finished");
					break;
				}
				atSlider->incRec();
				qApp->processEvents();
			}
		}
		else while(Running){
				
			doPlot(true);
			if(atSlider->record >= atSlider->maxRecord ) {
				atSlider->adjustRec(0);
				colourAuto->setChecked(colAutoSet); // reset auto col range
				Q3dviewAppendTextDisplay("\n Animation finished");
				break;
			}
			atSlider->incRec();
			qApp->processEvents();
		}

//		this->raise();
		Running=false;
		runBtn->setText("  Animate Time  ");

	}
}


// --------------------------------------

void Q3dWindow::doPlot(bool forceSameWindow)
{
		
	// slot invoked when Plot push button clicked
  	if(QuSlotDebug())
	{
		printf("Q3dWindow::doPlot \n");
		fflush(stdout);
	}
	
	if(dobj.is_nil()) return;

    autoPlot=true;

	// check that data has not been deleted (only edited_OK() signal resets objects
	if(	dataSlot->reValidate(false /* don't emit edited_OK */) == false) 
	{
		Q3dviewAppendTextDisplay("Data no longer valid\n");
		return;
	}

	if(nr <= 0 || ntheta <= 0 || nphi <= 0){
		Q3dviewAppendTextDisplay("Zero bin dimension\n");
		return;
	}

	// User settings
		
	bool OK;			
	double _zmin = colMin->text().toDouble(&OK);
	if(OK) zmin = (PLFLT) _zmin;
	double _zmax = colMax->text().toDouble(&OK);
	if(OK) zmax = (PLFLT) _zmax;
	
	alpha = (PLFLT) alphaSpin->value();
	
	// Initialize PLplot.
	
	setwindow(forceSameWindow);
    
    // To make sure the new width will be validated:
	// plwid only stores changes in width
	// If no change from the previous plot, the default
	// width is not stored in the device buffer.
	// Thus we force a change, to store it.
	plwid(100);
	pladv(0);
    
	plscol0a (0, 0 , 0 , 0 , 1.0 );
	plcol0(0);
	plschr(QPUI_CHAR_HEIGHT, 1.2*text_size_fld->value());

	plwid(4);
	
	if (colourLog->isChecked()) {
		zmin = SaferLog10(zmin);
		zmax = SaferLog10(zmax);
	
		for (int i = 0; i < ndata; i++) {  			
			zLog[i] = SaferLog10(z[i]);
		}	
	}

	
	// convert int viewing angles to PLFLT
	
	az = (PLFLT) azimuth->value();
	el = (PLFLT) elevation->value();
	
	// convert deg to rad
	el *= M_PI/180.;
	az = atan2(sin(az*M_PI/180.), cos(az*M_PI/180.));	//This forces az to be between -pi to pi radians.
	
	select_vpor(0);
	pllab("", "", timeText.c_str());

	// call axis specific plot 	

	if(isPolar) {
		doPlotPolar();
	}
	else {
		doPlotCartesian();
	}

#ifdef Q_OS_MACX
// fix Mac OS problem with plot not showing until focus change
	
	if(forceSameWindow) return; // avoid flicker

	win->show();
			
#endif


}

void Q3dWindow::doPlotCartesian()
{
	// slot invoked when Plot push button clicked
  	if(QuSlotDebug())
	{
		printf("Q3dWindow::doPlotCartesian \n");
		fflush(stdout);
	}
	
	select_vpor(-1);

	plcol0(0);
	plschr(QPUI_CHAR_HEIGHT, 1.0*title_size_fld->value());

	pllab("", "", titleSlot->text().toLatin1().data());

	plschr(QPUI_CHAR_HEIGHT, 1.0*text_size_fld->value());

	select_vpor(0);

	// set colour table
	colourTable->getCurrentTable().apply(alphaSpin->value());

	if(plotType->currentIndex() != 0){
		Q3dviewAppendTextDisplay("Cartesian plot only available for \"Data Points\" plot type \nSelect other plot type or set Axis to polar");
	}
	else {
		room = 1.2; //how much larger (factor) than the maximum radius of the plot should the 3d box be?
		cartesian_scatterplot();

	}

	drawcolscale();


}

// --------------------------------------

void Q3dWindow::cartesian_scatterplot(){

	char xaxisopt[50], yaxisopt[50], zaxisopt[50];

	if (RXLog->isChecked())
		sprintf(xaxisopt, "bnlstu");
	else
		sprintf(xaxisopt, "bnstu");
	if (YLog->isChecked())
		sprintf(yaxisopt, "bnlstu");
	else
		sprintf(yaxisopt, "bnstu");
	if (ZLog->isChecked())
		sprintf(zaxisopt, "bcdlmnstuv");
	else
		sprintf(zaxisopt, "bcdmnstuv");

	select_vpor(0);
	PLFLT half_boxdia = sqrt(3) / 1.8;
	plwind(-half_boxdia, half_boxdia, -half_boxdia + + 0.4*cos(el), half_boxdia + 0.4*cos(el) );
	plcol0(0);

	plw3d(1., 1., 1., userRmin - (room - 1.) * (userRmax - userRmin)/2., userRmax + (room - 1.) * (userRmax - userRmin)/2.,
			userThmin - (room - 1.) * (userThmax - userThmin)/2., userThmax + (room - 1.) * (userThmax - userThmin)/2.,
			userPhimin - (room-1.)*(userPhimax-userPhimin)/2., userPhimax + (room-1.)*(userPhimax-userPhimin)/2., el*180./M_PI, az*180./M_PI);

	plbox3(xaxisopt, RXLabSlot->text().toLatin1().data(), 0.0, 0, yaxisopt, ThYLabSlot->text().toLatin1().data(), 0.0, 0, zaxisopt, PhiZLabSlot->text().toLatin1().data(), 0.0, 0);

	vector <threevector*> data_pts;	

	for (int i = nr_start; i < nr_end; i++) {
		for (int j = nth_start; j < nth_end; j++) {
			for (int k = nphi_start; k < nphi_end; k++) {
				threevector *tv;
				if (colourLog->isChecked()) {
					PLFLT val = zLog[ i*ntheta*nphi + j*nphi + k];
					if( val < zmin ) continue; 
					if( val > zmax)  val = zmax; 
					tv = new threevector(r[i], theta[j], phi[k], val);
					data_pts.push_back(tv);
				}
				else {
					PLFLT val = z[ i*ntheta*nphi + j*nphi + k];
					if( val < zmin ) continue;
					if( val > zmax) val = zmax;
					tv = new threevector(r[i], theta[j], phi[k], val);
					data_pts.push_back(tv);
				}
			}
		}
	}

	if(data_pts.size() == 0){
		Q3dviewAppendTextDisplay("\n   No data in range");
		return;
	}

	quicksortthreevector(data_pts, 0, data_pts.size()-1, el, az); //sorting the points by depth.

	PLFLT mindepth = get_mappeddepth(data_pts[0]); //getmappeddepth returns the depth of a point when mapped into a 1x1x1 cube.
	PLFLT maxdepth = get_mappeddepth(data_pts[data_pts.size()-1]);
	PLFLT xpts[1], ypts[1], zpts[1];

	PLFLT dotsize = (PLFLT) symbol_size->value();
	dotsize /= maxdepth - mindepth + 0.1*dotsize ;
	PLINT symbol = symbolIndexConversion(symbol_shape->currentIndex()+1); // +1 as none is not not used here

	//Drawing the data points.
	for (int i=data_pts.size()-1; i >= 0; i--) {
		if ((double) data_pts[i]->getf() != zfillVal) {

			plcol1((data_pts[i]->getf() - zmin) / (zmax - zmin));

			//The size of the points gives the observer a sense of depth. ie. points further away are drawn smaller.
			PLFLT width = dotsize * (maxdepth - get_mappeddepth(data_pts[i]));

			plssym(0, width);

			xpts[0] = data_pts[i]->getx();
			ypts[0] = data_pts[i]->gety();
			zpts[0] = data_pts[i]->getz();
			plpoin3(1, xpts, ypts, zpts, symbol);
		}
	}
	
	// clean up
	for(int k=0; k<data_pts.size(); k++){
		delete data_pts[k];
	}
}

// -------------------------------------

void Q3dWindow::polar_scatterplot()
{
	select_vpor(0);
	PLFLT half_boxdia = sqrt(3.) / 1.8;
	plwind(-half_boxdia, half_boxdia, -half_boxdia + 0.5 * cos(el), half_boxdia + 0.5 * cos(el));
	plcol0(0);
	PLFLT rtop = room*userRmax;

	plw3d(1., 1., 1., -rtop, rtop, -rtop, rtop, -rtop, rtop, el*180./M_PI, az*180./M_PI);

	drawaxis();

	drawphi();

	drawtheta(true);

	vector <threevector*> data_pts;	

	//Sometimes theta[] is the polar angle and sometimes it's the latitude. If theta[] spans between positive and negative values we treat it as latitude
	if ((thetamin < 0) ^ (thetamax >= 0)) { 
		for (int i = nr_start; i < nr_end; i++) {
			for (int j = nth_start; j < nth_end; j++) {
				for (int k = nphi_start; k < nphi_end; k++) {
					PLFLT val;
					if (colourLog->isChecked()) {
						val = zLog[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) {
						val = zmax; 
					}
					}
					else {
						val = z[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}

					threevector *tv = new threevector(r[i], fabs(theta[j]), phi[k], val, 'r');
					data_pts.push_back(tv);

				}
			}
		}
	} else {
		for (int i = nr_start; i < nr_end; i++) {
			for (int j = nth_start; j < nth_end; j++) {
				for (int k = nphi_start; k < nphi_end; k++) {
					PLFLT val;
					if (colourLog->isChecked()) {
						val = zLog[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) {
						val = zmax; 
					}
					}
					else {
						val = z[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}

					threevector *tv = new threevector(r[i] * cos(phi[k]) * cos(theta[j]), r[i] * sin(phi[k]) * cos(theta[j]), r[i] * sin(theta[j]), val);
					data_pts.push_back(tv);
				}
			}
		}
	}

	if(data_pts.size() == 0){
		Q3dviewAppendTextDisplay("\n   No data in range");
		return;
	}
	
	quicksortthreevector(data_pts, 0, data_pts.size() - 1, el, az);

	PLFLT mindepth = get_mappeddepth(data_pts[0]);
	PLFLT maxdepth = get_mappeddepth(data_pts[data_pts.size() - 1]);

	PLFLT xpts[1], ypts[1], zpts[1];

	PLFLT dotsize = (PLFLT) symbol_size->value();

	dotsize /= maxdepth - mindepth + 0.1*dotsize ;
	PLINT symbol = symbolIndexConversion(symbol_shape->currentIndex()+1); // +1 as 'none' is not not used here

	for (int i=data_pts.size()-1; i > -1; i--) {
		PLFLT val = data_pts[i]->getf();
		if ((double) val != zfillVal) {
		
			plcol1((val - zmin) / (zmax - zmin));

			PLFLT width = dotsize * (maxdepth - get_mappeddepth(data_pts[i]));
			plssym(0.0, width);

			xpts[0] = data_pts[i]->getx();
			ypts[0] = data_pts[i]->gety();
			zpts[0] = data_pts[i]->getz();

			plpoin3(1, xpts, ypts, zpts, symbol);
		}
	}

	drawtheta(false);
	
	// clean up
	for(int k=0; k<data_pts.size(); k++){
		delete data_pts[k];
	}
	data_pts.clear();
}




// -------------------------------------

void Q3dWindow::polar_wedgeplot()
{
	select_vpor(0);
	PLFLT half_boxdia = sqrt(3.) / 1.8;
	plwind(-half_boxdia, half_boxdia, -half_boxdia + 0.5 * cos(el), half_boxdia + 0.5 * cos(el));
	plcol0(0);

	plw3d(1., 1., 1., -room*userRmax, room*userRmax, -room*userRmax, room*userRmax, -room*userRmax, room*userRmax, el*180./M_PI, az*180./M_PI);

	drawaxis();
	drawphi();
	drawtheta(true);
	
	vector <threevector*> data_pts;	
	vector <threevector*> data_pts_plus;	
	vector <threevector*> data_pts_minus;	

	//Sometimes theta[] is the polar angle and sometimes it's the latitude. If theta[] spans between positive and negative values we treat it as latitude
	if ((thetamin < 0) ^ (thetamax >= 0)) { 
		for (int i = nr_start; i < nr_end; i++) {
			for (int j = nth_start; j < nth_end; j++) {
				for (int k = nphi_start; k < nphi_end; k++) {
					PLFLT val;
					if (colourLog->isChecked()) {
						val = zLog[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}
					else {
						val = z[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}
					threevector *tv = new threevector(r[i], fabs(theta[j]), phi[k], val, 'r');
					data_pts.push_back(tv);
					threevector *tv_p = new threevector(r_plus[i], fabs(theta_plus[j]), phi_plus[k], val, 'r');
					data_pts_plus.push_back(tv_p);
					threevector *tv_m = new threevector(r_minus[i], fabs(theta_minus[j]), phi_minus[k], val, 'r');
					data_pts_minus.push_back(tv_m);

				}
			}
		}
	} 
	else {
		
		for (int i = nr_start; i < nr_end; i++) {
			for (int j = nth_start; j < nth_end; j++) {
				for (int k = nphi_start; k < nphi_end; k++) {
					PLFLT val;
					if (colourLog->isChecked()) {
						val = zLog[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}
					else {
						val = z[ i*ntheta*nphi + j*nphi + k];
						if( val < zmin ) continue;
						if( val > zmax) val = zmax;
					}

					threevector *tv = new threevector(r[i] * cos(phi[k]) * cos(theta[j]), r[i] * sin(phi[k]) * cos(theta[j]), r[i] * sin(theta[j]), val);
					data_pts.push_back(tv);

					threevector *tv_p = new threevector(r_plus[i] * cos(phi_plus[k]) * cos(theta_plus[j]), r_plus[i] * sin(phi_plus[k]) * cos(theta_plus[j]), r_plus[i] * sin(theta_plus[j]), val);
					data_pts_plus.push_back(tv_p);

					threevector *tv_m = new threevector(r_minus[i] * cos(phi_minus[k]) * cos(theta_minus[j]), r_minus[i] * sin(phi_minus[k]) * cos(theta_minus[j]), r_minus[i] * sin(theta_minus[j]), val);
					data_pts_minus.push_back(tv_m);
	
				}
			}
		}
	}

	if(data_pts.size() == 0){
		Q3dviewAppendTextDisplay("\n   No data in range");
		return;
	}

	quicksortthreevector(data_pts, data_pts_plus, data_pts_minus, 0, data_pts.size() - 1, el, az);


	PLFLT xpts[5], ypts[5], zpts[5];

	for (int i=data_pts.size()-1; i > -1; i--) {
		
		PLFLT val = data_pts[i]->getf();
		if ((double) val != zfillVal && !isnan(val) ) {

			plcol1((val - zmin) / (zmax - zmin));

			// Need to solve wrap around problem of phi.
			PLFLT plus_wrap(0);
			PLFLT minus_wrap(0);

			if(data_pts_plus[i]->getphi() - data_pts[i]->getphi() > M_PI) plus_wrap = -2.*M_PI;
			else if(data_pts_plus[i]->getphi() - data_pts[i]->getphi() < -M_PI) plus_wrap = 2.*M_PI;
			if(data_pts_minus[i]->getphi() - data_pts[i]->getphi() > M_PI) minus_wrap = -2.*M_PI;
			else if(data_pts_minus[i]->getphi() - data_pts[i]->getphi() < -M_PI) minus_wrap = 2.*M_PI;
			
			PLFLT wedge = (PLFLT) beta->value();

			if(plotType->currentIndex() == 1){	//constant theta: vary r, delta phi
				PLFLT cos_phiplus = cos(wedge*(data_pts_plus[i]->getphi() + plus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT cos_phiminus = cos(wedge*(data_pts_minus[i]->getphi() + minus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT sin_phiplus = sin(wedge*(data_pts_plus[i]->getphi() + plus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT sin_phiminus = sin(wedge*(data_pts_minus[i]->getphi() + minus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				xpts[0] = data_pts_minus[i]->getr() * sin(data_pts[i]->gettheta()) * cos_phiminus;
				ypts[0] = data_pts_minus[i]->getr() * sin(data_pts[i]->gettheta()) * sin_phiminus;
				zpts[0]	= data_pts_minus[i]->getr() * cos(data_pts[i]->gettheta());

				xpts[1] = data_pts_plus[i]->getr() * sin(data_pts[i]->gettheta()) * cos_phiminus;
				ypts[1] = data_pts_plus[i]->getr() * sin(data_pts[i]->gettheta()) * sin_phiminus;
				zpts[1] = data_pts_plus[i]->getr() * cos(data_pts[i]->gettheta());

				xpts[2] = data_pts_plus[i]->getr() * sin(data_pts[i]->gettheta()) * cos_phiplus;
				ypts[2] = data_pts_plus[i]->getr() * sin(data_pts[i]->gettheta()) * sin_phiplus;
				zpts[2] = data_pts_plus[i]->getr() * cos(data_pts[i]->gettheta());

				xpts[3] = data_pts_minus[i]->getr() * sin(data_pts[i]->gettheta()) * cos_phiplus;
				ypts[3] = data_pts_minus[i]->getr() * sin(data_pts[i]->gettheta()) * sin_phiplus;
				zpts[3] = data_pts_minus[i]->getr() * cos(data_pts[i]->gettheta());
			}
			else if (plotType->currentIndex() == 2){	//constant phi, vary r, delta theta
				PLFLT cos_thetaplus = cos(wedge*data_pts_plus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT cos_thetaminus = cos(wedge*data_pts_minus[i]->gettheta()  + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT sin_thetaplus = sin(wedge*data_pts_plus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT sin_thetaminus = sin(wedge*data_pts_minus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				xpts[0] = data_pts_minus[i]->getr() * sin_thetaplus * cos(data_pts[i]->getphi());
				ypts[0] = data_pts_minus[i]->getr() * sin_thetaplus * sin(data_pts[i]->getphi());
				zpts[0]	= data_pts_minus[i]->getr() * cos_thetaplus;

				xpts[1] = data_pts_plus[i]->getr() * sin_thetaplus * cos(data_pts[i]->getphi());
				ypts[1] = data_pts_plus[i]->getr() * sin_thetaplus * sin(data_pts[i]->getphi());
				zpts[1] = data_pts_plus[i]->getr() * cos_thetaplus;

				xpts[2] = data_pts_plus[i]->getr() * sin_thetaminus * cos(data_pts[i]->getphi());
				ypts[2] = data_pts_plus[i]->getr() * sin_thetaminus * sin(data_pts[i]->getphi());
				zpts[2] = data_pts_plus[i]->getr() * cos_thetaminus;

				xpts[3] = data_pts_minus[i]->getr() * sin_thetaminus * cos(data_pts[i]->getphi());
				ypts[3] = data_pts_minus[i]->getr() * sin_thetaminus * sin(data_pts[i]->getphi());
				zpts[3] = data_pts_minus[i]->getr() * cos_thetaminus;
			}
			else if(plotType->currentIndex() == 3){	//constant r: vary theta, delta phi
				PLFLT cos_phiplus = cos(wedge*(data_pts_plus[i]->getphi() + plus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT cos_phiminus = cos(wedge*(data_pts_minus[i]->getphi() + minus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT sin_phiplus = sin(wedge*(data_pts_plus[i]->getphi() + plus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				PLFLT sin_phiminus = sin(wedge*(data_pts_minus[i]->getphi() + minus_wrap) + (1.-wedge)*data_pts[i]->getphi());
				xpts[0] = data_pts[i]->getr() * sin(data_pts_plus[i]->gettheta()) * cos_phiminus;
				ypts[0] = data_pts[i]->getr() * sin(data_pts_plus[i]->gettheta()) * sin_phiminus;
				zpts[0]	= data_pts[i]->getr() * cos(data_pts_plus[i]->gettheta());

				xpts[1] = data_pts[i]->getr() * sin(data_pts_minus[i]->gettheta()) * cos_phiminus;
				ypts[1] = data_pts[i]->getr() * sin(data_pts_minus[i]->gettheta()) * sin_phiminus;
				zpts[1] = data_pts[i]->getr() * cos(data_pts_minus[i]->gettheta());

				xpts[2] = data_pts[i]->getr() * sin(data_pts_minus[i]->gettheta()) * cos_phiplus;
				ypts[2] = data_pts[i]->getr() * sin(data_pts_minus[i]->gettheta()) * sin_phiplus;
				zpts[2] = data_pts[i]->getr() * cos(data_pts_minus[i]->gettheta());

				xpts[3] = data_pts[i]->getr() * sin(data_pts_plus[i]->gettheta()) * cos_phiplus;
				ypts[3] = data_pts[i]->getr() * sin(data_pts_plus[i]->gettheta()) * sin_phiplus;
				zpts[3] = data_pts[i]->getr() * cos(data_pts_plus[i]->gettheta());
			}
			else if (plotType->currentIndex() == 4){	//constant phi, vary theta, delta r
				PLFLT rplus = wedge*data_pts_plus[i]->getr() + (1.-wedge)*data_pts[i]->getr();
				PLFLT rminus = wedge*data_pts_minus[i]->getr() + (1.-wedge)*data_pts[i]->getr();
				xpts[0] = rminus * sin(data_pts_plus[i]->gettheta()) * cos(data_pts[i]->getphi());
				ypts[0] = rminus * sin(data_pts_plus[i]->gettheta()) * sin(data_pts[i]->getphi());
				zpts[0]	= rminus * cos(data_pts_plus[i]->gettheta());

				xpts[1] = rplus * sin(data_pts_plus[i]->gettheta()) * cos(data_pts[i]->getphi());
				ypts[1] = rplus * sin(data_pts_plus[i]->gettheta()) * sin(data_pts[i]->getphi());
				zpts[1] = rplus * cos(data_pts_plus[i]->gettheta());

				xpts[2] = rplus * sin(data_pts_minus[i]->gettheta()) * cos(data_pts[i]->getphi());
				ypts[2] = rplus * sin(data_pts_minus[i]->gettheta()) * sin(data_pts[i]->getphi());
				zpts[2] = rplus * cos(data_pts_minus[i]->gettheta());

				xpts[3] = rminus * sin(data_pts_minus[i]->gettheta()) * cos(data_pts[i]->getphi());
				ypts[3] = rminus * sin(data_pts_minus[i]->gettheta()) * sin(data_pts[i]->getphi());
				zpts[3] = rminus * cos(data_pts_minus[i]->gettheta());
			}
			else if(plotType->currentIndex() == 5){	//constant r: vary phi, delta theta
				PLFLT cos_thetaplus = cos(wedge*data_pts_plus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT cos_thetaminus = cos(wedge*data_pts_minus[i]->gettheta()  + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT sin_thetaplus = sin(wedge*data_pts_plus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				PLFLT sin_thetaminus = sin(wedge*data_pts_minus[i]->gettheta() + (1.-wedge)*data_pts[i]->gettheta());
				xpts[0] = data_pts[i]->getr() * sin_thetaplus * cos(data_pts_minus[i]->getphi());
				ypts[0] = data_pts[i]->getr() * sin_thetaplus * sin(data_pts_minus[i]->getphi());
				zpts[0]	= data_pts[i]->getr() * cos_thetaplus;

				xpts[1] = data_pts[i]->getr() * sin_thetaminus * cos(data_pts_minus[i]->getphi());
				ypts[1] = data_pts[i]->getr() * sin_thetaminus * sin(data_pts_minus[i]->getphi());
				zpts[1] = data_pts[i]->getr() * cos_thetaminus;

				xpts[2] = data_pts[i]->getr() * sin_thetaminus * cos(data_pts_plus[i]->getphi());
				ypts[2] = data_pts[i]->getr() * sin_thetaminus * sin(data_pts_plus[i]->getphi());
				zpts[2] = data_pts[i]->getr() * cos_thetaminus;

				xpts[3] = data_pts[i]->getr() * sin_thetaplus * cos(data_pts_plus[i]->getphi());
				ypts[3] = data_pts[i]->getr() * sin_thetaplus * sin(data_pts_plus[i]->getphi());
				zpts[3] = data_pts[i]->getr() * cos_thetaplus;
			}
			else if(plotType->currentIndex() == 6){	//constant theta: vary phi, delta r
				PLFLT rplus = wedge*data_pts_plus[i]->getr() + (1.-wedge)*data_pts[i]->getr();
				PLFLT rminus = wedge*data_pts_minus[i]->getr() + (1.-wedge)*data_pts[i]->getr();
				xpts[0] = rminus * sin(data_pts[i]->gettheta()) * cos(data_pts_minus[i]->getphi());
				ypts[0] = rminus * sin(data_pts[i]->gettheta()) * sin(data_pts_minus[i]->getphi());
				zpts[0]	= rminus * cos(data_pts[i]->gettheta());

				xpts[1] = rplus * sin(data_pts[i]->gettheta()) * cos(data_pts_minus[i]->getphi());
				ypts[1] = rplus * sin(data_pts[i]->gettheta()) * sin(data_pts_minus[i]->getphi());
				zpts[1] = rplus * cos(data_pts[i]->gettheta());

				xpts[2] = rplus * sin(data_pts[i]->gettheta()) * cos(data_pts_plus[i]->getphi());
				ypts[2] = rplus * sin(data_pts[i]->gettheta()) * sin(data_pts_plus[i]->getphi());
				zpts[2] = rplus * cos(data_pts[i]->gettheta());

				xpts[3] = rminus * sin(data_pts[i]->gettheta()) * cos(data_pts_plus[i]->getphi());
				ypts[3] = rminus * sin(data_pts[i]->gettheta()) * sin(data_pts_plus[i]->getphi());
				zpts[3] = rminus * cos(data_pts[i]->gettheta());
			}

			xpts[4] = xpts[0];
			ypts[4] = ypts[0];
			zpts[4] = zpts[0];

			plfill3(4, xpts, ypts, zpts);	
			
		}
	}
	
	drawtheta(false);

	
	// clean up
	for(int k=0; k<data_pts.size(); k++){
		delete data_pts[k];
		delete data_pts_plus[k];
		delete data_pts_minus[k];
	}
}


// -------------------------------------

// Draws the axis. Note that depending on the angle el and az, different lines are drawn.
void Q3dWindow::drawaxis() {
	
	PLFLT rtop = room*userRmax;
	plschr(0., 0.8*text_size_fld->value());
	if (RXLog->isChecked()) {
		plbox3("bu", RXLabSlot->text().toLatin1().data(), 0.0, 0, "bu", RXLabSlot->text().toLatin1().data(), 0.0, 0, "bcu",
				RXLabSlot->text().toLatin1().data(), 0.0, 0);

		vector<PLFLT> ticks;
		logticks(SaferLog10(userRmin), rtop + SaferLog10(userRmin), ticks);

		PLFLT xline[3], yline[3], zline[3];
		PLFLT xtick[2], ytick[2], ztick[2];


		char tick_val[50];
		PLFLT pos;

		ztick[0] = ztick[1] = -rtop;
		
			
        plwid(QtWidth2PlWidth(axis_width_fld->value()));


		if (sin(el) > .001 ) {
			//Draw x axis
			if (az < M_PI / 2. && az > -M_PI / 2.) {
				//draw axis at minimum y
				ytick[0] = -rtop;

				//now drawing the ticks
				for (int i = 0; i < ticks.size(); i++) {
					if (ticks[i] > SaferLog10(userRmin)) {
						xtick[0] = xtick[1] = ticks[i] - SaferLog10(userRmin);
						ytick[1] = - rtop * 0.97;

						if (ticks[i] == (int) ticks[i]) {
							pos = (xtick[0] + rtop) / (2. * rtop);
							ytick[1] = -rtop * 0.95;
							sprintf(tick_val, "10#u%d", (int) ticks[i]);
							plmtex3("xp", 1.5, pos, 0.2, tick_val);

							if (i != 0) {
								pos = (rtop - xtick[0]) / (2 * rtop);
								plmtex3("xp", 1.5, pos, 0.2, tick_val);
							}
						}

						plline3(2, xtick, ytick, ztick);

						xtick[0] = xtick[1] = -ticks[i] + SaferLog10(userRmin);
						plline3(2, xtick, ytick, ztick);
					}
				}
			} else if (az > M_PI / 2. || az < -M_PI / 2.) {
				//draw axis at maximum y
				ytick[0] = rtop;

				//now drawing the ticks
				for (int i = 0; i < ticks.size(); i++) {
					if (ticks[i] > SaferLog10(userRmin)) {
						xtick[0] = xtick[1] = ticks[i] - SaferLog10(userRmin);
						ytick[1] = rtop * 0.97;

						if (ticks[i] == (int) ticks[i]) {
							pos = (xtick[0] + rtop) / (2. * rtop);
							ytick[1] = rtop * 0.95;
							sprintf(tick_val, "10#u%d", (int) ticks[i]);
							plmtex3("xs", 1.5, pos, 0.2, tick_val);

							if (i != 0) {
								pos = (-xtick[0] + rtop) / (2. * rtop);
								plmtex3("xs", 1.5, pos, 0.2, tick_val);
							}
						}

						plline3(2, xtick, ytick, ztick);

						xtick[0] = xtick[1] = -ticks[i] + SaferLog10(userRmin);
						plline3(2, xtick, ytick, ztick);
					}
				}
			}

			//Draw y axis
			if (az > 0 && az < M_PI) {
				//draw axis at minimum x
				xtick[0] = -rtop;

				//now drawing the ticks
				for (int i = 0; i < ticks.size(); i++) {
					if (ticks[i] > SaferLog10(userRmin)) {
						ytick[0] = ytick[1] = ticks[i] - SaferLog10(userRmin);
						xtick[1] = -rtop * 0.97;

						if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
							pos = (ytick[0] + rtop) / (2. * rtop);
							xtick[1] = -rtop * 0.95;
							sprintf(tick_val, "10#u%d", (int) ticks[i]);
							plmtex3("yp", 1.5, pos, 0.2, tick_val);

							if (i != 0) {
								pos = (-ytick[0] + rtop) / (2. * rtop);
								plmtex3("yp", 1.5, pos, 0.2, tick_val);
							}
						}

						plline3(2, xtick, ytick, ztick);

						ytick[0] = ytick[1] = -ticks[i] + SaferLog10(userRmin);
						plline3(2, xtick, ytick, ztick);
					}
				}
			} else if (az < 0 && az > -M_PI) {
				//draw axis at maximum x
				xtick[0] = rtop;

				for (int i = 0; i < ticks.size(); i++) {
					if (ticks[i] > SaferLog10(userRmin)) {
						ytick[0] = ytick[1] = ticks[i] - SaferLog10(userRmin);
						xtick[1] = rtop * 0.97;

						if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
							pos = (ytick[0] + rtop) / (2. * rtop);
							xtick[1] = rtop * 0.95;
							sprintf(tick_val, "10#u%d", (int) ticks[i]);
							plmtex3("ys", 1.5, pos, 0.2, tick_val);

							if (i != 0) {
								pos = (-ytick[0] + rtop) / (2. * rtop);
								plmtex3("ys", 1.5, pos, 0.2, tick_val);
							}
						}

						plline3(2, xtick, ytick, ztick);

						ytick[0] = ytick[1] = -ticks[i] + SaferLog10(userRmin);
						plline3(2, xtick, ytick, ztick);
					}
				}
			}
		}

		//Draw z axis
/*		if (cos(el) != 0) {
			//draw axis at xmin, ymin
			xline[0] = xline[1] = -rtop;
			yline[0] = yline[1] = -rtop;
			zline[0] = -rtop;
			zline[1] = rtop;
			if (!(az > 0 && az <= M_PI / 2.) )
				plline3(2, xline, yline, zline);

			//draw axis at xmin, ymax
			xline[0] = xline[1] = -rtop;
			yline[0] = yline[1] = rtop;
			zline[0] = -rtop;
			zline[1] = rtop;
			if (!(az > M_PI / 2. && az <= M_PI) )
				plline3(2, xline, yline, zline); 

			//draw axis at xmax, ymmin
			xline[0] = xline[1] = rtop;
			yline[0] = yline[1] = rtop;
			zline[0] = -rtop;
			zline[1] = rtop;
			if (!(az <= -M_PI / 2.) )
				plline3(2, xline, yline, zline);

			//draw axis at xmax, ymin
			xline[0] = xline[1] = rtop;
			yline[0] = yline[1] = -rtop;
			zline[0] = -rtop;
			zline[1] = rtop;
			if (!(az >= -M_PI / 2. && az <= 0) )
				plline3(2, xline, yline, zline);
		}
		*/
		//Draw z axis ticks
		if (cos(el) != 0) {
			if (!(az > 0 && az < M_PI / 2.)) { 
				//draw axis at xmin, ymin

				xtick[0] = -fabs(rtop);
				ytick[0] = -fabs(rtop);

				//now drawing the ticks
				if ((az <= 0 && az > -M_PI / 2.) || (az >= M_PI / 2. && az < M_PI))
					for (int i = 0; i < ticks.size(); i++) {
						if (ticks[i] > SaferLog10(userRmin)) {
							ztick[0] = ztick[1] = ticks[i] - SaferLog10(userRmin);

							if (az >= M_PI / 2. && az < M_PI)
								xtick[1] = xtick[0];
							else
								xtick[1] = xtick[0] * 0.97;

							if (az <= 0 && az > -M_PI / 2.)
								ytick[1] = ytick[0];
							else
								ytick[1] = ytick[0] * 0.97;

							if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
								pos = (ztick[0] + rtop) / (2. * rtop);

								if (az >= M_PI / 2. && az < M_PI)
									xtick[1] = xtick[0];
								else
									xtick[1] = xtick[0] * 0.95;
								if (az <= 0 && az > -M_PI / 2.)
									ytick[1] = ytick[0];
								else
									ytick[1] = ytick[0] * 0.95;
							}

							plline3(2, xtick, ytick, ztick);

							ztick[0] = ztick[1] = -ticks[i] + SaferLog10(userRmin);
							plline3(2, xtick, ytick, ztick);
						}
					}
			}

			if (!(az > M_PI / 2. && az < M_PI)) {
				//draw axis at xmin, ymax
				xtick[0] = -fabs(rtop);
				ytick[0] = fabs(rtop);

				//now drawing the ticks
				if ((az > 0 && az <= M_PI / 2.) || (az <= -M_PI / 2.) || az == M_PI)
					for (int i = 0; i < ticks.size(); i++) {
						if (ticks[i] > SaferLog10(userRmin)) {
							ztick[0] = ztick[1] = ticks[i] - SaferLog10(userRmin);

							if (az > 0 && az <= M_PI / 2.)
								xtick[1] = xtick[0];
							else
								xtick[1] = xtick[0] * 0.97;

							if (az <= -M_PI / 2. || az == M_PI)
								ytick[1] = ytick[0];
							else
								ytick[1] = ytick[0] * 0.97;

							if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
								pos = (ztick[0] + rtop) / (2. * rtop);

								if (az > 0 && az <= M_PI / 2.)
									xtick[1] = xtick[0];
								else
									xtick[1] = xtick[0] * 0.95;
								if (az <= -M_PI / 2. || az == M_PI)
									ytick[1] = ytick[0];
								else
									ytick[1] = ytick[0] * 0.95;
							}

							plline3(2, xtick, ytick, ztick);

							ztick[0] = ztick[1] = -ticks[i] + SaferLog10(userRmin);
							plline3(2, xtick, ytick, ztick);
						}
					}
			}

			if (!(az < -M_PI / 2.)) {
				//draw axis at xmax, ymax
				xtick[0] = fabs(rtop);
				ytick[0] = fabs(rtop);

				//now drawing the ticks
				if (az >= M_PI / 2. || (az >= -M_PI / 2. && az < 0))
					for (int i = 0; i < ticks.size(); i++) {
						if (ticks[i] > SaferLog10(userRmin)) {
							ztick[0] = ztick[1] = ticks[i] - SaferLog10(userRmin);

							if (az >= -M_PI / 2. && az < 0)
								xtick[1] = xtick[0];
							else
								xtick[1] = xtick[0] * 0.97;

							if (az >= M_PI / 2.)
								ytick[1] = ytick[0];
							else
								ytick[1] = ytick[0] * 0.97;

							if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
								pos = (ztick[0] + rtop) / (2. * rtop);

								if (az > -M_PI / 2. && az < 0)
									xtick[1] = xtick[0];
								else
									xtick[1] = xtick[0] * 0.95;
								if (az >= M_PI / 2.)
									ytick[1] = ytick[0];
								else
									ytick[1] = ytick[0] * 0.95;
								sprintf(tick_val, "10#u%d", (int) ticks[i]);
								plmtex3("zp", 1.5, pos, 0.2, tick_val);
								plmtex3("zs", 1.5, pos, 0.2, tick_val);

								if (i != 0) {
									pos = (-ztick[0] + rtop) / (2. * rtop);
									plmtex3("zp", 1.5, pos, 0.2, tick_val);
									plmtex3("zs", 1.5, pos, 0.2, tick_val);
								}
							}

							plline3(2, xtick, ytick, ztick);

							ztick[0] = ztick[1] = -ticks[i]	+ SaferLog10(userRmin);
							plline3(2, xtick, ytick, ztick);
						}
					}
			}

			if (!(az > -M_PI / 2. && az < 0)) {
				//draw axis at xmax, ymin
				xtick[0] = fabs(rtop);
				ytick[0] = -fabs(rtop);

				//now drawing the ticks
				if ((az <= -M_PI / 2. && az > -M_PI) || (az >= 0 && az < M_PI / 2.))
					for (int i = 0; i < ticks.size(); i++) {
						if (ticks[i] > SaferLog10(userRmin)) {
							ztick[0] = ztick[1] = ticks[i] - SaferLog10(userRmin);

							if (az <= -M_PI / 2. && az > -M_PI)
								xtick[1] = xtick[0];
							else
								xtick[1] = xtick[0] * 0.97;

							if (az >= 0 && az < M_PI / 2.)
								ytick[1] = ytick[0];
							else
								ytick[1] = ytick[0] * 0.97;

							if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {
								pos = (ztick[0] + rtop) / (2. * rtop);

								if (az <= -M_PI / 2.)
									xtick[1] = xtick[0];
								else
									xtick[1] = xtick[0] * 0.95;
								if (az >= 0 && az <= M_PI / 2.)
									ytick[1] = ytick[0];
								else
									ytick[1] = ytick[0] * 0.95;
								sprintf(tick_val, "10#u%d", (int) ticks[i]);
								plmtex3("zp", 1.5, pos, 0.2, tick_val);
								plmtex3("zs", 1.5, pos, 0.2, tick_val);

								if (i != 0) {
									pos = (-ztick[0] + rtop) / (2. * rtop);
									plmtex3("zp", 1.5, pos, 0.2, tick_val);
									plmtex3("zs", 1.5, pos, 0.2, tick_val);
								}
							}

							plline3(2, xtick, ytick, ztick);

							ztick[0] = ztick[1] = -ticks[i] + SaferLog10(userRmin);
							plline3(2, xtick, ytick, ztick);
						}
					}
			}

			//Putting Numerical values onto z axis
			for (int i = 0; i < ticks.size(); i++) {
				if (ticks[i] > SaferLog10(userRmin)) {
					if (ticks[i] == (int) ticks[i] && i != ticks.size() - 2) {

						pos = (ticks[i] - SaferLog10(userRmin) + rtop) / (2. * rtop);

						sprintf(tick_val, "10#u%d", (int) ticks[i]);
						plmtex3("zp", 1.5, pos, 0.2, tick_val);
						plmtex3("zs", 1.5, pos, 0.2, tick_val);

						if (i != 0) {
							pos = (-ticks[i] + SaferLog10(userRmin) + rtop) / (2. * rtop);
							plmtex3("zp", 1.5, pos, 0.2, tick_val);
							plmtex3("zs", 1.5, pos, 0.2, tick_val);
						}
					}
				}
			}
		}

		//Drawing mid lines
		if (sin(el) != 0) {
			if (el >= 0 && el <= M_PI / 2.) {
				zline[0] = zline[1] = -rtop;
				xline[0] = xline[1] = 0;
				yline[0] = -rtop;
				yline[1] = rtop;

				plline3(2, xline, yline, zline);
				plline3(2, yline, xline, zline);
			}
		}

	} 
	else
		plbox3("bnstu", RXLabSlot->text().toLatin1().data(), 0.0, 0, "abnstu", RXLabSlot->text().toLatin1().data(), 0.0, 0, "bcdmnstuv", RXLabSlot->text().toLatin1().data(), 0.0, 0);
}

// -------------------------------------

//Draws phi angles on the base of the box. Draws on the top of the box if angle of elevation < 0.
void Q3dWindow::drawphi() {
	PLFLT rtop = room*userRmax;
	plschr(0, 0.8*text_size_fld->value());

	if (sin(el) != 0) {
		plcol0(0);
		char angle[10];

		PLFLT height((el >= 0 && el < M_PI) ? -rtop : rtop);

		for (int j = 0; j < 12; j++) {
			double omega = 30. * j;
			sprintf(angle, "%d", (int) omega);
			omega *= M_PI / 180.;
			plptex3(0.7 * rtop * cos(omega), 0.7 * rtop * sin(omega), 
					height, rtop * cos(omega), rtop * sin(omega), 0.,
					-rtop * sin(omega), rtop * cos(omega), 0., 0.0, angle);
		}

		char phichar[] = "   ";
		string philabel = string(PhiZLabSlot->text().toLatin1().data());

		PLFLT domega = M_PI / 36.; //0.5*M_PI / strlen(philabel);
		PLFLT omega = az * M_PI / 180. + M_PI / 2. - philabel.size() * M_PI / 72.;

		plschr(0, 1.0*text_size_fld->value());
		
		for (int i=0; i<philabel.size(); i++) {
			if(philabel[i] == '#'){
        		strncpy(&(phichar[0]), &(philabel[i]), 3);
				i += 2;
			}
			else {
				phichar[0] = philabel[i];
				phichar[1] = '\0';
			}
			plptex3(0.88 * rtop * sin(omega), 0.88 * rtop * cos(omega), height, 
					rtop * cos(omega), -rtop * sin(omega), 0.,
					rtop * sin(omega), rtop * cos(omega), 0, 0.5, phichar);
			omega += domega;
		}
	}
}


// -------------------------------------


//Draws polar angles. This routine should be called two times:
//"bool firstdraw" should be true in the first call so it draws the labels at the back.
// "" false in the second call so it draws labels in the front, ensuring the labels aren't hidden by the data points.
void Q3dWindow::drawtheta(bool firstdraw) {
	//if 0 <= el <= 180, characters above the origin will be drawn in the 2nd call, otherwise characters below the origin will be drawn in the 2nd call.
	if (!(0 <= el && el <= 180))
		firstdraw = !firstdraw;

	// flush plplot no show bug
	plcol0(0);
	PLFLT rtop = room*userRmax;
	PLFLT xline[2], yline[2], zline[2];
	zline[0] = zline[1] = -rtop;
	xline[0] = xline[1] = -rtop;
	yline[0] = -rtop;
	yline[1] = -rtop*0.999;
	plline3(2, xline, yline, zline);
	 
	plcol0(8);
	// do theta	
	plschr(0, 0.8*text_size_fld->value());
	
	if (fabs(cos(el)) > 0.02) {
		char angle[10];

		PLFLT height((el >= 0 && el < M_PI) ? -room*zmax : room*zmax);

		PLFLT phi_az = M_PI - az;

		for (int j = 0; j <= 6; j++) {
			sprintf(angle, "%d", (int) j * 30);
			double omega = M_PI *j / 6.;

			if ((cos(omega) >= 0) ^ firstdraw)
				plptex3(0.85 * rtop * cos(phi_az) * sin(omega), 0.85 * rtop * sin(phi_az) * sin(omega), 0.85 * rtop * cos(omega), 
						-rtop * cos(phi_az) * sin(omega), -rtop * sin(phi_az) * sin(omega), -rtop * cos(omega), 
						-rtop * cos(phi_az) * cos(omega), -rtop * sin(phi_az) * cos(omega), rtop * sin(omega), 1.0, angle);
		}

		char thetachar[] = "   ";
		string thetalabel = string(ThYLabSlot->text().toLatin1().data());

		double domega = M_PI / 36.; //0.5*M_PI / strlen(thetalabel);
		double omega = M_PI / 2. + thetalabel.size() * M_PI / 72.;

	plschr(0, 1.0*text_size_fld->value());


		for (int i=0; i<thetalabel.size(); i++) {
			
			if(thetalabel[i] == '#'){
        		strncpy(&(thetachar[0]), &(thetalabel[i]), 3);
				i += 2;
			}
			else {
        		thetachar[0] = thetalabel[i];
				thetachar[1] = '\0';
			}
			if ((cos(omega) >= 0) ^ firstdraw)
				plptex3(rtop * cos(phi_az) * sin(omega), rtop * sin(phi_az) * sin(omega), rtop * cos(omega), 
						-rtop * cos(phi_az) * cos(omega), -rtop * sin(phi_az) * cos(omega), rtop * sin(omega), 
						rtop * cos(phi_az) * sin(omega), rtop * sin(phi_az) * sin(omega), rtop * cos(omega), 
						0.5, thetachar);
			omega -= domega;
		}
	}
	plcol0(0);

}


// --------------------------------------

void Q3dWindow::doPlotPolar()
{

	// slot invoked when Plot push button clicked
  	if(QuSlotDebug())
	{
		printf("Q3dWindow::doPlotPolar \n");
		fflush(stdout);
	}
	
	select_vpor(-1);

	plcol0(0);
	plschr(QPUI_CHAR_HEIGHT, 1.0*title_size_fld->value());

	pllab("", "", titleSlot->text().toLatin1().data());

	plschr(QPUI_CHAR_HEIGHT, 1.0*text_size_fld->value());

	select_vpor(0);

	// set colour table
	colourTable->getCurrentTable().apply(alphaSpin->value());

	if(plotType->currentIndex() == 0 ){
		room = 1.3; //how much larger (factor) than the maximum radius of the plot should the 3d box be?
		polar_scatterplot();
	}
	else {
		room = 1.25; //how much larger (factor) than the maximum radius of the plot should the 3d box be?
		polar_wedgeplot();
	}

	drawcolscale();


}


//----------------------------

//Sets up page and window.
void Q3dWindow::setwindow(bool forceSameWindow) {

	bool landscape;
    bool getNewWindow = ((plotWin_id->currentWindow == NULL )|| (newWindow->isChecked() && !forceSameWindow) );

    QPrinter::PageSize pageSize;
    switch(page_size->currentIndex())
    {
        case QPUI_A4:  default: case QPUI_DEFAULT:
        {
            pageSize = QPrinter::A4;
            break;
        }
        case QPUI_CUSTOM:
        {
            pageSize = QPrinter::Custom;
            break;
        }
        case QPUI_EXECUTIVE:
        {
            pageSize = QPrinter::Executive;
            break;
        }
        case QPUI_LEGAL:
        {
            pageSize = QPrinter::Legal;
            break;
        }
        case QPUI_LETTER:
        {
            pageSize = QPrinter::Letter;
        }
            
    }
    
    QPrinter::Orientation orientation;
	if(pageOrientation->currentIndex() == 0) {
		landscape = false;
		orientation = QPrinter::Portrait;
	}
	else {
		landscape = true;
		orientation = QPrinter::Landscape;
	}

    bool OK;
    double slotValue = pageWidth->text().toDouble(&OK);
    if(OK) width = slotValue;
    
    slotValue = pageHeight->text().toDouble(&OK);
    if(OK) height = slotValue;
    
	if(  !getNewWindow )
	{	
        // pick up existing window
        win = plotWin_id->focusWindow(width, height, DPI, pageSize, orientation, true);
        return;
	}
	
	

	if(width == height) width+=1;		// There seems to be a bug when saving jpgs with the same width and height, the entire jpg turns black. KaHo

    win = plotWin_id->newPlotWindow(width, height, DPI, pageSize, orientation, true);

	win->setSmooth(true);
	if(forceSameWindow) win->move(0,0);

    elevation = win->addSlider(Qt::Vertical, 0, 90, 1);
    elevation->setValue(startElevation);
    azimuth = win->addSlider(Qt::Horizontal, -180, 180, 1);
    azimuth->setValue(startAzimuth);
    connect(elevation, SIGNAL(valueChanged(int)), this, SLOT(changeView()) );
    connect(azimuth, SIGNAL(valueChanged(int)), this, SLOT(changeView()) );

    
    QLabel* delayLabel = new QLabel("Animation delay: ");
    win->addWidget(delayLabel, 0, 0, Qt::AlignRight);
    
    delay = new QSpinBox();
    delay->setMinimum(0);
    delay->setMaximum(500);
    delay->setValue(0);
    delay->setSingleStep(10);
    delay->setToolTip("Delay in msec between plots in animations");
    win->addWidget(delay, 0, 1, Qt::AlignLeft);
 
    QLabel *space1 = new QLabel("    ");
    win->addWidget(space1, 0, 2, Qt::AlignCenter);
    
    animateBtn = new QPushButton("  Animate View  ");
    animateBtn->setMinimumSize(animateBtn->sizeHint());
    animateBtn->setFocusPolicy(Qt::StrongFocus);
    connect(animateBtn, SIGNAL(clicked()), this, SLOT(doAnimate()) );
    win->addWidget(animateBtn, 0, 3, Qt::AlignCenter);
    
    QLabel *space2 = new QLabel("    ");
    win->addWidget(space2, 0, 4, Qt::AlignCenter);
    
    runBtn = new QPushButton("  Animate Time  ",this);
    runBtn->setMinimumSize(runBtn->sizeHint());
    runBtn->setFocusPolicy(Qt::StrongFocus);
    connect(runBtn, SIGNAL(clicked()), this, SLOT(doRun()));
    win->addWidget(runBtn, 0, 5, Qt::AlignCenter);
    

	pladv(0); // this is done again later, but clears a bug with plwid being ignored

}



// ----------------------

// Sets the page tab feedback
void Q3dWindow::setPage()
{
	bool readonly = true;
	if(page_size->currentIndex() == QPUI_CUSTOM) readonly = false;
	
	switch(dpiPage->currentIndex())
	{
		case QPUI_72_DPI:
		{
			DPI = 72;
			break;
		}
		case QPUI_150_DPI:
		{
			DPI = 150;
			break;
		}
		case QPUI_300_DPI:
		{
			DPI = 300;
			break;
		}
		case QPUI_600_DPI:
		{
			DPI = 600;
			break;
		}
		case QPUI_1200_DPI:
		{
			DPI = 1200;
			break;
		}
	}
	
	switch(pageUnits->currentIndex())
	{
		case QPUI_PIXELS: 
		{
			conv = (double) DPI;
			break;
		}
		case QPUI_INCHES: 
		{
			conv = 1.0;
			break;
		}
		case QPUI_CM: 
		{
			conv = 2.54;
			break;
		}
		case QPUI_MM: 
		{
			conv = 25.4;
			break;
		}
		case QPUI_PSPOINTS: 
		{
			conv = (double) DPI;
			break;
		}
	}


	if(readonly)
	{
		switch(page_size->currentIndex()){
			case QPUI_A4: default:
			{
				width = 8.3 * conv;
				height = 11.694 * conv;
				break;
			}
			case QPUI_EXECUTIVE:
			{
				width = 7.5 * conv;
				height = 10.0 * conv;
				break;
			}
			case QPUI_LEGAL:
			{
				width = 8.5 * conv;
				height = 14 * conv;
				break;
			}
			case QPUI_LETTER:
			{
				width = 8.5 * conv;
				height = 11.0 * conv;
				break;
			}
			case QPUI_DEFAULT:
			{
				width = 8.3 * conv;
				height = 11.694 * conv;
				break;
			}
        }
        pageWidth->setText(QString().setNum(width));
        pageHeight->setText(QString().setNum(height));
		
	}
	pageWidth->setReadOnly(readonly);
	pageHeight->setReadOnly(readonly);
	
	pageWidth->setEnabled(!readonly);
	pageHeight->setEnabled(!readonly);

    for(int i=0; i<plotWin_id->size(); i++){
        plotWin_id->windowAt(i)->setPrintRes(DPI);
    }

}

// ---------------------------


//	near_nice_number() and nice_min() are used to find nice values to draw contour lines.
//	nice_min() rounds down zmin to a nice number. How much it gets rounded down depends on the stepsize of the contour lines.
//	near_nice_number() is used to round stepsize into nice numbers.
//	Hence each contour level sits on (nice min) + integer * (nice number)


//	find a nice number near to x. Nice means 1,2, or 5 times power of 10
//	The Nice number is always larger in magnitude than x
PLFLT Q3dWindow::near_nice_number(PLFLT x) {
	double xlog, r, temp;
	int pow_of_10, pos, inum;

	pos = (x > 0); // for x < 0, need to strip and keep sign info
	xlog = SaferLog10(fabs(x));
	pow_of_10 = (int) xlog; // find nearest power of 10; if < 0, dec by 1


	if (pow_of_10 <= 0)
		pow_of_10--;
	temp = xlog - pow_of_10; // temp is now log of number part only

	if (temp > 1) {
		temp -= 1;
		pow_of_10++;
	} // fudge for vals 0.1-0.9
	r = pow(10.0, temp);
	//   inum = (int) r;
	inum = (int) r;
	if (r / inum > 1.0)
		inum++;
/*	if (inum > 5)
		inum = 10;
	else if (inum > 2)
		inum = 5;
	else if (inum > 1)
		inum = 2;
	else
		inum = 1;*/
	r = inum * pow(10.0, pow_of_10);
	if (!pos)
		r *= -1;

	return (PLFLT) r;

}

PLFLT Q3dWindow::nice_min(PLFLT zmin, PLFLT stepsize) {
	int pos = (zmin > 0);
	PLFLT nice_min;
	int steppow_of_10 = (int)floor(SaferLog10(stepsize));

	int zminpow_of_10 = (int)floor(SaferLog10(abs(zmin)));

	PLFLT tmp = SaferLog10(abs(zmin)) - zminpow_of_10;

	if (zminpow_of_10 - steppow_of_10 != 0) {
		if (pos)
			nice_min = floor(pow(10.0, tmp - steppow_of_10 + zminpow_of_10 - 1));
		else
			nice_min = ceil(pow(10.0, tmp - steppow_of_10 + zminpow_of_10 - 1));
		nice_min *= pow(10.0, steppow_of_10 + 1);

	} else //if the step size is close to the value of zmin don't round down zmin too much.
	{
		if (pos)
			nice_min = floor(pow(10.0, tmp - steppow_of_10 + zminpow_of_10));
		else
			nice_min = ceil(pow(10.0, tmp - steppow_of_10 + zminpow_of_10));
		nice_min *= pow(10.0, steppow_of_10);
	}

	if (!pos)
		nice_min *= -1;

	return nice_min;
}




//Draws the colourscale. This is achived by drawing 2000 lines in a window so it looks continuous.
void Q3dWindow::drawcolscale() {

	bool OK;

	plwid(QtWidth2PlWidth(axis_width_fld->value()));

	select_vpor(5);

	int nsteps = 200;
	if (pageOrientation->currentIndex() == 1) {
		
		// Landscape
		plwind(151.0, 161.0, zmin, zmax);
		PLFLT x[4], y[4];
	
		x[0]=151.0;
		x[1]=151.0;
		x[2]=161.0;
		x[3]=161.0;

		PLFLT zstep=(zmax-zmin)/nsteps;
		PLFLT i=0.;

		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./nsteps)
		{
			y[0] = col;
			y[3] = col;
			y[1] = col+zstep;
			y[2] = y[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}


		plcol0(0);
		if (colourLog->isChecked())
			plbox("", 0.0, 0, "bnlstv", 0.0, 0);
		else
			plbox("", 0.0, 0, "bnstv", 0.0, 0);

		plmtex("l", 6, 0.5, 0.5, colLabSlot->text().toLatin1().data());



		//Other colours
		PLFLT xm[] = { 0, 0, 1, 1 };
		PLFLT ym[] = { 0, 1, 1, 0 };
		char max[50], min[50];

		if (colourLog->isChecked()) {
			sprintf(max, ">10#u%.2f", zmax);
			sprintf(min, "<10#u%.2f", zmin);
		} 
		else {
			sprintf(max, ">%.2e", zmax);
			sprintf(min, "<%.2e", zmin);
		}

		select_vpor(6);
		plwind(0, 1, 0, 1);
		plcol1(1.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("t", 1.5, 0.5, 0.5, max);

		select_vpor(7);
		plwind(0, 1, 0, 1);
	//	plcol0(15);
	//	plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.5, 0.5, min);
	
	}
	else{				
		
		// Portrait
		// Put the colourbar horizontally so to maxmize use of space.
		
		PLFLT titleOffset = 0.0;
		
		plwind(zmin, zmax, 151.0, 161.0);

		PLFLT x[4], y[4];
	
		y[0]=151.0;
		y[1]=151.0;
		y[2]=161.0;
		y[3]=161.0;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			x[0] = col;
			x[3] = col;
			x[1] = col+zstep;
			x[2] = x[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}

		plschr(QPUI_CHAR_HEIGHT, 1.1*text_size_fld->value());

		plcol0(0);
		if (colourLog->isChecked())
			plbox("bnlstv", 0.0, 0, "", 0.0, 0);
		else
			plbox("bnstv", 0.0, 0, "", 0.0, 0);

		plmtex("t", 2.0+titleOffset, 0.5, 0.5, colLabSlot->text().toLatin1().data());


/*
		//Other colours
		PLFLT xm[] = { 0, 0, 1, 1 };
		PLFLT ym[] = { 0, 1, 1, 0 };
		char max[50], min[50];

		if (colourLog->isChecked()) {
			sprintf(max, ">10#u%.2f", zmax);
			sprintf(min, "<10#u%.2f", zmin);
		} else {
			sprintf(max, ">%.2g", zmax);
			sprintf(min, "<%.2g", zmin);
		}

		plschr(QPUI_CHAR_HEIGHT, 0.5);

		select_vpor(6);
		plwind(0, 1, 0, 1);
		plcol1(1.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.0, 0.0, max);

		select_vpor(7);
		plwind(0, 1, 0, 1);
		plcol1(0.0);
		plfill(4, xm, ym);
		plcol0(0);
		plbox("bc", 0.0, 0, "bc", 0.0, 0);
		plmtex("b", 1.5, 0.0, 0.0, min);
		
		plschr(QPUI_CHAR_HEIGHT, 0.6);
*/
	}

}





//Since lots of viewports are defined, only this part needs to be changed if the window size changes.
void Q3dWindow::select_vpor(int n)
{
	PLFLT x0, x1, y0, y1, xmean, ymean;

	PLFLT xcutoff;	//xcutoff, ycutoff is the one-sided fraction of the page that will be trimmed off if we cut a 1:sqrt(2) page out of the window.
	PLFLT ycutoff;

	bool landscape;
	
	// Getting plot option 

	if(pageOrientation->currentIndex() == 0) landscape = false;
	else landscape = true;

	//Try to fit the largest 1:sqrt(2) page into the window.

	xcutoff = (width*sqrt(2.) < (PLFLT) height? 0:((width-height/sqrt(2.))/2.)/(PLFLT)width);
	ycutoff = (width*sqrt(2.) > (PLFLT) height? 0:((height-width*sqrt(2.))/2.)/(PLFLT)height);

	if (landscape) {
		x0 = 0.1 + xcutoff;
		y0 = 0.2 + ycutoff;
		y1 = 0.9 - ycutoff;
		x1 = x0 + (y1 - y0) * ((PLFLT) width / (PLFLT) height);
		xmean = (x0 + x1) / 2;
		ymean = (y0 + y1) / 2;

		switch (n) {
		case -1: //vpor for the title
			plvpor(0.02, 0.98, .8, y1+.03);
			break;
		case 0: //main vpor
			plvpor(x0, x1, y0+0.02, y1-0.02);
			break;
		case 1: //top right of main
			plvpor(xmean, x1, ymean, y1);
			break;
		case 2: //top left of main
			plvpor(x0, xmean, ymean, y1);
			break;
		case 3: //bottom left of main
			plvpor(x0, xmean, y0, ymean);
			break;
		case 4: //bottom right of main
			plvpor(xmean, x1, y0, ymean);
			break;
		case 5: //col scale
			plvpor(x1 + 0.05, x1 + 0.10, y0 + 0.07, y1 - 0.07);
			break;
		case 6: //max col
			plvpor(x1 + 0.05, x1 + 0.10, y1 - 0.06, y1 - 0.03);
			break;
		case 7: //min col
			plvpor(x1 + 0.05, x1 + 0.10, y0 + 0.03, y0 + 0.06);
			break;
		case 8: //axes label
			plvpor(xmean, x1, y0 - 0.1, y0);
			break;
		}
	} else {
		x0 = 0.10 + xcutoff;
		x1 = 0.90 - xcutoff;
		y0 = 0.60 - (x1 - x0) * 0.5 * ((PLFLT) width / (PLFLT) height);
		y1 = 0.60 + (x1 - x0) * 0.5 * ((PLFLT) width / (PLFLT) height);
		xmean = (x0 + x1) / 2;
		ymean = (y0 + y1) / 2;

		switch (n) {
		case -1: //vpor for the title
			plvpor(0.02, 0.98, 0, y1+.03);
			break;
		case 0: //main vpor
			plvpor(x0, x1, y0, y1);
			break;
		case 1: //top right of main
			plvpor(xmean, x1, ymean, y1);
			break;
		case 2: //top left of main
			plvpor(x0, xmean, ymean, y1);
			break;
		case 3: //bottom left of main
			plvpor(x0, xmean, y0, ymean);
			break;
		case 4: //bottom right of main
			plvpor(xmean, x1, y0, ymean);
			break;
		case 5: //col scale
			plvpor(x0 + 0.10, x1 - 0.10, 0.10, 0.15);
			break;
		case 6: //max col
			plvpor(x1 - 0.07, x1 - 0.03, 0.10, 0.15);
			break;
		case 7: //min col
			plvpor(x0 + 0.03, x0 + 0.07, 0.10, 0.15);
			break;
		case 8: //axes label
			plvpor(xmean, x1, y0 - 0.03, y0);
			break;
		}
	}
}


PLFLT Q3dWindow::get_mappeddepth(threevector *tv){

	PLFLT mapped_x, mapped_y, mapped_z;
	
	if(isPolar){
		mapped_x = tv->getx();
		mapped_y = tv->gety();
		mapped_z = tv->getz();
	}
	else{
		// userR, userTh, userPhi are actually userX, Y, Z so this looks confusing
		mapped_x = (tv->getx() - userRmin)/(userRmax - userRmin);
		mapped_y = (tv->gety() - userThmin)/(userThmax - userThmin);
		mapped_z = (tv->getz() - userPhimin)/(userPhimax - userPhimin);
	}

	return mapped_x*sin(az)*cos(el) + mapped_y*cos(az)*cos(el) - mapped_z*sin(el);
}


PLFLT arrminNonZero(PLFLT a[], int size) {
	PLFLT minval=1.e31;

	for (int i = 1; i < size; i++) {
	
		if (a[i] < minval && a[i] > 1.e-30){
			minval = a[i];
		}
	}
	return minval;
}


void MinMax3dgrid(PLFLT data[], int nx, int ny, int nz, PLFLT &min, PLFLT &max, double fillVal) {
	int i, j, k;

	min = 1.e33;
	max = -1.e31;

	for (i=0; i < nx; i++) {
		for(j=0; j<ny; j++) {
			for(k=0; k<nz; k++) {
				double val = (double) data[i*ny*nz + j*nz + k];
				if ( val != fillVal && !isnan(val)) {
					if (min > val) min = val;
					if (max < val) max = val;
				}
			}
		}
	}
}

void quicksortthreevector(vector <threevector*> &vect, int start, int end, PLFLT el, PLFLT az){

	if (start < end) {
		threevector *pivot = vect[end];
		int bottom = start - 1;
		int top = end;

		bool done = false;
		while (!done) {
			while (!done) {
				bottom += 1;

				if (bottom == top) {
					done = true;
					break;
				}
				if (vect[bottom]->getdepth(el, az) > pivot->getdepth(el, az)) {
					vect[top] = vect[bottom];
					break;
				}
			}
			while (!done) {
				top = top - 1;

				if (top == bottom) {
					done = true;
					break;
				}
				if (vect[top]->getdepth(el, az) < pivot->getdepth(el, az)) {
					vect[bottom] = vect[top];
					break;
				}
			}
		}
		vect[top] = pivot;

		//pivot = top;
		quicksortthreevector(vect, start, top - 1, el, az);
		quicksortthreevector(vect, top + 1, end, el, az);
	}
}

void quicksortthreevector(vector <threevector*> &vect, vector <threevector*> &v2 ,vector <threevector*> &v3, int start, int end, PLFLT el, PLFLT az){

	if (start < end) {
		threevector *pivot = vect[end];
		threevector *pivot2 = v2[end];
		threevector *pivot3 = v3[end];
		int bottom = start - 1;
		int top = end;

		bool done = false;
		while (!done) {
			while (!done) {
				bottom += 1;

				if (bottom == top) {
					done = true;
					break;
				}
				if (vect[bottom]->getdepth(el, az) > pivot->getdepth(el, az)) {
					vect[top] = vect[bottom];
					v2[top] = v2[bottom];
					v3[top] = v3[bottom];
					break;
				}
			}
			while (!done) {
				top = top - 1;

				if (top == bottom) {
					done = true;
					break;
				}
				if (vect[top]->getdepth(el, az) < pivot->getdepth(el, az)) {
					vect[bottom] = vect[top];
					v2[bottom] = v2[top];
					v3[bottom] = v3[top];
					break;
				}
			}
		}
		vect[top] = pivot;
		v2[top] = pivot2;
		v3[top] = pivot3;

		//pivot = top;
		quicksortthreevector(vect, v2, v3, start, top - 1, el, az);
		quicksortthreevector(vect, v2, v3, top + 1, end, el, az);
	}
}



