#include "IntervalDialog.h"
#include "PlotWindow.h"
#include "qpui.h"

IntervalDialog::IntervalDialog(PlotWindow * parent) :
	QMainWindow(parent, Qt::Widget)
{
	this->frame=NULL;
	QWidget * mainWidget=new QWidget(this);
	
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	
	xWidget=new ScalarIntervalWidget(this);
    xWidget->setBarString("Selected Range");
	xWidget->setRangeName("Xrange");
	
	timeWidget=new TimeIntervalWidget(this, 1);
    timeWidget->setBarString("Selected range");
	
	mainLayout->addWidget(xWidget);
	mainLayout->addWidget(timeWidget);

    xWidget->expand();
    timeWidget->expand();

	// Buttons
	
	QWidget * captureWidget=new QWidget(mainWidget);
	QHBoxLayout * captureLayout=new QHBoxLayout;
	
	QWidget *buttonsWidget=new QWidget(mainWidget);
	QHBoxLayout * buttonsLayout=new QHBoxLayout;
	
	captureButton=new QPushButton("Get event", captureWidget);
	connect(captureButton, SIGNAL(clicked()), this, SLOT(capture()));
	captureLayout->addWidget(captureButton);
	
	addButton=new QPushButton("Add to events", captureWidget);
	connect(addButton, SIGNAL(clicked()), this, SLOT(add()));
	captureLayout->addWidget(addButton);
	
	captureWidget->setLayout(captureLayout);
	mainLayout->addWidget(captureWidget);
	
// 	QPushButton * resetButton=new QPushButton("Reset", buttonsWidget);
// 	connect(resetButton, SIGNAL(clicked()), this, SLOT(reset()));
// 	buttonsLayout->addWidget(resetButton);
	
	applyButton=new QPushButton("Apply", buttonsWidget);
	connect(applyButton, SIGNAL(clicked()), this, SLOT(apply()));
	buttonsLayout->addWidget(applyButton);
	applyButton->setDefault(true);	
	
	QPushButton * dismissButton=new QPushButton("Dismiss", buttonsWidget);
	connect(dismissButton, SIGNAL(clicked()), this, SLOT(dismiss()));
	buttonsLayout->addWidget(dismissButton);
	
	buttonsWidget->setLayout(buttonsLayout);
	mainLayout->addWidget(buttonsWidget);
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
// 	connect(slider, SIGNAL(changed(double, double)), this, SLOT(updateFields(double, double)));
	
#if QT_VERSION >= 0x040300
	mainLayout->setContentsMargins(0, 0, 0, 0);
	buttonsLayout->setContentsMargins(0, 0, 0, 0);
	captureLayout->setContentsMargins(0, 0, 0, 0);
#else
	mainLayout->setMargin(0.);
	buttonsLayout->setMargin(0.);
	captureLayout->setMargin(0.);
#endif
	buttonsLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	captureLayout->setSpacing(QSAS_profiles->current()->GUImargins);
	
	setVisible(false);
}

void IntervalDialog::setAsTimeInterval(DvTime tmin, DvTime tmax/*, double currentTmin, double currentTmax*/, FrameItem* frame)
{
	this->frame=frame;
	
	xWidget->setVisible(false);
	timeWidget->setVisible(true);
	captureButton->setVisible(true);
	addButton->setVisible(true);
	
	timeWidget->clearEventTable();
	timeWidget->setStart(tmin);
	timeWidget->setEnd(tmax);
	timeWidget->resetView();
	
// 	QDOS::Time start, end;
// 	start.set_time_rep(currentTmin);
// 	end.set_time_rep(currentTmax);
// 	startTimeWidget->setTime(start);
// 	endTimeWidget->setTime(end);
// 	
// 	slider->reset(tmin, tmax, currentTmin, currentTmax);
	adjustSize();
}

void IntervalDialog::setAsXInterval(double tmin, double tmax)
{
	xWidget->setVisible(true);
	timeWidget->setVisible(false);
	captureButton->setVisible(false);
	addButton->setVisible(false);
	
	xWidget->setInterval(tmin, tmax);
	
	adjustSize();
}

void IntervalDialog::setViewRange(DvEvent range)
{
	if(timeWidget->isVisible())
	{
		timeWidget->setInterval(range);
	}
}

void IntervalDialog::setViewRange(double min, double max)
{
	if(xWidget->isVisible())
	{
		xWidget->setInterval(min, max);
	}
}

void IntervalDialog::setAvailabilityRange(double min, double max)
{
	if(xWidget->isVisible())
	{
		xWidget->setAvailabilityRange(min, max);
	}
	else if(timeWidget->isVisible())
	{
		timeWidget->setAvailabilityRange(DvTime(min), DvTime(max));
	}
}

void IntervalDialog::apply()
{
	if(timeWidget->isVisible())
	{
		emit rescaleTime(timeWidget->getInterval());
	}
	else if(xWidget->isVisible())
	{
		emit rescaleScalar(xWidget->getStart(), xWidget->getEnd());
	}
}

void IntervalDialog::dismiss()
{
	setVisible(false);
}

void IntervalDialog::add()
{
	if(timeWidget->isVisible())
	{
		timeWidget->appendIntervalSlot();
	}
}

void IntervalDialog::capture()
{
	PlotWindow* p=(PlotWindow*)parent();
	if(p!=NULL)
	{
		double x_start, y_start, x_end, y_end;
		p->activateWindow();
		p->raise();
		hide();
		((PlotWindow*) parent())->plot->captureMouse(&x_start, &y_start, &x_end, &y_end, false);
		y_start=1.-y_start;
		y_end=1.-y_end;
		show();
		applyButton->setFocus();

		// Check that the selection is in the right frame
		if(frame!=NULL)
		{
			bool inFrame=false;
			int numPanels = frame->childCount();
			float panelHt = frame->PropertiesTab->_height_fld->value();
			double factor=(frame->page()->containsSpectrum()?0.9:1.);
			
			inFrame=(	frame->PropertiesTab->_loffset_fld->value() <= x_start &&  
					x_start <= frame->PropertiesTab->_loffset_fld->value() + frame->PropertiesTab->_width_fld->value()*factor
					&& 1. - frame->PropertiesTab->_toffset_fld->value() >= y_start
					&&
					y_start >= 1. - frame->PropertiesTab->_toffset_fld->value() - panelHt);
			
			if(!inFrame)
			{
				(void)QMessageBox::warning(this, "Value Return", "Sorry! Wrong frame in selection");
				return;
			}
			
			double x_offset = frame->PropertiesTab->_loffset_fld->value();
			double x_sz = frame->PropertiesTab->_width_fld->value()*factor;
			DvTime tmin = frame->tAxisTab->_axis_time_fld->getStart();
			DvTime tmax = frame->tAxisTab->_axis_time_fld->getEnd();
			
			double t_range = tmax.getDiffSecs(tmin);
			timeWidget->setInterval(tmin+t_range*(x_start-x_offset)/x_sz, tmin+t_range * (x_end-x_offset)/x_sz, false);
		}
	}
}

// void IntervalDialog::reset()
// {
// // 	startXWidget->setText(double_to_string(slider->min).c_str());
// // 	endXWidget->setText(double_to_string(slider->max).c_str());
// // 	
// // 	QDOS::Time start, end;
// // 	start.set_time_rep(slider->min);
// // 	end.set_time_rep(slider->max);
// // 	startTimeWidget->setTime(start);
// // 	endTimeWidget->setTime(end);
// // 	
// // 	slider->reset(slider->min, slider->max, slider->min, slider->max);
// }

// void IntervalDialog::updateFields(double start, double end)
// {
// 	QDOS::Time s, e;
// 	s.set_time_rep(start);
// 	e.set_time_rep(end);
// 	startTimeWidget->setTime(s);
// 	endTimeWidget->setTime(e);
// 	
// 	startXWidget->setText(double_to_string(start).c_str());
// 	endXWidget->setText(double_to_string(end).c_str());
// }

// void IntervalDialog::updateSlider()
// {
// 	slider->blockSignals(true);
// 	if(tAxisWidget->isVisible())
// 	{
// 		slider->setValue(startTimeWidget->value(), endTimeWidget->value());
// 	}
// 	else if(xAxisWidget->isVisible())
// 	{
// 		slider->setValue(startXWidget->text().toDouble(), endXWidget->text().toDouble());
// 	}
// 	slider->blockSignals(false);
// 	slider->update();
// }
