// qpui_axis_options.cc
//
// Plot Interface Implementation - Trace Options 
//
// Janet Barnes
// July 2002
//
// Tony Allen
// April 2008


#include "qpui.h"
#include "qdutil.h"
//#include "QbTimeEditor.h"


#include "qt_version.h"
#include <QGroupBox>
#include <QColorDialog>
#include <QToolTip>
#include "qbu.h"

extern QPalette QPUI_normal_palette;	
extern int qsas_restore_release;


//     QpTab CLASS (Base of all plot tabs)
//     ---------------------

void QpTab::setSlotColour(bool active)
{
// 	QColorGroup g = QColorGroup(GUIpalette.active());
// 	QColorGroup in_g = QColorGroup(GUIpalette.inactive());
// 	if(active) 
// 	{
// 		in_g.setColor(QColorGroup::Base, g.light());
// 		g.setColor(QColorGroup::Base, g.light());
// 	}
// 	else{
// 		in_g.setColor(QColorGroup::Base, g.midlight());
// 		g.setColor(QColorGroup::Base, g.midlight());
// 	}
// 	GUIpalette.setActive(g);
// 	GUIpalette.setInactive(in_g);

	if(active)
	{
		GUIpalette.setColor(QPalette::Active, QPalette::Base, QApplication::palette().color(QPalette::Active, QPalette::Base));
		GUIpalette.setColor(QPalette::Inactive, QPalette::Base, QApplication::palette().color(QPalette::Active, QPalette::Base));
	}
	else
	{
		GUIpalette.setColor(QPalette::Active, QPalette::Base, QApplication::palette().color(QPalette::Inactive, QPalette::Base));
		GUIpalette.setColor(QPalette::Inactive, QPalette::Base, QApplication::palette().color(QPalette::Inactive, QPalette::Base));
	}

}

QpTab::QpTab(QTabWidget *tabWidget,  const QString& name, QpTabType _tabType):
    QWidget(tabWidget)
{
	// Comstructor
	TabType = _tabType;
	GUIpalette = QPalette(qApp->palette());
}

//     QpAxisTab CLASS
//     ---------------------

QpAxisTab::QpAxisTab(QTabWidget *tabWidget,  const QString& name, QpuiItem *item, QpTabType _tabType):
		QpTab(tabWidget, name, _tabType)
{
	// Constructor
    userSetLinLog=false;
	
	I_Item = item; // will be either frame or panel in inherited classes QpYAxis and QpXAxis (or QpTAxis)

	create();
	
	connect(_frame_type_btn, SIGNAL(pressed()), this, SLOT(launchFrameTypeSelector()) );
}

QpAxisTab::QpAxisTab(QpAxisTab* tab, QpuiItem* item, QpTabType _tabType):
		QpTab((QTabWidget*)(tab->parent()), "Axis", _tabType)
{

    userSetLinLog=false;
	
	I_Item = item; // will be either frame or panel in inherited classes QpYAxis and QpXAxis (or QpTAxis)

	create();
	
	_axis_linlog_fld->setCurrentIndex(tab->_axis_linlog_fld->currentIndex());
	
	
	_axis_autoscale_fld->setCurrentIndex(tab->_axis_autoscale_fld->currentIndex());
	_axis_autolabel_fld->setChecked(tab->_axis_autolabel_fld->isChecked());
        _axis_xy_fld->setInterval(tab->_axis_xy_fld->getStart(), tab->_axis_xy_fld->getEnd());
	_axis_xy_fld->resetView();
	_axis_time_fld->setInterval(tab->_axis_time_fld->getInterval());
	
	_axis_y_nudge_fld->setChecked(tab->_axis_y_nudge_fld->isChecked());
	_axis_major_ticks_num_fld->setValue(tab->_axis_major_ticks_num_fld->value());
	_axis_minor_ticks_num_fld->setValue(tab->_axis_minor_ticks_num_fld->value());
	_axis_major_tickmarks_fld->setCurrentIndex(tab->_axis_major_tickmarks_fld->currentIndex());
	_axis_minor_tickmarks_fld->setCurrentIndex(tab->_axis_minor_tickmarks_fld->currentIndex());
	
	tick_time_fmt_fld->setText(tab->tick_time_fmt_fld->text());
	tick_time_fmt_auto_fld->setChecked(tab->tick_time_fmt_auto_fld->isChecked());
	
	time_labels_fmt_fld->setText(tab->time_labels_fmt_fld->text());
	time_labels_fmt_auto_fld->setChecked(tab->time_labels_fmt_auto_fld->isChecked());
	_label_fld->setText(tab->_label_fld->text());
	_label_char_size_fld->setValue(tab->_label_char_size_fld->value());
	_label_disp_fld->setValue(tab->_label_disp_fld->value());
	_label_rotation_fld->setValue(tab->_label_rotation_fld->value());
	
	PLINT family, style, weight;
	tab->_label_font_fld->getFontParams(family, style, weight);
	_label_font_fld->setFontParams(family, style, weight);
	_label_colour=tab->_label_colour;

	connect(_frame_type_btn, SIGNAL(pressed()), this, SLOT(launchFrameTypeSelector()) );
}

void QpAxisTab::create()
{
	QVBoxLayout *axis_top_layout = new QVBoxLayout(this);
	axis_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *axis_scale_groupbox = new QGroupBox(this);
	axis_scale_groupbox->setTitle("Scale");

	QVBoxLayout * axis_scale_layout=new QVBoxLayout;
	axis_scale_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_scale_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QHBoxLayout * subLayout1=new QHBoxLayout;
	QLabel *_axis_autoscale_lbl = new QLabel("Use: ");
	subLayout1->addWidget(_axis_autoscale_lbl, 0, Qt::AlignRight);
	
	_axis_autoscale_fld = new QComboBox();
	_axis_autoscale_fld->insertItem(QP_FROM_META, "Metadata");
	_axis_autoscale_fld->insertItem(QP_FROM_DATA, "Autoscale");
	_axis_autoscale_fld->insertItem(QP_FROM_USER, "User Set");
	_axis_autoscale_fld->setCurrentIndex(QSAS_profiles->current()->axisScaleType);
	
	subLayout1->addWidget(_axis_autoscale_fld, 0, Qt::AlignLeft);

	subLayout1->addStretch();
	
	linlog_label = new QLabel("Type: ");
	subLayout1->addWidget(linlog_label, 0, Qt::AlignRight);
	_axis_linlog_fld = new QComboBox();
	_axis_linlog_fld->insertItem(QP_LINEAR, "Linear");
	_axis_linlog_fld->insertItem(QP_LOG, "Log");
	QFontMetrics linlog_fm(_axis_linlog_fld->font());
	int linlog_w = linlog_fm.width("Linear");
	int linlog_h = linlog_fm.height();
	_axis_linlog_fld->setMinimumSize(linlog_w, linlog_h);


	subLayout1->addWidget(_axis_linlog_fld, 0, Qt::AlignLeft);

	// only show if X or Y axis
	linlog_label->setVisible(false);
	_axis_linlog_fld->setVisible(false);
	
	t_drop_label = new QLabel("From Object: ");
	subLayout1->addWidget(t_drop_label, 0, Qt::AlignRight);

	_t_data_fld = new QguWLinput(axis_scale_groupbox, "t_data_fld");
	_t_data_fld->setMinimumWidth(280);
	subLayout1->addWidget(_t_data_fld, 0, Qt::AlignLeft);
	_t_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
	
	// only show if t axis
	_t_data_fld->setVisible(false);
	t_drop_label->setVisible(false);
	
	axis_scale_layout->addLayout(subLayout1);
	
	_axis_xy_fld=new ScalarIntervalWidget(this);
	_axis_xy_fld->setRangeName(QString("X_axisRange"));

	_axis_time_fld=new TimeIntervalWidget(this);
	
	axis_scale_layout->addWidget(_axis_xy_fld);
	axis_scale_layout->addWidget(_axis_time_fld);
	_axis_time_fld->setVisible(false);
	
	_axis_y_nudge_fld=new QCheckBox("Nudge Y range");
	axis_scale_layout->addWidget(_axis_y_nudge_fld, Qt::AlignRight);
	_axis_y_nudge_fld->setChecked(false);

	QGroupBox *axis_ticks_groupbox = new QGroupBox;
	axis_ticks_groupbox->setTitle("Tickmarks");

	QGridLayout *axis_ticks_layout = new QGridLayout;
	
	axis_ticks_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_ticks_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* num_major_ticks_label = new QLabel( "# Major ticks: ");
	axis_ticks_layout->addWidget(num_major_ticks_label, 1, 0, Qt::AlignRight);
	_axis_major_ticks_num_fld = new QSpinBox;
	_axis_major_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_major_ticks_num_fld, 1, 1, Qt::AlignLeft);

	QLabel* major_tickmarks_label = new QLabel("Major tickmarks: ");
	axis_ticks_layout->addWidget(major_tickmarks_label, 1, 2, Qt::AlignRight);
	_axis_major_tickmarks_fld = new QComboBox();
	_axis_major_tickmarks_fld->insertItem(QPUI_NOTICKS, "None");
	_axis_major_tickmarks_fld->insertItem(QPUI_IN, "Inwards");
	_axis_major_tickmarks_fld->insertItem(QPUI_OUT, "Outwards");
	_axis_major_tickmarks_fld->insertItem(QPUI_GRID, "Grid");
	QFontMetrics major_ticks_fm(_axis_major_tickmarks_fld->font());
	int major_ticks_w = major_ticks_fm.width("Outwards");
	int major_ticks_h = major_ticks_fm.height();
	_axis_major_tickmarks_fld->setMinimumSize(major_ticks_w, major_ticks_h);

	_axis_major_tickmarks_fld->setToolTip("Select major tickmarks style");
	axis_ticks_layout->addWidget(_axis_major_tickmarks_fld, 1, 3, Qt::AlignLeft);
	_axis_major_tickmarks_fld->setCurrentIndex(QPUI_IN);
	
	QLabel* num_minor_ticks_label = new QLabel( "# Minor ticks: ");
	axis_ticks_layout->addWidget(num_minor_ticks_label, 3, 0, Qt::AlignRight);
	_axis_minor_ticks_num_fld = new QSpinBox;
	_axis_minor_ticks_num_fld->setFixedWidth(100);
	axis_ticks_layout->addWidget(_axis_minor_ticks_num_fld, 3, 1, Qt::AlignLeft);

	QLabel* minor_tickmarks_label = new QLabel( "Minor tickmarks: ");
	axis_ticks_layout->addWidget(minor_tickmarks_label, 3, 2, Qt::AlignRight);
	_axis_minor_tickmarks_fld = new QComboBox();
	_axis_minor_tickmarks_fld->insertItem(QPUI_NOTICKS, "None");
	_axis_minor_tickmarks_fld->insertItem(QPUI_IN, "Inwards");
	_axis_minor_tickmarks_fld->insertItem(QPUI_OUT, "Outwards");
	_axis_minor_tickmarks_fld->setMinimumSize(major_ticks_w,major_ticks_h);

	_axis_minor_tickmarks_fld->setToolTip("Select minor tickmarks style");
	axis_ticks_layout->addWidget(_axis_minor_tickmarks_fld, 3, 3, Qt::AlignLeft);
	_axis_minor_tickmarks_fld->setCurrentIndex(QPUI_IN);
	
	tick_time_fmt_label=new QLabel("Time ticks format");
	axis_ticks_layout->addWidget(tick_time_fmt_label, 5, 0, Qt::AlignRight);
	tick_time_fmt_fld=new QLineEdit;
	setSlotColour(false);
	tick_time_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(tick_time_fmt_fld, 5, 1, Qt::AlignRight);
	tick_time_fmt_auto_fld=new QCheckBox("Auto-format-ticks");
	axis_ticks_layout->addWidget(tick_time_fmt_auto_fld, 5, 2, Qt::AlignRight);
	tick_time_fmt_auto_fld->setChecked(true);
	
	
	time_labels_fmt_label=new QLabel("Time labels format");
	axis_ticks_layout->addWidget(time_labels_fmt_label, 7, 0, Qt::AlignRight);
	time_labels_fmt_fld=new QLineEdit;
	setSlotColour(false);
	time_labels_fmt_fld->setPalette(GUIpalette);
	
	axis_ticks_layout->addWidget(time_labels_fmt_fld, 7, 1, Qt::AlignRight);
	time_labels_fmt_auto_fld=new QCheckBox("Auto-format-label");
	axis_ticks_layout->addWidget(time_labels_fmt_auto_fld, 7, 2, Qt::AlignRight);
	time_labels_fmt_auto_fld->setChecked(true);
	
	tick_time_fmt_fld->setToolTip( QString("Time format as in the strftime C function:\n")+
				       QString("%A or %a for week day,\n")+
				       QString("%B or %b for month name,\n")+
				       QString("%d for day of month as integer,\n")+
				       QString("%H for hours,\n")+
				       QString("%j for day of year,\n")+
				       QString("%m for month as integer,\n")+
				       QString("%M for minutes,\n")+
				       QString("%S for seconds,\n")+
				       QString("%S%3 for seconds with 3 decimal places, etc, eg 12.345\n")+
				       QString("%Y or %y for year,\n")+
				       QString("...\n")+
				       QString("#u to switch to superscript,\n")+
				       QString("#d to switch to subscript,\n")+
				       QString("Note: # is alt-pound on UK Macs\n")
		     );
	
	time_labels_fmt_fld->setToolTip( QString("Time format as in the strftime C function:\n")+
				       QString("%A or %a for week day,\n")+
				       QString("%B or %b for month name,\n")+
				       QString("%d for day of month as integer,\n")+
				       QString("%H for hours,\n")+
				       QString("%j for day of year,\n")+
				       QString("%m for month as integer,\n")+
				       QString("%M for minutes,\n")+
				       QString("%S for seconds,\n")+
				       QString("%S%3 for seconds with 3 decimal places, etc, eg 12.345\n")+
				       QString("%Y or %y for year,\n")+
				       QString("...F\n")+
				       QString("#u to switch to superscript,\n")+
				       QString("#d to switch to subscript,\n")+
				       QString("Note: # is alt-pound on UK Macs\n")
		     );

	axis_ticks_groupbox->setLayout(axis_ticks_layout);
	
	axis_scale_layout->addWidget(axis_ticks_groupbox);

    _exp_label_params_groupbox=new QGroupBox("Exponential label position");

    QHBoxLayout * exp_layout=new QHBoxLayout;
    
    exp_layout->addWidget(new QLabel("Horizontal offset:"), Qt::AlignRight);

    _exp_label_offset_x_fld=new FloatSpinBox(-20., 20., 2, 0.02);
    exp_layout->addWidget(_exp_label_offset_x_fld,Qt::AlignLeft);
	_exp_label_offset_x_fld->setValue(0.);

    exp_layout->addWidget(new QLabel("Vertical offset:"), Qt::AlignRight);

    _exp_label_offset_y_fld=new FloatSpinBox(-20., 20., 2, 0.02);
    exp_layout->addWidget(_exp_label_offset_y_fld, Qt::AlignLeft);
	_exp_label_offset_y_fld->setValue(0.);

    exp_layout->addStretch(10);
    
    _exp_label_params_groupbox->setLayout(exp_layout);

    axis_scale_layout->addWidget(_exp_label_params_groupbox);

	axis_scale_groupbox->setLayout(axis_scale_layout);
	
	axis_top_layout->addWidget(axis_scale_groupbox);

	QGroupBox *axis_label_groupbox = new QGroupBox(this);

	axis_label_groupbox->setTitle("Label");

	QGridLayout *axis_label_layout = new QGridLayout;
	axis_label_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	axis_label_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* text_label = new QLabel("Text: ");
	axis_label_layout->addWidget(text_label, 1, 0, Qt::AlignRight);
	_label_fld = new QLineEdit; 
	_label_fld->setMinimumWidth(350);
	setSlotColour(false);
	_label_fld->setPalette(GUIpalette);

	axis_label_layout->addWidget(_label_fld, 1, 1, 1, 3, Qt::AlignLeft);
		
	_axis_autolabel_fld = new QCheckBox("Auto-label");
	_axis_autolabel_fld->setChecked(true);
	axis_label_layout->addWidget(_axis_autolabel_fld, 1, 4, Qt::AlignLeft);

	QLabel* label_char_size_label = new QLabel("Character size: ");
	axis_label_layout->addWidget(label_char_size_label, 3, 0, Qt::AlignRight);
	_label_char_size_fld = new FloatSpinBox(0.01, 40.,1, .1, axis_label_groupbox, "label_char_size");
	_label_char_size_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_char_size_fld, 3, 1, Qt::AlignLeft);
	_label_char_size_fld->setValue(QSAS_profiles->current()->BaseFontSize+QPUI_LABELS_CHAR_SIZE_ADD);
	
	QLabel* label_disp_label = new QLabel("Displacement: ");
	axis_label_layout->addWidget(label_disp_label, 3, 2, Qt::AlignRight);
	_label_disp_fld = new FloatSpinBox(0.01, 40., 1,.5, axis_label_groupbox, "label_disp");
	_label_disp_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_disp_fld, 3, 3, Qt::AlignLeft);
	_label_disp_fld->setValue(QPUI_LABELS_DISP);

	QLabel* label_rotation_label = new QLabel("Rotation angle: ");
	axis_label_layout->addWidget(label_rotation_label, 5, 0, Qt::AlignRight);
	_label_rotation_fld = new FloatSpinBox(0.0, 360.0, 1, 10., axis_label_groupbox, "label_rotation");
	_label_rotation_fld->setFixedWidth(100);
	axis_label_layout->addWidget(_label_rotation_fld, 5, 1, Qt::AlignLeft);

	QLabel* font_label = new QLabel("Font: ");
	axis_label_layout->addWidget(font_label, 7, 0, Qt::AlignRight);
	_label_font_fld = new FontButton;
	_label_font_fld->setToolTip("Select font for axis label");
	axis_label_layout->addWidget(_label_font_fld, 7, 1, Qt::AlignLeft);

	QLabel* colour_label = new QLabel("Colour: ");
	axis_label_layout->addWidget(colour_label, 7, 2, Qt::AlignRight);
	_label_colour_fld = new QPushButton("Colour...");
	_label_colour_fld->setFixedWidth(100);
	_label_colour_fld->setFixedHeight(30);
	QFont f = _label_colour_fld->font();
	f.setBold(TRUE);
	_label_colour_fld->setFont(f);
	axis_label_layout->addWidget(_label_colour_fld, 7, 3, Qt::AlignLeft);
	

	_event_label_fld = new QPushButton("Add Event label");
	axis_label_layout->addWidget(_event_label_fld, 8, 0, Qt::AlignLeft);
	_event_label_fld->setVisible(false);
	
	axis_label_groupbox->setLayout(axis_label_layout);
	
	axis_top_layout->addWidget(axis_label_groupbox);


	frame_type_groupbox = new QGroupBox(this);

	QGridLayout *frame_type_layout = new QGridLayout;
	frame_type_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	frame_type_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_frame_type_btn = new QPushButton("Change Frame Type...");
	_frame_type_btn->setMinimumWidth(100);
	frame_type_layout->addWidget(_frame_type_btn,0,0,Qt::AlignLeft);
	_frame_type_btn->setToolTip("Determines whether plot items will be TS or XY");

	frame_type_groupbox->setLayout(frame_type_layout);
	
	axis_top_layout->addWidget(frame_type_groupbox);

	axis_top_layout->addSpacing(4);
	axis_top_layout->addStretch(10);

	_frame_type_btn->setVisible(false);
	frame_type_groupbox->setVisible(false);

	_label_colour = Qt::black;

}



//  ---------------------

void QpAxisTab::handleAutoScale(int choice)
{
	_axis_autoscale_fld->setCurrentIndex(choice);
	if(choice == QP_FROM_USER)  _axis_y_nudge_fld->setEnabled(false);
	else   _axis_y_nudge_fld->setEnabled(true);
}

//  ---------------------

void QpAxisTab::setAxisAutoScale(int choice)
{
	int axis = this->I_axis;

	if ( QuSlotDebug() )printf( "QpAxisTab::setAxisAutoScale axis = %d \n", axis );
		

	if(choice == QP_FROM_USER)
	{
		_axis_time_fld->setAltPalette(false);
		_axis_xy_fld->setAltPalette(false);
		_axis_y_nudge_fld->setEnabled(false);
	}
	else
	{
		this->setAxisMinMax();
		if(axis == X_AXIS || axis == T_AXIS) refreshYZRanges(); // also tell Y axis and colourscale
		qApp->processEvents();
		_axis_time_fld->setAltPalette(true);
		_axis_xy_fld->setAltPalette(true);
		_axis_y_nudge_fld->setEnabled(true);
	}
	
	// only tell list view to make changes to selected items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Item->Tree()->setAxisOption(AXIS_AUTOSCALE, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisLinLog(int choice)
{
	int axis = this->I_axis;
	this->handleAutoScale(QP_FROM_USER);
    userSetLinLog = true; // block metadata updating
    
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisLinLog axis = %d \n", axis );

 
	if(this->isVisible() )
	{
		I_Item->Tree()->setAxisOption(AXIS_LINLOG, axis, choice);
	}
}

//  ---------------------

void QpAxisTab::setAxisRange()
{
	int axis = this->I_axis;
	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisRange axis = %d \n", axis );
		
	
	if(axis!=T_AXIS && _axis_xy_fld->hasFocus())
	{
		this->handleAutoScale(QP_FROM_USER);
		_axis_xy_fld->setAltPalette(false);
		_axis_time_fld->setAltPalette(false);
		
		if(axis==X_AXIS) refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->Tree()->setAxisOption(AXIS_MIN, axis, 0, 0.,  QString("%1").arg(this->_axis_xy_fld->getStart()));
			I_Item->Tree()->setAxisOption(AXIS_MAX, axis, 0, 0.,  QString("%1").arg(this->_axis_xy_fld->getEnd()));
		}
	}
	else if(axis==T_AXIS )// && _axis_time_fld->hasFocus())
	{
		this->handleAutoScale(QP_FROM_USER);
		setSlotColour(false);
		_t_data_fld->lineBox->setPalette(GUIpalette);
		_axis_xy_fld->setAltPalette(false);
		_axis_time_fld->setAltPalette(false);
		
		refreshYZRanges();
		
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
		
		if(this->isVisible() ) {
			I_Item->Tree()->setAxisOption(AXIS_MIN, axis, 0, 0., this->_axis_time_fld->getStart().getISOstring(0).c_str());
			I_Item->Tree()->setAxisOption(AXIS_MAX, axis, 0, 0., this->_axis_time_fld->getEnd().getISOstring(0).c_str());
		}
	}
	FrameItem *frameItem = dynamic_cast<FrameItem *>(I_Item);
	PanelItem *panelItem = dynamic_cast<PanelItem *>(I_Item);
	if(frameItem) frameItem->updateIsotropic();
	else if(panelItem) panelItem->frame()->updateIsotropic();
}

//  ---------------------

void QpAxisTab::setAxisNumMajorTicks(int choice)
{
	int axis = this->I_axis;
	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisNumMajorTicks  \n");
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_NUM_MAJOR_TICKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisNumMinorTicks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisNumMinorTicks axis = %d \n", axis );
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_NUM_MINOR_TICKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisMajorTickMarks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisMajorTickMarks axis = %d \n", axis );
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_MAJOR_TICKMARKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisMinorTickMarks(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisMinorTickMarks axis = %d \n", axis );
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_MINOR_TICKMARKS, axis, choice);
}

//  ---------------------

void QpAxisTab::setAxisExpLabelYOffset(double choice)
{
    int axis = this->I_axis;
    
    // only tell list view to make changes to selected frame items if this
    // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
    if ( QuSlotDebug() )
        printf( "QpAxisTab::setAxisExpLabelYOffset = %d \n", axis );
   
    if(this->isVisible() )  I_Item->Tree()->setAxisOption(AXIS_EXP_LABEL_Y, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisExpLabelXOffset(double choice)
{
    int axis = this->I_axis;
    
    // only tell list view to make changes to selected frame items if this
    // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
    if ( QuSlotDebug() )
        printf( "QpAxisTab::setAxisExpLabelXOffset = %d \n", axis );
   
    if(this->isVisible() )  I_Item->Tree()->setAxisOption(AXIS_EXP_LABEL_X, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabel()
{	
	int axis = this->I_axis;
	
	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisLabel axis = %d \n", axis );
		

	if(_label_fld->hasFocus())
	{
		// only change if user input
		_axis_autolabel_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
 		if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL, axis, 0, 0.0, this->_label_fld->text());
	}

}
//  ---------------------

void QpAxisTab::setAxisAutoLabel( bool choice)
{

	int axis = this->I_axis;
	
	if ( QuSlotDebug() )printf( "QpAxisTab::setAxisAutoLabel axis = %d \n", axis );
		
	
	if(choice)
	{
		_label_fld->clearFocus();
	
		this->setAutoLabelText();
	}
	setSlotColour(!choice);
	_label_fld->setPalette(GUIpalette);
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
 	if(this->isVisible() ) I_Item->Tree()->setAxisOption(AXIS_AUTOLABEL, axis, 0, 0.0, "", choice);
	

}


//  ---------------------

void QpAxisTab::setAxisLabelCharSize(double choice)
{
	int axis = this->I_axis;
		if ( QuSlotDebug() )printf( "QpAxisTab::setAxisLabelCharSize axis = %d \n", axis );

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_CHAR_SIZE, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelDisp(double choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelDisp axis = %d \n", axis );
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_DISP, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelRotation(double choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelRotation axis = %d \n", axis );
   
 	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_ROTATION, axis, 0, choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelFamily(int choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelFamily axis = %d \n", axis );
   
	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_FONT_FAMILY, axis, choice);
}

void QpAxisTab::setAxisLabelStyle(bool choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() )
		printf( "QpAxisTab::setAxisLabelStyle axis = %d \n", axis );
   
	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_FONT_STYLE, axis, 0, 0., "", choice);
}

void QpAxisTab::setAxisLabelWeight(bool choice)
{
	int axis = this->I_axis;
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisLabelWeight axis = %d \n", axis );
   
	if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_LABEL_FONT_WEIGHT, axis, 0, 0., "", choice);
}

//  ---------------------

void QpAxisTab::setAxisLabelColour()
{
	int axis = this->I_axis;
	if ( QuSlotDebug() ) printf( "QpAxisTab::setAxisLabelColour axis = %d \n", axis );
	
	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(_label_colour, &changed, this);
	if(changed)
	{
        QPalette line_palette = QPalette(QPUI_normal_palette);
        line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
        line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
        this->_label_colour_fld->setPalette(line_palette);
        _label_colour = c;
	}

	// only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	I_Item->Tree()->setAxisOption(AXIS_LABEL_COLOUR, axis, 0, 0., "", false, c);
}

//  ---------------------

void QpAxisTab::refreshYZRanges()
{
	// called when X or T axis changes: force update of Y and Z ranges
	FrameItem * frameItem = dynamic_cast<FrameItem*>(I_Item);
	if(frameItem)
	{
		if(frameItem->hasChildren())
		{
			PanelItem * panelItem;
			for(int i=0; i<frameItem->childCount(); ++i)
			{
				panelItem = dynamic_cast<PanelItem*>(frameItem->child(i));
				panelItem->updateYZRange();
			}
		}
	}
}


// ------------------------------

void QpAxisTab::launchFrameTypeSelector()
{
	// slot invoked by Change Type button
	QMenu * frameTypePopup = new QMenu;
	frameTypePopup->addAction("Time Series");
	frameTypePopup->addAction("XY Frame");
	connect(frameTypePopup, SIGNAL(triggered(QAction*)), I_Item->Tree(), SLOT(setFrameType(QAction*)) );
	
	frameTypePopup->exec(QCursor::pos());
	
}


//     QpXAxisTab CLASS
//     ---------------------

QpXAxisTab::QpXAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame):
      QpAxisTab(tabWidget, name, (QpuiItem*) frame, QP_XT_TAB)
{
	// Constructor
	if ( QuSlotDebug() )
		printf( "QpXAxisTab::QpXAxisTab name = %s \n", name.toLatin1().data() );
	
	I_Frame = frame; 
	I_axis = X_AXIS;
	
	// only show for X or Y axis
	
	_axis_xy_fld->setVisible(true);
	_axis_time_fld->setVisible(false);
	_axis_y_nudge_fld->setVisible(false);
	
	linlog_label->setVisible(true);
	_axis_linlog_fld->setVisible(true);
	_frame_type_btn->setVisible(true);
	frame_type_groupbox->setVisible(true);
	frame_type_groupbox->setTitle("Frame type:   X vs Y");

    _exp_label_params_groupbox->setVisible(true);
	
	tick_time_fmt_label->setVisible(false); // show only if time axis
	tick_time_fmt_fld->setVisible(false); // show only if time axis
	tick_time_fmt_auto_fld->setVisible(false); // show only if time axis
	time_labels_fmt_label->setVisible(false); // show only if time axis
	time_labels_fmt_fld->setVisible(false); // show only if time axis
	time_labels_fmt_auto_fld->setVisible(false); // show only if time axis

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	connect( _axis_xy_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );

	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );

    connect( _exp_label_offset_y_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelYOffset(double)));
    connect( _exp_label_offset_x_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelXOffset(double)));
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));


}

QpXAxisTab::QpXAxisTab(QpXAxisTab* tab, FrameItem* frame):
	QpAxisTab(tab, frame, QP_XT_TAB)
{
	// Constructor
	if ( QuSlotDebug() )
		printf( "QpXAxisTab::QpXAxisTab copy \n");

	I_Frame = frame; 
	I_axis = X_AXIS;
	
	// only show for X or Y axis
	
	_axis_xy_fld->setVisible(true);
	_axis_time_fld->setVisible(false);
	_axis_y_nudge_fld->setVisible(false);
	
	linlog_label->setVisible(true);
	_axis_linlog_fld->setVisible(true);
	_frame_type_btn->setVisible(true);
	frame_type_groupbox->setVisible(true);
	frame_type_groupbox->setTitle("Frame type:   X vs Y");

    _exp_label_params_groupbox->setVisible(true);
	
	tick_time_fmt_label->setVisible(false); // show only if time axis
	tick_time_fmt_fld->setVisible(false); // show only if time axis
	tick_time_fmt_auto_fld->setVisible(false); // show only if time axis
	time_labels_fmt_label->setVisible(false); // show only if time axis
	time_labels_fmt_fld->setVisible(false); // show only if time axis
	time_labels_fmt_auto_fld->setVisible(false); // show only if time axis

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	connect( _axis_xy_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );

	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );

    connect( _exp_label_offset_y_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelYOffset(double)));
    connect( _exp_label_offset_x_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelXOffset(double)));
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));
}

// -----------------------------------------------

void QpXAxisTab::setAxisMinMax()
{
	// slot called when autoscale is set on or object changed
	if ( QuSlotDebug() )
	{
		printf( "QpXAxisTab::setAxisMinMax  \n" );
		fflush(stdout);
	}
	
	double from, to;

	switch( _axis_autoscale_fld->currentIndex())
	{
	
		case QP_FROM_META:
		{
			bool found = I_Frame->getXmetaRange(from, to); // falls through to autoscale if metadata not present
            if(_axis_linlog_fld->currentIndex() == QP_LOG){
                if(from <= 0) from = to * 1.0e-6;
                if(to <= 0) found = false;                    
            }
			if(found)
			{
                _axis_xy_fld->setInterval(from, to);
			}
			break;
		}
		case QP_FROM_DATA:
		{
			QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
			bool found = I_Frame->getXRange(from, to);
			QApplication::restoreOverrideCursor();
            if(_axis_linlog_fld->currentIndex() == QP_LOG){
                if(from <= 0) from = to * 1.0e-6;
                if(to <= 0) found = false;                   
            }
			if(found)
			{
				_axis_xy_fld->setInterval(from, to);
			}
			break;
			
		}
		default: case QP_FROM_USER:
		{
			// ensure objects are set for Y range
			I_Frame->getYObjects();
		}
	}
	
}
  
// -------------------------------

void QpXAxisTab::setAutoLabelText()
{	
	if ( QuSlotDebug() )printf( "QpXAxisTab::setAutoLabelText  \n" );
	// refresh Axis Label
	DvString _label="X Axis";
	
	// use first valid X axis plot item we can find (dependent axis should all be the same)
	PanelItem * panelItem;
	for(int i=0; i<I_Frame->childCount(); ++i)
	{
		panelItem=dynamic_cast<PanelItem*>(I_Frame->child(i));
		TraceItem * traceItem = dynamic_cast<TraceItem*>(panelItem->child(0));
		SpectrumItem * spectrumItem = dynamic_cast<SpectrumItem*>(panelItem->child(0));

		// skip const trace types
		if(traceItem && traceItem->I_Type == QP_TRACE) traceItem->getLabel(X_AXIS, _label);
		else if(spectrumItem) spectrumItem->getLabel(X_AXIS, _label);
	}

	_label_fld->setText(_label.c_str());

}

// -------------------------------

void QpXAxisTab::interactiveRescale(double start, double end)
{
	if ( QuSlotDebug() )printf( "QpXAxisTab::interactiveRescale  \n" );
	_axis_xy_fld->setInterval(start, end);
    handleAutoScale(QP_FROM_USER);

    refreshYZRanges();
	
	PlotWindow* win=(PlotWindow *) sender();
	if(!win) return;
	
	win->setPLFocus(); // ensure replot goes to correct window
	I_Frame->page()->plot(win);
}


//     QpYAxisTab CLASS
//     ---------------------

QpYAxisTab::QpYAxisTab(QTabWidget *tabWidget,  const QString& name, PanelItem * panel):
      QpAxisTab(tabWidget, name, (QpuiItem*) panel, QP_Y_TAB)
{
	// Comstructor
	if ( QuSlotDebug() )
		printf( "QpYAxisTab::QpYAxisTab name = %s \n", name.toLatin1().data() );
	
	I_Panel = panel; 
	I_axis = Y_AXIS;
	
	// only show for X or Y axis
	linlog_label->setVisible(true);
	_axis_linlog_fld->setVisible(true);
	
	_label_rotation_fld->setValue(90.);
	
	_axis_xy_fld->setVisible(true);
	_axis_time_fld->setVisible(false);
	_axis_y_nudge_fld->setVisible(true);
	_axis_xy_fld->setRangeName(QString("Y_axisRange"));

    _exp_label_params_groupbox->setVisible(true);
	
	tick_time_fmt_label->setVisible(false); // show only if time axis
	tick_time_fmt_fld->setVisible(false); // show only if time axis
	tick_time_fmt_auto_fld->setVisible(false); // show only if time axis
	time_labels_fmt_label->setVisible(false); // show only if time axis
	time_labels_fmt_fld->setVisible(false); // show only if time axis
	time_labels_fmt_auto_fld->setVisible(false); // show only if time axis
	
		// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	connect( _axis_xy_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );

    connect( _exp_label_offset_y_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelYOffset(double)));
    connect( _exp_label_offset_x_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelXOffset(double)));
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));

	_event_label_fld->setVisible(true);
 	connect(_event_label_fld, SIGNAL(clicked()), I_Panel->Tree(), SLOT(makeEventItem()));

	connect(_axis_y_nudge_fld, SIGNAL(stateChanged(int)), this, SLOT(updateNudge()));
	
	_axis_autoscale_fld->insertItem(QP_AUTO_ALL_SELECTED, "Autoscale All Selected Panels");

}

QpYAxisTab::QpYAxisTab(QpYAxisTab* tab, PanelItem* panel):
	QpAxisTab(tab, panel, QP_Y_TAB)
{
// Comstructor
	if ( QuSlotDebug() )
		printf( "QpYAxisTab::QpYAxisTab copy\n");
	
	I_Panel = panel; 
	I_axis = Y_AXIS;
	
	// only show for X or Y axis
	linlog_label->setVisible(true);
	_axis_linlog_fld->setVisible(true);
	
	_label_rotation_fld->setValue(90.);
	
	_axis_xy_fld->setVisible(true);
	_axis_time_fld->setVisible(false);
	_axis_y_nudge_fld->setVisible(true);
	_axis_xy_fld->setRangeName(QString("Y_axisRange"));

    _exp_label_params_groupbox->setVisible(true);
	
	tick_time_fmt_label->setVisible(false); // show only if time axis
	tick_time_fmt_fld->setVisible(false); // show only if time axis
	tick_time_fmt_auto_fld->setVisible(false); // show only if time axis
	time_labels_fmt_label->setVisible(false); // show only if time axis
	time_labels_fmt_fld->setVisible(false); // show only if time axis
	time_labels_fmt_auto_fld->setVisible(false); // show only if time axis
	
		// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	connect( _axis_linlog_fld, SIGNAL(activated(int)), this, SLOT(setAxisLinLog(int)) );
	
	connect( _axis_xy_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );

    connect( _exp_label_offset_y_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelYOffset(double)));
    connect( _exp_label_offset_x_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelXOffset(double)));
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));

	_event_label_fld->setVisible(true);
	connect(_event_label_fld, SIGNAL(clicked()), I_Panel->Tree(), SLOT(makeEventItem()));

	connect(_axis_y_nudge_fld, SIGNAL(stateChanged(int)), this, SLOT(updateNudge()));	

	_axis_autoscale_fld->insertItem(QP_AUTO_ALL_SELECTED, "Autoscale All Selected Panels");
}

void QpYAxisTab::updateNudge()
{	
	if ( QuSlotDebug() ) printf( "QpYAxisTab::updateNudge  \n" );
		
	setAxisMinMax();
}

// -----------------------------------------------

void QpYAxisTab::setAxisMinMax()
{
	if ( QuSlotDebug() )
	{
		printf( "QpYAxisTab::setAxisMinMax \n" );
		fflush(stdout);
	}

	double from, to;
	
	switch( _axis_autoscale_fld->currentIndex())
	{
		case QP_FROM_META:
		{
            bool found = I_Panel->getYmetaRange(from, to);
            if(_axis_linlog_fld->currentIndex() == QP_LOG){
                if(from <= 0) from = to * 1.0e-6;
                if(to <= 0) found = false; 
            }
			if(found)
			{
				_axis_xy_fld->setInterval(from, to);
			}
			else return;
			
			break;
		}
		case QP_FROM_DATA:
		{
			bool found = I_Panel->getYRange(from, to);
            if(_axis_linlog_fld->currentIndex() == QP_LOG){
                if(from <= 0) from = to * 1.0e-6;
                if(to <= 0) found = false; 
            }
			if(found)
			{
				_axis_xy_fld->setInterval(from, to);
			}
			else return;
			break;
		}
		case QP_AUTO_ALL_SELECTED:
		{
			// do nothing if being reset by another tab
			if(this->isVisible()) I_Item->Tree()->setYRangeAllSelected();

		//	QApplication::restoreOverrideCursor();
			
			break;
		}
		Default: case QP_FROM_USER:
		{
			// do nothing
		}
	}

}

// -------------------------------

void QpYAxisTab::setAutoLabelText()
{	

	if ( QuSlotDebug() )
		printf( "QpYAxisTab::setAutoLabelText  \n" );

	// refresh Axis Label
	DvString _label="";
	I_Panel->getLabel(Y_AXIS, _label);
	_label_fld->setText(_label.c_str());
}


  

//     QpTAxisTab CLASS
//     ---------------------

QpTAxisTab::QpTAxisTab(QTabWidget *tabWidget,  const QString& name, FrameItem * frame):
      QpAxisTab(tabWidget, name, (QpuiItem*) frame, QP_XT_TAB)
{
	// Constructor
	if ( QuSlotDebug() )
		printf( "QpTAxisTab::QpTAxisTab name = %s \n", name.toLatin1().data() );

	
	I_Frame = frame; 
	I_axis = T_AXIS;
	
	// only show forT axis
	t_drop_label->setVisible(true);
	_t_data_fld->setVisible(true);
	_frame_type_btn->setVisible(true);
	frame_type_groupbox->setVisible(true);
	frame_type_groupbox->setTitle("Frame type:   Time Series");

    _exp_label_params_groupbox->setVisible(false);
	
	tick_time_fmt_label->setVisible(true);
	tick_time_fmt_fld->setVisible(true);
	tick_time_fmt_auto_fld->setVisible(true);
	time_labels_fmt_label->setVisible(true);
	time_labels_fmt_fld->setVisible(true);
	time_labels_fmt_auto_fld->setVisible(true);

	_axis_xy_fld->setVisible(false);
	_axis_time_fld->setVisible(true);
	_axis_y_nudge_fld->setVisible(false);
	
	connect(_t_data_fld, SIGNAL(edited_OK()), this, SLOT(setTData()));

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	
	connect( _axis_time_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));
	
	connect(tick_time_fmt_fld, SIGNAL(textEdited(const QString&)), this, SLOT(setTickTimeFormat(const QString&)));
	
	connect(tick_time_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTickTimeFormatAuto(int)));
	
	connect(time_labels_fmt_fld, SIGNAL(textEdited(const QString&)), this, SLOT(setTimeLabelsFormat(const QString&)));
	
	connect(time_labels_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTimeLabelsFormatAuto(int)));

}

QpTAxisTab::QpTAxisTab(QpTAxisTab* tab, FrameItem* frame):
	QpAxisTab(tab, frame, QP_XT_TAB)
{
	// Constructor
	if ( QuSlotDebug() )
		printf( "QpTAxisTab::QpTAxisTab copy \n");

	I_Frame = frame; 
	I_axis = T_AXIS;
	
	// only show forT axis
	t_drop_label->setVisible(true);
	_t_data_fld->setVisible(true);
	_frame_type_btn->setVisible(true);
	frame_type_groupbox->setVisible(true);
	frame_type_groupbox->setTitle("Frame type:   Time Series");

    _exp_label_params_groupbox->setVisible(false);
	
	tick_time_fmt_label->setVisible(true);
	tick_time_fmt_fld->setVisible(true);
	tick_time_fmt_auto_fld->setVisible(true);
	time_labels_fmt_label->setVisible(true);
	time_labels_fmt_fld->setVisible(true);
	time_labels_fmt_auto_fld->setVisible(true);

	_axis_xy_fld->setVisible(false);
	_axis_time_fld->setVisible(true);
	_axis_y_nudge_fld->setVisible(false);
	
	connect(_t_data_fld, SIGNAL(edited_OK()), this, SLOT(setTData()));

	// attach appropriate callbacks to widgets
	
	connect( _axis_autoscale_fld, SIGNAL(activated(int)), this, SLOT(setAxisAutoScale(int)) );
	
	connect( _axis_time_fld, SIGNAL(intervalUpdated()), this, SLOT(setAxisRange()) );
	
	connect(_axis_major_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMajorTicks(int)));
	connect( _axis_major_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMajorTickMarks(int)) );
	
	connect(_axis_minor_ticks_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setAxisNumMinorTicks(int)));
	connect( _axis_minor_tickmarks_fld, SIGNAL(activated(int)), this, SLOT(setAxisMinorTickMarks(int)) );
	
	connect( _label_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setAxisLabel()) );

	connect(_label_char_size_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelCharSize(double)));

	connect(_label_disp_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelDisp(double)));

	connect( _axis_autolabel_fld, SIGNAL(toggled(bool)), this, SLOT(setAxisAutoLabel(bool)) );

	connect(_label_rotation_fld, SIGNAL(valueChanged(double)), this, SLOT(setAxisLabelRotation(double)));

	connect( _label_font_fld, SIGNAL(familyChanged(int)), this, SLOT(setAxisLabelFamily(int)) );
	connect( _label_font_fld, SIGNAL(styleChanged(bool)), this, SLOT(setAxisLabelStyle(bool)) );
	connect( _label_font_fld, SIGNAL(weightChanged(bool)), this, SLOT(setAxisLabelWeight(bool)) );

	connect(_label_colour_fld, SIGNAL(clicked()), this, SLOT(setAxisLabelColour()));
	
	connect(tick_time_fmt_fld, SIGNAL(textEdited(const QString&)), this, SLOT(setTickTimeFormat(const QString&)));
	
	connect(tick_time_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTickTimeFormatAuto(int)));
	
	connect(time_labels_fmt_fld, SIGNAL(textEdited(const QString&)), this, SLOT(setTimeLabelsFormat(const QString&)));
	
	connect(time_labels_fmt_auto_fld, SIGNAL(stateChanged(int)), this, SLOT(setTimeLabelsFormatAuto(int)));
	

}

//  ---------------------

void QpTAxisTab::setTData()
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTData  \n" );
		fflush(stdout);
	}

	this->setTfromObj();

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
		setSlotColour(true);
		_t_data_fld->lineBox->setPalette(GUIpalette);
 		I_Item->Tree()->setAxisOption(AXIS_T_DATA, T_AXIS, 0, 0.0, this->_t_data_fld->text());
	}
}

void QpTAxisTab::setTickTimeFormat(const QString& str)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTickTimeFormat  \n" );
		fflush(stdout);
	}

	if(tick_time_fmt_fld->hasFocus())
	{
		// only change if user input
		tick_time_fmt_auto_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
 		if(this->isVisible() ) 	I_Item->Tree()->setAxisOption(AXIS_TICK_TIME_FORMAT, T_AXIS, 0, 0.0, str);
	}

}

void QpTAxisTab::setTickTimeFormatAuto(int state)
{
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTickTimeFormatAuto  %d\n", state );
		fflush(stdout);
	}
	
	if(state) tick_time_fmt_fld->clearFocus();
	setSlotColour(!state);
	tick_time_fmt_fld->setPalette(GUIpalette);
	
	if(state) tick_time_fmt_fld->setText(" ");

	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
 		I_Item->Tree()->setAxisOption(AXIS_TICK_TIME_FORMAT_AUTO, T_AXIS, state, 0.0, "");
	}
}

void QpTAxisTab::setTimeLabelsFormat(const QString& str)
{
	if ( QuSlotDebug() ) printf( "QpTAxisTab::setTimeLabelsFormat  \n" );
		
	
	if(time_labels_fmt_fld->hasFocus())
	{
		// only change if user input
		time_labels_fmt_auto_fld->setChecked(false);
	
		// only tell list view to make changes to selected frame items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
 		if(this->isVisible() ) I_Item->Tree()->setAxisOption(AXIS_TIME_LABELS_FORMAT, T_AXIS, 0, 0.0, str);
	}
}

void QpTAxisTab::setTimeLabelsFormatAuto(int state)
{
	if ( QuSlotDebug() ) printf( "QpTAxisTab::setTimeLabelFormatAuto  %d\n",  state );
		
	
	if(state) 
	{
		time_labels_fmt_fld->clearFocus();
		time_labels_fmt_fld->setText(" ");
	}
	setSlotColour(!state);
	time_labels_fmt_fld->setPalette(GUIpalette);
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
	if(this->isVisible() ) {
 		I_Item->Tree()->setAxisOption(AXIS_TIME_LABELS_FORMAT_AUTO, T_AXIS, state, 0.0, "");
	}
}


// ---------------------------

void QpTAxisTab::setTfromObj()
{
	// slot called when t data name field changed on tab
	if ( QuSlotDebug() )
	{
		printf( "QpTAxisTab::setTfromObj  \n" );
		fflush(stdout);
	}
	
	// Fetch timeinterval_var object
	DvObject_var tobj = _t_data_fld->getObject();
	if(tobj.is_nil()) return;
	
	if(tobj->is_event()) {
		_axis_time_fld->setInterval(tobj->event(0));
		handleAutoScale(QP_FROM_USER);
	}
	else{
		DvEvent tint = tobj->getTimeRange();
		// update range
		_axis_time_fld->setInterval(tint);
		handleAutoScale(QP_FROM_USER);
	}		
		refreshYZRanges();

}

// -----------------------------------

void QpTAxisTab::setAxisMinMax()
{
	// slot called when autoscale is set on or object changed
	if ( QuSlotDebug() ) printf( "QpTAxisTab::setAxisMinMax  \n" );

    double from, to;

	switch( _axis_autoscale_fld->currentIndex())
	{
	
		case QP_FROM_META:
		{
			if(I_Frame->getXmetaRange(from, to))
			{
				DvEvent interval(from, to);
				_axis_time_fld->setInterval(interval);
				_axis_time_fld->setAltPalette(true);
				break;
			}
			// else fall through to use data
		}
		case QP_FROM_DATA:
		{
			QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
			bool found = I_Frame->getXRange(from, to);
			QApplication::restoreOverrideCursor();
			
			if(found)
			{
				DvEvent interval(from, to);
				_axis_time_fld->setInterval(interval);
				_axis_time_fld->setAltPalette(true);
			}
		}
		Default: case QP_FROM_USER:
		{
			// ensure objects are set for Y range
			I_Frame->getYObjects();
		}
	}
}

// -------------------------------

void QpTAxisTab::setAutoLabelText()
{	
	if ( QuSlotDebug() )
		printf( "QpTAxisTab::setAutoLabelText  \n" );

	// refresh Axis Label
	_label_fld->setText("Universal Time");
}

// -------------------------------

void QpTAxisTab::interactiveRescale(DvEvent interval)
{

	if ( QuSlotDebug() ) printf( "QpTAxisTab::interactiveRescale  \n" );

	_axis_time_fld->setInterval(interval);
    handleAutoScale(QP_FROM_USER);

    refreshYZRanges();
	
	PlotWindow* win=(PlotWindow *) sender();
	if(!win) return;
	
	win->setPLFocus(); // ensure replot goes to correct window
	I_Frame->page()->plot(win);
}


QDomElement QpAxisTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Axis_Tab");
    
    xml.appendChild(createTextXMLNode(doc, "Autoscale", _axis_autoscale_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Autolabel", _axis_autolabel_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "X_Start", _axis_xy_fld->getStart()));
    xml.appendChild(createTextXMLNode(doc, "X_End", _axis_xy_fld->getEnd()));
    xml.appendChild(createTextXMLNode(doc, "Time_Start", _axis_time_fld->getStart().getISOstring(0)));
    xml.appendChild(createTextXMLNode(doc, "Time_End", _axis_time_fld->getEnd().getISOstring(0)));

    xml.appendChild(createTextXMLNode(doc, "Nudge", _axis_y_nudge_fld->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "Major_Ticks_Num", _axis_major_ticks_num_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Minor_Ticks_Num", _axis_minor_ticks_num_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Major_Tickmarks", _axis_major_tickmarks_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Minor_Tickmarks", _axis_minor_tickmarks_fld->currentIndex()));

    xml.appendChild(createTextXMLNode(doc, "Time_Tick_Format", tick_time_fmt_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Time_Tick_Auto", tick_time_fmt_auto_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "Time_Label_Format", time_labels_fmt_fld->text()));
    xml.appendChild(createTextXMLNode(doc, "Time_Label_Auto", time_labels_fmt_auto_fld->isChecked()));

    xml.appendChild(createTextXMLNode(doc, "Exp_Label_X_Offset", _exp_label_offset_x_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Exp_Label_Y_Offset", _exp_label_offset_y_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Label", _label_fld->text()));

    xml.appendChild(createTextXMLNode(doc, "Label_Char_Size", _label_char_size_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Label_Char_Disp", _label_disp_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Label_Char_Rotation", _label_rotation_fld->value()));

    PLINT family, style, weight;
    _label_font_fld->getFontParams(family, style, weight);
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Family", family));
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Style", style));
    xml.appendChild(createTextXMLNode(doc, "Label_Font_Weight", weight));

    xml.appendChild(createTextXMLNode(doc, "Label_Red", _label_colour.red()));
    xml.appendChild(createTextXMLNode(doc, "Label_Green", _label_colour.green()));
    xml.appendChild(createTextXMLNode(doc, "Label_Blue", _label_colour.blue()));

    return xml;
}

void QpAxisTab::fromXMLElement(QDomElement& el)
{
    QString s;
    int autoscale(0);
    if(getXMLNodeText(el, "Autoscale", s))
    {
        autoscale=s.toInt();
        _axis_autoscale_fld->setCurrentIndex(autoscale);
    }
    if(getXMLNodeText(el, "Autolabel", s))
    {
        _axis_autolabel_fld->setChecked(s.toInt());
    }
    QString from(""), to("");
    if(getXMLNodeText(el, "X_Start", s))
    {
        from=s;
    }
    if(getXMLNodeText(el, "X_End", s))
    {
        to=s;
    }
    _axis_xy_fld->setInterval(from.toDouble(), to.toDouble());
    _axis_xy_fld->resetView();

    from="";
    to="";
    if(getXMLNodeText(el, "Time_Start", s))
    {
        from=s;
    }
    if(getXMLNodeText(el, "Time_End", s))
    {
        to=s;
    }
    _axis_time_fld->setInterval(DvTime(from.toLatin1().data()), DvTime(to.toLatin1().data()));
    _axis_time_fld->resetView();
    

    if(autoscale==QP_FROM_USER)
    {
        _axis_time_fld->setAltPalette(false);
        _axis_xy_fld->setAltPalette(false);
    }
    else
    {
        _axis_time_fld->setAltPalette(true);
        _axis_xy_fld->setAltPalette(true);
    }

    if(getXMLNodeText(el, "Nudge", s))
    {
        _axis_y_nudge_fld->setChecked(s.toInt());
    }

    if(getXMLNodeText(el, "Major_Ticks_Num", s))
    {
        _axis_major_ticks_num_fld->setValue(s.toInt());
    }
    if(getXMLNodeText(el, "Minor_Ticks_Num", s))
    {
        _axis_minor_ticks_num_fld->setValue(s.toInt());
    }
    if(getXMLNodeText(el, "Major_Tickmarks", s))
    {
        _axis_major_tickmarks_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Minor_Tickmarks", s))
    {
        _axis_minor_tickmarks_fld->setCurrentIndex(s.toInt());
    }

    if(getXMLNodeText(el, "Time_Tick_Format", s))
    {
        tick_time_fmt_fld->setText(s);
    }
    if(getXMLNodeText(el, "Time_Tick_Auto", s))
    {
        tick_time_fmt_auto_fld->setChecked(s.toInt());
    }
    if(getXMLNodeText(el, "Time_Label_Format", s))
    {
        time_labels_fmt_fld->setText(s);
    }
    if(getXMLNodeText(el, "Time_Label_Auto", s))
    {
        time_labels_fmt_auto_fld->setChecked(s.toInt());
    }

    if(getXMLNodeText(el, "Exp_Label_X_Offset", s))
    {
        _exp_label_offset_x_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Exp_Label_Y_Offset", s))
    {
        _exp_label_offset_y_fld->setValue(s.toDouble());
    }

    if(getXMLNodeText(el, "Label", s))
    {
        _label_fld->setText(s);
    }
    if(getXMLNodeText(el, "Label_Char_Size", s))
    {
        _label_char_size_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Label_Char_Disp", s))
    {
        _label_disp_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Label_Char_Rotation", s))
    {
        _label_rotation_fld->setValue(s.toDouble());
    }

    PLINT family(PL_FCI_SANS), style(0), weight(0);
    if(getXMLNodeText(el, "Label_Font_Family", s))
    {
        family=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Font_Style", s))
    {
        style=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Font_Weight", s))
    {
        weight=s.toInt();
    }
    switch(family)
    {
        case PL_FCI_SERIF: _label_font_fld->setSerifFont(); break;
        case PL_FCI_MONO: _label_font_fld->setMonotypeFont(); break;
        case PL_FCI_SCRIPT: _label_font_fld->setDecorativeFont(); break;
        default: _label_font_fld->setSansFont(); break;
    }
    _label_font_fld->setItalic(style==PL_FCI_ITALIC);
    _label_font_fld->setBold(weight==PL_FCI_BOLD);
    
    int r(0), g(0), b(0);
    if(getXMLNodeText(el, "Label_Red", s))
    {
        r=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Green", s))
    {
        g=s.toInt();
    }
    if(getXMLNodeText(el, "Label_Blue", s))
    {
        b=s.toInt();
    }
    _label_colour = QColor(r,g,b);
    QPalette line_palette = QPalette(QPUI_normal_palette);
    line_palette.setColor(QPalette::Active ,QPalette::ButtonText, _label_colour);
    line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, _label_colour);
    _label_colour_fld->setPalette(line_palette);
}


