// qpui_page_options.cc
//
// Plot Interface Implementation - Page Options 
//
// Janet Barnes
// July 2002
//
//  Tony Allen
// February 2008

#include "qpui.h"
#include "qdutil.h"
#include <qtooltip.h>

//  QpLayoutTab Class
// ------------------


QpLayoutTab::QpLayoutTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page):
		QpTab(tabWidget, name, QP_LAYOUT_TAB)

{
	// Constructor for the Layout Tab widget shown in the Page Options

	I_Page = _page;
	
	create();

	connect(_row_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumRows(int)));
	connect(_col_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumCols(int)));
	connect( _autolayout_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLayout(bool)) );
	connect(_hgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setHGap(double)));
	connect(_vgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setVGap(double)));
	connect(_lmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setLMargin(double)));
	connect(_rmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setRMargin(double)));
	connect(_tmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setTMargin(double)));
	connect(_bmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setBMargin(double)));
	
}

QpLayoutTab::QpLayoutTab(QpLayoutTab* tab, PageItem* _page):
		QpTab((QTabWidget*)(tab->parent()), "Page Layout", QP_LAYOUT_TAB)
{
	I_Page=_page;
	
	create();
	
	//Copy parameters from input tab
	
	_autolayout_fld->setChecked(tab->_autolayout_fld->isChecked());
	_hgap_fld->setValue(tab->_hgap_fld->value());
	_vgap_fld->setValue(tab->_vgap_fld->value());
	_lmargin_fld->setValue(tab->_lmargin_fld->value());
	_rmargin_fld->setValue(tab->_rmargin_fld->value());
	_tmargin_fld->setValue(tab->_tmargin_fld->value());
	_bmargin_fld->setValue(tab->_bmargin_fld->value());
	
	_row_num_fld->setValue(tab->_row_num_fld->value());
	_col_num_fld->setValue(tab->_col_num_fld->value());
	
	connect(_row_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumRows(int)));
	connect(_col_num_fld, SIGNAL(valueChanged(int)), this, SLOT(setNumCols(int)));
	connect( _autolayout_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLayout(bool)) );
	connect(_hgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setHGap(double)));
	connect(_vgap_fld, SIGNAL(valueChanged(double)), this, SLOT(setVGap(double)));
	connect(_lmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setLMargin(double)));
	connect(_rmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setRMargin(double)));
	connect(_tmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setTMargin(double)));
	connect(_bmargin_fld, SIGNAL(valueChanged(double)), this, SLOT(setBMargin(double)));
}


void QpLayoutTab::create()
{
	QVBoxLayout *page_layout_top_layout = new QVBoxLayout(this);
	page_layout_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_layout_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_panels_groupbox = new QGroupBox(this);

	page_panels_groupbox->setTitle("Frame Layout");

// 	QWidget* page_panels_widget = new QWidget(page_panels_groupbox, "page_panels_widget");

	QGridLayout *page_panels_layout = new QGridLayout;
	page_panels_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_panels_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* row_num_label = new QLabel("Number of rows: ");
	page_panels_layout->addWidget(row_num_label, 0, 0, Qt::AlignRight);
	_row_num_fld = new QSpinBox;
	_row_num_fld->setMinimum(1);
	_row_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_row_num_fld, 0, 1, Qt::AlignLeft);

	QLabel* col_num_label = new QLabel("Number of columns: ");
	page_panels_layout->addWidget(col_num_label, 0, 2, Qt::AlignRight);
	_col_num_fld = new QSpinBox;
	_col_num_fld->setMinimum(1);
	_col_num_fld->setFixedWidth(100);
	page_panels_layout->addWidget(_col_num_fld, 0, 3, Qt::AlignLeft);

	_autolayout_fld = new QCheckBox("Autolayout Frames");
	page_panels_layout->addWidget(_autolayout_fld, 1, 0, 1, 2);
	_autolayout_fld->setChecked(true);

// 	page_panels_layout->addRowSpacing(0,0);
// 	page_panels_layout->addRowSpacing(1, _row_num_fld->height());
// 	page_panels_layout->addRowSpacing(2,0);

	page_panels_layout->setRowStretch(0,0);
	page_panels_layout->setRowStretch(1,1);
	page_panels_layout->setRowStretch(2,0);

	page_panels_layout->setColumnStretch(0,0);
	page_panels_layout->setColumnStretch(1,0);
	page_panels_layout->setColumnStretch(2,0);
	page_panels_layout->setColumnStretch(3,0);
	page_panels_layout->setColumnStretch(4,1);

	page_panels_groupbox->setLayout(page_panels_layout);
	
	page_layout_top_layout->addWidget(page_panels_groupbox);

	QGroupBox *page_margins_groupbox = new QGroupBox(this);

	page_margins_groupbox->setTitle("Frame Margins for Autolayout");

// 	QWidget* page_margins_widget = new QWidget(page_margins_groupbox, "page_margins_widget");

	QGridLayout *page_margins_layout = new QGridLayout;
	page_margins_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_margins_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* hgap_label = new QLabel("Horizontal gap: ");
	page_margins_layout->addWidget(hgap_label, 1, 0, Qt::AlignRight);
	_hgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "hgap");
	_hgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_hgap_fld, 1, 1, Qt::AlignLeft);
	_hgap_fld->setValue(QPUI_PAGE_HGAP);

	QLabel* vgap_label = new QLabel("Vertical gap: ");
	page_margins_layout->addWidget(vgap_label, 1, 2, Qt::AlignRight);
	_vgap_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "vgap");
	_vgap_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_vgap_fld, 1, 3, Qt::AlignLeft);
	_vgap_fld->setValue(QPUI_PAGE_VGAP);

	QLabel* lmargin_label = new QLabel("Left margin: ", page_margins_groupbox);
	page_margins_layout->addWidget(lmargin_label, 3, 0, Qt::AlignRight);
	_lmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "lmargin");
	_lmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_lmargin_fld, 3, 1, Qt::AlignLeft);
	_lmargin_fld->setValue(QPUI_PAGE_LMARGIN);

	QLabel* rmargin_label = new QLabel("Right margin: ");
	page_margins_layout->addWidget(rmargin_label, 3, 2, Qt::AlignRight);
	_rmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "rmargin");
	_rmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_rmargin_fld, 3, 3, Qt::AlignLeft);
	_rmargin_fld->setValue(QPUI_PAGE_RMARGIN);

	QLabel* tmargin_label = new QLabel("Top margin: ", page_margins_groupbox);
	page_margins_layout->addWidget(tmargin_label, 5, 0, Qt::AlignRight);
	_tmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "tmargin");
	_tmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_tmargin_fld, 5, 1, Qt::AlignLeft);
	_tmargin_fld->setValue(QPUI_PAGE_TMARGIN);

	QLabel* bmargin_label = new QLabel("Bottom margin: ", page_margins_groupbox);
	page_margins_layout->addWidget(bmargin_label, 5, 2, Qt::AlignRight);
	_bmargin_fld = new FloatSpinBox(0., 1., 2, 0.05, page_margins_groupbox, "bmargin");
	_bmargin_fld->setFixedWidth(100);
	page_margins_layout->addWidget(_bmargin_fld, 5, 3, Qt::AlignLeft);
	_bmargin_fld->setValue(QPUI_PAGE_BMARGIN);


	page_margins_layout->setRowStretch(0,0);
	page_margins_layout->setRowStretch(1,1);
	page_margins_layout->setRowStretch(2,0);
	page_margins_layout->setRowStretch(3,1);
	page_margins_layout->setRowStretch(4,0);
	page_margins_layout->setRowStretch(5,1);
	page_margins_layout->setRowStretch(6,0);

	page_margins_layout->setColumnStretch(0,0);
	page_margins_layout->setColumnStretch(1,0);
	page_margins_layout->setColumnStretch(2,0);
	page_margins_layout->setColumnStretch(3,0);
	page_margins_layout->setColumnStretch(4,1);

	page_margins_groupbox->setLayout(page_margins_layout);
	
	page_layout_top_layout->addWidget(page_margins_groupbox);

	page_layout_top_layout->addSpacing(4);
	page_layout_top_layout->addStretch(10);
}
// ----------------------------------------------

QDomElement QpLayoutTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Layout_Tab");
    xml.appendChild(createTextXMLNode(doc, "Num_Rows", _row_num_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Num_Cols", _col_num_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "Autolayout", _autolayout_fld->isChecked()));
    xml.appendChild(createTextXMLNode(doc, "H_Gap", _hgap_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "V_Gap", _vgap_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "L_Margin", _lmargin_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "R_Margin", _rmargin_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "T_Margin", _tmargin_fld->value()));
    xml.appendChild(createTextXMLNode(doc, "B_Margin", _bmargin_fld->value()));

    return xml;
}

void QpLayoutTab::fromXMLElement(QDomElement& el)
{
    QString s;
    if(getXMLNodeText(el, "Autolayout", s))
    {
        _autolayout_fld->setChecked(s.toInt());
    }
    if(getXMLNodeText(el, "H_Gap", s))
    {
        _hgap_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "V_Gap", s))
    {
        _vgap_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "L_Margin", s))
    {
        _lmargin_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "R_Margin", s))
    {
        _rmargin_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "T_Margin", s))
    {
        _tmargin_fld->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "B_Margin", s))
    {
        _bmargin_fld->setValue(s.toDouble());
    }
    

    // Num rows and Num cols are done at page level (must be done after reading the frames)
}


void QpLayoutTab::setAutoLayout(bool bChoice)
{
   // only tell list view to make changes to selected panel items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	if(QuSlotDebug()) printf("QpLayoutTab::setAutoLayout\n");

   I_Page->setFramesAuto(bChoice);
   
	
	// propagate to other selected pages if this is the one user modified
 	if(this->isVisible() ) I_Page->Tree()->setPageOption(AUTOLAYOUT, 0, 0.0, "", bChoice);

}

// ----------------------------------------------

void QpLayoutTab::updateRowCol(int numFrames)
{
	if(QuSlotDebug()) printf("QpLayoutTab::updateRowCol\n");
		
	
	if( numFrames <= 0) return;

	// called when Frame created or destroyed
	int row_num = (int) ceil((float) numFrames / _col_num_fld->value());
	_row_num_fld->clearFocus();
	_row_num_fld->setValue(row_num); 

    I_Page->setLayout();
	
}

void QpLayoutTab::setNumRows(int row_num)
{
	if(QuSlotDebug()) printf("QpLayoutTab::setNumRows\n");
	
	int col_num = (int) ceil((float) I_Page->childCount() / row_num);
	_col_num_fld->clearFocus();
	_col_num_fld->setValue(col_num); 

    I_Page->setLayout();
	
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(_row_num_fld->hasFocus() ) I_Page->Tree()->setPageOption(NUM_ROWS, row_num);
}

void QpLayoutTab::setNumCols(int col_num)
{
	if(QuSlotDebug()) printf("QpLayoutTab::setNumCols\n");

	int row_num = (int) ceil((float) I_Page->childCount()  / col_num);
	_row_num_fld->clearFocus();
	_row_num_fld->setValue(row_num); 

    I_Page->setLayout();
	
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(_col_num_fld->hasFocus() ) I_Page->Tree()->setPageOption(NUM_COLS, col_num);
}

void QpLayoutTab::setHGap(double gap)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	
	if(QuSlotDebug()) printf("QpLayoutTab::setHGap\n");

	I_Page->setLayout();

 	if(_hgap_fld->hasFocus() ) I_Page->Tree()->setPageOption(HGAP, 0, gap);
}

void QpLayoutTab::setVGap(double gap)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(QuSlotDebug()) printf("QpLayoutTab::setVGap\n");
      
	I_Page->setLayout();

 	if(_vgap_fld->hasFocus() ) I_Page->Tree()->setPageOption(VGAP, 0, gap);
}

void QpLayoutTab::setLMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(QuSlotDebug()) printf("QpLayoutTab::setLMargin\n");
	
	I_Page->setLayout();

 	if(_lmargin_fld->hasFocus() ) I_Page->Tree()->setPageOption(LMARGIN, 0, margin);
}

void QpLayoutTab::setRMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(QuSlotDebug()) printf("QpLayoutTab::setRMargin\n");
	
	I_Page->setLayout();

 	if(_rmargin_fld->hasFocus() ) I_Page->Tree()->setPageOption(RMARGIN, 0, margin);
}

void QpLayoutTab::setTMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(QuSlotDebug()) printf("QpLayoutTab::setTMargin\n");

	I_Page->setLayout();

 	if(_tmargin_fld->hasFocus() ) I_Page->Tree()->setPageOption(TMARGIN, 0, margin);
}

void QpLayoutTab::setBMargin(double margin)
{
	// only tell list view to make changes to selected page items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(QuSlotDebug()) printf("QpLayoutTab::setBMargin\n");

	I_Page->setLayout();

 	if(_bmargin_fld->hasFocus() ) I_Page->Tree()->setPageOption(TMARGIN, 0, margin);
}

   

// QpPageTab Class
//----------------


QpPageTab::QpPageTab(QTabWidget *tabWidget,  const QString& name, PageItem * _page):
		QpTab(tabWidget, name, QP_PAGE_TAB)
{
	// Constructor for the Page Tab widget shown in the Page Options
	I_Page = _page;
	create();
	
	connect( _page_size_fld, SIGNAL(activated(int)), this, SLOT(setPageSize(int)) );
	connect( _units_fld, SIGNAL(activated(int)), this, SLOT(setUnits(int)) );
	connect(_dpi_fld, SIGNAL(activated(int)), this, SLOT(setDPI(int)));
	connect( _width_fld, SIGNAL(textChanged(const QString&)),this, SLOT(setWidth(const QString&)) );
	connect( _height_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setHeight(const QString&)) );
	connect(_orientation_fld, SIGNAL(activated(int)), this, SLOT(setOrientation(int)) );
	connect( _show_ident_fld, SIGNAL(toggled(bool)), this, SLOT(setIdent(bool)) );

// 	page_size_widget->adjustSize();
}


QpPageTab::QpPageTab(QpPageTab* tab, PageItem* _page):QpTab((QTabWidget*)(tab->parent()), "Page Layout", QP_PAGE_TAB)
{
	I_Page = _page;
	create();
	
	// Copy parameters from tab
	
	_page_size_fld->setCurrentIndex(tab->_page_size_fld->currentIndex());
	_units_fld->setCurrentIndex(tab->_units_fld->currentIndex());
	oldUnits=tab->oldUnits;
	
	bool readonly=(_page_size_fld->currentIndex()!= QPUI_CUSTOM);
	_width_fld->setReadOnly(readonly);
	_height_fld->setReadOnly(readonly);
	_width_fld->setEnabled(!readonly);
	_height_fld->setEnabled(!readonly);
	
	_dpi_fld->setCurrentIndex(tab->_dpi_fld->currentIndex());
	
	switch(_dpi_fld->currentIndex())
	{
		case QPUI_150_DPI:
			_page->_conversion_factors[QPUI_PIXELS]=150.;
			_page->_conversion_factors[QPUI_PSPOINTS]=150.;
			break;
			
		case QPUI_300_DPI:
			_page->_conversion_factors[QPUI_PIXELS]=300.;
			_page->_conversion_factors[QPUI_PSPOINTS]=300.;
			break;
			
		case QPUI_600_DPI:
			_page->_conversion_factors[QPUI_PIXELS]=600.;
			_page->_conversion_factors[QPUI_PSPOINTS]=600.;
			break;
			
		case QPUI_1200_DPI:
			_page->_conversion_factors[QPUI_PIXELS]=1200.;
			_page->_conversion_factors[QPUI_PSPOINTS]=1200.;
			break;
			
		default:
			_page->_conversion_factors[QPUI_PIXELS]=72.;
			_page->_conversion_factors[QPUI_PSPOINTS]=72.;
			break;
	}
	
	_width_fld->setText(tab->_width_fld->text());
	_height_fld->setText(tab->_height_fld->text());
	
	_orientation_fld->setCurrentIndex(tab->_orientation_fld->currentIndex());
	_xwin_num_fld->setCurrentIndex(tab->_xwin_num_fld->currentIndex());
	
	_show_ident_fld->setChecked(tab->_show_ident_fld->isChecked());
	
	connect( _page_size_fld, SIGNAL(activated(int)), this, SLOT(setPageSize(int)) );
	connect( _units_fld, SIGNAL(activated(int)), this, SLOT(setUnits(int)) );
	connect(_dpi_fld, SIGNAL(activated(int)), this, SLOT(setDPI(int)));
	connect( _width_fld, SIGNAL(textChanged(const QString&)),this, SLOT(setWidth(const QString&)) );
	connect( _height_fld, SIGNAL(textChanged(const QString&)), this, SLOT(setHeight(const QString&)) );
	connect(_orientation_fld, SIGNAL(activated(int)), this, SLOT(setOrientation(int)) );
	connect( _show_ident_fld, SIGNAL(toggled(bool)), this, SLOT(setIdent(bool)) );
}

void QpPageTab::create()
{

	QVBoxLayout *page_properties_top_layout=new QVBoxLayout(this);
	page_properties_top_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_properties_top_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QGroupBox *page_size_groupbox = new QGroupBox(this);
	page_size_groupbox->setTitle("Size");

// 	QWidget* page_size_widget = new QWidget(page_size_groupbox, "page_size_widget");
	QGridLayout *page_size_layout = new QGridLayout;
	page_size_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_size_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* page_size_label = new QLabel("Page size: ");
	page_size_layout->addWidget(page_size_label, 1, 0, Qt::AlignRight);
	_page_size_fld = new QComboBox();
	_page_size_fld->insertItem(QPUI_A4, "A4");
	_page_size_fld->insertItem(QPUI_CUSTOM, "Custom");
	_page_size_fld->insertItem(QPUI_EXECUTIVE, "Executive");
	_page_size_fld->insertItem(QPUI_LEGAL, "Legal");
	_page_size_fld->insertItem(QPUI_LETTER, "Letter");
	_page_size_fld->insertItem(QPUI_DEFAULT, "Default");

	QFontMetrics page_size_fm(_page_size_fld->font());
	int page_size_w = page_size_fm.width("Executive");
	int page_size_h = page_size_fm.height();
	_page_size_fld->setMinimumSize(page_size_w, page_size_h);

	_page_size_fld->setToolTip("Select page size");
	page_size_layout->addWidget(_page_size_fld, 1, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageSize=="A4")
		_page_size_fld->setCurrentIndex(QPUI_A4);
	else if(QSAS_profiles->current()->PageSize=="Executive")
		_page_size_fld->setCurrentIndex(QPUI_EXECUTIVE);
	else if(QSAS_profiles->current()->PageSize=="Legal")
		_page_size_fld->setCurrentIndex(QPUI_LEGAL);
	else if(QSAS_profiles->current()->PageSize=="Letter")
		_page_size_fld->setCurrentIndex(QPUI_LETTER);
	else if(QSAS_profiles->current()->PageSize=="Default")
		_page_size_fld->setCurrentIndex(QPUI_DEFAULT);
	else _page_size_fld->setCurrentIndex(QPUI_CUSTOM);
	
	QLabel* units_label = new QLabel("Units: ");
	page_size_layout->addWidget(units_label, 1, 2, Qt::AlignRight);
	_units_fld = new QComboBox();
	_units_fld->insertItem(QPUI_PIXELS, "pixels");
	_units_fld->insertItem(QPUI_INCHES, "inches");
	_units_fld->insertItem(QPUI_CM, "cm");
	_units_fld->insertItem(QPUI_MM, "mm");
	_units_fld->insertItem(QPUI_PSPOINTS, "PS points");
	QFontMetrics units_fm(_units_fld->font());
	int units_w = units_fm.width("PS points");
	int units_h = page_size_fm.height();
	_units_fld->setMinimumSize(units_w, units_h);

	_units_fld->setToolTip("Select units for page size");
	page_size_layout->addWidget(_units_fld, 1, 3, Qt::AlignLeft);

	oldUnits = _units_fld->currentIndex();
	
	QLabel* dpi_label = new QLabel("Resolution: ");
	page_size_layout->addWidget(dpi_label, 1, 4, Qt::AlignRight);
	_dpi_fld=new QComboBox();
	_dpi_fld->insertItem(QPUI_72_DPI, "72 dpi");
	_dpi_fld->insertItem(QPUI_150_DPI, "150 dpi");
	_dpi_fld->insertItem(QPUI_300_DPI, "300 dpi");
	_dpi_fld->insertItem(QPUI_600_DPI, "600 dpi");
	_dpi_fld->insertItem(QPUI_1200_DPI, "1200 dpi");
	switch(QSAS_profiles->current()->PageDPI)
	{
		case 150:
			I_Page->_conversion_factors[QPUI_PIXELS]=150.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
			_dpi_fld->setCurrentIndex(QPUI_150_DPI);
			break;
		case 300:
			I_Page->_conversion_factors[QPUI_PIXELS]=300.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
			_dpi_fld->setCurrentIndex(QPUI_300_DPI);
			break;
		case 600:
			I_Page->_conversion_factors[QPUI_PIXELS]=600.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
			_dpi_fld->setCurrentIndex(QPUI_600_DPI);
			break;
		case 1200:
			I_Page->_conversion_factors[QPUI_PIXELS]=1200.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=1200.;
			_dpi_fld->setCurrentIndex(QPUI_1200_DPI);
			break;
		default:
			I_Page->_conversion_factors[QPUI_PIXELS]=72.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
			_dpi_fld->setCurrentIndex(QPUI_72_DPI);
			break;
	}
    
	page_size_layout->addWidget(_dpi_fld, 1, 5, Qt::AlignLeft);
	
	QLabel* width_label = new QLabel("Width: ");
	page_size_layout->addWidget(width_label, 3, 0, Qt::AlignRight);
	_width_fld = new QLineEdit;
	_width_fld->setFixedWidth(120);

	page_size_layout->addWidget(_width_fld, 3, 1, Qt::AlignLeft);

	QLabel* height_label = new QLabel("Height: ");
	page_size_layout->addWidget(height_label, 3, 2, Qt::AlignRight);
	_height_fld = new QLineEdit;
	_height_fld->setFixedWidth(120);
	page_size_layout->addWidget(_height_fld, 3, 3, Qt::AlignLeft);
	
	
	// set appropriate text in width and height fields
	if(QSAS_profiles->current()->PageSize=="Custom")
	{
		double conv = I_Page->_conversion_factors[_units_fld->currentIndex()];
		double w=QSAS_profiles->current()->PageWidth;
		double h=QSAS_profiles->current()->PageHeight;
		_width_fld->setText(QString().setNum(w*conv));
		_height_fld->setText(QString().setNum(h*conv));
	}
	setPageSize(_page_size_fld->currentIndex());

	QLabel* orientation_label = new QLabel("Orientation: ");
	page_size_layout->addWidget(orientation_label, 5, 0, Qt::AlignRight);
	_orientation_fld = new QComboBox();
	_orientation_fld->insertItem(QPUI_PORTRAIT, "Portrait");
	_orientation_fld->insertItem(QPUI_LANDSCAPE, "Landscape");
	QFontMetrics orientation_fm(_orientation_fld->font());
	int orientation_w = orientation_fm.width("Landscape");
	int orientation_h = orientation_fm.height();
	_orientation_fld->setMinimumSize(orientation_w, orientation_h);

	_orientation_fld->setToolTip("Select portrait or landscape");
	page_size_layout->addWidget(_orientation_fld, 5, 1, Qt::AlignLeft);
	
	if(QSAS_profiles->current()->PageOrientation=="Portrait")
		_orientation_fld->setCurrentIndex(QPUI_PORTRAIT);
	else _orientation_fld->setCurrentIndex(QPUI_LANDSCAPE);

	page_size_layout->setRowStretch(0,0);
	page_size_layout->setRowStretch(1,1);
	page_size_layout->setRowStretch(2,0);
	page_size_layout->setRowStretch(3,1);
	page_size_layout->setRowStretch(4,0);
	page_size_layout->setRowStretch(5,1);
	page_size_layout->setRowStretch(6,0);

	page_size_layout->setColumnStretch(0,0);
	page_size_layout->setColumnStretch(1,0);
	page_size_layout->setColumnStretch(2,0);
	page_size_layout->setColumnStretch(3,0);
	page_size_layout->setColumnStretch(4,0);
	page_size_layout->setColumnStretch(5,0);
	page_size_layout->setColumnStretch(6,1);
	
	page_size_groupbox->setLayout(page_size_layout);

	page_properties_top_layout->addWidget(page_size_groupbox);

	QGroupBox *page_output_groupbox = new QGroupBox(this);

	page_output_groupbox->setTitle("Output");

// 	QWidget* page_output_widget = new QWidget(page_output_groupbox, "page_output_widget");

	page_output_layout = new QGridLayout;
	page_output_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	page_output_layout->setMargin(QSAS_profiles->current()->GUImargins);

    _xwin_num_fld = new PlotID(" Window:", I_Page->Tree()->plotWindows);
	page_output_layout->addWidget(_xwin_num_fld, 1, 0, Qt::AlignLeft);

    QLabel *pad = new QLabel("        ");
    page_output_layout->addWidget(pad, 1, 1);

	_show_ident_fld = new QCheckBox("Show user identification");
	_show_ident_fld->setChecked(true);
	page_output_layout->addWidget(_show_ident_fld, 1, 2, Qt::AlignRight);

	page_output_layout->setRowStretch(0,0);
	page_output_layout->setRowStretch(1,1);
	page_output_layout->setRowStretch(2,0);
	page_output_layout->setRowStretch(3,1);
	page_output_layout->setRowStretch(4,0);

	page_output_layout->setColumnStretch(0,0);
	page_output_layout->setColumnStretch(1,0);
	page_output_layout->setColumnStretch(2,0);
	page_output_layout->setColumnStretch(3,0);
	page_output_layout->setColumnStretch(4,1);
	
	page_output_groupbox->setLayout(page_output_layout);

	page_properties_top_layout->addWidget(page_output_groupbox);

	page_properties_top_layout->addSpacing(4);
	page_properties_top_layout->addStretch(10);
}





QDomElement QpPageTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Page_Tab");

    int _units = _units_fld->currentIndex();
    double conv = I_Page->_conversion_factors[_units];
    
    xml.appendChild(createTextXMLNode(doc, "Page_Size", _page_size_fld->currentIndex()));

    double _width = _width_fld->text().toDouble();
    _width /= conv;
    xml.appendChild(createTextXMLNode(doc, "Width", _width));
    double _height = _height_fld->text().toDouble();
    _height /= conv;
    xml.appendChild(createTextXMLNode(doc, "Height", _height));
    xml.appendChild(createTextXMLNode(doc, "Units", _units));
    xml.appendChild(createTextXMLNode(doc, "DPI", _dpi_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Orientation", _orientation_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Show_User", _show_ident_fld->isChecked()));

    return xml;
}

void QpPageTab::fromXMLElement(QDomElement& el)
{
    QString s;
    int _page_size;
    if(getXMLNodeText(el, "Page_Size", s))
    {
        _page_size=s.toInt();
    }
    _page_size_fld->setCurrentIndex(_page_size);
    double _width, _height;
    int _units;
    if(getXMLNodeText(el, "Width", s))
    {
        _width=s.toDouble();
    }
    if(getXMLNodeText(el, "Height", s))
    {
        _height=s.toDouble();
    }
    if(getXMLNodeText(el, "Units", s))
    {
        _units=s.toInt();
    }
    _units_fld->setCurrentIndex(_units);
    oldUnits = _units;
    bool readonly = (_page_size != QPUI_CUSTOM);
    _width_fld->setReadOnly(readonly);
    _height_fld->setReadOnly(readonly);
    _width_fld->setEnabled(!readonly);
    _height_fld->setEnabled(!readonly);

    int _dpi;
    if(getXMLNodeText(el, "DPI", s))
    {
        _dpi=s.toInt();
    }
    _dpi_fld->setCurrentIndex(_dpi);
    switch(_dpi)
    {
        case QPUI_150_DPI:
            I_Page->_conversion_factors[QPUI_PIXELS]=150.;
            I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
            break;
            
        case QPUI_300_DPI:
            I_Page->_conversion_factors[QPUI_PIXELS]=300.;
            I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
            break;
            
        case QPUI_600_DPI:
            I_Page->_conversion_factors[QPUI_PIXELS]=600.;
            I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
            break;
            
        case QPUI_1200_DPI:
            I_Page->_conversion_factors[QPUI_PIXELS]=1200.;
            I_Page->_conversion_factors[QPUI_PSPOINTS]=1200.;
            break;
            
        default:
            I_Page->_conversion_factors[QPUI_PIXELS]=72.;
            I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
            break;
    }
    double conv = I_Page->_conversion_factors[_units];
    _width *= conv;
    _width_fld->setText(QString::number(_width));
    _height *= conv;
    _height_fld->setText(QString::number(_height));

    if(getXMLNodeText(el, "Orientation", s))
    {
        _orientation_fld->setCurrentIndex(s.toInt());
    }

    if(getXMLNodeText(el, "Show_User", s))
    {
        _show_ident_fld->setChecked(s.toInt());
    }
}

void QpPageTab::setPageSize(int page_size)
{
	if( QuSlotDebug() ) cout << " QpPageTab::setPageSize  " << endl;

	bool readonly = (page_size != QPUI_CUSTOM);
	double conv=I_Page->_conversion_factors[_units_fld->currentIndex()];

	if(readonly)
	{
		double width = I_Page->_width_in_inches[page_size];
        DvString strVal = "";
        strVal.append(width * conv, 2);
		_width_fld->setText(strVal.c_str());
		double height = I_Page->_height_in_inches[page_size];
        strVal = "";
        strVal.append(height * conv, 2);
		_height_fld->setText(strVal.c_str());
		
	}
	_width_fld->setReadOnly(readonly);
	_height_fld->setReadOnly(readonly);
	_width_fld->setEnabled(!readonly);
	_height_fld->setEnabled(!readonly);
      
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
 	if(this->isVisible() )
    {
        I_Page->Tree()->setPageOption(PAGE_SIZE, page_size);
        I_Page->updateIsotropicFrames();
    }
}

void QpPageTab::setUnits(int units)
{
	if( QuSlotDebug() ) cout << " QpPageTab::setUnits  " << endl;

	double conv = I_Page->_conversion_factors[units];
	double oldConv = I_Page->_conversion_factors[oldUnits];
	conv /= oldConv;
	oldUnits = units;

	double width = atof(_width_fld->text().toLatin1().data());
    DvString strVal = "";
    strVal.append(width * conv, 2);
	_width_fld->setText(strVal.c_str());
	double height = atof(_height_fld->text().toLatin1().data());
    strVal = "";
    strVal.append(height * conv, 2);
	_height_fld->setText(strVal.c_str());
     
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Page->Tree()->setPageOption(PAGE_UNITS, units);
}

void QpPageTab::setDPI(int dpi_index)
{
	if( QuSlotDebug() ) cout << " QpPageTab::setDPI  " << endl;
	
	double old_dpi=I_Page->_conversion_factors[_units_fld->currentIndex()];

	switch(dpi_index)
	{
		case QPUI_150_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=150.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=150.;
			break;
			
		case QPUI_300_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=300.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=300.;
			break;
			
		case QPUI_600_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=600.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=600.;
			break;
			
		case QPUI_1200_DPI:
			I_Page->_conversion_factors[QPUI_PIXELS]=1200.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=1200.;
			break;
			
		default:
			I_Page->_conversion_factors[QPUI_PIXELS]=72.;
			I_Page->_conversion_factors[QPUI_PSPOINTS]=72.;
			break;
	}

	double new_dpi=I_Page->_conversion_factors[_units_fld->currentIndex()];
	
	double ratio=new_dpi/old_dpi;
	
	if(ratio!=1.)
	{
		double width = atof(_width_fld->text().toLatin1().data());
		double height = atof(_height_fld->text().toLatin1().data());
        DvString strVal ="";
        strVal.append(width * ratio, 2);
		_width_fld->setText(strVal.c_str());
        strVal ="";
        strVal.append( height * ratio, 2);
		_height_fld->setText(strVal.c_str());
	}
	
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Page->Tree()->setPageOption(PAGE_DPI, dpi_index);
	
    for(int i=0; i<_xwin_num_fld->size(); i++){
        _xwin_num_fld->windowAt(i)->setPrintRes((int)I_Page->_conversion_factors[QPUI_PSPOINTS]);
    }
	
}

void QpPageTab::setWidth(const QString &value)
{   
		if( QuSlotDebug() ) cout << " QpPageTab::setWidth  " << endl;

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
 	if(_width_fld->hasFocus() )
    {
        I_Page->Tree()->setPageOption(PAGE_WIDTH, 0, 0.0, value);
        I_Page->updateIsotropicFrames();
    }

}

void QpPageTab::setHeight(const QString &value)
{   
	if( QuSlotDebug() ) cout << " QpPageTab::setHeight  " << endl;

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

 	if(_height_fld->hasFocus() )
    {
        I_Page->Tree()->setPageOption(PAGE_HEIGHT, 0, 0.0, value);
        I_Page->updateIsotropicFrames();
    }

}

void QpPageTab::setOrientation(int iChoice)
{
	if( QuSlotDebug() ) cout << " QpPageTab::setOrientation  " << endl;

	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
	if(this->isVisible() )
    {
        I_Page->Tree()->setPageOption(PAGE_ORIENTATION, iChoice);
        I_Page->updateIsotropicFrames();
    }
}



void QpPageTab::setIdent(bool bChoice)
{   
	if( QuSlotDebug() ) cout << " QpPageTab::setIdent  " << endl;
	// only tell list view to make changes to selected panel items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
  
 	if( this->isVisible() ) I_Page->Tree()->setPageOption(IDENT, 0, 0.0, "", bChoice);

}


