// qpui_spectrogram.cc
//
// Plot Interface Implementation - SpectroGram Object
//
// Janet Barnes
// July 2002
//
// Tony Allen
// Feb 2008, Jan 2014

#include "qpui.h"
#include "DvObj.h"
#include "qdutil.h"
#include "qwui.h"

extern int qsas_restore_release;

//     SpectrumItem CLASS
//     ------------------


SpectrumItem::SpectrumItem(const QString& name, PanelItem* parent, const QString& WLcontext, int position):
		PlotItem(name, parent, WLcontext, position)
{
	I_Type = QP_SPECTROGRAM;

	if(name.isEmpty()){
		setNextName();
	}

	ColourscaleTab = new QpColourscaleTab(tabWidget, "ColourscaleTab", this);
	ColourscaleTab->setVisible(false);
	
	DataTab = new QpDataTab(tabWidget, "DataTab", this, QP_SPECTROGRAM);
	DataTab->setVisible(false);

	if(frameType() == QP_TS){
		DataTab->_z_data_fld->set3DSubsetDef("r");
	}
	else{
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
	}
		
	_z_obj_id=0;
	_y_start_obj_id=0;
	_y_end_obj_id=0;
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	
	y_data_start_str = "";
	y_data_end_str = "";
	x_data_start_str = "";
	x_data_end_str = "";

	// starts as not set (used by fill_X_Data and fill_Y_Data to find when toggle changes)
	XYtoggle_in_fill_x = false;
	XYtoggle_in_fill_y = false;
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	zdo_id_getZRange = 0;
	
	x_min_data = 0;
	x_max_data = 0;
	y_min_data = 0;
	y_max_data = 0;
	z_min_data = 0;
	z_max_data = 0;
	
	if(parent && parent->isInterlaced()) DataTab->interlace_toggle->setChecked(true);
	
	setIcon(QP_SPECTROGRAM);

}

SpectrumItem::SpectrumItem(SpectrumItem* item, PanelItem* parent, int position):
		PlotItem(item, parent, position)
{
	I_Type = QP_SPECTROGRAM;

	ColourscaleTab = new QpColourscaleTab(item->ColourscaleTab, this);
	ColourscaleTab->setVisible(false);

	DataTab = new QpDataTab(item->DataTab, this);
	DataTab->setVisible(false);

	if(frameType() == QP_TS){
		DataTab->_z_data_fld->set3DSubsetDef("r");
	}
	else{
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
	}
	
	_z_obj_id=0;
	_y_start_obj_id=0;
	_y_end_obj_id=0;
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	
	y_data_start_str = "";
	y_data_end_str = "";
	x_data_start_str = "";
	x_data_end_str = "";

	// starts as not set (used by fill_X_Data and fill_Y_Data to find when toggle changes)
	XYtoggle_in_fill_x = false;
	XYtoggle_in_fill_y = false;
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	zdo_id_getZRange = 0;
	
	x_min_data = 0;
	x_max_data = 0;
	y_min_data = 0;
	y_max_data = 0;
	z_min_data = 0;
	z_max_data = 0;

qApp->processEvents();
	
	if(parent && parent->isInterlaced()) DataTab->interlace_toggle->setChecked(true);

	setIcon(QP_SPECTROGRAM);

}

SpectrumItem::~SpectrumItem()
{
	if(DataTab) delete DataTab;
	if(ColourscaleTab) delete ColourscaleTab;
	foreach(PlotItem* i, linkedPlots)
	{
		i->linkedPlots.removeAll(this);
	}
}

// -------------------------------------------

void SpectrumItem::setDataZObj(QString txt)
{
	if(txt.isEmpty()) return;

	DataTab->_z_data_fld->setText(txt);
	DataTab->_z_data_fld->reValidate(true);
}

// -------------------------------------------

void SpectrumItem::setDataYObj(QString txt)
{
	if(txt.isEmpty()) return;

	DataTab->_y_start_data_fld->setText(txt);
	DataTab->_y_start_data_fld->reValidate(true);
}

// -------------------------------------------

void SpectrumItem::setDataXObj(QString txt)
{
	if(txt.isEmpty()) return;

	DataTab->_x_start_data_fld->setText(txt);
	DataTab->_x_start_data_fld->reValidate(true);
}

// ------------------------------------

void SpectrumItem::clearObj_id()
{
   // called when we need to force a range update
	_z_obj_id = 0;
	_y_start_obj_id = 0;
	_x_start_obj_id = 0;
}

// -------------------------------------------

void SpectrumItem::replaceData(QString obj_name)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceData" << endl;
	if(obj_name.isEmpty()) return;
	
	DvString oldCompText = compText( DataTab->_z_data_fld->text().toLatin1().data() );

	if(oldCompText.empty())
	{
		if(this->frameType() == QP_TS)
		{
			DataTab->_z_data_fld->getDefaultComp(QS_1D, QS_SEQ); // set data slot
		}
		else
		{
			DataTab->_z_data_fld->getDefaultComp(QS_2D, QS_NOT_SEQ); // set data slot
		}
	}
	else
	{
		DataTab->_z_data_fld->setTextKeepComp(obj_name); // set data slot

	}
	this->setName(DataTab->_z_data_fld->text()); // set item name
	this->WLcontext = obj_name;

}

// -------------------------------------------

void SpectrumItem::replaceComponent(QString comp)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceComponent" << endl;
	if(comp.isEmpty()) return;
	
	DvString  newName = baseObjectText(DataTab->_z_data_fld->text().toLatin1().data());
	newName += getNewComp(DataTab->_z_data_fld->text().toLatin1().data(), comp.toLatin1().data());
	
	this->setName(newName.c_str());                      // set item name
	DataTab->_z_data_fld->setText(newName.c_str()); // set data slot
	DataTab->_z_data_fld->reValidate(true);

}

// -------------------------------------------
void SpectrumItem::replaceTIdata(QString tivl)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceTI with '" << tivl.toLatin1().data() << "'" << endl;
	if(tivl.isEmpty()) return;
	
	DataTab->_mask_data_fld->setText(tivl);  // set TI slot
	DataTab->_mask_data_fld->reValidate();


}

// -------------------------------------------
void SpectrumItem::replaceTIcomp(QString comp)
{
	if(QuSlotDebug()) cout << "SpectrumItem::replaceTI with '" << comp.toLatin1().data() << "'" << endl;
	if(comp.isEmpty()) return;
	
	DvString newName = baseObjectText(DataTab->_mask_data_fld->text().toLatin1().data());
	newName += comp.toLatin1().data();
	DataTab->_mask_data_fld->setText(newName.c_str());  // set TI slot

	DataTab->_mask_data_fld->reValidate();



}

// -------------------------------------------

void SpectrumItem::refreshTabs()
{
	if(QuSlotDebug()) cout << "SpectrumItem::refreshTabs" << endl;
	DataTab->_z_data_fld->reValidate();

}

// -------------------------------------------

void SpectrumItem::showTabs()
{
	if(QuSlotDebug()) cout << "SpectrumItem::showTabs" << endl;

	// ensure all tabs are hidden
	int Ntabs = tabWidget->count();
	for(int i = Ntabs-1; i >= 0; i--)
	{
		if(tabWidget->widget(i))
		{
			tabWidget->widget(i)->setVisible(false);
			tabWidget->removeTab(i);
		}
	}

	tabWidget->addTab(DataTab, "Data");

	tabWidget->addTab(ColourscaleTab, "Colourscale");
	// value return is always shown
	tabWidget->addTab(page()->ValueReturnTab, "Value Return");
	
	if(this->Tree()->I_LastTab == QP_COLOUR_TAB)  tabWidget->setCurrentIndex(tabWidget->indexOf(ColourscaleTab));
	else	tabWidget->setCurrentIndex(tabWidget->indexOf(DataTab));
}

// ------------------------------------------

void SpectrumItem::changeFrameType(int frameType)
{
	// called when Frame changes type XY <-> TS

    if(frameType == QP_TS)
	{
		DataTab->_x_start_label_fld->setVisible(false);
		DataTab->_x_start_data_fld->setVisible(false);
		DataTab->_XYtoggle->setVisible(false);
		DataTab->gap_toggle->setVisible(true);
		DataTab->interlace_toggle->setVisible(true);
		DataTab->timeAsSec_toggle->setVisible(false);
		DataTab->_z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_SEQ, QS_TT));
		DataTab->_z_data_fld->set3DSubsetDef("r");
		QString baseTxt = DataTab->_z_data_fld->textObject();
		DataTab->_z_data_fld->setText(baseTxt);
		DataTab->_z_data_fld->reValidate(true);
	}
	else
	{
		DataTab->_x_start_label_fld->setVisible(true);
		DataTab->_x_start_data_fld->setVisible(true);
		DataTab->_XYtoggle->setVisible(true);
		DataTab->gap_toggle->setVisible(false);
		DataTab->interlace_toggle->setVisible(false);
		DataTab->timeAsSec_toggle->setVisible(true);
		DataTab->_z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_SEQ, QS_ANY));
		DataTab->_z_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_NOT_SEQ, QS_ANY));
		DataTab->_z_data_fld->set3DSubsetDef("rp", true);
		QString baseTxt = DataTab->_z_data_fld->textObject();
		DataTab->_z_data_fld->setText(baseTxt);
		DataTab->_z_data_fld->reValidate(true);
	}
	
	DataTab->setDataTypeTitle();
	
	// force axis update
	xdo_id_getXRange = 0;
	ydo_id_getYRange = 0;
	_z_obj_id = 0;
	
	DataTab->setZData();
}



// ---------------------------------------

void SpectrumItem::get_dobjs()
{
    if( QuSlotDebug() ) cout << "QpSpectrumItem::get_dobjs " <<  endl;
	_zdo = getZDO(); // sets _zdo
	if(_zdo.is_nil()) return;

	// Apply Time Interval Mask if set
	DvObject_var TImaskObj = DataTab->_mask_data_fld->getObject();
	if( TImaskObj.is_ok()) {
		DvEvent range = TImaskObj->getTimeRange();
		_zdo = _zdo->subSequence(range);
	}
 
	_nx_data = fill_X_Data(); // sets _xdo
	_ny_data = fill_Y_Data(); // sets _ydo
	

}

// ---------------------------------------

void SpectrumItem::getLabel(const QpAxes axis, DvString& label)
{
	if( QuSlotDebug() ) cout << "QpSpectrumItem::getLabel for axis " << (int)axis <<  endl;

	// fetch label for axis
  
	// test on is_nil() done after default label is set below
  
	DvObject_var obj;
	DvString default_label;
	switch(axis)
	{
		case X_AXIS:
			// if time series plot default label is Universal Time
			if(frameType() == QP_TS)
			{
				label="Universal Time";
				return;
			}
			if(DataTab->timeAsSec_toggle->isChecked())
			{
				label="Seconds";
				return;
			}
			else label="X Axis";

			obj=_xdo;
			break;

		case Y_AXIS:
			label="Y Axis";
			obj=_ydo;
			break;

		case Z_AXIS:
			label="Z Axis";

			obj=_zdo;
			break;
			
		case COL_AXIS:
			label="Colour Bar";

			obj=_zdo;
			break;

		default:
			return;
	}
	if(obj.is_nil()) return;

	if(axis == COL_AXIS)
	{
		label = obj->getXrefText(LABLAXIS);
		
		DvString units=obj->getXrefText(UNITS);
		units.trimmed();
		if(units.size() )
		{
			label += " ";
			label += units.replace(" summed", "");
		}
	}
	else
	{

		default_label = obj->getXrefText(LABLAXIS);
		if(default_label.empty()) default_label = obj->getXrefText(FIELDNAM);
		if(default_label.empty()) default_label = obj->getXrefText(OBJECT_NAME);
		if(!default_label.empty()) label=default_label;
	
		DvString units = obj->getXrefText(UNITS);
		units.trimmed();
		if(units.size() )
		{
			label += " (";
			label += units;
			label += ")";
		}
		DvString scid = obj->getXrefText(SC_ID);
		scid.trimmed();
		if( !scid.empty() )
		{
			label += " SC=";
			label += scid;
		}
	}
	
	label = label.replace("theta","#gh");
	label = label.replace("phi","#gf");
	
   
	return;
}

// -------------------------------------

int SpectrumItem::getXData(DvObject_var &xdo, DvObject_var &xdo_end){
	int nx_data;
  
  // if we don't understand xdo_end object call alternative method
	if(xdo_end.is_nil() && xdo_end->not_dbl() && xdo_end->not_time()) return getXData( xdo);
  
	 
	if (xdo->is_time())
	{
		nx_data = xdo->seqSize();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
	  
      // create full array
		for (int i=0; i< nx_data; i++)
		{
			_x_start_all[i] = xdo->time(i).getEpoch2000();
		}
	}
	else if ( xdo->is_dbl() )
	{
		if(xdo->arraySize() == 1)
		{
			nx_data = xdo->seqSize();
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
		  
			for (int i=0; i< nx_data; i++)
			{
				// dbl(i) runs along sequence 
				_x_start_all[i] = xdo->dbl(i);
			}
		}
		else if(xdo->nDims() == 1)
		{
			nx_data = xdo->Dims()[0];
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
		  
			for (int i=0; i< nx_data; i++)
			{
				// dbl(i) runs along array dim 
				_x_start_all[i] = xdo->dbl(i);
			}
		}
		else
		{
			return 0;
		}
  
	}
	else
	{
		return 0;
	}
  
	if (xdo_end->is_time() && xdo_end->seqSize() == nx_data)
	{
		for (int i=0; i< nx_data; i++)
		{
			_x_end_all[i] = xdo_end->time(i).getEpoch2000();
		}
	  
	}
	else if ( xdo_end->is_dbl() && xdo_end->seqSize() == nx_data)
	{
		for (int i=0; i< nx_data; i++)
		{
			_x_end_all[i] = xdo_end->dbl(i);
		}

	}
	else
	{
		return getXData(xdo);
	}

	return nx_data;
}


// ----------------------------------------------

int SpectrumItem::getXData(DvObject_var &xdo, DvObject_var &xdo_plus, DvObject_var &xdo_minus){
	int nx_data;
 
	_x_start_all.clear();
	_x_end_all.clear();
	
	if(frameType() == QP_TS && DataTab->interlace_toggle->isChecked() )
	{
		// Make spectra plot interlaced
		int nSpectra = panel()->numberSpectra();
		int nThis = panel()->numberThisSpectrum(this);
		if( xdo->is_time() ) return getXData(xdo, nSpectra, nThis);
	}

    // Allow for data set with single value for delta_plus and delta_minus
	if(xdo_plus.is_nil() ) {
      // if no deltas call basic xdo only method
		if( xdo_minus.is_nil()) {
			cout << " X Object delta plus and minus are empty" << endl;
			return getXData(xdo);
		}
		xdo_plus = new DvObject(0.0);
	}
	else if(xdo_minus.is_nil() ) xdo_minus = new DvObject(0.0);
  	
  // if we don't understand xdo_deltas objects call alternative method
	if( xdo_plus->not_time() && xdo_plus->not_dbl() && xdo_plus->not_int()  ) {
		cout << " X Object delta plus not allowed data type" << endl;
		return getXData(xdo);
	}
	if(xdo_minus->not_time() && xdo_minus->not_dbl() && xdo_minus->not_int()) {
		cout << " X Object delta minus not allowed data type" << endl;
		return getXData(xdo);
	}

	if( xdo->is_time() )
	{
		nx_data = xdo->seqSize();

		DvObject_var d_minus = new DvObject(0.0, nx_data);
		if( xdo_minus->is_time() && xdo_minus->seqSize() == nx_data)
		{
			for(int i=0; i<nx_data; i++)
			{
				d_minus->dbl(i) = xdo_minus->time(i).getEpoch2000();
			}
		}
		else if( xdo_minus->is_dbl() && xdo_minus->seqSize() == nx_data)
		{

			d_minus = xdo_minus->changeUnitsTo( DvString("1>s") );
			
		}
		else if( xdo_minus->is_dbl() && xdo_minus->seqSize() == 1 )
		{
			DvObject_var  dm = xdo_minus->changeUnitsTo( DvString("1>s") );
		 
			for(int i=0; i<nx_data; i++)
			{
				d_minus->dbl(i) = dm->dbl(0);
			}
		}
		else
		{
			cout << " X Object delta minus sequence length != time sequence length" << endl;
			return getXData(xdo);
		}
	 
		DvObject_var d_plus = new DvObject(0.0, nx_data);
		if( xdo_plus->is_time() && xdo_plus->seqSize() == nx_data)
		{
			for(int i=0; i<nx_data; i++)
			{
				d_plus->dbl(i) = xdo_plus->time(i).getEpoch2000();
			}
		}
		else if( xdo_plus->is_dbl() && xdo_plus->seqSize() == nx_data)
		{

			d_plus = xdo_plus->changeUnitsTo( DvString("1>s") );
			
		}
		else if( xdo_plus->is_dbl() && xdo_plus->seqSize() == 1 )
		{
			DvObject_var  dm = xdo_plus->changeUnitsTo( DvString("1>s") );
		 
			for(int i=0; i<nx_data; i++)
			{
				d_plus->dbl(i) = dm->dbl(0);
			}
		}
		else
		{
			cout << " X Object delta plus sequence length != time sequence length" << endl;
			return getXData(xdo);
		}

		// create  arrays
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		for (int i=0; i< nx_data; i++)
		{
			double value_ip = xdo->time(i).getEpoch2000();
			double value_im = value_ip;
			value_ip += d_plus->dbl(i);
			value_im -= d_minus->dbl(i);

			_x_end_all[i] = value_ip;
			_x_start_all[i] = value_im;
		}

	}
	else if( xdo->is_dbl() && xdo->arraySize() == 1)
	{
		nx_data = xdo->seqSize();
	  	  
		DvObject_var ss_start = xdo - xdo_minus;
		
		DvObject_var ss_end = xdo + xdo_plus;
			
	 
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		for (int i=0; i< nx_data; i++)
		{
			_x_start_all[i] = ss_start->dbl(i);
			_x_end_all[i] = ss_end->dbl(i);
		}

	}
	else if ( xdo->is_dbl() && xdo->arraySize() > 1 )
	{
		if(xdo->nDims() == 1)
		{
			nx_data = xdo->Dims()[0];
		 
			DvObject_var m_start = xdo - xdo_minus;
			DvObject_var m_end = xdo + xdo_plus;

		
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
			for (int i=0; i< nx_data; i++)
			{
				_x_start_all[i] = m_start->dbl(i);
				_x_end_all[i] = m_end->dbl(i);
			}

		}
		else
		{
			cout << "X Object matrix must be 1-D" << endl;
			return 0;
		}
  
	}
	else
	{
		cout << " X Object not time or scalar sequence nor 1D array" << endl;
		return 0;
	}
  
  // in the case of monotonic decreasing X the deltas may be applied the wrong way
  // ensure if decreasing that start > end
  
	if (_x_start_all[0] > _x_start_all[nx_data-1])
	{
    // decreasing bins, ensure start > end for each bin
		for(int i=0; i<nx_data; i++)
		{
			if(_x_start_all[i] < _x_end_all[i] )
			{
				double hold = _x_end_all[i];
				_x_end_all[i] = _x_start_all[i];
				_x_start_all[i] = hold;
			}
		}
	}
	
	// remove white gaps 
	if(frameType() == QP_TS && DataTab->gap_toggle->isChecked() )
	{
		for(int i=1; i<nx_data; i++)
		{
			double diff = (_x_start_all[i] - _x_end_all[i-1]) * 0.5;
			_x_end_all[i-1] += diff;
			_x_start_all[i] -= diff;
		}
	}

	return nx_data;
}

// ----------------------------------------------

int SpectrumItem::getXData(DvObject_var &tdo, int nSpectra, int nItem){

	_x_start_all.clear();
	_x_end_all.clear();

	int nx_data;
	double half_spacing=0.1; // to ensure something plots

	if ( tdo.is_ok() && tdo->is_time() )
	{
		nx_data = tdo->seqSize();
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);

	  // create  array
		for (int i=0; i< nx_data; i++)
		{
			double value_i = tdo->time(i).getEpoch2000();
			_x_start_all[i] = value_i;
		}
	  // find bin widths
		double diff[nx_data + 1];
		 
		for(int i=1; i<nx_data; i++)
		{
			diff[i] = _x_start_all[i] - _x_start_all[i-1];
		}
	  // handle ends
		diff[0] = diff[1];
		diff[nx_data] = diff[nx_data-1];
		 
	  // set start end of bin using widths
		for(int i=0; i < nx_data; i++)
		{
			_x_end_all[i]   = _x_start_all[i]  + diff[i] * (((double) (nItem + 1) / nSpectra) - 0.5) ;
			_x_start_all[i] = _x_start_all[i]  + diff[i] * (((double) nItem / nSpectra) - 0.5)  ;	  
		}

	}
  
	return nx_data;
}

// ----------------------------------------------

int SpectrumItem::getXData(DvObject_var &xdo){
	_x_start_all.clear();
	_x_end_all.clear();

	if(xdo.is_nil()) return 0;

	int nx_data = xdo->seqSize();
	double half_spacing=0.1; // to ensure something plots

	if( xdo->is_time() )
	{
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);

		double med=getSpacingMedian(xdo);
	    if(med > 0.) {
			// create array
			for (int i=0; i< nx_data; i++){
				double value_i = xdo->time(i).getEpoch2000();
				_x_start_all[i] = value_i;
				_x_end_all[i] = value_i + med;
			}
		}
		else{
			// create array
			for (int i=0; i< nx_data; i++)
			{
				double value_i = xdo->time(i).getEpoch2000();
				_x_start_all[i] = value_i;
			}
			// find bin widths
			double diff[nx_data + 1];
		 
			for(int i=1; i<nx_data; i++)
			{
				diff[i] = _x_start_all[i] - _x_start_all[i-1];
			}
			// handle ends
			diff[0] = diff[1];
			diff[nx_data] = diff[nx_data-1];
		 
			// set start end of bin using widths
			for(int i=0; i < nx_data; i++)
			{
				_x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
				_x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
			}		
		}

	}
	else if( xdo->is_dbl() && xdo->arraySize() == 1)
	{
		_x_start_all.resize(nx_data);
		_x_end_all.resize(nx_data);
		half_spacing = xdo->get_spacing();
	  
		for (int i=0; i< nx_data; i++)
		{
			double value_i =  xdo->dbl(i);
			_x_start_all[i] = value_i;
			_x_end_all[i] = value_i + 2* half_spacing;
		}
	}
	else if( xdo->is_dbl() && xdo->arraySize() > 1 )
	{
		if(xdo->nDims() == 1)
		{
			nx_data = xdo->Dims()[0];
			_x_start_all.resize(nx_data);
			_x_end_all.resize(nx_data);
			for (int i=0; i< nx_data; i++)
			{
				_x_start_all[i] = xdo->dbl(i);
			}
		 // find bin widths
			double diff[nx_data + 1];
		 
			for(int i=1; i<nx_data; i++)
			{
				diff[i] = _x_start_all[i] - _x_start_all[i-1];
			}
		 // handle ends
			diff[0] = diff[1];
			diff[nx_data] = diff[nx_data-1];
		 
		 // set start end of bin using widths
			for(int i=0; i < nx_data; i++)
			{
				_x_end_all[i] = _x_start_all[i] + 0.5 * diff[i+1];
				_x_start_all[i] = _x_start_all[i] - 0.5 * diff[i];
			}
		}
		else
		{
			cout << "X Object matrix must be 1-D" << endl;
			return 0;
		}
  
	}
	else
	{
		cout << " X Object not time or scalar sequence nor 1D array" << endl;
		return 0;
	}
  
	return nx_data;
}

// -------------------------------------------------

int SpectrumItem::getYData(DvObject_var &ydo, DvObject_var &ydo_end)
{
    if ( QuSlotDebug() )
	{
		printf( "SpectrumItem::getYData, with ydo end\n" );
		fflush(stdout);
	}
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	

	int ny_data=0;
  

	if( ydo->seqSize() == _nx_data)
	{
		for(int nx=0; nx < _nx_data; ++nx){

			if(ydo->nDims() != 1)
			{
				cout << "Y Object is sequence but not 1-D" << endl;
				return -1;
			}
			
			ny_data = ydo->Dims()[0];
			_y_start_all[nx].resize(ny_data);
            for(size_t i=0; i<ny_data; i++) { _y_start_all[nx][i] = ydo->asDouble(nx*ny_data+i);
            }
			
		}
	}
	else if(ydo->seqSize() == 1)
	{
		ny_data = ydo->Dims()[0];
		for(int nx=0; nx < _nx_data; ++nx){
						
			if(ydo->nDims() != 1)
			{
				cout << "Y Object matrix must be 1-D" << endl;
				return -1;
			}
			
			_y_start_all[nx].resize(ny_data);
            for(size_t i=0; i<ny_data; i++) _y_start_all[nx][i] =  ydo->asDouble(i);
		}
	}
	else
	{
		plmtex("lv",-2.0,0.75,0.0, "Y Object not a matrix or matrix sequence");
		return -1;
	}
  
  // Handle y_end
  
	if( ydo_end.is_nil()) return getYData( ydo);
  
	if( ydo_end->seqSize() == _nx_data)
	{
		for(int nx=0; nx< _nx_data; ++nx){

			ny_data = ydo_end->Dims()[0];
			if(_y_start_all[nx].size() != ny_data)
			{
				cout <<  "Y End Sequence [nx] dimension error" << endl;
				return -1;
			}
			_y_end_all[nx].resize(ny_data);
			for(size_t i=0; i<ny_data; i++) _y_end_all[nx][i] = ydo_end->asDouble(nx*ny_data+i);
		}
	}
	else if( ydo_end->seqSize() == 1)
	{
		ny_data = ydo_end->Dims()[0];
		for(int nx=0; nx< _nx_data; ++nx){

			if(_y_start_all[nx].size() != ny_data)
			{
				cout << "Y End Object dimension error" << endl;
				return -1;
			}
			_y_end_all[nx].resize(ny_data);
			for(size_t i=0; i<ny_data; i++) _y_end_all[nx][i] = ydo_end->asDouble(i);

		}
	}
	return ny_data;
  
}


// -------------------------------------------------

int SpectrumItem::getYData(DvObject_var &ydo, DvObject_var &ydo_plus, DvObject_var &ydo_minus)
{
	if ( QuSlotDebug() )
	{
		printf( " SpectrumItem::getYData, deltas\n" );
		fflush(stdout);
	}
	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);
	int ny_data=0;
	double value_i;



	if(ydo->nDims() == 1)
	{
		// Allow for data set with single value for delta_plus and delta_minus
		if(ydo_plus.is_nil() ) {
			// if no deltas call basic xdo only method
			if( ydo_minus.is_nil()) {
				cout << " Y Object delta plus and minus are empty" << endl;
				return getYData( ydo);
			}
			ydo_plus = new DvObject(0.0, ydo->Dims());
		}
		else if(ydo_minus.is_nil() ) ydo_minus = new DvObject(0.0, ydo->Dims());

		DvObject_var ms_end = ydo + ydo_plus;
		DvObject_var ms_start = ydo - ydo_minus;
	 	  
		if(ms_start->seqSize() != _nx_data && ms_start->seqSize() != 1)
		{
			cout << "delta minus sequence of wrong length" << endl;
			return -1;
		}
		if(ms_end->seqSize() != _nx_data && ms_end->seqSize() != 1)
		{
			cout << "delta plus sequence of wrong length" << endl;
			return -1;
		}
	  
		for(int nx=0; nx< _nx_data; ++nx){
			int nx_s = nx;
			if(ms_start->seqSize() == 1) nx_s = 0;
			int nx_e = nx;
			if(ms_end->seqSize() == 1) nx_e = 0;
			ny_data = ydo->Dims()[0];
			_y_start_all[nx].resize(ny_data);
			_y_end_all[nx].resize(ny_data);


			for(size_t i=0; i<ny_data; i++){
				_y_start_all[nx][i] = ms_start->asDouble(nx_s*ny_data + i);
				_y_end_all[nx][i] = ms_end->asDouble(nx_e*ny_data + i);
			}
		}
	}
	else{
		return -1;
	}

	// in the case of monotonic decreasing Y the deltas may be applied the wrong way
	// ensure if decreasing that start > end
  
	for(int j=0; j< _nx_data; j++)
	{
		ny_data = _y_start_all[j].size();
		if ( (_y_start_all[j])[0] > (_y_start_all[j])[ny_data-1])
		{
			// decreasing bins, ensure start > end for each bin
			for(int i=0; i<ny_data; i++)
			{
				if((_y_start_all[j])[i] < (_y_end_all[j])[i] )
				{
					double hold = (_y_end_all[j])[i];
					(_y_end_all[j])[i] = (_y_start_all[j])[i];
					(_y_start_all[j])[i] = hold;
				}
			}
		}
	}
	return ny_data;
}

// -------------------------------------------------

int SpectrumItem::getYData(DvObject_var &ydo)
{
	if ( QuSlotDebug() )
	{
		printf( " SpectrumItem::getYData, basic\n" );
		fflush(stdout);
	}

	_y_start_all.clear();
	_y_end_all.clear();
	_y_start_all.resize(_nx_data);
	_y_end_all.resize(_nx_data);

	
	int ny_data=0;

	if(ydo->seqSize() == _nx_data)
	{
		for(int nx=0; nx< _nx_data; ++nx){
			if(ydo->nDims() == 1)
			{
				ny_data = ydo->Dims()[0];
				_y_start_all[nx].resize(ny_data);
				_y_end_all[nx].resize(ny_data);
				if(ny_data == 1){
					double value = ydo->asDouble(0);
					_y_start_all[nx][0] = value;
					_y_end_all[nx][0] = value+1.;
				}
				else
				{
					for(size_t i=0; i<ny_data-1; i++){ 
						_y_start_all[nx][i] = ydo->asDouble(nx*ny_data + i);
						_y_end_all[nx][i] = ydo->asDouble(nx*ny_data + i+1);
					}
					_y_start_all[nx][ny_data-1] = ydo->asDouble(nx*ny_data + ny_data-1);
					double lastValminus = ydo->asDouble(nx*ny_data + ny_data-2);
					_y_end_all[nx][ny_data-1] = ( 2.0*_y_start_all[nx][ny_data-1] - lastValminus);
				}
			}
			else{
				cout << "Y Object is sequence but not 1-D" << endl;
				return -1;
			}
		}
	}
	else 
	{
		if(ydo->nDims() != 1)
		{
			cout << "Y Object matrix must be 1-D" << endl;
			return -1;
		}

		for(int nx=0; nx< _nx_data; nx++){
	  
			ny_data = ydo->Dims()[0];
			_y_start_all[nx].resize(ny_data);
			_y_end_all[nx].resize(ny_data);
			if(ny_data == 1){
				double value = ydo->asDouble(0);
				_y_start_all[nx][0] = value;
				_y_end_all[nx][0] = value+1.;
			}
			else
			{

				for(size_t i=0; i<ny_data-1; i++){ 
					_y_start_all[nx][i] = ydo->asDouble(i);
					_y_end_all[nx][i] = ydo->asDouble(i+1);
				}
				_y_start_all[nx][ny_data-1] = ydo->asDouble(ny_data-1);
				double lastValminus = ydo->asDouble(ny_data-2);
				_y_end_all[nx][ny_data-1] = ( 2.0*_y_start_all[nx][ny_data-1] - lastValminus);
			}
		}
	}
	
	return ny_data;
  
}

// -------------------------------------------------

bool SpectrumItem::isZConformal(int nx_len, int ny_len)
{
   // does something only for 2D matrix or 2D matrix sequence
	if ( QuSlotDebug() )
	{
		printf( "SpectrumItem::isZConformal nx=%d, ny=%d\n", nx_len , ny_len );
		fflush(stdout);
	}


	if( _zdo->is_nil() || _zdo->not_dbl() ) return true; // do nothing


	if(  _zdo->nDims() != 2) return true; // do nothing, not 2D array
	
	int n0 = _zdo->Dims()[0];
	int	n1 = _zdo->Dims()[1];

	if(DataTab->_XYtoggle->isChecked() && n0 == ny_len && n1 == nx_len)
	{
		return true;
	}
	else if(n0 == nx_len && n1 == ny_len)
	{
		return true;
	}
	else return false;
}

//----------------------------

void SpectrumItem::getSpectrogramName(string& name)
{
  // fetch name of spectrogram object (for display in listview)

	name = DataTab->_z_data_fld->text().toLatin1().data();
	if(name == "")
	{
		if(frameType() == QP_XY)
			name = "XY SpectroGram";
		else
			name = "TS SpectroGram";
	}
}

//----------------------------

string SpectrumItem::getTitle()
{
	// fetch title for current spectrogram object

	if(_zdo.is_nil())
	{
		return string("");
	}
	DvString title = _zdo->getXrefText(FIELDNAM);
	if(title.size() == 0) title += DataTab->_z_data_fld->text().toLatin1().data();
  
	return (string) title;
}
// --------------------------------

int SpectrumItem::getXmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( "QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	
	int isLog = 0;
	
	// find SCALETYP
		
	if (!_xdo.is_nil() && _xdo->xref_exists(SCALETYP) )
	{
		DvString scaleType = _xdo->getXrefText(SCALETYP);
		if(!scaleType.empty() )
		{
			if(scaleType == "LOG") isLog = 1;
		}
	}
	return isLog;
}


// -----------------------------------------------------

bool SpectrumItem::getXmetaRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getXmetaRange" << endl;

    bool rangeFound = false;
	
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;

	if ( _nx_data == 0 )   return false;
	
	if(frameType() == QP_TS)
	{
		DvObject_var tdo = _zdo->getTimeTags();
		if(tdo.is_nil() ) return false;
	
	// Change to use bins as Y range tests against bin edges which include delta_plus/minus
	//	xmin = tdo[0].getEpoch2000();
	//	xmax = tdo[tdo->sequence_size()-1].getEpoch2000();
		xmin = _x_start_all[0];
		xmax = _x_end_all[_nx_data-1];
		rangeFound = true;
	}
	else
	{
		if( DataTab->timeAsSec_toggle->isChecked() )
		{
			DvObject_var tdo = _zdo->getTimeTags();
			if(tdo.is_nil()) return false;
		
		//	xmin = tdo[0].getEpoch2000();
		//	xmax = tdo[tdo->sequence_size()-1].getEpoch2000();
			xmin = _x_start_all[0];
			xmax = _x_end_all[_nx_data-1];
			rangeFound = true;
		}
		else
		{
			if (_xdo->xref_exists(SCALEMIN) && _xdo->xref_exists(SCALEMAX))
			{
				DvObject_var smin = _xdo->get_xref(SCALEMIN);
				DvObject_var smax = _xdo->get_xref(SCALEMAX);
				if(smin.is_nil() || smax.is_nil()) return false;
			
				xmin = smin->asDouble();
				xmax = smax->asDouble();
				rangeFound = true;
			}
		}
	}

	if(rangeFound) return rangeFound;
	else return getXRange( xmin,  xmax);

}
// -----------------------------------------------------

bool SpectrumItem::getXRange(double& xmin, double& xmax)
{
	// gets x min and max values for this spectrogram object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getXRange" << endl;
    
	// reject if no Z object yet
	if(_zdo.is_nil()) return false;
  	
	xmin = DBL_MAX;
	xmax = -DBL_MAX;
	
	if ( _xdo.is_nil() )   return false;
	
	if(_xdo->get_id() == xdo_id_getXRange)
	{
		xmin = x_min_data;
		xmax = x_max_data;
		return true;
	}
	xdo_id_getXRange = _xdo->get_id();
  
	for(int i=0; i<_nx_data; i++)
	{
		double mini = _x_start_all[i];
		double maxi = _x_end_all[i];
		// may not be increasing bins
		if(mini < maxi)
		{
			if(mini < xmin) xmin = mini;
			if(maxi > xmax) xmax = maxi;
		}
		else
		{
			if(maxi < xmin) xmin = maxi;
			if(mini > xmax) xmax = mini;
		}
	}
   
   // store for reuse
	x_min_data = xmin;
	x_max_data = xmax;
    
	return true;
}

// --------------------------------

int SpectrumItem::getYmetaIsLog()
{
	if ( QuSlotDebug() )
	{
		printf( " QpSpectrumItem::getXmetaIsLog \n" );
		fflush(stdout);
	}
	 
	int isLog=0;
	// find SCALETYP
		
	if (!_ydo.is_nil() && _ydo->xref_exists(SCALETYP) )
	{
		DvString scaleType = _ydo->getXrefText(SCALETYP);
		if(!scaleType.empty() )
		{
			if(scaleType == "LOG")  isLog = 1;
		}
	}
	return isLog;
}

// ------------------------------------

bool SpectrumItem::getYmetaRange(double& ymin, double& ymax)
{
	// gets y min and max values for whole Y axis
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getYmetaRange" << endl;

    ymin=0;
	ymax=0;
	bool rangeFound = false;
  
	if(_zdo.is_nil()) return false;
	
	// reject if no X object yet (also traps no z object)
	if ( _xdo.is_nil() )   return false;
  
	if ( _ydo.is_nil() ) return false;

	if (_ydo->xref_exists(SCALEMIN) && _ydo->xref_exists(SCALEMAX))
	{
		DvObject_var smin = _ydo->get_xref(SCALEMIN);
		DvObject_var smax = _ydo->get_xref(SCALEMAX);
		if(!smin.is_nil() && !smax.is_nil() ){
			ymin = smin->asDouble();
			ymax = smax->asDouble();
			rangeFound = true;
		}
	}
	
	if(rangeFound) return rangeFound;
	else	return getYRange(ymin, ymax);
	
}

// ---------------------------------------

bool SpectrumItem::getYRange(double& ymin, double& ymax)
{
	// gets y min and max values for all x values in range
	// xmin - xmax for the current spectrogram object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getYRange" << endl;
	
	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		xmax = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		xmin = frame()->xAxisTab->_axis_xy_fld->getStart();
		xmax = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	if(xmin >= xmax) return false;
	
	ymin = DBL_MAX;
	ymax = -DBL_MAX;
	
	bool recalc = false;

	// reject if no Z objects yet
	if(_zdo.is_nil()) return false;
	if ( _xdo.is_nil() )   return false;
	if ( _ydo.is_nil() ) return false;
	
	if(x_range_min != xmin || x_range_max != xmax)
	{
		// keep x range used to calculate y range
		x_range_min = xmin;
		x_range_max = xmax;
		recalc = true;
	}
	
	if( _ydo->get_id() != ydo_id_getYRange) recalc = true;
		
	ydo_id_getYRange = _ydo->get_id();

	if(!recalc)
	{
		ymin = y_min_data;
		ymax = y_max_data;
		return true;
	}

	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
      	
		// skip if out of range
		if(dx1 < dx2)
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		// find ymin and ymax for each x bin
		for(int j=0; j<_y_start_all[nx].size(); j++)
		{
			double ymini = (_y_start_all[nx])[j];
			double ymaxi = (_y_end_all[nx])[j];
			// may be decreasing bin
			if(ymini < ymaxi)
			{
				if(ymax < ymaxi) ymax = ymaxi;
				if(ymin > ymini) ymin = ymini;
			}
			else
			{
				// decreasing bin
				if(ymax < ymini) ymax = ymini;
				if(ymin > ymaxi) ymin = ymaxi;
			}
		}
	}

	// store to reuse if nothing changed
	y_min_data = ymin;
	y_max_data = ymax;

	return true;

}

// -----------------------------------------------------

bool SpectrumItem::getZmetaRange(double& zmin,double& zmax)
{
	// gets z min and max values using metadata
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZmetaRange" << endl;
	
	bool rangeFound = false;
	
	if(_zdo.is_nil()) return false;

	if (_zdo->xref_exists(SCALEMIN) && _zdo->xref_exists(SCALEMIN))
	{
		DvObject_var smin = _zdo->get_xref(SCALEMIN);
		if(smin.is_nil()) return false;
		zmin = smin->asDouble();
		
		DvObject_var smax = _zdo->get_xref(SCALEMAX);
		if(smax.is_nil()) return false;
		zmax = smax->asDouble();
		rangeFound = true;
	}
				
	// set lin/log from metadata
	
	if (_zdo->xref_exists(SCALETYP) && !ColourscaleTab->userSetColLinLog )
	{
        int scaleTypeFlag = QP_LOG;
		DvString scaleType = _zdo->getXrefText(SCALETYP);
		if( !scaleType.empty() ) 
		{
			if (scaleType == "LOG")
				scaleTypeFlag = QP_LOG;
			else
				scaleTypeFlag = QP_LINEAR;

		}
        ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(scaleTypeFlag);
	}
	
	return rangeFound;
}

// ----------------------------------

bool SpectrumItem::getZRange(double &zmin, double &zmax)
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::getZRange" << endl;

	double xmin, xmax;
	
	if(frameType()==QP_TS)
	{
		xmin = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		xmax = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		xmin = frame()->xAxisTab->_axis_xy_fld->getStart();
		xmax = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	double ymin = panel()->yAxisTab->_axis_xy_fld->getStart();
	double ymax = panel()->yAxisTab->_axis_xy_fld->getEnd();

	// only try if x, y ranges available and valid
	if(xmin >= xmax || ymin >= ymax) return false;

	zmin = DBL_MAX;
	zmax = -DBL_MAX;
	bool recalc = false;

	 // fill X will reject if no Z object yet
	if(_zdo.is_nil()) return false;
	if( _zdo->get_id() != zdo_id_getZRange) recalc = true;
		
	zdo_id_getZRange = _zdo->get_id();

	if ( _xdo.is_nil() )   return false;
	if ( _ydo.is_nil() ) return false;
  
	QpAxisTab * axisTab;
	if(panel()->frameType() == QP_TS)  axisTab = frame()->tAxisTab;
	else  axisTab = frame()->xAxisTab;

	if(x_range_min_z != xmin || x_range_max_z != xmax )
	{
		x_range_min_z = xmin;
		x_range_max_z = xmax;
		recalc = true;
	}
	
	axisTab = panel()->yAxisTab;
	if(y_range_min_z != ymin || y_range_max_z != ymax )
	{
		y_range_min_z = ymin;
		y_range_max_z = ymax;
		recalc = true;
	}

	if(!recalc)
	{
		zmin = z_min_data;
		zmax = z_max_data;
		return true;
	}
	
	if(_nx_data != _zdo->seqSize() && _zdo->seqSize() != 1) return false;
  
	if( !isZConformal(_nx_data, _ny_data) )  return false;

	int XtoY = 0;
	if(DataTab->_XYtoggle->isChecked()) XtoY = 1;
	
	if( fill_Z_Data( XtoY,false) == -1)  return false;

	double zmin_for_log = DBL_MAX;

	for(int nx=0; nx<_x_start_all.size(); nx++)
	{
		double dx1 =  _x_start_all[nx] ;
		double dx2 =  _x_end_all[nx];
		// skip if out of range
		if(dx1 < dx2)
		{
			if(dx1 < xmin || dx2 > xmax) continue;
		}
		else
		{
			if(dx2 < xmin || dx1 > xmax) continue;
		}

		if (_z_data_all[nx].size() != _y_start_all[nx].size()) return false;
		for(int j=0; j<_y_start_all[nx].size();  j++)
		{
			double dy1 =  (_y_start_all[nx])[j] ;
			double dy2 =  (_y_end_all[nx])[j];
			// skip if out of range
			if(dy1 < dy2)
			{
				if(dy1 < ymin || dy2 > ymax) continue;
			}
			else
			{
				if(dy2 < ymin || dy1 > ymax) continue;
			}
			double value_j = (_z_data_all[nx])[j];
			if(value_j == missing_value) continue;
			 
			if(value_j < zmin_for_log && value_j != 0 ) zmin_for_log = value_j;
			
			if(value_j < zmin) 	zmin = value_j;
			if(value_j > zmax) zmax = value_j;
			
		}
	}
	// set lin/log from metadata
	
	if (_zdo->xref_exists(SCALETYP) && !ColourscaleTab->userSetColLinLog )
	{
        int scaleTypeFlag = QP_LOG;
		DvString scaleType = _zdo->getXrefText(SCALETYP);
		if( !scaleType.empty() ) 
		{
			if (scaleType == "LOG")
				scaleTypeFlag = QP_LOG;
			else
				scaleTypeFlag = QP_LINEAR;

		}
        ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(scaleTypeFlag);
	}
	 
	if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG)
	{
		if(zmin_for_log == DBL_MAX)
		{
			zmin = 1.0; // defensive just in case we found no non-zero values
			zmax = 10.0;
		}
		else zmin = zmin_for_log;
	}

	// keep for reuse if nothing changed   
	z_min_data = zmin;
	z_max_data = zmax;

	return true;
}



// ------------------------------------

DvObject_var SpectrumItem::getZDO()
{
	// get zdo object
	if ( QuSlotDebug() ) cout << "SpectrumItem::getZDO" << endl;

	DvObject_var zdo = DataTab->_z_data_fld->getObject();
	if(zdo.is_ok())
	{
		if(zdo->is_int()) zdo = zdo->convertToDbl();
		
		if(_z_obj_id != zdo->get_id())
		{
			_z_obj_id = zdo->get_id();
			
			// get z object fill value
			DvObject_var fillObj = new DvObject(-1.0);
			if( zdo->xref_exists(FILL_VALUE)) fillObj = zdo->get_xref(FILL_VALUE);
			missing_value = fillObj->asDouble();
		}
	 
	}
	else _z_obj_id = 0;
	
	return zdo;
}

// ------------------------------------

int SpectrumItem::fill_X_Data()
{
    if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_X_Data " << _zdo.is_nil() << endl;
	if (_zdo.is_nil() ) return 0; // cannot guarantee to create xdo so don't try till zdo exists

	DvObject_var xdo_end, xdo_delta_plus, xdo_delta_minus;
	
	// find x data object _xdo
	if(frameType() == QP_TS)
	{
		_xdo = _zdo->getTimeTags();
		if(_xdo.is_nil())
		{
			return 0;
		}
		
		if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
		if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
		if(_zdo->xref_exists(DEPEND_END_0))  xdo_end = _zdo->get_xref(DEPEND_END_0);

	}
	else
	{
		// X-Y Spectrogram
		
		if(DataTab->timeAsSec_toggle->isChecked())
		{
			DvObject_var tt = _zdo->getTimeTags();
			DvObject_var tOffset = DataTab->_x_start_data_fld->getObject();
			if(tt->not_time() || tOffset->not_time() ) return 0;
			
			DvTime t0 = tOffset->time(0);
			DvObject_var ss = new DvObject(0.0, tt->seqSize());
			for(int i=0; i<tt->seqSize(); i++)
			{
				ss->dbl(i) = tt->time(i).getDiffSecs(t0);
			}
			_xdo = ss;
			if(tt->xref_exists(DELTA_PLUS))  xdo_delta_plus = tt->get_xref(DELTA_PLUS);
			if(tt->xref_exists(DELTA_MINUS)) xdo_delta_minus = tt->get_xref(DELTA_MINUS);
		}
		else
		{
			// use X data slot object
		
			_xdo = DataTab->_x_start_data_fld->getObject();

			if(_xdo.is_ok() && _xdo->is_dbl())
			{
				if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
				if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
			}
			else
			{
				// Nothing in X slot
				if(_zdo->seqSize() == 1){
					if( DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_2)) _xdo = _zdo->get_xref(DEPEND_2);
						else  _xdo = makeIndexObject(_zdo, 2);

						if(_zdo->xref_exists(DEPEND_END_2)) xdo_end = _zdo->get_xref(DEPEND_END_2);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						if(_zdo->xref_exists(DEPEND_1)) _xdo = _zdo->get_xref(DEPEND_1);
						else  _xdo = makeIndexObject(_zdo, 1);

						if(_zdo->xref_exists(DEPEND_END_1)) xdo_end = _zdo->get_xref(DEPEND_END_1);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					if (XYtoggle_in_fill_x != DataTab->_XYtoggle->isChecked() )
					{
						XYtoggle_in_fill_x = DataTab->_XYtoggle->isChecked();
						_x_start_obj_id = 0; // force recalc as XY toggle changed
					}
				}
				else
				{
					if(DataTab->_XYtoggle->isChecked() )
					{
						// swap X and Y is on
						if(_zdo->xref_exists(DEPEND_2)) _xdo = _zdo->get_xref(DEPEND_2);
						else  _xdo = makeIndexObject(_zdo, 2);

						if(_zdo->xref_exists(DEPEND_END_2)) xdo_end = _zdo->get_xref(DEPEND_END_2);
						else if(!_xdo.is_nil())
						{
							if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
							if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
						}
					}
					else
					{
						// use dep 0 
						if(_zdo->xref_exists(DEPEND_0))
							_xdo = _zdo->get_xref(DEPEND_0);
						else
						{
							_xdo = makeIndexObject(_zdo, 0);
						}
					}
					if(_zdo->xref_exists(DEPEND_END_0))
						xdo_end = _zdo->get_xref(DEPEND_END_0);
					else if(!_xdo.is_nil())
					{
						if(_xdo->xref_exists(DELTA_PLUS))  xdo_delta_plus = _xdo->get_xref(DELTA_PLUS);
						if(_xdo->xref_exists(DELTA_MINUS)) xdo_delta_minus = _xdo->get_xref(DELTA_MINUS);
					}
				}
			} // end else on X slot obj set
		} // end else on use T as Sec toggle
	} // end if-else on spectrum type

	// Apply Time Interval Mask if set
	DvObject_var TImaskObj = DataTab->_mask_data_fld->getObject();
	if( TImaskObj.is_ok()) {
		DvEvent range = TImaskObj->getTimeRange();
		_xdo = _xdo->subSequence(range);
		if(!xdo_end.is_nil()) xdo_end = xdo_end->subSequence(range);
			if(!xdo_delta_plus.is_nil() && xdo_delta_plus->seqSize() > 1) xdo_delta_plus = xdo_delta_plus->subSequence(range);
			if(!xdo_delta_minus.is_nil() && xdo_delta_minus->seqSize() > 1) xdo_delta_minus = xdo_delta_minus->subSequence(range);
	}

	if(_xdo.is_nil()) return 0;

	_x_start_obj_id = _xdo->get_id();
	if( !xdo_end.is_nil() ) _x_end_obj_id = xdo_end->get_id();

	int nx_data;
	
	if( !xdo_delta_plus.is_nil() || !xdo_delta_minus.is_nil())
		nx_data = getXData(_xdo, xdo_delta_plus, xdo_delta_minus);
	else if( !xdo_end.is_nil() )
		nx_data = getXData(_xdo, xdo_end);	
	else
		nx_data = getXData(_xdo);

	return nx_data;
  
}

void SpectrumItem::refreshX()
{
	_x_start_obj_id=0;
	_x_end_obj_id=0;
	//	get_dobjs();
	_nx_data = fill_X_Data();
}

// -----------------------------------------------------

int SpectrumItem::fill_Y_Data()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::fill_Y_Data" << endl;

	if ( _zdo.is_nil() ) return -1; // cannot guarantee to create xdo so don't try till zdo exists
  	
	DvObject_var  ydo = DataTab->_y_start_data_fld->getObject();
	DvObject_var ydo_end;
	DvObject_var ydo_delta_minus;
	DvObject_var ydo_delta_plus;
 
	if(ydo.is_nil())
	{
		if(_zdo->nDims() < 1) return -1; // give up
		
		// use metadata off 
		if( DataTab->_XYtoggle->isChecked() )
		{
			if(_zdo->nDims() == 1)
			{
				// Depend 0 is being used, so Y is Dep 1
				if(_zdo->xref_exists(DEPEND_1))
					ydo = _zdo->get_xref(DEPEND_1);
				else
					ydo = makeIndexObject(_zdo, 1);
			
				if(_zdo->xref_exists(DEPEND_END_1))
					ydo_end = _zdo->get_xref(DEPEND_END_1);
			}
			else
			{
				// no Y object but swap XY is on
				if(_zdo->xref_exists(DEPEND_1))
					ydo = _zdo->get_xref(DEPEND_1);
				else
				{
					ydo = makeIndexObject(_zdo, 1);
				}
		
				if(_zdo->xref_exists(DEPEND_END_1))
					ydo_end = _zdo->get_xref(DEPEND_END_1);
			}
		}
		else
		{
			if(_zdo->nDims() == 1)
			{
				// Depend 0 is being used, so Y is Dep 1
				if(_zdo->xref_exists(DEPEND_1))
					ydo = _zdo->get_xref(DEPEND_1);
				else
					ydo = makeIndexObject(_zdo, 1);
			
				if(_zdo->xref_exists(DEPEND_END_1))
					ydo_end = _zdo->get_xref(DEPEND_END_1);
			}
			else
			{
				if(_zdo->xref_exists(DEPEND_2))
				{
					ydo = _zdo->get_xref(DEPEND_2);
				}

				if(ydo.is_nil())
					ydo = makeIndexObject(_zdo, 2);
			
				if(_zdo->xref_exists(DEPEND_END_2))
					ydo_end = _zdo->get_xref(DEPEND_END_2);
			}
			
		}
	
		if (XYtoggle_in_fill_y != DataTab->_XYtoggle->isChecked() )
		{
			XYtoggle_in_fill_y = DataTab->_XYtoggle->isChecked();
			_y_start_obj_id = 0; // force update
		}

	}
  
	// ydo must exist by now as we make one if need be
	if(ydo->xref_exists(DELTA_PLUS))
		ydo_delta_plus = ydo->get_xref(DELTA_PLUS);
	if(ydo->xref_exists(DELTA_MINUS))
		ydo_delta_minus = ydo->get_xref(DELTA_MINUS);


	// Apply Time Interval Mask if set
	DvObject_var TImaskObj = DataTab->_mask_data_fld->getObject();
	if( TImaskObj.is_ok()) {
		DvEvent range = TImaskObj->getTimeRange();
		ydo = ydo->subSequence(range);
		if(!ydo_end.is_nil()) ydo_end = ydo_end->subSequence(range);
		if(!ydo_delta_plus.is_nil() && ydo_delta_plus->seqSize() > 1) ydo_delta_plus = ydo_delta_plus->subSequence(range);
		if(!ydo_delta_minus.is_nil() && ydo_delta_minus->seqSize() > 1) ydo_delta_minus = ydo_delta_minus->subSequence(range);
	}

	if(ydo.is_nil()) return -1;

	_y_start_obj_id = ydo->get_id();
	if(!ydo_end.is_nil()) _y_end_obj_id = ydo_end->get_id();
		
	// fill the Y axis bins for each x bin (inhomogeneous arrays OK)
	int ny_data;
	if( !ydo_end.is_nil() )
	{
		ny_data = getYData( ydo, ydo_end);
	}
	else if( !ydo_delta_plus.is_nil() || !ydo_delta_minus.is_nil())
	{

		ny_data = getYData(ydo, ydo_delta_plus, ydo_delta_minus);
	}
	else
	{
		ny_data = getYData(ydo );
	}
	// NOTE: getYData() returns -1 for error, last array dim for sequences, array dim for matrix

	_ydo = ydo;

	return ny_data;

}

// -------------------------------------

int SpectrumItem::fill_Z_Data( int XtoY, bool inPlot)
{
	if ( QuSlotDebug() ) cout << " QpSpectrumItem::fill_Z_Data " << endl;

	if(_zdo->not_dbl())
	{
		if ( QuSlotDebug() ) cout << "z object wrong data type" << endl;
		return -1;
	}
	if(_nx_data != _y_start_all.size())
	{
		if ( QuSlotDebug() ) cout << "y dimension incorrect: check object has DEPEND_1 (and DEPEND_2 if XY spectrum)" << endl;
		return -1;
	}
	
	_z_data_all.clear();
	_z_data_all.resize(_nx_data);
	

	if( _zdo->seqSize() > 1 )
	{
 	 
		if(_zdo->nDims() == 1)
		{
			// Try 1D Mat sequence
			for(int nx=0; nx<_nx_data; nx++){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);

				for ( int i=0; i<ny; i++) {
					(_z_data_all[nx])[i] = _zdo->asDouble(nx*ny+i);

				}
			}
		}
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix sequence, but not 1D or 2D with one record");
            else
                std::cout << "Z data is a matrix sequence, but not 1D or 2D with one record" << std::endl;
			return -1;
		}

	}
	else if( _zdo->seqSize() == 1 )
	{
		if(_zdo->nDims() == 2)
		{
			double value_j;
			for(int nx=0; nx< _nx_data; nx++){
				int ny = _y_start_all[nx].size();
				_z_data_all[nx].resize(ny);
				for(int j=0; j< ny; j++)
				{
					if (XtoY == 1) value_j =  _zdo->dbl(0, j, nx);
					else      value_j =  _zdo->dbl(0, nx, j);
			 
					(_z_data_all[nx])[j] = value_j;
				}
			}
		}
        else if(_zdo->nDims() == 1)
        {
            // Try 1D Mat sequence
            for(int nx=0; nx<_nx_data; nx++){
                int ny = _y_start_all[nx].size();
                _z_data_all[nx].resize(ny);
                
                for ( int i=0; i<ny; i++) {
                    (_z_data_all[nx])[i] = _zdo->asDouble(nx*ny+i);
                    
                }
            }
        }
		else
		{
            if(inPlot)
			    plmtex("lv",-2.0,0.5,0.0, "Z data is a matrix, but not 2D or 1D sequence");
            else
                std::cout << "Z data is a matrix, but not 2D or 1D sequence" << std::endl;
			return -1;
		}
  
	}
	else
	{
        if(inPlot)
		    plmtex("lv",-2.0,0.5,0.0, "Z data is not a matrix or matrix sequence");
        else
            std::cout << "Z data is not a matrix or matrix sequence" << std::endl;
		return -1;
	}

	return 0;
}
// ----------------------------------

void SpectrumItem::setDefaultOptions()
{
	DataTab->timeAsSec_toggle->setChecked(false);
	DataTab->_XYtoggle->setChecked(false);
	DataTab->gap_toggle->setChecked(false);
	DataTab->interlace_toggle->setChecked(false);
	
	
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_max_fld->setText("");
	ColourscaleTab->_colourscale_auto_fld->setCurrentIndex(QP_FROM_META);
	ColourscaleTab->_colourscale_linlog_fld->setCurrentIndex(QP_LINEAR);
	ColourscaleTab->_colourscale_min_fld->setText("");
	ColourscaleTab->_colourscale_show_colour_bar_fld->setChecked(true);
	ColourscaleTab->_colourscale_colour_bar_width_fld->setValue(QPUI_COLOUR_BAR_WIDTH);
	ColourscaleTab->_colourscale_colour_bar_height_fld->setValue(QPUI_COLOUR_BAR_HEIGHT);
	ColourscaleTab->_colourscale_colour_bar_voffset_fld->setValue(QPUI_COLOUR_BAR_VOFFSET);
	ColourscaleTab->_colourscale_colour_bar_disp_fld->setValue(QPUI_COLOUR_BAR_DISP);
// 	ColourscaleTab->_colourscale_table_fld->setCurrentItem(QPUI_RAINBOW); //TODO
	ColourscaleTab->_colourscale_label_fld->setText("");
	ColourscaleTab->_colourscale_autolabel_fld->setChecked(true);
	
	updateAxes();
}



// --------------------------------------

void SpectrumItem::updateAxes()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::updateAxes" << endl;

	frame()->updateAxes();
			
	if(ColourscaleTab->_colourscale_auto_fld->currentIndex() != QP_FROM_USER) ColourscaleTab->setColourscaleMinMax();

	if(ColourscaleTab->_colourscale_autolabel_fld->isChecked())  ColourscaleTab->setColourScaleAutoLabel();


}

void SpectrumItem::addCustomColourTableTo(ColourTablePalette& palette)
{
    ColourTable current=ColourscaleTab->_colourtable_fld->getCurrentTable();
    if(!stdPalette.contains(current))
    {
        if(!palette.contains(current))
        {
            palette.push_back(current);
        }
    }
}

// --------------------------------------

QpuiReturnStatus SpectrumItem::plot()
{
	if ( QuSlotDebug() ) cout << "QpSpectrumItem::plot" << endl;
	
		// set the colour table
// 	page()->set_colour_table((QpuiColourTable)ColourscaleTab->_colourscale_table_fld->currentItem());
    ColourscaleTab->_colourtable_fld->getCurrentTable().apply();
	
	// plots the SpectroGram Object TS or XY
	
	double nominal_spacing = DBL_MAX;
	
	if ( _zdo.is_nil() ){
		plmtex("lv",-2.0,0.5,0.0,"No Z data object");
		return QPUI_FAILURE;
	}
	
	if(!DataTab->_z_data_fld->isValid())
	{
		plmtex("lv",-10.0,0.5,0.0,"Trace Plot ERROR: Invalid Z Slot");
		return QPUI_FAILURE;
	}
		
	if ( _nx_data == 0 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad X axis object");
		return QPUI_FAILURE;
	}
		
	if ( _ny_data == -1 ){
		plmtex("lv",-2.0,0.5,0.0,"Bad Y axis object(s)");
		return QPUI_FAILURE;
	}
	
	if( !DataTab->_y_start_data_fld->text().isEmpty()){
		// test Z and Y joined
		
		if( !_ydo->isJoined(*_zdo) ) {
			plmtex("lv",-2.0,0.5,0.0,"Z data and Y axis objects not joined");
			_ydo = DvObject_var(); // set to null
			return QPUI_FAILURE;
		}
	}
	
	// --------------------------------------------------
		// Z data object
	if(_nx_data != _zdo->seqSize() && _zdo->seqSize() != 1)
	{
		plmtex("lv",-2.0,0.5,0.0,"Z and X lengths differ");
		return QPUI_FAILURE;
	}
	if( !isZConformal(_nx_data, _ny_data))  {
		plmtex("lv",-2.0,0.5,0.0, "Z 2D matrix dimensions don't match axes");
		return QPUI_FAILURE;
	}

	int XtoY = 0;
	if(DataTab->_XYtoggle->isChecked()) XtoY = 1;
	
	if( fill_Z_Data( XtoY) == -1)  return QPUI_FAILURE;
	
	// set up colour scale
	double zmin = ColourscaleTab->_colourscale_min_fld->value();
	double zmax = ColourscaleTab->_colourscale_max_fld->value();

	
	if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG)  {
		zmin = SafeLog10(zmin);
		zmax = SafeLog10(zmax);
	}

    if(zmin >= zmax)
    {
        plmtex("lv",-2.0,0.5,0.0, "Invalid Colourscale Range");
        return QPUI_FAILURE;
    }
   
		// find  T axis offset for plot
	
	double x_offset=0.0;
	double x_min, x_max;
	
	if(frameType() == QP_TS)
	{
		x_min = frame()->tAxisTab->_axis_time_fld->getStart().getEpoch2000();
		x_max = frame()->tAxisTab->_axis_time_fld->getEnd().getEpoch2000();
	}
	else
	{
		x_min = frame()->xAxisTab->_axis_xy_fld->getStart();
		x_max = frame()->xAxisTab->_axis_xy_fld->getEnd();
	}
	
	if ( frameType() == QP_TS )
	{
		x_offset = x_min; // this is Epoch2000 value of range start in seconds
	}
	
	double zrange = zmax - zmin;
	PLFLT x[4], y[4];
	PLFLT minCellX = fabs( DataTab->minCellWidth->value() * (x_max - x_min)) ;
	
	// plot data 
	
	page()->setSmooth(false);

	double y_min = panel()->yAxisTab->_axis_xy_fld->getStart();
	double y_max = panel()->yAxisTab->_axis_xy_fld->getEnd();

	double dx1, dx2, dy1, dy2, zval;
	bool isXLog=(frame()->xAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG);
	bool isYLog=(panel()->yAxisTab->_axis_linlog_fld->currentIndex() == QP_LOG);
	bool isZLog=(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG);
	
	for(int nx=0; nx<_x_start_all.size(); ++nx)
	{
		dx1 =  _x_start_all[nx] ;
		dx2 =  _x_end_all[nx];
		
		// skip if both bin edges are out of range
		if((dx1 < x_min || dx1 > x_max) && ((dx2 < x_min || dx2 > x_max))) continue;
		
		if(isXLog)
		{
			x[0] = (PLFLT) SafeLog10(dx1 - x_offset);
			x[2] = (PLFLT) SafeLog10(dx2 - x_offset);
		}
		else
		{
			x[0] = (PLFLT) (dx1 - x_offset);
			x[2] = (PLFLT) (dx2 - x_offset);
		}

		if( fabs(x[2] - x[0]) < minCellX ) {
			if(x[2] >= x[0]) x[2] = x[0] + minCellX;
			else x[2] = x[0] - minCellX;
		}
		
		x[1] = x[0];
		x[3] = x[2];
		
		for(int j=0; j<_y_start_all[nx].size(); ++j)
		{
			dy1 = (_y_start_all[nx])[j];
			dy2 = (_y_end_all[nx])[j];
		
					// skip if out of range
			if((dy1 < y_min || dy1 > y_max) && (dy2 < y_min || dy2 > y_max)) continue;
					
			if(isYLog)
			{
				y[0] = SafeLog10(dy1);
				y[1] = SafeLog10(dy2);
			}
			else
			{
				y[0] = (PLFLT) dy1;
				y[1] = (PLFLT) dy2;
			}
			y[2] = y[1];
			y[3] = y[0];
		
			zval=(_z_data_all[nx])[j];
			if( zval != missing_value && isfinite(zval)) // ALBAN: changed for nothing plotted if data gap
			{
				PLFLT col;
				if(isZLog){
					col=(SafeLog10(zval) - zmin)/zrange;
				}
				else{
					col=(zval-zmin)/zrange;
				}
				
				if(col <0.) col=0.;
				if(col>=0.999) col=0.999;
				
				plcol1(col);
				
				
				plfill(4, x, y);
			}
	
		}
	}

	plcol0(1);

	// set colour bar
	if(ColourscaleTab->_colourscale_show_colour_bar_fld->isChecked())
	{
		PLFLT x1win_save, x2win_save, y1win_save, y2win_save;
		PLFLT x1vp_save, x2vp_save, y1vp_save, y2vp_save;
	
		plgvpw(&x1win_save, &x2win_save, &y1win_save, &y2win_save);
		plgvpd(&x1vp_save, &x2vp_save, &y1vp_save, &y2vp_save);
		string label="";

		PLFLT chsize_ndc, dummy;
		
		chsize_ndc=getCharHeight();
	
		x[0] = x2vp_save + ColourscaleTab->_colourscale_colour_bar_disp_fld->value() * chsize_ndc*1.5;
		x[2] = x[0] +  ColourscaleTab->_colourscale_colour_bar_width_fld->value() * chsize_ndc;
		float ht = (y2vp_save - y1vp_save) ;
		y[0] = y1vp_save + ht * ColourscaleTab->_colourscale_colour_bar_voffset_fld->value();
		y[1] = y[0] + ht * ColourscaleTab->_colourscale_colour_bar_height_fld->value();
        
        if(x[0] < 0.) x[0] = 0.;
        if(x[2] > 1.) x[2] = 1.;
        if(y[0] < 0.) y[0] = 0.;
        if(y[1] > 1.) y[1] = 1.;
      
		plvpor(x[0],x[2], y[0], y[1]);
		
		plwind(0., 1., zmin, zmax);
	
		x[0]=0.;
		x[1]=0.;
		x[2]=1.;
		x[3]=1.;

		PLFLT zstep=(zmax-zmin)/200.;
		PLFLT i=0.;
		for(PLFLT col=zmin;col<zmax;col+=zstep, i+=1./200.)
		{
			y[0] = col;
			y[3] = col;
			y[1] = col+zstep;
			y[2] = y[1];
			if(i>=1.) i=0.999;
			plcol1(i);
			plfill(4, x, y);
		}
	
		plcol0(1);

        page()->setSmooth(true);
		string yopt = "bcmtsv";
		if(ColourscaleTab->_colourscale_linlog_fld->currentIndex() == QP_LOG) yopt += "l";
		plschr(0., frame()->DisplayTab->_ticks_char_size_fld->value());
		
		plwid(QtWidth2PlWidth(frame()->DisplayTab->_line_width_fld->value()));

		PLFLT _ticks_length = ColourscaleTab->_colourscale_ticklength_fld->value();
		
		plsmaj(0., _ticks_length);
		plsmin(0., _ticks_length*0.6);

        PLLabelDefaults y_label_opt;
        y_label_opt.exp_label_disp = 2.0 + (PLFLT) ColourscaleTab->_exp_label_offset_y_fld->value();
        y_label_opt.exp_label_pos = 1.02 +(PLFLT) ColourscaleTab->_exp_label_offset_x_fld->value();
        y_label_opt.exp_label_just = 0.0;
        plslabelfunc(NULL, &y_label_opt);
		plbox("bc", 0.0, 0, yopt.c_str(), 0.0, 0);
        plslabelfunc(NULL, NULL);
        
	
		plwid(1);

		label = ColourscaleTab->_colourscale_label_fld->text().toLatin1().data();
		Q_plmtex("l", 1., 0.5, 0.5, SafePGPString(label).c_str());
		plvpor(x1vp_save, x2vp_save, y1vp_save, y2vp_save);
		plwind(x1win_save, x2win_save, y1win_save, y2win_save);
		page()->setSmooth(false);
		
	}
	
	// this just overplots empty box. Fixes loss of T date label in panel
	// Plplot bug?
	plbox("b", 0.0, 0, "", 0.0, 0); 

	return QPUI_SUCCESS;
}



//     ----------------------

QDomElement SpectrumItem::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Plot_Item");
    xml.setAttribute("Type", "Spectrogram");
    xml.appendChild(createTextXMLNode(doc, "Name", getName()));
    xml.appendChild(DataTab->toXMLElement(doc, QP_SPECTROGRAM));
    QString linkText="";
    if(!getLinkText().isEmpty())
    {
        linkText=getLinkText()+"S";
    }
    xml.appendChild(createTextXMLNode(doc, "Link_Text", linkText));

    xml.appendChild(ColourscaleTab->toXMLElement(doc));

    xml.appendChild(createTextXMLNode(doc, "Missing_Value", missing_value));

    return xml;
}

void SpectrumItem::fromXMLElement(QDomElement& el)
{
    QString s;
    
    if(getXMLNodeText(el, "Name", s))
    {
        setName(s);
    }

    QDomElement dataElement=el.firstChildElement("Data_Tab");
    if(!dataElement.isNull())
    {
        DataTab->fromXMLElement(dataElement, I_Type);
		if(frameType()== QP_XY)
		{
			if(getXMLNodeText(el, "X_Start_Data", s))
            {
				DataTab->_x_start_data_fld->setText(s);
			}
			if(getXMLNodeText(el, "X_End_Data", s))
            {
	//			DataTab->_x_end_data_fld->setText(s);
			}
			if(getXMLNodeText(el, "Time_As_Sec", s))
            {
				DataTab->timeAsSec_toggle->setChecked(s.toInt());
			}

			DataTab->_x_start_data_fld->reValidate();
//			DataTab->_x_end_data_fld->reValidate();
		}
	}

    
    QDomElement colourscaleElement=el.firstChildElement("Colourscale_Tab");
    if(!colourscaleElement.isNull())
    {
        ColourscaleTab->fromXMLElement(colourscaleElement);
    }

	if(getXMLNodeText(el, "Link_Text", s))
	{
		setLinkText(s);
	}
	
    if(getXMLNodeText(el, "Missing_Value", s))
    {
        missing_value=(bool)s.toDouble();
    }
	
	DvObject_var o=getWLObject(getName().toLatin1().data());
	if(o.is_ok()) WLcontext=getName();
	
//	DataTab->_y_start_data_fld->reValidate();
//	DataTab->_z_data_fld->reValidate(true);
}


DvObject_var makeIndexObject(DvObject_var &zdo, int indxFlag){
	
    int indx = indxFlag-1;
	size_t len=0;

	if (indxFlag == 0 && zdo->seqSize() > 1 )
	{
     // use sequence length
		len = zdo->seqSize();
	}
	else if( zdo->seqSize() > 1 )
	{
        // use index
		if(zdo->nDims() > indx)
		{
			len = zdo->Dims()[indx];
		}
		else
		{
			len = zdo->seqSize();
		}
  
	}
	else if( zdo->seqSize() == 1 )
	{
        // use index
		if(zdo->nDims() > indx)
		{
			len = zdo->Dims()[indx];
		}
  
	}
  
  // create a matrix to return
	if (len > 0)
	{
        // create matrix whose values are the index 
		vector <size_t> d;
		d.push_back(len);
		DvObject_var dimIdxObj = new DvObject(0.0, d);
		for (int i=0; i<len; i++) dimIdxObj->dbl(i) = i+1;
		return dimIdxObj;
	}
	else
	{
    // This should never be reached, but create something in case
		cout << "CANNOT create axis object " << indxFlag << " (t-axis=0; x-axis=1, yaxis=2)"  << endl;
		DvObject_var dimIdxObj = new DvObject(0.0);
		return dimIdxObj;
	}
  
}
 
 
