// qpui_trace_options.cc
//
// Plot Interface Implementation - Trace Options 
//
// Janet Barnes
//
// July 2002
//
//  Tony Allen
//  Jan 2008

#include "qpui.h"
#include "qdutil.h"

#include <qcolordialog.h>
#include <qtooltip.h>

extern QPalette QPUI_normal_palette;



// QpTraceTab Class
// ----------------

QpTraceTab::QpTraceTab(QTabWidget *tabWidget,  const QString& name, TraceItem * _trace):
		QpTab(tabWidget, name, QP_TRACE_TAB)
{
	// Constructor for the Trace Options Tab
   
	// keep track of item which owns this tab
	I_Trace = _trace;
	
	create();
	
	_custom_symbol_fld->setEnabled(false);

	connect( _line_style_fld, SIGNAL(activated(int)), this, SLOT(setLineStyle(int)) );
	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));
	connect( _connect_style_fld, SIGNAL(activated(int)), this, SLOT(setConnectStyle(int)) );
	connect( _line_scale_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineScale(double)) );
	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));
	connect( _symbol_shape_fld, SIGNAL(activated(int)), this, SLOT(setSymbolShape(int)) );
	connect(_symbol_size_fld,SIGNAL(valueChanged(double)), this, SLOT(setSymbolSize(double)));
	connect(_symbol_colour_fld, SIGNAL(clicked()), this, SLOT(setSymbolColour()));
	connect(_custom_symbol_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomSymbol(const QString&)));
}

QpTraceTab::QpTraceTab(QpTraceTab* tab, TraceItem* _trace):
		QpTab((QTabWidget*)(tab->parent()), "TraceTab", QP_TRACE_TAB)
{
	// Constructor for the Trace Options Tab
   
	// keep track of item which owns this tab
	I_Trace = _trace;
	
	create();

    _line_style_fld->setCurrentIndex(tab->_line_style_fld->currentIndex());
    _line_width_fld->setValue(tab->_line_width_fld->value());
    _connect_style_fld->setCurrentIndex(tab->_connect_style_fld->currentIndex());
	_line_scale_fld->setValue(tab->_line_scale_fld->value());
	_symbol_shape_fld->setCurrentIndex(tab->_symbol_shape_fld->currentIndex());
	_symbol_size_fld->setValue(tab->_symbol_size_fld->value());
	_custom_symbol_fld->setText(tab->_custom_symbol_fld->text());
	_custom_symbol_fld->setEnabled(tab->_custom_symbol_fld->isEnabled());
	
    _c_data_fld->setText(tab->_c_data_fld->text());
    c_range_auto->setChecked(tab->c_range_auto->isChecked());
    c_range_log->setChecked(tab->c_range_log->isChecked());
    _show_colour_bar->setChecked(tab->_show_colour_bar->isChecked());
    colLabSlot->setText(tab->colLabSlot->text());
    _exp_label_offset_x->setValue(tab->_exp_label_offset_x->value());
    _exp_label_offset_y->setValue(tab->_exp_label_offset_y->value());
    _colour_bar_width->setValue(tab->_colour_bar_width->value());
    _colour_bar_height->setValue(tab->_colour_bar_height->value());
    _colour_bar_disp->setValue(tab->_colour_bar_disp->value());
    _colour_bar_voffset->setValue(tab->_colour_bar_voffset->value());
    _colourscale_ticklength->setValue(tab->_colourscale_ticklength->value());
    _colour_on->setChecked(tab->_colour_on->isChecked());
    _size_on->setChecked(tab->_size_on->isChecked());
    _c_min_fld->setText(tab->_c_min_fld->text());
    _c_max_fld->setText(tab->_c_max_fld->text());
    
	connect( _line_style_fld, SIGNAL(activated(int)), this, SLOT(setLineStyle(int)) );
	connect(_line_width_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineWidth(double)));
	connect( _connect_style_fld, SIGNAL(activated(int)), this, SLOT(setConnectStyle(int)) );
	connect( _line_scale_fld, SIGNAL(valueChanged(double)), this, SLOT(setLineScale(double)) );
	connect(_line_colour_fld, SIGNAL(clicked()), this, SLOT(setLineColour()));
	connect( _symbol_shape_fld, SIGNAL(activated(int)), this, SLOT(setSymbolShape(int)) );
	connect(_symbol_size_fld,SIGNAL(valueChanged(double)), this, SLOT(setSymbolSize(double)));
	connect(_symbol_colour_fld, SIGNAL(clicked()), this, SLOT(setSymbolColour()));
	connect(_custom_symbol_fld, SIGNAL(textChanged(const QString &)), this, SLOT(setCustomSymbol(const QString&)));
}

void QpTraceTab::create()
{
	QVBoxLayout *trace_top_layout = new QVBoxLayout( this);
	trace_top_layout->setSpacing(4);

	QGroupBox *trace_line_groupbox=new QGroupBox(this);

	trace_line_groupbox->setTitle("Line");

// 	QWidget* trace_widget = new QWidget(trace_line_groupbox, "trace_widget");

	QGridLayout *trace_layout = new QGridLayout;
	trace_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	trace_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* style_label = new QLabel("Line style: ");
	trace_layout->addWidget(style_label,1,0,Qt::AlignRight);
	_line_style_fld=new QComboBox();
	_line_style_fld->insertItem(QP_NOLINE, "None");
	_line_style_fld->insertItem(QP_SOLID, "Solid");
	_line_style_fld->insertItem(QP_DASHED, "Dashed");
	_line_style_fld->insertItem(QP_DOTDASH, "Dot-dash");
	_line_style_fld->insertItem(QP_DOTTED, "Dotted");
	_line_style_fld->insertItem(QP_DASHDOTDOTDOT, "Dash-dot-dot-dot");
	QFontMetrics line_style_fm(_line_style_fld->font());
	int line_style_w=line_style_fm.width("Dash-dot-dot-dot");
	int line_style_h=line_style_fm.height();
	_line_style_fld->setMinimumSize(line_style_w,line_style_h);
	_line_style_fld->setToolTip("Select line style");
	trace_layout->addWidget(_line_style_fld,1,1,Qt::AlignLeft);

	_line_style_fld->setCurrentIndex(QP_SOLID);

	QLabel* line_width_label=new QLabel("Line width: ");
	trace_layout->addWidget(line_width_label,1,2,Qt::AlignRight);
	_line_width_fld = new FloatSpinBox(0.1,20.,1,0.1, trace_line_groupbox, "line_width");
	_line_width_fld->setValue(QSAS_profiles->current()->LineWidth);
	_line_width_fld->setFixedWidth(100);
	trace_layout->addWidget(_line_width_fld,1,3,Qt::AlignLeft);
	
	QLabel* line_scale_label=new QLabel("Pattern scale: ");
	trace_layout->addWidget(line_scale_label,3,0,Qt::AlignRight);
	_line_scale_fld = new FloatSpinBox(0.1,20.,1,0.1, trace_line_groupbox, "line_width");
	_line_scale_fld->setValue(QSAS_profiles->current()->LineWidth);
	_line_scale_fld->setFixedWidth(100);
	trace_layout->addWidget(_line_scale_fld,3,1,Qt::AlignLeft);

	QLabel* connect_label = new QLabel("Connect style: ");
	trace_layout->addWidget(connect_label,5,0,Qt::AlignRight);
	_connect_style_fld = new QComboBox();
	_connect_style_fld->insertItem(QPUI_STRAIGHT, "Straight");
	_connect_style_fld->insertItem(QPUI_HIST, "Histogram");
	_connect_style_fld->insertItem(QPUI_NOCONNECT, "None");
	QFontMetrics connect_style_fm(_connect_style_fld->font());
	int connect_style_w=connect_style_fm.width("Histogram");
	int connect_style_h=connect_style_fm.height();
	_connect_style_fld->setMinimumSize(connect_style_w,connect_style_h);
	_connect_style_fld->setToolTip("Select style for connecting points");
	trace_layout->addWidget(_connect_style_fld,5,1,Qt::AlignLeft);

	QLabel* line_colour_label=new QLabel("Line colour: ");
	trace_layout->addWidget(line_colour_label,5,2,Qt::AlignRight);
	_line_colour_fld = new QPushButton("Colour...");
	_line_colour_fld->setFixedWidth(100);
	_line_colour_fld->setFixedHeight(30);
	QFont f = QFont(qApp->font());
	f.setBold(TRUE);
	_line_colour_fld->setFont(f);
	trace_layout->addWidget(_line_colour_fld,5,3,Qt::AlignLeft);

	trace_layout->setRowStretch(0,0);
	trace_layout->setRowStretch(1,1);
	trace_layout->setRowStretch(2,0);
	trace_layout->setRowStretch(3,1);
	trace_layout->setRowStretch(4,0);

	trace_layout->setColumnStretch(0,0);
	trace_layout->setColumnStretch(1,0);
	trace_layout->setColumnStretch(2,0);
	trace_layout->setColumnStretch(3,0);
	trace_layout->setColumnStretch(4,1);

	trace_line_groupbox->setLayout(trace_layout);
	
	trace_top_layout->addWidget(trace_line_groupbox);

	QGroupBox *trace_trace_symbols_groupbox=new QGroupBox(this);

	trace_trace_symbols_groupbox->setTitle("Symbols");

	QGridLayout *trace_symbols_layout=new QGridLayout;
	trace_symbols_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	trace_symbols_layout->setMargin(QSAS_profiles->current()->GUImargins);

	QLabel* shape_label=new QLabel("Shape: ");
	trace_symbols_layout->addWidget(shape_label,1,0,Qt::AlignRight);
	_symbol_shape_fld=new QComboBox();
	_symbol_shape_fld->addItem("None");
	_symbol_shape_fld->addItem("Point");
	_symbol_shape_fld->addItem("Plus sign");
	_symbol_shape_fld->addItem("Asterisk");
	_symbol_shape_fld->addItem("Circle");
	_symbol_shape_fld->addItem("Cross");
	_symbol_shape_fld->addItem("Square");
	_symbol_shape_fld->addItem("Triangle");
	_symbol_shape_fld->addItem("Circled Point");
	_symbol_shape_fld->addItem("Filled Diamond");
	_symbol_shape_fld->addItem("Diamond");
	_symbol_shape_fld->addItem("Star");
	_symbol_shape_fld->addItem("Wide cross");
	_symbol_shape_fld->addItem("Filled Square");
	_symbol_shape_fld->addItem("Filled Circle");
	_symbol_shape_fld->addItem("Filled Star");
	_symbol_shape_fld->addItem("Left Arrow");
	_symbol_shape_fld->addItem("Right Arrow");
	_symbol_shape_fld->addItem("Up Arrow");
	_symbol_shape_fld->addItem("Down Arrow");
	_symbol_shape_fld->addItem("Custom...");
	QFontMetrics symbol_shape_fm(_symbol_shape_fld->font());
	int symbol_shape_w=symbol_shape_fm.width("Filled Triangle");
	int symbol_shape_h=symbol_shape_fm.height();
	_symbol_shape_fld->setMinimumSize(symbol_shape_w,symbol_shape_h);
	_symbol_shape_fld->setToolTip("Select symbol shape");
	trace_symbols_layout->addWidget(_symbol_shape_fld,1,1,Qt::AlignLeft);
	oldSymbolShape = 0;	
	
	QLabel* size_label = new QLabel("Size: ");
	trace_symbols_layout->addWidget(size_label,1,2,Qt::AlignRight);
	_symbol_size_fld = new FloatSpinBox(0.1,20.,1,0.5,trace_trace_symbols_groupbox, "symbol_size");
	_symbol_size_fld->setFixedWidth(100);
	trace_symbols_layout->addWidget(_symbol_size_fld,1,3,Qt::AlignLeft);
	_symbol_size_fld->setValue(1.0);
	
	
	QLabel* custom_label = new QLabel("Custom symbol or text: ");
	trace_symbols_layout->addWidget(custom_label,3,0,Qt::AlignRight);
	_custom_symbol_fld = new QLineEdit;
	_custom_symbol_fld->setFixedWidth(100);
	trace_symbols_layout->addWidget(_custom_symbol_fld,3,1,Qt::AlignLeft);
	
	QLabel* symbols_colour_label=new QLabel("Colour: ");
	trace_symbols_layout->addWidget(symbols_colour_label,3,2,Qt::AlignRight);
	_symbol_colour_fld=new QPushButton("Colour...");
	_symbol_colour_fld->setFixedWidth(100);
	_symbol_colour_fld->setFixedHeight(30);
	_symbol_colour_fld->setFont(f); // declared bold above
	trace_symbols_layout->addWidget(_symbol_colour_fld,3,3,Qt::AlignLeft);

	trace_symbols_layout->setRowStretch(0,0);
	trace_symbols_layout->setRowStretch(1,1);
	trace_symbols_layout->setRowStretch(2,0);
	trace_symbols_layout->setRowStretch(3,1);
	trace_symbols_layout->setRowStretch(4,0);

	trace_symbols_layout->setColumnStretch(0,0);
	trace_symbols_layout->setColumnStretch(1,0);
	trace_symbols_layout->setColumnStretch(2,0);
	trace_symbols_layout->setColumnStretch(3,0);
	trace_symbols_layout->setColumnStretch(4,1);

	trace_trace_symbols_groupbox->setLayout(trace_symbols_layout);
	trace_top_layout->addWidget(trace_trace_symbols_groupbox);


	QGroupBox *trace_trace_col_var_groupbox=new QGroupBox(this);
	trace_trace_col_var_groupbox->setTitle("Optional Colour Variable");
	
	QGridLayout *trace_col_var_layout=new QGridLayout;
	trace_col_var_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	trace_col_var_layout->setMargin(QSAS_profiles->current()->GUImargins);

	_description_fld=new QLabel("If used, sets symbol colour or size by variable (X-Y-Z plots)");
	trace_col_var_layout->addWidget(_description_fld,0,0,1,4,Qt::AlignCenter);
	
	_c_label_fld=new QLabel("Colour variable: ");
	trace_col_var_layout->addWidget(_c_label_fld,1,0,Qt::AlignRight);
	_c_data_fld=new QguWLinput(trace_trace_col_var_groupbox,"c_data");
	_c_data_fld->setMinimumWidth(450);
	_c_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
	trace_col_var_layout->addWidget(_c_data_fld,1,1,1,3,Qt::AlignLeft);
	
	connect(_c_data_fld, SIGNAL(edited_OK()), this, SLOT(setColData()));
	connect(_c_data_fld, SIGNAL(slotEmpty()), this, SLOT(setColDataOff()));
	
	_size_on = new QCheckBox("Apply to size");
	trace_col_var_layout->addWidget(_size_on,2,0,Qt::AlignLeft);
	_size_on->setChecked(false);
	connect(_size_on, SIGNAL(clicked(bool)), this, SLOT(setSizeOn(bool)));

	_colour_on = new QCheckBox("Apply to Colour");
	trace_col_var_layout->addWidget(_colour_on,2,1,Qt::AlignLeft);
	_colour_on->setChecked(true);
	connect(_colour_on, SIGNAL(clicked(bool)), this, SLOT(setColourOn(bool)));

	c_range_auto = new QCheckBox("Auto Scale");
	trace_col_var_layout->addWidget(c_range_auto,2,2, Qt::AlignLeft);
	c_range_auto->setChecked(true);
	connect(c_range_auto, SIGNAL(clicked(bool)), this, SLOT(setAutoColour(bool)));

	c_range_log = new QCheckBox("Log Scale");
	trace_col_var_layout->addWidget(c_range_log,2,3, Qt::AlignLeft);
	c_range_log->setChecked(false);
	connect(c_range_log, SIGNAL(clicked(bool)), this, SLOT(setLogColour(bool)));

	minLabel = new QLabel("Min value: ");
	trace_col_var_layout->addWidget(minLabel, 4, 0, Qt::AlignRight);
	_c_min_fld = new QLineEdit();
	_c_min_fld->setMinimumWidth(50);
	_c_min_fld->setMaximumWidth(100);
	_c_min_fld->setText("0.0");
	_c_min_fld->setToolTip("Minimum data value (Avoid log(0) )");
	connect(_c_min_fld, SIGNAL(textEdited(const QString & )), this, SLOT(setColUserMin(const QString &)));
	trace_col_var_layout->addWidget(_c_min_fld, 4, 1, Qt::AlignLeft);

	maxLabel = new QLabel("Max value: ");
	trace_col_var_layout->addWidget(maxLabel, 4, 2, Qt::AlignRight);
	_c_max_fld = new QLineEdit();
	_c_max_fld->setMinimumWidth(50);
	_c_max_fld->setMaximumWidth(100);
	_c_max_fld->setText("1.e6");
	_c_max_fld->setToolTip("Maximum data value");
	connect(_c_max_fld, SIGNAL(textEdited(const QString & )), this, SLOT(setColUserMax(const QString &)));
	trace_col_var_layout->addWidget(_c_max_fld, 4, 3, Qt::AlignLeft);


	_show_colour_bar = new QCheckBox("Show Colour Bar");
	trace_col_var_layout->addWidget(_show_colour_bar,5,0,Qt::AlignLeft);
	_show_colour_bar->setChecked(false);
	connect(_show_colour_bar, SIGNAL(clicked(bool)), this, SLOT(showColourBar(bool)));
	useColBarOn=true; // default colour bar on once C object dropped

	colour_table_label = new QLabel("Colour Table: ");
	trace_col_var_layout->addWidget(colour_table_label, 5, 2, Qt::AlignRight);
    colourTable = new ColourTableButton;
    trace_col_var_layout->addWidget(colourTable, 5, 3, Qt::AlignLeft);
	connect(colourTable, SIGNAL(selectedProfile(int)), this, SLOT(setColourTableProfile(int)));
	connect(colourTable, SIGNAL(selectedSession(int)), this, SLOT(setColourTableSession(int)));
	connect(colourTable, SIGNAL(selectedStd(int)), this, SLOT(setColourTableStd(int)));

	colLabLab = new QLabel("Colour bar label: ");
	trace_col_var_layout->addWidget(colLabLab, 6, 0, Qt::AlignRight);
	
	colLabSlot = new QLineEdit();
	colLabSlot->setMinimumWidth(200);
	connect(colLabSlot, SIGNAL(textEdited(const QString& )), this, SLOT(setUserColLab(const QString& )));
	trace_col_var_layout->addWidget(colLabSlot, 6, 1, 1, 3);
	userColLab = false;

	QGroupBox* _exp_params_groupbox=new QGroupBox("Exponent offset");
    QHBoxLayout * exp_layout=new QHBoxLayout;
    
    exp_layout->addWidget(new QLabel("Horizontal offset:"), Qt::AlignRight);

    _exp_label_offset_x=new FloatSpinBox(-20., 20., 2, 0.02);
    exp_layout->addWidget(_exp_label_offset_x,Qt::AlignLeft);
	_exp_label_offset_x->setValue(0.);
	connect( _exp_label_offset_x, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelXOffset(double)));
    exp_layout->addWidget(new QLabel("Vertical offset:"), Qt::AlignRight);

    _exp_label_offset_y=new FloatSpinBox(-20., 20., 2, 0.02);
    exp_layout->addWidget(_exp_label_offset_y, Qt::AlignLeft);
	_exp_label_offset_y->setValue(0.);
	connect( _exp_label_offset_y, SIGNAL(valueChanged(double)), this, SLOT(setAxisExpLabelYOffset(double)));

    exp_layout->addStretch(10);
    
    _exp_params_groupbox->setLayout(exp_layout);

    trace_col_var_layout->addWidget(_exp_params_groupbox, 7, 0, 1, 4);

	QGroupBox* _bar_params_groupbox=new QGroupBox("Colour Bar Position");
    QGridLayout * bar_layout=new QGridLayout;

	QLabel* _colour_bar_width_lbl = new QLabel("Width: ");
	bar_layout->addWidget( _colour_bar_width_lbl, 0, 0, Qt::AlignRight);
	_colour_bar_width = new FloatSpinBox( 0.01, 40., 1, .1, trace_trace_col_var_groupbox, "colour_bar_width");
	_colour_bar_width->setFixedWidth(100);
	bar_layout->addWidget( _colour_bar_width, 0, 1, Qt::AlignLeft);
	_colour_bar_width->setValue(QPUI_COLOUR_BAR_WIDTH);
	connect( _colour_bar_width, SIGNAL(valueChanged(double)), this, SLOT(setColBarWidth(double)));

	QLabel* _colour_bar_height_lbl = new QLabel("Height Fraction: ");
	bar_layout->addWidget( _colour_bar_height_lbl, 0, 2, Qt::AlignRight);
	_colour_bar_height = new FloatSpinBox( 0.2,10.,2,.1,trace_trace_col_var_groupbox, "colour_bar_height");
	_colour_bar_height->setFixedWidth(100);
	bar_layout->addWidget( _colour_bar_height, 0, 3, Qt::AlignLeft);
	_colour_bar_height->setValue(QPUI_COLOUR_BAR_HEIGHT);
	connect( _colour_bar_height, SIGNAL(valueChanged(double)), this, SLOT(setColBarHeight(double)));
	
	QLabel* _colour_bar_disp_lbl = new QLabel("Displacement: ");
	bar_layout->addWidget( _colour_bar_disp_lbl, 1, 0, Qt::AlignRight);
	_colour_bar_disp = new FloatSpinBox( -60.0,10.,1,.2,trace_trace_col_var_groupbox, "colour_bar_disp");
	_colour_bar_disp->setFixedWidth(100);
	bar_layout->addWidget( _colour_bar_disp, 1, 1, Qt::AlignLeft);
    _colour_bar_disp->setValue(QPUI_COLOUR_BAR_DISP);
	connect( _colour_bar_disp, SIGNAL(valueChanged(double)), this, SLOT(setColBarHoffset(double)));

	QLabel* _colour_bar_voffset_lbl = new QLabel("Vertical Offset: ");
	bar_layout->addWidget( _colour_bar_voffset_lbl, 1, 2, Qt::AlignRight);
	_colour_bar_voffset = new FloatSpinBox( -10., 10., 2, .1, trace_trace_col_var_groupbox, "colour_bar_voffset");
	_colour_bar_voffset->setFixedWidth(100);
	bar_layout->addWidget( _colour_bar_voffset, 1, 3, Qt::AlignLeft);
	_colour_bar_voffset->setValue(QPUI_COLOUR_BAR_VOFFSET);
	connect( _colour_bar_voffset, SIGNAL(valueChanged(double)), this, SLOT(setColBarVoffset(double)));
	
	QLabel* colourscale_ticklength_disp_lbl=new QLabel("Ticks length: ");
	bar_layout->addWidget(colourscale_ticklength_disp_lbl, 2, 0, Qt::AlignRight);
	_colourscale_ticklength= new FloatSpinBox(0.01, 40., 1, .5, this, "ticks_length");
	_colourscale_ticklength->setFixedWidth(100);
	bar_layout->addWidget( _colourscale_ticklength, 2, 1, Qt::AlignLeft);
	_colourscale_ticklength->setValue(QPUI_TICKS_LENGTH);
	connect( _colourscale_ticklength, SIGNAL(valueChanged(double)), this, SLOT(setColBarTickLen(double)));

    _bar_params_groupbox->setLayout(bar_layout);
    trace_col_var_layout->addWidget(_bar_params_groupbox, 8, 0, 1, 4);

	trace_trace_col_var_groupbox->setLayout(trace_col_var_layout);
	trace_top_layout->addWidget(trace_trace_col_var_groupbox);

	if(I_Trace->I_Type != QP_TRACE) {
		_c_label_fld->setEnabled(false);
		_c_data_fld->setEnabled(false);
		_c_min_fld->setEnabled(false);
		_c_max_fld->setEnabled(false);
		colourTable->setEnabled(false);
		colLabSlot->setEnabled(false);
		c_range_auto->setEnabled(false);
		c_range_log->setEnabled(false);
		minLabel->setEnabled(false);
		maxLabel->setEnabled(false);
		colLabLab->setEnabled(false);
		colour_table_label->setEnabled(false);
		_description_fld->setEnabled(false);
		_show_colour_bar->setEnabled(false);
		_exp_label_offset_x->setEnabled(false);
		_exp_label_offset_y->setEnabled(false);
		_colour_bar_width->setEnabled(false);
		_colour_bar_height->setEnabled(false);
		_colour_bar_disp->setEnabled(false);
		_colour_bar_voffset->setEnabled(false);
		_colourscale_ticklength->setEnabled(false);
	}
	
	trace_top_layout->addSpacing(4);
	trace_top_layout->addStretch(10);

}

// Trace Options
// ------------

void QpTraceTab::setColUserMin(const QString & txt)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColUserMin  " << endl;
	c_range_auto->setChecked(false);
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_MIN, 0, 0.0, txt);
	
}

void QpTraceTab::setColUserMax(const QString & txt)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColUserMax  " << endl;
	c_range_auto->setChecked(false);
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_MAX, 0, 0.0, txt);
	
}

void QpTraceTab::setAutoColour(bool on)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setAutoColour  " << endl;
	I_Trace->updateColourBar();
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_AUTO_COL_BAR, 0, 0.0, "", on);

}

void QpTraceTab::setLogColour(bool on)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setLogColour  " << endl;
	I_Trace->updateColourBar();
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LOG_COL_BAR, 0, 0.0, "", on);

}

void QpTraceTab::setUserColLab(const QString& txt)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setUserColLab  " << endl;
	if(txt.isEmpty() ) userColLab = false;
	else userColLab = true;

	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_USER_COL_LAB, 0, 0.0, txt);

}


void QpTraceTab::setColDataOff()
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColDataOff  " << endl;

	// if field is empty show colour choice
	_symbol_colour_fld->setEnabled(true);
	_show_colour_bar->setChecked(false);
	_symbol_shape_fld->setCurrentIndex(oldSymbolShape);
}

void QpTraceTab::setColData()
{

	if( QuSlotDebug() ) cout << " QpTraceTab::setColData  " << endl;
	// slot called when col data name field changed on widget
		
	// is change to base object or component...
	DvString BaseName = baseObjectText( _c_data_fld->text().toLatin1().data() ); // new value
	DvString CompText = compText( _c_data_fld->text().toLatin1().data() ); // new value


	bool compOnly = false;
	if(old_BaseName == BaseName && old_Comp != CompText) compOnly = true;
	old_BaseName = BaseName;
	old_Comp = CompText;
	
	// if field is used hide colour choice
	_symbol_colour_fld->setEnabled(false);
	oldSymbolShape = _symbol_shape_fld->currentIndex();
	
	if(_symbol_shape_fld->currentIndex() == 0) _symbol_shape_fld->setCurrentIndex(1);
	 
	I_Trace->WLcontext = BaseName.c_str();
	if(this->isVisible() ) {
		// propagate to selected items 

 		if(compOnly) I_Trace->Tree()->setTraceOption(TRACE_COL_COMP, 0, 0.0, CompText.c_str());
		else I_Trace->Tree()->setTraceOption(TRACE_COL_DATA, 0, 0.0, BaseName.c_str());

	}
		
	I_Trace->updateColourBar(); // also sets _cdo 
	
	_show_colour_bar->setChecked(useColBarOn);

	
}


void QpTraceTab::setAxisExpLabelXOffset(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setAxisExpLabelXOffset  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_EXP_COL_BAR_X, 0, val);
}

void QpTraceTab::setAxisExpLabelYOffset(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setAxisExpLabelYOffset  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_EXP_COL_BAR_Y, 0, val);
}

void QpTraceTab::setColBarWidth(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColBarWidth  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_BAR_W, 0, val);
}

void QpTraceTab::setColBarHeight(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColBarHeight  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_BAR_H, 0, val);
}

void QpTraceTab::setColBarHoffset(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColBarHoffset  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_BAR_HOFF, 0, val);
}

void QpTraceTab::setColBarVoffset(double val)
{
	if( QuSlotDebug() ) cout << " QpTraceTab::setColBarVoffset  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_BAR_VOFF, 0, val);
}

void QpTraceTab::setColBarTickLen(double val)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setColBarTickLen  " << endl;
  // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_COL_BAR_TICK, 0, val);
}


void QpTraceTab::showColourBar(bool on)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::showColourBar  " << endl;
   	// only tell list view to make changes to selected trace items if this
   	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   	useColBarOn=on;
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SHOW_COL_BAR, 0, 0.0, "", on);
}

void QpTraceTab::setColourOn(bool on)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setColourOn  " << endl;
   	// only tell list view to make changes to selected trace items if this
   	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SET_C_COL, 0, 0.0, "", on);
}

void QpTraceTab::setSizeOn(bool on)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setSizeOn  " << endl;
   	// only tell list view to make changes to selected trace items if this
   	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SET_C_SIZE, 0, 0.0, "", on);
}

void QpTraceTab::setColourTableProfile(int choice)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setColourTable  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SET_COL_TABLE_PROF, choice);
}

void QpTraceTab::setColourTableSession(int choice)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setColourTable  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SET_COL_TABLE_SESS, choice);
}

void QpTraceTab::setColourTableStd(int choice)
{
 	if( QuSlotDebug() ) cout << " QpTraceTab::setColourTable  " << endl;
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SET_COL_TABLE_STD, choice);
}

void QpTraceTab::setLineStyle(int choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setLineStyle \n" );

   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
     if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LINE_STYLE, choice);
}

void QpTraceTab::setLineWidth(double choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setLineWidth \n" );
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LINE_WIDTH, 0, (float)choice);
}

void QpTraceTab::setConnectStyle(int choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setConnectStyle \n" );
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_CONNECT_STYLE, choice);
}

void QpTraceTab::setLineScale(double choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setLineScale \n" );
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LINE_SCALE, 0, (float)choice);
}

void QpTraceTab::setLineColour()
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setLineColour \n" );

	bool changed = false;
	// ask the user for colour choice
	QColor c = QuGetColour(I_Trace->_line_colour, &changed, this);

	if(changed)
	{
		I_Trace->userLineColour = true;
	}
	setLineColour(c);

	// only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
 	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LINE_COLOUR, 0, 0., "", false, c);
}

void QpTraceTab::setLineColour(QColor c)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setLineColour \n" );

	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
	line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
	this->_line_colour_fld->setPalette(line_palette);
	I_Trace->_line_colour = c;
}

void QpTraceTab::setSymbolShape(int choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setSymbolShape \n" );

   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   oldSymbolShape = choice;
    if(this->isVisible() )
	{
		if(choice==20) // custom
		{
			_custom_symbol_fld->setEnabled(true);
		}
		else
		{
			_custom_symbol_fld->setEnabled(false);
		}
		I_Trace->Tree()->setTraceOption(TRACE_SYMBOL_SHAPE, choice);
	}
}

void QpTraceTab::setSymbolSize(double choice)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setSymbolSize \n" );
   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SYMBOL_SIZE, 0, choice);
}

void QpTraceTab::setCustomSymbol(const QString& text)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setCustomSymbol \n" );
	if(this->isVisible() )
	{
		I_Trace->Tree()->setTraceOption(TRACE_CUSTOM_SYMBOL, 0, 0., text);
	}

}

void QpTraceTab::setSymbolColour()
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setSymbolColour \n" );
	
   bool changed = false;
   // ask the user for colour choice
   QColor c = QuGetColour(I_Trace->_symbol_colour, &changed, this);
   if(changed)
   {
		I_Trace->userSymbolColour = true;
   }
        setSymbolColour(c);

   // only tell list view to make changes to selected trace items if this
   // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
   
    if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SYMBOL_COLOUR, 0, 0., "", false, c);
}

void QpTraceTab::setSymbolColour(QColor c)
{
	if ( QuSlotDebug() ) printf( "QpTraceTab::setSymbolColour \n" );
  
	QPalette line_palette = QPalette(QPUI_normal_palette);
	line_palette.setColor(QPalette::Active ,QPalette::ButtonText, c);
	line_palette.setColor(QPalette::Inactive, QPalette::ButtonText, c);
	this->_symbol_colour_fld->setPalette(line_palette);
	I_Trace->_symbol_colour=c;
 
}

QDomElement QpTraceTab::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Trace_Tab");
    
    xml.appendChild(createTextXMLNode(doc, "Connect_Style", _connect_style_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Line_Style", _line_style_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Line_Width", _line_width_fld->value()));
	xml.appendChild(createTextXMLNode(doc, "Line_Scale", _line_scale_fld->value()));

    xml.appendChild(createTextXMLNode(doc, "Symbol_Shape", _symbol_shape_fld->currentIndex()));
    xml.appendChild(createTextXMLNode(doc, "Symbol_Size", _symbol_size_fld->value()));
	xml.appendChild(createTextXMLNode(doc, "Custom_Symbol", _custom_symbol_fld->text()));
	xml.appendChild(createTextXMLNode(doc, "Symbol_variable", _c_data_fld->text()));
    
	xml.appendChild(createTextXMLNode(doc, "Apply_To_Size", _size_on->isChecked()));
	xml.appendChild(createTextXMLNode(doc, "Apply_To_Colour", _colour_on->isChecked()));
	xml.appendChild(createTextXMLNode(doc, "Auto_Scale", c_range_auto->isChecked()));
	xml.appendChild(createTextXMLNode(doc, "Log_Scale", c_range_log->isChecked()));
	xml.appendChild(createTextXMLNode(doc, "Show_Colour_Bar", _show_colour_bar->isChecked()));
	xml.appendChild(createTextXMLNode(doc, "Col_Min", _c_min_fld->text()));
	xml.appendChild(createTextXMLNode(doc, "Col_Max", _c_max_fld->text()));
	xml.appendChild(createTextXMLNode(doc, "Col_Label", colLabSlot->text()));

    ColourTable colTable=colourTable->getCurrentTable();

    int _colour_table_type;
    int _colour_table_id;
    if(stdPalette.contains(colTable))
    {
        _colour_table_type=0;
        _colour_table_id=stdPalette.indexOf(colTable);
    }
    else
    {
        _colour_table_type=1;
        _colour_table_id=savedPalette.indexOf(colTable);
    }
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_Type", _colour_table_type));
    xml.appendChild(createTextXMLNode(doc, "Colour_Table_ID", _colour_table_id));

    xml.appendChild(createTextXMLNode(doc, "Exp_Offset_X", _exp_label_offset_x->value()));
    xml.appendChild(createTextXMLNode(doc, "Exp_Offset_Y", _exp_label_offset_y->value()));

    xml.appendChild(createTextXMLNode(doc, "Col_Bar_W", _colour_bar_width->value()));
    xml.appendChild(createTextXMLNode(doc, "Col_Bar_H", _colour_bar_height->value()));
    xml.appendChild(createTextXMLNode(doc, "Col_Bar_Disp", _colour_bar_disp->value()));
    xml.appendChild(createTextXMLNode(doc, "Col_Bar_Voff", _colour_bar_voffset->value()));
    xml.appendChild(createTextXMLNode(doc, "Col_Bar_Tick", _colourscale_ticklength->value()));


    return xml;
}

void QpTraceTab::fromXMLElement(QDomElement& el)
{
    QString s;
    if(getXMLNodeText(el, "Connect_Style", s))
    {
        _connect_style_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Line_Style", s))
    {
        _line_style_fld->setCurrentIndex(s.toInt());
    }
    if(getXMLNodeText(el, "Line_Width", s))
    {
        _line_width_fld->setValue(s.toDouble());
    }
	if(getXMLNodeText(el, "Line_Scale", s))
    {
        _line_scale_fld->setValue(s.toDouble());
    }
    
    if(getXMLNodeText(el, "Symbol_Shape", s))
    {
        _symbol_shape_fld->setCurrentIndex(s.toInt());
		oldSymbolShape = s.toInt();
		if(s.toInt()==20) // custom
		{
			_custom_symbol_fld->setEnabled(true);
		}
		else
		{
			_custom_symbol_fld->setEnabled(false);
		}
    }
    if(getXMLNodeText(el, "Symbol_Size", s))
    {
        _symbol_size_fld->setValue(s.toDouble());
    }
	if(getXMLNodeText(el, "Custom_Symbol", s))
    {
        _custom_symbol_fld->setText(s);
    }
	if(getXMLNodeText(el, "Symbol_variable", s))
    {
        _c_data_fld->setText(s);
		_c_data_fld->reValidate(true);
    }
	
	if(getXMLNodeText(el, "Apply_To_Size", s))
	{
		_size_on->setChecked(s.toInt());
	}
	if(getXMLNodeText(el, "Apply_To_Colour", s))
	{
		_colour_on->setChecked(s.toInt());
	}
	if(getXMLNodeText(el, "Auto_Scale", s))
	{
		c_range_auto->setChecked(s.toInt());
	}
	if(getXMLNodeText(el, "Log_Scale", s))
	{
		c_range_log->setChecked(s.toInt());
	}
	if(getXMLNodeText(el, "Show_Colour_Bar", s))
	{
		_show_colour_bar->setChecked(s.toInt());
	}
	if(getXMLNodeText(el, "Col_Min", s))
    {
        _c_min_fld->setText(s);
    }
	if(getXMLNodeText(el, "Col_Max", s))
    {
        _c_max_fld->setText(s);
    }
	if(getXMLNodeText(el, "Col_Label", s))
    {
        colLabSlot->setText(s);
    }
    int _colour_table_type(0);
    int _colour_table(0);
    if(getXMLNodeText(el, "Colour_Table_ID", s))
    {
        _colour_table=s.toInt();
    }
    if(getXMLNodeText(el, "Colour_Table_Type", s))
    {
        _colour_table_type=(s.toInt());
    }
    if(_colour_table_type!=0)
    {
        colourTable->setSessionPalette(_colour_table+importedPaletteOffset);
    }
    else
    {
        colourTable->setStdPalette(_colour_table);
    }

    if(getXMLNodeText(el, "Exp_Offset_X", s))
    {
        _exp_label_offset_x->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Exp_Offset_Y", s))
    {
        _exp_label_offset_y->setValue(s.toDouble());
    }

    if(getXMLNodeText(el, "Col_Bar_W", s))
    {
        _colour_bar_width->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Col_Bar_H", s))
    {
        _colour_bar_height->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Col_Bar_Disp", s))
    {
        _colour_bar_disp->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Col_Bar_Voff", s))
    {
        _colour_bar_voffset->setValue(s.toDouble());
    }
    if(getXMLNodeText(el, "Col_Bar_Tick", s))
    {
        _colourscale_ticklength->setValue(s.toDouble());
    }

}

// QpDataTab class
//------------------


QpDataTab::QpDataTab(QTabWidget *tabWidget,  const QString& name, PlotItem * _plot, QpItemType plotType):
		QpTab(tabWidget, name, QP_DATA_TAB)
{
	// Constructor for the Data Tab for the Trace Options
  
	// keep track of item which owns this tab
	I_Plot = _plot;
	I_PlotType = plotType;

	create();

    
	if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
	{
		connect(_y_data_fld, SIGNAL(edited_OK()), this, SLOT(setYData()));
		connect(_y_data_fld, SIGNAL(hadDropEvent()), this, SLOT(propagateLink()));
		if(I_PlotType == QP_REG_LINE) {
			connect(_mask_data_fld, SIGNAL(edited_OK()), this, SLOT(resetLinReg())); // refresh
			connect(_mask_data_fld, SIGNAL(slotEmpty()), this, SLOT(resetLinReg())); // refresh
		}
		else {
			connect(_mask_data_fld, SIGNAL(edited_OK()), this, SLOT(setTIData())); // refresh
			connect(_mask_data_fld, SIGNAL(slotEmpty()), this, SLOT(setTIData())); // refresh
		}
		
        _gap_detection_fld->setChecked(QSAS_profiles->current()->gapToleranceType >= 0);
        _gap_threshold_fld->setValue(QSAS_profiles->current()->gapTolerance);
        _gap_threshold_type_fld->setCurrentIndex((QSAS_profiles->current()->gapToleranceType>= 0 ?QSAS_profiles->current()->gapToleranceType : 1));
		connect(_x_data_fld, SIGNAL(edited_OK()), this, SLOT(setXData()));
		connect(timeAsSec_toggle, SIGNAL(toggled(bool)), this, SLOT(useTimeAsSec(bool)) );
		connect(_show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend()) );
		connect( _legend_text_fld, SIGNAL(textChanged(const QString &)) ,this, SLOT(setLegendText(const QString &)) );
		connect(_auto_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLegend(bool)) );
        connect(_gap_detection_fld, SIGNAL(toggled(bool)), this, SLOT(gap_detection_toggled(bool)));
        connect(_gap_threshold_fld, SIGNAL(valueChanged(double)), this, SLOT(gap_threshold_changed(double)));
        connect(_gap_threshold_type_fld, SIGNAL(activated(int)), this, SLOT(gap_threshold_type_changed(int)));
	}
	else if(I_PlotType == QP_SPECTROGRAM)
	{
//		connect(_z_data_fld, SIGNAL(edited_OK()), this, SLOT(setZData())); // now called from setMetaMenu()
		connect(_mask_data_fld, SIGNAL(edited_OK()), this, SLOT(setTI_ZData())); // refresh
		connect(_mask_data_fld, SIGNAL(slotEmpty()), this, SLOT(setTI_ZData())); // refresh
		connect(_XYtoggle, SIGNAL(clicked()), this, SLOT(setXYswap()));
		connect(_y_start_data_fld, SIGNAL(edited_OK()), this, SLOT(setYStartData()));
		connect(_y_start_data_fld->lineBox, SIGNAL(textEdited(const QString &)), this, SLOT(checkYStartData(const QString &)));
	//	connect(_y_end_data_fld, SIGNAL(edited_OK()), this, SLOT(setYEndData()));
		connect(_x_start_data_fld, SIGNAL(edited_OK()), this, SLOT(setXStartData()));
//		connect(_x_end_data_fld, SIGNAL(edited_OK()), this, SLOT(setXEndData()));
		connect(timeAsSec_toggle, SIGNAL(toggled(bool)), this, SLOT(useTimeAsSec(bool)) );
		connect(gap_toggle, SIGNAL(toggled(bool)), this, SLOT(gap_toggle_changed(bool)));
		connect(interlace_toggle, SIGNAL(toggled(bool)), this, SLOT(interlace_toggle_changed(bool )));
		connect(minCellWidth, SIGNAL(valueChanged(double)), this, SLOT(minCellWidth_changed(double )));
	}
}

QpDataTab::QpDataTab(QpDataTab* tab, PlotItem* _plot):
		QpTab((QTabWidget*)(tab->parent()), "DataTab", tab->TabType)
{
	I_Plot = _plot;
	I_PlotType = tab->I_PlotType;

	create();
    
	if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
	{
		_auto_legend_fld->setChecked(tab->_auto_legend_fld->isChecked());
		_legend_text_fld->setText(tab->_legend_text_fld->text());
		_show_legend_fld->setChecked(tab->_show_legend_fld->isChecked());
		_x_data_fld->setText(tab->_x_data_fld->text());
		_y_data_fld->setText(tab->_y_data_fld->text());

		connect(_y_data_fld, SIGNAL(edited_OK()), this, SLOT(setYData()));
		connect(_y_data_fld, SIGNAL(hadDropEvent()), this, SLOT(propagateLink()));
		connect(_x_data_fld, SIGNAL(edited_OK()), this, SLOT(setXData()));
		connect(_mask_data_fld, SIGNAL(edited_OK()), this, SLOT(setTIData())); // refresh

        _gap_detection_fld->setChecked(tab->_gap_detection_fld->isChecked());
        _gap_threshold_fld->setValue(tab->_gap_threshold_fld->value());
        _gap_threshold_type_fld->setCurrentIndex(tab->_gap_threshold_type_fld->currentIndex());
        _gap_detection_fld->setVisible(tab->_gap_detection_fld->isVisible());
        _gap_params_container->setVisible(tab->_gap_params_container->isVisible());

		connect(timeAsSec_toggle, SIGNAL(toggled(bool)), this, SLOT(useTimeAsSec(bool)) );
		connect(_show_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setShowLegend()) );
		connect( _legend_text_fld, SIGNAL(textChanged(const QString &)) ,this, SLOT(setLegendText(const QString &)) );
		connect(_auto_legend_fld, SIGNAL(toggled(bool)), this, SLOT(setAutoLegend(bool)) );
        connect(_gap_detection_fld, SIGNAL(toggled(bool)), this, SLOT(gap_detection_toggled(bool)));
        connect(_gap_threshold_fld, SIGNAL(valueChanged(double)), this, SLOT(gap_threshold_changed(double)));
        connect(_gap_threshold_type_fld, SIGNAL(activated(int)), this, SLOT(gap_threshold_type_changed(int)));
	}
	else if(I_PlotType == QP_SPECTROGRAM)
	{
		_z_data_fld->setText(tab->_z_data_fld->text());
		_y_start_data_fld->setText(tab->_y_start_data_fld->text());
		_x_start_data_fld->setText(tab->_x_start_data_fld->text());
		_XYtoggle->setChecked(tab->_XYtoggle->isChecked());
		
//		connect(_z_data_fld, SIGNAL(edited_OK()), this, SLOT(setZData())); // now called from setMetaMenu()
		connect(_mask_data_fld, SIGNAL(edited_OK()), this, SLOT(setTI_ZData())); // refresh
		connect(_XYtoggle, SIGNAL(clicked()), this, SLOT(setXYswap()));
		connect(_y_start_data_fld, SIGNAL(edited_OK()), this, SLOT(setYStartData()));
		connect(_x_start_data_fld, SIGNAL(edited_OK()), this, SLOT(setXStartData()));
		connect(timeAsSec_toggle, SIGNAL(toggled(bool)), this, SLOT(useTimeAsSec(bool)) );
		connect(gap_toggle, SIGNAL(toggled(bool)), this, SLOT(gap_toggle_changed(bool)));
		connect(interlace_toggle, SIGNAL(toggled(bool)), this, SLOT(interlace_toggle_changed(bool)));
		connect(minCellWidth, SIGNAL(valueChanged(double)), this, SLOT(minCellWidth_changed(double )));
	}
}

void QpDataTab::create()
{
	old_BaseName = "";
	old_BaseXName = "";
	old_XComp = "";
	old_YComp = "";
		
	_y_data_fld=NULL;
	_x_data_fld=NULL;
	_z_data_fld=NULL;
	_mask_data_fld=NULL;
	
	QVBoxLayout *data_top_layout = new QVBoxLayout(this);
	data_top_layout->setSpacing(4);

	QGroupBox *data_plot_groupbox = new QGroupBox(this);
	
	QGridLayout *data_plot_layout=new QGridLayout;
	data_plot_layout->setSpacing(QSAS_profiles->current()->GUImargins);
	data_plot_layout->setMargin(QSAS_profiles->current()->GUImargins);

	if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
	{
		_z_data_fld = NULL;
		_y_label_fld=new QLabel("Y: ");
		data_plot_layout->addWidget(_y_label_fld,1,0,Qt::AlignRight);
		_y_data_fld=new QguWLinput(data_plot_groupbox,"y_data");
		_y_data_fld->setMinimumWidth(500);
		data_plot_layout->addWidget(_y_data_fld,1,1,1,2,Qt::AlignLeft);

		_x_label_fld = new QLabel("X: ");
		data_plot_layout->addWidget(_x_label_fld,3,0,Qt::AlignRight);
		_x_data_fld = new QguWLinput(data_plot_groupbox,"x_data");
		_x_data_fld->setMinimumWidth(500);
		data_plot_layout->addWidget(_x_data_fld,3,1,1,2,Qt::AlignLeft);
		
		timeAsSec_toggle = new QCheckBox("Use time as sec from T1");
		data_plot_layout->addWidget(timeAsSec_toggle,5,1,1,2,Qt::AlignLeft);
		timeAsSec_toggle->setChecked(false);

		if(I_Plot->frameType() == QP_TS) timeAsSec_toggle->setVisible(false);

        QHBoxLayout * horizontalLayout=new QHBoxLayout;
        _gap_detection_fld=new QCheckBox("Gap detection");
        horizontalLayout->addWidget(_gap_detection_fld);
        
        _gap_params_container=new QWidget;
        QHBoxLayout * gapLayout=new QHBoxLayout;
        _gap_params_container->setLayout(gapLayout);
        
        gapLayout->addWidget(new QLabel("Threshold:", _gap_params_container), Qt::AlignRight);
        _gap_threshold_fld=new FloatSpinBox(0.1, 100000.,1, .1, _gap_params_container, "");
        gapLayout->addWidget(_gap_threshold_fld, Qt::AlignLeft);
        _gap_threshold_type_fld=new QComboBox;
        _gap_threshold_type_fld->addItem("seconds");
        _gap_threshold_type_fld->addItem("x Median Spacing (         s)");
        _gap_threshold_type_fld->setSizeAdjustPolicy(QComboBox::AdjustToContents);
        gapLayout->addWidget(_gap_threshold_type_fld, Qt::AlignLeft);
        gapLayout->addStretch(10);
        
        horizontalLayout->addWidget(_gap_params_container);
        data_plot_layout->addLayout(horizontalLayout, 6, 0, 1, 3);

        if(I_Plot->frameType() == QP_XY) {
            _gap_detection_fld->setVisible(false);
            _gap_params_container->setVisible(false);
        }

		// add interval mask to all traces
		_mask_label_fld=new QLabel("Event:");
		data_plot_layout->addWidget(_mask_label_fld,8,0,Qt::AlignRight);
		_mask_data_fld=new QguWLinput(data_plot_groupbox,"mask_data");
		_mask_data_fld->setMinimumWidth(500);
		_mask_data_fld->setToolTip("Optional Time Interval Mask");
		data_plot_layout->addWidget(_mask_data_fld,8,1,1,2,Qt::AlignLeft);
        
		data_plot_layout->setRowStretch(0,0);
		data_plot_layout->setRowStretch(1,1);
		data_plot_layout->setRowStretch(2,0);
		data_plot_layout->setRowStretch(3,1);
		data_plot_layout->setRowStretch(4,0);

		data_plot_layout->setColumnStretch(0,0);
		data_plot_layout->setColumnStretch(1,0);
		data_plot_layout->setColumnStretch(2,0);
		data_plot_layout->setColumnStretch(3,0);
		data_plot_layout->setColumnStretch(4,10);
		
		data_plot_groupbox->setLayout(data_plot_layout);
        
		data_top_layout->addWidget(data_plot_groupbox);
		
		QGroupBox *data_legend_groupbox=new QGroupBox(this);

		data_legend_groupbox->setTitle("Legend");

		QGridLayout *data_legend_layout=new QGridLayout;
		data_legend_layout->setSpacing(QSAS_profiles->current()->GUImargins);
		data_legend_layout->setMargin(QSAS_profiles->current()->GUImargins);

		_show_legend_fld = new QCheckBox("Include in legend");
		_show_legend_fld->setChecked(true);
		data_legend_layout->addWidget(_show_legend_fld,1,0,Qt::AlignCenter);

		QLabel *legend_label_fld=new QLabel("Text:");
		data_legend_layout->addWidget(legend_label_fld,1,1,Qt::AlignRight);
		_legend_text_fld=new QLineEdit;
		_legend_text_fld->setMinimumWidth(350);
		data_legend_layout->addWidget(_legend_text_fld,1,2,1,3,Qt::AlignLeft);
	
		setSlotColour(false);
		_legend_text_fld->setPalette(GUIpalette);
		_auto_legend_fld = new QCheckBox("Auto-legend");
		_auto_legend_fld->setChecked(true);
		data_legend_layout->addWidget(_auto_legend_fld,1,5,Qt::AlignCenter);

		data_legend_layout->setRowStretch(0,0);
		data_legend_layout->setRowStretch(1,1);
		data_legend_layout->setRowStretch(2,0);

		data_legend_groupbox->setLayout(data_legend_layout);

		data_top_layout->addWidget(data_legend_groupbox);

		data_type_groupbox = new QGroupBox(this);
		setDataTypeTitle();
		
		// info box 
		meta_groupbox=new QGroupBox(this);
		meta_groupbox->setTitle("Metadata");
		
		QGridLayout *meta_layout=new QGridLayout;
		meta_layout->setSpacing((int)(QSAS_profiles->current()->GUImargins*2.5));
		meta_layout->setMargin((int)(QSAS_profiles->current()->GUImargins*2.5));

		QPushButton *meta_btn = new QPushButton("Object Info");
		meta_btn->setMinimumWidth(100);
		meta_layout->addWidget(meta_btn,0,0,Qt::AlignLeft);
		meta_btn->setToolTip("Get metadata from data object for info and cut & paste");
		metaMenu = new QMenu();
		meta_btn->setMenu(metaMenu);
		
	
		metaText = new QLineEdit();
		metaText->setMinimumWidth(400);
		meta_layout->addWidget(metaText,0,2,Qt::AlignLeft);

		meta_layout->setColumnStretch(0,0);
		meta_layout->setColumnStretch(1,0);
		meta_layout->setColumnStretch(2,10);
		
		meta_groupbox->setLayout(meta_layout);

		data_top_layout->addWidget(meta_groupbox);

		// data type

		QGridLayout *data_type_layout=new QGridLayout;
		data_type_layout->setSpacing((int)(QSAS_profiles->current()->GUImargins*2.5));
		data_type_layout->setMargin((int)(QSAS_profiles->current()->GUImargins*2.5));

		QPushButton *_type_btn = new QPushButton("Change Plot Type...");
		_type_btn->setMinimumWidth(100);
		data_type_layout->addWidget(_type_btn,0,0,Qt::AlignLeft);

		QPushButton *_frame_type_btn = new QPushButton("Change Frame Type...");
		_frame_type_btn->setMinimumWidth(100);
		data_type_layout->addWidget(_frame_type_btn,0,2,Qt::AlignLeft);

		_frame_type_btn->setToolTip("Sets whether ALL plot items in frame will be TS or XY");	

		data_type_layout->setColumnStretch(0,0);
		data_type_layout->setColumnStretch(1,1);
		data_type_layout->setColumnStretch(2,0);

		data_type_groupbox->setLayout(data_type_layout);
		data_top_layout->addWidget(data_type_groupbox);
		connect(_type_btn, SIGNAL(pressed()), this, SLOT(launchTypeSelector()) );
		connect(_frame_type_btn, SIGNAL(pressed()), this->I_Plot->frame()->xtAxisTab, SLOT(launchFrameTypeSelector()) );
		connect(_y_data_fld, SIGNAL(edited_OK()), this, SLOT(setMetaMenu()) );

	}
	else if(I_PlotType == QP_SPECTROGRAM)
	{
	
		QLabel* z_label_fld  =new QLabel("Z: ");
		data_plot_layout->addWidget(z_label_fld, 1, 0, Qt::AlignRight);
		_z_data_fld = new QguWLinput(data_plot_groupbox, "z_data");
		_z_data_fld->setMinimumWidth(500);
		data_plot_layout->addWidget( _z_data_fld, 1, 1, 1, 3, Qt::AlignLeft);

		_XYtoggle = new QCheckBox("Swap default X and Y axes");
		data_plot_layout->addWidget(_XYtoggle, 7, 2, Qt::AlignLeft);
		_XYtoggle->setChecked(false);
		
		QLabel* y_start_label_fld=new QLabel("Y: ");
		QFont it_font = QFont(qApp->font());
		it_font.setItalic(true);
		y_start_label_fld->setFont(it_font);
		data_plot_layout->addWidget(y_start_label_fld, 3, 0, Qt::AlignRight);
		_y_start_data_fld = new QguWLinput(data_plot_groupbox, "y_start_data");
		_y_start_data_fld->setMinimumWidth(500);
		data_plot_layout->addWidget( _y_start_data_fld, 3, 1, 1, 3, Qt::AlignLeft);

		_x_start_label_fld = new QLabel("X: ");
		_x_start_label_fld->setFont(it_font);
		data_plot_layout->addWidget(_x_start_label_fld, 5, 0, Qt::AlignRight);
		_x_start_data_fld  =new QguWLinput(data_plot_groupbox, "x_start_data");
		_x_start_data_fld->setMinimumWidth(500);
		data_plot_layout->addWidget( _x_start_data_fld, 5, 1, 1, 3, Qt::AlignLeft);

		timeAsSec_toggle = new QCheckBox("Use time as sec from T1");
		data_plot_layout->addWidget(timeAsSec_toggle, 7, 1, 1, 2, Qt::AlignLeft);
		timeAsSec_toggle->setChecked(false);

		gap_toggle = new QCheckBox("Fill t gaps");
		data_plot_layout->addWidget(gap_toggle, 7, 1, Qt::AlignLeft);
		gap_toggle->setChecked(false);
        
		interlace_toggle = new QCheckBox("Interlace spectra");
		data_plot_layout->addWidget(interlace_toggle, 7, 2, Qt::AlignLeft);
		interlace_toggle->setChecked(false);
  
  		QLabel * minCellLab = new QLabel("minimum cell width: ");
		data_plot_layout->addWidget(minCellLab, 8, 1, Qt::AlignRight);
		minCellWidth = new FloatSpinBox(0.0, .01, 4, .0001, this, "minCellWidth");
		minCellWidth->setValue(0);
		minCellWidth->setToolTip("minimum width of cells as fraction of X axis");
		data_plot_layout->addWidget(minCellWidth, 8, 2, Qt::AlignLeft);

      		
		data_plot_layout->setColumnStretch(0,0);
		data_plot_layout->setColumnStretch(1,1);
		data_plot_layout->setColumnStretch(2,1);

		data_plot_layout->setRowStretch(0,0);
		data_plot_layout->setRowStretch(1,1);
		data_plot_layout->setRowStretch(2,0);
		data_plot_layout->setRowStretch(3,1);
		data_plot_layout->setRowStretch(4,0);
		data_plot_layout->setRowStretch(5,1);

		// add interval mask to all traces
		_mask_label_fld=new QLabel("Event:");
		data_plot_layout->addWidget(_mask_label_fld,9,0,Qt::AlignRight);
		_mask_data_fld=new QguWLinput(data_plot_groupbox,"mask_data");
		_mask_data_fld->setMinimumWidth(500);
		_mask_data_fld->setToolTip("Optional Time Interval Mask");
		data_plot_layout->addWidget(_mask_data_fld,9,1,1,2,Qt::AlignLeft);


		data_plot_groupbox->setLayout(data_plot_layout);

		data_top_layout->addWidget(data_plot_groupbox);

		data_type_groupbox=new QGroupBox(this);
		setDataTypeTitle();

		// info box 
		meta_groupbox=new QGroupBox(this);
		meta_groupbox->setTitle("Metadata");
		
		QGridLayout *meta_layout=new QGridLayout;
		meta_layout->setSpacing((int)(QSAS_profiles->current()->GUImargins*2.5));
		meta_layout->setMargin((int)(QSAS_profiles->current()->GUImargins*2.5));

		QPushButton *meta_btn = new QPushButton("Object Info");
		meta_btn->setMinimumWidth(100);
		meta_layout->addWidget(meta_btn,0,0,Qt::AlignLeft);
		meta_btn->setToolTip("Get metadata from data object for info and cut & paste");
		metaMenu = new QMenu();
		meta_btn->setMenu(metaMenu);

		metaText = new QLineEdit();
		metaText->setMinimumWidth(400);
		meta_layout->addWidget(metaText,0,2,Qt::AlignLeft);

		meta_layout->setColumnStretch(0,0);
		meta_layout->setColumnStretch(1,0);
		meta_layout->setColumnStretch(2,10);
		
		meta_groupbox->setLayout(meta_layout);

		data_top_layout->addWidget(meta_groupbox);

		// data type
		QGridLayout *data_type_layout=new QGridLayout;
		data_type_layout->setSpacing((int)(QSAS_profiles->current()->GUImargins*2.5));
		data_type_layout->setMargin((int)(QSAS_profiles->current()->GUImargins*2.5));

		QPushButton *_type_btn = new QPushButton("Change Plot Type...");
		_type_btn->setMinimumWidth(100);
		data_type_layout->addWidget(_type_btn,0,0,Qt::AlignLeft);
	
		QPushButton *_frame_type_btn = new QPushButton("Change Frame Type...");
		_frame_type_btn->setMinimumWidth(100);
		data_type_layout->addWidget(_frame_type_btn,0,2,Qt::AlignLeft);

		_frame_type_btn->setToolTip("Sets whether ALL plot items in frame will be TS or XY");

		data_type_layout->setColumnStretch(0,0);
		data_type_layout->setColumnStretch(1,1);
		data_type_layout->setColumnStretch(2,0);
		
		data_type_groupbox->setLayout(data_type_layout);

		data_top_layout->addWidget(data_type_groupbox);
		
		connect(_type_btn, SIGNAL(pressed()), this, SLOT(launchTypeSelector()) );
		connect(_frame_type_btn, SIGNAL(pressed()), this->I_Plot->frame()->xtAxisTab, SLOT(launchFrameTypeSelector()) );
		connect(_z_data_fld, SIGNAL(edited_OK()), this, SLOT(setMetaMenu()) );

	}
		
	_mask_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));


	QString PlotTitle = "Plot Object";

    if(I_PlotType == QP_SPECTROGRAM)
    {
        PlotTitle = "Spectrogram Data";
        if(I_Plot->frameType() == QP_TS)
        {
            _z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_ANY, QS_TT));

            _y_start_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
            _y_start_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
            
            _x_start_label_fld->setVisible(false);
            _x_start_data_fld->setVisible(false);
			
            _XYtoggle->setVisible(false);
            gap_toggle->setVisible(true);
            interlace_toggle->setVisible(true);
            timeAsSec_toggle->setVisible(false);
        }
        else
        {
            _z_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_ANY, QS_ANY));
            _z_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_NOT_SEQ, QS_ANY));
        
            _y_start_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
            _y_start_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));

            _x_start_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
            _x_start_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));

            
            _x_start_label_fld->setVisible(true);
            _x_start_data_fld->setVisible(true);

            _XYtoggle->setVisible(true);
            gap_toggle->setVisible(false);
            interlace_toggle->setVisible(false);
            timeAsSec_toggle->setVisible(true);
        	_mask_label_fld->setVisible(false);
        	_mask_data_fld->setVisible(false);
        }
    }
    else
    {
        if(I_Plot->frameType() == QP_TS)
        {
    
            _x_label_fld->setVisible(false);
            _x_data_fld->setVisible(false);
            timeAsSec_toggle->setVisible(false);
 
            if(I_PlotType == QP_CONST_XT)
            {
            // set to const t trace
           		_gap_detection_fld->setVisible(false);
            	_gap_params_container->setVisible(false);
                PlotTitle = "Const  T  Data";
                _y_label_fld->setText("Time Constant: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
                _show_legend_fld->setChecked(false);
            }
            else if(I_PlotType == QP_CONST_Y)
            {
           		_gap_detection_fld->setVisible(false);
            	_gap_params_container->setVisible(false);
                PlotTitle = "Const  Y  Data";
                _y_label_fld->setText("Y Constant: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
                _show_legend_fld->setChecked(false);
            }
            else if(I_PlotType == QP_REG_LINE)
            {
           		_gap_detection_fld->setVisible(false);
            	_gap_params_container->setVisible(false);
                PlotTitle = "Regression Line Data";
                _y_label_fld->setText("Y: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_TT));
                _show_legend_fld->setChecked(false);
            }
            else
            {
           		_gap_detection_fld->setVisible(true);
            	_gap_params_container->setVisible(true);
                PlotTitle = "Trace Data";
                _y_label_fld->setText("Y: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_TT));
                _show_legend_fld->setChecked(true);
            }
        }
        else
        {
            timeAsSec_toggle->setVisible(true);
            _gap_detection_fld->setVisible(false);
            _gap_params_container->setVisible(false);
			
            if(I_PlotType == QP_CONST_XT)
            {
                PlotTitle = "Const  X  Data";
                _y_label_fld->setText("X Constant: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));

                _x_label_fld->hide();
                _x_data_fld->hide();
                timeAsSec_toggle->hide();
                _show_legend_fld->setChecked(false);
            }
            else if(I_PlotType == QP_CONST_Y)
            {
                PlotTitle = "Const  Y  Data";
                _y_label_fld->setText("Y Constant: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
            
                _x_label_fld->hide();
                _x_data_fld->hide();
                timeAsSec_toggle->hide();
                _show_legend_fld->setChecked(false);
            }
            else if(I_PlotType == QP_REG_LINE)
            {
                PlotTitle = "Regression Line Data";
                _y_label_fld->setText("Y: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
                _y_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
            
                _x_label_fld->show();
                _x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
                _x_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
                _x_data_fld->show();
                timeAsSec_toggle->show();
                _show_legend_fld->setChecked(false);
            }
            else
            {
                PlotTitle = "Trace Data";
                _y_label_fld->setText("Y: ");
                _y_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
                _y_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));

                _x_label_fld->show();
                _x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
                _x_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
                _x_data_fld->show();
                timeAsSec_toggle->show();
                _show_legend_fld->setChecked(false);
            }
        }
    }
    
    data_plot_groupbox->setTitle(PlotTitle);
	
	data_top_layout->addSpacing(4);
	data_top_layout->addStretch(10);

}

//---------------------------------------------

QpDataTab::~QpDataTab(){
// QT deletes everything
}


//---------------------------------------------

void QpDataTab::setMetaMenu()
{
	if ( QuSlotDebug() ) printf( "QpDataTab::setMetaMenu \n" );

    DvObject_var dobj; 
	QString dobj_text;
	
	if(I_PlotType == QP_SPECTROGRAM) {
		dobj =  _z_data_fld->getObject();
		dobj_text = _z_data_fld->text();
	}
	else {
		dobj = _y_data_fld->getObject();
		dobj_text = _y_data_fld->text();
	}

    if(dobj.is_nil()) return;
	
	metaMenu->clear();
	metaActions.clear();
	
	// list xrefs
	DvNode * next = dobj->first_xref();
	while(next){
		QString name = QString(next->name().c_str());
		QAction *nextAction = metaMenu->addAction(name, this, SLOT(setMetaText()) );
		nextAction->setData(name);
		metaActions.push_back(nextAction);
		
		next=next->next;
	}

	// In special case of time slice add extra action to show time
	// dobj is already subset over selected range
	DvObject_var d0 = dobj->getTimeTags();
	if(d0.is_ok() ) {
		QString valStr;
		if( d0->seqSize() > 1 ){
			valStr = QString(d0->asTime(0).getISOstring(0).c_str());
            valStr += QString("/");
            valStr += QString(d0->asTime(d0->seqSize()-1).getISOstring(0).c_str());
		}
		else{
			valStr = QString(d0->asTime(0).getISOstring(0).c_str());
		}
				
		metaText->setText(valStr);
		meta_groupbox->setTitle("Time Range");
		
		QAction *nextAction = metaMenu->addAction("TIME SLICE", this, SLOT(setMetaText()) );
		nextAction->setData(valStr);
		metaActions.push_back(nextAction);
	}
	// call from here rather than using connect to ensure order of setMetaMenu() and setZData()
	if(I_PlotType == QP_SPECTROGRAM)  setZData();
}

//---------------------------------------------

void QpDataTab::setMetaText()
{
	if ( QuSlotDebug() ) printf( "QpDataTab::setMetaText \n" );

	QAction *action = qobject_cast<QAction *>(sender());
	if (action){
		DvObject_var dobj;
		if(I_PlotType == QP_SPECTROGRAM) dobj = _z_data_fld->getObject();
		else dobj = _y_data_fld->getObject();
		
		if(dobj.is_nil()) return;
		
		QString str = action->data().toString();
		if( dobj->xref_exists( str.toLatin1().data()) ){
			QString valStr = dobj->getXrefText(str.toLatin1().data()).c_str();
			metaText->setText(valStr);
			meta_groupbox->setTitle(action->data().toString() );
		}
		else{
			//QString valStr = dobj->getXrefText(str.toLatin1().data()).c_str();
			metaText->setText(action->data().toString());
			meta_groupbox->setTitle("Time Range");
		}
	}
}


//---------------------------------------------

void QpDataTab::setDataTypeTitle()
{

	string thisType;
	if (I_PlotType == QP_TRACE)				thisType = "    Plot Type:   Trace            ";
	else if (I_PlotType == QP_CONST_XT && I_Plot->frameType() == QP_TS)		thisType = "    Plot Type:   Const T Trace  ";
	else if (I_PlotType == QP_CONST_XT && I_Plot->frameType() == QP_XY)		thisType = "    Plot Type:   Const X Trace  ";
	else if (I_PlotType == QP_CONST_Y)		thisType = "    Plot Type:   Const Y Trace  ";
	else if (I_PlotType == QP_REG_LINE)		thisType = "    Plot Type:   Linear Fit Trace  ";
	else if (I_PlotType == QP_SPECTROGRAM)	thisType = "    Plot Type:   Spectrogram     ";
	else;
		
	if(I_Plot->frameType() == QP_TS) 
	{
		thisType += "                    Frame Type:  Time Series";
	}
	else 
	{
		thisType += "                    Frame Type:  X-Y";
	}
	data_type_groupbox->setTitle(thisType.c_str());

}
// --------------------------------------------

void QpDataTab::useTimeAsSec(bool setOn)
{
	if( QuSlotDebug() ) cout << " QpDataTab::useTimeAsSec  " << setOn << endl;

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	SpectrumItem *I_Spectrum = dynamic_cast<SpectrumItem*>(I_Plot);
	if(I_Trace)
	{
		if(setOn) {
			_x_label_fld->setText("T1: ");
			_x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
			if( !(_y_data_fld->text().isEmpty()) && _x_data_fld->text().isEmpty() )
			{
				_x_data_fld->setText(baseObjectText(_y_data_fld->text().toLatin1().data()).c_str());
				_x_data_fld->reValidate();
				_x_data_fld->setTimeDropdownDefault();
			}
		}
		else
		{
			_x_label_fld->setText("X: ");
			_x_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
			_x_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
			
			_x_data_fld->setText("");
			_x_data_fld->reValidate();

		}
		
		// only tell list view to make changes to selected trace items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

		if(this->isVisible() ) 
		{
			// propagate to selected items if not hard linked
			if(I_Trace->linkedPlots.size() == 0) 
			{
 				I_Plot->Tree()->setPlotOption(USE_TIME_AS_SEC, setOn);
			}
		}
	
		// pass on to linked items	
		if(!(I_Trace->Tree()->isLink))
		{
			I_Trace->Tree()->isLink = true; // stops infinite recursion;
			foreach(PlotItem* item, I_Trace->linkedPlots)
			{
				if(item==I_Trace) continue;
				if(item->DataTab->timeAsSec_toggle->isChecked() != setOn) item->DataTab->timeAsSec_toggle->toggle();
			}

			I_Trace->Tree()->isLink = false;
		}

		I_Trace->updateAxes();
		DvString legend_text = I_Trace->getLegendText();
		_legend_text_fld->setText(legend_text.c_str());
		
	}
	if(I_Spectrum)
	{
		if(setOn) {
			_x_start_label_fld->setText("T1: ");
			_x_start_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));

		//	_x_end_label_fld->setVisible(false);
		//	_x_end_data_fld->setVisible(false);
			if( !(_z_data_fld->text().isEmpty()) )
			{
				_x_start_data_fld->setTextKeepComp(baseObjectText(_z_data_fld->text().toLatin1().data()).c_str());
			}
		//	_x_start_data_fld->reValidate();
		}
		else
		{
			_x_start_label_fld->setText("X: ");
			_x_start_data_fld->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
			_x_start_data_fld->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_NOT_SEQ, QS_ANY));
			
		//	_x_end_label_fld->setVisible(false);
		//	_x_end_data_fld->setVisible(false);
			_x_start_data_fld->setText("");
			_x_start_data_fld->reValidate();
		}
		
		// only tell list view to make changes to selected spectra items if this
		// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

		if(this->isVisible() ) 
		{
			// propagate to selected items if not hard linked
			if(I_Spectrum->linkedPlots.size() == 0) 
			{
 				I_Plot->Tree()->setPlotOption(USE_TIME_AS_SEC, setOn);
			}
		}
	
		// pass on to linked items	
 		if(!(I_Spectrum->Tree()->isLink))
		{
			I_Spectrum->Tree()->isLink = true; // stops infinite recursion;
			foreach(PlotItem* item, I_Spectrum->linkedPlots)
			{
				if(item==I_Spectrum) continue;
				if(item->DataTab->timeAsSec_toggle->isChecked() != setOn) item->DataTab->timeAsSec_toggle->toggle();
			}
// 			list <PlotItem*>::iterator it;
// 			for( it=I_Spectrum->linkedPlots.begin(); it != I_Spectrum->linkedPlots.end(); it++)
// 			{
// 				if( *it == I_Spectrum) continue;
// 				if((*it)->DataTab->timeAsSec_toggle->isChecked() != setOn) (*it)->DataTab->timeAsSec_toggle->toggle();
// 			}
			I_Spectrum->Tree()->isLink = false;
		}
		
		I_Spectrum->updateAxes();

	}


}
// --------------------------------------------

void QpDataTab::launchTypeSelector()
{
	// slot invoked by Change Type button
	QMenu * typePopup = new QMenu;
	typePopup->addAction("Trace");
	typePopup->addAction("Spectrogram");
	typePopup->addAction("Const X or T Trace");
	typePopup->addAction("Const Y Trace");
	
	connect(typePopup, SIGNAL(triggered(QAction*)), I_Plot->Tree(), SLOT(setPlotType(QAction*)) );
	I_Plot->Tree()->itemFromTab = I_Plot;
	
	typePopup->exec(QCursor::pos());
	
}

void QpDataTab::setLegendText(const QString &legend_text)
{
  	if( QuSlotDebug() ) cout << " QpDataTab::setLegendText = " << legend_text.toLatin1().data() << endl;

	if(_legend_text_fld->hasFocus()) _auto_legend_fld->setChecked(false);

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	if( !I_Trace) return;
	
	// only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

 	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_LEGEND_TEXT, 0, 0., legend_text);
}

void QpDataTab::setAutoLegend(bool autoOn)
{
	if ( QuSlotDebug() ) printf( "QpDataTab::setAutoLegend  %d\n", autoOn );
		
	if(autoOn)
	{
		_legend_text_fld->clearFocus();
		TraceItem * traceItem = dynamic_cast<TraceItem*>(I_Plot);
		if(traceItem){
			DvString legend_text = traceItem->getLegendText();
			_legend_text_fld->setText(legend_text.c_str());
		}
	}
	
	setSlotColour(!autoOn);
	_legend_text_fld->setPalette(GUIpalette);
	
	// only tell list view to make changes to selected frame items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)	
 	if(this->isVisible() ) I_Plot->Tree()->setTraceOption(LEGEND_AUTO_TEXT, 0, 0.0, "", autoOn);
	

} 


void QpDataTab::setShowLegend()
{
	if( QuSlotDebug() ) cout << " QpDataTab::setShowLegend = " <<  endl;

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	if( !I_Trace) return;
	I_Plot->panel()->LegendTab->updateNumTraces();
	 
  	// only tell list view to make changes to selected trace items if this
	// is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
	int showLegend = (int) this->_show_legend_fld->isChecked();
 	if(this->isVisible() ) I_Trace->Tree()->setTraceOption(TRACE_SHOW_LEGEND, showLegend);
}

void QpDataTab::setXData()
{
	if( QuSlotDebug() ) cout << " QpDataTab::setXData  " << endl;

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	if( !I_Trace) return;


	// is change to base object or component...
	DvString BaseName = baseObjectText( _x_data_fld->text().toLatin1().data() ); // new value
	DvString CompText = compText( _x_data_fld->text().toLatin1().data() ); // new value
	
	bool compOnly = false;
	if(old_BaseXName == BaseName && old_XComp != CompText) compOnly = true;
	
	old_BaseXName = BaseName;
	old_XComp = CompText;

	if(this->isVisible() ) {

		// propagate to selected items if not hard linked
		if(I_Trace->linkedPlots.size() == 0) 
		{
 			if(compOnly) I_Trace->Tree()->setTraceOption(TRACE_XCOMP, 0, 0.0, CompText.c_str());
 			else I_Trace->Tree()->setTraceOption(TRACE_XDATA, 0, 0.0, BaseName.c_str());
		}
	}
	
	// pass on the change to linked items: component selection should NOT propagate
	if(!(I_Trace->Tree()->isLink)){
		I_Trace->Tree()->isLink = true; // stops infinite recursion;
		foreach(PlotItem* item, I_Trace->linkedPlots)
		{
			if(item==I_Trace) continue;
			if(compOnly) item->replaceXComponent(CompText.c_str());
			else item->replaceXData(BaseName.c_str());
		}

		I_Trace->Tree()->isLink = false;
	}
		
	I_Trace->updateAxes();

	if(_auto_legend_fld->isChecked())
	{
		DvString legend_text = I_Trace->getLegendText();
		_legend_text_fld->setText(legend_text.c_str());
	}

}

void QpDataTab::resetLinReg()
{
	if( QuSlotDebug() ) cout << " QpDataTab::resetLinReg  " << endl;
	
	setTIData();

	setYData();
	
}

void QpDataTab::setTIData()
{
	if( QuSlotDebug() ) cout << " QpDataTab::setTIData  " << endl;
	// slot called when y data name field changed on widget

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	if( !I_Trace) return;	
	
	
	// is change to base object or component...
	DvString BaseName = baseObjectText( _mask_data_fld->text().toLatin1().data() ); // new value
	DvString CompText = compText( _mask_data_fld->text().toLatin1().data() ); // new value
	bool compOnly = false;
	if(old_BaseTIName == BaseName ) compOnly = true;
	old_BaseTIName = BaseName;

	if(this->isVisible() ) {
		// propagate to selected items 
	
		if(compOnly) I_Trace->Tree()->setTraceOption(TRACE_TICOMP, 0, 0.0, CompText.c_str());
		else  I_Trace->Tree()->setTraceOption(TRACE_TIDATA, 0, 0.0, BaseName.c_str());
	}

	I_Trace->updateAxes(); // also sets _ydo
	
	I_Trace->updateColourBar(); // for X-Y-Z traces
}


void QpDataTab::setYData()
{
	qApp->processEvents(); // ensure link propogation happens first so that axes are correct

	if( QuSlotDebug() ) cout << " QpDataTab::setYData  '" << _y_data_fld->text().toLatin1().data() << "'" << endl;
	
	// slot called when y data name field changed on widget
	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	if( !I_Trace) return;
	
	I_Trace->linFit_var = new DvObject(); // set var ptr to nil to force recalculation
	
	// is change to base object or component...
	DvString BaseName = baseObjectText( _y_data_fld->text().toLatin1().data() ); // new value
    DvString CompText = compText( _y_data_fld->text().toLatin1().data() ); // new value

	DvString compChange = getCompChange(old_YComp.c_str(), CompText.c_str());

	bool compOnly = false;
	if(old_BaseName == BaseName && old_YComp != CompText) compOnly = true;
	old_BaseName = BaseName;
	old_YComp = CompText;

	
	// if item uses data name change it
	if(  !(_y_data_fld->text().isEmpty())) {
		I_Trace->setName(_y_data_fld->text());
	}
	I_Trace->WLcontext = BaseName.c_str();
	if(this->isVisible() ) {
		// propagate to selected items 

 		if(compOnly) I_Trace->Tree()->setTraceOption(TRACE_YCOMP, 0, 0.0, compChange.c_str());
		else I_Trace->Tree()->setTraceOption(TRACE_YDATA, 0, 0.0, BaseName.c_str());

	}
		
	I_Trace->updateAxes(); // also sets _ydo needed for legend
		
	if(_auto_legend_fld->isChecked())
	{
		DvString legend_text = I_Trace->getLegendText();
		_legend_text_fld->setText(legend_text.c_str());
	}
	
	updateGapThreshold();

}


void QpDataTab::propagateLink()
{
	if( QuSlotDebug() ) cout << " QpDataTab::propagateLink  " << endl;

	TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
	SpectrumItem *I_Spectrum = dynamic_cast<SpectrumItem*>(I_Plot);
	if( !I_Trace && !I_Spectrum) return;

	DvString BaseName;
	DvString CompText;
	if(I_Trace){
		BaseName = baseObjectText( _y_data_fld->text().toLatin1().data() ); // new value
		CompText = compText( _y_data_fld->text().toLatin1().data() ); // new value
	}
	else {
		BaseName = baseObjectText( _z_data_fld->text().toLatin1().data() ); // new value
		CompText = compText( _z_data_fld->text().toLatin1().data() ); // new value
	}
	// set default trace if not user set

	if(I_Trace){
		QColor defColour = QColor(0,0,0);
		if(BaseName.contains("_C1_") || BaseName.contains("_dr1_")|| BaseName.contains("_at1_")){
			defColour = I_Trace->C1_colour;
		}
		else if(BaseName.contains("_C2_") || BaseName.contains("_dr2_")|| BaseName.contains("_at2_")){
			defColour = I_Trace->C2_colour;
		}
		else if(BaseName.contains("_C3_") || BaseName.contains("_dr3_")|| BaseName.contains("_at3_")){
			defColour = I_Trace->C3_colour;
		}
		else if(BaseName.contains("_C4_") || BaseName.contains("_dr4_")|| BaseName.contains("_at4_")){
			defColour = I_Trace->C4_colour;
		}
	
		if(!I_Trace->userLineColour ){
			I_Trace->TraceTab->setLineColour(defColour);
		}
		if(!I_Trace->userSymbolColour ){
			I_Trace->TraceTab->setSymbolColour(defColour);
		}
	}
	
	// pass on the drop to linked items	: component selection should NOT propagate
	if(!(I_Plot->Tree()->isLink) )
	{
		bool discard = false; // used to find onceOnly linkage
		I_Plot->Tree()->isLink = true; // stops infinite recursion;
		foreach(PlotItem* item, I_Plot->linkedPlots)
		{
			if(item==I_Plot) continue;
			item->replaceData(BaseName.c_str());

			if(item->getLinkText().contains("**"))
			{
				item->setLinkText("");
				item->linkedPlots.clear();
				discard=true;
			}
		}

		if(discard) 
		{
			// also clear linkage on drop target
			I_Plot->setLinkText("");
			I_Plot->linkedPlots.clear();
		}
		I_Plot->Tree()->isLink = false;
	}

}

void QpDataTab::gap_detection_toggled(bool b)
{
    if( QuSlotDebug() ) cout << " QpDataTab::gap_detection_toggled " << b << endl;

    if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
    {
        TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);

        if( !I_Trace) return;

        // only tell list view to make changes to selected trace items if this
        // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)
        updateGapThreshold();
        if(this->isVisible() )
        {
            I_Trace->Tree()->setTraceOption(TRACE_GAP_DETECTION, 0, 0., "", b);
        }
    }
}

void QpDataTab::gap_threshold_changed(double d)
{
    if( QuSlotDebug() ) cout << " QpDataTab::gap_threshold_changed " << d << endl;

    if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
    {
        TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
        if( !I_Trace) return;
        
        // only tell list view to make changes to selected trace items if this
        // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

        updateGapThreshold();
        if(this->isVisible() )
        {
            I_Trace->Tree()->setTraceOption(TRACE_GAP_THRESHOLD, 0, d);
        }
    }
}

void QpDataTab::gap_threshold_type_changed(int i)
{
    if( QuSlotDebug() ) cout << " QpDataTab::gap_threshold_type_changed " << i << endl;

    TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
    if( !I_Trace) return;
    
    // only tell list view to make changes to selected trace items if this
    // is the tab being user modified (avoids infinite loop from signals as hidden tabs change)

    updateGapThreshold();
    if(this->isVisible() )
    {   I_Trace->Tree()->setTraceOption(TRACE_GAP_THRESHOLD_TYPE, i);
        
    }
}

void QpDataTab::updateGapThreshold()
{
 

   if( QuSlotDebug() ) cout << " QpDataTab::updateGapThreshold " << endl;

    if(I_PlotType == QP_TRACE || I_PlotType == QP_CONST_XT || I_PlotType == QP_CONST_Y || I_PlotType == QP_REG_LINE)
    {
        TraceItem *I_Trace = dynamic_cast<TraceItem*>(I_Plot);
		
        if(!I_Trace || I_Trace->_ydo.is_nil()) return;
       
        if(_gap_detection_fld->isChecked())
        {
            _gap_params_container->setEnabled(true);

            DvObject_var D0 = I_Trace->_ydo->getDep0();

            if(D0.is_nil()/* && _gap_threshold_type_fld->currentIndex()!=0*/)
            {
                gap_threshold=-1.;
                return;
            }
            double med=getSpacingMedian(D0);

            if(med>0)
            {
                _gap_threshold_type_fld->setItemText(1, QString("x Median Spacing (%1s)").arg(med));
            }
            else
            {
                _gap_threshold_type_fld->setItemText(1, QString("x Median Spacing (?s)"));
            }
            
            if(_gap_threshold_type_fld->currentIndex()==0)
            {
                gap_threshold=_gap_threshold_fld->value();
            }
            else
            {
                
                if(med>0)
                {
                    gap_threshold=_gap_threshold_fld->value()*med;
                }
                else
                {
                    gap_threshold=-1.;
                }
            }
        }
        else
        {
            _gap_params_container->setEnabled(false);
            gap_threshold=-1.;
        }
    }
}

QDomElement QpDataTab::toXMLElement(QDomDocument& doc, QpItemType I_Type) const
{
    QDomElement xml=doc.createElement("Data_Tab");
    switch(I_Type)
    {
        case QP_SPECTROGRAM:
            xml.appendChild(createTextXMLNode(doc, "Z_Data", _z_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "Y_Start_Data", _y_start_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "TI_MASK_Data", _mask_data_fld->text()));
//             xml.appendChild(createTextXMLNode(doc, "Y_End_Data", _y_end_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "Fill_Gaps", gap_toggle->isChecked()));
            if(_XYtoggle) xml.appendChild(createTextXMLNode(doc, "Swap_XY", _XYtoggle->isChecked()));
            xml.appendChild(createTextXMLNode(doc, "Interlace_Spectra", interlace_toggle->isChecked()));
            xml.appendChild(createTextXMLNode(doc, "X_Start_Data", _x_start_data_fld->text()));
      //      xml.appendChild(createTextXMLNode(doc, "X_End_Data", _x_end_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "Time_As_Sec", timeAsSec_toggle->isChecked()));

                break;
        default:
            xml.appendChild(createTextXMLNode(doc, "Time_As_Sec", timeAsSec_toggle->isChecked()));

            xml.appendChild(createTextXMLNode(doc, "Y_Data", _y_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "TI_Mask_Data", _mask_data_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "X_Data", _x_data_fld->text()));

            xml.appendChild(createTextXMLNode(doc, "Gap_Detection", _gap_detection_fld->isChecked()));
            xml.appendChild(createTextXMLNode(doc, "Gap_Threshold", _gap_threshold_fld->value()));
            xml.appendChild(createTextXMLNode(doc, "Gap_Threshold_Type", _gap_threshold_type_fld->currentIndex()));


            xml.appendChild(createTextXMLNode(doc, "Autolegend", _auto_legend_fld->isChecked()));
            xml.appendChild(createTextXMLNode(doc, "Legend", _legend_text_fld->text()));
            xml.appendChild(createTextXMLNode(doc, "Show_Legend", _show_legend_fld->isChecked()));
            break;
    }
    

    return xml;
}

void QpDataTab::fromXMLElement(QDomElement& el, QpItemType I_Type)
{
 	if(QuSlotDebug()) cout <<  "QpDataTab restore from XML (>>) " << I_Type  <<endl;

    QString s;

    switch(I_Type)
    {
        case QP_SPECTROGRAM:
			if(getXMLNodeText(el, "Z_Data", s))
            {
                _z_data_fld->setText(s);
				old_BaseName = baseObjectText(s.toLatin1().data());
				old_Comp = s.remove(old_BaseName.c_str()).toLatin1().data();
            }
			if(getXMLNodeText(el, "Y_Start_Data", s))
            {
                _y_start_data_fld->setText(s);
            }
            if(getXMLNodeText(el, "TI_Mask_Data", s))
            {
                _mask_data_fld->setText(s);
                old_BaseTIName = baseObjectText(s.toLatin1().data());
            }
			if(getXMLNodeText(el, "Fill_Gaps", s))
            {
                gap_toggle->setChecked(s.toInt());
            }

			if(getXMLNodeText(el, "Swap_XY", s))
            {
               if(_XYtoggle) _XYtoggle->setChecked(s.toInt());
            }
			if(getXMLNodeText(el, "Interlace_Spectra", s))
            {
                interlace_toggle->setChecked(s.toInt());
            }
            _z_data_fld->reValidate(false);
            _y_start_data_fld->reValidate(false);
			
            break;

        default:
            if(getXMLNodeText(el, "X_Data", s))
            {
                _x_data_fld->setText(s);
            }
            if(getXMLNodeText(el, "Y_Data", s))
            {
                _y_data_fld->setText(s);
                old_BaseName = baseObjectText(s.toLatin1().data());
				old_YComp = compText(s.toLatin1().data());
            }
            if(getXMLNodeText(el, "TI_Mask_Data", s))
            {
                _mask_data_fld->setText(s);
                old_BaseTIName = baseObjectText(s.toLatin1().data());
            }

            if(getXMLNodeText(el, "Gap_Detection", s))
            {
                _gap_detection_fld->setChecked(s.toInt());
            }
            if(getXMLNodeText(el, "Gap_Threshold", s))
            {
                _gap_threshold_fld->setValue(s.toDouble());
            }
            if(getXMLNodeText(el, "Gap_Threshold_Type", s))
            {
                _gap_threshold_type_fld->setCurrentIndex(s.toInt());
            }

            if(getXMLNodeText(el, "Time_As_Sec", s))
            {
                timeAsSec_toggle->setChecked(s.toInt());
            }

            if(getXMLNodeText(el, "Autolegend", s))
            {
                _auto_legend_fld->setChecked(s.toInt());
            }
            if(getXMLNodeText(el, "Legend", s))
            {
                _legend_text_fld->setText(s);
            }
            if(getXMLNodeText(el, "Show_Legend", s))
            {
                _show_legend_fld->setChecked(s.toInt());
            }

           _x_data_fld->reValidate(true);
            _y_data_fld->reValidate(true); // true so that GapThreshold update works

			
            break;
    }
    

}
