#ifndef COLOUR_TABLE_H
#define COLOUR_TABLE_H

#include <QString>
#include <QVector>
#include <QColor>
#include <QIcon>
#include <QPixmap>
#include <QPainter>
#include <QPen>
#include <QBrush>
#include <vector>
#include "plplot.h"
#include <math.h>
#include <qcolordialog.h>

class ColourTable;


class ColourTable
{
    
    public:
        ColourTable(const QString& name, const QVector<double>& points, const QVector<double>& H, const QVector<double>& S, const QVector<double>& V, bool isRGB=false);

        ColourTable(const QString& name="");

        ColourTable(const ColourTable& table);

        ~ColourTable();

        void operator=(const ColourTable&);

        void setPresetRainbow();
        void setPresetSpectrum();
        void setPresetSpectrumDark();
        void setPresetRainbowExt();
        void setPresetGrayWhiteHigh();
        void setPresetGrayWhiteLow();
        void setPresetInvRedBlue();
        void setPresetRedBlue();
        void setPresetEarth();

        void fromString(const QString&);
        QString toString() const;

        void apply(double alpha=1.0) const;
        void clear();
        QString name;


        bool operator==(const ColourTable&) const;
        bool operator!=(const ColourTable&) const;

        void drawColourBar(QPainter*, int, int);
        
        void updateIcon();
        
        QIcon icon;
                
        int isRGB;
        int size;
        PLFLT* i;
        PLFLT* h;
        PLFLT* s;
        PLFLT* v;
    protected:


};

typedef ColourTable TOTO;

typedef QVector<ColourTable> ColourTablePalette;

// used in KaHo 2D and 3D plots

PLFLT arrmax(PLFLT a[], int size);
PLFLT arrmin(PLFLT a[], int size);
void logticks(PLFLT min, PLFLT max, std::vector<PLFLT> &ticks);
inline double SafeLog10(double d)
{
    // return "safe" log value
    return (d < 1.0e-35)?-35:log10(d);
}

QColor QuGetColour(QColor inC, bool *newChoice, QWidget * parent);

#endif
